#++
Cat::DifferentialFunctionCat -- category of functions of differential variables

Domains in Cat::DifferentialFunctionCat(DV) are a Cat::AbelianSemiGroup and
contain functions that depend on differential variables from the domain DV, an
element of Cat::DifferentialVariable. (We cannot use Cat::Rng etc., as we must
allow for linear functions where multiplication is not possible. Nonlinear
domains should also be in Cat::PartialDifferentialing).

The following methods must be provided by domains in this category:
	"diff" :     differentiate with respect to an element of DV or
		     with respect to a parameter
	"indets" :   return indeterminates of function as a set;
	             with option "DiffVars" only those in DV
	"diffVars" : returns differential variables of function as ordered list
		     (leader first)
	"subs" :     substitute an expression for an element of DV
	"solve" :    solve for an element of DV;
	             if not possible return FAIL
and the methods of a Cat::AbelianSemiGroup.

For these methods default implementations are provided:
	"Variables" :  the domain DV
	"order" :      order of a function
	"class" :      class of a function
	"diffVars" :   abbreviation for "indets" with option "DiffVars"
	"leader" :     leader of a function (with respect to order in DV)
	"freeOf" :     checks whether functions depends on differential variable
	"diffSubs" :   substitute for a differential variable and its derivatives
	"isConst" :    checks whether a function is a constant (wrt to DV)
	"dimension" :  dimension of variety defined by list of functions
	"orderDim" :   like "dimension" but takes only variables of given order
	               into account
	"jacobian" :   Jacobian of a list of functions
	"symbol" :     leading part of Jacobian
	"totalDiff" :  total derivative with respect to an independent
	               variable (can be given by its index)
	"simplify" :   purely algebraic simplification
	"autoReduce" : simplification including differential operations
Some of these are not very efficient/powerful; some are valid only under
additional assumptions.


Authors: Eduard Niess, Werner M. Seiler, Uni Karlsruhe
Created: ??? (1996)
Last update: December 15th, 1997
++#

alias(TYPE_ERR=error("wrong type of argument"),
      ARG_ERR=error("wrong no of args")):


Cat::DifferentialFunctionCat := CategoryConstructor(

	Cat::DifferentialFunctionCat,

	[ DV ],

	[ DVcoerce, DVdiff, DVisone, DViszero, DVorder, DVclass,
	  DVdim, DVdimq, DVderiv, DVm2r, DVtype, DVindex,
	  notation, nn, smerge ],

	(if args(0)<>1 then error("wrong no of args") end_if;
	 if (domtype(DV)<>DOM_DOMAIN) or (DV::hasProp=FAIL) or
            not(DV::hasProp(Cat::DifferentialVariableCat)) then
             error("arg must be a Cat::DifferentialVariable");
	 end_if;

	 DVcoerce := DV::coerce;
	 DVdiff := DV::totalDiff;
	 DVisone := DV::isone;
	 DViszero := DV::iszero;
	 DVorder := DV::order;
	 DVclass := DV::class;
	 DVindex := DV::index;
	 DVdim := DV::dim;
	 DVdimq := DV::dimq;
	 DVderiv := DV::derivativeOf;
	 DVm2r := DV::m2r;
	 DVtype := DV::vartype;
	 notation := DV::notation;
	 nn := DV::numIndVar;
	 smerge := listtools::singleMerge;
	),

	[ Cat::AbelianSemiGroup ],

	[],


	# ================= #
	# undefined methods #
	# ================= #


	"diff", "indets", "subs", "solve",


	# ============== #
	# simple methods #
	# ============== #

	"Variables" = DV,


	"diffVars" = fun(sort([op(this::indets(args(1),hold(DiffVars)))],
			      DV::greater)),


	"isConst" = proc(phi)
	local JV;
	begin
	    JV := this::diffVars(phi);
	    if nops(JV)=0 then TRUE
	    elif nops(JV)>1 then FALSE
	    else (DVisone(JV[1]) or DViszero(JV[1]))
	    end_if;
	end_proc,


	"order" = (if DV::_less=DV::orderLex then
	    fun((this::diffVars(args(1));
	         if %=[] then 0 else DVorder(%[1]) end_if))
	else
	    fun((this::diffVars(args(1));
	         if %=[] then 0
	         else max(op(map(this::diffVars(args(1)),DVorder))) end_if))
	end_if),


	"class" = (if DV::_less=DV::orderLex then
	    fun((this::diffVars(args(1));
	         if %=[] then 0 else DVclass(%[1]) end_if))
	else
	    fun((this::diffVars(args(1));
	         if %=[] then return(0)
	         else max(op(map(%,DVorder))) end_if;
	         select(%2,fun(bool(DVorder(args(1))=args(2))),%);
	         min(map(%,DVclass))))
	end_if),


	"leader" = fun((this::diffVars(args(1));
                        if %=[] then DV::one else %[1] end_if)),


	"freeOf" = func(bool(contains(this::diffVars(f),jv)=0), f,jv),


        # not very efficient! should store computed derivatives of ex #
	"diffSubs" = proc(f,eq)
	local nof,nf,JVar,jvar,dex,d,jv,ex;
	begin
	    if testargs() then
		if args(0)<>2 then error("wrong no of args") end_if;
		if domtype(f)<>this or
		   type(eq)<>"_equal" then TYPE_ERR end_if;
		jv := DVcoerce(op(eq,1));
		if jv=FAIL then TYPE_ERR end_if;
		ex := this::coerce(op(eq,2));
		if ex=FAIL then TYPE_ERR end_if;
	    else
		jv := op(eq,1);
		ex := op(eq,2);
	    end_if;

	    nof := this::zero;
	    nf := f;
	    while nf<>nof do
		nof := nf;
		JVar := this::diffVars(nf);
		for jvar in JVar do
		    if notation="repeated" then
			d := DVderivativeOf(jvar,jv);
		    else
			d := DVm2r(DVderivativeOf(jvar,jv));
		    end_if;
		    if nops(d)<>0 then
			dex := this::totalDiff(ex,d);
			nf := this::subs(nf,jvar=dex);
		    end_if;
		    if DV::greater(jv,jvar) then break end_if;
		end_for;
	    end_while;

	    nf;
	end_proc,


	# primitive default implementation of dimension methods;
	  assumes that system is simplified and contains only
	  algebraically independent equations #

	"dimension" = func(DVdim(q)-nops(sys), sys,q),


	"orderDim" = func(DVdimq(q)-nops(sys), sys,q),


	# without knowing more about the domain one cannot do much
	  simplification and reduction!
	  a basic implementation based on "solve" and "leader" should
	  be provided (especially for general expressions where nothing
	  else is possible) #

	"simplify" = id,


	"autoReduce" = id,


        # ================ #
        # differentiations #
        # ================ #

	"D" = proc(x,l)
	local dvars;
	begin
	    if testargs() then
		x := this::coerce(x);
		if x=FAIL then TYPE_ERR end_if;
		if not(testtype(l,Type::ListOf(DOM_INT))) then TYPE_ERR end_if;
	    end_if;

	    if l=[] then return(x) end_if;
	    dvars := this::diffVars(x);
	    l := map(l,fun(dvars[args(1)]));
	    this::diff(x,op(l));
	end_proc,


	"totalDiff" = (if this::hasProp(Cat::CommutativeRing) then
	    proc(sys,mu)
            local i,tmp,sysflag;
            begin
                if args(0)=1 then return(sys) end_if;
		sysflag := bool(domtype(sys)=DOM_LIST);
                if not(sysflag) then sys := [sys] end_if;
                if domtype(mu)<>DOM_LIST then mu := [args(2..args(0))] end_if;

                if testargs() then
                    sys := map(sys,this::coerce);
                    if contains(sys,FAIL)<>0 then TYPE_ERR end_if;
                    for i from 1 to nops(mu) do
                        if domtype(mu[i])=DOM_INT then
                            if mu[i]<1 or mu[i]>nn then
                                TYPE_ERR
                            end_if;
                        else
                            tmp := DVcoerce(mu[i]);
                            if tmp=FAIL or DVtype(tmp)<>"Indep" then
                                TYPE_ERR
                            end_if;
                            mu[i] := DVindex(tmp);
                        end_if;
                    end_for;
                end_if;

                tmp := (sys,map(sys,this::diffVars));
		if _and(op(map(op(tmp,2),fun(bool(args(1)=[]))))) then
		    if sysflag then return([this::zero $ nops(sys)])
		    else return(this::zero) end_if;
		end_if;
                for i in mu do
                    tmp := this::totalDiffJM(op(tmp,1),i,this::jacobian(tmp));
                 end_for;
                 if sysflag then op(tmp,1) else op(tmp,[1,1]) end_if;
            end_proc
	else toBeDefined end_if),


	"totalDiffJM" = (if this::hasProp(Cat::CommutativeRing) then
            proc(sys,mu,JM)
            local n,vars,nvars,dvars,tdvars,rvars,rows,i,j,k,pos,res,dv,t;
            begin
                if testargs() then
                    if args(0)<>3 then ARG_ERR end_if;
                    sys := map(sys,this::coerce);
                    if domtype(mu)=DOM_INT then
                        if mu<1 or mu>nn then TYPE_ERR end_if;
                    else
                        mu := DVcoerce(mu);
                        if mu=FAIL then TYPE_ERR end_if;
                        mu := DVindex(mu);
                    end_if;
                    if domtype(JM)<>Dom::SparseMatrix(DV,this,DV::greater) then
                        TYPE_ERR
                    end_if;
                    if nops(sys)<>JM::nrows(JM) then
                        error("incompatible jacobian");
                    end_if;
                end_if;

                n := JM::nrows(JM);
                vars := JM::allIndices(JM);
                if vars=[] then
                    return(([this::zero $ n], [[] $ n]));
                end_if;
                rows := JM::rows(JM);
                nvars := select(vars,fun(contains({"Deriv","Dep"},
                                                  DVtype(args(1)))));
                dvars := map(nvars,DVdiff,mu);
		tdvars := map(dvars,this::convert);
                vars := [ [] $ n ];

                for i from 1 to n do         # for each equation #
		    if rows[i]=[] then
			sys[i] := this::zero;
			next;
		    end_if;
                    rvars := map(rows[i],op,1);
                    vars[i] := [ 0 $ nops(rvars) ];
                    j := 0;
		    k := 0;
                    res := [ this::zero $ nops(rows[i]) ];
                    for t in rows[i] do      # compute derivative #
                        case DVtype(t[1])
                        of "Dep" do
                        of "Deriv" do
                            pos := contains(nvars,t[1]);
			    k := k+1;
                            res[k] := this::_mult(t[2],tdvars[pos]);
                            j := j+1;
                            vars[i][j] := dvars[pos];
                            break;
                        of "Indep" do
                            if DVindex(t[1])=mu then
				k := k+1;
                                res[k] := t[2];
                            end_if;
                            break;
                        end_case;
                    end_for;
		    sys[i] := this::_plus(op(res,1..k));
                    vars[i] := [op(vars[i],1..j)];
                    k :=1;
                    while k<=nops(rvars) do   # check variables #
                        if contains(vars[i],rvars[k])>0 or
                           this::freeOf(res,rvars[k]) then
                            rvars[k] := NIL
                        else k := k+1 end_if;
                    end_while;
                    vars[i] := listtools::merge(vars[i],rvars,DV::greater);
                end_for;
                (sys,vars);
            end_proc
        else NIL end_if),


	# ================ #
	# jacobian, symbol #
	# ================ #

	"jacobian" = (if this::hasProp(Cat::CommutativeRing) then
            proc(sys)
            local v,vars,varlist,i,j,f,r,rows,SEM;
            begin
		if args(0)>1 then vars := args(2) end_if;
		if domtype(sys)<>DOM_LIST then
		    sys := [sys];
		    if args(0)>1 then vars := [vars] end_if;
		end_if;
                if testargs() then
		    if args(0)<1 or args(0)>2 then ARG_ERR end_if;
		    sys := map(sys,this::coerce);
		    if contains(sys,FAIL)>0 then TYPE_ERR end_if;
		    if args(0)=2 then
			if {op(map(vars,domtype))}<>{DOM_LIST} then
			    TYPE_ERR
			end_if;
			vars := map(vars,map,DVcoerce);
			if has(vars,FAIL) then TYPE_ERR end_if;
		    end_if;
                end_if;

                if args(0)=1 then vars := map(sys,this::diffVars) end_if;
                varlist := [];
                for v in vars do
                    varlist := smerge(v,varlist,DV::greater);
                end_for;
                if varlist=[] then error("empty matrix") end_if;
                # should return zero matrix (of what size?) #

		rows := [ [] $ nops(sys) ];
		i := 0;
                for f in sys do
		    i := i+1;
		    if vars[i]=[] then next end_if;
		    r := [ 0 $ nops(vars[i]) ];
		    j := 0;
		    for v in vars[i] do
			j := j+1;
			r[j] := [v,this::diff(f,v)];
		    end_for;
		    rows[i] := r;
		end_for;
		SEM := Dom::SparseMatrix(DV,this,DV::greater);
		SEM(nops(sys),varlist,rows);
            end_proc
        else toBeDefined end_if),


	"symbol" = proc(sys)
	local n,v,vars,varlist,i,j,f,r,rows,SEM;
	begin
	    SEM := Dom::SparseMatrix(DV,this,DV::greater);
	    if testargs() then
		if args(0)<>1 then ARG_ERR end_if;
		if domtype(sys)<>SEM then
		    if domtype(sys)<>DOM_LIST then sys := [sys] end_if;
		    sys := map(sys,this::coerce);
		    if contains(sys,FAIL)>0 then TYPE_ERR end_if;
		end_if;
	    end_if;

	    if domtype(sys)=DOM_LIST then
                vars := map(sys,this::diffVars);
                n := max(op(map(vars,fun(max(op(map(args(1),DVorder)))))));
                vars := map(vars,select,
                                 fun(bool(DVorder(args(1))=args(2))),n);
                varlist := [];
                for v in vars do
                    varlist := smerge(v,varlist);
                end_for;
                if varlist=[] then error("empty matrix") end_if;
                # should return zero matrix (of what size?) #

		rows := [ [] $ nops(sys) ];
		i := 0;
                for f in sys do
		    i := i+1;
		    if vars[i]=[] then next end_if;
		    r := [ 0 $ nops(vars[i]) ];
		    j := 0;
		    for v in vars[i] do
			j := j+1;
			r[j] := [v,this::diff(f,v)];
		    end_for;
		    rows[i] := r;
		end_for;
		SEM := Dom::SparseMatrix(DV,this,DV::greater);
		SEM(nops(sys),varlist,rows);
	    else
		n := DVorder(SEM::allIndices(sys)[1]);
		SEM::purge(sys,fun(bool(DVorder(args(1))<>args(2))),n);
	    end_if;
	end_proc

):

unalias(TYPE_ERR,ARG_ERR):

