#++
Cat::DifferentialVariableCat -- category of differential variables

Its purpose is to provide basic data structures and procedures for
differential variables. In geometric approaches one may consider it as an
implementation of adapted charts in jet bundles.  Every instance of this
category has a fixed number of independent and dependent variables which can
be obtained with the methods "numIndVar" and "numDepVar", respectively.

Differential variables are classified into four types: "Indep" are the
independent, "Dep" the dependent variables, "Deriv" derivatives; for some
compatibility reasons there is a fourth type "Const" which basically contains
0 and 1.  The method "varype" returns this type as a string.

For derivatives two kinds of notations are supported: multi index and repeated
index notation. The method "notation" returns a string "multi" or "repeated"
indicating the used notation for a specific domain. The methods "multiIndex" and
"repeatedIndex" return the indices in the respective notation. In the case of
repeated indices, it is always assumed that a sorted list (lowest index first)
is returned. For conversion, two auxiliary methods "m2r" and "r2m" are
provided. Internally, the independent and dependent variables are always
indexed. The method "index" returns this index. For derivatives it returns the
index of the corresponding dependent variable.

Each domain should provide two methods "newMulti" and "newRepeated" to generate
differential variables.  They take as first argument the type of the variable, as
second argument its index and, for derivatives, as third argument its multi or
repeated index, respectively.  These two methods are mainly thought for the use
in programs and not for interactive use.  Therefore they should be fast!  The
category provides a default implementation which assumes that convert can handle
such input.

Cat::DifferentialVariableCat is a subcategory of Cat::OrderedSet. It provides
several orders but the user can also implement his own by an assignment to the
method "_less".  Alternatively, he can use the method "setOrder".  It takes one
argument which may be either a string representing the name of one of the
already implemented orders or his own procedures.  This procedures should take
two differential variables as arguments and return TRUE or FALSE.  Currently
implemented are:
	"DegreeOrder" -- derivatives are first ordered according to their order
		         then lexicographically. This is the default for "_less".
	"LexOrder"    -- derivatives are ordered purely lexicographically.

The following methods have to been defined in domains of this category
	vartype :   type of a differential variable
	index :     index of a differential variable
	numIndVar : number of independent variables
	numDepVar : number of dependent variables
	notation :  used notation ("multi" or "repeated")
The last three are not functions but should yield directly the corresponding
constants.

Default implementations are provided for:
	repeatedIndex : repeated index of a derivative
			(if this::notation="multi")
	multiIndex :    multi index of a derivative
	                (if this::notation="repeated")
        newRepeated :   generation of differential variables given
                        the repeated index
        newMulti :      generation of differential variables given
                        the multi index
	m2r :           converts a multi index into a repeated index
	r2m :           converts a repeated index into a multi index
	allRepeated :   list of all repeated indices corresponding to a
	                given multi index
	class :         class of a derivative
	order :         order of a derivative
	dim :           number of differential variables up to a given order
	dimq :          number of differential variables of a given order
	diff :          differentiates a differential variable wrt to some
			variable (treats all variables as ordinary variables)
	totalDiff :     differentiates a differential variable wrt to an
			independent variables ("true" differentiation)
	derivativeOf :  checks whether a differential variable is the
	                the derivative of another one
	int :     	integrates a differential variable
	indVariables :	list of all independent variables
	depVariables :	list of all dependent variables
	derivatives :   list all derivatives up to a given order
	setOrder :      changes the order
	greater :       opposite of _less


Authors: Eduard Niess, Werner M. Seiler, Uni Karlsruhe
Created: ??? (1996)
Last update: January 16th, 1998
++#

alias(TYPE_ERR = error("wrong type of argument"),
      ARG_ERR = error("wrong no of args"),
      IND_ERR = error("illegal index")):


Cat::DifferentialVariableCat := CategoryConstructor(

	Cat::DifferentialVariableCat,

	[], [], [], [ Cat::OrderedSet ], [],


	# ================= #
	# undefined methods #
	# ================= #

	"vartype", "index", "numIndVar", "numDepVar", "notation",


	"repeatedIndex" = (if this::notation="multi" then
	       fun(this::m2r(this::multiIndex(args(1))));
	    else toBeDefined end_if),


	"multiIndex" = (if this::notation="repeated" then
	       fun(this::r2m(this::repeatedIndex(args(1))));
	    else toBeDefined end_if),


	# ================= #
	# auxiliary methods #
	# ================= #

	"raise" = proc(l,mu)
	local i;
	begin
	    if domtype(mu)=DOM_INT then l[mu] := l[mu]+1
	    else
		for i in mu do l[i] := l[i]+1 end_for;
	    end_if;
	    l;
	end_proc,


        "m2r" = fun([ (i $ args(1)[i]) $ hold(i)=1..this::numIndVar ]),


	"r2m" = proc(l)
	local i,res;
	begin
	    res := [ 0 $ this::numIndVar ];
	    for i in l do
		res[i] := res[i]+1;
	    end_for;
	    res;
	end_proc,
	
	# returns a list of all possible realizations of a given multi-index
	  as repeated index. #
	"allRepeated" = proc(l)
	local i,ll,res;
	begin
	    res := [];
	    for i from 1 to this::numIndVar do
		if not(iszero(l[i])) then
		    ll := l;
		    ll[i] := ll[i]-1;
		    res := res . map(this::allRepeated(ll),
				     fun([args(2)].args(1)),i);
		end_if;
	    end_for;
	    if res = [] then [[]]
	    else res end_if;
	end_proc,


	"newMulti" =
	    (if this::notation="multi" then this::convert
	     else fun(this::convert(args(1),args(2),this::m2r(args(3))))
	     end_if),


	"newRepeated" =
	    (if this::notation="repeated" then this::convert
	     else fun(this::convert(args(1),args(2),this::r2m(args(3))))
	     end_if),


	# ================ #
	# "simple" methods #
	# ================ #


        "zero" = fun(this::convert(0)),


        "iszero" = fun(bool(args(1)=this::zero)),


        "one" = fun(this::convert(1)),


        "isone" = fun(bool(args(1)=this::one)),


	"class" = fun((if this::vartype(args(1))<>"Deriv" then 0
		       else this::repeatedIndex(args(1)); %[1] end_if)),


	"order" = (if this::notation="multi" then
	       fun((if this::vartype(args(1))<>"Deriv" then 0
	            else _plus(op(this::multiIndex(args(1)))) end_if))
	    else
	       fun((if this::vartype(args(1))<>"Deriv" then 0
	            else nops(this::repeatedIndex(args(1))) end_if))
	    end_if),


	"dim" = proc(q)
	option remember;
	begin
	    this::numDepVar * binomial(q+this::numIndVar, q);
	end_proc,


	"dimq" = proc(q)
	option remember;
	begin
	    this::numDepVar * binomial(q-1+this::numIndVar, q);
	end_proc,


	# =============== #
	# differentiation #
	# =============== #

	# this is more or less a hack needed for domains like
	  Dom::DifferentialExpression in order to provide an
	  interface for the system function "diff"; differential
	  variables are treated like ordinary variables; the
	  output is either 0 or 1 (and not this::zero or this::one!) #
	"diff" = fun((case args(0)
		      of 0 do ARG_ERR;
		      of 1 do return(args(1));
		      of 2 do
			  if testargs() then this::convert(args(2))
			  else args(2) end_if;
			  if args(1)=% then return(1)
			  else return(0) end_if;
		      otherwise return(0);
		      end_case)),


	# the "real" differentiation #
	"totalDiff" = fun((if testargs() then
		this::checkTotalDiff(args())
	    else case args(0)
		of 1 do return(args(1));
		of 2 do
		    if domtype(args(2))=DOM_LIST then
			this::totalDiffN(args());
		    else this::totalDiff2(args()) end_if;
		    break;
		otherwise this::totalDiffN(args(1),[args(2..args(0))]);
		end_case;
	    end_if)),


	"totalDiff2" = (if this::notation="multi" then
	    fun((case this::vartype(args(1))
	    	 of "Const" do return(this::zero);
	    	 of "Indep" do
	             if this::index(args(1))=args(2) then return(this::one)
                     else return(this::zero) end_if;
	    	 of "Dep" do
	             return(this::newRepeated("Deriv",this::index(args(1)),
					      [args(2)]));
	    	 of "Deriv" do
	             this::raise(this::multiIndex(args(1)),args(2));
	             return(this::newMulti("Deriv",this::index(args(1)),%));
	    	 end_case))
	else
	    fun((case this::vartype(args(1))
	    	 of "Const" do return(this::zero);
	    	 of "Indep" do
	             if this::index(args(1))=args(2) then return(this::one)
                     else return(this::zero) end_if;
	    	 of "Dep" do
	             return(this::newRepeated("Deriv",this::index(args(1)),
					      [args(2)]));
	    	 of "Deriv" do
	             insert_ordered(args(2),this::repeatedIndex(args(1)));
	             return(this::newRepeated("Deriv",this::index(args(1)),%));
	    	 end_case))
	end_if),


	"totalDiffN" = (if this::notation="multi" then
	    fun((case this::vartype(args(1))
	         of "Const" do return(this::zero);
	         of "Indep" do
	             if nops(args(2))=1 and this::index(args(1))=args(2)[1] then
	                 return(this::one)
                     else return(this::zero) end_if;
	         of "Dep" do
	             return(this::newMulti("Deriv",this::index(args(1)),
					   this::r2m(args(2))));
		 of "Deriv" do
		     this::raise(this::multiIndex(args(1)),args(2));
		     return(this::newMulti("Deriv",this::index(args(1)),%));
	         end_case))
	else
	    fun((case this::vartype(args(1))
	         of "Const" do return(this::zero);
	         of "Indep" do
	             if nops(args(2))=1 and this::index(args(1))=args(2)[1] then
	                 return(this::one)
                     else return(this::zero) end_if;
	         of "Dep" do
	             return(this::newRepeated("Deriv",this::index(args(1)),
					      sort(args(2))));
	         of "Deriv" do
	             listtools::merge(sort(args(2)),
				      this::repeatedIndex(args(1)));
	             return(this::newRepeated("Deriv",this::index(args(1)),%));
	         end_case))
	end_if),


	"checkTotalDiff" = proc(jv)
	local mu,tmp,i;
	begin
	    if args(0)=1 then return(jv) end_if;
	    if args(0)>2 then mu := [args(2..args(0))]
	    else mu := args(2) end_if;
	    if domtype(jv)<>this then
		jv := this::coerce(jv);
		if jv=FAIL then TYPE_ERR end_if;
	    end_if;
	    if domtype(mu)=DOM_LIST then
		for i from 1 to nops(mu) do
		    case domtype(mu[i])
		    of DOM_INT do break;
		    of this do
			if this::vartype(mu[i])<>"Indep" then TYP_ERR end_if;
			mu[i] := this::index(mu[i]);
		        break;
		    otherwise
		        tmp := this::coerce(mu[i]);
		        if tmp=FAIL or
			   this::vartype(tmp)<>"Indep" then TYPE_ERR end_if;
		        mu[i] := this::index(tmp);
		    end_case;
		end_for;
	    else
		case domtype(mu)
		of DOM_INT do break;
		of this do
		    if this::vartype(mu)<>"Indep" then TYP_ERR end_if;
		    mu := this::index(mu);
		    break;
		otherwise
		    tmp := this::coerce(mu);
		    if tmp=FAIL or
		       this::vartype(tmp)<>"Indep" then TYPE_ERR end_if;
		    mu := this::index(tmp);
		end_case;
	    end_if;

            if domtype(mu)=DOM_LIST then this::totalDiffN(jv,mu)
	    else this::totalDiff2(jv,mu) end_if;
	end_proc,


	"derivativeOf" = proc(j1,j2)
	local jt,res,mi1,mi2,i;
	begin
	    if testargs() then
		if domtype(j1)<>this then
		    j1 := this::coerce(j1);
		    if j1=FAIL then TYPE_ERR end_if;
		end_if;
		if domtype(j2)<>this then
		    j2 := this::coerce(j2);
		    if j2=FAIL then TYPE_ERR end_if;
		end_if;
	    end_if;

	    if this::vartype(j1)<>"Deriv" then return([]) end_if;
	    jt := this::vartype(j2);
	    if jt<>"Deriv" and jt<>"Dep" then return([]) end_if;
	    if this::index(j1)<>this::index(j2) then return([]) end_if;

	    res := [ 0 $ this::numIndVar ];
	    mi1 := this::multiIndex(j1);
	    mi2 := this::multiIndex(j2);
	    for i from 1 to this::numIndVar do
		res[i] := mi1[i]-mi2[i];
		if res[i]<0 then return([]) end_if;
	    end_for;

	    if this::notation="repeated" then return(this::m2r(res)) end_if;
	    res;
	end_proc,


	"int" = proc(jv,i)
	local nind,pos,isderiv,j;
	begin
	    if domtype(i)=this and this::vartype(i)="Indep" then
	        i := this::index(i);
	    end_if;
	    if testargs() then
		if domtype(jv)<>this then
		    jv := this::coerce(jv);
		    if jv=FAIL then TYPE_ERR end_if;
		end_if;
		if domtype(i)<>DOM_INT then
		    i := this::coerce(i);
		    if i=FAIL or this::vartype(i)<>"Indep" then TYPE_ERR end_if;
		    i := this::index(i);
		end_if;
		if (i<1) or (i>this::numIndVar) then IND_ERR end_if;
	    end_if;

	    if this::vartype(jv)<>"Deriv" then return(FAIL) end_if;

	    if this::notation="multi" then
		nind := this::multiIndex(jv);
		if iszero(nind[i]) then	return(FAIL) end_if;
		nind[i] := nind[i]-1;
		isderiv := FALSE;
		for j in nind do
		    if not(iszero(j)) then
			isderiv := TRUE;
			break;
		    end_if;
		end_for;
		if isderiv then
		    return(this::newMulti("Deriv",this::index(jv),nind));
		else
		    return(this::newMulti("Dep",this::index(jv),[]));
		end_if;
	    else
		nind := this::repeatedIndex(jv);
		pos := contains(nind,i);
		if iszero(pos) then return(FAIL) end_if;
		nind[pos] := NIL;
		if nind = [] then
			return(this::newRepeated("Dep",this::index(jv),[]));
		else
			return(this::newRepeated("Deriv",this::index(jv),nind));
		end_if;
	    end_if;
	end_proc,


	# ========= #
	# variables #
	# ========= #

	"indVariables" = fun([this::newRepeated("Indep",j) $
			      hold(j)=1..this::numIndVar]),
			      
			      
	"depVariables" = fun([this::newRepeated("Dep",j) $
			      hold(j)=1..this::numDepVar]),
			      
			      
	"derivatives" = proc(q)
	local nIV,nDV,qq,oIndList,nIndList,ind,j,indList,jv;
	begin
	    if testargs() then
		if args(0)<1 or args(0)>2 then
		    error("wrong number of args");
		end_if;
		if domtype(q)<>DOM_INT then
		    error("first arg must be an integer");
		end_if;
		if q<0 then error("order must be non-negative") end_if;
		if args(0)=2 then
		    if domtype(args(2))<>DOM_INT then
			error("second arg must be an integer");
		    end_if;
		    if (args(2)<1) or (args(2)>this::numIndVar) then
			error("illegal class");
		    end_if;
		end_if;
	    end_if;

	    nIV := this::numIndVar;
	    nDV := this::numDepVar;

	    if args(0)=1 then
		if iszero(q) then return([]) end_if;

		oIndList := [[j] $ j=1..nIV];
		indList := oIndList;
		for qq from 2 to q do
		    nIndList := [];
		    for ind in oIndList do
			nIndList := nIndList . [ind . [j] $ j=ind[qq-1]..nIV];
	  	    end_for;
		    oIndList := nIndList;
		    indList := indList . nIndList;
		end_for;

		jv := [];
		for ind in indList do
		    jv := jv . [this::newRepeated("Deriv",j,ind) $ j=1..nDV];
		end_for;
		jv;

	    else
		if iszero(q) then return([]) end_if;

		oIndList := [[j] $ j=args(2)..nIV];
		for qq from 2 to q do
		    nIndList := [];
		    for ind in oIndList do
			nIndList := nIndList . [ind . [j] $ j=ind[qq-1]..nIV];
		    end_for;
		    oIndList := nIndList;
		end_for;

		jv := [];
		for ind in oIndList do
		    jv := jv . [this::newRepeated("Deriv",j,ind) $ j=1..nDV];
		end_for;
		jv;
	    end_if;
	end_proc,


	# ======== #
	# ordering #
	# ======== #

	"orderLex" = proc(jv1,jv2)
	local t1,t2,o1,o2,ind,inds2;
	begin
	    if testargs() then
	       if domtype(jv1)<>this then
		    jv1 := this::coerce(jv1);
		    if jv1=FAIL then TYPE_ERR end_if;
		end_if;
		if domtype(jv2)<>this then
		    jv2 := this::coerce(jv2);
		    if jv2=FAIL then TYPE_ERR end_if;
		end_if;
            end_if;

	    t1 := this::vartype(jv1);
	    t2 := this::vartype(jv2);

	    if t2 = "Const" then return(FALSE) end_if;

	    case t1
	    of "Const" do return(TRUE);
	    of "Indep" do
                if t2="Indep" then
		    return(bool(this::index(jv1)<this::index(jv2)));
                else return(TRUE) end_if;
            of "Dep" do
                case t2
                of "Indep" do return(FALSE);
                of "Dep" do return(bool(this::index(jv1)<this::index(jv2)));
		otherwise return(TRUE);
                end_case;
	    end_case;

	    if t2="Indep" or t2="Dep" then return(FALSE) end_if;

	    o1 := this::order(jv1);
	    o2 := this::order(jv2);

	    if o1=o2 then
		inds2 := this::multiIndex(jv2);
		for ind in this::multiIndex(jv1) do
		    if inds2=[] then break end_if;
		    if ind<>inds2[1] then return(bool(ind>inds2[1])) end_if;
		    inds2[1] := NIL;
		end_for;
		bool(this::index(jv1)<this::index(jv2));
	    else bool(o1<o2) end_if;
	end_proc,


	"pureLex" = proc(jv1,jv2)
	local t1,t2,i,ind,inds2;
	begin
	    if testargs() then
		if domtype(jv1)<>this then
		    jv1 := this::coerce(jv1);
		    if jv1=FAIL then TYPE_ERR end_if;
		end_if;
		if domtype(jv2)<>this then
		    jv2 := this::coerce(jv2);
		    if jv2=FAIL then TYPE_ERR end_if;
		end_if;
	    end_if;

	    t1 := vartype(jv1);
	    t2 := vartype(jv2);

	    if t2 = "Const" then return(FALSE) end_if;

	    case t1
            of "Const" do return(TRUE);
            of "Indep" do
                if t2 = "Indep" then
                    return(bool(this::index(jv1)<this::index(jv2)));
                else return(TRUE) end_if;
            of "Dep" do
                case t2
                of "Indep" do return(FALSE);
                of "Dep" do return(bool(this::index(jv1)<this::index(jv2)));
                otherwise return(TRUE);
                end_case;
	    end_case;

	    if t2="Indep" or t2="Dep" then return(FALSE) end_if;

	    inds1 := this::multiIndex(jv1);
	    inds2 := this::multiIndex(jv2);
	    for i from 1 to this::numIndVar do
		if inds1[i]<inds2[i] then return(FALSE)
		elif inds1[1]>inds2[i] then return(TRUE)
		end_if;
	    end_for;
	    FALSE
	end_proc,


	"_less" = this::orderLex,


        "greater" = fun(this::_less(args(2),args(1))),


	"setOrder" = proc(s)
	begin
	    if domtype(s)=DOM_STRING then
		case s
		of "DegreeOrder" do this::_less := this::orderLex; break;
		of "LexOrder" do this::_less := this::pureLex; break;
		otherwise error("unknown order");
		end_case;
	    else this::_less := s end_if;
	    this::greater := fun(this::_less(args(2),args(1)));
	    NIL;
	end_proc,

	# ============= #
	# miscellaneous #
	# ============= #

	"random" = proc()
	local r,ri,rd;
	begin
	    r := random(1..3);
	    case r()
	    of 1 do
		ri := random(1..this::numIndVar);
		return(this::newRepeated("Indep",ri()));
	    of 2 do
		rd := random(1..this::numDepVar);
		return(this::newRepeated("Dep",rd()));
	    of 3 do
		ri := random(1..this::numIndVar);
		rd := random(1..this::numDepVar);
		return(this::newRepeated("Deriv", rd(), sort([ri() $ r()])));
	    end_case;
	end_proc

)():


unalias(TYPE_ERR,IND_ERR):
