#++
Cat::FiniteMonoidRing -- category of free algebras over an ordered monoid
	with finite support

A Cat::FiniteMonoidRing(R,S) is a Cat::MonoidRing(R,S) with the additional
property that each element is only of finite support. It defines the
following additional methods:
   mapcoeffs(x,f) :   applies the function f to every coefficient in x,
                      thus f must map elements of R to elements of R
   terms(x) :         a (sorted) list of all terms of x with non-vanishing
                      coefficient
   coeff(x) :         a list of all coefficients (order matching that of
		      terms(x), i.e. the first entry is the coefficient of
		      the first entry of support(x) etc.)
   coeff(x,s) :       the coefficient of the generator s
   nterms(x) :        the number of monomials in x
   nthmonomial(x,n) : the n-th monomial of x
   nthterm(x,n) :     the n-th term of x
   nthcoeff(x,n) :    the n-th coefficient of x

Default implementations are provided for lcoeff, lterm, isMonomial, nthmonomial,
nthterm, nthcoeff, mapcoeffs, degree and nterms. Some of these are not very
efficient!

Author: Werner M. Seiler, Uni Karlsruhe
Created: July 3rd, 1997
Last updated: August 20th, 1997
++#

Cat::FiniteMonoidRing := CategoryConstructor(

	Cat::FiniteMonoidRing,

	[R,S],

	[Rzero,Riszero,Sone,Sisone,Sdegree],

	(if args(0)<>2 then error("wrong no of args") end_if;
	if (domtype(R)<>DOM_DOMAIN) or (domtype(S)<>DOM_DOMAIN) or
	   (R::hasProp=FAIL) or (S::hasProp=FAIL) then
		error("arguments must be domains");
	end_if;
	if R::hasProp(Cat::Ring) then
		Rzero := R::zero;
		Riszero := R::iszero;
	else error("first argument must belong to Cat::Ring") end_if;
	if S::hasProp(Cat::OrderedMonoid) then
		Sone := S::one;
		Sisone := S::isone;
	elif S::hasProp(Cat::OrderedAbelianMonoid) then
		Sone := S::zero;
		Sisone := S::iszero;
	else error("second argument must be an ordered monoid")	end_if;
	Sdegree := S::degree;
	),

	[Cat::MonoidRing(R,S)],

	[],


        # ====================================================== #
	# implementations for methods defined in Cat::MonoidRing #
	# ====================================================== #

        "lcoeff" = fun((if this::iszero(args(1)) then Rzero
                        else this::coeffs(args(1))[1] end_if)),


        "lterm" = fun((if this::iszero(args(1)) then Sone
                        else this::support(args(1))[1] end_if)),


        "isMonomial" = fun(bool(this::nterms(args(1))=1)),


	"degree" = (if Sdegree<>FAIL then
			fun(max(op(map(this::terms(args(1)),Sdegree))))
		    else NIL end_if),


	"_divide" = (if R::hasProp(Cat::Field) then
			fun(this::mapcoeffs(args(1),R::_divide,args(2)))
		    else NIL end_if),


	# ================== #
	# additional methods #
	# ================== #

	"terms",

	"nterms" = fun(nops(this::terms(args(1)))),


        "nthterm" = fun((if this::nterms(args(1))<args(2) then
                             error("term does not exist")
                         else this::terms(args(1))[args(2)] end_if)),


        "nthcoeff" = fun((if this::nterms(args(1))<args(2) then
                             error("coefficient does not exist")
                          else this::coeff(args(1))[args(2)] end_if)),


        "nthmonomial" = fun((if this::nterms(args(1))<args(2) then
                                 error("monomial does not exist")
                             else
                                this::monomial(this::nthcoeff(args(1),args(2)),
                                               this::nthterm(args(1),args(2)))
                             end_if)),


	"mapcoeffs" = proc()
	local x,f,rargs,r;
	begin
	    if args(0)<2 then error("too few arguments") end_if;
            x := args(1);
            f := args(2);
            rargs := args(3..args(0));
	    r := fnct(this::lcoeff(x),rargs);
	    if Riszero(r) then
		 return(this::mapcoeffs(this::reductum(x),fnct,rargs));
	    end_if;
	    this::_plus(this::monomial(r,this::lterm(x)),
		  	this::mapcoeffs(this::reductum(x),fnct,rargs));
	end_proc

):
