#++
Cat::MonoidRing -- category of free algebras over an ordered monoid

A Cat::MonoidRing(R,S) is a Cat::Ring and a Cat::LeftModule(S) where
R belongs to Cat::Ring and S to Cat::OrderedMonoid or
Cat::OrderedAbelianMonoid. A domain in this category represents a free
algebra where the support come from the monoid and the coefficients
from the ring. Thus an element of such a domain is a formal sum of
monomials. A monomial is a term consisting of a coefficient from R
and a generator from S.  Domains where the formal sum contains always
only finitely many non-vanishing monomials belong to the subcategory
Cat::FiniteMonoidRing(R,S).

Cat::MonoidRing is the basic super category for anything "that looks
like a polynomial or a series". This includes for example differential
operators or forms, Ore polynomials or a tensor algebra.

A Cat::MonoidRing has besides the obvious methods stemming from the super
categories the following methods:
   lmonomial(x) :    the leading monomial of x (with respect to
                     the ordering of S)
   lcoeff(x) :       the coefficient of the lmonomial(x)
   lterm(x) :        the term of the lmonomial(x)
   reductum(x) :     x - lmonomial(x)
   const(x) :        the coefficient of the constant term
   monomial(r,s):    generates a monomial from its components
   coeff(x,s) :      the coefficient of x for a given generator
   isMonomial(x) :   TRUE, if x consists of only one monomial
Default implementations are provided for lmonomial, reductum and const. If S
has a method degree, then this method is extended to the monoid ring.

Author: Werner M. Seiler, Uni Karlsruhe
Created: July 1st, 1997
Last Updated: August 20th, 1997
++#

Cat::MonoidRing := CategoryConstructor(

	Cat::MonoidRing,

	[R,S],

	[Rzero,Riszero,Sone,Sisone],

	(if args(0)<>2 then error("wrong no of args") end_if;
	if (domtype(R)<>DOM_DOMAIN) or (domtype(S)<>DOM_DOMAIN) or
	   (R::hasProp=FAIL) or (S::hasProp=FAIL) then
		error("arguments must be domains");
	end_if;
	if R::hasProp(Cat::Ring) then
		Rzero := R::zero;
		Riszero := R::iszero;
	else error("first argument must belong to Cat::Ring") end_if;
	if S::hasProp(Cat::OrderedMonoid) then
		Sone := S::one;
		Sisone := S::isone;
	elif S::hasProp(Cat::OrderedAbelianMonoid) then
		Sone := S::zero;
		Sisone := S::iszero;
	else error("second argument must be ordered monoid") end_if;
	),

	[Cat::Ring, Cat::LeftModule(R)],

	[],


	"coeff", "monomial", "lcoeff", "lterm", "isMonomial", "const",

	"lmonomial" = fun((if this::iszero(args(1)) then
                                error("not defined for zero")
                           end_if;
                           this::monomial(this::lcoeff(args(1)),
                                          this::degree(args(1))))),


        "reductum" = fun(this::_subtract(args(1),this::lmonomial(args(1)))),


        "const" = fun(this::coeff(args(1),Sone)),


        "degree" = (if S::degree<>FAIL then toBeDefined
		    else NIL end_if),


	"_divide" = (if R::hasProp(Cat::Field) then toBeDefined
		    else NIL end_if)

):
