# $Date: 1999/04/28 14:52:06 $ $Author: stefanw $ $Revision: 1.35.2.1 $ #

#++
Dom::AlgebraicExtension -- domain constructor for simple algebraic extension

Dom::AlgebraicExtension(F, p [,a])

F - field
p - irreducible polynomial or expression defining adjoined element
a - name of the adjoined element (identifier)

Dom::AlgebraicExtension(F,p,a) creates a simple algebraic extension F(a) where a
are the roots of an irreducible polynomial defined by p. p must either be an
equation, a polynomial expression or a DOM_POLY.

- If p is a polynomial expression or a DOM_POLY, it is converted into a
  univariate polynomial over F in the variable a. If a is not given then p
  must have exactly one unknown which then takes the role of a.

- If p is an equation x=y, it is changed into the expression x-y. x-y is then
  used as defining polynomial, such that the same rules as above apply.

It is not tested whether p is irreducible or not!

Methods:-
conjNorm(x) - returns the norm of x, ie. the product of all conjugates of x
conjTrace(x) - returns the trace of x, i.e. the sum of all conjugates of x

++#

Dom::AlgebraicExtension:= DomainConstructor(
    Dom::AlgebraicExtension,
    [ F , PExpr, Alias ],
    [ P ],
    (case args(0)
     of 2 do 
         Alias:= null(); 
         break;
     of 3 do
         if domtype(Alias) <> DOM_IDENT then
	     error("illegal variable type")
         end_if;
         Alias:= [ Alias ];
         break;
     otherwise error("wrong no of args")
     end_case;
     if F::hasProp(Cat::Field) <> TRUE then
        error("no Cat::Field given")
     end_if;
     if not F::hasProp(Ax::normalRep) then
        error("field has no normal representation")
     end_if;
     if type(PExpr) = "_equal" then
         PExpr:= op(PExpr,1) - op(PExpr,2)
     end_if;
     if F = Dom::Rational then
	P:= poly(PExpr, Alias, Expr);
	if not testtype(P, Type::PolyOf(Type::Rational,1)) then
	    error("not a univariate polynomial")
	end_if;
     elif F::constructor = Dom::IntegerMod then
	P:= poly(PExpr, Alias, hold(IntMod)(F::constructor_args[1]));
	if P = FAIL then
	    error("not a polynomial")
	end_if;
	if nops(op(P,2)) <> 1 then
	    error("multivariate polynomial")
	end_if;
     else
	P:= poly(PExpr, Alias, F);
	if P = FAIL then
	    error("not a polynomial")
	end_if;
        if nops(op(P,2)) <> 1 then
	    error("multivariate polynomial")
        end_if;
     end_if;
     if degree(P) = 0 then
     	error("constant polynomial")
     end_if;
     P:= multcoeffs(P, 1/lcoeff(P));
     PExpr:= (expr(P) = 0);
     Alias:= op(P,[2,1]);
    ),
    Dom::BaseDomain,
    [ Cat::Field, Cat::Algebra(F), Cat::VectorSpace(F)  ],
    [ (if F::hasProp(Ax::canonicalRep) then Ax::canonicalRep else Ax::normalRep end_if) ],

    "characteristic" = F::characteristic,

    "degreeOverPrimeField" = 
    (if F::hasProp(Dom::AlgebraicExtension) or F::hasProp(Dom::GaloisField) then
	  if F::degreeOverPrimeField=UNKNOWN then UNKNOWN
          else degree(P)*F::degreeOverPrimeField 
	  end_if
    elif F::hasProp(Dom::IntegerMod) or F::hasProp(Dom::Rational) then
          degree(P)  
    else
	  UNKNOWN;
    end_if ), 

    "variable" = Alias,                                         
    
    "zero" = new(this, P-P),

    "one" = new(this, P^0),

    "_plus" = fun((
        map([args()],this::coerce);
        if contains(%,FAIL)=0 then
    	   new(this, _plus(op(map(%, extop, 1))))
        else FAIL
    	end_if
    )),

    "_negate" = fun(new(this, -extop(args(1),1))),

    "_subtract" = fun((map([args()],this::coerce);
           new(this, _subtract(op(map(%,extop,1)))) )),

    "iszero" = fun(iszero(extop(args(1), 1))),

    "_mult" = (if F::hasProp(Ax::systemRep) then
        fun((if args(0) = 2 then
    	    if domtype(args(2)) <> this then
    	    	if domtype(args(2)) = DOM_INT then
    	    	    this::intmult(args())
    	    	elif testtype(args(2), F) then
    	    	    new(this, multcoeffs(extop(args(1),1), F::convert(args(2))))
    	    	else
    	    	    (domtype(args(2)))::_mult(args())
    	    	end_if
    	    elif domtype(args(1)) <> this then
    	    	if domtype(args(1)) = DOM_INT then
    	    	    this::intmult(args(2), args(1))
    	    	elif testtype(args(1), F) then
    	    	    new(this, multcoeffs(extop(args(2),1), F::convert(args(1))))
    	    	else
    	    	    FAIL
    	    	end_if
    	    else
    	        new(this, divide(extop(args(1),1) * extop(args(2),1), P, Rem))
    	    end_if
    	elif args(0) = 1 then
    	    args(1)
    	else
    	    _mult(args(1..(args(0) div 2)));
    	    _mult(args(((args(0) div 2) + 1)..args(0)));
	    if (domtype(%2))::_mult <> FAIL then (domtype(%2))::_mult(%2, %1)
	    elif (domtype(%1))::_mult <> FAIL then (domtype(%1))::_mult(%2, %1)
	    else _mult(%2, %1)
	    end_if
    	end_if))
    else
        fun((if args(0) = 2 then
    	    if domtype(args(2)) <> this then
    	    	case domtype(args(2))
    	    	of DOM_INT do this::intmult(args()); break;
    	    	of F do new(this, multcoeffs(extop(args(1),1), args(2))); break;
    	    	otherwise (domtype(args(2)))::_mult(args());
    	    	end_case
    	    elif domtype(args(1)) <> this then
    	    	case domtype(args(1))
    	    	of DOM_INT do this::intmult(args(2), args(1)); break;
    	    	of F do new(this, multcoeffs(extop(args(2),1), args(1))); break;
    	    	otherwise FAIL;
    	    	end_case
    	    else
    	        new(this, divide(extop(args(1),1) * extop(args(2),1), P, Rem))
    	    end_if
    	elif args(0) = 1 then
    	    args(1)
    	else
    	    _mult(args(1..(args(0) div 2)));
    	    _mult(args(((args(0) div 2) + 1)..args(0)));
	    if (domtype(%2))::_mult <> FAIL then (domtype(%2))::_mult(%2, %1)
	    elif (domtype(%1))::_mult <> FAIL then (domtype(%1))::_mult(%2, %1)
	    else _mult(%2, %1)
	    end_if
    	end_if))
    end_if),
    
    "intmult" = fun(new(this, multcoeffs(extop(args(1),1), args(2)))),

    "_invert" = proc(c)
	local q,c1,c2,d,d1,d2,r,r1,r2,l;
    begin
	c:= extop(c,1);
	if degree(c) = 0 then return(new(this, mapcoeffs(c, 1/id))) end_if;
	l:= 1/lcoeff(c);
	c:= multcoeffs(c, l);
	c1:= extop(this::one,1);
	d1:= extop(this::zero,1);
	c2:= d1;
	d2:= c1;
	d:= P;
	while not iszero(d) do
	    q:= divide(c,d); r:= op(q,2); q:= op(q,1);
            r1:= c1-q*d1; r2:= c2-q*d2;
            c:= d; c1:= d1; c2:= d2;
            d:= r; d1:= r1; d2:= r2;
	end_while;
        new(this, multcoeffs(c1, l / lcoeff(c)))
    end_proc,

    "conjNorm" = (if F::constructor = Dom::IntegerMod then
        fun(F::convert(resultant(P, extop(args(1),1), hold(Alias))))
    else
        fun(resultant(P,extop(args(1),1),  hold(Alias)))
    end_if),
    
    "conjTrace" = proc(a)
    local i,result,x;

    begin
    a:=extop(a,1);
    x:=poly(Alias, op(a,2..3));
    result:=coeff(a,0);
    for i from 1 to degree(P)-1 do
	a:=divide(a*x, P, hold(Rem));
	result:=result + coeff(a,Alias,i)
    end_for; 
    F::convert(result)
    end_proc,
    

    # convert -- convert expression into algebraic number
   
      The expression must be a polynomial of the same type as the polynomial P
      defining the adjoinded element. It is interpreted as a polynomial in the
      adjoined element. #
     
    "convert" =
    (if F::constructor = Dom::IntegerMod then
       proc(xx)
         local x;
       begin
         x:=xx;
         if domtype(x) = this then
           return(x)
         end_if;
         if domtype(x)=F then x:=expr(x); end_if;
   	    x:= poly(x, op(P, 2..3));
   	    if x = FAIL then
                 x:=denom(xx);
                 if has(x, this::variable) then
                   return(this::_mult(this::new(numer(xx)),
                                     this::_invert(this::new(x))))
                 end_if;
                 return(FAIL)
          end_if;
   	    new(this, divide(x, P, Rem))
        end_proc
    elif F::hasProp(Dom::Rational) then
        proc(xx) local x;
        begin
          x:=xx;
   	    if domtype(x) = this then
                return(x)
            end_if;
   	    x:= poly(x, op(P,2), Expr);
   	    if not testtype(x, DOM_POLY) then
                 x:=denom(xx);
                 if has(x, this::variable) then
                   return(this::_mult(this::new(numer(xx)),
                                     this::_invert(this::new(x))))
                 end_if;
		return(FAIL) 
	    end_if;
   	    new(this, divide(x, P, Rem))
        end_proc
    else
       proc(xx)
        local x;
                 
       begin
         x:=xx;
   	    if domtype(x) = this then
                return(x)
            end_if;
   	    x:= poly(x, op(P, 2..3));
   	    if not testtype(x, Type::PolyOf(F)) then
                 x:=denom(xx);
                 if has(x, this::variable) then
                   return(this::_mult(this::new(numer(xx)),
                                     this::_invert(this::new(x))))
                 end_if;
                 return(FAIL) end_if;
   	    new(this, divide(x, P, Rem))
        end_proc
    end_if),
   
    "convert_to" = proc(x, T) begin
        if domtype(x) = this then
	    case T
	    of this do return(x);
	    of DOM_POLY do return(extop(x,1));
	    of DOM_EXPR do return(expr(extop(x,1)));
	    end_case;
	    x:= extop(x, 1);
	    if degree(x) = 0 then
		x:= lcoeff(x);
		return((if domtype(x) = T then x else T::coerce(x) end_if))
	    end_if
        end_if;
        FAIL
    end_proc,

    "expr" = fun(expr(extop(args(1),1))),
        
    "random" = (if F = Dom::Rational then
	fun(new(this,
	    poly(
	randpoly([hold(Alias)], Expr, Degree=degree(P)-1,Terms=infinity),Expr) ))
    elif F::constructor = Dom::IntegerMod then
	fun(new(this, 
	    randpoly([hold(Alias)], op(P,3), Degree=degree(P)-1,Terms=infinity) ))
    else
	fun(new(this, 
	    randpoly([hold(Alias)], F, Degree=degree(P)-1,Terms=infinity) ))
    end_if)

):

# end of file #
