#++
Dom::DerivativeProduct(S,order) -- Monoid of derivatives with respect to some
	variables given in the list S. order specifies the used ordering.

Dom::DerivativeProduct(S,order) defines a Cat::OrderedAbelianMonoid.

Six different orderings are provided:
    LexOrder :       pure lexicographic
    RevLexOrder :    reverse lexicographic
    InvLexOrder :    inverse lexicographic
    DegreeOrder :    total degree, lexicographic
    DegRevLexOrder : total degree, reverse lexicographic
    DegInvLexOrder : total degree, inverse lexicographic

Besides the monoid operations there are the following methods implemented:
    degree :    degree(x) gives the total degree,
                degree(x,s) the degree in s
    exponents : list of all non-vanishing exponents
    variables : list of all derivatives with non-vanishing exponents
    func_call : application to elements of some domain (that domain
		must have a method diff)

As usually the operation in Dom::DerivativeProduct(S,order) is understood as a
multiplication, but the operation in a Cat::AbelianMonoid is an addition,
Dom::DerivativeProduct(S,order) is also a Cat::OrderedMonoid. _mult is defined
as _plus and one as zero.  This is a hack around the basic problem that
traditionally in computer algebra an abelian monoid is *not* a monoid,
as the operation in the former one is addition and the latter one
multiplication. Thus it is not possible to define an abelian multiplicative
monoid.


Author: Werner M. Seiler, Uni Karlsruhe
Created: August 8th, 1997
Last Update: September 5th, 1997
++#

alias(REP(x)=extop(x,1),
      DIFF_ERR=error("cannot differentiate"),
      TYPE_ERR=error("wrong type of argument")):

Dom::DerivativeProduct := DomainConstructor(

    Dom::DerivativeProduct,

    [ S, order ],

    [ dim ],

    (if args(0)<1 or args(0)>2 then error("wrong number of arguments") end_if;
     if domtype(S)=DOM_LIST then
	 dim := nops(S);
     else error("first argument must be list") end_if;

     if args(0)<2 then order := LexOrder
     elif contains({ LexOrder, RevLexOrder, InvLexOrder,
		     DegreeOrder, DegRevLexOrder, DegInvLexOrder },
                   order)=0 then error("undefined ordering") end_if;
     ),

    Dom::BaseDomain,

    [ Cat::OrderedAbelianMonoid, Cat::OrderedMonoid ],

    [ Ax::canonicalRep ],


    # ================= #
    # "local" functions #
    # ================= #

    "Sless" = fun(bool(contains(hold(S),args(1))<contains(hold(S),args(2)))),


    # sorts and remove duplicates from list #
    # needed only if S is an ordered set #
    "normalize" = proc(x)
	local i;
	begin
	    if x=[] then return(x) end_if;
	    x := sort(x,this::Sless);
	    i := 1;
	    while i<nops(x) do
		if (x[i])[1]=(x[i+1])[1] then
		    (x[i])[2] := (x[i])[2]+(x[i+1])[2];
		    x[i+1] := NIL;
		else
		    i := i+1;
		end_if;
	    end_while;
	    return(x);
	end_proc,


    # ========================= #
    # representation, in/output #
    # ========================= #

    "convert" = proc(x)
        local i,res,e,f,facts;
        begin
            case domtype(x)
            of this do return(x);
            of DOM_LIST do
                if ({op(map(args(1),domtype))} = {DOM_INT}) and
                   (nops(x)=dim) then return(new(this,x))
                else return(FAIL) end_if;
            of DOM_EXPR do
                case type(x)
                of "_mult" do
                    res := [ 0 $ dim ];
                    facts := op(x);
                    for f in facts do
                        case type(f)
                        of "_power" do
                            i := contains(hold(S),op(f,1));
                            if i=0 then return(FAIL) end_if;
                            e := op(f,2);
                            if testtype(e,Type::PosInt) then
                                res[i] := res[i]+e;
                            elif e<>0 then return(FAIL) end_if;
                            break;
                        of DOM_IDENT do
                            i := contains(hold(S),f);
                            if i>0 then res[i] := res[i]+1;
                            else return(FAIL) end_if;
                            break;
                        otherwise return(FAIL);
                        end_case;
                    end_for;
                    return(new(this,res));
                of "_power" do
                    i := contains(hold(S),op(x,1));
                    if i=0 then return(FAIL) end_if;
                    e := op(args(1),2);
                    if testtype(e,Type::PosInt) or (e=0) then
                        res := [ 0 $ dim ];
                        res[i] := e;
                        return(new(this,res));
                    else return(FAIL) end_if;
                end_case;
                return(FAIL);
            of DOM_IDENT do
                i := contains(hold(S),x);
                if i=0 then return(FAIL) end_if;
                res := [ 0 $ dim ];
                res[i] := 1;
                return(new(this,res));
            end_case;
            FAIL;
        end_proc,


    "convert_to" = proc(x,t)
	local i,k,res;
	begin
	    if domtype(x)<>this then return(FAIL) end_if;
	    case t
            of DOM_LIST do
                res := [];
                i := 1;
                for k in REP(x) do
                    if k>0 then res := append(res,[hold(S)[i],k]) end_if;
                    i := i+1;
                end_for;
                return(res)
            of this do return(x);
            end_case;
            return(FAIL);
        end_proc,


    "expr" = proc(x)
        local i,k,res;
        begin
           res := [];
           i := 1;
           for k in REP(x) do
               if k>1 then
                   res := append(res,_power(hold(D)(hold(S)[i]),k));
               elif k=1 then
                   res := append(res,hold(D)(hold(S)[i]));
               end_if;
               i := i+1;
           end_for;
           case nops(res)
           of 0 do return(1);
           of 1 do return(res[1]);
           otherwise return(_mult(op(res)));
           end_case;
        end_proc,


    # ========= #
    # orderings #
    # ========= #

    "_less" = (case order
	of LexOrder do       this::lexless; break;
	of RevLexOrder do    this::revlexless; break;
	of InvLexOrder do    this::invlexless; break;
	of DegreeOrder do    this::deglexless; break;
	of DegRevLexOrder do this::degrevlexless; break;
	of DegInvLexOrder do this::deginvlexless; break;
     end_case),


    "lexless" = proc(x1:this,x2:this)
        local i,l1,l2;
        begin
            l1 := REP(x1);
            l2 := REP(x2);
            i := 1;
            while (i<=dim) and (l1[i]=l2[i]) do i := i+1 end_while;
	    if (i>dim) then FALSE else bool(l1[i]<l2[i]) end_if;
        end_proc,


    "revlexless" = proc(x1:this,x2:this)
        local i,l1,l2;
        begin
            l1 := REP(x1);
            l2 := REP(x2);
            i := dim;
            while (i>0) and (l1[i]=l2[i]) do i := i-1 end_while;
	    if (i=0) then FALSE else bool(l1[i]>l2[i]) end_if;
        end_proc,


    "invlexless" = proc(x1:this,x2:this)
        local i,l1,l2;
        begin
            l1 := REP(x1);
            l2 := REP(x2);
            i := dim;
            while (i>0) and (l1[i]=l2[i]) do i := i-1 end_while;
	    if (i=0) then FALSE else bool(l1[i]<l2[i]) end_if;
        end_proc,


    "deglexless" = proc(x1:this,x2:this)
        local d1,d2;
        begin
            d1 := this::degree(x1);
            d2 := this::degree(x2);
            if d1<d2 then TRUE
            elif d1>d2 then FALSE
	    else this::lexless(x1,x2)
            end_if;
        end_proc,


    "degrevlexless" = proc(x1:this,x2:this)
        local d1,d2;
        begin
            d1 := this::degree(x1);
            d2 := this::degree(x2);
            if d1<d2 then TRUE
            elif d1>d2 then FALSE
	    else this::revlexless(x1,x2)
            end_if;
        end_proc,


    "deginvlexless" = proc(x1:this,x2:this)
        local d1,d2;
        begin
            d1 := this::degree(x1);
            d2 := this::degree(x2);
            if d1<d2 then TRUE
            elif d1>d2 then FALSE
	    else this::invlexless(x1,x2)
            end_if;
        end_proc,


    # =========== #
    # arithmetics #
    # =========== #

    "zero" = new(this,[ 0 $ dim ]),


    "_negate" = FAIL,


    "_plus" = fun((case args(0)
	of 0 do return(this::zero);
	of 1 do return(args(1));
	of 2 do return(new(this,zip(REP(args(1)),REP(args(2)),_plus)));
	end_case;
	args(0) div 2;
        this::_plus(this::_plus(args(1..%)),
                    this::_plus(args((%+1)..args(0)))))),


    "_mult" = this::_plus,


    "one" = this::zero,


    "isone" = this::iszero,


    # ============= #
    # miscellaneous #
    # ============= #

    "degree" =
        fun((case args(0)
             of 1 do return(_plus(op(REP(args(1)))));
             of 2 do
	         contains(hold(S),args(2));
		 if (%=0) then return(0);
		 else return(REP(args(1))[%]);
		 end_if;
             otherwise error("too many argument");
             end_case)),


    "variables" = proc()
        local i,k,res;
        begin
            res := [];
            i := 1;
            for k in REP(args(1)) do
                if k>0 then res := append(res,hold(S)[i]) end_if;
                i := i+1;
            end_for;
            res;
        end_proc,


    "exponents" = fun(select(REP(args(1)),fun(args(1)>0))),


    "func_call" = proc(Dop:this,f)
	local F,d,iz,z,i,j,k,res;
	begin
	    #if this::isone(Dop) then return(f) end_if;#
	    f := context(f);
	    F := domtype(f);
	    if F=DOM_EXPR or F=DOM_POLY then
		d := diff;
		iz := fun(bool(args(1)=0));
		z := 0;
	    elif F::hasProp<>FAIL and
	         F::hasProp(Cat::PartialDifferentialRing) then
	    	d := F::diff;
		iz := F::iszero;
		z := F::zero;
	    else DIFF_ERR end_if;

	    if testargs() then
		if d=FAIL then DIFF_ERR end_if;
	    end_if;

	    res := f;
	    i := 1;
	    for j in REP(Dop) do
		for k from 1 to j do
		    res := d(res,hold(S)[i]);
		    if iz(res) then return(z) end_if;
		end_for;
		i := i+1;
	    end_for;
	    res;
	end_proc,


    "TeX" = fun(generate::tex(this::expr(args(1)))),


    "random" =proc()
	local i,j,r1,r2,res;
	begin
	    r1 := random(1..dim);
	    r2 := random(1..5);
	    res := [0 $ dim];
	    for i from 1 to dim do
		j:=r1();
		res[j] := res[j]+r2();
	    end_for;
	    new(this,res);
	end_proc
):

unalias(REP,DIFF_ERR,TYPE_ERR):
