#++
Dom::DifferentialPolynomial(DV,R) -- differential polynomial with coefficients
        from the partial differential ring R in the differential variables
        from DV. Optionally an ordering can be given as third argument.

The implementation is based on Dom::FreeAlgebra using Dom::PowerProduct. The
implemented functions are those of a Cat::DifferentialFunctionCat.


Authors: Eduard Niess, Werner M. Seiler
Created: ??? (1996)
Last update: October 16th, 1997
++#

alias(REP(x)=extop(x,1),
      VARS(x)=extop(x,2),
      ARG_ERR=error("wrong no of args"),
      TYPE_ERR=error("wrong type of argument")):


Dom::DifferentialPolynomial := DomainConstructor(

	Dom::DifferentialPolynomial,

	[ DV, R, order ],

	[ Rone, Rzero, Risone, Riszero, Rdiff, Rcoerce, Rmult, Rindets,
	  Rdiffvars, Rconvto, Rexpr,
	  DVcoerce, PP, PPvars, PPconvto, PPdiff,
	  orders, baseflag, RDF, nn ],

	(orders := { LexOrder, RevLexOrder, InvLexOrder,
		     DegreeOrder, DegRevLexOrder, DegInvLexOrder };

	case args(0)
	of 1 do
	    R := Dom::Integer;
	    order := LexOrder;
	    break;
	of 2 do
	    if contains(orders,R) then
		order := R;
		R := Dom::Integer;
	    else
		if domtype(R)=DOM_STRING then error("unknown order") end_if;
		order := DegreeOrder;
	    end_if;
	    break;
	of 3 do break;
	otherwise ARG_ERR;
	end_case;

	if domtype(DV)<>DOM_DOMAIN or DV::hasProp=FAIL or
	   not(DV::hasProp(Cat::DifferentialVariableCat)) then
	    error("first argument must be Cat::DifferentialVariableCat");
	end_if;
	RDF := Cat::RestrictedDifferentialFunctionCat(DV,"Indep");
	if domtype(R)<>DOM_DOMAIN or R::hasProp=FAIL or
	   not(R::hasProp(Cat::PartialDifferentialRing)) then
	    error("second arg must be Cat::PartialDifferentialRing");
	end_if;
	if R::hasProp(Cat::DifferentialFunctionCat) and
	   not(R::hasProp(RDF)) then
	    error("coefficients may depend only on independent variables")
	end_if;
	if contains(orders,order)=0 then error("unknown order") end_if;
	baseflag := R::hasProp(RDF);

	Rone := R::one;
	Rzero := R::zero;
        Risone := R::isone;
        Riszero := R::iszero;
        Rdiff := R::diff;
        Rcoerce := R::coerce;
	Rexpr := R::expr;
        Rmult := R::_mult;
        Rindets := R::indets;
	if baseflag then
            Rdiffvars := R::diffVars;
	    Rconvto := R::convert_to;
	end_if;

        DVcoerce := DV::coerce;

        PP := Dom::DifferentialProduct(DV,order);
        PPvars := PP::variables;
        PPconvto := PP::convert_to;
        PPdiff := PP::diff;

        nn := DV::numIndVar;
	),

	Dom::MonoidAlgebra(R,PP),

	# caution! not all polynomial operations are yet implemented! #
	[ Cat::DifferentialFunctionCat(DV), Cat::PolynomialCat(R) ],

	[(if R::hasProp(Ax::canonicalRep) then Ax::canonicalRep end_if),
	 (if R::hasProp(Ax::normalRep) then Ax::normalRep end_if)],


	# ========================= #
	# representation, in/output #
	# ========================= #

        # The representation extends the one used in Dom::FreeModule.
          The second slot is used to store the differential variables
          occuring in the differential polynomials. This allows for a
          more efficient implementation of many of the differential
          methods. But it implies that some code from Dom::FreeModule
          must be duplicated. Note that if the coefficient ring also
          belongs to Cat::DifferentialFunctionCat, the variables of the
          coefficients are not taken into account here! #

        "collectVars" = proc(l)
        local i,j,res;
        begin
            if l=[] then return(l) end_if;
            l := map(map(l,op,2),PPvars);
            res := [];
            for i in l do
                for j in i do
                    if contains(res,j)>0 then next end_if;
                    res := insert_ordered(j,res,DV::greater);
                end_for;
            end_for;
            res;
        end_proc,


        "generate1" = fun((
            if args(0)=2 then new(this,[[args(1),args(2)]],PPvars(args(2)))
            else new(this,[[args(1),args(2)]],args(3)) end_if)),


        "generate" = fun((
            if args(0)=2 then new(this,args(1),args(2))
            else new(this,args(1),this::collectVars(args(1))) end_if)),


        "multMonom" = fun((if Risone(args(1)[1]) then args(1)[2]
                           else hold(_mult)(Rexpr(args(1)[1]),args(1)[2]) 
			   end_if)),


        "convert_to" = fun((case args(2)
            of "diff" do
            of "D" do
                if this::iszero(args(1)) then return(0) end_if;
                map(REP(args(1)), 
		    fun(subsop(args(1),2=PPconvto(args(1)[2],args(2)))),
                    args(2));
                if nops(%)=1 then return(this::multMonom(%[1]))
                else return(hold(_plus)(op(map(%,this::multMonom)))) end_if;
            otherwise return(this::moduleConvertTo(args()));
            end_case)),


        "diffVars" = (if baseflag then
            proc(dp:this)
            local l,i,v,res;
            begin
                if this::iszero(dp) then return([]) end_if;
                l := map(this::coeff(dp),Rdiffvars);
                res := l[1];
                l[1] := NIL;
                for v in l do
                    for i in v do
                        if contains(res,i)>0 then next end_if;
                        res := insert_ordered(i,res,DV::greater);
                        if nops(res)>nn then break end_if;
                    end_for;
                    if nops(res)>nn then break end_if;
                end_for;
                VARS(dp) . res;
            end_proc
        else fun(VARS(args(1))) end_if),


	"indets" = proc(dp)
	local res,pos,flag,c,cres,opts;
	begin
	    if testargs() then
		if args(0)<1 or args(0)>3 then ARG_ERR end_if;
		dp := this::convert(dp);
		if dp=FAIL then TYPE_ERR end_if;
	    end_if;

            if this::iszero(dp) then return({}) end_if;
	    if args(0)>1 then
		opts := [args(2..args(0))];
		pos := contains(opts,hold(DiffVars));
		flag := bool(pos<>0);
		if flag then opts[pos] := NIL end_if;
	    else flag := FALSE end_if;

            res := {op(VARS(dp))};
	    if not(baseflag) and flag then return(res) end_if;

	    if Rindets<>FAIL then       # indets of coefficients #
		cres := [];
		if baseflag and not(flag) then
		    for c in this::coeff(l) do
			cres := cres union Rindets(c,op(opts));
		    end_for;
		else
		    if flag then
			for c in this::coeff(l) do
			    cres := cres union Rindets(c,hold(DiffVar));
			end_for;
		    else
			for c in this::coeff(l) do
			    cres := cres union Rindets(c,op(opts));
			end_for;
		    end_if;
		end_if;
		res := cres union res;
	    end_if;

	    res;
	end_proc,


        # =========== #
        # arithmetics #
        # =========== #

        # check _negate, _plus, _mult !
          should mapcoeffs be modified ? #


        # ==================== #
        # differential methods #
        # ==================== #

        # we must be careful about what must be differentiated #
        # not very efficient for multiple differentiations! #
        "diff" = proc(dp)
	local res,x,vars;
	begin
	    if args(0)=1 then return(dp)
	    else vars := [args(2..args(0))] end_if;
	    if testargs() then
	        dp := this::coerce(dp);
	        if dp=FAIL then TYPE_ERR end_if;
	        vars := map(vars,fun((DVcoerce(args(1));
	                              if %=FAIL then args(1) else % end_if)));
	    end_if;

            res := dp;
            for x in vars do
                if this::iszero(res) then return(res) end_if;
                if domtype(x)<>DV then
                    res := this::diffR(res,x);
                else
                    res := this::diff2(res,x);
                    if baseflag and DVtype(x)="Indep" then
                        res := this::_plus(res,this::diffR(res,x));
                    end_if;
                end_if;
            end_for;
            res;
	end_proc,


        # "polynomial differentiation" #
        "diff2" = proc(dp,x)
        local l,flag,i,k,n,tmp,res;
        begin
            if contains(VARS(dp),x)=0 then return(this::zero) end_if;
            l := REP(dp);
            res := [ 0 $ nops(l) ];
            k := 0;
            flag := FALSE;
            for i in l do
                tmp := PPdiff(i[2],x);
                n := op(tmp,1);
                if n<>0 then
                    flag := flag or bool(n=1);
                    k := k+1;
                    res[k] := [Rmult(n,i[1]),op(tmp,2)];
                end_if;
            end_for;
            res := this::normalizeRep([op(res,1..k)]);
            if flag then this::generate(res)
            else this::generate(res,VARS(dp)) end_if;
        end_proc,


        # differentiation of coefficients wrt parameters #
        "diffR"  = fun(this::mapcoeffs(args(1),Rdiff,args(2..args(0)))),


        # ============= #
        # miscellaneous #
        # ============= #

	"subs" = proc(dp,eq)
        local res;
        begin
	    error("Sorry, not yet implemented");
	end_proc,


	"solve" = proc(dp,dv)
	local res;
	begin
	    error("Sorry, not yet implemented");
	end_proc,


	"evalp" = proc(dp)
	local i,res;
	begin
	    res := dp;
	    for i from 2 to args(0) do
		res := this::subs(res,args(i));
	    end_for;
	    res;
	end_proc,


	"mainvar" = this::leader,


	"multcoeffs" = fun(this::mapcoeffs(args(1),fun(Rmult(args(2),args(1))),
					   args(2)))

):

unalias(REP,VARS,ARG_ERR,TYPE_ERR):



