#++
Dom::DifferentialVariables -- domain of differential variables
                              (element of Cat::DifferentialVariableCat)

Depending on the arguments this domain provides two fairly different
implementations of differential variables. The first possibility is to give
as arguments two list of identifiers (DOM_IDENT). The first list will be
considered as the names of the independent variables and the second list as
the names of the dependent ones. Dom::DifferentVariables uses then repeated
index notation.

The second possibility is to give three arguments, namely identifiers used
to generate the names of the independent and dependent variables and
derivatives, respectively. Numbers given behind the two first identifiers
are interpreted as the numbers of variables. Dom::DifferentialVariables
uses now multi index notation.

Examples for the generation of domains and differential variables.
    repeated index:
	JB := Dom::DifferentialVariable([x, y, z], [u, v]);
	j1 := JB(y);                  independent variable y
	j2 := JB(u);                  dependent variable u
	j3 := JB(v([x,x,z]));         derivative v_{xxz}
    multi index:
	IJB := Dom::DifferentialVariable(x[3], u[2], p);
	ij1 := IJB(x[2]);             independent variable x_2
	ij2 := IJB(u[1]);             dependent variable u_1
	ij3 := IJB(p[2]([2,0,1]));    derivative of u_2 with respect to
				      x_1 (twice) and x_3 (once)
Other possibilities are the diff or D notation of MuPAD or the direct
entering of the representation. Output in these notation is also possible
with convert_to(jv,"diff") or convert_to(jv,"D").

The representation consists of three slots: the first contains the
type of the differential variable, the second one the upper index numbering
the independent or dependent variables, the third one the lower index
for the derivative. The form of the lower index depends on the chosen
notation; it is either a multi or a repeated index.


Authors: Eduard Niess, Werner M. Seiler, Uni Karlsruhe
Created: ??? (1996)
Last update: January 16th, 1998
++#

alias(TYPE(x)=extop(x,1),
      INDEX(x)=extop(x,2),
      DERIV_INDEX(x)=extop(x,3),
      ARG_ERR=error("wrong no of args"),
      TYPE_ERR=error("wrong type of argument")):


Dom::DifferentialVariable := DomainConstructor(

	Dom::DifferentialVariable,

	[ ivar, dvar ],

	[ nn, mm, IndexedFlag, IVar, DVar, PVar ],

	(if args(0)<2 or args(0)>3 then ARG_ERR end_if;

	# if IndexedFlag=TRUE then we use multi index notation
	  otherwise repeated index notation #
	IndexedFlag := bool((type(ivar)="_index") or (type(dvar)="_index"));

	IVar := ivar;               # list of indep. variables #
	DVar := dvar;               # list of dep. variables #
	if IndexedFlag then
	    if type(ivar)<>"_index" then IVar := ivar[1] end_if;
	    if type(dvar)<>"_index" then DVar := dvar[1] end_if;
	    if args(0)=2 then pvar := op(DVar,1) 
	    else pvar := args(3) end_if;
	    if domtype(op(IVar,1))<>DOM_IDENT or
	       domtype(op(DVar,1))<>DOM_IDENT or
	       domtype(pvar)<>DOM_IDENT then
		error("names of variables must be identifiers");
	    end_if;
	    nn := op(IVar,2);
	    mm := op(DVar,2);
	    if nn=1 then IVar := [op(IVar,1)]
	    else IVar := [op(IVar,1)[i] $ hold(i) = 1..nn] end_if;
	    if mm=1 then
	        DVar := [op(DVar,1)];
	        PVar := [pvar];
	    else
	        DVar := [op(DVar,1)[i] $ hold(i) = 1..mm];
	        PVar := [pvar[i] $ hold(i) = 1..mm];
	    end_if;
	else
	    if args(0)<>2 then ARG_ERR end_if;
	    if domtype(ivar)<>DOM_LIST then IVar := [ivar] end_if;
	    if domtype(dvar)<>DOM_LIST then DVar := [dvar] end_if;
	    for nn in ivar do
		if domtype(nn)<>DOM_IDENT and type(nn)<>"_index" then
		    error("variables must be (indexed) identifiers");
		end_if;
	    end_for;
	    for mm in dvar do
		if domtype(mm)<>DOM_IDENT and type(mm)<>"_index" then
		    error("variables must be (indexed) identifiers");
		end_if;
	    end_for;
	    nn := nops(IVar);
	    mm := nops(DVar);
	    pvar := NIL;
	end_if;

	if nn=0 or mm=0 then error("empty list of variables") end_if;
	),

	Dom::BaseDomain,

	[ Cat::DifferentialVariableCat ],

	[ Ax::canonicalRep, Ax::normalRep ],


        # ========================= #
	# representation, in/output #
	# ========================= #

	"newMulti" = (if IndexedFlag then
		fun((if args(0)=2 then new(this,args(1),args(2),[])
		     else new(this,args(1),args(2),args(3)) end_if))
	    else
		fun((if args(0)=2 then new(this,args(1),args(2),[])
		     else new(this,args(1),args(2),this::m2r(args(3))) end_if))
	    end_if),


	"newRepeated" = (if IndexedFlag then
		fun((if args(0)=2 then new(this,args(1),args(2),[])
		     else new(this,args(1),args(2),this::r2m(args(3))) end_if))
	    else
		fun((if args(0)=2 then new(this,args(1),args(2),[])
		     else new(this,args(1),args(2),args(3)) end_if))
	    end_if),


	"convert" = proc(e)
	local i,j,ind,derind,mu,pos,funname;
	begin
            if args(0)>1 then          # direct entering of representation #
		if args(0)>3 or
		   domtype(args(1))<>DOM_STRING or
		   domtype(args(2))<>DOM_INT then return(FAIL) end_if;

                if testargs() then
                    case args(1)
		    of "Const" do
                        if args(2)<0 or args(2)>1 then return(FAIL) end_if;
                        break;
		    of "Indep" do
                        if args(2)<0 or args(2)>nn then return(FAIL) end_if;
                        break;
		    of "Dep" do
                        if args(2)<0 or args(2)>mm then return(FAIL) end_if;
                        break;
		    of "Deriv" do break;
		    otherwise return(FAIL);
  		    end_case;

		    if args(0)=3 then
                        if args(1)<>"Deriv" then return(FAIL) end_if;
                        if not(testtype(args(3),Type::ListOf(DOM_INT))) then
			    return(FAIL);
                        end_if;
                    end_if;
		end_if;

		if args(0)=3 then
		    if IndexedFlag then
			if nops(args(3))<>nn then return(FAIL) end_if;
			for i in args(3) do
			    if i<0 then return(FAIL) end_if;
			end_for;
			derind := args(3)
		    else
			for i in args(3) do
			    if i<0 or i>nn then return(FAIL) end_if;
		        end_for;
			derind := sort(args(3))
		    end_if;
		else derind := [] end_if;
		return(new(this,args(1),args(2),derind));
            end_if;

	    case type(e)                            # conversion #
	    of this do return(e);
	    of DOM_INT do
		if e=0 or e=1 then return(new(this,"Const",e,[]))
		else return(FAIL) end_if;
	    of "_index" do
	    of DOM_IDENT do
		ind := contains(hold(IVar),e);
		if ind>0 then return(new(this,"Indep",ind,[])) end_if;
		ind := contains(hold(DVar),e);
		if ind>0 then return(new(this,"Dep",ind,[])) end_if;
		return(FAIL);
	    of "function" do
		if IndexedFlag then ind := contains(hold(PVar),op(e,0))
		else ind := contains(hold(DVar),op(e,0)) end_if;
		if ind=0 then return(this::convert(op(e,0))) end_if;
		derind := op(e,1);		# check for derivative #
		if nops(e)=1 and domtype(derind)=DOM_LIST then
		    if IndexedFlag then
			if nops(derind)>nn then return(FAIL) end_if;
			for i in derind do
			    if not(testtype(i,Type::NonNegInt)) then
				return(FAIL)
			    end_if;
			end_for;
		    else
			derind := [ 0 $ nops(op(e,1)) ];
			j := 0;
			for i in op(e,1) do
			    j := j+1;
			    pos := contains(hold(IVar),i);
			    if pos=0 then return(FAIL) end_if;
			    derind[j] := pos;
			end_for;
			derind := sort(derind);
		    end_if;
		    return(new(this,"Deriv",ind,derind));
		else 	# check for dependent variable with arguments #
		    ind := contains(hold(DVar),op(e,0));
		    derind := {op(e)};
		    if ind=0 or derind<>{op(IVar)} then return(FAIL) end_if;
		    return(new(this,"Dep",ind,[]));
		end_if;
	    of "D" do
		if nops(op(e))=1 then
		    if nn<>1 then return(FAIL) end_if;
		    funname := this::convert(op(e,1));
		    if funname=FAIL or TYPE(funname)<>"Dep" then
		        return(FAIL)
		    end_if;
		    return(new(this,"Deriv",INDEX(funname),[1]));
		end_if;
		if nops(op(e))<>2 then return(FAIL) end_if;
		funname := this::convert(op(e,2));
		if funname=FAIL or TYPE(funname)<>"Dep" then return(FAIL) end_if;
		ind := INDEX(funname);
		mu := op(e,1);
		if domtype(mu)<>DOM_LIST then mu := [mu] end_if;
		if IndexedFlag then
		    derind := [ 0 $ nn ];
		    for i in mu do
			if domtype(i)<>DOM_INT or i<1 or i>nn then
			    return(FAIL);
			end_if;
			derind[i] := derind[i]+1;
		    end_for;
		else
		    derind := mu;
		    for i in derind do
			if domtype(i)<>DOM_INT or i<1 or i>nn then
			    return(FAIL)
			end_if;
		    end_for;
		    derind := sort(derind);
		end_if;
		return(new(this,"Deriv",ind,derind));
	    of "diff" do
		funname := this::convert(op(e,[1,0]));
		if funname=FAIL or TYPE(funname)<>"Dep" then return(FAIL) end_if;
		ind := INDEX(funname);
		if IndexedFlag then
		    derind := [ 0 $ nn ];
		    for i in [op(e,2..nops(e))] do
			pos := contains(hold(IVar),i);
			if pos=0 then return(FAIL) end_if;
			derind[pos] := derind[pos]+1;
		    end_for;
		else
		    derind := [ 0 $ nops(e)-1 ];
		    j := 0;
		    for i in [op(e,2..nops(e))] do
		        j := j+1;
			pos := contains(hold(IVar),i);
			if pos=0 then return(FAIL) end_if;
			derind[j] := pos;
		    end_for;
		    derind := sort(derind);
		end_if;
		return(new(this,"Deriv",ind,derind));
	    otherwise return(FAIL);
	    end_case;
	end_proc,


	"convert_to" = proc(jv,T)
	local f,vars;
	begin
	    if testargs() then
		jv := this::coerce(jv);
		if jv=FAIL then return(FAIL) end_if;
	    end_if;

	    case T
	    of this do return(jv);
	    of DOM_EXPR do return(this::expr(jv));
	    of "diff" do
		case TYPE(jv)
		of "Const" do return(INDEX(jv));
		of "Indep" do return(hold(IVar)[INDEX(jv)]);
		of "Dep" do return(hold(DVar)[INDEX(jv)](op(hold(IVar))));
		of "Deriv" do
		    f := hold(DVar)[INDEX(jv)](op(hold(IVar)));
		    vars := map(this::repeatedIndex(jv),
				fun(hold(IVar)[args(1)]));
		    return(hold(diff)(f,op(vars)));
		otherwise error("internal bug");
		end_case;
	    of "D" do
		case TYPE(jv)
		of "Const" do return(INDEX(jv));
		of "Indep" do return(hold(IVar)[INDEX(jv)]);
		of "Dep" do return(hold(DVar)[INDEX(jv)]);
		of "Deriv" do
		    f := hold(DVar)[INDEX(jv)];
		    vars := this::repeatedIndex(jv);
		    return(hold(D)(vars,f));
		otherwise error("internal bug");
		end_case;
	    otherwise return(FAIL);
	    end_case;
	end_proc,


	"expr" = fun((case TYPE(args(1))
	    of "Const" do return(INDEX(args(1)));
	    of "Indep" do return(hold(IVar)[INDEX(args(1))]);
	    of "Dep" do return(hold(DVar)[INDEX(args(1))]);
	    of "Deriv" do
		if IndexedFlag then
		    return(hold(PVar)[INDEX(args(1))](DERIV_INDEX(args(1))));
		else
		    map(DERIV_INDEX(args(1)), fun(hold(IVar)[args(1)]));
		    return(hold(DVar)[INDEX(args(1))](%));
		end_if;
	    otherwise error("internal bug");
	    end_case)),


	# ************************* #
	# ***** miscellaneous ***** #
	# ************************* #

	"indVariables" = IVar,
	
	
	"depVariables" = DVar,
	

	"zero" = new(this,"Const",0,[]),


	"one" = new(this,"Const",1,[]),


	"iszero" = fun(bool(TYPE(args(1))="Const" and INDEX(args(1))=0)),


	"isone" = fun(bool(TYPE(args(1))="Const" and INDEX(args(1))=1)),


	"index" = fun(INDEX(args(1))),


	"repeatedIndex" = (if IndexedFlag then
		fun(this::m2r(DERIV_INDEX(args(1))))
	    else fun(DERIV_INDEX(args(1))) end_if),


	"multiIndex" = (if IndexedFlag then
		fun(DERIV_INDEX(args(1)))
	    else fun(this::r2m(DERIV_INDEX(args(1)))) end_if),


	"vartype" = fun(TYPE(args(1))),


	"notation" = (if IndexedFlag then "multi" else "repeated" end_if),


	"numIndVar" = nn,


	"numDepVar" = mm

):


unalias(TYPE, INDEX, DERIV_INDEX, TYPE_ERR):

