# $Date: 1998/02/03 09:05:06 $  $Author: kg $  $Revision: 1.59 $ #

#++
Dom::ExpressionField -- domain of arithmetical MuPAD expressions
		   supposed to be a field. 

Dom::ExpressionField( [ Normal [, IsZero ]] )

Normal - function used to normalize intermediate expressions,
	 must be of type Expression -> Expression
IsZero - function used to detect 0, must be of type Expression -> Boolean
	 and return TRUE iff the argument is zero

This domain assumes that the expressions occuring are valid representations
of field elements; the field is assumed to be of characteristic 0. The
function 'Normal' is used to normalize an expression, the function 'IsZero'
is used to test an expression for 0.

The call 'Dom::ExpressionField()' creates a domain which is similar to 
the domain 'Dom::ArithmeticalExpression'. In this case only the standard 
simplifications are done. Some methods are defined additionaly (see
below).

If 'IsZero' is missing then 'iszero @ Normal' is used instead. 

If a normalizing function other than 'id' is given it is assumed that
this functions returns a normal form where the zero element is uniquely
represented by the constant 0.

The elements of this domain are created by 'new' and have one operand:
the expression representing the element of the field.

Examples:

Dom::ExpressionField( normal ) creates a field of rational expressions over
the rationals where the intermediate expressions occuring are normalized
by 'normal'.

In Dom::ExpressionField( id, iszero @ normal ) the intermediate expressions
are not normalized, but 0 is detected correctly.


The following methods are defined additionaly:

conjugate(x)	-- computes the complex conjugate of x
Re(x)		-- computes the real part of x
Im(x)		-- computes the imaginary part of x
norm(x)		-- computes the norm of x 
numer(x)	-- return the numerator of x
denom(x)	-- return the denominator of x
sqrfree(x)	-- returns a sqrfree representation of x
factor(x)	-- factorize x
diff(...)	-- computes the derivative of an expression
max(...)	-- returns the maximum of the arguments
min(...)	-- returns the minimum of the arguments
++#

Dom::ExpressionField := DomainConstructor(
    Dom::ExpressionField,
    [ Normal, IsZero ],
    [ ],
    ( case args(0)
      of 0 do
	  Normal:= id;
	  IsZero:= iszero;
	  break;
      of 1 do 
	  if Normal = id then 
	      IsZero:= iszero 
	  else 
	      IsZero:= iszero @ Normal 
	  end_if; 
	  break;
      of 2 do 
          break;
      otherwise error("wrong no of args");
      end_case ),
    (if Normal = id and IsZero = iszero then Dom::ArithmeticalExpression
     else Dom::BaseDomain 
     end_if),
    [ Cat::Field, Cat::DifferentialRing ],
    [ ( if Normal = id then
            Ax::effectiveOperation("_divide"), 
 	    Ax::effectiveOperation("_mult"),
            Ax::effectiveOperation("_invert"),
	    (if IsZero = iszero then Ax::systemRep end_if)
	else Ax::normalRep 
	end_if )
    ],

    "new" = proc(x)
    begin
	if args(0) <> 1 then error("wrong no of args") end_if;
	x:= this::convert(x);
	if x = FAIL then error("illegal argument") end_if;
	x
    end_proc, 

    "characteristic" = 0,
    
    "convert" = (if not this::hasProp(Ax::systemRep) then
		    fun((
			if domtype(args(1)) = this then return(args(1)) end_if;
			(Dom::ArithmeticalExpression)::convert(args(1));
			if % = FAIL then FAIL else
	    		    Normal(%);
	    		    if % = FAIL then FAIL else new(this, %) end_if
			end_if
		    ))
		 else 
		    (Dom::ArithmeticalExpression)::convert
		 end_if),

    "length" = (if not this::hasProp(Ax::systemRep) then
		   fun(length(extop(args(1),1)))
               else length end_if),

    "expr" = (if not this::hasProp(Ax::systemRep) then
	         fun(extop(args(1),1))
	     else id end_if),

    "conjugate" = (if not this::hasProp(Ax::systemRep) then
		      fun(new( this,Normal(conjugate(extop(args(1),1))) ))
		  else conjugate end_if),

    "Re" = (if not this::hasProp(Ax::systemRep) then
	       fun(new( this,Normal(Re(extop(args(1),1))) ))
	   else Re end_if),

    "Im" = (if not this::hasProp(Ax::systemRep) then
	       fun(new( this,Normal(Im(extop(args(1),1))) ))
           else Im end_if),

    "norm" = (if not this::hasProp(Ax::systemRep) then 
	         fun(new( this,Normal(abs(extop(args(1),1))) ))
	      else abs end_if),

    "abs"  = (if not this::hasProp(Ax::systemRep) then
                 fun(new( this,Normal(abs(extop(args(1),1))) ))
              else abs end_if),

    "zero" = (if not this::hasProp(Ax::systemRep) then new(this, 0) else 0 end_if),

    "one" = (if not this::hasProp(Ax::systemRep) then new(this, 1) else 1 end_if),

    "iszero" = (if not this::hasProp(Ax::systemRep) then
	           fun( IsZero(extop(args(1),1)) ) 
	       else iszero end_if),
	
    "equal" =  (if not this::hasProp(Ax::systemRep) then
		    fun(IsZero(extop(args(1),1)-extop(args(2),1)))
		else bool @ _equal end_if),

    "_plus" =  (if not this::hasProp(Ax::systemRep) then
        proc() local p, l;
        begin
            if map({args()}, domtype) = {this} then
                new(this, Normal(_plus(op(map([args()], extop, 1)))))
            else
                l:= map([args()], this::convert);
                if (p:= contains(l, FAIL)) = 0 then
                    new(this, Normal(_plus(op(map(l, extop, 1)))))
                else
                    (args(p))::_plus(
                        new(this, Normal(_plus(map(op(l,1..p-1),extop,1))) ),args(p..args(0)) )
                end_if
             end_if
         end_proc
    else _plus end_if),

    "_negate" = (if not this::hasProp(Ax::systemRep) then
	           fun(new(this, Normal(-extop(args(1), 1))))
	       else _negate end_if),

    "max" = (if not this::hasProp(Ax::systemRep) then
		fun( new(this,max(op(map([args()],extop,1)))) )
	    end_if),

    "min" = (if not this::hasProp(Ax::systemRep) then
                fun( new(this,min(op(map([args()],extop,1)))) )
            end_if),

    "_subtract" =  (if not this::hasProp(Ax::systemRep) then
		   fun( new(this, Normal(extop(args(1),1)-extop(args(2),1))) )
	       else
	           fun( args(1)-args(2) )
	       end_if),

    "_mult" =  (if not this::hasProp(Ax::systemRep) then
        proc() local p, l;
        begin
            if map({args()}, domtype) = {this} then
                new(this, Normal(_mult(op(map([args()], extop, 1)))))
            else
                l:= map([args()], this::convert);
                if (p:= contains(l, FAIL)) = 0 then
                    new(this, Normal(_mult(op(map(l, extop, 1)))))
                else
                    (args(p))::_mult(
                        new(this, Normal(_mult(map(op(l,1..p-1),extop,1))) ),args(p..args(0)) )
                end_if
             end_if
         end_proc
    else _mult end_if),

    "_invert" = (if not this::hasProp(Ax::systemRep) then
	           fun( new(this,Normal(1/extop(args(1),1))) )
	       else _invert end_if),

    "_divide" =  (if not this::hasProp(Ax::systemRep) then 
	           fun(new(this, Normal(extop(args(1),1) / extop(args(2),1))))
	       else
	           fun( args(1) / args(2) )
	       end_if),

    "intmult" = (if not this::hasProp(Ax::systemRep) then
    	fun(new(this, Normal(extop(args(1),1) * args(2))))
    else _mult end_if),

    "_power" = (if not this::hasProp(Ax::systemRep) then
    	proc(x,n) begin
    	    if domtype(x) <> this then 
    	        x:= this::coerce(x);
    	        if x = FAIL then error("Illegal power") end_if
     	    elif domtype(n) <> this then 
    	        n:= this::coerce(n);
    	        if n = FAIL then error("Illegal power") end_if
    	    end_if;
    	    new(this, Normal(extop(args(1),1) ^ extop(args(2),1)))
    	end_proc
    else _power end_if),

    "D" = (if not this::hasProp(Ax::systemRep) then
    	fun((
    	    case args(0)
    	    of 1 do
    		new(this, Normal(D(extop(args(1),1)))); break;
    	    of 2 do
    		new(this, Normal(D(args(1), extop(args(2),1)))); break;
    	    otherwise error("wrong no of args");
    	    end_case
    	))
    else D end_if),

    "diff" = (if not this::hasProp(Ax::systemRep) then
        fun( new(this, Normal(diff(extop(args(1),1), 
        	  op(map([args(2..args(0))],extop,1))))) )
    else diff end_if),

    "normal" = (if not this::hasProp(Ax::systemRep) then
    	fun(new(this, Normal(normal(extop(args(1),1)))))
    else normal end_if),

    "simplify" = (if not this::hasProp(Ax::systemRep) then
	fun(new(this, Normal(simplify(extop(args(1),1),args(2..args(0))))))
    else simplify end_if),

    "expand" = (if not this::hasProp(Ax::systemRep) then
        fun(new(this, Normal(expand( extop(args(1),1) )) ))
    else expand end_if),

    "combine" = (if not this::hasProp(Ax::systemRep) then
        fun(new(this, Normal(combine(extop(args(1),1),args(2..args(0))))))
    else combine end_if),

    "radsimp" = (if not this::hasProp(Ax::systemRep) then
	fun(new(this, Normal(radsimp(extop(args(1),1)))))
    else radsimp end_if),

    "numer" =  (if not this::hasProp(Ax::systemRep) then
    	fun(new(this, Normal(numer(extop(args(1),1)))))
    else numer end_if),

    "denom" =  (if not this::hasProp(Ax::systemRep) then
    	fun(new(this, Normal(denom(extop(args(1),1)))))
    else denom end_if),

    "gcd" = fun((if select([args()], not iszero) = [] then this::zero else this::one end_if)),

    "lcm" = fun((if select([args()], iszero) <> [] then this::zero else this::one end_if)),

    "factor" = (if not this::hasProp(Ax::systemRep) then
		    fun( [ new(this, Factor(extop(args(1),1))) ] )
		else Factor end_if),
		
    "Factor" = (if not this::hasProp(Ax::systemRep) then
		    fun( new(this, Factor(extop(args(1),1))) )
		else Factor end_if),

    "sqrfree" = (if not this::hasProp(Ax::systemRep) then
		    proc(x) local i; begin
			if this::iszero(x) then return(this::zero) end_if;
			x:= sqrfree(extop(x,1));
			if x = FAIL then return(FAIL) end_if;
			if nops(x) = 1 then
			    x:= x[1]
			else
			    x:= hold(_mult)(x[1], hold(_power)(x[2*i], x[2*i+1])
			    			  $ i=1..(nops(x) div 2))
			end_if;
			[ new(this, x) ]
		    end_proc
		 else
		    proc(x) local i; begin
			if iszero(x) then return(0) end_if;
			x:= sqrfree(x);
			if x = FAIL then return(FAIL) end_if;
			if nops(x) = 1 then
			    x:= x[1]
			else
			    x:= hold(_mult)(x[1], hold(_power)(x[2*i], x[2*i+1])
			    			  $ i=1..(nops(x) div 2))
			end_if;
			[ x ]
		    end_proc
		 end_if),

    "randomIdent" = genident("R"),

    "random" = (if not this::hasProp(Ax::systemRep) then
        fun((
            randpoly([this::randomIdent], hold(Expr));
            if iszero(%) then 0 else
                expr(randpoly([this::randomIdent], hold(Expr))) / expr(%)
            end_if;
            new(this, Normal(%))
        ))
    end_if),

    "float" = (if not this::hasProp(Ax::systemRep) then
    	fun(new(this, Normal(
	    float(extop(args(1),1))
        ) ))
    else float end_if),

    "nops" = (if not this::hasProp(Ax::systemRep) then fun(nops(extop(args(1),1)))
    else nops end_if),

    "subs" = (if not this::hasProp(Ax::systemRep) then
    	fun(new(this, Normal(
	    subs(extop(args(1),1), args(2..args(0)))
        ) ))
    else subs end_if),

    "op" = (if not this::hasProp(Ax::systemRep) then
        proc()
            local t;
        begin
            if args(0) = 2 and has(args(2),0) then
                op(extop(args(1),1), args(2..args(0)))
            else
                t:= op(extop(args(1),1), args(2..args(0)));
                if t <> FAIL then new( this,Normal(t) ) else FAIL end_if
            end_if
        end_proc
    else op end_if),

    "subsop" = (if not this::hasProp(Ax::systemRep) then
    	fun(new(this, Normal(
	    subsop(extop(args(1),1), args(2..args(0)))
        ) ))
    else subsop end_if),

    "subsex" = (if not this::hasProp(Ax::systemRep) then
    	fun(new(this, Normal(
            subsex(extop(args(1),1), args(2..args(0)))
        ) ))
    else subsex end_if),

    "pivotSize" = (if not this::hasProp(Ax::systemRep) then
	fun(length(extop(args(1),1)))
    else length end_if)
):

# end of file #
