#++
Dom::ExteriorAlgebra(R,S) -- the exterior algebra over a vector space with
        a basis given by the ordered set S (can also be a list; the ordering
        is then given by the position in the list) and coefficients in the
        ring R.

The implementation is based on Dom::FreeModule(R,Dom::ExteriorProduct(S)) from
which all arithmetic functions and the representation are inherited.  The
representation is extended by a flag signalling whether or not an exterior form
is normalized.

Additional operations are:
    normal :       normalizes all base forms and coefficients
    isnormalized : checks whether an exterior form is normalized
    wedge :        alternative to _mult (used in output)
    variables :    all variables from S apparing in wedge products


Author: Werner M. Seiler, Uni Karlsruhe
Created: August 4th, 1997
Last update: October 12th, 1997
++#

alias(REP(x)=extop(x,1),
      NORM(x)=extop(x,2)):


Dom::ExteriorAlgebra := DomainConstructor(

    Dom::ExteriorAlgebra,

    [ R, S ],

    [ Riszero, Rone, Rplus, Rmult, Rnegate,
      EPS, EPSisnormalized, EPSnormal, EPSsign, EPSiszero, EPSwedge,
      EPSnegate, EPSequal, EPSexpr, EPSone, EPSless, EPSvars,
      remDup, merge ],

    (if domtype(R)=DOM_DOMAIN and (R::hasProp<>FAIL) and
	R::hasProp(Cat::Ring) then
         Riszero := R::iszero;
	 Rone := R::one;
         Rplus := R::_plus;
         Rmult := R::_mult;
         Rnegate := R::_negate;
     else error("first argument must be Cat::Ring") end_if;
     if (domtype(S)=DOM_LIST) or
        ((domtype(S)=DOM_DOMAIN) and (S::hasProp<>FAIL) and
	 (S::hasProp(Cat::OrderedSet))) then
         EPS := Dom::ExteriorProduct(S);
         EPSisnormalized := EPS::isnormalized;
         EPSnormal := EPS::normal;
	 EPSsign := EPS::sign;
         EPSiszero := EPS::iszero;
         EPSwedge := EPS::wedge;
         EPSnegate := EPS::_negate;
         EPSequal := EPS::equal;
	 EPSexpr := EPS::expr;
	 EPSone := EPS::one;
	 EPSless := EPS::_less;
	 EPSvars := EPS::variables;
     else error("second argument must be Cat::OrderedSet or list") end_if;

     remDup := listtools::removeDuplicates;
     merge := listtools::merge;
    ),

    Dom::FreeModule(R,EPS),

    [ Cat::LeftModule(R), Cat::Ring ],

    [ ],


    # ========================= #
    # representation, in/output #
    # ========================= #

    "swapSign" = fun((if EPSsign(args(1)[2])=-1 then
		[Rnegate(args(1)[1]),EPSnegate(args(1)[2])]
	    else args(1) end_if)),


    "generate1" = fun((if EPSisnormalized(args(2)) then
	    new(this,[this::swapSign([args(1),args(2)])],TRUE)
	else new(this,[this::swapSign([args(1),args(2)])],FALSE) end_if)),


    "generate" = fun(new(this,map(args(1),this::swapSign),FALSE)),


    "normalGenerate" = fun((new(this,args(1),TRUE))),


    "isnormalized" = fun(NORM(args(1))),


    "normal" = proc(x:this)
    local i,res;
    begin
        if NORM(x) then return(x) end_if;
        res := [];
        for i in REP(x) do
            i[2] := EPSnormal(i[2]);
            if not(EPSiszero(i[2])) then
                if EPSsign(i[2])=-1 then
                    res := append(res,[Rnegate(i[1]),EPSnegate(i[2])]);
                else res := append(res,i) end_if;
            end_if;
        end_for;
        res := sort(res,fun(EPSless(args(2)[2],args(1)[2])));
        i := 1;
        while i<nops(res) do
	    if EPSequal(res[i][2],res[i+1][2]) then
		res[i][1] := Rplus(res[i][1],res[i+1][1]);
		res[i+1] := NIL;
	    else i:=i+1 end_if
        end_while;
        this::normalGenerate(select(res,fun(not(Riszero(args(1)[1])))));
    end_proc,


    "exprMonomial" = proc(m)
    begin
	r := m[1];
	s := m[2];
	if EPSsign(s)=-1 then
	    r := Rnegate(r);
	    s := EPSnegate(s);
	end_if;
	hold(_mult)(r,EPSexpr(s));
    end_proc,


    "variables" = proc(x:this)
    local l,i,res;
    begin
        l := map(this::terms(x),EPSvars);
        if l=[] then return(l) end_if;
        res := [];
        for i in l do
            res := remDup(merge(res,i,EPSless));
        end_for;
        res;
    end_proc,


    # =========== #
    # arithmetics #
    # =========== #

    "one" = this::generate1(Rone,EPSone),


    "_mult" = fun((if args(0)=0 then this::one
        elif args(0)=1 then args(1)
        elif domtype(args(1))=this then this::wedge(args())
        else this::smult(args()) end_if)),


    "wedge" = fun((case args(0)
        of 0 do return(this::one);
        of 1 do return(args(1));
        of 2 do return(this::wedge2(args(1),args(2)));
        otherwise
            args(0) div 2;
            this::wedge(this::wedge(args(1..%)),
                        this::wedge(args(%+1..args(0))));
        end_case)),


    "wedge2" = proc(w1,w2)
    local i,l2,res;
    begin
	if domtype(w1)<>this then
	    w1 := this::convert(w1);
	    if w1=FAIL then TYPE_ERR end_if;
	end_if;
	if domtype(w2)<>this then
	    w2 := this::convert(w2);
	    if w2=FAIL then TYPE_ERR end_if;
	end_if;
        res := [];
        l2 := REP(w2);
        for i in REP(w1) do
            res := res . map(l2,fun([Rmult(i[1],args(1)[1]),
                                     EPSwedge(i[2],args(1)[2])]));
        end_for;
        res := select(res,fun(not(EPSiszero(args(1)[2]))));
        if not(R::hasProp(Ax::noZeroDivisors)) then
            res := select(res,fun(not(Riszero(args(1)[1]))));
        end_if;
        return(this::generate(res));
    end_proc,


    # this is essentially the same hack as in Dom::ExteriorProduct
      using the fact that ^ is translated by MuPAD into _power. But
      now we must be more careful, as here it could really happen
      that a power is meant! #
    "_power" =
	fun((if domtype(args(2))=DOM_INT then this::power(args(1),args(2))
	     else this::wedge(args()) end_if)),


    "power" = domains::repeatedSquaring(this,"wedge"),


    "equal" = fun((if not(NORM(args(1))) then
                       this::equal(this::normal(args(1)),args(2))
                   elif not(NORM(args(2))) then
                       this::equal(args(1),this::normal(args(2)))
                   else
                       bool(REP(args(1))=REP(args(2)))
                   end_if)),


    # ============= #
    # miscellaneous #
    # ============= #

    # Most of the following methods are copied from Dom::FreeModule. Special
      versions are need in order to keep information on the normalization
      of the argument #

    "_plus" = fun((case args(0)
	of 0 do return(this::zero);
	of 1 do return(args(1));
	of 2 do
	    if testargs() then
		this::convert(args(1));
		this::convert(args(2));
		if (%=FAIL) or (%2=FAIL) then TYPE_ERR
		else return(this::plus2(%2,%)) end_if;
	    else
		return(this::plus2(args(1),args(2)));
	    end_if;
	otherwise
	    args(0) div 2;
	    this::_plus2(this::_plus(args(1..%)),
			 this::_plus(args((%+1)..args(0))));
	end_case)),


    "plus2" = proc(x1,x2)
	local i,l1,l2,tmp,res;
	begin
	    l1 := REP(x1);
	    l2 := REP(x2);
            res := [ 0 $ nops(l1)+nops(l2) ];
	    i := 0;
            while (l1<>[]) and (l2<>[]) do
		if this::greater2(l1[1],l2[1]) then
		    i := i+1;
                    res[i] := l1[1];
                    l1[1] := NIL;
                elif this::equal2(l1[1],l2[1]) then
                    tmp := Rplus(l1[1][1],l2[1][1]);
                    if not Riszero(tmp) then
			i := i+1;
                        res[i] := [tmp,l1[1][2]];
                    end_if;
                    l1[1] := NIL;
                    l2[1] := NIL;
                else
		    i := i+1;
                    res[i] := l2[1];
                    l2[1] := NIL;
                end_if;
            end_while;
            if (l1<>[]) then res := op(res,1..i) . l1
            elif (l2<>[]) then res := op(res,1..i) . l2
	    else res := op(res,1..i)
            end_if;
	    if this::isnormalized(x1) and this::isnormalized(x2) then
		this::normalGenerate(res);
	    else this::generate(res) end_if;
	end_proc,


    "reductum" = proc(x)
	local l;
	begin
            if testargs() then
                x := this::convert(x);
                if x=FAIL then TYPE_ERR end_if;
            end_if;
	    case this::nterms(x)
	    of 0 do error("not defined for zero");
	    of 1 do return(this::zero);
	    otherwise
		l := REP(x);
		l[1] := NIL;
		if this::isnormalized(x) then return(this::normalGenerate(l));
		else return(this::generate(l)) end_if;
	    end_case;
	end_proc,


    "mapcoeffs" = fun((if this::iszero(x) then return(this::zero) end_if;
	select(map(REP(args(1)),
		   fun(subsop(args(1),
			      1=args(2)(args(1)[1],args(3..args(0))))),
		   args(2..args(0))),
	       fun(not(Riszero(args(1)[1]))));
	if this::isnormalized(args(1)) then this::normalGenerate(%)
	else this::generate(%) end_if)),


    "characteristic" = R::characteristic

):

unalias(REP,NORM):
