#++
Dom::ExteriorProduct(S) -- monoid of exterior products over the ordered
	set S of variables (S can also be a list; the ordering of the variables
	is then determined by the position in the list).

Dom::ExteriorProduct(S,order) defines a Cat::OrderedMonoid.  The representation
consists of a list of elements of S, a sign coefficient which is either -1, 0
or 1 and a flag signalling whether the list is normalized.  If the list is
normalized, the entries are sorted according to the ordering in S and no
element of S occurs more than once.  Equality etc. can be tested only after
normalization.

Besides the monoid operations there are the following methods implemented:
    normal :       normalizes an element
    isnormalized : checks whether an element is normalized
    degree :       form degree (number of factors in product)
    variables :    list of all variables
    sign :         sign of an element
    zero :         zero element
    iszero :       checks whether an element is zero
    wedge :        alternative to _mult (used in output)


Author: Werner M. Seiler, Uni Karlsruhe
Created: July 31st, 1997
Last update: Sepember 20th, 1997
++#


alias(REP(x)=extop(x,1),
      SIGN(x)=extop(x,2),
      NORM(x)=extop(x,3)):


Dom::ExteriorProduct := DomainConstructor(

    Dom::ExteriorProduct,

    [ S ],

    [ finite, Sexpr, Scoerce ],

    (if args(0)<>1 then error("wrong number of arguments") end_if;
     if domtype(S)=DOM_LIST then
	 finite := nops(S);
         if finite=0 then error("empty list") end_if;
     elif domtype(S)=DOM_DOMAIN and (S::hasProp<>FAIL) and
	  S::hasProp(Cat::OrderedSet) then
	 finite := 0;
	 Sexpr := S::expr;
	 Scoerce := S::coerce;
	 Srandom := S::random;
     else error("argument must be ordered set or list");
     end_if;
     ),

    Dom::BaseDomain,

    [ Cat::OrderedMonoid ],

    [ ],


    # ================= #
    # "local" functions #
    # ================= #

    "Sless" = (if finite>0 then
		   fun(bool(contains(hold(S),args(1))<
                            contains(hold(S),args(2))))
	       else S::_less end_if),


    # total degree, lexicographic ordering #
    "less" = proc(x1,x2)
    local l1,l2,n1,n2;
    begin
	l1 := REP(x1);
	l2 := REP(x2);
        n1 := nops(l1);
        n2 := nops(l2);
        if n1<n2 then return(TRUE)
        elif n1>n2 then return(FALSE)
        else
            while l1<>[] do
                if this::Sless(l1[1],l2[1]) then return(TRUE)
                elif l1[1]=l2[1] then
                    l1[1] := NIL;
                    l2[1] := NIL;
                else return(FALSE) end_if;
            end_while;
        end_if;
        bool(SIGN(x1)<SIGN(x2));
    end_proc,


    # checks for non-empty intersection between two lists #
    "overlap" = proc(l1,l2)
    local i;
    begin
        for i in l1 do
            if contains(l2,i)>0 then return(TRUE) end_if;
        end_for;
        FALSE;
    end_proc,

    # checks whether all elements of a list are elements of another one #
    "subset" = proc(l1,l2)
    local i;
    begin
	for i in l1 do
	    if contains(l2,i)=0 then return(FALSE) end_if;
	end_for;
	TRUE;
    end_proc,


    # ========================= #
    # representation, in/output #
    # ========================= #

    "convert" = (if finite>0 then
        fun((case domtype(args(1))
             of this do return(args(1));
             of DOM_LIST do
                 if this::subset(args(1),hold(S)) then
                     return(new(this,args(1),1,FALSE))
                 else return(FAIL) end_if;
             of DOM_IDENT do
                 if contains(hold(S),args(1))=0 then return(FAIL) end_if;
                 return(new(this,[args(1)],1,TRUE));
             of DOM_EXPR do
		case type(args(1))
		of "_power" do
		    this::convert(op(args(1),1));
		    this::convert(op(args(1),2));
		    if (%=FAIL) or (%2=FAIL) then return(FAIL)
		    else return(this::wedge(%2,%)) end_if;
		of "_mult" do
                    if this::subset([op(args(1))],hold(S)) then
                        return(new(this,[op(args(1))],1,FALSE));
		    elif (nops(args(1))=2) and (op(args(1),2)=-1) then
		    	this::convert(op(args(1),1));
		    	if %<>FAIL then return(this::_negate(%))
		    	else return(FAIL) end_if;
                    else return(FAIL) end_if;
		of "_negate" do
		    this::convert(op(args(1),1));
		    if %<>FAIL then return(this::_negate(%))
		    else return(FAIL) end_if;
		of "function" do
		    if op(args(1),0)=hold(wedge) then
			return(this::convert(hold(_mult)(op(args(1)))));
		    else return(FAIL) end_if;
		otherwise
		    if args(1)=0 then return(this::zero)
		    elif args(1)=1 then return(this::one)
		    else return(FAIL) end_if;
                end_case;
             end_case;
             FAIL))
    else
        fun((case domtype(args(1))
             of this do return(args(1));
             of DOM_LIST do
                 if testtype(args(1),Type::ListOf(S)) then
                     return(new(this,args(1),1,FALSE))
                 else return(FAIL) end_if;
             of DOM_EXPR do
		case type(args(1))
		of "_power" do
		of "_mult" do
                    map([op(args(1))],Scoerce);
		    if contains(%,FAIL)=0 then return(new(this,%,1,FALSE))
                    else return(FAIL) end_if;
		of "_negate" do
		    this::convert(op(args(1),1));
		    if %<>FAIL then return(this::_negate(%))
		    else return(FAIL) end_if;
		of "function" do
		    if op(args(1),0)=hold(wedge) then
			return(this::convert(hold(_mult)(op(args(1)))));
		    else return(FAIL) end_if;
		otherwise
		    if args(1)=0 then return(this::zero)
		    elif args(1)=1 then return(this::one)
		    else return(FAIL) end_if;
                end_case;
             otherwise
                 Scoerce(args(1));
                 if %=FAIL then return(FAIL)
                 else return(new(this,[%],1,TRUE));
	 	 end_if;
             end_case))
    end_if),


    "convert_to" =
        fun((if args(0)<>2 then error("wrong no of args") end_if;
	     if domtype(args(1))<>this then return(FAIL) end_if;
             case args(2)
             of DOM_LIST do return(REP(args(1)))
             of this do return(args(1));
             end_case;
             return(FAIL))),


    "wedgeexpr" = (if finite>0 then
	fun((if nops(args(1))=1 then args(1)[1]
	     else hold(wedge)(op(args(1))) end_if))
    else
	fun((if nops(args(1))=1 then Sexpr(args(1)[1])
	     else hold(wedge)(op(map(args(1),Sexpr))) end_if))
    end_if),


    "expr" = fun((if REP(args(1))=[] then SIGN(args(1))
		  elif SIGN(args(1))=1 then this::wedgeexpr(REP(args(1)))
		  else _negate(this::wedgeexpr(REP(args(1)))) end_if)),


    "isnormalized" = fun(NORM(args(1))),


    "normal" = proc(x:this)
	local c,l,n,i,tmp,flag;
	begin
            if NORM(x) then return(x) end_if;
            c := SIGN(x);
            if c=0 then return(this::zero) end_if;
            l := REP(x);
            n := nops(l)-1;
            flag := TRUE;

            while flag do
                flag := FALSE;
                for i from 1 to n do
                    if l[i]=l[i+1] then return(this::zero)
                    elif this::Sless(l[i+1],l[i]) then
                        flag := TRUE;
                        c := -c;
                        tmp := l[i];
                        l[i] := l[i+1];
                        l[i+1] := tmp;
                    end_if;
                end_for;
            end_while;

            new(this,l,c,TRUE);
	end_proc,


    "sign" = fun(SIGN(args(1))),


    "_less" = fun((
        if not(NORM(args(1))) then
            return(this::_less(this::normal(args(1)),args(2)))
        elif not(NORM(args(2))) then
            return(this::_less(args(1),this::normal(args(2))))
        else this::less(args(1),args(2)) end_if)),


    "equal" = fun((if not(NORM(args(1))) then
                       this::equal(this::normal(args(1)),args(2))
                   elif not(NORM(args(2))) then
                       this::equal(args(1),this::normal(args(2)))
                   else
                       bool((REP(args(1))=REP(args(2))) and
                            (SIGN(args(1))=SIGN(args(2))))
                   end_if)),

    # =========== #
    # arithmetics #
    # =========== #

    "zero" = new(this,[],0,TRUE),


    "iszero" = fun((if NORM(args(1)) then bool(SIGN(args(1))=0)
                    else this::iszero(this::normal(args(1))) end_if)),


    "one" = new(this,[],1,TRUE),


    "isone" = fun((if NORM(args(1)) then
                       bool(REP(args(1))=[] and SIGN(args(1))=1)
                   else this::isone(this::normal(args(1))) end_if)),


    "_invert" = FAIL,


    "_negate" = fun(new(this,REP(args(1)),-SIGN(args(1)),NORM(args(1)))),


    "wedge" = fun((case args(0)
        of 0 do return(this::one);
        of 1 do return(args(1));
        of 2 do
            if this::iszero(args(1)) or this::iszero(args(2)) then
                return(this::zero);
            elif this::overlap(REP(args(1)),REP(args(2))) then
                return(this::zero);
            else
                return(new(this, REP(args(1)).REP(args(2)),
                                 SIGN(args(1))*SIGN(args(2)),FALSE))
            end_if;
        otherwise
            args(0) div 2;
            this::wedge(this::wedge(args(1..%)),
                        this::wedge(args(%+1..args(0))));
        end_case)),


    "_mult" = this::wedge,


    # The following is really a hack exploiting the fact that in MuPAD
      the symbol for _power is a kind of a wedge, namely ^. This is
      used to allow for entering wedge products with ^. If this is done,
      a call to _power is generated which is "redirected" to wedge. #
    "_power" = this::wedge,


    # fast left multiplication with element of S #
    "lmult" = fun((if contains(REP(args(2)),args(1))<>0 then this::zero
	           else new(this,[args(1)].REP(args(2)), SIGN(args(2)),FALSE)
		   end_if)),


    # ============= #
    # miscellaneous #
    # ============= #

    "degree" = fun(nops(REP(args(1)))),


    "variables" = fun(REP(args(1))),


    "TeX" = fun(generate::tex(this::expr(args(1)))),


    "random" = (if finite>0 then
	proc()
	local i,j,r,res;
	begin
	    r := random(1..dim);
	    res := [];
	    for i from 1 to r() do
		j := r();
		if contains(res,hold(S)[j])=0 then
		    res := append(res,hold(S)[j])
		end_if;
	    end_for;
            if nops(res)>1 then new(this,res,1,FALSE)
            else new(this,res,1,TRUE) end_if;
	end_proc;
    elif Srandom<>FAIL then
	proc()
	local i,r,s,res;
	begin
	    r := random(1..10);
	    res := [];
	    for i from 1 to r() do
                s := Srandom();
		if contains(res,s)=0 then res := append(res,s) end_if;
	    end_for;
	    if nops(res)>1 then new(this,res,1,FALSE)
            else new(this,res,1,TRUE) end_if;
	end_proc;
    else FAIL end_if)
):

unalias(REP,NORM,SIGN):
