#++
Dom::FreeModule(R,S)  -- free module over the ring R with (finite) support
        in the ordered set S (S can also be a list; the ordering is then
	defined by the position in the list)

An element is represented as a domain element with operands:

0 - domain Dom::FreeModule(R,S)
1 - a list [[r1,s1], ...,[rn,sn]] of lists of one element of R and one
    element of S, with s1 > s2 > ... > sn

Elements are generated exclusively with the two methods generate and generate1.
The first one expects as argument a list as described above; generate1 takes as
arguments one element of R and one of S. Domains based on Dom::FreeModule which
want to extend the representation by further operands (computable from the
list) can thus simply overwrite these two methods and then still inherit all
other methods implemented here.

The provided methods are essentially those of a Cat::FiniteMonoidRing. Some
methods like _mult, _divide, convert, convert_to or expr are implemented under
different names, so that the implementation can be more easily inherited by
other domains build on Dom::FreeModule. For example, scalar multiplication is
called smult and _mult is identical to smult. If now an algebra is constructed
upon Dom::FreeModule, it will have to redefine _mult. But it can still make use
of the here implemented scalar multiplication by calling smult.


Authors: Stefan Wehmeier, Uni Paderborn
         Werner M. Seiler, Uni Karlsruhe
Created: June 30th, 1997
Last update: October 14th, 1997
++#

alias(REP(x)=extop(x,1),
      TYPE_ERR=error("wrong type of argument")):


Dom::FreeModule := DomainConstructor(

    Dom::FreeModule,

    [ R, S ],

    [ Rzero, Riszero, Rone, Risone, Rcoerce, Rexpr, Rplus, Rmult, Rnegate,
      Rdivide, Sone, Sisone, Sexpr, Sdegree, Smult,
      maxr, finite ],

    (if args(0) <> 2 then error("wrong number of arguments") end_if;
     if (domtype(R)=DOM_DOMAIN) and (R::hasProp<>FAIL) and
	R::hasProp(Cat::Rng) then
     	 Rzero := R::zero;
     	 Riszero := R::iszero;
	 Rone := R::one;
	 Risone := R::isone;
    	 Rcoerce := R::coerce;
     	 Rexpr := R::expr;
	 Rplus := R::_plus;
	 Rmult := R::_mult;
	 Rnegate := R::_negate;
	 Rdivide := R::_divide;
     else error("first argument must be Cat::Rng") end_if;
     if domtype(S)=DOM_LIST then
         finite := TRUE;
	 maxr := nops(S);
	 Sexpr := id;
	 Sone := FAIL;
	 Sisone := FAIL;
         Sdegree := fun(1);
	 Smult := FAIL;
     elif domtype(S)=DOM_DOMAIN and (S::hasProp<>FAIL) and
	  S::hasProp(Cat::OrderedSet) then
         finite := FALSE;
	 maxr := 10;        # one should check for a finite set! #
     	 Sexpr := S::expr;
     	 Sone := (if S::hasProp(Cat::Monoid) then S::one
	          elif S::hasProp(Cat::AbelianMonoid) then S::zero
	          else S::one;           # will probably be FAIL ! #
	          end_if);
     	 Sisone := (if S::hasProp(Cat::Monoid) then S::isone
	           elif S::hasProp(Cat::AbelianMonoid) then S::iszero
	           else S::isone;        # will probably be FAIL ! #
	           end_if);
	 Smult := (if S::hasProp(Cat::Monoid) then S::_mult
		   else FAIL end_if);
         Sdegree := S::degree;
     else error("second argument must be list or Cat::OrderedSet") end_if;
    ),

    Dom::BaseDomain,

    [ Cat::LeftModule(R) ],

    [ (if R::hasProp(Ax::canonicalRep) then Ax::canonicalRep end_if),
      (if R::hasProp(Ax::normalRep) then Ax::normalRep end_if) ],


    # =============== #
    # local functions #
    # =============== #

    "Srandom" = (if finite then
	     fun((random(1..nops(hold(S)));%();hold(S)[%]))
	 else S::random end_if),


    "Scoerce" = (if finite then
	     fun((if contains(hold(S),args(1))=0 then FAIL;
		  else args(1) end_if))
	 else S::coerce end_if),


    "greater2" = (if finite then
	    fun(bool(contains(hold(S),args(1)[2])<contains(hold(S),args(2)[2])))
	else
	    fun(S::_less(args(2)[2],args(1)[2]))
	end_if),


    # removes multiple occurrence of generators #
    #    x - list of lists of the form [r,e] #
    "normalizeRep" = proc(x)
    local i;
    begin
        if x=[] then return([]) end_if;
        x := sort(x,this::greater2);
        i := 1;
        while i<nops(x) do
	    if x[i][2]=x[i+1][2] then
		x[i][1] := Rplus(x[i][1],x[i+1][1]);
		x[i+1] := NIL;
	    else i:=i+1 end_if;
        end_while;
        select(x, fun(not(Riszero(args(1)[1]))));
    end_proc,


    "coerceSummand" = fun([Rcoerce(args(1)[1]),this::Scoerce(args(1)[2])]),


    # ========================= #
    # representation, in/output #
    # ========================= #

    "generate" = fun(new(this,args(1))),


    "generate1" = fun(new(this,[[args(1),args(2)]])),


    "moduleConvert" = proc(x)
    local r,s,f,res,rres,sres;
    begin
	if domtype(S)<>DOM_LIST and
	   (S::hasProp(Cat::Monoid) or S::hasProp(Cat::AbelianMonoid)) then
	    r := Rcoerce(x);
	    if r<>FAIL then
		if Riszero(r) then return(this::zero)
		else return(this::generate1(r,Sone)) end_if;
	    end_if;
	end_if;
	if Rone<>FAIL then
	    s := this::Scoerce(x);
	    if s<>FAIL then return(this::generate1(Rone,s)) end_if;
	end_if;
	case domtype(x)
	of this do return(x);
	of DOM_LIST do
	    if nops(x)=2 and (r:=Rcoerce(x[1]))<>FAIL and
			     (s:=this::Scoerce(x[2]))<>FAIL then
		if Riszero(r) then return(this::zero)
		else return(this::generate1(r,s));
		end_if;
	    elif {op(map(x,domtype))}<>{DOM_LIST} then return(FAIL)
	    elif {op(map(x,nops))}<>{2} then return(FAIL)
	    else
		x := map(x,this::coerceSummand);
		if has(x,FAIL) then return(FAIL);
		else
	 	    return(this::generate(this::normalizeRep(
			  		  map(x,this::coerceSummand))));
		end_if;
	  end_if;
	of DOM_EXPR do
	    x := expand(x);
	    case type(x)
	    of "_plus" do
		res := this::zero;
		for r in op(x) do
		    s := this::convert(r);
		    if s=FAIL then return(FAIL) end_if;
		    res := this::_plus(res,s);
		end_for;
		return(res);
	    of "_mult" do
		rres := [];
		sres := [];
		for f in op(x) do
		    r := Rcoerce(f);
		    if r<>FAIL then rres := append(rres,r);
		    else sres := append(sres,f) end_if;
		end_for;
		if rres=[] then
		    if Rone=FAIL then return(FAIL)
		    else rres := Rone end_if;
		else rres := Rmult(op(rres)) end_if;
		if nops(sres)=1 then sres := this::Scoerce(sres[1])
		else
		    s := this::Scoerce(hold(_mult)(op(sres)));
		    if s=FAIL and Smult<>FAIL then
			sres := map(sres,this::Scoerce);
			if contains(sres,FAIL)>0 then return(FAIL) end_if;
			sres := Smult(op(sres));
		    else sres := s end_if;
		    if sres=FAIL then return(FAIL) end_if;
		end_if;
		return(this::generate1(rres,sres));
	    end_case;
	of DOM_POLY do return(this::convert(expr(x)));
	of DOM_ARRAY do return(this::convert([op(x)]));
	otherwise
	    if x=0 then return(this::zero) end_if;
	    if x=1 and this::one<>FAIL then return(this::one) end_if;
	end_case;
        FAIL
    end_proc,


    "convert" = this::moduleConvert,


    "moduleConvertTo" = fun((case args(2)
	of this do return(args(1));
	of DOM_LIST do return(REP(args(1)));
	of R do
	    if (domtype(S)<>DOMLIST) and
	       (S::hasProp(Cat::Monoid) or S::hasProp(Cat::AbelianMonoid)) then
		if this::nterms(args(1))>1 then return(FAIL)
		elif this::iszero(args(1)) then return(Rzero)
		elif Sisone(REP(args(1))[1][2]) then return(REP(args(1))[1][1])
		else return(FAIL) end_if;
	    else return(FAIL) end_if;
	of S do
	    if Risone=FAIL then return(FAIL) end_if;
	    if this::nterms(args(1))=1 and Risone(REP(args(1))[1][1]) then
		return(REP(args(1))[1][2])
	    else return(FAIL) end_if;
	end_case;
	FAIL)),


    "convert_to" = this::moduleConvertTo,


    "exprMonomial" = (if Risone=FAIL then
	    if Sisone=FAIL then
	        fun(hold(_mult)(Rexpr(args(1)[1]),Sexpr(args(1)[2])))
	    else
		fun((if Sisone(args(1)[2]) then Rexpr(args(1)[1])
		     else hold(_mult)(Rexpr(args(1)[1]),Sexpr(args(1)[2]))
		     end_if))
	    end_if
	elif Sisone=FAIL then
	    fun((if Risone(args(1)[1]) then Sexpr(args(1)[2])
		 else hold(_mult)(Rexpr(args(1)[1]),Sexpr(args(1)[2])) end_if))
	else
	    fun((if Risone(args(1)[1]) then Sexpr(args(1)[2])
		 elif Sisone(args(1)[2]) then Rexpr(args(1)[1])
		 else hold(_mult)(Rexpr(args(1)[1]),Sexpr(args(1)[2])) end_if))
	end_if),


    "expr" = fun((case this::nterms(args(1))
                  of 0 do return(0);
                  of 1 do return(this::exprMonomial(REP(args(1))[1]));
                  otherwise
		      hold(_plus)(this::exprMonomial(REP(args(1))[i])
				  $ hold(i)=1..nops(REP(args(1))) );
		      #hold(_plus)(this::exprMonomial(i)
				  $ i in REP(args(1)) );#
                  end_case)),


    "equal" = fun(bool(REP(args(1))=REP(args(2)))),


    # ====================== #
    # "polynomial" functions #
    # ====================== #

    "terms" = fun((if this::iszero(args(1)) then []
                   else map(REP(args(1)),op,2)
                   end_if)),


    "coeff" = fun((if args(0)=1 then
			[op(map(REP(args(1)),op,1))]
		   elif this::Scoerce(args(2))<>FAIL then
			contains(this::terms(args(1)),this::Scoerce(args(2)));
			if %=0 then Rzero
			else op(op(REP(args(1)),%),1)
			end_if
		   else error("wrong arguments") end_if)),


    "const" = (if not(finite) and (S::hasProp(Cat::Monoid) or
                  		   S::hasProp(Cat::AbelianMonoid)) then
        fun((select(REP(args(1)),Sisone);
             if %=[] then Rzero
             else %[1][1] end_if))
    else NIL end_if),


    "lcoeff" = fun((if this::iszero(args(1)) then Rzero
 		    else op(REP(args(1)),[1,1]) end_if)),


    "lterm" = fun((if this::iszero(args(1)) then Sone
		   else op(REP(args(1)),[1,2]) end_if)),


    "lmonomial" = fun((if this::iszero(args(1)) then args(1)
                       else this::generate1(op(REP(args(1))[1])) end_if)),


    "tcoeff" = fun((this::nterms(args(1));
		    if iszero(%) then Rzero
 		    else op(REP(args(1)),[%,1]) end_if)),


    "tterm" = fun((this::nterms(args(1));
		   if iszero(%) then Sone
		   else op(REP(args(1)),[%,2]) end_if)),


    "tmonomial" = fun((this::nterms(args(1));
		       if iszero(%) then args(1)
                       else this::generate1(op(REP(args(1))[%])) end_if)),


    "nthmonomial" = fun((if testargs() then
			     if this::nterms(args(1))<args(2) or args(2)<1 then
                                 error("monomial does not exist")
			     end_if;
			 end_if;
                         this::generate1(op(REP(args(1))[args(2)])))),


    "nthterm" = fun((if testargs() then
		         if this::nterms(args(1))<args(2) or args(2)<1 then
                             error("monomial does not exist")
		         end_if;
		     end_if;
                     REP(args(1))[args(2)][2])),


    "nthcoeff" = fun((if testargs() then
			  if this::nterms(args(1))<args(2) or args(2)<1 then
                              error("monomial does not exist")
			  end_if;
		      end_if;
		      REP(args(1))[args(2)][1])),


    "reductum" = proc(x)
	local l;
	begin
            if testargs() then
                x := this::convert(x);
                if x=FAIL then TYPE_ERR end_if;
            end_if;
	    case this::nterms(x)
	    of 0 do error("not defined for zero");
	    of 1 do return(this::zero);
	    otherwise
		l := REP(x);
		l[1] := NIL;
		return(this::generate(l));
	    end_case;
	end_proc,


    "monomial" = fun(this::convert([args(1),args(2)])),


    "isMonomial" = fun(bool(this::nterms(args(1))=1)),


    "nterms"=fun(nops(REP(args(1)))),


    "degree" = (if Sdegree<>FAIL then
		    fun(max(op(map(this::terms(args(1)),Sdegree,
				   args(2..args(0))))))
		else NIL end_if),


    "degreevec" = (if Sdegreevec<>FAIL then
			fun(Sdegreevec(this::lterm(args(1))))
		   else NIL end_if),


    # =========== #
    # arithmetics #
    # =========== #

    "zero" = this::generate([]),


    "iszero" = fun(bool(REP(args(1))=[])),


    "_mult" = this::smult,


    "intmult" = fun(this::smult(args(2),args(1))),


    "smult" = proc(lambda,x)
    begin
        if testargs() then
	    if args(0)<>2 then error("wrong no of arguments") end_if;
            lambda := Rcoerce(lambda);
            if lambda=FAIL then TYPE_ERR end_if;
            x := this::convert(x);
            if x=FAIL then TYPE_ERR end_if;
        end_if;
	if Riszero(lambda) or this::iszero(x) then return(this::zero) end_if;
	mapcoeffs(x,fun(Rmult(args(2),args(1))),lambda);
    end_proc,


    "_negate" = fun((if this::iszero(args(1)) then args(1)
		     else mapcoeffs(args(1),Rnegate) end_if)),


    # operates only on the representation!
      in- and output are lists of monomials #
    "plus2" = proc(l1,l2)
	local i,tmp,res;
	begin
            res := [ 0 $ nops(l1)+nops(l2) ];
	    i := 0;
            while (l1<>[]) and (l2<>[]) do
		if this::greater2(l1[1],l2[1]) then
		    i := i+1;
                    res[i] := l1[1];
                    l1[1] := NIL;
                elif l1[1][2]=l2[1][2] then
                    tmp := Rplus(l1[1][1],l2[1][1]);
                    if not Riszero(tmp) then
			i := i+1;
                        res[i] := [tmp,l1[1][2]];
                    end_if;
                    l1[1] := NIL;
                    l2[1] := NIL;
                else
		    i := i+1;
                    res[i] := l2[1];
                    l2[1] := NIL;
                end_if;
            end_while;
            if (l1<>[]) then [op(res,1..i)] . l1
            elif (l2<>[]) then [op(res,1..i)] . l2
	    else [op(res,1..i)] end_if;
	end_proc,


    "_plus" = fun((case args(0)
	of 0 do return(this::zero);
	of 1 do return(args(1));
	of 2 do
	    if testargs() then
		this::convert(args(1));
		this::convert(args(2));
		if (%=FAIL) or (%2=FAIL) then TYPE_ERR
		else this::plus2(REP(%2),REP(%)) end_if;
	    else
		this::plus2(REP(args(1)),REP(args(2)));
	    end_if;
	    return(this::generate(%));
	otherwise
	    args(0) div 2;
	    this::_plus(this::_plus(args(1..%)),
			this::_plus(args((%+1)..args(0))));
	end_case)),


    "sdivide" = (if Rdivide<>FAIL then
	proc(x,lambda)
	begin
            if testargs() then
                x := this::convert(x);
                if x=FAIL then TYPE_ERR end_if;
                lambda := Rcoerce(lambda);
                if lambda=FAIL then TYPE_ERR end_if;
            end_if;
	    if this::iszero(x) or Risone(lambda) then
		return(x);
	    end_if;
	    this::mapcoeffs(x,Rdivide,lambda);
	end_proc;
    else NIL end_if),


    "_divide" = this::sdivide,


    "_invert" = FAIL,


    # ============= #
    # miscellaneous #
    # ============= #

    "mapcoeffs" = fun((if this::iszero(x) then return(this::zero) end_if;
	    this::generate(
		select(map(REP(args(1)),
			   fun(subsop(args(1),
				      1=args(2)(args(1)[1],args(3..args(0))))),
			   args(2..args(0))),
		       fun(not(Riszero(args(1)[1]))))))),


    "TeX" = fun(expr2text(this::expr(args(1)))),


    "random" = (if R::random<>FAIL and this::Srandom<>FAIL then
	fun((random(1..maxr);%();
	     this::generate(this::normalizeRep(
			    [[R::random(),this::Srandom()] $ hold(i)=1..%]))))
	end_if),


    "coeffRing" = R,


    "characteristic" = R::characteristic

):

unalias(REP,TYPE_ERR):
