# $Date: 1997/12/15 15:02:17 $ $Author: stefanw $ $Revision: 1.28 $ #


#++

Dom::GaloisField -- domain constructor for finite fields

Dom::GaloisField(P,n <,pol>)
or
Dom::GaloisField(q)

P               - Prime number
                  or Dom::IntegerMod(p) where p is a prime number
		  or Dom::GaloisField 
n               - integer 
pol (optional) - an irreducible polynomial of degree n over Z/pZ or over
		  the underlying Dom::GaloisField, respectively
q		- integer (= p^n)

Dom::GaloisField creates the finite field Fp^n = Fp[X]/(poly).
If poly is not given as a parameter, it is chosen at random.
An element is represented as Domain element with operands

0 -  domain Dom::GaloisField(P,n,pol)
1 -  polynomial of degree <= n-1 with coefficient ring
        -   IntMod(P)    if P integer 
        -   IntMod(p)    if P=Dom::IntegerMod(p)
        -   P            if P Dom::GaloisField 

Entries:

PrimeField - Dom::IntegerMod(p), where p is the characteristic
Matrices  - the matrices over the field
size      - the number of elements in the field

   
Methods:

frobenius - the frobenius automorphism x->x^q, where p is the size of the underlying
            Dom::IntegerMod or Dom::GaloisField, respectively
conjugates(x) - list of all conjugates of x
             (= images of x under all F-automorphisms,
	     where F is the underlying Dom::GaloisField / Dom::IntegerMod , 
	     respectively)
	     Any conjugate is only listed once even if it occurs as the
	     image of x under several automorphisms          
conjTrace(x) -  computes the trace (= sum of conjugates ) of an element x
conjNorm(x)  -  computes the norm  (= product of conjugates ) of an element x
order(x)     - computes the order (= least integer k with x^k=1 ) of an element
               x  
isSquare(x) - tests whether there exists y such that y^2=x 
randomprimitive() - returns a generator of the multiplicative Group of the field
		    chosen at random                
isBasis(l) - checks whether the operands of the list l form a basis of the field
isNormal(x) - tests whether the set of conjugates of x is a basis 
randomnormal() - returns a normal element chosen at random      
++#


Dom::GaloisField:= DomainConstructor::new(
	Dom::GaloisField,
	[P,n,pol],

	# static values #
	[ characteristic,size,PrimeField,VariablesInUse,LogTable ],
	(
	 if args(0) < 1 or args(0) > 3 then error("wrong no of args"); end_if; 
         if args(0) = 1 then
		if domtype(P) <> DOM_INT then
			error("Argument must be integer");
		end_if;
		if P<0 then
			error("Number of elements cannot be less than zero");
		end_if;
		size:=P;
		if isprime(P) then 
			n:=1
		else
			n:=numlib::ispower(P);
                	if n=FALSE then
		        	error("Number of elements must be prime power");
                	end_if;
			P:=op(n,1);

			n:=op(n,2);
		end_if;
		characteristic:=P;
		PrimeField:=Dom::IntegerMod(P);
		VariablesInUse:=[];
		
         elif domtype(P)= DOM_INT then 
		if not isprime(P) then error("P must be prime"); 
		else 
			characteristic:=P;
			size:=P^n;
			PrimeField:=Dom::IntegerMod(P);
		
                        VariablesInUse:=[];
		end_if;
	 elif domtype(P)<> DOM_DOMAIN or
	  (P::constructor <> Dom::IntegerMod and P::constructor <> Dom::GaloisField )
         or not P::hasProp(Cat::Field) then 
                 error("First Argument must be prime or Dom::IntegerMod(p), p prime or Dom::GaloisField");
         else 
		characteristic := P::characteristic;
         	if P::constructor = Dom::IntegerMod then
			PrimeField:=P;
			P:=characteristic;
			size:=P^n;
                        VariablesInUse:=[];
		else   # Dom::GaloisField over another Dom::GaloisField #
			PrimeField:=P::PrimeField;
			size:=(P::size)^n;
                        VariablesInUse:=P::VariablesInUse;
		end_if;
	 end_if;       
         if domtype(n) <> DOM_INT then
		error("Second argument must be integer");
	 end_if;
	 if n < 1 then 
		error("Degree over underlying Field must be >= 1 ")
	 end_if;
	 if args(0)=3 then
		userinfo(5,"Testing argument",pol);
		if domtype(pol) = DOM_EXPR or domtype(pol)= DOM_IDENT
	   	or domtype(pol) = DOM_POLY  then
			if domtype(P)=DOM_INT then                          
				pol:= poly(pol,IntMod(P));
		        elif P::constructor = Dom::IntegerMod then
                                pol:= poly(pol,IntMod(P::characteristic));
			else pol:= poly(pol,P);
			end_if;
	        else 
			error("Third argument must be polynomial");
	 	end_if;
	        if nops(op(pol,2))<> 1 then
			error("Polynomial in one variable needed");
		end_if;
                if has(VariablesInUse,op(pol,[2,1])) then 
                        error("Variable already used in underlying Dom::GaloisField");
                end_if;
		if degree(pol) <> n then
			error("Polynomial has wrong degree");
		end_if;
		if not faclib::is_irred_gf(pol) then 
			error("Polynomial not irreducible");
		end_if;
		
		VariablesInUse:=VariablesInUse.op(pol,2); 

          else  # i.e. if args(0) = 2 #
		if domtype(P)=DOM_INT then
			pol:=faclib::random_irred_gf(IntMod(P),n);
                elif P::constructor = Dom::IntegerMod then
    			pol:=faclib::random_irred_gf(IntMod(P::characteristic),n);
		else
                	pol:=faclib::random_irred_gf(P,n);
		end_if;
		userinfo(3,"Choosing random irreducible polynomial",pol);
		VariablesInUse:=VariablesInUse.op(pol,2);      

	  end_if),


	 (if domtype(P)=DOM_INT then 
           	Dom::AlgebraicExtension(Dom::IntegerMod(P),pol) 
           else
		Dom::AlgebraicExtension(P,pol)
	   end_if), 
         
         

	 [ Cat::Field,Cat::Algebra(PrimeField),Cat::VectorSpace(PrimeField),
           (if domtype(P)=DOM_DOMAIN then Cat::VectorSpace(P) end_if),
           (if domtype(P)=DOM_DOMAIN then Cat::Algebra(P) end_if) ],

         # axioms #
	 [ Ax::canonicalRep ],

	 # entries #

	 "size" = size,

	 # degreeOverPrimeField inherited from Dom::AlgebraicExtension #

	 "zero" = new(this,poly(0,op(pol,2..3))),

	 "one" = new(this,poly(1,op(pol,2..3))),

	 "iszero" = fun (  iszero(extop(args(1),1))) ,

	 "characteristic"=characteristic,

	 "Matrices"=Dom::Matrix(this),

	 "PrimeField"=PrimeField,

    "Variable"=op(pol,[2,1]),
   
    "VariablesInUse"=VariablesInUse,

	 
    "convert" = proc(x)
         
         begin
            if domtype(x)=this then
                return(x);  
	    end_if;

	    x:=(this::getSuperDomain())::convert(x); 
	    if x = FAIL then return(FAIL) end_if;
            extsubsop(x,0=this)                        
	 end_proc,      


     "convert_to" = fun
         (
	
	   ( case args(2)
	    of DOM_POLY do return(extop(args(1),1));                    
	    of this do return(args(1));
	    end_case;
      
	    FAIL )
	 ),

      # elementnumber : orders the field elements as
	  0,1, ...,P-1,X,X+1,...,X+P-1,X^2,....,
	  (P-1)X^(n-1)+(P-1)X^(n-2)+...+(P-1) # 

      "elementnumber"= fun(
	        evalp(mapcoeffs(subsop(extop(args(1),1),3=Expr),_mod,P),
		this::Variable=P)
	),


      #   expr and print are inherited from SuperDomain 
	Dom::AlgebraicExtension(....)  #

      #  _plus, _subtract, and _mult inherited from SuperDomain #

     "_power" = fun
    ((
        if domtype(args(2)) <> DOM_INT then
            error("no integer power")
        end_if;
        if iszero(args(2)) then this::one
	elif iszero(args(1)) then this::zero
        else
	        new(this, powermod(extop(args(1),1), args(2) mod (size-1), pol))
        end_if
    )),
  
      # faster than the inherited _power would be #

      #  _invert,_divide,intmult,random,TeX inherited from SuperDomain #

        
     "frobenius" = fun
     	 (
	 (if domtype(P) = DOM_DOMAIN then _power(args(1),P::size)  
     	  else _power(args(1),characteristic) end_if )
     	 ),

     "conjugates" = proc(x)
	 local result,i,a;

         begin
	     result:=[x];
	     a:=this::frobenius(x);
	     while a<>x do
		 result:=result.[a];
		 a:=this::frobenius(a);
	     end_while;
             return(result);
	 end_proc,

    /*  conjTrace inherited from AlgebraicExtension; faster than        
     	    
             "conjTrace" = proc(x)
		 local a;

         	begin
		a:=this::conjugates(x);
	        this::intmult(this::_plus(op(a)),n div nops(a));
	 	end_proc,

       would be */
 
     	   
     # conjNorm inherited from AlgebraicExtension #		     

       


	  "order"= proc(a)
	  local k,l,primes,lngth,i,j,suf,b,c,powarr,bas2l,no;

	  begin
            if a=this::zero then return(infinity); end_if;
	    if a=this::one then return(1); end_if;
            k:=size-1;
	    if isprime(k) then return(k); end_if;
	    l:=ifactor(k);
	    no:=floor(ln(k div op(l,2))/ln(2));
                       # number of powers to be computed #
	    lngth:=nops(l) div 2;
	    primes:=[op(l,2*i) $ hold(i)=1..lngth ];
	    suf:=[op(l,2*i+1) $ hold(i)=1..lngth ];
            powarr:=array(0..no);
	    powarr[0]:=a;
	    for i from 1 to no do 
		powarr[i]:=powarr[i-1]*powarr[i-1];
	    end_for;
		# now, powarr[i] = a^(2^i) for all i #
	    for i from 1 to lngth do
		repeat
			c:=k div primes[i];
			bas2l:=[];
			b:=c;
			for j from 0 to no do
			     if b mod 2=1 then bas2l:=append(bas2l,j); end_if;
			     b:=b div 2;
			     if b=0 then break; end_if;
			end_for;
			# now, bas2l contains all numbers j such that the
			  binary representation of c has a "1" at position j #
			
			b:=this::_mult(powarr[op(bas2l,j)] $ hold(j)=1..nops(bas2l));
			# this gives b=a^c #
			if b=this::one then		
		          suf[i]:=suf[i]-1;
			  k:=c;
			else 
			  break;
			end_if;
		until suf[i]=0 end_repeat;
	    end_for;
	    return(k);
	  end_proc,

	
	"isSquare"=fun

	 (
	 (if this::characteristic=2 then TRUE
	 else
	      bool(this::_power(args(1),(this::size-1) div 2) =this::one);
	 end_if)
	 ),

   	 "ln" = proc(x:this,g:this)
    	# compute log(x) w.r.t. the base g #
    	local ord,Q,gQ,gQk,lastg,i,ginv,xg,position;

    	begin
    	if iszero(g) then 
      		error("Zero cannot be base of logarithm")
    	end_if;
    	if iszero(x) then
      		return(infinity)
    	end_if;
    	userinfo(3, "Computing giant steps");
    	ord:=this::order(g);
    	Q:=ceil(sqrt(ord));
    	userinfo(5, "Size of giant step: ".expr2text(Q));
    	gQ:=g^Q;
    	gQk:=[this::one];
    	lastg:=this::one;
    	for i from 1 to Q do
       		lastg:=lastg*gQ;
        	gQk:=append(gQk, lastg);
        	if lastg=x then
            		userinfo(3, "Found logarithm in giant step");
            		return(Q*i)
        	end_if
    	end_for;
    	ginv:=g^(-1);
    	xg:=x*ginv;
    	for i from 1 to Q do
      		if (position:=contains(gQk,xg))<>0 then
         		return((position-1)*Q+i)
      		end_if;
      		xg:=xg*ginv;
    	end_for;    
    	userinfo(2, expr2text(x). " is not in the subgroup generated by ".
                expr2text(g));
    	infinity
    	end_proc,

	
    "randomprimitive"=proc()
	  local a,c,k,l,result,i,j;

	  begin
	  k:=this::size-1;
	  if isprime(k) then
		 repeat
			 result:=this::random(); 
		 until result<>(this::one) and result<>(this::zero) end_repeat;
		 return(result);
	  end_if;
	  l:=ifactor(k);
	   result:=this::one;
	  for i in [[l[2*j],l[2*j+1]] $ hold(j)=1..nops(l) div 2] do
		c:=k div i[1];
		repeat
			a:=this::random();
		until not iszero(a) and a^c<>this::one end_repeat;
		result:=result*a^(k div (i[1]^i[2]));
	   end_for;
	   return(result);
	   end_proc,
	
		

	  		

       "isBasis" = proc(x)
         
	  local M;
	  
	  begin
	    if domtype(x)<> DOM_LIST then
                 userinfo(1,"domtype of argument was",domtype(x));
		 error ("Argument must be list");
	    end_if;
	    if nops(x) <> n then return(FALSE); end_if;
	    M:=this::Matrices(map(x,this::conjugates));
	    if linalg::rank(M) = n then 
 		return(TRUE)
	    else
 	        return(FALSE);
	    end_if;
	  end_proc,
	

	"isNormal"=proc(x)
	local a,j,i,l;

	begin
	a:=this::conjugates(x);
	if nops(a)<>n then return(FALSE); end_if;
	# generate circulant #
        l:=[([op(a,j) $ hold(j)=i..n].[op(a,j) $ hold(j)=1..i-1])
			 $ hold(i)=1..n];
	return(bool(linalg::rank(this::Matrices(l))=n));
	end_proc,
	

	"randomnormal" =proc()
	local a;
	begin
	 repeat 
		a:=this::random(); 
	 until this::isNormal(a) end_repeat;
	 a;
	end_proc
     
):

# end of file #
