# domain for sets of numerical intervals, with either closed or open bounds:
  the main methods are _union and _intersect:

>> a:=Dom::IntervalSet(0..3,Open(5)..20);
>> b:=Dom::IntervalSet(Open(2)..10);
>> a union b;

                                  [0..20]

>> a intersect b;

                         [Open(2)..3, Open(5)..10]
#

Dom::IntervalSet := DomainConstructor(
# name #
    Dom::IntervalSet,
# arguments #
    [ ],
# local variables #
    [ ],
# initialisation #
    (if args(0) <> 0 then error("wrong no of args") end_if),
# super-domains #
    Dom::BaseDomain,
# categories #
    [ Cat::Field ],
# axioms #
    [ ],
# methods #
    "characteristic" = 0,
    
    # s is a set of intervals, each one being a list [a<x,x<=b,x] #
    "new" = proc(s) begin
       # enables to write Dom::IntervalSet([...],[...]) #
       if type(s)<>DOM_LIST then s:=[args()] end_if;
       if type(s)<>DOM_LIST then error("list of intervals expected") end_if;
       new(Dom::IntervalSet,s) 
     end_proc,

    "print" = proc(r) begin extop(r,1) end_proc,
  
    "_union" = proc() local l; begin
       if args(0)=0 then error("wrong number of arguments") end_if;
       l:=map([args()],extop,1);
       while nops(l)>1 do
          l[1]:=this::union(l[1],l[2]); l[2]:=NIL;
       end_while;
       new(Dom::IntervalSet,l[1])
    end_proc,

    # compares two left interval bounds (either a or Open(a)) to give the lower bound #
    "leqleft" = proc(a,b) begin
       if type(a)="function" then 
          if type(b)="function" then bool(op(a)<=op(b)) # Open(a)<=Open(b) iff a<=b #
          else bool(op(a)<b) # Open(a)<=b iff a<b #
          end_if
       else # a<=b or a<=Open(b) #
          if type(b)="function" then bool(a<=op(b)) else bool(a<=b) end_if
       end_if
    end_proc,

    # compares two right interval bounds (either a or Open(a)) to give the lower bound #
    "leqright" = proc(a,b) begin
       if type(a)="function" then 
          if type(b)="function" then bool(op(a)<=op(b)) # Open(a)<=Open(b) iff a<=b #
          else bool(op(a)<=b) # Open(a)<=b iff a<=b #
          end_if
       else # a<=b or a<=Open(b) #
          if type(b)="function" then bool(a<op(b)) else bool(a<=b) end_if
       end_if
    end_proc,

   # tells if -infinity..a and b..infinity may or not have an intersection #
    "le" = proc(a,b) begin
       if type(a)="function" then 
          if type(b)="function" then bool(op(a)<=op(b)) # Open(a)<Open(b) iff a<=b #
          else bool(op(a)<=b) # Open(a)<b iff a<=b #
          end_if
       else # a<b or a<Open(b) #
          if type(b)="function" then bool(a<=op(b)) else bool(a<b) end_if
       end_if
    end_proc,

    "maxright" = proc(a,b) begin
       if this::leqright(a,b) then b else a end_if
    end_proc,

    # checks if the right bound a joins with the left bound b (i.e. there is no hole inbetween) #
    "isjoin" = proc(a,b) begin
       if type(a)="function" then 
          if type(b)="function" then bool(op(a)>op(b)) # Open(a),Open(b) #
          else bool(op(a)>=b) # Open(a),b #
          end_if
       else # a,b or a,Open(b) #
          if type(b)="function" then bool(a>=op(b)) else bool(a>=b) end_if
       end_if
    end_proc,

    # binary union, s and t are two lists #
    "union" = proc(s,t) local u,x,y,lo,hi; begin
       if t=[] then return(s) elif s=[] then return(t) end_if;
       u:=[]; 
       while s<>[] and t<>[] do
          x:=s[1]; y:=t[1];
          if this::leqleft(op(x,1),op(y,1)) then s[1]:=NIL; lo:=op(x,1); hi:=op(x,2);
          else t[1]:=NIL; lo:=op(y,1); hi:=op(y,2);
          end_if;
          while s<>[] and t<>[] do
             x:=s[1]; y:=t[1];
             if this::leqleft(op(x,1),op(y,1)) then
                if this::isjoin(hi,op(x,1)) then s[1]:=NIL; hi:=this::maxright(hi,op(x,2)) else break end_if
             else # op(y,1)<op(x,1) #
                if this::isjoin(hi,op(y,1)) then t[1]:=NIL; hi:=this::maxright(hi,op(y,2)) else break end_if
             end_if
          end_while;
          if s<>[] and t<>[] then u:=append(u,lo..hi) end_if
       end_while;
       # either s is empty or t is empty #
       t:=s.t;
       while t<>[] do
          y:=t[1]; if this::isjoin(hi,op(y,1)) then t[1]:=NIL; hi:=this::maxright(hi,op(y,2)) else break end_if;
       end_while;
       u:=append(u,lo..hi) . t
    end_proc,

    "_intersect" = proc() local l; begin
       if args(0)=0 then error("wrong number of arguments") end_if;
       l:=map([args()],extop,1);
       while nops(l)>1 do
          l[1]:=this::intersect(l[1],l[2]); l[2]:=NIL;
       end_while;
       new(Dom::IntervalSet,l[1])
    end_proc,

    # binary intersection, s and t are two lists #
    "intersect" = proc(s,t) local u,x,y; begin
       u:=[]; 
       while s<>[] and t<>[] do
          x:=s[1]; y:=t[1];
          if this::leqleft(op(x,1),op(y,1)) then
             if this::le(op(x,2),op(y,1)) then s[1]:=NIL # no intersection with x #
             elif this::leqright(op(x,2),op(y,2)) then s[1]:=NIL; u:=append(u,op(y,1)..op(x,2))
             else t[1]:=NIL; u:=append(u,op(y,1)..op(y,2))
             end_if
          else # op(y,1)<op(x,1) #
             if this::le(op(y,2),op(x,1)) then t[1]:=NIL # no intersection with y #
             elif this::leqright(op(y,2),op(x,2)) then t[1]:=NIL; u:=append(u,op(x,1)..op(y,2))
             else s[1]:=NIL; u:=append(u,op(x,1)..op(x,2))
             end_if
          end_if
       end_while;
       u
    end_proc
)():
