# $Date: 1997/03/05 12:08:23 $ $Author: olli $ $Revision: 1.31 $ #
#++
Dom::Integer -- domain of integer numbers

An integer is represented by a system-integer (DOM_INT). There is
no DOM_EXT to represent an integer. This fact is expressed by the axiom
'Ax::systemRep'.

Methods:-
testtype(x,D)      - returns TRUE if x is of type DOM_INT
_invert(x)         - returns x if x is 1 or -1 and FAIL otherwise
_divide(x,y)       - returns x/y if y divides x and FAIL otherwise
divides(x,y)       - returns TRUE iff x divides y
euclideanDegree(x) - returns the euclidean degree of x, ie., abs(x)
unitNormal(x)      - returns the unit normal part of x, ie., abs(x)
unitNormalRep(x)   - returns the unit normal rep. of x, ie. the
		     list [abs(x), sign(x), -sign(x)]
gcd(x1,x2,...)     - returns the gcd of the xi
gcdex(x,y)         - returns g,v,w such that g = gcd(x,y) = x*v + y*w
lcm(x1,x2,...)     - returns the lcm of the xi
quo(x,y)           - returns the euclidean quotient of x and y, ie., x div y
rem(x,y)           - returns the euclidean reminder of x and y, ie., modp(x,y)
convert(x)         - returns the value of x converted to Dom::Integer
min(x,...)	   - returns the minimal argument
max(x,...)	   - returns the maximal argument
factor(x)	   - returns the factorisation of x
irreducible(x)	   - returns TRUE iff x is prime
random()	   - returns a random number in the range -999..999
random(n)	   - returns a random number between 0 and n-1
random(m..n)	   - returns a random number between m and n
++#

Dom::Integer:= DomainConstructor(

    Dom::Integer,
    [ ],
    [ ],
    NIL,
    Dom::Numerical,
    [ Cat::EuclideanDomain, Cat::FactorialDomain, Cat::DifferentialRing, Cat::OrderedSet ],
    [ Ax::canonicalRep, Ax::systemRep, Ax::canonicalOrder,
      Ax::canonicalUnitNormal, Ax::closedUnitNormals,
      Ax::effectiveOperation("_divide"),
      Ax::effectiveOperation("_mult")
    ],

    "info_str" = "domain of integer numbers",
    
    "testtype" = fun((
	if domtype(args(1)) = DOM_INT then TRUE else FAIL end_if
    )),

    "_invert" = proc()
	local p;
    begin
	p:= proc() begin FAIL end_proc;
	p(1):= 1;
	p(-1):= -1;
	p
    end_proc(),

    "_divide" = fun((
	if modp(args(1), args(2)) = 0 then (args(1) div args(2)) else FAIL end_if
    )),

    "divides" = fun(
	bool(modp(args(2), args(1)) = 0)
    ),

    "isUnit" = proc()
	local p;
    begin
	p:= proc() begin FALSE end_proc;
	p(1):= TRUE;
	p(-1):= TRUE;
	p
    end_proc(),

    "associates" = fun(
	bool(abs(args(1)) = abs(args(2)))
    ),

    "euclideanDegree" = abs,

    "unitNormal" = abs,

    "unitNormalRep" = fun((
        if args(1) = 0 then
            [ 0, 1, 1 ]
        else
	    [ abs(args(1)), sign(args(1)), sign(args(1)) ]
        end_if
    )),

    "gcd" = igcd,

    "gcdex" = igcdex,

    "lcm" = ilcm,

    "quo" = _div,

    "rem" = modp,

    "min" = min,

    "max" = max,

    "factor" = ifactor,

    "irreducible" = isprime,

    "divide" = fun(
        op([ args(1) div args(2), modp(args(1), args(2)) ])
    ),

    "convert" = fun((
       if domtype(args(1)) = DOM_INT then args(1) else FAIL end_if
    )),

    "convert_to" = fun((
	if domtype(args(1)) <> DOM_INT then FAIL else
	    case args(2)
	    of Dom::Integer do
	    of Dom::Rational do
	    of DOM_INT do
	    of Dom::Numerical do args(1); break;
	    of DOM_FLOAT do
	    of Dom::Float do float(args(1)); break;
	    otherwise FAIL;
	    end_case
	end_if
    )),
    
    "randomGen" = table((-999..999)=random(-999..999)),
    
    "random" = fun((
        if args(0) = 0 then
            this::randomGen[-999..999]()
        else
            if not contains(this::randomGen, args(1)) then
                sysassign(this::randomGen[args(1)], random(args(1)))
            end_if;
            this::randomGen[args(1)]()
        end_if
    ))

)():

