#++
Dom::LinearDifferentialFunction(DV,R) -- linear functions in the
	differential variables from DV with coefficients from the ring R

The implemented methods are those of a Cat::DifferentialFunctionCat.  The
implementation is based on Dom::FreeModule. The simplification is done by
Gaussian elimination. Additional methods are
    makeMatrix : extracts the coefficient matrix of a system
		 (list of functions)
    makeSystem : takes a coefficient matrix and transforms it back to a system

For the linearity only the dependent variables and the derivatives are
considered.  This affects especially the method "jacobian": even if the
elements of the coefficient ring may dependent on the independent variables
(i.e. we have variable coefficients), this is *not* taken into account in the
construction of the jacobian! This is necessary to avoid algebraic problems
with the matrices. Linear functions do not form a ring and thus we cannot
build matrices upon them. Jacobians and symbols are now always matrices over
the coefficient ring.


Author: Eduard Niess, Werner M. Seiler, Uni Karlsruhe
Date created: ??? (1996)
Last update: October 15th, 1997
++#

alias(REP(x)=extop(x,1),
      ARG_ER=error("wrong no of args"),
      TYPE_ERR=error("wrong type of argument")):


Dom::LinearDifferentialFunction := DomainConstructor(

	Dom::LinearDifferentialFunction,

	[ DV, R ],

	[DVone, DVzero, DVcoerce, DVisone, DViszero, DVdiff, DVindex, DVtype,
	 DVorder, DVconvto,
	 Rone, Rzero, Riszero, Risone, Rexpr, Rcoerce, Rmult, Rdiff, Rinvert,
	 Rindets, Rnegate, Rdiffvars,
	 constCoeff, swap, SMR, TyLO, TyAT, nn, mm],

	(if (args(0)<1) or (args(0)>2) then ARG_ERR end_if;
	if args(0)=1 then R := Dom::ExpressionField(normal) end_if;
	if domtype(R)<>DOM_DOMAIN or R::hasProp=FAIL or
	   not(R::hasProp(Cat::Rng)) then
	    error("second arg must be a Cat::Rng");
	end_if;
	constCoeff := not(R::hasProp(Cat::DifferentialFunctionCat(DV)));
	if not(constCoeff) and
	   not(R::hasProp(Cat::RestrictedDifferentialFunctionCat(DV,{"Indep"})))
	   then
            error("non-linear functions");
        end_if;
	if domtype(DV)<>DOM_DOMAIN or DV::hasProp=FAIL or
           not(DV::hasProp(Cat::DifferentialVariableCat)) then
	    error("first arg must be Cat::DifferentialVariableCat");
	end_if;

	DVone := DV::one;
	DVzero := DV::zero;
	DVcoerce := DV::coerce;
	DVisone := DV::isone;
	DViszero := DV::iszero;
	DVdiff := DV::totalDiff;
	DVindex := DV::index;
	DVtype := DV::vartype;
	DVorder := DV::order;
	DVconvto := DV::convert_to;

	Rone := R::one;
	Rzero := R::zero;
	Riszero := R::iszero;
	Risone := R::isone;
	Rexpr := R::expr;
	Rcoerce := R::coerce;
	Rmult := R::_mult;
	Rplus := R::_plus;
	Rinvert := R::_invert;
	Rindets := R::indets;
	Rdiffvars := R::diffVars;
	Rnegate := R::_negate;
	Rdiff := R::diff;

	swap := fun([args(1)[2],args(1)[1]]);
	SMR := Dom::SparseMatrix(DV,R,DV::greater);
	TyLO := Type::ListOf;
	TyAT := Type::AnyType;
	nn := DV::numIndVar;
	mm := DV::numDepVar;
	),

	Dom::FreeModule(R,DV),

	[ Cat::DifferentialFunctionCat(DV), Cat::LeftModule(R) ],

	[(if R::hasProp(Ax::normalRep) then Ax::normalRep end_if),
	 (if R::hasProp(Ax::canonicalRep) then Ax::canonicalRep end_if)],


	# ========================= #
	# representation, in/output #
	# ========================= #

	"convert" = proc(x)
	local r,s,f,res,rres,sres;
	begin
	    case domtype(x)
	    of this do return(x);
	    of DOM_LIST do
		if nops(x)=2 and (r:=Rcoerce(x[1]))<>FAIL and
		   (s:=DVcoerce(x[2]))<>FAIL then
		    if DVtype(s)="Indep" then return(FAIL)
		    elif Riszero(r) then return(this::zero)
		    else return(this::generate1(r,s));
		    end_if;
		elif {op(map(x,domtype))}<>{DOM_LIST} then return(FAIL)
		elif {op(map(x,nops))}<>{2} then return(FAIL)
		else
		    x := map(x,this::coerceSummand);
		    if has(x,FAIL) then return(FAIL);
		    else
		        for s in x do
		            if DVtype(s[2])="Indep" then return(FAIL) end_if;
		        end_for;
	 		return(this::generate(this::normalizeRep(x)));
		    end_if;
		end_if;
	    of DOM_ARRAY do return(this::convert([op(x)]));
	    of DOM_EXPR do
		x := expand(x);
		case type(x)
		of "_plus" do
		    res := this::zero;
		    for r in op(x) do
			s := this::convert(r);
			if s=FAIL then return(FAIL) end_if;
			res := this::_plus(res,s);
		    end_for;
		    return(res);
		of "_mult" do
		    rres := [];
		    sres := [];
		    for f in op(x) do
			r := Rcoerce(f);
			if r<>FAIL then
			    rres := append(rres,r);
			else
			    s := DVcoerce(f);
			    if s=FAIL or DVtype(s)="Indep" then
				return(FAIL)
			    end_if;
			    sres := append(sres,s);
			end_if;
		    end_for;
		    if rres=[] then rres := Rone
		    else rres := Rmult(op(rres)) end_if;
		    case nops(sres)
		    of 0 do
			sres := DVone;
			break;
		    of 1 do
			sres := sres[1];
			break;
		    otherwise return(FAIL);
		    end_case;
		    return(this::generate1(rres,sres));
		end_case;
	    otherwise
		if x=0 then return(this::zero) end_if;
		if x=1 then return(this::one) end_if;
		r := Rcoerce(x);
		if r<>FAIL then
		    if Riszero(r) then return(this::zero)
		    else return(this::generate1(r,DVone)) end_if;
		end_if;
		s := DVcoerce(x);
		if s<>FAIL then
		    if DVtype(s)="Indep" then return(FAIL) end_if;
		    return(this::generate1(Rone,s))
		end_if;
		return(FAIL);
	    end_case;
	    FAIL
	end_proc,


        "convDiff" = fun((if DVisone(args(1)[2]) then Rexpr(args(1)[2])
                elif Risone(args(1)[1]) then DVconvto(args(1)[2],args(2))
                else hold(_mult)(Rexpr(args(1)[1]),
                                 DVconvto(args(1)[2],args(2))) end_if)),


        "convert_to" = fun((case args(2)
            of "diff" do
            of "D" do
                case this::nterms(args(1))
                of 0 do return(0);
                of 1 do return(this::convDiff(REP(args(1))[1]),args(2));
                otherwise
		    hold(_plus)(this::convDiff(REP(args(1))[i],args(2))
			        $ hold(i)=1..nops(REP(args(1))) );
                end_case;
                break;
            otherwise this::moduleConvertTo(args());
            end_case)),


	"diffVars" = (if constCoeff then
	    fun((this::terms(args(1));
		 contains(%,DVone);
		 if %=0 then return(%2)
		 else [op(%2,1..%-1)] end_if))
	else
	    proc(l)
	    local pos,res1,res2,k,v;
	    begin
	        if this::iszero(l) then return([]) end_if;
	        res1 := this::terms(l);
	        pos := contains(res1,DVone);
	        if pos>0 then res1[pos] := NIL end_if;
	        res2 := [];
	        for k in map(this::coeff(l),Rdiffvars) do
		    for v in k do
		        if contains(res2,v)>0 then next end_if;
		        res2 := insert_ordered(v,res2,DV::greater);
		        if nops(res2)=nn then break end_if;
		    end_for;
		    if nops(res2)=nn then break end_if;
		end_for;
		res1 . res2;
	    end_proc
	end_if),


	"indets" = proc(l)
	local res,pos,flag,c,cres,opts;
	begin
	    if testargs() then
		if args(0)<1 or args(0)>3 then ARG_ERR end_if;
		l := this::convert(l);
		if l=FAIL then TYPE_ERR end_if;
	    end_if;

            if this::iszero(l) then return({}) end_if;
	    if args(0)>1 then
		opts := [args(2..args(0))];
		pos := contains(opts,hold(DiffVars));
		flag := bool(pos<>0);
		if flag then opts[pos] := NIL end_if;
	    else flag := FALSE end_if;

            res := this::terms(l);
	    pos := contains(res,DVone);
	    if pos<>0 then res[pos] := NIL end_if;
	    res := {op(res)};
	    if constCoeff and flag then return(res) end_if;

	    if Rindets<>FAIL then       # indets of coefficients #
		cres := [];
		if constCoeff and not(flag) then
		    for c in this::coeff(l) do
			cres := cres union Rindets(c,op(opts));
		    end_for;
		else
		    if flag then
			for c in this::coeff(l) do
			    cres := cres union Rindets(c,hold(DiffVar));
			end_for;
		    else
			for c in this::coeff(l) do
			    cres := cres union Rindets(c,op(opts));
			end_for;
		    end_if;
		end_if;
		res := cres union res;
	    end_if;

	    res;
	end_proc,


	"one" = this::generate1(Rone,DVone),


        # ==================== #
        # differential methods #
        # ==================== #

        "diff" = fun((case args(0)
            of 0 do ARG_ERR;
            of 1 do return(args(1));
            of 2 do ;
                if testargs() then
                    DVcoerce(args(2));
                    if %<>FAIL then return(this::diff2(args(1),%))
                    else return(this::diff2(args())) end_if;
                else return(this::diff2(args())) end_if;
            otherwise
                if testargs() then
                    map([args(2..args(0))],
                        fun((DVcoerce(args(1));
                             if %=FAIL then args(1) else % end_if)));
                    return(this::diffn(args(1),op(%)));
                else return(this::diffn(args())) end_if;
            end_case)),


	"diff2" = proc(l,x)
	local pos;
	begin
	    if domtype(x)<>DV then     # differentiation wrt parameter #
		if Rdiff=FAIL then return(this::zero) end_if;
		this::mapcoeffs(l,Rdiff,x);
	    elif not(constCoeff) and DVtype(x)="Indep" then
		this::mapcoeffs(l,Rdiff,x);
	    else
		l := REP(l);
		pos := contains(this::terms(l),x);
		if pos=0 then this::zero
		else print(l[pos][1]);this::generate1(l[pos][1],DVone) end_if;
	    end_if;
	end_proc,


        "diffn" = proc(l)
        local avars,ivars,dvars,tmp;
        begin
            avars := [];
            ivars := [];
            dvars := [];
            for x in [args(2..args(0))] do
                if domtype(x)<>DV  then
                    avars := append(avars,x);
                elif DVtype(x)="Indep" then
                    ivars := append(ivars,x);
                else
                    dvars := append(dvars,x);
                end_if;
            end_for;
            if constCoeff then
                if ivars<>[] then return(this::zero) end_if;
            else
                avars := avars . ivars;
            end_if;
            if Rdiff=FAIL and avars<>[] then return(this::zero) end_if;

            case nops(dvars)
            of 0 do return(this::mapcoeffs(l,Rdiff,op(avars)));
            of 1 do
                return(this::mapcoeffs(this::diff2(l,dvars[1]),
                                       Rdiff,op(avars)))
            otherwise return(this::zero);
            end_case;
        end_proc,


	"totalDiff" = (if constCoeff then
	    proc(sys,mu)
	    local i,tmp,l,nl,sysflag;
	    begin
                if args(0)=1 then return(sys) end_if;
		sysflag := bool(domtype(sys)=DOM_LIST);
                if not(sysflag) then sys := [sys] end_if;
	        if domtype(args(2))<>DOM_LIST then mu := [args(2..args(0))]
	        else mu := args(2) end_if;

	        if testargs() then
                    sys := map(sys,this::coerce);
                    if contains(sys,FAIL)<>0 then TYPE_ERR end_if;
                    for i from 1 to nops(mu) do
                        if domtype(mu[i])=DOM_INT then
                            if mu[i]<1 or mu[i]>nn then TYPE_ERR end_if;
                        else
                            tmp := DVcoerce(mu[i]);
                            if tmp=FAIL or DVtype(tmp)<>"Indep" then
                                TYPE_ERR
                            end_if;
                            mu[i] := DVindex(tmp);
                        end_if;
                    end_for;
	        end_if;

                for i from 1 to nops(sys) do
                    if this::iszero(sys[i]) then next end_if;
	            l := REP(sys[i]);
	            nl := nops(l);
	            if DVisone(l[nl][2]) then l[nl] := NIL end_if;
	            if l=[] then sys[i] := this::zero end_if;
                    sys[i] := this::generate(
                                map(l,fun(subsop(args(1),
                                              2=DVdiff(args(1)[2],args(2)))),
                                    mu));
                end_for;
                if sysflag then sys else sys[1] end_if;
            end_proc
        else
	    proc(sys,mu)
	    local i,j,tmp,l,nl;
	    begin
                if args(0)=1 then return(sys) end_if;
                if domtype(sys)<>DOM_LIST then sys := [sys] end_if;
	        if domtype(args(2))<>DOM_LIST then mu := [args(2..args(0))]
	        else mu := args(2) end_if;

	        if testargs() then
                    sys := map(sys,this::coerce);
                    if contains(sys,FAIL)<>0 then TYPE_ERR end_if;
                    for i from 1 to nops(mu) do
                        if domtype(mu[i])=DOM_INT then
                            if mu[i]<1 or mu[i]>nn then
                                TYPE_ERR
                            end_if;
                        else
                            tmp := DVcoerce(mu[i]);
                            if tmp=FAIL or DVtype(tmp)<>"Indep" then
                                TYPE_ERR
                            end_if;
                            mu[i] := DVindex(tmp);
                        end_if;
                    end_for;
	        end_if;

                for j in mu do
                    for i from 1 to nops(sys) do
                        if this::iszero(sys[i]) then next end_if;
                        res := this::mapcoeffs(sys[i],R::totalDiff,[j]);
	                l := REP(sys[i]);
	                nl := nops(l);
	                if DVisone(l[nl][2]) then l[nl] := NIL end_if;
	                if l=[] then sys[i] := res end_if;
                        sys[i] := this::_plus(res,
                             this::generate(
                                map(l,fun(subsop(args(1),
                                              2=DVdiff(args(1)[2],args(2)))),
                                    j)));
                    end_for;
                end_for;
                sys;
            end_proc
        end_if),


        # ================= #
        # matrix operations #
        # ================= #

	"jacobian" = fun((
	    if testargs() then this::checkMakeMatrix(args())
	    else this::makeMatrix(args()) end_if;
	    op(SMR::horizSplit(%,DVone),1))),


	"makeMatrix" = func((
	    if testargs() then this::checkMakeMatrix(args()) end_if;
	    map(sys, extop, 1);
	    map(%, map, swap);
	    if args(0)=1 then SMR(%) else SMR(varlist,%) end_if), sys,varlist),


	"checkMakeMatrix" = proc(sys,varlist)
	begin
	    if domtype(sys)<>DOM_LIST then
		return(this::checkMakeMatrix(map(args(),fun([args(1)]))));
	    end_if;
	    sys := map(sys,this::coerce);
	    if has(sys,FAIL) then TYPE_ERR end_if;
	    if args(0)>1 then
		varlist := map(varlist,DVcoerce);
		if contains(varlist,FAIL)<>0 then TYPE_ERR end_if;
	    end_if;
	    sys := map(sys, extop, 1);
	    sys := map(sys, map, swap);
	    if args(0)=1 then SMR(sys) else SMR(varlist,sys) end_if;
	end_proc,


        "makeSystem" = func((
            if testargs() then
                if args(0)<>1 then ARG_ERR end_if;
                if domtype(jm)<>SMR then TYPE_ERR end_if;
            end_if;
            map(SMR::rows(jm), map, swap);
            map(%, select, fun(not(Riszero(args(1)[1]))));
            map(%, this::generate)), jm),


        "symbol" = fun((
            case domtype(args(1))
            of DOM_LIST do
                map(args(1),extop,1);
                max(op(map(%,op,[1,2])));
                map(%2,select,fun(bool(DVorder(args(1)[2])=args(2))),%);
                map(%,map,swap);
                SMR(%);
                break;
            of SMR do
                max(map(SMR::allIndices(args(1)),DVorder));
                SMR::purge(args(1),fun(bool(DVorder(args(1))=args(2))),%);
                break;
            otherwise TYPE_ERR
            end_case)),

        # ============= #
        # miscellaneous #
        # ============= #

	"leader" = this::lterm,


	"freeOf" = (if constCoeff then
	    fun(bool(contains(this::terms(args(1)),args(2))=0))
	else
	    fun((if contains(ltypes,DVtype(args(2)))=0 then
		     bool(contains(this::terms(args(1)),args(2))=0)
		 else
		     _and(op(map(this::coeff(args(1)),R::freeOf,args(2))));
		 end_if))
	end_if),


	"solve" = proc(l,jv)
	local jl,c,pos;
	begin
	    if testargs() then
		if args(0)<>2 then ARG_ERR end_if;
		if domtype(l)<>this then l := this::coerce(l) end_if;
		if l=FAIL then TYPE_ERR end_if;
		if domtype(jv)<>DV then	jv := DVcoerce(jv) end_if;
		if jv=FAIL then TYPE_ERR end_if;
	    end_if;

	    l := REP(l);
	    jl := this::terms(l);
	    pos := contains(jl,jv);
	    if pos=0 then return(FAIL) end_if;
	    if nops(jl)=1 then return(this::zero) end_if;
	    c := Rinvert(l[pos][1]);
	    if c=FAIL then return(FAIL) end_if;
	    l[pos] := NIL;
	    this::generate(map(l,fun([Rnegate(Rmult(args(1)[1],args(2))),
				      args(1)[2]]),c));
	end_proc,


	"subs" = proc(l,eq)
	local jl,pos,c,jv,ex,nl;
	begin
	    if testargs() then
		if args(0)<>2 then ARG_ERR end_if;
		if type(eq)<>"_equal" then TYPE_ERR end_if;
		if domtype(l)<>this then l := this::coerce(l) end_if;
		if l=FAIL then TYPE_ERR end_if;
		jv := op(eq,1);
		if domtype(jv)<>DV then	jv := DVcoerce(jv) end_if;
		if jv=FAIL then TYPE_ERR end_if;
		ex := op(eq,2);
		if domtype(ex)<>R then
		    c := Rcoerce(ex);
		    if c=FAIL then
			c := DVcoerce(ex);
			if c=FAIL then
			    ex := this::coerce(ex);
			    if ex=FAIL then TYPE_ERR end_if;
			else ex := c end_if;
		    else ex := c end_if;
		end_if;
	    else
		jv := op(eq,1);
		ex := op(eq,2);
	    end_if;
	    if DVtype(jv)="Const" then
		error("substitution for a constant not allowed");
	    elif DVtype(jv)="Indep" then       # subst. in coeffs #
	        if constCoeff then return(l) end_if;
		return(this::mapcoeffs(l,R::subs,eq));
	    end_if;

	    jl := this::terms(l);
	    pos := contains(jl,jv);
	    if pos=0 then return(l) end_if;
	    l := REP(l);
	    c := l[pos][1];
	    l[pos] := NIL;
	    jl[pos] := NIL;

	    case domtype(ex)
	    of R do                 # substitution by ring element #
		if not(Riszero(ex)) then
		    nl := nops(l);
		    if l=[] or l[nl][2]<>DVone then
	    		l := append(l,[Rmult(c,ex),DVone]);
		    else
			c := Rplus(l[nl][1],Rmult(c,ex));
			if Riszero(c) then l[nl] := NIL
			else l[nl][1] := c end_if;
		    end_if;
		end_if;
		return(this::generate(l));
	    of DV do                # substitution by differential variable #
		if not(DViszero(ex)) then
		    pos := contains(jl,ex);
		    if pos<>0 then
			c := Rplus(l[pos][1],c);
			if Riszero(c) then l[pos] := NIL
			else l[pos][1] := c end_if;
		    else
			l := insert_ordered([c,ex],l,
					    fun(bool(args(1)[2]>args(2)[2])))
		    end_if;
		end_if;
		return(this::generate(l));
            of this do              # substitution by linear function #
		if not(this::iszero(ex)) then
		    ex := map(REP(ex),
			      fun([Rmult(args(2),args(1)[1]),args(1)[2]]),c);
		    return(this::plus2(l,ex));
		end_if;
		return(this::generate(l));
	    otherwise TYPE_ERR;
	    end_case;
	end_proc,


        # ============== #
        # simplification #
        # ============== #

	# only primitive form without dependency list, Jacobian etc. #
	"simplify" = proc(sys)
	local M,GE,newsys;
	begin
	    if testargs() then
		if domtype(sys)<>DOM_LIST then TYPE_ERR end_if;
		sys := map(sys,this::coerce);
		if contains(sys,FAIL)<>0 then TYPE_ERR end_if;
	    end_if;

	    M := this::makeMatrix(sys);
	    GE := SMR::gaussElim(M,hold(Primitive),hold(All));
	    newsys := this::makeSystem(op(GE,1));
	    return(newsys);
	end_proc,

	"autoreduce" = fun(error("sorry, not yet implemented"))

):

unalias(REP,ARG_ERR,TYPE_ERR):

