#++ ---------------- OrePoly.mu ---------------------

Author: Winfried Fakler,  University of Karlsruhe
Date Created: 16 February  1997
Last Update : 30 September 1997


Description:
This is the domain of univariate skew polynomials the so-called
Ore-polynomials. In this implementation the coefficient ring needs to be 
commutative and to be a differential ring. (The reasons for that are 
efficiency and simplicity. Maybe we will extend it later.). 
A multiplication in an Ore-Domain is defined by x*a = \sigma(a)*x+\delta(a),
where \sigma is an injective ring endomorphism and \delta a \sigma derivation
i.e. it suffice the two conditions \delta(a+b) = \delta(a)+\delta(b) and
\delta(a*b) = \sigma(a)*\delta(b)+\delta(a)*b.

Call:

 Dom::UnivariateSkewPolynomial(var, sigma, delta, < Ring >) 
 where
   var        => indeterminate of type DOM_INDET;
   sigma      => injective ring endomorphism: Ring -> Ring
   delta      => sigma-Derivation: Ring -> Ring 
   opt: Ring  => arbitrary commutative ring of the Domains-package;
                 otherwise Dom::ExpressionField(normal) is choosen
                 automatically


Methods:
 
 - used Parameter:
    ++ a,b : this Domain
    ++ p   : DOM_POLY
    ++ e   : any of {DOM_EXPR, DOM_LIST}
    ++ n   : DOM_INT
    ++ a,b = a, b are Ore-polynomials of this Category
    ++ n   = n is a nonnegative integer
    ++ e   = e is an arbitrary linear operator in var or
    ++       it's coefficients vector or a polynomial expression in var

 - new(e)
    ++ creates a Domain element.
 - leftShift(p)    (INTERNAL-function)
    ++ computes a left shift of the coefficient vector of p, which is the
    ++ same as p*poly(var,[var],Ring). 
 - rightShift(p)   (INTERNAL-function)
    ++ computes a right shift of the coefficient vector of p whereby the 
    ++ ground term is thrown away. This is the same as 
    ++ divide(p,poly(var,[var],Ring),Quo). 
 - ore_mult(a, b)
    ++ computes the multiplication a*b defined by var*a=sigma(a)*var+delta(a).
 - _mult(a,b,...)
    ++ multiplies a,b,... by the non-commutative definition for multiplication
    ++ of Ore-polynomials.
 - _power(a, n)
    ++ computes the n-th power of a by the previous defined operation _mult.
 - leftDivide(a, b)
    ++ computes q, r, such that a=b*q+r with deg(r) < deg(b) and 
    ++ returns a table of quotient=q, remainder=r.
 - rightDivide(a, b)
    ++ computes q, r, such that a=q*b+r with deg(r) < deg(b) and 
    ++ returns a table of quotient=q, remainder=r.

All Entries:
Rep, TeX, TeXCoeff, TeXTerm, TeXindet, _invert, _mult, _negate, _plus, _po\
wer, _subtract, allAxioms, allCategories, allEntries, allSuperDomains, cha\
racteristic, coeff, coeffRing, coerce, content, convert, convert_to, degre\
e, degreevec, equal, equiv, euclideanDegree, expr, getAxioms, getCategorie\
s, getSuperDomain, getTermOrder, hasProp, indets, info, intmult, isone, is\
zero, lcoeff, leftDivide, leftExactQuotient, leftExtendedEuclid, leftGcd, \
leftLcm, leftQuotient, leftRemainder, leftShift, lmonomial, lterm, mainvar\
, mapcoeffs, monic, mult, multcoeffs, negate, new, norm, nterms, nthcoeff,\
 nthmonomial, nthterm, one, ore_mult, plus, poly, pquo, print, printElem, \
printMonomial, printTerm, random, reductum, rightDivide, rightExactQuotien\
t, rightExtendedEuclid, rightGcd, rightLcm, rightQuotient, rightRemainder,\
 rightShift, ringmult, sign, subs, subsex, tcoeff, testtype, undefinedEntr\
ies, vectorize, whichEntry, zero


See: 
 -  M. Bronstein & M. Petkovsek (1993). On Ore Rings, Linear Operators 
    and Factorisation. Bericht Nr. 200  ETH Zuerich 
 -  M. Bronstein & M. Petkovsek (1996). An introduction to pseudo-linear
    algebra. Theoretical Computer Science 157 No. 1, 3-33.
 -  O. Ore (1933). Theory of non-commutative polynomials. Annals of
    Mathematics 34, 480-508.

Example:

>> ore := Dom::UnivariateSkewPolynomial(P,id,fun(diff(args(1),x))); 
>> p1 := ore(P^2+(x+1)*P+2*x);
>> p2 := ore(P+1);
>> p1::leftDivide(p1, p2);

++#


# Dependencies:

 OrePolyCat.mu, PolyExplicit.mu, UniPoly.mu

#

alias( rep(x) = extop(x, 1),
       argstest = (if map({args()}, domtype)<>{this} 
                      then error("illegal arguments") end_if)
):


sysassign(Dom::UnivariateSkewPolynomial, DomainConstructor(

  # name: #            Dom::UnivariateSkewPolynomial,

  # parameter: #       [ var, sigma, delta, R ],

  # static value: #    [ Rs, UP, coerce, hasField ],

  # initialization: #  (if iszero(args(0)) or args(0) > 4 then 
                          error("wrong no of args")
                        end_if;
                        if domtype(var) <> DOM_IDENT or
                           domtype(dvar) <> DOM_IDENT
                          then error("not a variable")
                        end_if;
                        if args(0) = 4 then
                          if R::hasProp(Dom::BaseDomain) = FAIL 
                            then error("illegal coefficient ring") end_if;
                          if not(R::hasProp(Cat::CommutativeRing) and
                                 R::hasProp(Cat::DifferentialRing) and
                                 #iszero( R::characteristic() ) and#
                                 R::hasProp(Ax::normalRep)  or
                                 R::hasProp(
                                   Dom::ExpressionField(id, iszero@normal))) 
			     then error("illegal coefficient ring") end_if
                         else R := Dom::ExpressionField(normal) 
                        end_if;
                        if R::hasProp(Dom::Rational) 
                           then Rs := {R, DOM_RAT} else Rs := {R} end_if;
                        UP := Dom::UnivariatePolynomial(var, R);
                        coerce := UP::new;
                        hasField := bool(R::hasProp(Cat::Field));
 ) ,

  # super-domain: #     Dom::UnivariatePolynomial(var, R),

  # categories: #       [ Cat::UnivariateSkewPolynomialCat(R) ],

  # axioms: #           [ Ax::normalRep, 
                         (if R::hasProp(Ax::canonicalRep) 
                           then Ax::canonicalRep end_if) ],

  # entries: #

  "new" = proc(e)
    local i;
    begin
      if args(0) <> 1 then error("wrong no of args") end_if;
      if type(e)=DOM_LIST then
        e := map(e, expr);
        e := _plus(e[i]*var^(i-1) $i=1..nops(e))
        end_if;
      poly(expr(e), [var], R);
      if % = FAIL then return(FAIL)
       else new(this, %) end_if
    end_proc, 

  "rightShift" = proc(x) local l;
     begin
       l := map(poly2list(x), fun([args(1)[1],args(1)[2]-1]) );
       if op(l, [nops(l),2])<0 then l[nops(l)] := NIL end_if;
       #if iszero(nops(l)) then l:=[[R::zero,0]] end_if;#
       poly(l, [var], R)
     end_proc,

  "leftShift" = proc(x) 
     begin
       poly(map(poly2list(x),fun([args(1)[1],args(1)[2]+1])), [var], R)
     end_proc,

  "ore_mult" = proc(a, b)
     local s, l;
     begin
       a := rep(a);
       b := rep(b);
       s := poly(R::zero,[var],R);
       while degree(a)>0 do
         s := s+multcoeffs(b,coeff(a,0));
         a := this::rightShift(a);
         b := this::leftShift(mapcoeffs(b,sigma))+mapcoeffs(b,delta)
       end_while;
       new(this, s+multcoeffs(b,coeff(a,0)))
     end_proc,

  "mult" = proc() local i, m;
     begin
       if testargs() then
          if map({args()}, domtype) <> {this} then 
             error("cannot multiply arguments") end_if;
       end_if; 
       m := args(args(0));
       for i from args(0)-1 downto 1 do  
         m := this::ore_mult(args(i), m)
       end_for;
       m
     end_proc,

  "_mult" = proc() 
      local sel, h;
      begin
        if testargs() then 
          if map({args()},domtype) minus 
             {this,op(Rs),Dom::Integer,DOM_INT} <> {}
            then  error("cannot multiply arguments") end_if
        end_if;
        sel := split([args()], testtype, Dom::Integer); 
        if iszero(nops(sel[1])) then h := (Dom::Integer)::one; 
           else h := (Dom::Integer)::_mult(op(sel[1])) end_if;
        sel := map(sel[2],fun((if domtype(args(1))=this then args(1) else
                              new(this,poly(expr(args(1)),[var],R)) end_if)));
        this::ringmult(this::mult(op(sel)), h)
      end_proc, 

 
  "_power" = proc()  
     begin
       args(2);
       if iszero(%) then this::one
       else
         if domtype(%) = DOM_INT and % > 0 then  
           this::_mult(args(1) $args(2)) 
         else error("NonNegativeInteger as exponent expected") end_if
       end_if
     end_proc,

  "leftDivide" = 
   (if hasField then
      proc(a, b)
       local q, h, da, db, inv;
       begin
         argstest;
         q := this::zero;
         da := degree(a);
         db := degree(b); 
         inv :=  R::_invert(lcoeff(b));
         while da >= db and not iszero(a) do
           # NOTE: this is not very efficient! #
           h :=UP::_mult(coerce(R::_mult(inv,lcoeff(a))),coerce(var^(da-db)));
           q := q + h;
           a := a - this::_mult(b, h);
           da := degree(a)
         end_while;
         table(quotient = q, remainder = a)
       end_proc
     end_if ),

  "rightDivide" = 
   (if hasField then
      proc(a, b)
        local q, h, da, db, inv;
        begin
          argstest;
          q := this::zero;
          da := degree(a);
          db := degree(b); 
          inv :=  R::_invert(lcoeff(b));
          while da >= db and not iszero(a) do
            # NOTE: this is not very efficient! #
            h :=UP::_mult(coerce(R::_mult(inv,lcoeff(a))),coerce(var^(da-db)));
            q := q + h;
            a := a - this::_mult(h, b);
            da := degree(a)
          end_while;
          table(remainder = a, quotient = q)
        end_proc
     end_if ),

#
 -------------------------

   Forbidden Functions 
            
 -------------------------
#
  
  "_divide" = FAIL, "D" = FAIL, "Dpoly" = FAIL, "SPolynomial" = FAIL, 
  "decompose" = FAIL, "diff" = FAIL, "dimension" = FAIL, "divex" = FAIL, 
  "divide" = FAIL,  "divides" = FAIL, "evalp" = FAIL, "factor" = FAIL, 
  "gcd" = FAIL, "gcdex" = FAIL, "groebner" = FAIL, "idealGenerator" = FAIL, 
  #"invert" = FAIL,# "irreducible" = FAIL, "lcm" = FAIL, "normalForm" = FAIL, 
  "pdivide" = FAIL, "pquo" = "FAIL", "prem" = FAIL, "quo" = FAIL, "rem" = FAIL,
  "resultant" = FAIL, "sqrfree" = FAIL,

#
 -------------------------

    Output-Procedures 
            
 -------------------------
#

  "TeXCoeff" = proc(x)
     local str, ex;
     begin
       str := R::TeX(x);
       ex  := R::expr(x);
       case type(ex)
         of "_plus" do str := " + {\\left( ".str." \\right)}"; break;
         of "_mult" do if op(ex, nops(ex)) <> -1 then str := " + {".str."}"
                        else str := " - {".substring(str, 1, strlen(str)-1)."}"
                       end_if; break;
         otherwise
           if x = R::one then str := " + " 
             elif x = -R::one then str := " - "
               elif sign(ex) <> -1 then str := " + {".str."}"
               else str := " - {".substring(str, 1, strlen(str)-1)."}"     
           end_if
       end_case;
       str
     end_proc

) ):

unalias( rep, argstest ):






