# $Date: 1997/08/07 10:54:51 $ $Author: stefanw $ $Revision: 1.2 $ #

#++
Dom::PolynomialIdeal -- domain of polynomial ideals

Dom::PolynomialIdeal(polys, unks <, ord>)

polys - list/set of polynomials
unks  - list/set of variables
ord   - (optional) ordering (LexOrder, DegreeOrder, DegInvLexOrder)
	default is DegInvLexOrder

Methods:-
basis		- returns a basis of the ideal
normal(p)	- returns a normal form of p
setbasis(l)	- defines the ideal from its basis l

Examples:
>> II:=Dom::PolynomialIdeal([x^2-x+2*y^2,x+2*y-1],[x,y]);

                              2      2
Dom::PolynomialIdeal([-1 x + x  + 2 y , x + 2 y + -1], [x, y],

DegInvLexOrder)

>> II::basis;

                                 2
                      [-4 x + 3 x  + 1, x + 2 y + -1]

>> II::normal(x^3*y^4);

                                           5
                                   4   26 y
                               -1 y  + -----
                                         9

>> II::setbasis([x,y]);

                                  [x, y]


>> II::normal(x^3*y^4);

                                     0
++#

Dom::PolynomialIdeal:= DomainConstructor(
    Dom::PolynomialIdeal,
    [ polys, unks, ord ],
    [ R, NumInd, SysRep ],
    ( case args(0) of 2 do ord:=DegInvLexOrder; of 3 do break;
      otherwise  error("wrong no of args") end_case;
      if not contains({DOM_SET,DOM_LIST},type(polys)) or nops(polys)=0 then
	error("non empty list/set of polynomials expected") end_if;
      NumInd:=nops(unks);
      R:=(if type(op(polys,1))=DOM_POLY then op(polys,[1,3])
          else Dom::Rational end_if);
      if R=hold(Expr) then R:=Dom::Rational
          elif has(R,IntMod) then R:=Dom::IntegerMod(op(R))
      end_if;
      SysRep:= bool(R::constructor <> Dom::IntegerMod and 
      		    R::hasProp(Ax::normalRep))
    ),
    Dom::BaseDomain,
    [ ( if NumInd = 1 then
	    Cat::UnivariatePolynomialCat(R)
	else
	    Cat::PolynomialCat(R)
	end_if ) ],
    [ ( if SysRep then Ax::systemRep end_if),
      ( if R::hasProp(Ax::canonicalRep) then
	    Ax::canonicalRep
	elif R::hasProp(Ax::normalRep) then
	    Ax::normalRep
	end_if )
    ],

    "convert" = proc(x) begin new(this,x) end_proc,

    "expr" = (if not SysRep then
	fun(expr(extop(args(1),1)))
    else
	expr
    end_if),

    "zero" = this::convert(R::zero),

    "one" = this::convert(R::one),

    "iszero" = (if R::constructor = Dom::IntegerMod then
	fun(iszero(extop(args(1),1)))
    elif not R::hasProp(Ax::normalRep) then
        proc(x) local c; begin
            if iszero(extop(x,1)) then return(TRUE) end_if;
            for c in [ coeff(extop(x,1)) ] do
                if not R::iszero(c) then return(FALSE) end_if
            end_for;
            TRUE
        end_proc
    else
        iszero
    end_if),

    "indets" = (if not SysRep then
	fun( op(extop(args(1),1), 2) )
    else
	fun( op(args(1), 2) )
    end_if),

    "basis" = groebner::gbasis(polys, ord),

    "normal" = proc(p) begin groebner::normalf(p, this::basis, ord) end_proc,

    "setbasis" = proc(l) begin this::basis:=l end_proc

):

