
#++ ---------------- PolyExplicit.mu ---------------------

Author: Winfried Fakler,  University of Karlsruhe
Date Created: 23 June     1996
Last Update :  9 April    1998


Description:
This domain represents polynomials  over arbitrary commutative rings. 
It is somehow a basic domain for polynomials from which one can easy
create new polynomial domains.
Most of the functions for DOM_POLY are available.

Call:

 Dom::PolynomialExplicit(Vars, < Ring >, < Order >) 
 where
   Vars       => list of indeterminates; 
                 empty list denotes arbitrary many indeterminates
   opt: Ring  => arbitrary commutative ring of the Domains-package;
                 otherwise Dom::ExpressionField(normal) is choosen
                 automatically
   opt: Order => set a term order. Currently is possible: 
                 LexOrder, DegreeOrder, DegInvLexOrder; 
                 otherwise LexOrder is choosen automatically

Methods: 

 - used Parameter:
    ++ a,b,t  : this Domain
    ++ c      : Ring
    ++ d,v,ais: DOM_LIST
    ++ t  = term  (e.g. x*y)
    ++ d  = degree vector
    ++ v  = variable list
    ++ ais= list or set of elements of this Domain

 - adaptIndets(a, b, ...)
    ++ determines the indeterminates of all arguments, extends them such that 
    ++ all have the same indeterminates and returns this adapted arguments.
 - dimension(ais, <Order>)
    ++ computes the dimension of the affine variety generated by the 
    ++ polynomials in ais. It is assumed that ais is already a Grbner basis.
 - getTermOrder()
    ++ returns the chosen term order in 'Order'. 
 - groebner(ais, <Order>, <Reorder>)
    ++ returns reduced normalized Grbner-basis for the polynomials ais.
    ++ (see GroebnerPackage).
 - int(a)
 - int(a, x)
 - int(a, x = x0..x1)
    ++ int(a, x) returns the antiderivative of the polynomial a
    ++ with respect to x, or FAIL if the antiderivative is not an element
    ++ of this Domain. If a has only one variable (e.g. if indets(a)=[x]),
    ++ it is possible to call simply int(a).
    ++ int( f, x = x0..x1 ) returns the definite integral of a from x0 to x1,
    ++ or FAIL if the definite integral of a is not an element
    ++ of this Domain.
 - isone(a) 
    ++ tests if a is the multiplicative unit of the Domain.
 - printMonomial(c, d, v) 
    ++ returns an ordered expression c*v^d. 
 - printTerm(d)
 - printTerm(d, v) if Vars=[]
    ++ returns an ordered sequence of the indeterminates Vars resp. v 
    ++ (with their powers d).
 - monic(a)
    ++ divides a by its leading coefficient, i.e. the leading coefficient of
    ++ of the resulting polynomial is 1.
 - normalForm(a, ais, <Order>)
    ++ returns the normal form of a modulo the polynomials ais. 
 - reductum(a)
    ++ returns the reductum of a, i.e  a-lmonomial(a).
 - ringmult(a, c)
    ++ multiplies a by the coefficient ring element c.
 - Rep(a)
    ++ returns the representation of a.
 - sign(a)
    ++ determines the sign of a, which is 0 if a is zero, 1 if a is either
    ++ a positive constant or a positive monomial or an unevaluated positive
    ++ single expression, -1 otherwise.
 - SPolynomial(a,b, <Order>)
    ++ returns the S-Polynomial of a and b.
 - TeXCoeff(c)
    ++ returns a string, which consists of the TeX representation of c.
 - TeXTerm(t)
    ++ returns a string, which consists of the ordered TeX representation of t.

All Entries:
D, Dpoly, Factor, Rep, SPolynomial, TeX, TeXCoeff, TeXTerm, TeXindet, _div\
ide, _invert, _mult, _negate, _plus, _power, _subtract, allAxioms, allCate\
gories, allEntries, allSuperDomains, associates, characteristic, coeff, co\
effRing, coerce, content, convert, convert_to, decompose, degree, degreeve\
c, diff, dimension, divides, equal, equiv, evalp, expr, factor, gcd, getAx\
ioms, getCategories, getSuperDomain, getTermOrder, groebner, hasProp, inde\
ts, info, intmult, irreducible, isUnit, isone, iszero, lcm, lcoeff, lmonom\
ial, lterm, mainvar, mapcoeffs, monic, mult, multcoeffs, negate, new, norm\
, normalForm, nterms, nthcoeff, nthmonomial, nthterm, one, plus, poly, pri\
mpart, print, printElem, printMonomial, printTerm, random, reductum, resul\
tant, ringmult, sign, solve, sqrfree, subs, subsex, tcoeff, testtype, unde\
finedEntries, unitNormal, unitNormalRep, whichEntry, zero


Example:

>> PE := Dom::PolynomialExplicit([x,y,z],Dom::Integer);

        Dom::PolynomialExplicit([x, y, z], Dom::Integer, LexOrder)

>> p1 := PE(3*x-2);

                                  3 x - 2

>> p2 := PE(3*x^2-2*x+1);

                                 2
                              3 x  - 2 x + 1

>> p3 := PE(5*x^3*y^2*z-z^3+5*y^2+1+x);

                          3  2            2    3
                       5 x  y  z + x + 5 y  - z  + 1

>>  5*p3+2*p1^2*3*p2;   

          4       3  2          3        2               2      3
     162 x  + 25 x  y  z - 324 x  + 270 x  - 115 x + 25 y  - 5 z  + 29


++#

alias(
      rep(x) = extop(args(x),1)
      ):

sysassign(Dom::PolynomialExplicit, DomainConstructor(

  # name: #            Dom::PolynomialExplicit,
  # parameter: #       [ Vars, R, Order ],
  # static value: #    [ NumVars, IntDom, GcdDom, EucDom, Field, Rs ],
  # initialization: #  (if iszero(args(0)) or args(0) > 3 then 
                          error("wrong no of args")
                        end_if;
                        if domtype(Vars) <> DOM_LIST then
                          error("illegal indeterminates")
                        end_if;
                        if Vars <> [] then
                          if traperror(poly(1,Vars)) <> 0 then
                             error("illegal indeterminates") end_if
                        end_if;
                        if R::hasProp(Dom::BaseDomain) = FAIL then
                           R := Dom::ExpressionField(normal) 
                         elif not(R::hasProp(Cat::CommutativeRing) and
                                  R::hasProp(Ax::normalRep) or
                                  R::hasProp(
                                   Dom::ExpressionField(id, iszero@normal)))
                             then error("illegal coefficient ring")
                        end_if;
                        if R::hasProp(Dom::Rational) 
                           then Rs := {R, DOM_RAT} else Rs := {R} end_if;
                        #if R::hasProp(Dom::ExpressionField() ) 
                           then Rs := {R, DOM_EXPR, DOM_RAT} 
                           else Rs := {R} end_if;#
                        if contains({LexOrder, DegreeOrder, DegInvLexOrder},
                                     args(args(0))) 
                           then Order := args(args(0))                     
                           else Order := LexOrder end_if;
                        NumVars := nops(Vars);
                        IntDom := bool(R::hasProp(Cat::IntegralDomain));
                        GcdDom := bool(R::hasProp(Cat::GcdDomain));
                        EucDom := bool(R::hasProp(Cat::EuclideanDomain));
                        Field  := bool(R::hasProp(Cat::Field)); ),
  # super-domain: #     Dom::BaseDomain,
  # categories: #       [ (if NumVars = 1 then
                             Cat::UnivariatePolynomialCat(R)
                           else Cat::PolynomialCat(R) end_if) ],
  # axioms: #           [ Ax::normalRep, 
                        (if R::hasProp(Ax::canonicalRep) 
                           then Ax::canonicalRep end_if) ],
  # entries: #

  "new" = (if NumVars = 0 then
            proc(p)
              local pol; 
              begin
                if args(0) <> 1 then error("wrong no of args") end_if;
                pol := poly(expr(p), R);
                case pol
                  of FAIL do pol := poly(expr(p), [hold(_dummy)], R);
                             if pol = FAIL then return(FAIL) end_if;
                  otherwise new(this, pol) end_case
              end_proc
          else 
            proc(p)
              local pol;
              begin
                if args(0) <> 1 then error("wrong no of args") end_if;
                #new(this, poly(expr(p), hold(Vars), R) )#
                pol := poly(expr(p), hold(Vars), R);
                if pol=FAIL then return(FAIL) else new(this, pol) end_if
              end_proc
          end_if ), 

  "expr" = fun( expr(rep(1)) ),

#  "coerce" = fun( new(this, args(1)) ),#

  "Rep" = fun(rep(1)),

  "poly" = this::Rep,

#  "print" = this::expr, #

  "convert" = this::new,

  "indets" = (if Vars =[] then fun( op(rep(1), 2) )
               else fun( hold(Vars) ) end_if),

  "mainvar" = (if Vars =[] then fun( op(rep(1), 2)[1] )
                else fun( hold(Vars)[1] ) end_if),

  "sign" = proc(p) 
     local ep;
     begin
       if this::iszero(p) then return( 0 ) end_if;
       ep := this::expr(p);
       case type(ep)
         of "_plus" do return( hold(sign)(p) );
         of "_mult" do if sign(op(ep, nops(ep))) <> -1 
                         then return( 1 )
                         else return( -1 ) end_if;
         of DOM_IDENT do return( 1 );
         otherwise
           expr2text(ep);
           if strmatch(substring(%,0,1), "-") then return( -1 )
           else return( 1 ) end_if  
       end_case
    end_proc,

  "getTermOrder" = fun( Order ),

#  "allMethods" = proc()#
  "allEntries" = proc()
    local ae,i;
    begin
      ae := sort([op(DomainConstructor::allEntries(this))]);
      ae := map(ae, fun( (if type(domattr(this,args(1)))<>DOM_FAIL 
                             then args(1) else null() end_if) ));
      ae := [_concat(ae[i],", ")$hold(i)=1..nops(ae)-1].[ae[nops(ae)]];  
      print(Unquoted, _concat(op(ae)));
      return(null())
    end_proc,

 #
  ------------------

   Ringoperationen

  ------------------
 #

  "adaptIndets" = (if NumVars = 0 then   # siehe Dom::Polynomial #
    proc() 
      local pols, varset;
      begin
        case args(0)
          of 1 do return(rep(1));
          of 0 do return();
        end_case;
        pols  := map(args(), extop, 1);
        varset := { map(pols, op, 2) };
        if nops(varset) = 1 then 
            pols 
        else 
            map(pols, fun(poly(args(1), sort([ op(map(varset, op)) ]), R)))
        end_if
      end_proc 
     end_if),

  "plus" = (if NumVars <> 0 then  
     fun( 
       (if testargs() then
          if map({args()}, domtype) <> {this} then  
             error("cannot add arguments") end_if 
        end_if;
        new(this, _plus(extop(args(i), 1) $ hold(i)=1..args(0))) ) )
    else
     fun( 
       (if testargs() then
          if map({args()}, domtype) <> {this} then  
             error("cannot add arguments") end_if 
        end_if;
        new(this, _plus(this::adaptIndets( args() ))) ) ) 
    end_if),

  "_plus" = proc() 
      local sel, h;
      begin
        if testargs() then
          if map({args()},domtype) minus 
             {this,op(Rs),Dom::Integer,DOM_INT} <> {}
            then  error("cannot add arguments") end_if
        end_if;
        sel := split([args()], testtype, Dom::Integer);
        h   := (Dom::Integer)::_plus(op(sel[1]));
        sel := split(sel[2], fun( contains(Rs, domtype(args(1))) )); 
        h   := R::_plus(op(sel[1]), R::convert(h)); 
        this::plus(op(sel[2]), this::convert(h)) 
      end_proc, 

  "_negate" = fun( new(this, - rep(1)) ),
  "negate" = this::_negate,  #MuPAD 1.3 #

#~  "_subtract" = fun(_plus(args(1), _negate(args(2)))), ~#
  "_subtract" = fun( new(this, rep(1)-rep(2)) ),

  "mult" = (if NumVars <> 0 then 
     fun(
       (if testargs() then
          if map({args()}, domtype) <> {this} then 
             error("cannot multiply arguments") end_if 
        end_if;
        new(this, _mult(rep(i) $ hold(i)=1..args(0))) ) )
    else
     fun(
       (if testargs() then
          if map({args()}, domtype) <> {this} then 
             error("cannot multiply arguments") end_if 
        end_if;
        new(this, _mult(this::adaptIndets( args() ))) ) )
    end_if), 

  "_mult" = proc() 
      local sel, h;
      begin
        if testargs() then 
          if map({args()},domtype) minus 
             {this,op(Rs),Dom::Integer,DOM_INT} <> {}
            then  error("cannot multiply arguments") end_if
        end_if;
        sel := split([args()], testtype, Dom::Integer); 
        if iszero(nops(sel[1])) then h := (Dom::Integer)::one; 
           else h := (Dom::Integer)::_mult(op(sel[1])) end_if;
        sel := split(sel[2], fun( contains(Rs, domtype(args(1))) )); 
        if iszero(nops(sel[1])) then h := R::intmult(R::one, h)  
           else h := R::intmult(R::_mult(op(sel[1])), h) end_if;
        if nops(sel[2]) = 1 then this::ringmult(op(sel[2]), h) 
           else this::ringmult(this::mult(op(sel[2])), h) end_if
      end_proc, 

  "ringmult" = fun(
     (if testargs() then
        if not testtype(args(2), R) then 
           error("not a ring element ") end_if 
      end_if;
      new(this, multcoeffs(rep(1), args(2))) ) ),

  "intmult" = fun( 
     (if testargs() then
        if not contains({DOM_INT, Dom::Integer}, domtype(args(2))) 
           then error("not an Integer") end_if
      end_if;
      new(this, multcoeffs(rep(1), args(2))) ) ),

  "_power" = fun(
     (if testargs() then
        if not contains({DOM_INT, Dom::Integer}, domtype(args(2))) 
               or (args(2) < 0)
           then error("not an NonNegativeInteger") end_if  
      end_if;
      new(this, _power(rep(1), args(2))) ) ), 

  "zero" = this::new(R::zero), 
 
  "one" = this::new(R::one),

 #
  ------------------------------------

   Operationen zur Konsistenzwahrung 

  ------------------------------------
 #

  "_invert" = proc(a) 
     begin
       if this::degree(a) > 0 then return(FAIL) end_if;
       a:= R::_invert(this::lcoeff(a));
       if a = FAIL then FAIL else this::convert(a) end_if
     end_proc,

  "_divide" = 
     (if IntDom then
       if NumVars <> 0 then
         fun( (divide(rep(1), rep(2), hold(Exact));
              if % = FAIL then FAIL else new(this, %) end_if) )
       else
         fun( (divide(this::adaptIndets(args(1), args(2)), hold(Exact));
              if % = FAIL then FAIL else new(this, %) end_if) ) end_if 
      end_if),

  "norm" = fun( (if args(0) = 1 then norm(rep(1)) 
                   else norm(rep(1), args(2..args(0)))
                  end_if) ),

  "diff" = fun( (if args(0)= 1 then args(1)
     else 
      new(this, diff(rep(1),args(2..args(0)))) end_if) ),

 # "intmult" see near "_mult" #

  "random" = (if NumVars <> 0 then
     fun( (SEED := random(); new(this, randpoly(hold(Vars), R ))) )
    else
     proc() local i, rs;
       begin
        SEED := random(); 
        r := [];
        for i from 1 to random(1..4)() do
          r := append(r, genident() )
        end_for;
        new(this, randpoly(r, R ))
     end_proc
    end_if), 

  "int" = fun( this::new(int(rep(1),args(2..args(0)))) ),

 #
  --------------------------------------

   Zugriffsfunktionen auf Koeffizienten

  --------------------------------------
 #
  
  "degree" = fun( (if args(0) = 1 then degree(rep(1))
     else degree(rep(1), args(2..args(0))) end_if) ),

  "degreevec" = fun( (if args(0) = 1 then degreevec(rep(1), Order)
     else degreevec(rep(1),args(2..args(0))) end_if) ),

  "lcoeff" = fun( (if args(0) = 1 then lcoeff(rep(1), Order)
     else lcoeff(rep(1), args(2..args(0))) end_if) ),

  "lmonomial" = fun( (if args(0) = 1 then  
        new(this, lmonomial(rep(1), Order))
       else 
        new(this, lmonomial(rep(1), args(2..args(0))))
       end_if) ),

  "lterm" = fun( (if args(0) = 1 then 
       new(this, lterm(rep(1), Order))
     else 
       new(this, lterm(rep(1), args(2..args(0))))
     end_if) ),

  "reductum" = fun( (if args(0) = 1 then args(1)- this::lmonomial(args(1))
       else args(1)-this::lmonomial(args(1..args(0))) end_if) ),

  "nthcoeff" = fun( nthcoeff(rep(1), args(2)) ),

  "nthmonomial" = fun( new(this, nthmonomial(rep(1), args(2))) ),

  "nthterm" = fun( new(this, nthterm(rep(1), args(2))) ),

  "nterms" = fun( nterms(rep(1)) ),

  "tcoeff" = fun( tcoeff(rep(1)) ),

  "coeff" = 
        proc(p, x, n)
          local cl;
          begin
            if this::iszero(p) then return( this::zero ) end_if;
            cl := [];
            case args(0)
              of 1 do while not this::iszero(p) do 
                        cl := append(cl, this::lcoeff(p));
                        p  := this::reductum(p)
                      end_while;
                      return( op(cl) );
              otherwise  
                this::new( coeff(rep(1), args(2..args(0))) )
            end_case
          end_proc,

  "mapcoeffs" = fun( 
     this::new(mapcoeffs(rep(1), args(2..args(0)))) ),

 #
  --------------------------------------

   Standardarithmetik auf den Elementen

  --------------------------------------
 #      

  "gcd" = 
    (if GcdDom then 
      if NumVars <> 0 then 
       fun( new(this, gcd(rep(i) $ hold(i)=1..args(0))) )  
      else 
       fun( new(this, gcd(this::adaptIndets(args()))) )  
      end_if end_if),

  "gcdex" = 
    ( if GcdDom and Numvars = 1 then 
       fun( 
         (case args(0)
           of 2 do gcdex(rep(1), rep(2)); break;
           of 3 do gcdex(rep(1), rep(2), args(3)); break;
           otherwise error("wrong number of arguments")
           end_case; 
           (new(this, %[1]), new(this, %[2]), new(this, %[3]))) )  end_if ),

  "lcm" =
    (if GcdDom then
      if NumVars <> 0 then  
        fun( new(this, lcm(rep(i) $ hold(i)=1..args(0))) )
      else 
        fun( new(this, lcm(this::adaptIndets(args()))) )  
      end_if end_if),

  "content" = 
     ( if GcdDom then
         fun( (content(rep(1)); R::new(%)) )  end_if ),

  "multcoeffs" = fun( this::new(multcoeffs(rep(1), args(2))) ),

  "divide" =   
    ( if Field and (NumVars = 1) then 
        proc(x,y,o) 
          begin
            if args(0) = 2 then
                x := divide(this::poly(x), this::poly(y));
            else
                x := divide(this::poly(x), this::poly(y), o);
            end_if;
            case domtype(x)
            of DOM_POLY do this::new(x); break;
            of DOM_FAIL do FAIL; break;
            otherwise (new(this, x[1]), new(this, x[2])) 
            end_case
          end_proc 
      end_if ), 

  "pdivide" = (if NumVars = 1 then  # pdivide(p,q) -> b,s,r mit b*p = s*q+r #
    proc() 
     begin
       case args(0) 
         of 1 do error("Illegal arguments"); 
         of 2 do 
           pdivide(rep(1), rep(2));
           (%[1], new(this, %[2]), new(this, %[3])); break;
         otherwise
           pdivide(rep(1), extop(args(2), 1), args(3..args(0)));
           new(this, %)
       end_case
     end_proc end_if), 

  "pquo" = (if NumVars = 1 then
    fun( this::pdivide(args(1..args(0)), Quo) ) end_if),

  "prem" = (if NumVars = 1 then
    fun( this::pdivide(args(1..args(0)), Rem) ) end_if), 

  "evalp" = fun( 
     this::new(evalp(rep(1), args(2..args(0)))) ), 

  "icontent" = ( if R = Dom::Integer then
     fun( icontent(expr(rep(1)), Vars) ) end_if ),

  "iszero" = fun( iszero(rep(1)) ),

  "isone" = fun( bool(args(1) = this::one) ),

  "Dpoly" = fun( 
   (if args(0) = 1 then new(this, Dpoly(rep(1)))
     else new(this, Dpoly(args(1), rep(2))) end_if) ),

  "D" = this::Dpoly,

  "factor" = 
    (if R = Dom::Integer or R = Dom::Rational or 
       #  (R::constructor = Dom::IntegerMod and Field) #
        Field then
       proc(x) local f, i; begin
         x:= extop(x,1);
	 f:= factor(x);
	 [ this::new(f[1]),
	   (new(this, f[2*i]), f[2*i+1]) $i=1..((nops(f)-1) div 2) ]
       end_proc
      else FAIL
     end_if),

  "sqrfree" = 
    (if R = Dom::Integer or R = Dom::Rational or 
        (R::constructor = Dom::IntegerMod and Field) then
       proc(x) local f, i; begin
         x:= extop(x,1);
	 f:= sqrfree(x);
	 [ this::new(f[1]),
	   (new(this, f[2*i]), f[2*i+1]) $i=1..((nops(f)-1) div 2) ]
       end_proc
     end_if),

  "decompose" = proc()
     begin
       case args(0) 
         of 1 do if nops(indets(args(1))) <> 1 then 
                    error("sorry, you must specify a variable") end_if;
                    decompose(rep(1)); break;
         of 2 do decompose(rep(1), args(2)); break;
       end_case;
       map(%, fun(new(this, args(1))))
     end_proc,

  "resultant" = ( if NumVars <> 0 then
      fun( (case args(0) 
              of 2 do this::convert(
                      resultant(rep(1), rep(2))); break;
              of 3 do this::convert(
                      resultant(rep(1), rep(2), args(3))); 
                      break;
              otherwise error("wrong no of arguments")
            end_case) )
    else
      fun( (case args(0) 
              of 2 do this::convert(
                      resultant(this::adaptIndets( args() )) ); break;
              of 3 do this::convert(
                      resultant(this::adaptIndets(args(1), args(2)),args(3))); 
                      break;
              otherwise error("wrong no of arguments")
            end_case) ) 
    end_if ),     

  "monic" = 
    ( if Field then
       fun( this::mapcoeffs(args(1),_mult,_invert(lcoeff(args(1)))) ) end_if ),

 #
  ------------------------------

   Groebner Basis Operationen

  ------------------------------
 #

  "groebner" = ( if Field then
    proc(ais) # optional: order, reorder #
      local order;
      begin
        if testargs() then
          if not testtype([op(ais)], Type::ListOf(this)) then
            error("all polynomials needs to have the same type") end_if
        end_if;
        if args(0)=1 then order:=Order else order:=args(2..args(0)) end_if;
        map(groebner::gbasis(map([op(ais)], this::poly),order), 
            fun( new(this,args(1)) ))
      end_proc
    end_if),

  "normalForm" = ( if Field then
    proc(a, ais) # optional: order #
      local order;
      begin
        if testargs() then
          if not testtype([a, op(ais)], Type::ListOf(this)) then
            error("all polynomials needs to have the same type") end_if
        end_if;
        if args(0)=2 then order:=Order else order:=args(3) end_if;
        new(this,groebner::normalf(rep(1),(map([op(ais)],this::poly),order))) 
      end_proc
    end_if),

  "SPolynomial" = ( if Field then
    proc(a, b) # optional: order #
      local order;
      begin
        if testargs() then
          if not testtype([a, b], Type::ListOf(this)) then
            error("both polynomials needs to have the same type") end_if
        end_if;
        if args(0)=2 then order:=Order else order:=args(3) end_if;
        new(this,groebner::spoly(rep(1),rep(2),order)) 
      end_proc
    end_if),

  "dimension" = ( if Field then
    proc(ais) # optional: order #
      local order;
      begin
        if testargs() then
          if not testtype([op(ais)], Type::ListOf(this)) then
            error("all polynomials needs to have the same type") end_if
        end_if;
        if args(0)=1 then order:=Order else order:=args(2) end_if;
        groebner::dimension(map([op(ais)], this::poly),order)
      end_proc
    end_if),

 #
  ---------------------

   Ausgabeoperationen

  ---------------------
 #

  "print" = 
     proc(p) 
       local monos, lm, vars;
       begin
         p := rep(1);
         if iszero(p) then return( R::zero ) else monos:=[] end_if;
         vars:=op(p,2);
         while not iszero(p) do
           lm := lmonomial(p,Order);
           monos := append(monos, 
             this::printMonomial(lcoeff(lm,Order),degreevec(lm,Order),vars));
           p := p-lm;
         end_while;
         if nops(monos) = 1 then monos[1]
          else hold(_plus)( op(monos) ) end_if
     end_proc,

  "printMonomial" = 
     proc(c,dvec,vars)
       local term, ec;
       begin
         term:=this::printTerm(dvec,vars);
         if iszero(nops(term)) 
           then return(R::print(c))
           else   # prevent some brackets and coefficients like 1, -1 #
             ec := R::expr(c);
             case type(ec)
               of "_plus" do return(hold(_mult)(R::print(c),term));
               of "_mult" do if sign(op(ec, nops(ec))) <> -1
                         then return(hold(_mult)(R::print(c),term))
                         else return(hold(_mult)(R::print(-c),term,-1)) end_if;
             otherwise
               if expr2text(R::one) = expr2text(1) 
                 then case nops([term]) #prevent output error with hold(_mult)#
                    of 1 do if c = R::one then return(term) 
                             elif c = -R::one then return(-term)
                               elif sign(c) <> -1  
                                 then return(hold(_mult)(R::print(c),term)) 
                                 else 
                                   return(hold(_mult)(R::print(-c),term,-1)) 
                             end_if;
                    otherwise
                      if c = R::one then return(hold(_mult)(term)) 
                        elif c = -R::one then return(hold(_mult)(term,-1))
                          elif sign(c) <> -1
                            then return(hold(_mult)(R::print(c),term)) 
                            else return(hold(_mult)(R::print(-c),term,-1))
                       end_if; 
                    end_case
               else return(hold(_mult)(R::print(c),term)) end_if
             end_case;
           end_if
     end_proc,

  "printTerm" = (if NumVars <> 0 then
     proc(dvec)
       local xis, i;
       begin
         xis := []; 
         for i from 1 to NumVars do
           if dvec[i] = 1 then xis := append(xis, Vars[i])
             elif dvec[i] > 1 then xis := append(xis, Vars[i]^dvec[i])
           end_if
         end_for;
         op(xis)
       end_proc
    else
      proc(dvec,vars)
       local xis, i;
       begin
         xis := []; 
         for i from 1 to nops(vars) do
           if dvec[i] = 1 then xis := append(xis, vars[i])
             elif dvec[i] > 1 then xis := append(xis, vars[i]^dvec[i])
           end_if
         end_for;
         op(xis)
       end_proc
    end_if),

  "TeXindet" = fun( "{".expr2text(args(1))."}" ),

  "TeXCoeff" = proc(x)
     local str, ex;
     begin
       str := R::TeX(x);
       ex  := R::expr(x);
       case type(ex)
         of "_plus" do str := " + {\\left( ".str." \\right)}"; break;
         of "_mult" do if sign(op(ex, nops(ex))) <> -1 then 
                         str := " + {".str."}" end_if; break;
         otherwise
           if contains({DOM_INT, Dom::Integer, DOM_RAT, Dom::Rational},
                       domtype(x)) or 
              R::constructor = Dom::ExpressionField 
             then case x
                    of  R::one do str := " + "; break;
                    of -R::one do str := " - "; break;
                    otherwise
                      if sign(op(ex, nops(ex))) <> -1 then 
                         str := " + {".str."}"
                        else str := " - {".R::TeX(-x)."}" end_if
                  end_case
             else str := " + {\\left( ".str." \\right)}" end_if
       end_case;
       str
     end_proc,

  "TeXTerm" = proc(x)
     local vars, dvec, str, i;
     begin
        dvec := this::degreevec(x);
        vars := map(this::indets(x), this::TeXindet);
        str := ""; 
        for i from 1 to nops(vars) do
          if dvec[i] = 1 then str := str.vars[i]
           elif dvec[i] > 1 then str := str.vars[i]."^{".expr2text(dvec[i])."}"
          end_if
        end_for;
        str
      end_proc,       

   "TeX" = proc(x) 
      local str, ex; 
      begin
        if iszero(this::degree(x)) then "{".R::TeX(this::lcoeff(x))."}"
         else
          str := this::TeXCoeff(this::lcoeff(x));
          if strmatch(substring(str, 0, 3), " + ") then 
            str := substring(str, 3, strlen(str)-3).
                   (this::TeXTerm(this::lterm(x)))
          else str := str.(this::TeXTerm(this::lterm(x))) end_if;
          x := this::reductum(x);
          while not iszero(this::degree(x)) do
            str := str.(this::TeXCoeff(this::lcoeff(x))).
                   (this::TeXTerm(this::lterm(x)));
            x := this::reductum(x)
          end_while;
          case type( (ex := this::expr(x)) )
            of "_mult" do if sign(op(ex, nops(ex))) <> -1 then 
                             str := str." + {".R::TeX(this::lcoeff(x))."}"
                            else 
                             str := str." - {".R::TeX(-this::lcoeff(x))."}" 
                          end_if; break;
            otherwise
              R::TeX(this::lcoeff(x));
              if strmatch(substring(%, 0, 1), "-") then
                  str := str." - {".substring(%, 1, strlen(%)-1)."}"
                else 
                  str := str." + {".R::TeX(this::lcoeff(x))."}" end_if
          end_case
        end_if
      end_proc
 
) ):

unalias( rep ):






