#++
Dom::PowerProduct(S,order) -- abelian monoid of power products over the ordered
	set S of variables (S can also be a list; the ordering of the variables
	is then determined by the position in the list). The optional parameter
	order is a string naming the used ordering.

Dom::PowerProduct(S,order) defines a Cat::OrderedAbelianMonoid. The
representation depends on the type of S. If S is a list and thus finite,
exponents vectors are used.  Otherwise the representation is a list of pairs
[s,i] with s an element of S and i a positive integer. This list is ordered
according to the ordering defined in S.

Six different orderings are provided:
    LexOrder :       pure lexicographic
    RevLexOrder :    reverse lexicographic
    InvLexOrder :    inverse lexicographic
    DegreeOrder :    total degree, lexicographic
    DegRevLexOrder : total degree, reverse lexicographic
    DegInvLexOrder : total degree, inverse lexicographic

Besides the monoid operations there are the following methods implemented:
    degree :    degree(x) gives the total degree,
                degree(x,s) the degree in s
    exponents : list of all non-vanishing exponents
    variables : list of all variables with non-vanishing exponents
    diff :      differentiation with respect to an element of S
                diff(x,s) yields the tuple (n,dx) where n is the
                exponent of s in x and dx is x with the exponent of s
                reduced by 1

As usually the operation in Dom::PowerProduct(S,order) is understood as a
multiplication, but the operation in a Cat::AbelianMonoid is an addition,
Dom::PowerProduct(S,order) is also a Cat::OrderedMonoid. _mult is defined
as _plus and one as zero.  This is a hack around the basic problem that
traditionally in computer algebra an abelian monoid is *not* a monoid,
as the operation in the former one is addition and the latter one
multiplication. Thus it is not possible to define an abelian multiplicative
monoid.


Author: Werner M. Seiler, Uni Karlsruhe
Created: July 3rd, 1997
Last updated: October 15th, 1997
++#

alias(REP(x)=extop(x,1),
      ARG_ERR=error("wrong no of args"),
      TYPE_ERR=error("wrong type of argument")):


Dom::PowerProduct := DomainConstructor(

    Dom::PowerProduct,

    [ S, order ],

    [ dim, Scoerce, Srandom ],

    (if args(0)<1 or args(0)>2 then error("wrong number of arguments") end_if;
     if domtype(S)=DOM_LIST then
	 dim := nops(S);
	 if dim=0 then error("empty list") end_if;
     elif domtype(S)=DOM_DOMAIN and (S::hasProp<>FAIL) and
	  S::hasProp(Cat::OrderedSet) then
	 dim := 0;
	 Scoerce := S::coerce;
	 Srandom := S::random;
     else error("first argument must be Cat::OrderedSet or list") end_if;

     if args(0)<2 then order := LexOrder;
     elif contains({ LexOrder, RevLexOrder, InvLexOrder,
		     DegreeOrder, DegRevLexOrder, DegInvLexOrder },
                   order)=0 then error("undefined ordering") end_if;
     ),

    Dom::BaseDomain,

    [ Cat::OrderedAbelianMonoid, Cat::OrderedMonoid ],

    [ Ax::canonicalRep ],


    # =============== #
    # "local" methods #
    # =============== #

    "Sless" = (if dim>0 then
		   fun(bool(contains(hold(S),args(1))<
                            contains(hold(S),args(2))))
	       else S::_less end_if),


    # sorts and remove duplicates from list #
    # needed only if S is an ordered set #
    "normalize" = proc(x)
	local i;
	begin
	    if x=[] then return(x) end_if;
	    x := sort(x,fun(this::Sless(args(1)[1],args(2)[1])));
	    i := 1;
	    while i<nops(x) do
		if x[i][1]=x[i+1][1] then
		    x[i][2] := x[i][2]+x[i+1][2];
		    x[i+1] := NIL;
		else
		    i := i+1;
		end_if;
	    end_while;
	    return(x);
	end_proc,


    # also only needed if S is an ordered set #
    "coerceFactor" = fun(([Scoerce(args(1)[1]),args(1)[2]])),


    # ========================= #
    # representation, in/output #
    # ========================= #

    "convert" = (if dim>0 then
        proc(x)
        local i,res,e,f,facts;
        begin
            case domtype(x)
            of this do return(x);
            of DOM_LIST do
                if testtype(x,Type::ListOf(Type::NonNegInt)) and
                   (nops(x)=dim) then return(new(this,x))
                else return(this::convert(hold(_mult)(op(x)))) end_if;
            of DOM_EXPR do
                case type(x)
                of "_mult" do
                    res := [ 0 $ dim ];
                    facts := op(x);
                    for f in facts do
                        case type(f)
                        of "_power" do
                            i := contains(hold(S),op(f,1));
                            if i=0 then return(FAIL) end_if;
                            e := op(f,2);
                            if testtype(e,Type::PosInt) then
                                res[i] := res[i]+e;
                            elif e<>0 then return(FAIL) end_if;
                            break;
                        of DOM_IDENT do
                            i := contains(hold(S),f);
                            if i>0 then res[i] := res[i]+1;
                            else return(FAIL) end_if;
                            break;
                        otherwise return(FAIL);
                        end_case;
                    end_for;
                    return(new(this,res));
                of "_power" do
                    i := contains(hold(S),op(x,1));
                    if i=0 then return(FAIL) end_if;
                    e := op(x,2);
                    if testtype(e,Type::PosInt) or (e=0) then
                        res := [ 0 $ dim ];
                        res[i] := e;
                        return(new(this,res));
                    else return(FAIL) end_if;
                end_case;
                return(FAIL);
            of DOM_IDENT do
                i := contains(hold(S),x);
                if i=0 then return(FAIL) end_if;
                res := [ 0 $ dim ];
                res[i] := 1;
                return(new(this,res));
            end_case;
            FAIL;
        end_proc
    else
        proc(x)
        local s,res,e,f,facts;
        begin
            case domtype(x)
            of this do return(x);
            of DOM_LIST do
                if testtype(x,Type::ListOf(S)) then
                    return(this::convert(hold(_mult)(op(x))))
                end_if;
		if nops(x)=2 and (s:=Scoerce(x[1]))<>FAIL and
		   testtype(x[2],Type::PosInt) then
		    return(new(this,[[s,x[2]]]));
		elif {op(map(x,domtype))}<>{DOM_LIST} then return(FAIL)
		elif {op(map(x,nops))}<>{2} then return(FAIL)
		else
		    x := map(x,this::coerceFactor);
		    if has(x,FAIL) then return(FAIL)
		    else return(new(this,this::normalize(x))) end_if;
		end_if;
            of DOM_EXPR do
                case type(x)
                of "_mult" do
                    res := [];
                    facts := op(x);
                    for f in facts do
                        case type(f)
                        of "_power" do
                            s := Scoerce(op(f,1));
                            if s=FAIL then return(FAIL) end_if;
                            e := op(f,2);
                            if testtype(e,Type::PosInt) then
                                res := append(res,[s,e]);
                            elif e<>0 then return(FAIL) end_if;
                            break;
                        otherwise
                            s := Scoerce(f);
                            if s=FAIL then return(FAIL) end_if;
			    res := append(res,[s,1]);
                            break;
                        end_case;
                    end_for;
                    return(new(this,this::normalize(res)));
                of "_power" do
                    s := Scoerce(op(x,1));
                    if s=FAIL then return(FAIL) end_if;
                    e := op(x,2);
                    if testtype(e,Type::PosInt) then
                        return(new(this,[[s,e]]));
                    else return(FAIL) end_if;
		otherwise
                    s := Scoerce(x);
                    if s=FAIL then return(FAIL) end_if;
                    return(new(this,[[s,1]]));
                end_case;
                return(FAIL);
            otherwise
                s := Scoerce(x);
                if s=FAIL then return(FAIL) end_if;
                return(new(this,[[s,1]]));
            end_case;
            FAIL;
        end_proc
    end_if),


    "convert_to" = (if dim>0 then
	proc(x,t)
	local i,j,k,res;
	begin
	    case t
            of DOM_LIST do
                res := [ 0 $ dim ];
                i := 1;
                j := 1;
                for k in REP(x) do
                    if k>0 then
                        j := j+1;
                        res[j] := [hold(S)[i],k]
                    end_if;
                    i := i+1;
                end_for;
                return([op(res,1..j)])
            of this do return(x);
            of DOM_EXPR do return(this::expr(x));
	    otherwise return(FAIL);
            end_case;
        end_proc;
    else
        fun((case args(2)
             of DOM_LIST do return(REP(args(1)));
             of this do return(args(1));
             of DOM_EXPR do return(this::expr(args(1)));
	     otherwise return(FAIL);
             end_case))
    end_if),



    "expr" = (if dim>0 then
        proc(x)
        local i,j,k,res;
        begin
           res := [ 0 $ dim ];
           i := 1;
           j := 1;
           for k in REP(x) do
               if k>1 then
                   j := j+1;
                   res[j] := hold(_power)(hold(S)[i],k);
               elif k=1 then
                   j := j+1;
                   res[j] := hold(S)[i];
               end_if;
               i := i+1;
           end_for;
           case j
           of 0 do return(1);
           of 1 do return(res[1]);
           otherwise return(hold(_mult)(op(res,1..j)));
           end_case;
        end_proc;
    else
        fun((if this::isone(args(1)) then 1
	     elif nops(REP(args(1)))=1 then
		REP(args(1))[1];
		if %[2]=1 then %[1] else hold(_power)(op(%)) end_if;
             else hold(_mult)(op(map(REP(args(1)),
                              fun((if args(1)[2]=1 then args(1)[1]
                                   else hold(_power)(op(args(1))) end_if)))))
             end_if))
    end_if),


    # ========= #
    # orderings #
    # ========= #

    "_less" = (case order
	of LexOrder do       this::lexless; break;
	of RevLexOrder do    this::revlexless; break;
	of InvLexOrder do    this::invlexless; break;
	of DegreeOrder do    this::deglexless; break;
	of DegRevLexOrder do this::degrevlexless; break;
	of DegInvLexOrder do this::deginvlexless; break;
     end_case),


    "lexless" = (if dim>0 then
        proc(x1:this,x2:this)
        local i,l1,l2;
        begin
            l1 := REP(x1);
            l2 := REP(x2);
            i := 1;
            while (i<=dim) and (l1[i]=l2[i]) do i := i+1 end_while;
	    if (i>dim) then FALSE else bool(l1[i]<l2[i]) end_if;
        end_proc
    else
        proc(x1:this,x2:this)
        local l1,l2;
        begin
            l1 := REP(x1);
            l2 := REP(x2);
            while (l1<>[]) and (l2<>[]) do
                if this::Sless(l1[1][1],l2[1][1]) then return(FALSE)
                elif l1[1][1]=l2[1][1] then
                    if l1[1][2]<l2[1][2] then return(TRUE)
                    elif l1[1][2]=l2[1][2] then
                        l1[1] := NIL;
                        l2[1] := NIL;
                    else return(FALSE) end_if;
                else return(TRUE) end_if;
            end_while;
            FALSE
        end_proc
    end_if),


    "revlexless" = (if dim>0 then
        proc(x1:this,x2:this)
        local i,l1,l2;
        begin
            l1 := REP(x1);
            l2 := REP(x2);
            i := dim;
            while (i>0) and (l1[i]=l2[i]) do i := i-1 end_while;
	    if (i=0) then FALSE else bool(l1[i]>l2[i]) end_if;
        end_proc
    else
        proc(x1:this,x2:this)
        local l1,l2;
        begin
            l1 := revert(REP(x1));
            l2 := revert(REP(x2));
            while (l1<>[]) and (l2<>[]) do
                if this::Sless(l2[1][1],l1[1][1]) then return(TRUE)
                elif l1[1][1]=l2[1][1] then
                    if l1[1][2]>l2[1][2] then return(TRUE)
                    elif l1[1][2]=l2[1][2] then
                        l1[1] := NIL;
                        l2[1] := NIL;
                    else return(FALSE) end_if;
                else return(FALSE) end_if;
            end_while;
            FALSE
        end_proc
    end_if),


    "invlexless" = (if dim>0 then
        proc(x1:this,x2:this)
        local i,l1,l2;
        begin
            l1 := REP(x1);
            l2 := REP(x2);
            i := dim;
            while (i>0) and (l1[i]=l2[i]) do i := i-1 end_while;
	    if (i=0) then FALSE else bool(l1[i]<l2[i]) end_if;
        end_proc
    else
        proc(x1:this,x2:this)
        local l1,l2;
        begin
            l1 := revert(REP(x1));
            l2 := revert(REP(x2));
            while (l1<>[]) and (l2<>[]) do
                if this::Sless(l1[1][1],l2[1][1]) then return(TRUE)
                elif l1[1][1]=l2[1][1] then
                    if l1[1][2]<l2[1][2] then return(TRUE)
                    elif l1[1][2]=l2[1][2] then
                        l1[1] := NIL;
                        l2[1] := NIL;
                    else return(FALSE) end_if;
                else return(FALSE) end_if;
            end_while;
            FALSE
        end_proc
    end_if),


    "deglexless" = proc(x1:this,x2:this)
        local d1,d2;
        begin
            d1 := this::degree(x1);
            d2 := this::degree(x2);
            if d1<d2 then TRUE
            elif d1>d2 then FALSE
	    else this::lexless(x1,x2)
            end_if;
        end_proc,


    "degrevlexless" = proc(x1:this,x2:this)
        local d1,d2;
        begin
            d1 := this::degree(x1);
            d2 := this::degree(x2);
            if d1<d2 then TRUE
            elif d1>d2 then FALSE
	    else this::revlexless(x1,x2)
            end_if;
        end_proc,


    "deginvlexless" = proc(x1:this,x2:this)
        local d1,d2;
        begin
            d1 := this::degree(x1);
            d2 := this::degree(x2);
            if d1<d2 then TRUE
            elif d1>d2 then FALSE
	    else this::invlexless(x1,x2)
            end_if;
        end_proc,


    # =========== #
    # arithmetics #
    # =========== #

    "zero" = (if dim>0 then new(this,[ 0 $ dim ])
     else new(this,[]) end_if),


    "_negate" = FAIL,


    "_plus" = (if dim>0 then
        fun((case args(0)
             of 0 do return(this::zero);
             of 1 do return(args(1));
             of 2 do return(new(this,zip(REP(args(1)),REP(args(2)),_plus)));
             end_case;
             args(0) div 2;
             this::_plus(this::_plus(args(1..%)),
                         this::_plus(args((%+1)..args(0))))))
    else
        proc()
        local i,l1,l2,res;
        begin
            case args(0)
            of 0 do return(this::zero);
            of 1 do return(args(1));
            of 2 do
                if testargs() then
                    l1 := this:convert(args(1));
                    if l1=FAIL then TYPE_ERR
                    else l1 := REP(l1) end_if;
                    l2 := this:convert(args(2));
                    if l2=FAIL then TYPE_ERR
                    else l2 := REP(l2) end_if;
                else
                    l1 := REP(args(1));
                    l2 := REP(args(2));
                end_if;
                res := [ 0 $ nops(l1)+nops(l2) ];
                i := 0;
                while (l1<>[]) and (l2<>[]) do
                    i := i+1;
                    if this::Sless(l1[1][1],l2[1][1]) then
                        res[i] := l1[1];
                        l1[1] := NIL;
                    elif (l1[1][1]=l2[1][1]) then
                        res[i] := [l1[1][1],l1[1][2]+l2[1][2]];
                        l1[1] := NIL;
                        l2[1] := NIL;
                    else
                        res[i] := l2[1];
                        l2[1] := NIL;
                    end_if;
                end_while;
                if l1<>[] then res := [op(res,1..i)] . l1
                elif l2<>[] then res := [op(res,1..i)] . l2
                else res := [op(res,1..i)] end_if;
                return(new(this,res));
            end_case;
            i := args(0) div 2;
            this::_plus(this::_plus(args(1..i)),
                        this::_plus(args(i+1..args(0))));
        end_proc;
    end_if),


    "_subtract" = (if dim>0 then
        proc(x,y)
        local res,i,j,k;
        begin
            if testargs() then
                if args(0)<>2 then ARG_ERR end_if;
                if domtype(x)<>this or domtype(y)<>this then TYPE_ERR end_if;
            end_if;
            res := REP(x);
            j := 0;
            for i in REP(y) do
                j := j+1;
                res[j] := res[j]-i;
                if res[j]<0 then return(FAIL) end_if;
            end_for;
            new(this,res);
        end_proc
    else
        proc(x,y)
        local i,l1,l2,res;
        begin
            if testargs() then
                if args(0)<>2 then ARG_ERR end_if;
                if domtype(x)<>this or domtype(y)<>this then TYPE_ERR end_if;
            end_if;
            l1 := REP(args(1));
            l2 := REP(args(2));
            res := [ 0 $ nops(l1)+nops(l2) ];
            i := 0;
            while (l1<>[]) and (l2<>[]) do
                if this::Sless(l1[1][1],l2[1][1]) then
                    i := i+1;
                    res[i] := l1[1];
                    l1[1] := NIL;
                elif (l1[1][1]=l2[1][1]) then
                    tmp := l1[1][2]-l2[1][2];
                    if tmp<0 then return(FAIL) end_if;
                    i := i+1;
                    res[i] := [l1[1][1],tmp];
                    l1[1] := NIL;
                    l2[1] := NIL;
                else return(FAIL) end_if;
            end_while;
            if l2<>[] then return(FAIL) end_if;
            if l1<>[] then res := [op(res,1..i)] . l1
            else res := [op(res,1..i)] end_if;
            return(new(this,res));
        end_proc;
    end_if),


    "_mult" = this::_plus,


    "_invert" = this::_negate,


    "_divide" = this::_subtract,


    "one" = this::zero,


    "isone" = this::iszero,


    # ============= #
    # miscellaneous #
    # ============= #

    "degree" = (if dim>0 then
        fun((case args(0)
             of 1 do return(_plus(op(REP(args(1)))));
             of 2 do
	         contains(hold(S),args(2));
		 if (%=0) then return(0);
		 else return(REP(args(1))[%]);
		 end_if;
             otherwise error("too many argument");
             end_case))
    else
        fun((case args(0)
             of 1 do return(_plus(op(this::exponents(args(1)))));
             of 2 do
                 contains(this::variables(args(1)),args(2));
                 if %=0 then return(0);
                 else return(this::exponents(args(1))[%]);
                 end_if;
             otherwise return(FAIL);
             end_case))
        end_if),


    "variables" = (if dim>0 then
        proc()
        local i,k,res;
        begin
            res := [];
            i := 1;
            for k in REP(args(1)) do
                if k>0 then res := append(res,hold(S)[i]) end_if;
                i := i+1;
            end_for;
            res;
        end_proc;
    else
        fun(map(REP(args(1)),op,1))
    end_if),


    "exponents" = (if dim>0 then
        fun(select(REP(args(1)),fun(args(1)>0)))
    else
        fun(map(REP(args(1)),op,2))
    end_if),


    "diff" =(if dim>0 then
        proc(x)
        local i,l,pos,n,expv;
        begin
            if testargs() then
		if args(0)<1 then ARG_ERR end_if;
                if domtype(x)<>this then TYPE_ERR end_if;
		for i in [args(2..args(0))] do
		    if contains(hold(S),i)=0 then TYPE_ERR end_if;
		end_for;
            end_if;

	    case args(0)
	    of 1 do return(x);
	    of 2 do
                l := REP(x);
                pos := contains(hold(S),args(2));
                if l[pos]=0 then return((0,this::zero)) end_if;
                n := l[pos];
                l[pos] := n-1;
                return((n,new(this,l)));
	    otherwise
		l := REP(x);
		expv := [ 0 $ dim ];
		for i in [args(2..args(0))] do
		    pos := contains(hold(S),i);
		    expv[pos] := expv[pos]+1;
		    if expv[pos]>l[pos] then return((0,this::zero)) end_if;
		end_for;
		n := 1;
		for i from 1 to dim do
		    if expv[i]=0 then next end_if;
		    for j from l[i] downto l[i]-expv[i]+1 do
			n := j*n;
		    end_for;
		    l[i] := l[i]-expv[i];
		end_for;
		return((n,new(this,l)));
	    end_case;
        end_proc
    else
        proc(x)
        local i,j,l,lvars,pos,n,expv,vars;
        begin
            if testargs() then
		if args(0)<1 then ARG_ERR end_if;
                if domtype(x)<>this then TYPE_ERR end_if;
		for i in [args(2..args(0))] do
		    if domtype(i)<>S then TYPE_ERR end_if;
		end_for;
            end_if;

	    case args(0)
	    of 1 do return(x);
	    of 2 do
                l := REP(x);
                pos := contains(map(l,op,1),args(2));
                if pos=0 then return((0,this::zero)) end_if;
                n := l[pos][2];
                if n=1 then l[pos] := NIL
                else l[pos][2] := n-1 end_if;
                return((n,new(this,l)));
	    otherwise
		l := REP(x);
		lvars := map(l,op,1);
		expv := [];
		vars := [];
		for v in [args(2..args(0))] do
		    pos := contains(lvars,v);
		    if pos=0 then return((0,this::zero)) end_if;
		    n := l[pos][2];
		    pos := contains(vars,v);
		    if pos=0 then
			vars := insert_ordered(v,vars,this::Sless);
			expv := insert_ordered([v,1],expv,
					fun(this::Sless(args(1)[1],args(2)[1])));
		    else
			expv[pos][2] := expv[pos][2]+1;
			if expv[pos][2]>n then return((0,this::zero)) end_if;
		    end_if;
		end_for;
		n := 1;
		i := 1;
		while i<=nops(l) do
		    if l[i][1]<>expv[1][1] then i := i+1;
		    else
		        for j from l[i][2] downto l[i][2]-expv[1][2]+1 do
			    n := j*n;
		        end_for;
		        l[i][2] := l[1][2]-expv[1][2];
			expv[1] := NIL;
			if l[i][2]=0 then l[i] := NIL
			else i := i+1 end_if;
		    end_if;
		end_while;
		return((n,new(this,l)));
	    end_case;
        end_proc
    end_if),


    "TeX" = fun(generate::tex(this::expr(args(1)))),


    "random" = (if dim>0 then
	proc()
	local i,j,r1,r2,res;
	begin
	    r1 := random(1..dim);
	    r2 := random(1..5);
	    res := [0 $ dim];
	    for i from 1 to dim do
		j:=r1();
		res[j] := res[j]+r2();
	    end_for;
	    new(this,res);
	end_proc;
    elif Srandom<>FAIL then
	proc()
	local i,r,res;
	begin
	    r := random(1..10);
	    res := [];
	    for i from 1 to r() do
		res := append(res,[Srandom(),r()]);
	    end_for;
	    new(this,res);
	end_proc;
    else FAIL end_if)
):

unalias(REP,TYPE_ERR):
