#++
Dom::SparseMatrix(C,R) -- sparse matrices with entries from the commutative
			  ring R and column indices from the ordered set C

Besides general matrix operations the main functionality is a procedure to
compute efficiently row echelon forms.

Examples for the generation of sparse matrices:

	>> A := array(1..3,1..3,[[2,0,4],[0,0,5],[3,6,0]]);

	>> M := Dom::Matrix(Dom::Integer)([[2,0,4],[0,0,5],[3,6,0]]);


	>> S:=Dom::SparseMatrix(Dom::Integer, Dom::Integer);

                  Dom::SparseMatrix(Dom::Integer, Dom::Integer)

                                +-       -+
                                | 2, 0, 4 |
                                |         |
                                | 0, 0, 5 |
                                |         |
                                | 3, 6, 0 |
                                +-       -+


	>> S(3, [1,2,3], [[[1,2],[3,4]],[[3,5]],[[1,3],[2,6]]]);

	>> S(3, [1,2,3], [[2,0,4],[0,0,5],[3,6,0]]);

	>> S([1,2,3], [[[1,2],[3,4]],[[3,5]],[[1,3],[2,6]]]);

	>> S([[[1,2],[3,4]],[[3,5]],[[1,3],[2,6]]]);

	>> S([1,2,3], [[2,0,4],[0,0,5],[3,6,0]]);

	>> S([1,2,3], A);

	>> S([1,2,3], M);

	>> S(%);


Authors: Eduard Niess, Werner M. Seiler,  Uni Karlsruhe
Created: ??? (1996)
Last update: October 8th, 1997
++#


alias(NROWS(x) = extop(x,1),
      NCOLS(x) = extop(x,2),
      ALL_INDICES(x) = extop(x,3),
      ROWS(x) = extop(x,4),
      ARG_ERR = error("wrong no of args"),
      COL_ERR = error("column index out of range"),
      ROW_ERR = error("row index out of range"),
      TYPE_ERR = error("illegal type of argument")):


Dom::SparseMatrix := DomainConstructor(

	Dom::SparseMatrix,

	[C, R ],

	[ Rzero, Rone, Risone, Riszero, Rcoerce, Rnegate, Rplus, Rmult, Rexpr,
	  Mat, Q, QMat, Qinvert, Qconvert, Qmult, remDup, setDiff, less,
	  Ccoerce, TyLO, TyAT, noChecks ],

	(if args(0)<2 or args(0)>3 then ARG_ERR end_if;
	if domtype(C)<>DOM_DOMAIN or C::hasProp=FAIL or
	   not(C::hasProp(Cat::OrderedSet)) then
	    error("first argument must be Cat::OrderedSet")
	end_if;
	if domtype(R)<>DOM_DOMAIN or R::hasProp=FAIL or
	   not(R::hasProp(Cat::Rng)) then
	    error("second argument must be Cat::Rng");
	end_if;

	if args(0)=2 then less := C::_less;
	else less := args(3) end_if;

	Rzero := R::zero;
	Riszero := R::iszero;
	Rcoerce := R::coerce;
	Rnegate := R::_negate;
	Rplus := R::_plus;
	Rmult := R::_mult;
	Rexpr := R::expr;
	Mat := Dom::Matrix(R);
	if R::hasProp(Cat::IntegralDomain) then
	    Q := Dom::Fraction(R);
	    QMat := Dom::Matrix(Q);
	    Qinvert := Q::_invert;
	    Qconvert := Q::convert;
	    Qmult := Q::_mult;
	    Rone := R::one;
	    Risone := R::isone;
	end_if;

	Ccoerce := C::coerce;
	remDup := listtools::removeDuplicates;
	setDiff := listtools::setDifference;
	TyLO := Type::ListOf;
	TyAT := Type::AnyType;
	noChecks := FALSE;
	),

	Dom::BaseDomain,

	[ Cat::MatrixCat(R) ],


	[],

#
The representation has four slots:
   NCols is the number of columns,
   NRows the number of rows
   AllIndices is a list containing all labels of columns; it must always
	 have length NCols and must always be orderd
   Rows is a list; each element represents one row;
	 a row is a list; each element represents one entry;
	 an entry is a list with two elements: the first one is the label,
	 the second one is the corresponding value
The representation is neither canonical nor normal, as there are no checks
whether some entries vanish.
#

	# ============= #
	# local methods #
	# ============= #

	"sortRow" =
	    fun(sort(args(1),fun(less(args(1)[1],args(2)[1])))),


	"containsIndex" = proc(r,c)
	local pos;
	begin
	    pos := 1;
	    while r<>[] and r[1][1]<>c do
		r[1] := NIL;
		pos := pos+1;
	    end_while;
	    if r=[] then 0 else pos end_if;
	end_proc,


	"indices" = fun(map(args(1),fun(args(1)[1]))),


	"entries" = fun(map(args(1),fun(args(1)[2]))),


	"greater" = func((
	    if r1=[] then FALSE
	    elif r2=[] then TRUE
	    elif r1[1][1]=r2[1][1] then bool(nops(r2)<nops(r1))
	    else bool(less(r1[1][1],r2[1][1]))
	    end_if), r1,r2),


	# removes the gcd of a row #
	"makePrimitive" = proc(r)
	local ld,g,i;
	begin
	    if nops(r)=1 then return(r[1][2],[[r[1][1],Rone]]) end_if;
	    ld := this::entries(r);
	    g := gcd(op(ld));
	    if Risone(g) then return(Rone,r) end_if;
	    ld := map(ld,R::_divide,g);
	    for i from 1 to nops(ld) do
		r[i][2] := ld[i];
	    end_for;
	    return(g,r);
	end_proc,


	# computes linear combination of two rows #
	"addRows" = proc(d1,r1,d2,r2)
	local res,c,e,summe,k;
	begin
	    if r1=[] then
		if not(Risone(d2)) then
		    r2 := map(r2, fun([args(1)[1],Rmult(args(1)[2],args(2))]),
			      d2);
		end_if;
		return(r2);
	    end_if;
	    if r2=[] then
		if not(Risone(d1)) then
		    r1 := map(r1, fun([args(1)[1],Rmult(args(1)[2],args(2))]),
			      d1);
		end_if;
		return(r1);
	    end_if;

	    if noChecks or not(Risone(d1)) then
		r1 := map(r1, fun([args(1)[1],Rmult(args(1)[2],args(2))]), d1);
	    end_if;
	    if noChecks or not(Risone(d2)) then
		r2 := map(r2, fun([args(1)[1],Rmult(args(1)[2],args(2))]), d2);
	    end_if;

	    res := [ 0 $ nops(r1)+nops(r2)];
	    k := 0;
	    while r1<>[] do
	        if r2=[] then return([op(res,1..k)].r1) end_if;
		c := r1[1][1];
		e := r1[1][2];
		r1[1] := NIL;
		while r2<>[] and less(r2[1][1],c) do
		    k := k+1;
		    res[k] := [r2[1][1],r2[1][2]];
		    r2[1] := NIL;
		end_while;
		if r2<>[] and r2[1][1]=c then
		    summe := Rplus(e,r2[1][2]);
		    if noChecks or not(Riszero(summe)) then
		        k := k+1;
			res[k] := [c,summe];
		    end_if;
		    r2[1] := NIL;
		else
		    k := k+1;
		    res[k] := [c,e];
		end_if;
	    end_while;

	    [op(res,1..k)] . r2;
	end_proc,


        "extractIndices" = proc(l)
        local res,i,r;
        begin
            res := [];
            for r in l do
                if r<>[] then
                    for i in r do
                        if i<>[] then
                            if contains(res,i[1])=0 then
                                res := append(res,i[1]);
                            end_if;
                        end_if;
                    end_for;
                end_if;
            end_for;
	    res;
        end_proc,


        "coerceEntry" = fun((if nops(args(1))<>2 then FAIL
            else [Ccoerce(args(1)[1]),Rcoerce(args(1)[2])] end_if)),


	"newMat" = proc(inds,x)
	local i,j,k1,k2,ni,ind,nr,nc,rows,r,e;
	begin
	    if domtype(inds)<>DOM_LIST or inds=[] then return(FAIL) end_if;
	    inds := map(inds,Ccoerce);
	    if contains(inds,FAIL)<>0 then return(FAIL) end_if;
	    inds := sort(inds,less);

	    case domtype(x)
	    of DOM_ARRAY do       # only two-dimensional arrays allowed! #
		if op(x,[0,1])<>2 then return(FAIL) end_if;
		nr := op(x,[0,2,2]);
		nc := op(x,[0,3,2]);
		break;
	    of DOM_LIST do
		x := Mat::convert(x);
		if x=FAIL then return(FAIL) end_if;
	    otherwise:
		if not(x::hasProp(Cat::MatrixCat)) then return(FAIL) end_if;
		i := x::dimen(x);
		nr := op(i,1);
		nc := op(i,2);
	    end_case;

	    rows := [ 0 $ nr ];
	    k1 := 0;
	    ni := nops(inds);
	    for i from 1 to nr do
		r := [ 0 $ ni ];
		k2 := 0;
		j := 1;
		for ind in inds do
		    e := Rcoerce(x[i,j]);
		    if not(Riszero(e)) then
		        k2 := k2+1;
		        r[k2] := [ind,e];
		    end_if;
		    j := j+1;
		end_for;
		k1 := k1+1;
		rows[k1] := [op(r,1..k2)];
	    end_for;
	    return(new(this,nr,nc,inds,rows));
	end_proc,


	# ========================= #
	# representation, in/output #
	# ========================= #


	"convert" = proc(x)
	local nr,inds,rows;
	begin
	    if testargs() then
		if (args(0)<1) or (args(0)>4) then ARG_ERR end_if;
	    end_if;

	    case args(0)
	    of 1 do
		if domtype(x)=this then return(x) end_if;
		if not(testtype(x,TyLO(TyLO(TyLO(TyAT))))) then
		    return(FAIL)
		end_if;
		nr := nops(x);
		rows := x;
		inds := this::extractIndices(x);
		break;
	    of 2 do
		case domtype(x)
		of DOM_INT do
		    nr := args(1);
		    inds := args(2);
		    rows := [];
		    break;
		of DOM_LIST do
		    if testtype(args(2),TyLO(TyLO(TyLO(TyAT)))) then
			nr := nops(args(2));
			inds := args(1);
			rows := args(2);
		    else return(this::newMat(args())) end_if;
		    break;
		otherwise return(FAIL);
		end_case;
		break;
	    of 3 do
		nr := args(1);
		inds := args(2);
		if not(testtype(args(3),TyLO(TyLO(TyLO(TyAT))))) then
			return(this::newMat(inds,args(3)));
		end_if;
		rows := args(3);
		break;
	    otherwise return(FAIL);
	    end_case;

	    if testargs() then
		if (domtype(nr)<>DOM_INT) or (nr<1) or
		   (domtype(inds)<>DOM_LIST) or (inds=[]) then
		    return(FAIL)
		end_if;
		inds := map(inds,Ccoerce);
		if contains(inds,FAIL) then return(FAIL) end_if;
		rows := map(rows, map, this::coerceEntry);
                if has(rows,FAIL) then return(FAIL) end_if;
	    end_if;

	    for i from 1 to nops(rows) do
		rows[i] := this::sortRow(rows[i]);
	    end_for;
	    if nops(rows)<nr then
		rows := rows . [[] $ nr-nops(rows)];
	    end_if;
	    new(this,nr,nops(inds),sort(inds,less),rows);
        end_proc,


	"convert_to" = proc(A,t)
	begin
	    if testargs() then
		if args(0)<>2 then ARG_ERR end_if;
		if domtype(A)<>this then TYPE_ERR end_if;
	    end_if;

	    case t
	    of DOM_LIST do return(ROWS(A));
	    of DOM_ARRAY do
	    of DOM_EXPR do
		return(this::expr(A));
	    end_case;
	    FAIL;
	end_proc,


	"ncols" = fun(NCOLS(args(1))),


	"nrows" = fun(NROWS(args(1))),


	"dimen" = fun([NROWS(args(1)),NCOLS(args(1))]),


	"allIndices" = fun(ALL_INDICES(args(1))),


	"rows" = fun(ROWS(args(1))),


	"isDense" = fun(FALSE),


	"isSparse" = fun(TRUE),


	"_index" = proc(A,i,c)
	local r;
	begin
	    if testargs() then
	        if domtype(A)<>this then TYPE_ERR end_if;
		if domtype(c)<>C then
		    c := Ccoerce(c);
		    if c=FAIL then COL_ERR end_if;
		end_if;
		if contains(ALL_INDICES(A),c)=0 then COL_ERR end_if;
		if (i<1) or (i>NROWS(A)) then ROW_ERR end_if;
	    end_if;

	    r := ROWS(A)[i];
	    pos := this::containsIndex(r,c);
	    if pos=0 then Rzero else r[pos][2] end_if;
	end_proc,


	"row" = fun((
	    if testargs() then
		if args(0)<>2 then ARG_ERR end_if;
		if domtype(args(1))<>this then TYPE_ERR end_if;
	    end_if;
	    case type(args(2))
	    of DOM_INT do
		if testargs() then
		    if (args(2)<0) or
		       (args(2)>NROWS(args(1))) then ROW_ERR end_if;
	    	end_if;
		return(ROWS(args(1))[args(2)]);
	    of DOM_LIST do return(this::extract(args(1),args(2)));
	    of "_range" do
		return(this::extract(args(1),[ $ args(2) ]));
	    otherwise TYPE_ERR
	    end_case)),


	# extracts the rows with indices in the list l and
	  returns them as a new matrix #
	"extract" = proc(A,l)
	local i,j,ra,nrows;
	begin
	    ra := ROWS(A);
	    nrows := [ 0 $ nops(l)];
	    j := 0;
	    for i in l do
		j := j+1;
		nrows[j] := ra[i];
	    end_for;
	    new(this,nops(l),NCOLS(A),ALL_INDICES(A),nrows);
	end_proc,


	# expr returns a sparse matrix as an array in the usual MuPAD form #
	"expr" = proc(A)
	local AA,i,j,allinds,ind,nr,nc,r;
	begin
	    nc := NCOLS(A);
	    nr := NROWS(A);
	    if nc=0 then error("matrix with zero columns") end_if;
	    if nr=0 then error("matrix with zero rows") end_if;

	    AA := array(1..nr, 1..nc, [[Rexpr(Rzero) $ nc] $ nr]);
	    i := 0;
	    allinds := ALL_INDICES(A);
	    for r in ROWS(A) do
		i := i+1;
		j := 0;
		for ind in allinds do
		    if r=[] then break end_if;
		    j := j+1;
		    if ind=r[1][1] then
			AA[i,j] := Rexpr(r[1][2]);
			r[1] := NIL;
		    end_if;
		end_for;
	    end_for;
	    AA;
	end_proc,


	"iszero" = fun(bool(ROWS(args(1)) = [[] $ NROWS(args(1))])),


	"map" = func(new(this,NROWS(A),NCOLS(A),ALL_INDICES(A),
			 map(ROWS(A),
			     fun(map(args(1),
				     fun([args(1)[1],
					  args(2)(args(1)[2],args(3..args(0)))]),
				     args(2..args(0)))),
			     f,args(3..args(0)))), A,f),


	# ======================================= #
	# manipulation and generation of matrices #
	# ======================================= #

	"set_index" = proc(A,i,c,d)
	local rs,r,j,pos,newr;
	begin
	    if testargs() then
	        if domtype(A)<>this then TYPE_ERR end_if;
		if domtype(c)<>C then
		    c := Ccoerce(c);
		    if c=FAIL then COL_ERR end_if;
		end_if;
		if contains(ALL_INDICES(A),c)=0 then COL_ERR end_if;
		if (i<1) or (i>NROWS(A)) then ROW_ERR end_if;
		d := Rcoerce(d);
	    end_if;

	    rs := ROWS(A);
	    pos := this::containsIndex(rs[i],c);
	    if pos=0 then
		r := rs[i];
		newr := [ 0 $ NROWS(A) ];
		k := 0;
		while r<>[] and less(r[1][1],c) do
		    k := k+1;
		    newr[k] := r[1];
		    r[1] := NIL;
		end_while;
		newr[k+1] := [c,d];
		newr := [op(newr,1..k+1)] . r;
		rs[i] := newr;
	    else rs[i][pos][2] := d end_if;
	    new(this,NROWS(A),NCOLS(A),ALL_INDICES(A),rs);
	end_proc,


	"setRow" = fun((
	    if testargs() then return(this::checkSetRow(args())) end_if;
	    new(this,NROWS(args(1)),NCOLS(args(1)),ALL_INDICES(args(1)),
		subsop(ROWS(args(1)),args(2)=args(3))))),


	"checkSetRow" = proc(A,i,r)
	local nr,inds;
	begin
	    if args(0)<>3 then ARG_ERR end_if;
	    if domtype(A)<>this then TYPE_ERR end_if;
	    nr := NROWS(A);
	    if i<0 or i>nr then ROW_ERR end_if;
	    r := map(r,this::coerceEntry);
	    if has(r,FAIL) then TYPE_ERR end_if;
	    r := this::sortRow(r);
	    inds := ALL_INDICES(A);
	    for ind in this::indices(r) do
		if contains(inds,ind)=0 then COL_ERR end_if;
	    end_for;
	    new(this,nr,NCOLS(A),inds,subsop(ROWS(A),i=r))
	end_proc,


	# removes columns which contain only zeros.
	  This effects only the values of ALL_INDICES and NCOLS #
	"elimZeroCols" = proc(A)
	local inds,r;
	begin
	    inds := [];
	    for r in ROWS(A) do
		inds := remDup(listtools::merge(inds,this::indices(r),less));
	    end_for;
	    new(this,NROWS(A),nops(inds),inds,ROWS(A));
	end_proc,


	# eliminates all columns with an index C such that crit(C)=TRUE #
	"purge" = proc(A,crit)
	local ra,inds,change,k,rargs;
	begin
	    if testargs() then
		if domtype(A)<>this then TYPE_ERR end_if;
	    end_if;
	    rargs := args(3..args(0));

	    inds := [ 0 $ nops(ALL_INDICES(A)) ];
	    change := FALSE;
	    k := 0;
	    for c in ALL_INDICES(A) do
		if not(crit(c,rargs)) then
		    k := k+1;
		    inds[k] := c
		else change := TRUE end_if;
	    end_for;

	    if change then
	        ra := map(ROWS(A), select, fun(not(crit(args(1)[1],rargs))));
	        new(this,NROWS(A),k,[op(inds,1..k)],ra);
	    else return(A) end_if;
	end_proc,


	# like "purge", however, with the additional assumption that crit
	  respects the ordering of the indices #
	"sortedPurge" = proc(A,crit)
	local ra,i,inds,rargs;
	begin
	    if testargs() then
		if domtype(A)<>this then TYPE_ERR end_if;
	    end_if;
	    rargs := args(3..args(0));

	    inds := ALL_INDICES(A);
	    ra := ROWS(A);
	    if crit(inds[1]) then
		while inds<>[] and crit(inds[1],rargs) do
		    inds[1] := NIL
		end_while;
		for i from 1 to NROWS(A) do
		    while ra[i]<>[] and crit(ra[i][1][1],rargs) do
			ra[i][1] := NIL
		    end_while;
		end_for;
	    end_if;
	    new(this,NROWS(A),nops(inds),inds,ra);
	end_proc,


	"delRow" = func((
	    if testargs() then
	        if domtype(A)<>this then TYPE_ERR end_if;
		if (i<0) or (i>NROWS(A)) then ROW_ERR end_if;
	    end_if;

	    new(this, NROWS(A)-1, NCOLS(A), ALL_INDICES(A),
		subsop(ROWS(A),i=NIL))), A,i),


	# inserts a row r at the top of the matrix #
	"consRow" = func((
	    if testargs() then
		if domtype(A)<>this then TYPE_ERR end_if;
		r := map(r,this::coerceEntry);
		if has(r,FAIL) then TYPE_ERR end_if;
		r := this::sortRow(r);
		if nops(setDiff(this::indices(r),ALL_INDICES(A)))<>
		   nops(this::indices(r)) then COL_ERR end_if;
	    end_if;

	    new(this, NROWS(A)+1, NCOLS(A), ALL_INDICES(A), [r].ROWS(A))), A,r),


	# appends a row at the end of the matrix #
	"appendRow" = func((
	    if testargs() then
		if domtype(A)<>this then TYPE_ERR end_if;
		r := map(r,this::coerceEntry);
		if has(r,FAIL) then TYPE_ERR end_if;
		r := this::sortRow(r);
		if nops(setDiff(this::indices(r),ALL_INDICES(A)))<>
		   nops(this::indices(r)) then COL_ERR end_if;
	    end_if;

	    new(this, NROWS(A)+1, NCOLS(A), ALL_INDICES(A),
		append(ROWS(A),r))), A,r),



	# stacks matrices #
	"stackMatrix" = fun((case args(0)
	    of 0 do error("no arguments given");
	    of 1 do return(args(1));
	    of 2 do
		if testargs() then
		    if domtype(args(1))<>this or
		       domtype(args(2))<>this then TYPE_ERR end_if;
		    if ALL_INDICES(args(1))<>
		       ALL_INDICES(args(2)) then COL_ERR end_if;
		end_if;
	        return(new(this,NROWS(args(1))+NROWS(args(2)),
			   NCOLS(args(1)),ALL_INDICES(A),
			   ROWS(args(1)).ROWS(args(2))));
	    otherwise
		args(0) div 2;
		this::stackMatrix(this::stackMatrix(args(1..%)),
				  this::stackMatrix(args(%+1..args(0))));
	    end_case)),


	# concats matrices #
	"concatMatrix" = fun((case args(0)
	    of 0 do error("no arguments given");
	    of 1 do return(args(1));
	    of 2 do
		if testargs() then
		    if NROWS(args(1))<>NROWS(args(2)) then
		        error("incompatible dimensions");
		    end_if;
		    if not(less(ALL_INDICES(args(1))[nops(ALL_INDICES(args(1)))],
		                ALL_INDICES(args(2))[1])) then
		        error("incompatible matrices");
		    end_if;
		end_if;
	        ALL_INDICES(args(1)) . ALL_INDICES(args(2));
	        return(new(this,NROWS(args(1)),nops(%),%,
			   zip(ROWS(args(1)),ROWS(args(2)),_concat)));
	    otherwise
		args(0) div 2;
		this::concatMatrix(this::concatMatrix(args(1..%)),
				   this::concatMatrix(args(%+1..args(0))));
	    end_case)),


	# splits the matrix A into two at the column given by c.
	  The first column of the right matrix is enumerated by the
	  first index greater or equal to c. #
	"horizSplit" = proc(A,c)
	local Rinds,Linds,Rrows,Lrows,ra,r,l,k,kl,ki,n;
	begin
	    if testargs() then
		if domtype(A)<>this then TYPE_ERR end_if;
	    end_if;

	    Rinds := ALL_INDICES(A);
	    if less(c,Rinds[1]) or (c=Rinds[1]) then return((0,A))
	    elif less(Rinds[nops(Rinds)],c) then return((A,0)) end_if;
	    n := nops(Rinds);
	    Linds := [ 0 $ n ];
	    ki := 0;
	    while less(Rinds[1],c) do
	        ki := ki+1;
		Linds[ki] := Rinds[1];
		Rinds[1] := NIL;
	    end_while;
	    Linds := [op(Linds,1..ki)];

            ra := ROWS(A);
	    Lrows := [ 0 $ NROWS(A) ];
	    Rrows := [ 0 $ NROWS(A) ];
	    for k from 1 to NROWS(A) do
	        r := ra[k];
		l := [ 0 $ nops(r) ];
		kl := 0;
		while r<>[] and less(r[1][1],c) do
		    kl := kl+1;
		    l[kl] := r[1];
		    r[1] := NIL;
		end_while;
		Lrows[k] := l;
		Rrows[k] := r;
	    end_for;
	    ( new(this, NROWS(A), ki, Linds, Lrows),
	      new(this, NROWS(A), n-ki, Rinds, Rrows) );
	end_proc,


	"swapRow" = proc(A,i,j)
	local ra,tmp;
	begin
	    if testargs() then
		if (domtype(A)<>this) or (domtype(i)<>DOM_INT) or
		   (domtype(j)<>DOM_INT) then TYPE_ERR end_if;
		if (i<0) or (j<0) or
		   (i>NROWS(A)) or (j>NROWS(A)) then ROW_ERR end_if;
	    end_if;

	    if (i=j) then return(A) end_if;
	    ra := ROWS(A);
	    tmp := nrows[i];
	    nrows[i] := nrows[j];
	    nrows[j] := tmp;
	    new(this,NROWS(A),NCOLS(A),ALL_INDICES(A),ra);
	end_proc,


	"transpose" = proc(A)
	local ra,nrows,inds,nr,nc,i,j,idx,pos;
	begin
	    if testargs() then
		if domtype(A)<>this then TYPE_ERR end_if;
	    end_if;
	    nr := NROWS(A);
	    nc := NCOLS(A);
	    if nr<>nc then error("only possible for square matrices") end_if;

	    nrows := [[ 0 $ nr] $ nc];
	    idx := [ 0 $ nc ];
	    inds := ALL_INDICES(A);
	    ra := ROWS(A);
	    for j from 1 to nr do
		for i in ra[j] do
		    pos := contains(inds,i[1]);
		    idx[pos] := idx[pos]+1;
		    nrows[pos][idx[pos]] := [inds[j],i[2]];
		end_for;
	    end_for;
	    for i from 1 to nr do
	        nrows[i] := [op(nrows[i],1..idx[i])];
	    end_for;
	    new(this,nc,nr,inds,nrows);
	end_proc,


	# ================ #
	# row echelon form #
	# ================ #

	"pivot" = proc(A,i)
	local r;
	begin
	    if testargs() then
	        if domtype(A)<>this then TYPE_ERR end_if;
		if (i<0) or (i>NROWS(A)) then ROW_ERR end_if;
	    end_if;

	    r := ROWS(A)[i];
	    if r=[] then error("empty row") end_if;
	    [r[1][1],r[1][2]];
	end_proc,


	"pivots" = proc(A)
	local res,r,k;
	begin
	    res := [ 0 $ NROWS(A) ];
	    k := 0;
	    for r in ROWS(A) do
		if r=[] then next end_if;
		k := k+1;
		res[k] := r[1];
	    end_for;
	    if k<NROWS(A) then [op(res,1..k)] else res end_if;
	end_proc,


	# For GCD domains a primitive row echelon form is provided.
	  It is computed by a fraction-free elimination where every row is made
	  primitive by division by its GCD. #
	"gaussElim" = proc(A)
	local all,prim,ltr,pivs,pivlen,pivind,rk,pivrow,anrows,notSorted,
	      oldr,newr,early,tmp,piv,c,fd,r,pr,len,rind,i,j,k,l,q;
	begin
	    case args(0)
	    of 1 do
		all := FALSE;
		prim := FALSE;
		break;
	    of 2 do
		all := bool(args(2)=hold(All));
		prim := bool(args(2)=hold(Primitive));
		if not(all or prim) then error("unknown option") end_if;
		break;
	    of 3 do
		if {args(2),args(3)}<>{hold(All),hold(Primitive)} then
		    error("unknown option");
		end_if;
		all := TRUE;
		prim := TRUE;
		break;
	    otherwise ARG_ERR;
	    end_case;
	    if testargs() then
		if not(R::hasProp(Cat::IntegralDomain)) then
		    error("not a Cat::IntegralDomain");
		end_if;
		if prim and not(R::hasProp(Cat::GcdDomain)) then
		    error("primitive form only possible for a Cat::GcdDomain");
		end_if;
		if domtype(A)<>this then TYPE_ERR end_if;
	    end_if;

	    anrows := NROWS(A);
	    if prim then
		ltr := QMat(anrows,anrows,Qconvert(Rone),hold(Diagonal));
	    else
		ltr := Mat(anrows,anrows,Rone,hold(Diagonal));
	    end_if;
	    pivs := [ 0 $ anrows ];

	    r := ROWS(A);
	    for i from 1 to anrows do
		while r[i]<>[] do
		    if not(Riszero(r[i][1][2])) then break end_if;
		    r[i][1] := NIL;
		end_while;
	    end_for;

	    # bubble sort #
	    notSorted := TRUE;
	    while notSorted do
		notSorted := FALSE;
		oldr := r[1];
		for i from 2 to anrows do
		    newr := r[i];
		    if this::greater(newr,oldr) then
			r[i] := oldr;
			r[i-1] := newr;
			ltr := ltr::swapRow(ltr,i-1,i);
			notSorted := TRUE;
		    else oldr := newr end_if;
		end_for;
	    end_while;

	    early := FALSE;
	    for i from 1 to anrows do
		if r[i]=[] then
		    rk := i-1;
		    early := TRUE;
		    break;
		end_if;

		# search good pivot (with "short" row) #
		pivind := r[i][1][1];
		pivlen := nops(r[i]);
		pivrow := i;
		k := 0;
		for j from i+1 to anrows do
		    rind := this::indices(r[j]);
		    if rind=[] or pivind<>rind[1] then break end_if;
		    len := nops(rind);
		    k := k+1;
		    if len<pivlen then
			pivlen := len;
			pivrow := j;
		    end_if;
		end_for;

		if prim then
		    tmp := this::makePrimitive(r[pivrow]);
		    if op(tmp,1)<>Rone then
			r[pivrow] := op(tmp,2);
			q := Qinvert(Qconvert(op(tmp,1)));
			for l from 1 to anrows do
			    if not(Riszero(ltr[pivrow,l])) then
				ltr[pivrow,l] := Qmult(q,
						       Qconvert(ltr[pivrow,l]));
			    end_if;
			end_for;
		    end_if;
		end_if;
		piv := r[pivrow][1][2];
		pivs[i] := piv;

		# elimination #
		if not(iszero(k)) then
		    if pivrow<>i then
			pr := r[pivrow];
			r[pivrow] := r[i];
			r[i] := pr;
			ltr := ltr::swapRow(ltr,i,pivrow);
		    end_if;
		    if prim then pr := op(tmp,2)
		    else pr := r[i] end_if;
		    pr[1] := NIL;
		    for j from i+1 to i+k do
			newr := r[i+1];
			c := newr[1][2];
			newr[1] := NIL;
			newr := this::addRows(piv,newr,-c,pr);
			for l from 1 to anrows do
			    fd := piv*ltr[i+1,l] - c*ltr[i,l];
			    ltr[i+1,l] := fd;
			end_for;
			for l from i+2 to i*2+k+1-j do
			    r[l-1] := r[l];
			    ltr := ltr::swapRow(ltr,l-1,l);
			end_for;
			for l from i*2+k+2-j to anrows do
			    if not(this::greater(r[l],newr)) then break	end_if;
			    r[l-1] := r[l];
			    ltr := ltr::swapRow(ltr,l-1,l);
			end_for;
			r[l-1] := newr;
		    end_for;
		end_if;
	    end_for;

	    if early then pivs := [op(pivs,1..rk)]
	    else rk := anrows end_if;
	    A := new(this,NROWS(A),NCOLS(A),ALL_INDICES(A),r);
	    if all then (A,ltr,pivs,rk)
	    else A end_if;
	end_proc,


	# =========== #
	# arithmetics #
	# =========== #

	"_negate" = fun(this::map(args(1),Rnegate)),


	"_plus" = fun((case args(0)
	    of 1 do return(args(1));
	    of 2 do return(this::plus2(args(1),args(2)));
	    otherwise
		args(0) div 2;
		this::plus2(this::_plus(args(1..%)),
			    this::_plus(args(%+1..args(0))));
	    end_case)),


	"plus2" = proc(a,b)
	local i,j,k,nr,nc,inds,ra,rb,newrows,newr,ta,tb;
	begin
	    nr := NROWS(a);
	    nc := NCOLS(a);
	    inds := ALL_INDICES(a);
	    if nr<>NROWS(b) or inds<>ALL_INDICES(b) then
		error("incompatible matrices");
	    end_if;
	    ra := ROWS(a);
	    rb := ROWS(b);

	    newrows := [ 0 $ nr ];
	    for i from 1 to nr do
		newr := [ 0 $ nc ];
		k := 0;
		for j from 1 to nc do
		    ta := select(ra[i],
				 fun(bool(op(args(1),1)=args(2))),inds[j]);
		    tb := select(rb[i],
				 fun(bool(op(args(1),1)=args(2))),inds[j]);
		    if ta<>[] then ta := [op(ta,[1,2])] end_if;
		    if tb<>[] then tb := [op(tb,[1,2])] end_if;
		    t := Rplus(op(ta),op(tb));
		    if not(Riszero(t)) then
		        k := k+1;
			newr[k] := [op(inds,j),t];
		    end_if;
		end_for;
		newrows[i] := [op(newr,1..k)];
	    end_for;
	    new(this,nr,nc,inds,newrows);
	end_proc,


	"_mult" = fun((case args(0)
	    of 1 do return(args(1));
	    of 2 do return(this::mult2(args(1),args(2)));
	    otherwise
		args(0) div 2;
		this::mult2(this::_mult(args(1..%)),
			    this::_mult(args(%+1..args(0))));
	    end_case)),


	"mult2" = proc(a,b)
	local nca,nra,ncb,nrb,i,k,newr,r,res,ind,vtmp,allinds,dim,idx;
	begin
	    if domtype(b)<>this then TYPE_ERR end_if;

	    case domtype(a)
	    of this do return(this::thisMult(a,b));
	    of DOM_ARRAY do
		dim := [op(a,[0,2,2]),op(a,[0,3,2])];
		break;
	    of Mat do
		dim := a::dimen(a);
		break;
	    otherwise
		if domtype(a)<>R  then
		    a := Rcoerce(a);
		    if a=FAIL then TYPE_ERR end_if;
		end_if;
		return(this::map(b,fun(Rmult(args(2),args(1))),a));
	    end_case;

	    nra := op(dim,1);
	    nca := op(dim,2);
	    nrb := NROWS(b);
	    ncb := NCOLS(b);
	    allinds := ALL_INDICES(b);
	    if nca<>nrb then error("incompatible matrix dimensions") end_if;
	    res := [ [0 $ ncb] $ nra];
	    idx := [ 0 $ nra ];
	    r := ROWS(b);
	    for ind in allinds do
		vtmp := array(1..nra,[Rzero $ nra]);
		for i from 1 to nrb do
		    newr := r[i];
		    if newr=[] or newr[1][1]<>ind then next end_if;
		    for k from 1 to nra do
			if Riszero(a[k,i]) then next end_if;
			vtmp[k] := vtmp[k] + a[k,i]*newr[1][2];
		    end_for;
		    newr[1] := NIL;
		    r[i] := newr;
		end_for;
		for k from 1 to nra do
		    if Riszero(vtmp[k]) then next end_if;
		    idx[k] := indx[k]+1;
		    res[k][idx[k]] := [ind,vtmp[k]];
		end_for;
	    end_for;
	    for i from 1 to nra do
	        res[i] := [op(res[i],1..idx[i])];
	    end_for;
	    new(this,nra,nca,allinds,res);
	end_proc,


	"thisMult" = proc(a,b)
	local nr,nc,ra,rb,newrows,newr,inds,i,j,ta,tb,c,t;
	begin
	    nr := NROWS(a);
	    nc := NCOLS(a);
	    inds := ALL_INDICES(a);
	    if nr<>nc or NROWS(b)<>NCOLS(b) then
		error("not a square matrix")
	    end_if;
	    if inds<>ALL_INDICES(b) then error("incompatible matrices") end_if;

	    b := this::transpose(b);
	    newrows := [];
	    for ra in ROWS(a) do
		newr := [];
		j := 1;
		for rb in ROWS(b) do
		    c := [];
		    for i in inds do
			ta := select(ra,fun(bool(op(args(1),1)=args(2))),i);
			tb := select(rb,fun(bool(op(args(1),1)=args(2))),i);
			if ta<>[] and tb<>[] then
			    c := append(c,Rmult(op(ta,[1,2]),op(tb,[1,2])));
			end_if;
		    end_for;
		    t := Rplus(op(c));
		    if not(Riszero(t)) then
			newr := append(newr,[op(inds,j),t]);
		    end_if;
		    j := j+1;
		end_for;
		newrows := append(newrows,newr);
	    end_for;
	    new(this,nr,nc,inds,newrows);
	end_proc,


	"characteristic" = R::characteristic
):

unalias(NROWS,NCOLS,ALL_INDICES,ROWS,ARG_ERR,COL_ERR,ROW_ERR,TYPE_ERR):

