
#++ ---------------- UniRatF.mu ---------------------

Author: Winfried Fakler,  University of Karlsruhe
Date Created: 24 September 1997
Last Update : 12 October   1997

Description:
This file contains the domain for rational functions over the rationals or 
algebraic extenstions over the rationals. At the moment only univariate 
rational functions are supported. This restriction comes from the fact that 
currently a factorization over algebraic extensions is possible only for 
univariate polynomials.
NOTE: This domain is still under construction! Therefore nearly everything
      could be changed in the future.
      There will be an update with the next patch-version.
CAUTION: Be careful in using factor/Factor! If the argument contains something
         one would not expect for a univariate rational function, e.g a 
         second variable, it will failure! 

Call: (provisionally)

 Dom::UnivariateRationalFunctions(Var, minpolys, xis) 
 where
   Var        => indeterminate of type DOM_INDET; 
   minpolys   => list of minimal polynomials (written as expressions)
   xis        => list of indeterminates w.r.t minpolys

incomplete todo list:
- conjNorm
- algNormalRep 
- extendedEuclid w.r.t algNormalRep


Methods: 

 - used Parameter:
    ++ a : this Domain
    ++ n : DOM_INT

 - definingPolynomials()
    ++ returns a list of the defined minimal polynomials of this domain. 

Example:

>> RF:=Dom::UnivariateRationalFunctions(x,[aa^2+aa+1,bb^3-2],[aa,bb]);
                                              2        3
  Dom::UnivariateRationalFunctions(x, [aa + aa  + 1, bb  - 2], [aa, bb])

>> f:=RF((x^3+108)/(x^2+108));
                                  3
                                 x  + 108
                                 --------
                                  2
                                 x  + 108

>> Factor(f);
                     2                        2              2
            1(x + bb (- 3aa - 3))(x + 3bb )(x + 3aabb )
            ---------------------------------------------------
                      (x - 12aa - 6)(x + 12aa + 6)

>> f1:=x*(aa^2-1)/(bb^2*aa+4)+f;
               2                          3        2
      108aabb  + x(- 108aa - 216 ) + x (aa(bb  - 1) + 2) + 432
      --------------------------------------------------------------
                             2    2       2
                    108aabb  + x (aabb  + 4) + 432
>> Factor(f1);
                2                         3       2
      (108aabb  + x(- 108aa - 216) + x (aabb  - aa + 2) + 432)
      --------------------------------------------------------------
                     2
               (aabb  + 4)(x - 12aa - 6)(x + 12aa + 6)

++#

# Dependencies:

  ExprFld.mu
#

alias(
      Reduce(f) = proc(f) local num,den,i;
                    begin
                     # f:=extop(f,1);#
                      num:=expr(numer(f)): den:=expr(denom(f));
                      for i from nops(minpolys) downto 1 do
                        num:=divide(num,minpolys[i],[xis[i]],Rem);
                        den:=divide(den,minpolys[i],[xis[i]],Rem); 
                      end_for;
                      collect(num,[hold(Var)])/collect(den,[hold(Var)])
                    end_proc,
      rep(x) = extop(x,1)
      ):

sysassign(Dom::UnivariateRationalFunctions, DomainConstructor(

  # name: #            Dom::UnivariateRationalFunctions,
  # parameter: #       [ Var, minpolys, xis ],
  # static value: #    [ hasAlgExt ],
  # initialization: #  (if iszero(args(0)) or args(0) > 3 then 
                          error("wrong no of args")
                        end_if;
                        if domtype(Var) <> DOM_IDENT then
                          error("not a variable")
                        end_if; 
                        if args(0)=1 
                          then minpolys:=[]: xis:=[]: hasAlgExt:=FALSE
                          else  hasAlgExt:=TRUE
                        end_if; ),
  # super-domain: #     (if args(0)=1 then Dom::ExpressionField(normal)
                          else 
                           Dom::ExpressionField(Reduce(f), iszero@normal)
                         end_if),
  # categories: #       [ Cat::Field, Cat::DifferentialRing ],
  # axioms: #           [ Ax::normalRep ],
  # entries: #

 "definingPolynomials"=fun( minpolys ),

 "lcm" = proc(a, b)
    begin
      case args(0)
        of 2 do break;
        of 1 do return(this::gcd(a));
        of 0 do return(this::one);
         otherwise
           b:=this::lcm(args(2..args(0)))
      end_case;
      this::_mult(a, this::_divide(b, this::gcd(a,b)))
    end_proc,

 "D" = proc()
   local l;
   begin
     case args(0)
       of 1 do return( this::new(
               subs(D(rep(args(1))),map(op(xis),fun(D(args(1))=0)))
                                ) ); break;
       of 2 do l:=args(1);
               return( new(this, normal(subs(
                 eval(D(l,rep(args(2)))), map(op(xis),fun(D(l,args(1))=0))
                                ))) ); break;
       otherwise error("wrong no of args")
     end_case
   end_proc,

 "diff" = proc() 
    local vars;
    begin
      if args(0)=1 then return( args(1) )
       else
        vars := {args(2..args(0))};
        if nops(vars)=nops(vars minus {op(xis)}) then 
          return( this::new(diff(rep(args(1)), op(vars))) )
         else error("Illegal variable") end_if
      end_if
    end_proc,

 "factor" = (if hasAlgExt then
   proc(a)
    local num,den,i,AEbeg,F;
    begin
      a:=rep(a);
      num:=numer(a): den:=denom(a);
      AEbeg:="Dom::AlgebraicExtension(";
      F:="Dom::Rational";
      for i from 1 to nops(minpolys) do
        F:=AEbeg.F.",".expr2text(minpolys[i]=0).",".expr2text(xis[i]).")";
      end_for;
      F:=eval(text2expr(F));
      if den=1 then
        num:=faclib::algfactor(poly(num,[hold(Var)],F));
        return( map(num, expr) )
      else
        num:=faclib::algfactor(poly(num,[hold(Var)],F));
        den:=faclib::algfactor(poly(den,[hold(Var)],F));
        if nops(den) = 1 then
          den:=_concat(den, [-1])
        else
          num[1]:=expr(num[1])*expr(den[1])^(-1);
          den[1]:= NIL;
          for i from 2 to nops(den) step 2 do den[i]:=_negate(den[i]) end_for
        end_if;
        return( map(_concat(num,den),expr) )
      end_if
    end_proc
   end_if;),

# "Factor"=(Cat::FactorialDomain)::Factor #

 "Factor"= proc(a) 
   local i; 
   begin
     a:= this::factor(a);
     if nops(a) = 1 then
       a[1]
     else
       hold(_mult)(a[1], hold(_power)(a[2*i], a[2*i+1])
                                $ i=1..(nops(a) div 2))
     end_if
   end_proc

) ):

unalias( rep, Reduce ):

