# factorization of bivariate polynomials over a field F
             or of univariate polynomials over F[y]

  The algorithm implemented in faclib::FACTOR is based on Hensel lifting
  and short vector computation. It works for the factorization of an
  univariate polynomial over F[y][x] over any ring y. 
  Reference: von zur Gathen, Hensel and Newton Methods in Valuation Rings, Math.Comp.
		166(1984), pp. 637-661.
  The complexity is O(n^6 m^2 + n log q) operations in F_q when
  F=F_q, and where n = deg(f,x) and m = deg(f,y).
#

# factors a general bivariate polynomial f in F[x,y] where F is a field #
faclib::bifactor:=proc(f)
local factlist,result,F,l,i,j;
begin
   if type(f)<>DOM_POLY then error("not a polynomial") end_if;
   if nops(op(f,2))<>2 then error("bivariate polynomial expected") end_if;
   F:=op(f,3);
   if F::hasProp(Cat::Field)<>TRUE then
	 error("coefficient ring should be a field")
   end_if;
   l:=sqrfree(f);
   result:=[op(l,1)];
   for i from 2 to nops(l) step 2 do
	factlist:=faclib::bfactor(op(l,i));
	for j from 3 to nops(factlist) step 2 do
		factlist[j]:=op(l,i+1)*factlist[j];
	end_for;
	result[1]:=result[1]*factlist[1];
	result:=result.[op(factlist,2..nops(factlist))];
    end_for;
    result
end_proc:






# bfactor factors a bivariate squarefree polynomial f in F[x,y] 
  where F is a field #
faclib::bfactor := proc(f)
local x,y,n,m,F,R,xy;
begin
   F:=op(f,3);
   xy:=op(f,2); x:=op(xy,1); y:=op(xy,2);
   n:=degree(f,x); m:=degree(f,y);
   if n>m then R:=x; x:=y; y:=R end_if;
   # now degree(f,x)<=degree(f,y) #
   R:=Dom::DistributedPolynomial([y],F);
   f:=poly(f,[x],R);
   f:=faclib::udfactor(f);
   # f[1] is a polynomial in y #
   [(if degree(f[1])=0 then coeff(f[1],0)
   else op(factor(poly(f[1],xy,F))) end_if),
   (poly(f[2*n],xy,F),f[2*n+1])$hold(n)=1..nops(f) div 2]
end_proc:

# factors an univariate polynomial f over R[x] where R=F[y] #
faclib::udfactor := proc(f) local l,n,x,R,e;
begin
   x:=op(f,2); R:=op(f,3);
   if nops(x)=1 then x:=op(x) else error("univariate polynomial expected") end_if;
   l:=[content(f)]; f:=primpart(f);
   if (n:=ldegree(f))>0 then
      l:=append(l,subsop(f,1=x),n); f:=divide(f,subsop(f,1=x^n),hold(Quo));
   end_if;
   if degree(f)=0 then return(l) end_if;
   repeat
      e:=faclib::FACTOR(f,x,R);
      if e<>FAIL then 
         userinfo(1,"found one factor:",e);
         l:=append(l,e,1); f:=divide(f,e,hold(Quo)) 
      end_if;
   until e=FAIL or degree(f)<=1 end_repeat;
   append(l,f,1)
end_proc:

# faclib::findN(n,m,k) gives an upper bound for the number of steps
  in Hensel lifting, where n is the degree in x of f (main unknown),
  m is the degree in y of f and k the degree in x of the factor h of 
  f mod p used in Hensel lifting. The upper bound proved by von zur
  Gathen is ceil(2*n*m/k). #
faclib::findN:=proc(n,m,k) begin ceil(2*n*m/k) end_proc:

# returns a proper factor of a squarefree univariate polynomial f in R[x] where R=F[y],
  or FAIL if f is irreducible. We suppose in addition that:
  (1) content(f)=1 i.e. f is primitive
  (2) ldegree(f)=0

  Input: f is a univariate polynomial (DOM_POLY)
         x is the variable of f = op(f,[2,1])
         R is the coefficient domain of f = op(f,3)
#
faclib::FACTOR := proc(f,x,R)
local n,a,y,F,i,df,N,l,g,h,fmodp,e,s,t,tmp,k,z,j,e1,K,pnormg,ptoi,dg,Pone,R_divide,Rmult,m,st,ok,p,q;
begin
   y:=op(R::constructor_args[1]); F:=R::constructor_args[2];
   df := Dpoly(f);
   if iszero(df) then
      f:=poly(expr(f),[y],Dom::DistributedPolynomial([x],F));
      f:=faclib::FACTOR(f,y,op(f,3));
      if f<>FAIL then f:=poly(expr(f),[x],R) end_if;
      return(f)
   end_if;
   n:=degree(f);
   # step 1 #
   a:=lcoeff(f);
   m:=max(degree(coeff(f,i)) $ i=0..n); 
   userinfo(1,"n=",n,"m=",m);
   # step 2 #

   # n e w #
   # always try evaluation point zero first (simple substitution) #

  if not iszero(coeff(a,0)) and not iszero(resultant
	 (poly(mapcoeffs(f,evalp,y=F::zero),[x],F),
          poly(mapcoeffs(df,evalp,y=F::zero),[x],F) ) )
	then p:=F::zero


  elif (F::constructor<>Dom::IntegerMod and F::constructor<>Dom::GaloisField) 
	or F::size > 3*degree(f,x) then

    ok:=FALSE; repeat   
      # try a linear random polynomial y+a (we are in a field) #
      p:=F::random(); 
      # y+c divides a iff subs(a,y=c)=0
        and y+c divides resultant(f,df) iff resultant(subs(f,y=c),subs(df,y=c))=0 #
     if not iszero(evalp(a,y=-p)) then
         g:=poly(mapcoeffs(f,evalp,y=-p),[x],F);
         h:=poly(mapcoeffs(df,evalp,y=-p),[x],F);
         if not iszero(resultant(g,h)) then ok:=TRUE end_if
      end_if;
     until ok end_repeat; 

   else

   # for finite fields: find linear polynomial that does not divide the 
     resultant of f and df, if such a polynomial exists #

   # new: try some random choices first #

     ok:=FALSE; 
     for i from 1 to 10 do
	 p:=F::random();
 	      if not iszero(evalp(a,y=-p)) then
	         g:=poly(mapcoeffs(f,evalp,y=-p),[x],F);
       	         h:=poly(mapcoeffs(df,evalp,y=-p),[x],F);
         	 if not iszero(resultant(g,h)) then 
			ok:=TRUE;
			break;
		 end_if
      	      end_if;
     end_for; 

     if not ok then
         q:=F::size;
         g:=resultant(f,df);
     
         h:=R(y^q-y);
     
		# as y^q-y is the product of all linear polynomials,
		  (y^q-y)/gcd(y^q-y,resultant(f,df)) is the product
  		   of all (y-c) where c runs through all suitable
		   evaluation points; they can be found by factoring #
	
	 g:=divide(h,gcd(h,g),hold(Exact));
	 g:=divide(g,gcd(g,a),hold(Exact));

		# only non-roots of a remain #
	
	 if degree(g)=0 then 
		# bad luck : there is no good evaluation point #
		# field has to be extended; if an extension of prime 
		  degree > max(m,n) is chosen, the factorization remains
		  the same 
		  (see v.z.Gathen, Factoring Sparse Multivariate Polynomials,
		   Proc. 24th Symp. Found. Comp. Sci.,IEEE, 1983, pp.172-179) #
	
		p:=nextprime(max(m,n)+1);
		userinfo(2,"Field does not contain good evaluation point");
		userinfo(2,"Constructing extension of degree",p);
		h:=Dom::DistributedPolynomial([y],Dom::GaloisField(
			F,p));
		f:=poly(expr(f),[x],h);
		g:=faclib::FACTOR(f,x,h);
		if g=FAIL then
			return(FAIL)
		else
		   return(poly(g,[x],R));
		end_if;
	  else		
		h:=factor(g);
		p:=coeff(op(h,2),0);
		# simply choose the first factor (anyone would be o.k) #
	  end_if
     end_if
    end_if;

	# end of recent changes #

   p:=R(poly(hold(_plus)(y,p),[y],F));
   userinfo(1,"using p=",p);
   # step 3: Factorization modulo p #
   fmodp := subsop(mapcoeffs(f,divide,p,hold(Rem)),3=F);
   st:=time();
   l:=factor(fmodp);
   userinfo(1,"time for factorization mod p=",time()-st);
   if nops(l)=3 then return(FAIL) end_if; # f is irreducible #
   # choose h with the highest possible degree #
   h:=l[2];
   for i from 2 to nops(l) div 2 do
      if degree(l[2*i])>degree(h) then h:=l[2*i] end_if
   end_for;
   g:=divide(fmodp,h,hold(Quo));
   userinfo(2,"g=",g);
   userinfo(2,"h=",h);
   # step 4: Hensel lifting #
   st:=time();
   tmp:=pdioe(g,h,poly(1,[x],F)); s:=tmp[1]; t:=tmp[2]; # s*g + t*h = 1 #
   g:=subsop(g,3=R); h:=subsop(h,3=R);
   s:=subsop(s,3=R); t:=subsop(t,3=R);
   Pone:=poly(1,[x],R);
   k:=degree(h); dg:=n-k; # degree that g should have #
   N:=faclib::findN(n,m,k); 
   userinfo(1,"N=",N,"degree(h)=",k);
   pnormg:=poly(x^(dg+1),[x],R);
   ptoi:=p;
   # h is monic, thus the divisor is always 1 #
   R_divide:=R::_divide; R::_divide:=fun(args(1));
   Rmult:=R::_mult; R::_mult:=fun(divide(Rmult(args()),ptoi,hold(Rem)));
   for i from 2 to N do 
      userinfo(2,"lifting to order ".i);
      ptoi:=Rmult(ptoi,p); # do not use _mult here !!! #
      e:=f-g*h;
      tmp:=divide(s*e,h); # s*e = tmp[1]*h + tmp[2] #
      h := h + tmp[2];
      if i<N then # only h is needed at the end #
         e:=divide(e,pnormg,hold(Rem));
         g:=g + t*e + g*tmp[1];
         g := divide(g,pnormg,hold(Rem)); # g <- g mod x^(n-k+1) #
      end_if;
   end_for;
   userinfo(1,"time for Hensel lifting=",time()-st);
   # restore domain operations #
   R::_divide:=R_divide; R::_mult:=Rmult;
   # step 5: short vector computation #
   p:=poly(p,[x],R);
   g:=[[coeff(p^N*poly(x^i,[x],R),j) $ j=0..n-1] $ hold(i)=0..k-1,
       [coeff(h*poly(x^i,[x],R),j) $ j=0..n-1] $ hold(i)=0..n-k-1];
   st:=time();
   g:=faclib::ReducedBasis(g,n,R); l:=g[2]; g:=g[1];
   userinfo(1,"time for short vector computation=",time()-st);
   z:=g[1]; # short vector #
   userinfo(2,"short vector is",z);
   z:=_plus(poly(z[j],[x],R)*poly(x^(l[j]-1),[x],R) $ j=1..n);
   userinfo(2,"short polynomial is",z);
   # step 6 #
   K:=Dom::Fraction(R);
   f:=poly(f,[x],K); z:=poly(z,[x],K);
   st:=time();
   e1:=gcd(f,z);
   userinfo(1,"time for gcd=",time()-st);
 
   if e1<>poly(1,[x],K) then
	 primpart(mapcoeffs(poly(numer(expr(e1)),[x],R),_mult,a))
   else 
	 FAIL
   end_if
end_proc:

# Input: f[1] ... f[n] are linearly independent row vectors in R^n, where (R,w) is
  a non-Archimedian Euclidean valuation ring.
  Output: a reduced sequence g[1] ... g[n] and a (n x n) permutation matrix A
  such that g1.A ... gn.A generate the R-module M = \sum fi.R 
  (A is represented in linear form: the i-th base vector is in column A[i]).
  w is Euclidean means that it exists 0 < beta < 1 such that for all a,b in R:
  (1) w(a) >= 1 if a<>0
  (2) it exists q in R such that w(a-q*b) <= beta*w(b) if b<>0
  the property (2) is some kind of Euclidean division. It is assumed that
  the element q is returned by the method pseudoDivide(a,b).

  Examples of valuations are:
  (i) R=Z and w(a)=|a| with beta=1/2 and pseudoDivide(a,b)=round(a/b)
  (ii) R=F[y] and w(a)=2^degree(a) with beta=1/2 and pseudoDivide(a,b)=quo(a,b)
#

faclib::ReducedBasis := proc(g,n,R)
local k,A,i,u,q,w,ww,v,j,tmp,m;
begin
   userinfo(2,"compute a reduced basis of dimension ".n." over",R);
   w:=fun((if iszero(args(1)) then 0 else 2^degree(args(1)) end_if));
   ww:=fun(max(op(map(args(1),w))));
   # convert from DistributedPolynomial to DOM_POLY #
   for i from 1 to n do g[i]:=map(g[i],extop,1) end_for;
   k:=1; A:=[$1..n];
   while k<=n do
      u:=ww(g[k]);
      m:=k;
      for i from k+1 to n do
         if (q:=ww(g[i]))<u then m:=i; u:=q end_if
      end_for;
      if k<>m then
         # exchange vectors k and m #
	 q:=g[k]; g[k]:=g[m]; g[m]:=q;
      end_if;
      for i from 1 to k-1 do
         q:=divide(g[k][i],g[i][i],hold(Quo));
         if not iszero(q) then
            for j from 1 to n do g[k][j]:=g[k][j]-q*g[i][j] end_for;
         end_if
      end_for;
      q:=ww(g[k]);
      if q=u then # the valuation did not decrease #
         v:=w(g[k][k]);
         if v<q then # interchange column k and that with the biggest valuation #
	    for m from k+1 to n do
               if w(g[k][m])=q then break end_if
	    end_for;
            # exchange column k and column m #
	    for i from 1 to n do
	       q:=g[i][k]; g[i][k]:=g[i][m]; g[i][m]:=q;
	    end_for;
	    tmp:=A[k]; A[k]:=A[m]; A[m]:=tmp;
         end_if;
         k:=k+1;
      else # necessary ww(g[k])<u #
         k:=1;
         for i from 1 to k-1 do
            if ww(g[i])<=q then k:=i end_if
         end_for
      end_if
   end_while;
   g,A
end_proc:
