# factorization of univariate polynomials over Z[x]
  
  The algorithm implemented in faclib::FACTOR is based on Hensel lifting
  and short vector computation. 
  Reference: von zur Gathen and Gerhard, Modern Computer Algebra, chap. 17
 and von zur Gathen, Hensel and Newton Methods in Valuation Rings, Math.Comp.
		166(1984), pp. 637-661.
  The complexity is O~(n^12 + n^10 log^2 |f|) bit operations
  where n = deg(f) and |f| is the |f|=norm(f,2).

Examples:
>> faclib::lll(poly(x^4-10*x^2+1));

                                  2    4
                    [1, poly(-10 x  + x  + 1, [x]), 1]

>> faclib::lll(poly(x^8-40*x^6+352*x^4-960*x^2+576));
#

# factors a univariate polynomial f in Z[x] #
faclib::lll:=proc(f)
local factlist,result,l,i,j;
begin
   if lazy_and(args(0)=2,args(2)<>hold(Quadratic)) then
	error("Only admissible second argument is Quadratic") 
   end_if;
   if type(f)<>DOM_POLY then error("not a polynomial") end_if;
   if nops(op(f,2))<>1 then error("univariate polynomial expected") end_if;
   if op(f,3)<>Expr or map({coeff(f)},type)<>{DOM_INT} then
      error("integer coefficients expected") end_if;
   l:=sqrfree(f);
   result:=[op(l,1)];
   for i from 2 to nops(l) step 2 do
	if args(0)=1 then
		factlist:=faclib::lll2(op(l,i));
	else
		factlist:=faclib::lll2(op(l,i),hold(Quadratic));
	end_if;
	for j from 3 to nops(factlist) step 2 do
		factlist[j]:=op(l,i+1)*factlist[j];
	end_for;
	result[1]:=result[1]*factlist[1];
	result:=result.[op(factlist,2..nops(factlist))];
    end_for;
    result
end_proc:

# factors an univariate polynomial f over Z[x] #
faclib::lll2:=proc(f) local l,n,x,e;
begin
   x:=op(f,[2,1]);
   l:=[content(f)]; f:=primpart(f);
   if (n:=ldegree(f))>0 then
      l:=append(l,subsop(f,1=x),n); f:=divide(f,poly(x^n),hold(Quo));
   end_if;
   if degree(f)=0 then return(l) end_if;
   repeat
      if args(0)=1 then
      	 e:=faclib::lll3(f,x);
      else
	 e:=faclib::lll3(f,x,hold(Quadratic));
      end_if;
      if e<>FAIL then 
         userinfo(1,"found one factor:",e);
         l:=append(l,e,1); f:=divide(f,e,hold(Quo)) 
      end_if;
   until e=FAIL or degree(f)<=1 end_repeat;
   append(l,f,1)
end_proc:

# returns a proper factor of a squarefree univariate polynomial f in Z[x],
  or FAIL if f is irreducible. We suppose in addition that:
  (1) content(f)=1 i.e. f is primitive
  (2) ldegree(f)=0

  Input: f is a univariate polynomial (DOM_POLY)
         x is the variable of f = op(f,[2,1])
#
faclib::lll3:=proc(f,x)
local n,a,i,df,N,l,g,h,fmodp,e,s,t,ss,tt,uu,tmp,k,z,j,e1,pnormg,ptoi,dg,Pone,
B,ok,p,gg,hh;
begin
   df := Dpoly(f);
   n:=degree(f);
   # step 1 #
   a:=lcoeff(f); B:=2^(2*n^2)*norm(f,2)^(2*n);
   userinfo(1,"n=".n." B=".expr2text(B));
   # step 2 #
   ok:=FALSE; 
   # find p not dividing a*res(f,f') #
   p:=1; repeat
      p:=nextprime(p+1);
      if a mod p<>0 then
         g:=poly(f,IntMod(p));
         g:=gcd(g,Dpoly([1],g));
         ok:=bool(degree(g)=0);
      end_if;
   until ok end_repeat;
   userinfo(1,"using p=".p);
   # step 3: Factorization modulo p #
   fmodp := poly(f,IntMod(p));
   l:=factor(fmodp);
   if nops(l)=3 then return(FAIL) end_if; # f is irreducible #
   # choose h with the highest possible degree #
   h:=l[2];
   for i from 2 to nops(l) div 2 do
      if degree(l[2*i])>degree(h) then h:=l[2*i] end_if
   end_for;
   g:=divide(fmodp,h,hold(Quo));
   userinfo(2,"g=",g,"h=",h);
   # step 4: Hensel lifting #
   tmp:=pdioe(g,h,poly(1,[x],IntMod(p))); s:=tmp[1]; t:=tmp[2]; # s*g + t*h = 1 #
   Pone:=poly(1,[x],IntMod(p));
   k:=degree(h); dg:=n-k; # degree that g should have #
   N:=ceil(ln(B)/ln(p));
   userinfo(1,"N=".N." degree(h)=".k);
   pnormg:=poly(x^(dg+1),[x],IntMod(p));
   ptoi:=p; i:=1;
   # h is monic, thus the divisor is always 1 #
   if args(0)=2 then # use linear lifting #
   while ptoi<=B do
      ptoi:=ptoi*p; i:=i+1;
      userinfo(2,"lifting to ".ptoi);
      g:=poly(g,IntMod(ptoi)); h:=poly(h,IntMod(ptoi));
      e:=poly(f,IntMod(ptoi))-g*h;
      tmp:=divide(poly(s,IntMod(ptoi))*e,h); # s*e = tmp[1]*h + tmp[2] #
      h := h + tmp[2];
      if i<N then # only h is needed at the end #
         g:=g + poly(t,IntMod(ptoi))*e + g*tmp[1];
      end_if;
      userinfo(3,"g=".expr2text(expr(g))." h=".expr2text(expr(h)));
  end_while;
   else # quadratic lifting #
      while ptoi<=B do
         ptoi:=ptoi*ptoi;
         i :=2*i;
         userinfo(2,"lifting to ".ptoi); 
         g:=poly(g,IntMod(ptoi)); h:=poly(h,IntMod(ptoi));
         e:=poly(f,IntMod(ptoi))-g*h;
         s:=poly(s,IntMod(ptoi));
         tmp:=divide(s*e,h); # s*e = tmp[1]*h + tmp[2] #
         h := h + tmp[2];
         # try lattice reduction #
         hh:=poly(h,[x],Expr);
         gg:=[poly(p^i*x^j,[x])$hold(j)=0..k-1,
              hh*poly(x^j,[x])$hold(j)=0..n-k-1];
         gg:=[[coeff(gg[i],j) $ hold(i)=1..n] $ hold(j)=0..n-1];
         gg:=lllint(gg,All)[2];
        	userinfo(1,"computed short vectors at time=",time());
         z:=poly(_plus(gg[j][1]*x^(j-1)$hold(j)=1..n),[x]);
         userinfo(1,"short polynomial is",expr(z));
         # step 6 #
         e1:=gcd(f,z);
         if degree(e1)<>0 then 
                return(primpart(mapcoeffs(e1,_mult,a))) 
         end_if;

         if i<N then # only h is needed at the end #
		t:=poly(t,IntMod(ptoi));
		g:=g + t*e + g*tmp[1];
	        uu:=divide(s*(s*g-poly(1,[x],IntMod(ptoi))),h);
		ss:=op(uu,2);
		uu:=op(uu,1);	
		tt:=t*(t*h+s*g-poly(1,[x],IntMod(ptoi)))+g*(uu+t*s);
                s:=s-ss;
                t:=t-tt;
	 end_if;
         if i < N then
        	 userinfo(3,"g=".expr2text(expr(g))." h=".expr2text(expr(h)));
         else
                userinfo(3,"h=".expr2text(expr(h)));
	 end_if;
       end_while;
    end_if;
   userinfo(1,"finished Hensel lifting at time=".time());
   # step 5: short vector computation #
   h:=poly(h,[x],Expr);
   g:=[poly(p^N*x^i,[x])$hold(i)=0..k-1, h*poly(x^i,[x])$hold(i)=0..n-k-1];
   g:=[[coeff(g[i],j) $ hold(i)=1..n] $ hold(j)=0..n-1];
   g:=lllint(g,All)[2];
   userinfo(1,"computed short vectors at time=",time());
   z:=poly(_plus(g[j][1]*x^(j-1)$j=1..n),[x]);
   userinfo(1,"short polynomial is",expr(z));
   # step 6 #
   e1:=gcd(f,z);
   if degree(e1)<>0 then primpart(mapcoeffs(e1,_mult,a)) else FAIL end_if
end_proc:
