# $Date: 1999/08/04 09:10:34 $ $Author: fakler $ $Revision: 1.1.2.2 $ #

#++ ---------------- diracInt.mu ---------------------

Author: Winfried Fakler,  University of Paderborn
Date Created:  2 June 1999
Last Update :  7 June 1999


Description:
This file contains functions for integrating generalized functions containing
Dirac's delta function.
							  
Functions: 

 - used Parameter:
    ++ f,a,b :DOM_EXPR
    ++ x     :DOM_IDENT				  
    ++ f     = (complex) generalized function (distribution)
    ++ x     = indeterminate of f					  
    ++ a,b   = constant expression w.r.t. x, representing lower and upper limit

 - intlib::diracInt(f,x)
    ++ returns an (formal) antiderivative of the generalized function f 
    ++ w.r.t. x, wherby the antiderivative F is defined as follows:
    ++
    ++              F:=int(g(x)*dirac(p(x)),x)
    ++    and
    ++              F= sum( g(x0)/diff(p,x)(x0)*heaviside(x-x0) with p(x0)=0 )
    ++ NOTE: p(x) needs to be solvable by the method linsolve.
                               
 - intlib::diracDef(f,x,a,b)
    ++ returns the definite integral of the generalized function f 
    ++ w.r.t. x in the range of a to b, wherby the integral F is defined
    ++ as follows:
    ++              F:=int(g(x)*dirac(x-x0),x=a..b)
    ++    and
    ++             { f(x0)       a<x0<b and g continuous in x0
    ++        F =  { undefined   a=x0 or b=x0
    ++             { 0           x0 not in (a,b)

See:
  [1] Foellinger, O. (1986). Laplace- und Fourier-Transformation. Huethig, 
      4. Auflage.

Examples:
++#

intlib::diracInt:= 
  proc(_f_,_x_) 
    local g,delta,s,r,x0,gx0,p;
  begin
 //print("entering diracInt with ",_f_,_x_);
    case type(_f_)
      of "_plus" do //print("case _plus");
        return( map(_f_, proc(e,_x_) begin (if hastype(e,"dirac") then
                            intlib::diracInt(e,_x_) 
                          else intlib::int(e,_x_) 
                          end_if) end_proc, _x_)  ); break;
        //return(map(_f_,int,_x_));
      of "dirac" do //print("case dirac");
        g:=1; delta:=_f_; break;
      of "_mult" do //print("case _mult");
        s:=split(_f_,hastype,"dirac");
        if type(s[1])="dirac" then delta:=s[1]; g:=s[2]*s[3] else return(0)
        end_if; break;
      otherwise //print("case otherwise");
        return(FAIL);
    end_case;

    // now _f_(_x_)=g(_x_)*delta(p(_x_))
    p:=op(delta); 
    if degree(p,[_x_],_x_)=1 then
      r:=0;
      x0:=-coeff(p,[_x_],_x_,0)/coeff(p,[_x_],_x_,1);
      if not has(x0,I)  //heaviside at most nonzero for real values
         and traperror((gx0:=eval(subs(g/diff(p,_x_),_x_=x0))))=0
                        // g continuous in x0
        then
          if domtype(coeff(p,[_x_],_x_,1))=DOM_IDENT then
            r:=r+gx0*heaviside(p);
          else r:=r+gx0*heaviside(_x_-x0); end_if;
        else return(FAIL)
      end_if;
      r;
    else FAIL
    end_if;
  end_proc:

        
intlib::diracDef:=
  proc(_f_,_x_,a,b,pv) 
    local g,delta,s,r,x0,gx0,p;
  begin
 //print("entering diracDef with ",_f_,_x_);
    case type(_f_)
      of "_plus" do //print("case _plus");
        return( map(_f_, proc(e,_x_) begin
                          if hastype(e,"dirac") then
                            intlib::diracDef(e,_x_,a,b,pv) 
                          else intlib::definite(e,_x_,a,b,pv) 
                          end_if
                       end_proc, _x_)); break;
      of "dirac" do //print("case dirac");
        g:=1; delta:=_f_; break;
      of "_mult" do //print("case _mult");
        s:=split(_f_,hastype,"dirac");
        if type(s[1])="dirac" then delta:=s[1]; g:=s[2]*s[3] else return(0)
        end_if; break;
      otherwise //print("case otherwise");
        return(FAIL);
    end_case;

    // now f(_x_)=g(_x_)*delta(p(_x_))
    p:=op(delta);
    if degree(p,[_x_],_x_)=1 then
      r:=0;
      x0:=-coeff(p,[_x_],_x_,0)/coeff(p,[_x_],_x_,1);
      if a=x0 or b=x0 then return(undefined)
      elif not has(x0,I) //heaviside at most nonzero for real values
        and (is(a<x0) and is(x0<b))=TRUE // x0 element in range (a,b)
        and traperror((gx0:=eval(subs(g/diff(p,_x_),_x_=x0))))=0
                         // g continuous in x0
        then r:=r+gx0;
        else return(FAIL)            
      end_if;
      r;
    else FAIL
    end_if;
  end_proc:
        