# #
# $Date: 1997/05/28 17:03:32 $  $Author: frankp $   $Revision: 1.20 $ #
# #
# frankp, 1994/08/16 #
# paul, rouillie@lorraine.loria.fr (Hessenberg's algorithm), 1996/12/18 #

#++
  charpoly.mu

    linalg::charPolynomial() -- returns the characteristic polynomial

	linalg::charPolynomial(A,x)
	
	A: square matrix over a commutative ring
	x: valid polynomial indeterminate
	 
	Computes the characteristic polynomial of the matrix A
	in x, i.e. det( x*E(n)-A ), where E(n) is the 
	identity matrix with n=dimen(A).

	The coefficient domain of A has to be a commutative ring.

        Hessenberg's algorithm is used for matrices over Dom::Float and
        Dom::IntegerMod. 

        The algorithm works in any field, and requires only O(n^3) coefficient 
        operations instead of O(n^4) for other methods like det(x*Id-A).
        But the entries of the matrix could swell extremely, so that
        the usage of Hessenberg's algorithm is restricted to the stated 
        coefficient domains.

  	Reference: A Course in Computational Algebraic Number Theory,
 	           by Henri Cohen, GTM 138, Springer Verlag, 
	           algorithm 2.2.9 page 55 
++#

linalg::charPolynomial:= proc(H,x)
    local r;
begin
    if testargs() then
        if args(0) <> 2 then error("expecting 2 arguments") end_if;
        if H::hasProp( Cat::MatrixCat ) <> TRUE then
            error("first argument is not of 'Cat::MatrixCat'")
        end_if;
        r:= H::dimen(H);
        if op(r,1) <> op(r,2) then
            error("expecting a square matrix")
        end_if:
        if poly(x) = FAIL then
            error("illegal indeterminate")
        end_if
    end_if;

    r:= H::coeffRing;
    if r::hasProp( Cat::Field ) = TRUE then
        if r = Dom::Float or r::constructor = Dom::IntegerMod then
            Dom::DistributedPolynomial([x],r)( linalg::hessenberg( H,x ) )
        else
            linalg::det( linalg::charMatrix(H,x) )
        end_if 
    elif r::hasProp( Cat::CommutativeRing ) = TRUE then
        linalg::det( linalg::charMatrix(H,x) )
    else
        error("expecting matrix over 'Cat::CommutativeRing'")
    end_if
end_proc:

linalg::hessenberg:= proc(H,x)
    local m,n,i,t,j,u,p,R,
          Rzero, Riszero, Rmult, Rdivide, Rplus;
begin
    # step 1: initialize #
    n:= H::dimen(H)[1]; 
    R:= H::coeffRing; 

    Rzero:= R::zero;
    Riszero:= R::iszero;
    Rmult:= R::_mult;
    Rdivide:= R::_divide;
    Rplus:= R::_plus;

    # step 2: search for non-zero #
    for m from 2 to n-1 do
        i:= m+1; while i<=n and Riszero((t:= H[i,m-1])) do i:= i + 1 end_while;

        if i <= n then
            if i > m then
                H:= assign_elems(H,([[i,j],H[m,j]], [[m,j],H[i,j]]) $ j = m-1..n);
                H:= assign_elems(H,([[j, i], H[j, m]], [[j, m], H[j, i]]) $ j = 1..n);
            end_if;

            # step 3: eliminate #
            for i from m+1 to n do
                if not Riszero( (u:= Rdivide(H[i,m-1],t)) ) then
                    H:=assign_elems(H,[[i,j],Rplus(H[i,j],-Rmult(u,H[m,j]))]
                          $ j=m..n, [[i,m-1],Rzero]);
                    H:=assign_elems(H,[[j,m],Rplus(H[j,m],Rmult(u,H[j,i]))]
                          $ j=1..n);
                end_if
            end_for
        end_if;
   end_for;

   # step 5 #
   p[0]:= poly(1,[x],R);
   # step 6 #
   for m from 1 to n do
       p[m]:= poly(hold(_plus)(x,-H[m,m]),[x],R) * p[m-1]; 
       t:= R::one;
       for i from 1 to m-1 do
           t:= Rmult( t, H[m-i+1,m-i] );
           p[m]:= p[m] - multcoeffs(p[m-i-1],Rmult(t,H[m-i,m]))
       end_for
   end_for;
   p[n]
end_proc:

# end of file #
