# #
# $Date: 1999/09/30 17:23:22 $  $Author: frankp $   $Revision: 1.6.2.1 $ #
# #
# frankp, 16/05/95 #

#++
  cholesky.mu

        linalg::cholesky -- computes the Cholesky decomposition

        cholesky(S [,isPositiveDefinite])
	cholesky(H [,isPositiveDefinite])

	S	          : symmetric matrix
        H                 : Hermitian matrix
        isPositiveDefinite: (optional) ident

	cholesky(H) computes the Cholesky decomposition for a 
	positive definite matrix H over a field.
	This is a decomposition in a product H=G*transpose(G) such that
	G is lower triangular and has positive entries on the main
	diagonal (the Cholesky factor of H).
	If H is not positive definit (i.e. such a decomposition does
	not exist), then an error occures.
	
	The call cholesky(H,isPositiveDefinite) just tests if the 
        Hermitian matrix H is really positive definite and returns 
        TRUE and FALSE, respectively.

	The same holds if the coefficient ring of the regarded
	matrix does not provide a method "conjugate". Then the matrix
	must be symmetric.
++#

linalg::cholesky := proc(A)
    name linalg::cholesky;
    local R, Rplus, Rzero, Rmult, Rcoerce, R_divide, R_negate, Rconjugate,
	  t, i, j, k, n;
begin
    if testargs() then
	if args(0) < 1 or args(0) > 2 then
	    error("expecting 1 or 2 arguments")
	end_if;
	if A::hasProp( Cat::MatrixCat ) <> TRUE then
	    error("first argument is not of 'Cat::MatrixCat'")
	end_if;
	R := A::coeffRing;
	if not R::hasProp( Cat::Field ) then
	    error("expecting matrix over a field")
	end_if;
	if R::conjugate = FAIL then
	    if not A::equal(A, A::transpose(A)) then
		error("expecting a symmetric matrix")
	    end_if
	elif not A::equal(A, A::transpose(A::conjugate(A))) then
	    error("expecting a Hermitian matrix")
	end_if;
	if args(0) = 2 then
	    if args(2) <> hold(isPositiveDefinite) then
		error("option 'isPositiveDefinite' expected")
	    end_if
	end_if
    end_if;

    t := args(0);
    R := A::coeffRing;
    Rzero := R::zero;
    Rplus := R::_plus;
    Rmult := R::_mult;
    Rcoerce := R::coerce;
    R_negate := R::_negate;
    R_divide := R::_divide;
    Rconjugate := R::conjugate;
    if Rconjugate = FAIL then Rconjugate := id end_if;

    n := op(A::dimen(A),1);
    for j from 1 to n do
	for k from 1 to j-1 do
	    A[j,j] := Rplus( A[j,j],R_negate(Rmult(A[j,k],A[j,k])) );
	end_for;

	k := A[j,j];
	userinfo(1,"check if ",k," is real and positive");
	if Rconjugate(k) <> k then
	    # A[j,j] is not real, hence A is not pos. def. #
	    if t = 1 then
                error("given matrix is not positive definit")
            else
                A := FAIL;
                break # j #
            end_if
        elif traperror( bool(k<=Rzero) ) <> 0 then
            error("unable to compute the Cholesky factor over ".expr2text(R))
	elif k <= Rzero then
	    # A[j,j] is not positive, hence A is not pos. def. #
	    if t = 1 then
		error("given matrix is not positive definit")
	    else
		A := FAIL; 
		break # j #
	    end_if
	end_if;

	# compute G #
	(A[j,k] := Rzero) $ hold(k)=j+1..n; # upper triangular of G is 0 #

	if (A[j,j]:= Rcoerce( sqrt( A[j,j] ) )) = FAIL then
	    error("unable to compute the Cholesky factor over ".expr2text(R))
	end_if;

	for i from j+1 to n do
	    for k from 1 to j-1 do
		A[i,j] := Rplus( A[i,j],R_negate(Rmult(A[i,k],A[j,k])) )
	    end_for;
	    A[i,j] := R_divide( A[i,j],A[j,j] )
	end_for
    end_for;

    if t = 2 then bool( A <> FAIL ) else A end_if
end_proc:

# end of file #
