# #
# $Date: 1997/03/19 11:21:00 $  $Author: frankp $   $Revision: 1.5 $ #
# #
# zimmerma, 9/10/95 #

#++
  hermite.mu

	linalg::hermiteForm  --  compute the Hermite normal form of a matrix

	hermiteForm( A )

	A - Matrix

	hermiteForm(A) computes the Hermite normal form of the matrix A,
	i.e. an upper-triangular matrix such that
	val(A[i,j])<=beta*val(A[j,j]) for j>i, and if A is a square matrix,
	the product of the diagonal elements equals det(A).

	The Hermite normal form is obtained from A by:

	(i) subtracting the product of a row by a coefficient to another row
	(ii) exchanging two rows
	(iii) multiplying a row by -1.

	This algorithm needs a pseudo-division method Div such that 
	if q=Div(a,b), then val(a-q*b)<=beta*val(b).
	For integer entries, val(k)=abs(k), beta=1/2 and Div(a,b)=round(a/b).

	Examples:
	>> export(Dom):
	>> MZ:= Matrix(Integer):
	>> A:=MZ( [[9,-36,30], [-36,192,-180], [30,-180,180]] ):
	>> linalg::hermiteForm( A );

				 +-         -+
				 | 3,  0, 30 |
				 |           |
				 | 0, 12,  0 |
				 |           |
				 | 0,  0, 60 |
				 +-         -+

	>> B:= MZ([[3,7,11], [13,17,19]]):
	>> linalg::hermiteForm( B );

				+-           -+
				| 1, -11, -25 |
				|             |
				| 0,  40,  86 |
				+-           -+
++#

linalg::hermiteForm := proc(A)
    local i,j,v,m,n,i0,v0,q,Div,k,changed,Sign;
begin
   if testargs() then
	if args(0) <> 1 then error("expecting one argument") end_if;
	if A::hasProp( Cat::MatrixCat ) <> TRUE then
	    error("argument is not of 'Cat::MatrixCat'")
	elif A::coeffRing <> Dom::Integer then
	    error("expecting matrix over 'Dom::Integer'")
	end_if
   end_if;

   v:=abs; Div:=fun(round(args(1)/args(2))); 
   Sign:=sign;
   k:= A::dimen( A );
   n:= k[1]; m:= k[2];
   for j from 1 to min(n,m) do
      repeat
	 # choose the non-zero entry of smallest valuation #
	 v0:=infinity; changed:=FALSE;
	 for i from j to n do
	    if not iszero(A[i,j]) then
	       if v(A[i,j])<v0 then i0:=i; v0:=v(A[i,j]) end_if
	    end_if
	 end_for;
         if v0<>infinity and not iszero(A[i0,j]) then
	    # now reduce the other entries #
	    for i from 1 to n do
	       if i=i0 then next end_if;
	       q:=Div(A[i,j],A[i0,j]);
	       if not iszero(q) then # subtract q * (row i0) to row i #
		  changed:=TRUE;
		  for k from j to m do A[i,k]:=A[i,k]-q*A[i0,k] end_for
	       end_if
	    end_for;
         end_if
      until changed=FALSE end_repeat;
      if v0<>infinity and i0<>j then # exchange row i0 and row j #
	 A:= A::swapRow( A,i0,j,j..m )
      end_if;
      if sign(A[j,j])=-1 then # multiply by -1 #
         for k from j to m do A[j,k]:=-A[j,k] end_for
      end_if
   end_for;

   A
end_proc:

# end of file #

