# #
# $Date: 1999/06/02 15:45:40 $  $Author: frankp $   $Revision: 1.14.2.1 $ #
# #
# frankp, 7/06/96 #

#++
  jordan.mu

        linalg::jordanForm -- computes the Jordan form of a square matrix

	jordanForm( A [,All] )

	A  : square matrix over a Cat::Field
	All: option

	jordanForm( A ) computes the Jordan form of A, hence it
	determines a transformation matrix Q such that the matrix

		J = Q^(-1) * A * Q

	has Jordan form.
	If option 'All' is given then the list [J,Q] is returned.

	Note: The secondary diagonal of J is below the main diagonal.
	Use
	
    	    tanspose(Q)*transpose(A)*transpose(Q^(-1))

	to get this diagonal above the main diagonal.
++#

linalg::jordanForm:= proc(A)
    local n, R, ev, evList, kern, kernList, e, Q, i, j, t, 
          addVectorList, ID, iv, d, B, nullSpace, gaussElim;
begin
    if testargs() then
	if args(0) < 1 or args(0) > 2 then
	    error("expecting 1 or 2 arguments")
	end_if;
	if A::hasProp( Cat::MatrixCat ) <> TRUE then
	    error("first argument is not of 'Cat::MatrixCat'")
	end_if;
	R:= A::coeffRing;
	if not R::hasProp( Cat::Field ) then
	    error("expecting matrix over 'Cat::Field'");
	end_if;
	n:= A::dimen(A);
	if n[1] <> n[2] then
	    error("expecting a square matrix")
	else 
	    n:= n[1]
	end_if;
	if args(0) = 2 then
	    if args(2) <> hold(All) then
		error("option 'All' expected")
	    end_if
	end_if
    else
	R:= A::coeffRing;
	n:= A::dimen(A)[1];
    end_if;

    userinfo(2,"compute eigenvalues of A and their algebraic multiplicity");
    evList:= linalg::eigenValues(A,hold(Multiple));
    if evList = FAIL then
	userinfo(1,"eigenvalues can not be computed");
	return( FAIL )
    elif _plus( op(evList,[i,2]) $ i=1..nops(evList) ) <> n then
	userinfo(1,"characteristic polynomial does not split completely into linear factors");
	return( FAIL )
    end_if;

    ID:= A::newThis( n,n,subs(fun(hold(XX)),hold(XX)=R::one),hold(Diagonal) );
    nullSpace:= linalg::nullSpace;
    gaussElim:= A::gaussElim;
 
    Q:= null();  # the transformation matrix #
    for ev in evList do 
	userinfo(3,"compute main vectors chain for ",ev);
	B:= A::_plus( A, A::_mult( ID, R::_negate(ev[1]) ) );
	kern:= nullSpace(B);
	if nops(kern) = 0 then
	    userinfo( 1,"unable to determine kernel (zero recognizing problem?)");
	    return( FAIL )
	end_if;

	kernList:= [kern];
	e:= 2;
	while nops(kern) <> ev[2] do
	    userinfo(4,"create main vectors");
	    kern:= nullSpace( B::_power( B,e) );
	    kernList:= kernList . [kern];
	    e:= e + 1
	end_while;
	e:= e - 1;

	if e > 1 then

	    # (1) Extend basis of B^1 to basis of B^e = V #
	    addVectorList:= [{$1..nops(kernList[1])}];
	    for i from 2 to e do
	        userinfo(3,"extend basis of (ev*E-A)^",i-1," to basis of (ev*E-A)^",i);
                # determine vectors which can be added to kernList[e] #
		t:= gaussElim( _concat( op(kernList[i-1]),op(kernList[i]) ) )[4];
		t:= {$1..nops(kernList[i])} minus map( t, _plus, -nops(kernList[i-1]) );
		# insert the right number of lin. indep. vectors to kernList[i-1] #
		t:= {op(t,j) $ j=1..nops(kernList[i])-nops(kernList[i-1])};

		addVectorList:= addVectorList . [t];
	    end_for;

	    # (2) Determine all vectors b of B^i with b not in B V #
	    for i from 1 to e-1 do
		t:= gaussElim( _concat( B,op(kernList[i]) ) )[4];
		t:= ( {$1..nops(kernList[i])} minus map( t minus {$1..n}, _plus, -n ) ) 
			intersect addVectorList[i];
		for iv in t do
		    Q:= _concat( Q,kernList[i][iv] );
		    if i > 1 then
		        Q:= _concat( Q, B::_mult( B::_power(B,d-1),kernList[i][iv])
		                $ d=2..i )
		    end_if
		end_for
	    end_for;

	    # (3) Add highest main vectors to Q #
	    for iv in addVectorList[e] do
		Q:= _concat( Q,kernList[e][iv] );
		Q:= _concat( Q, B::_mult( B::_power(B,d-1),kernList[i][iv])
		       $ d=2..e )
	    end_for
	else
	    Q:= _concat(Q, op(kern))
	end_if
    end_for;

    # compute Jordan matrix #
    if domtype( Q ) <> domtype( A ) then
	Q:= A::coerce( Q );
    end_if;
    B:= A::_mult( Q::_invert(Q), A, Q );

    if args(0) = 2 then [ B, Q ]
    else B
    end_if
end_proc: 

# end of file #

