# #
# $Date: 1999/01/29 08:52:44 $  $Author: frankp $   $Revision: 1.58.2.1 $ #
# #
# frankp, 24/02/94 #

#++
  linsolve.mu

	linalg::linearSolve  --  solve a linear system of equations

	linearSolve( A [,B] )
	linearSolve( A {,v,l} )
	linearSolve( A {,v,Special,Unique} )

        A, B            - matrices
	v               - vector
        l               - set or list with exactly ncols(A) elements
	Special, Unique - identifier (options)

        linearSolve(A,B) solves the systems A*x=col(B,i), i=1..ncols(B) 
        and returns a matrix representing the special solutions of the 
        ncols(B) systems.
        If B is missing the the system A[1..r,1..c-1]*c=col(A,c)
        will be solved.

        linearSolve(A,v,l) solves the system A*x=B. If the system
        is undetermined, then the general solution of the system 
        will be expressed by the sum of a special solution and a
        linear combination of the general solution of the corresponding
        homogeneus system with scalars in l. Only those scalars are
        used which correspond to a kernel vector of A.
        If v is missing then the system A[1..r,1..c-1]*c=col(A,c)
        will be solved.


        linearSolve(A,v,Special) returns a special solution of the system.
        linearSolve(A,v,Unique) returns an unique solution if one exits,
        otherwise [] will be returned.
        If v is missing then the system A[1..r,1..c-1]*c=col(A,c)
        will be solved.


	linearSolve returns [] if the system has no solution,
	and FAIL if the solution could not be computed over the 
	coefficient domain of A.

	Solutions are always elements of the domain Matrix(A::coeffRing).
++#

linalg::linearSolve := proc(A,B,Flag)
    name linalg::linearSolve;
    local r, c, t, R, i, j, k, ns, rs, cB, s, v, l,
	  RisField, newThis, R_divide, Rone, R_negate, Riszero, Rmult, 
	  Rcoerce, Rzero;
begin
    if A::hasProp( Cat::MatrixCat ) <> TRUE then
        error("first argument is not of 'Cat::MatrixCat'")
    end_if;

    R := A::coeffRing;
    if R::hasProp( Cat::IntegralDomain ) <> TRUE then
        error("expecting matrix over Cat::IntegralDomain")
    end_if;

    t:= A::dimen(A);
    r:= t[1]; c:= t[2];

    if A::hasProp( Cat::SquareMatrixCat ) then
	t:= A::getSuperDomain(A)
    else
	t:= type(A)
    end_if;
    newThis:= t::newThis;

    if testargs() and args(0) > 1 then
	if args(0) > 3 then
	    error("wrong no of args")
	end_if;
	if not contains( {hold(Unique),hold(Special)},B )
        and not contains( {DOM_SET,DOM_LIST},domtype(B) ) then
	    if not testtype( B,t ) then
                error("types of matrices don't match")
	    end_if;
	    B:= t::coerce( B );
	    s:= B::dimen(B);
	    if s[1] <> r then error("invalid operands") end_if;
	    if args(0) = 3 then
		if not contains( {hold(Unique),hold(Special)},Flag ) then
	            if contains( {DOM_SET,DOM_LIST},domtype(Flag) ) then
                        if nops(Flag) <> c then
                            error("list or set must have ".expr2text(c)." entries")
			end_if;
			if has(map(Flag,testtype,R),FALSE) then
                	    error("list or set contains an invalid entry")
            		end_if
		    else
			error("invalid 3rd argument")
		    end_if
                end_if
	    end_if
	elif args(0) > 2 then error("wrong no of args")
	elif contains({DOM_SET,DOM_LIST},domtype(B)) then
            if nops(B) <> c-1 then
                error("list or set must have ".expr2text(c-1)." entries")
            end_if;
	    if has(map(B,testtype,R),FALSE) then
		error("list or set contains an invalid entry")
	    end_if
	end_if
    end_if;

    if args(0) > 1 then
	if not contains( {hold(Unique),hold(Special)},B )
        and not contains( {DOM_SET,DOM_LIST},domtype(B) ) then
	    B:= t::coerce(B);
	    cB:= B::dimen(B)[2];
	    A:= A::concatMatrix( A,B );
	else
	    Flag:= B;
	    c:= c - 1; cB:= 1
	end_if
    else
	c:= c - 1; cB:= 1
    end_if;

    userinfo(2,"perform Gaussian elimination");
    Riszero:= R::iszero;

    s:= A::gaussElim( A );
    rs:= s[2];  # rank of (A,B) #
    s:= s[1];

    # rank(A,B) = rank(A) ? #
    if rs > 0 then
	for t from rs to c do
	    if not Riszero( s[rs,t] ) then break end_if
	end_for;
	if t > c then return( [] ) end_if
    end_if;

    RisField:= R::hasProp( Cat::Field );

    userinfo(2,"compute row reduced form");
    s:= linalg::gaussJordan( A,hold(All) );

    A:= s[1]; ns:= s[4];

    # each col-no > c does not belong to ker(A) #
    ns:= ns intersect {j $ j=1..c};

    # does an unique solution exist ? #
    if nops(ns) = 0 then  # rang(A) = c #
	if RisField then
            # the right side of A is the solution #
	    return( A[1..c,c+1..cB+c] )
	else
	    R_divide:= R::_divide;

	    s:= A[1..c,c+1..cB+c];
	    for i from 1 to c do
	        (s[i,j]:= R_divide( s[i,j],A[i,i] )) $ j=1..cB;
		if s::has( s,FAIL ) then
		    userinfo(3,"no solution over its coefficient domain");
		    return( FAIL )
                end_if
            end_for;

	    return( s )
	end_if
    elif Flag = hold(Unique) or cB > 1 then
	return( [] )
    end_if;


    userinfo(3,"search for special solution");

    if RisField then
	# compute special solution over fields #
	s:= newThis(c,1);

	j:= 1;
	for i from 1 to c do
	    if not contains( ns,i ) then
		s[i,1]:= A[j,c+1];
		j:= j + 1;
		if j > r then break # i # end_if
	    end_if
	end_for;

	if Flag = hold(Special) then
	   return( s )
	end_if

    elif Flag <> hold(Special) then
	error("sorry, can determine nullspace only over fields")
    else
	# compute special solution over integral domains #
	R_divide:= R::_divide;

        s:= newThis(c,1);

        j:= 1;
        for i from 1 to c do
            if not contains( ns,i ) then
                if (s[i,1]:= R_divide( A[j,c+1],A[j,i] )) = FAIL then
                        userinfo(3,"no solution over its coefficient domain");
                        return( FAIL )
		end_if;
                j:= j + 1;
                if j > r then break # i # end_if
            end_if
        end_for;

	return( s )
    end_if;


    userinfo(2,"solve the homogeneous system");

    R_negate:= R::_negate;
    Rone:= R::one;
    Rzero:= R::zero;

    rs:= [];
    ns:= sort( [op(ns)] );
    l:= min( r,c );
    if not contains( {DOM_LIST,DOM_SET},domtype(Flag) ) then
        for i from 1 to nops(ns) do
            k:= ns[i];
            v:= map( [op(A[1..l,k..k])], R_negate );
            for j from 1 to i do
                if j = i then
                    v:= [op(v,1..k-1),Rone,op(v,k..l+j-1)];
                else
                    t:= ns[j];
                    v:= [op(v,1..t-1),Rzero,op(v,t..l+j-1)]
                end_if
            end_for;

            rs:= rs . [newThis( c,1,[op(v,1..min(c,l+i))] )]
        end_for;

        [s, rs]
    else
	Rmult:= R::_mult;
 	Rcoerce:= R::coerce;	

        for i from 1 to nops(ns) do
            k:= ns[i];
	    R:= Rcoerce(op(Flag,k));
            v:= map( [op(A[1..l,k..k])], fun( Rmult( R_negate(args(1)),R ) ) );
            for j from 1 to i do
                if j = i then
                    v:= [op(v,1..k-1),R,op(v,k..l+j-1)]
                else
                    t:= ns[j];
                    v:= [op(v,1..t-1),Rzero,op(v,t..l+j-1)]
                end_if
            end_for;

            rs:= rs . [newThis( c,1,[op(v,1..min(c,l+i))] )]
        end_for;

	A::_plus( s,op(rs) )
    end_if

end_proc:

# end of file #
