#-

linopt::findMinimum -- minimize a linear program

Calling sequence:

linopt::findMinimum(A,b,c)

Parameter:

A -- matrix with m rows and n columns
b -- vector with m rows
c -- vector with n rows

Summary:

findMinimum(A,b,c) tries to find the minimum of the standard
Linear Optimization Problem Ax=b, x>=0, (c | x) = min
using the Simplex algorithm. Note, that rank(A) must be m.
findMinimum returns the vector x or [ FAIL, err ] where err is
one of the following messages:
  NO_SOLUTION 	 - The LOP has no solution.
  NO_CONVERGENCE - Can't find the minimum after 100 iterations
                   within the Simplex algorithm.
  UNBOUNDED      - the linear objective function f is unbounded
                   ( f(x) = x[1]*c[1] + ... + x[n]*c[n] )
                   
  
Possible errors:

wrong no of args              - findMinimum expects 3 arguments
expecting a matrix            - first argument must be a matrix
incompatible args             - b or c aren't vectors or the 
                                vectors are incompatible, for
                                example different coefficient rings
rank of the matrix must be m  - rank(A) must be m

-#

linopt::findMinimum := proc()
local A, b, c, m, n, AA, i, j, B, cc, xx, 
      J, result, x, JJ, p, q, J0, iter1, M;

begin

if testargs() then
        if args(0) <> 3 then error("wrong no of args") end_if;
        A := args(1); b := args(2); c := args(3);
        if A::hasProp( Cat::MatrixCat ) <> TRUE then
            error("expecting a matrix")
        end_if;
        m := linalg::nrows(A); n := linalg::ncols(A);
        if not testtype(b, linalg::VectorOf(A::coeffRing, m)) then
            error("incompatible args")
        end_if;
        if not testtype(c, linalg::VectorOf(A::coeffRing, n)) then 
            error("incompatible args")
        end_if;
        if linalg::rank(A) < m then
           error("rank of the matrix must be ".m)
        end_if;
else
        A := args(1); b := args(2); c := args(3);
        m := linalg::nrows(A); n := linalg::ncols(A);
end_if;

AA := A;
for i from 1 to m do
	if b[i] < 0 then 
		for j from 1 to n do AA[i,j] := - A[i,j] end_for;
		b[i] := - b[i]
	end_if
end_for;

M := Dom::Matrix(A::coeffRing);

B :=  M( m , n + m);      # matrix of dimension (m, n + m) #
cc := M( [ 0 $ n + m ] ); # 0-vector of dimension n+m      #
xx := M( [ 0 $ n + m ] ); # 0-vector of dimension n+m      #

for i from 1 to m do
	for j from 1 to n do B[i,j] := AA[i,j] end_for;
	B[i,n + i] := 1;
	cc[n + i] := 1;
	xx[n + i] := b[i]; 
end_for;

J := [ n + i $ hold(i) = 1..m ]; # [ n+1, ..., n+m ] #

userinfo(1, "Looking for a valid edge.");

result := linopt::simplex(B, cc, J, xx);

xx := result[2]; 

if xx = FAIL then 
   return( [ FAIL, result[1] ] )
end_if;

if linalg::scalarProduct(cc, xx) > 0 then
   userinfo(1, "no valid points (simplex)");
   return( [ FAIL, hold(NO_SOLUTION) ] )
end_if;

userinfo(3, "Valid edge: ".expr2text(xx));

x := M( [ xx[i] $ hold(i)=1..n ] ); # vector of dimension n #

JJ := result[1]; 
J := [];
for i from 1 to m do
	if JJ[i] <= n then J := append( J, JJ[i] ) end_if
end_for;

if nops(J) < m then 
	p := nops(J);
	J0 := sort( [ op( { j $ hold(j)=1..n } minus { op(J) } ) ] );
	for i from 1 to m do
		for j from 1 to p do
			AA[i,j] := A[i,J[j]]
		end_for;
		for j from 1 to n - p do
			AA[i,j + p] := A[i,J0[j]]
		end_for
	end_for;
	q := linopt::gauss(AA);
	for i from p + 1 to m do
		J := append( J, J0[ q[i] - p ] )
	end_for;
	J := sort(J)
end_if; 
iter1 := result[3];

userinfo(1, "Searching for a minimum.");

result := linopt::simplex(A, c, J, x);

if result[2] = FAIL then
   return( [ FAIL, result[1] ] )
end_if;

userinfo(5, "");
userinfo(5, "SOLUTION :");
userinfo(5, "Basismenge   : ".expr2text(result[1]));
userinfo(5, "Optimal Edge : ".expr2text( [ result[2][i,1] $ hold(i)=1..n ]));
userinfo(5, "Minimal value: ".expr2text(linalg::scalarProduct(c,x)));
userinfo(5, "Iterations in Phase I  : ".iter1);
userinfo(5, "Iterations in Phase II : ".result[3]);

result[2]

end_proc:

linopt::simplex := proc()
local A, c, J, xJ, m, n, x, v, y, w, maxiter, zeilen, iter, AJ, AJinv, 
      i, j, k, l, itemp, lambda, temp, M;

begin

userinfo(1, "Entering the Simplex algorithm.");

A := args(1); c := args(2);
J := args(3); xJ := args(4); 

m := linalg::nrows(A); n := linalg::ncols(A);

M := Dom::Matrix(A::coeffRing);

x := M( [0 $ n] ); v := M( [0 $ n] ); # 0-vectors of dimension n #
y := M( [0 $ m] ); w := M( [0 $ m] ); # 0-vectors of dimension m #

maxiter := 100;
zeilen := [ i $ i = 1..m ]; # [ 1, ..., m ] #

for iter from 0 to maxiter do
	AJ := linalg::extractMatrix(A, zeilen, J);
	AJinv := AJ^(-1);
	for i from 1 to m do y[i] := c[J[i]] end_for;
	y := linalg::transpose(AJinv) * y ;
	v := ( linalg::transpose(A) * y ) + (c * (-1) );
	for i from 1 to m do v[J[i]] := 0 end_for;
	for j from 1 to n do if v[j] > 0 then break end_if end_for;
	if j = n + 1 then 
		return(op([J,xJ,iter])) 
	end_if;
	k := j;
	for i from 1 to m do w[i] := A[i,k] end_for;
	w := AJinv * w ; 
	for i from 1 to m do if w[i] > 0 then break end_if end_for;
	if i = m + 1 then 
           userinfo(1, "Objective function is unbounded (Simplex).");
           return( [ hold(UNBOUNDED), FAIL ] );
        end_if;
	l := i;
	itemp := l;
	lambda := xJ[J[l]]/w[l]; 
	for i from itemp + 1 to m do 
		if w[i] > 0 then
			temp := xJ[J[i]]/w[i];
			if temp < lambda then 
				l := i;
				lambda := temp
			end_if
		end_if
	end_for;
	for j from 1 to n do x[j] := 0 end_for;
	for i from 1 to m do x[J[i]] := xJ[J[i]] - lambda * w[i] end_for; 
	x[J[l]] := 0;
	x[k] := lambda;
	xJ := x;
	J[l] := k;
	J := sort(J); 
end_for;

userinfo(1, "No convergence after ".maxiter." steps!");

return( [ hold(NO_CONVERGENCE), FAIL ] );

end_proc:

linopt::gauss := proc(A)
local i, j, k, m, n, r, index;

begin

m := linalg::nrows(A);
n := linalg::ncols(A);
r := 0;
k := 1;
for j from 1 to n do
	for i from k to m do
		if A[i,j] <> 0 then
			A := linalg::swapRow(A,k,i);
			break;
		end_if;
	end_for;
	if A[k,j] <> 0 then
		r := r + 1;
		index[k] := j;
		A := linalg::multRow(A,k,1/A[k,j]);
		for i from k+1 to m do
			A := linalg::addRow(A,k,i,-A[i,j]);
		end_for;
		k := k + 1;
	end_if;
	if k > m then break;
	end_if;
end_for;
for k from r downto 2 do
	for i from 1 to k-1 do
		A := linalg::addRow(A,k,i,-A[i,index[k]]);
	end_for;
end_for;

[ index[i] $ hold(i)=1..r ]; # char. indices #

end_proc:
