#-

linopt::maximize -- maximize a linear program
linopt::minimize -- minimize a linear program

Calling sequence:

linopt::minimize( f, cons [, NonNegative ][, int_cons] )
linopt::maximize( f, cons [, NonNegative ][, int_cons] )

Parameter:

f           -- a linear expression
cons        -- a set or list of linear constraints
NonNegative -- (optional) name
int_cons    -- (optional) set of variables or the name All

Summary:

The expression 'f' is the linear objective function to be minimized/maximized
subject to the linear constraints 'cons'. The function minimize/maximize
returns either a set of equations describing the optimal solution to the
specified linear program, or the empty set in the case where no feasible
solution to 'cons' exists, or UNBOUNDED in the case where the solution is 
unbounded. FAIL is returned, if no solution is found.

The equations returned by minimize/maximize can be substituted back into 
the objective function 'f' to obtain the value of the objective function at
optimal solution.

A third parameter may be used to specify that all variables are constrained
to be non-negative; such constraints may also be listed explicitly.

A fourth parameter may be used to specify the variables that are constrained
to be integer.

References:

* V. Chvatal, Linear Programming, Freeman & Company, 1983
* K.-H. Kiyek and F. Schwarz, Mathematik fuer Informatiker, Band 2,
  Teubner, 1991
* NEOS Guide: Optimization Software
  http://www.mcs.anl.gov/home/otc/Guide/SoftwareGuide/

Examples:

>> c := {3*x+4*y-3*z <= 23, 5*x-4*y-3*z <= 10, 7*x+4*y+11*z <= 30}:
>> f := -x + y + 2*z:
>> linopt::maximize(f, c union {x>=0,y>=0,z>=0});

                        {z = 1/2, y = 49/8, x = 0}

>> linopt::maximize(f, c, NonNegative);

                        {z = 1/2, y = 49/8, x = 0}

>> linopt::maximize(f, c);
 
                        UNBOUNDED
 
>> linopt::maximize(x, {x<=-1,x>=0}); 
 
                        {}

>> linopt::maximize(c1+c2,{2*c1<=1,2*c2<=1});

			{c1 = 1/2, c2 = 1/2}

>> c := {2*x1+x2+x3=4, 2*x1+3*x2+x4=6}:
>> f := -x1-x2:

>> linopt::minimize(f, c, NonNegative); subs(f,%);

                    {x3 = 0, x4 = 0, x2 = 1, x1 = 3/2}

                                   -5/2

>> linopt::minimize(f, c, NonNegative, All); subs(f,%);

                     {x2 = 0, x3 = 0, x1 = 2, x4 = 2}

                                    -2

>> linopt::minimize(f, c, NonNegative, {x1}); subs(f,%);

                   {x1 = 1, x4 = 0, x3 = 2/3, x2 = 4/3}

                                   -7/3

Knapsack problem ; x1, x2, x3, x4 elements of {0,1}

>> c := {20*x1+15*x2+20*x3+5*x4 <= 25}: f := 10*x1+15*x2+16*x3+x4:
>> c := c union {x1>=0, x1<=1, x2>=0, x2<=1, x3>=0, x3<=1, x4>=0, x4<=1}:
>> linopt::maximize(f, c, All);

                     {x1 = 0, x2 = 0, x3 = 1, x4 = 1}


See also: linopt::setup, linopt::findMinimum

-#

linopt::maximize := proc(f)
   # Note that we want to compute the maximum, #
   # so we have to find the minimum of -f.     # 
   begin linopt::minimize(-f, args(2..args(0))) end_proc:

linopt::minimize := proc(f, constraints, condition, int_cons)
begin
   case args(0) 
     of 2 do 
        return( linopt::minimize_generic(f, constraints) )
     of 3 do 
        if condition = NonNegative then
           return( linopt::minimize_generic(f, constraints, hold(NonNegative) ) )
        else
           int_cons := condition; 
           condition := hold(Unrestricted);
        end_if;
        break;
     of 4 do
        break;
     otherwise
        error("wrong no of args")
  end_case;

  if int_cons = {} then
     return( linopt::minimize_generic(f, constraints, condition) )
  elif int_cons = All then
     int_cons := indets(constraints)
  elif not testtype(int_cons, Type::SetOf(DOM_IDENT)) then
     error("Use a set of variables or the name All to specifiy integer constraints")
  end_if;

  linopt::Land_Doig(f, constraints, condition, int_cons)

end_proc:

linopt::minimize_generic := proc(f, constraints, condition)
   local A, b, c, l, m, n, r, 
         cons, vars, subs_set, ind;
begin

   l := linopt::linear_system(args(2..args(0)));
   if l = {} or l = UNBOUNDED then return(l) end_if;
   A := l[1]; b := l[2]; m := l[3]; n := l[4]; r := l[5];
   vars := l[6]; subs_set := l[7]; cons := l[8];

   # rank(A) = m and m < n #
 
   if Type::Linear(f, [ indets(f) ]) = FALSE then
      error("linear function expected.")
   end_if;
 
   # Substitute variables of f with respect to subs_set. #
   f := subs(f, subs_set);
 
   # Convert it to a n-dim. vector with respect to vars #
   c := linalg::expr2Matrix( [ f ], vars);
   ind := indets(c[2][1,1]);
   if ind <> {} then
      userinfo(3, "indeterminate(s) ".expr2text(op(ind))." unbounded.");
      return( UNBOUNDED )
   end_if;
   c := linalg::transpose( c[1] );
    
   # Find the Minimum of the standard LOP Ax=b, x>=0, (c | x) = min   #
   # using the Simplex algorithm. Note, that rank(A) must be nrows(A) #
 
   r := linopt::findMinimum(A, b, c);

   if r[1] = FAIL then
      case r[2]
        of NO_SOLUTION do
           return( {} )
        of UNBOUNDED do
           userinfo(3, "Linear objective function is unbounded.");
           return( UNBOUNDED );
        of NO_CONVERGENCE do
           userinfo(3, "Can't find optimal solution (no convergence).");
           return( FAIL );
        otherwise
           error("Unknown error ", r[2])
      end_case
   end_if;
 
   # Back substitution #
 
   linopt::backsubs(r, vars, n, subs_set)
 
end_proc:

linopt::linear_system := proc(constraints, condition)
   local Ab, A, b, i, l, m, n, r, t, x, cons,
         vars, subs_set, R, Rzero, Riszero;
begin

   l := linopt::setup(args());
   if l = [] then
      return({})
   end_if;
   cons := [ op(l[1]) ]; vars := [ op(l[2]) ]; subs_set := l[3];

   userinfo(5, "Standard Linear Optimization Problem:");
   userinfo(5, "Constraints  :"); userinfo(5, cons);
   userinfo(5, "Variables    :"); userinfo(5, vars);
   userinfo(5, "Substitutions:"); userinfo(5, subs_set);

   if cons = [] then return( UNBOUNDED ) end_if;

   # convert the list                                              #
   #   [ f_1(x_1,...,x_n) = c_1 , ... , f_m(x_1,...,x_n) = c_m ]   #
   # to a (m,n)-matrix and a m-dim. vector.  vars contains a list  #
   # of all indeterminates x_1, ..., x_n .                         #

   Ab := linalg::expr2Matrix(cons, vars);
   A  := Ab[1]; b := Ab[2];
   m  := linalg::nrows(A); n := linalg::ncols(A);

   Ab := linalg::concatMatrix( A ,b );
   t  := linalg::gaussElim(Ab, All);
   Ab := t[1]; r := t[2]; # r = rank(A,b) #

   R := Ab::coeffRing; Riszero := R::iszero; Rzero := R::zero;
   i := 1; while Riszero(Ab[r, i]) do i := i+1 end_while;

   if i = n+1 then
      # rank(A) <> rank(A,b) #
      return( {} )
   elif r < m then
      if r = n then
         # unique solution of Ax=b  #
         # no need for optimization #
         x := linopt::linsolve( Ab, n); 
         # Is x >= 0 ? #
         for i from 1 to n do 
             if x[i] < Rzero then return( {} ) end_if
         end_for;
         # Back substitution #
         return( linopt::backsubs(x, vars, n, subs_set) )
      else
         # Rows of A aren't linear independant. #
         # Delete unnecessary constraints.      #
         A := linalg::extractMatrix( Ab, 1..r, 1..n);
         b := linalg::extractMatrix( Ab, 1..r, n+1..n+1);
      end_if
   elif n = m then
      # unique solution of Ax=b  #
      # no need for optimization #
      x := linopt::linsolve( Ab, m);
      # Is x >= 0 ? #
      for i from 1 to n do 
          if x[i] < Rzero then return( {} ) end_if 
      end_for;
      # Back substitution #
      return( linopt::backsubs(x, vars, n, subs_set) )
   end_if;

   return( [ A, b, m, n, r, vars, subs_set, cons ] )

end_proc:

linopt::backsubs := proc(r, vars, n, subs_set)
   local i, s;
begin
   s := {};
   for i from 1 to n do
       s := s union {vars[i] = r[i]}
   end_for;
   subs(subs_set, s);
end_proc:

# Solve the linear system of equations Ax = b over an integral domain,  #
# where A is an upper triangular square matrix with m rows and columns. # 
# A and b are stored in the matrix Ab, meaning that b[i] = Ab[i, m+1].  #

linopt::linsolve := proc(Ab, m)
   local i, j, x;
begin
   x := [ Ab[i, m+1] $ i=1..m ];
   for i from m downto 1 do
       x[i] := x[i] / Ab[i,i];
       for j from i-1 downto 1 do
           x[j] := x[j] - Ab[j,i] * x[i]
       end_for;
   end_for;
   x
end_proc:
