
#-

linopt::setup - Setup a Standard Linear Optimization Problem

Calling sequence:

linopt::setup(set [, NonNegative ] [, Unsimplified])

Parameter:

set          - a set of linear constraints. A linear constraint can be
               an equation or a non-strict inequality.
NonNegative  - (optional) name. 
               All variables are assumed to be non-negative.
Unsimplified - (optional) name.
               No simplication of the constraints is done.

Summary:

linopt::setup returns a list [ eqn , vars, subs ] where
- eqn is a list of equations in a form with isolated variables
  on the left-hand side. Those variables form a basis for the
  corresponding linear system (see linopt::findMinimum), and do 
  not occur on the right hand side of any equation. Slack 
  variables are introduced to deal with inequalities. Unrestricted
  variables are transformed to be the difference of two variables.
- vars is a set containing all variables of eqn. All these
  variables are assumed to be non-negative.
- subs is a set of transformation used for any variable.
or the empty list. The empty list indicates, that no LOP can suffice
the constraints. The value [ {}, {}, {} ] means, that the solution
of any linear objective function is unbounded.
Use eqn and vars to get a Standard Linear Optimization Problem:
Ax = b , x >= 0

Example:

Constraints:

>> c := {3*x+4*y-3*z <= 23, 5*x-4*y-3*z <= 10, 7*x+4*y+11*z <= 30}:

Compute standard LOP Ax' = b , x' = (x,y,z) >= 0
Note, that x, y, z are assumed to be non-negative.

>> s := linopt::setup(c);

[{X1 + 5 X4 - 5 X5 - 4 X6 + 4 X7 - 3 X8 + 3 X9 = 10,

   X3 + 3 X4 - 3 X5 + 4 X6 - 4 X7 - 3 X8 + 3 X9 = 23,

   X2 + 7 X4 - 7 X5 + 4 X6 - 4 X7 + 11 X8 - 11 X9 = 30},
 
   {X2, X3, X4, X5, X6, X7, X8, X9, X1},
 
   {z = X8 - X9, x = X4 - X5, y = X6 - X7}]

X1, X2, X3 are slack variables.
X4, X5 are introduced because x is not assumed to be non-negative.

>> linalg::expr2Matrix(s[1], s[2]);

             -- +-                               -+  +-  -+ --
             |  | 0, 0, 5, -5, -4,  4, -3,  3,  1 |  | 10 |  |
             |  |                                 |  |    |  |
             |  | 0, 1, 3, -3,  4, -4, -3,  3,  0 |, | 23 |  |
             |  |                                 |  |    |  |
             |  | 1, 0, 7, -7,  4, -4, 11, -11, 0 |  | 30 |  |
             -- +-                               -+  +-  -+ --

Compute standard LOP Ax' = b , x' = (x,y,z) >= 0
All variables should be non-negative. Use 
linopt::setup(c, NonNegative) as a shortcut for
linopt::setup(c union {x>=0,y>=0,z>=0})

>> s := linopt::setup(c, NonNegative);

[{X10 + 5 X13 - 4 X14 - 3 X15 = 10, X11 + 7 X13 + 4 X14 + 11 X15 = 30,
 
   X12 + 3 X13 + 4 X14 - 3 X15 = 23}, {X10, X11, X12, X13, X14, X15},
 
   {z = X15, x = X13, y = X14}]

X10, X11, X12 are slack variables.

>> linalg::expr2Matrix(s[1], s[2]);


                   -- +-                  -+  +-  -+ --
                   |  | 1, 0, 7,  4, 11, 0 |  | 30 |  |
                   |  |                    |  |    |  |
                   |  | 0, 1, 3,  4, -3, 0 |, | 23 |  |
                   |  |                    |  |    |  |
                   |  | 0, 0, 5, -4, -3, 1 |  | 10 |  |
                   -- +-                  -+  +-  -+ --

-#

linopt::setup := proc(set, cond, reduce)
   local nargs, boundvars, freevars, vars, new_set,
         rep_set, non_neg, subs_set, con, f, v, c, x,
         insert_non_neg, eqn, s, red_set;
begin

   # Definition of local procedures #

   # Note, that this procedure modifies non-local #
   # variables, namely non_neg and subs_set.      #

   insert_non_neg := proc(x)
      local eqn, pos_x, neg_x;
   begin
      pos_x := genident(); neg_x := genident();
      non_neg := non_neg union { pos_x, neg_x };
      eqn := x = pos_x - neg_x ;
      subs_set := subs_set union { eqn };
      eqn;
   end_proc:


   # main program #

   reduce := TRUE;

   nargs := args(0); 
   case nargs 
    of 1 do
       cond := hold(Unrestricted);
       break;
    of 2 do
      if cond = hold(Unsimplified) then
         reduce := FALSE;
         cond   := hold(Unrestricted);
      end_if;
      break;
    of 3 do
      if reduce <> hold(Unsimplified) then
         error("Illegal option")
      else
         reduce := FALSE
      end_if;
      break;
    otherwise
      error("wrong number of arguments")
   end_case;
   
   if nargs > 1 and cond <> hold(NonNegative) and cond <> hold(Unrestricted) then
      error("Illegal option")
   end_if;

   if not testtype(set, Type::SetOf(Type::Relation)) then
      error("set of equations or non-strict inequalities expected.")
   end_if;

   if set = {} then
      return( [ {}, {}, {} ] )
   end_if;

   vars := {}; new_set := {}; non_neg := {}; 
   subs_set := {}; red_set := {};

   for con in set do
       f := op(con, 1) - op(con, 2);
       v := indets(f);
       if Type::Linear(f, [ op(v) ]) = FALSE then
          error("constraints must be linear.")  
       end_if; 

       case type(con) 
         of "_less" do
            error("Can't handle strict inequalities.")
         of "_unequal" do
            error("equations or non-strict inequalities expected.")
         of "_equal" do 
            if reduce and nops(v) = 1 then
               red_set := red_set union { con };
            elif nops(v) = 0 then
               if op(set) then
                  break
               else
                  return( [] );
               end_if
            else
               vars := vars union v;
               c := linopt::constant(f);
               new_set := new_set union { f - c = -c };
            end_if;
            break;
         of "_leequal" do
            if reduce and nops(v) = 1 then
               red_set := red_set union { con };
            else
               if type(-f) = DOM_IDENT then 
                    # -x <= 0 -> x >= 0 #
                    non_neg  := non_neg union { -f }
               elif nops(v) = 0 then    
                    if op(set) then  
                       break
                    else   
                       return( [] );
                    end_if 
               else
                    # insert slack variable #
                    s := genident();
                    vars := vars union v;
                    non_neg  := non_neg union { s };
                    c := linopt::constant(f);
                    new_set := new_set union { f - c + s = -c }
               end_if
            end_if
       end_case
   end_for;
   if nargs > 1 and cond = hold(NonNegative) then
      if reduce and red_set <> {} then
         red_set := red_set union map(indets(set), func(x >= 0))
      else
         non_neg := vars union non_neg
      end_if
   end_if; 

   if red_set <> {} then
      s := linopt::simplify(red_set);
      if s = [] then
         return( [] )  # no solution #
      end_if;
      eqn      := s[1]; 
      non_neg  := non_neg union s[3];
      new_set  := subs(new_set, eqn);
      subs_set := subs_set union eqn;
      new_set  := new_set union s[2] ;
      vars     := (vars union s[4]) minus s[5];
   end_if;

   freevars  := vars minus non_neg;
   boundvars := vars intersect non_neg;
   new_set   := subs( new_set, map( freevars, insert_non_neg) );
   rep_set   := map(boundvars, func(x = genident()) );
   subs_set  := subs_set union rep_set;
   non_neg   := subs(non_neg, rep_set);
   new_set   := subs(new_set, rep_set);

   [ new_set , non_neg, subs_set ]

end_proc:

# return the constant term of the linear function f #
 
linopt::constant := proc(f)
   local c;
begin 
   case type(f)
      of "_mult" do
         return(0);
      of "_plus" do
         # if the sum contains a constant, then it is the last op #
         c := op(f, nops(f));
         return((if testtype(c, NUMERIC) then c else 0 end_if))
      of DOM_IDENT do
         return(0);
   end_case
end_proc:
