
#-

linopt::simplify - Simplification of univariante linear constraints

Calling sequence:

linopt::simplify( set )

Parameter:

set          - a set of univariante linear constraints. Each
               constraint must be an equation or a non-strict
               inequality.

Summary:

linopt::simplify returns a list [ eqn, ineq, non_neg ] where
eqn -     a set of equations of the form x=c, c constant, 
          or the empty set.
ineq    - a set of equations of the form x+s=c resp. -x+s=-c, 
          c constant. Each equation corresponds to the inequality
          x <= c resp. x >= c. s is a slack variable.
non_neg - a set of variables that are assumed to be non-negative.
          Each variable corresponds to the inequality x >= 0.
or the empty list, if constraints are inconsistent (no solution exists.

Note, that indets(eqn) intersect non_neg = {} and non_neg is a
subset of indets(ineq).

linopt::simplify({}) returns simply [{}, {}, {}].

Example:

>> linopt::simplify({2*x-5<=3,x>=0,y>=2,y<=2});

                     [{y = 2}, {x + X1 = 4}, {x, X1}]

>> linopt::simplify({5-3*x<=-4,2*x-1<=3});

                                    []

>> linopt::simplify({x>=2,x<=2,y>=1,z>=-1});

           [{x = 2}, {- y + X2 = -1, - z + X3 = 1}, {X2, X3, y}]

-#

linopt::simplify := proc( set )
   local bounded, c1, c2, c3, con, eqn, ineq,
         non_neg, relop, s1, s2, t, vars, x;
begin
	
	#-
	  Fist step: combine 1) x  = c1 and x  = c2 
	                     2) x <= c1 and x <= c2
			     3) x >= c1 and x >= c2
	  1) if c1 = c2 then x = c1 else NO_SOLUTION
	  2) min(c1, c2)
	  3) max(c1, c2)
	  
	   t[x, "_equal"]   = c -> x  = c
	   t[x, "_leequal"] = c -> x <= c 
	   t[x, "_gequal"]  = c -> x >= c
	-#
	
	for con in set do
            con := linopt::normal(con);
	    x := con[1]; relop := con[2]; c1 := con[3];
		c2 := t[x, relop]; 
		if type(c2) = "_index" then
		   t[x, relop] := c1
		elif relop = "_equal" then 
		     if c1 <> c2 then return( [] ) end_if
	    elif relop = "_leequal" then
		     if c1 < c2 then t[x, relop] := c1 end_if
	    elif c1 > c2 then t[x, relop] := c1 end_if
	end_for;
	
	vars := indets(set); bounded := {};
	eqn := {}; ineq := {}; non_neg := {};
	
	#-
	   Second step: combine x = c1 and x <= c2 and x >= c3
	   
	   t[x, "_equal"]   = c1 -> x  = c1
	   t[x, "_leequal"] = c2 -> x <= c2 
	   t[x, "_gequal"]  = c3 -> x >= c3
	-#
	
	for x in vars do
	    c1 := t[x, "_equal"]; c2 := t[x, "_leequal"]; 
		c3 := t[x, "_gequal"];
		if type(c1) <> "_index" then
		   if type(c2) <> "_index" then
		      if c1 <= c2 then
			     if type(c3) <> "_index" then
				    if c1 >= c3 then
					   eqn := eqn union {x = c1};
                                           bounded := bounded union {x};
					else 
					   return( [] )
					end_if
				 else
				    eqn := eqn union {x = c1};
				    bounded := bounded union {x};
				 end_if
			  else
			     return( [] )
			  end_if
		   elif type(c3) <> "_index" then
		      if c1 >= c3 then
			     eqn := eqn union {x = c1};
			     bounded := bounded union {x};
			  else
			     return( [] )
			  end_if
		   else
		      eqn := eqn union {x = c1};
		      bounded := bounded union {x};
		   end_if
		elif type(c2) <> "_index" then
		   if type(c3) <> "_index" then
                       if c3 < c2 then
                          if c3 >= 0 then
                             s1 := genident();
                             non_neg := non_neg union {s1, x};
                             # x<=c2 -> x+s1=c2 #
                             ineq := ineq union {x+s1=c2};
                             if c3 <> 0 then
                                s2 := genident();
                                non_neg := non_neg union {s2};
                                # x>=c3 -> -x+s2=-c3 #
                                ineq := ineq union {-x+s2=-c3}
                             end_if
                          else
                             s1 := genident(); s2 := genident();
                             non_neg := non_neg union {s1, s2};
                             ineq := ineq union {x+s1=c2, -x+s2=-c3}
                          end_if
		       elif c2 = c3 then
			    eqn := eqn union {x = c2};
                            bounded := bounded union {x};
		       else
			    return( [] )
		       end_if
		   else
                       s1 := genident(); 
                       non_neg := non_neg union {s1};
                       # x<=c2 -> x+s1=c2 # 
                       ineq := ineq union {x+s1=c2}; 
		   end_if
		elif type(c3) <> "_index" then
		     if c3 < 0 then
                            s2 := genident();
                            non_neg := non_neg union {s2};
                            # x>=c3 -> -x+s2=-c3 # 
                            ineq := ineq union {-x+s2=-c3}
			 else
			    non_neg := non_neg union {x};
			    if c3 > 0 then
                               s2 := genident(); 
                               non_neg := non_neg union {s2};
                               # x>=c3 -> -x+s2=-c3 #  
                               ineq := ineq union {-x+s2=-c3}
			    end_if
		     end_if
		end_if
    end_for;
	
    [ eqn , ineq , non_neg , vars minus bounded minus non_neg, bounded ]
	
end_proc:

# Normalize univariante linear constraint.       #
# con : f(x) <op> c, c constant,                 #
#       f(x) univariante linear function,        #
#       <op> = _equal or <relop> = _leequal      #
# Returns a list [ x, relop, b ] where x is the  #
# indeterminate of f(x), b is a constant and     #
# relop is one of the following strings          #
#  "_leequal", "_equal", "_gequal" corresponding #
# to the relation operands <=, =, >= .           #
# Example:                                       #
# linopt::normal(-2*x-5<=3) -> [x, "_gequal", -4]#

linopt::normal := proc( con )
   local c, cx, f, relop, x;
begin
   f := op(con, 1) - op(con, 2);
   c := -linopt::constant(f); 
   x := f + c; relop := type(con);
   if type(x) = "_mult" then
      cx := op(x, 2); x := x/cx; c := c/cx;
	  if cx < 0 and relop = "_leequal" then
		 relop := "_gequal" 
	  end_if
   end_if;
   [ x , relop, c ] 
end_proc:			    
