# $Date: 1996/12/10 19:04:31 $  $Author: zimmerma $   $Revision: 1.3 $ #
# #
# olli #

#++
	numeric::bairstow  --  computing all zeros of an univariate 
	                       polynomial

	bairstow( p , <max_it> )

	p : univariate polynomial with real coefficients
	max_it : (optional) maximal number of iterations

        The method of Bairstow for determining all zeros of an univariate 
        polynomial with real coefficients.

	This function returns FAIL if the used method does not converge.
	Otherwise a set of all the zeros is returned.

	Examples:

	>> numeric::bairstow(x^6-2*x^5-4*x^4+13*x^3-24*x^2+18*x-4);

	{0.3819660112, 0.7320508075, 2.618033988, -2.732050807, 

	0.5 - 1.322875655 I, 0.5 + 1.322875655 I}

	>>  numeric::bairstow(-2*x^5-4*x^4+13*x^3-24*x^2+18*x-4);

	{-4.288322145, 0.382698279, 0.700560672, 0.602531597 + 1.173251614 I,

	0.602531597 - 1.173251614 I}
++#

numeric::bairstow := proc(po, max_it)
  local A, a, b, C, c, cont, eps1, eps2, i, n,
        p, p1, q, q1, r, s, t, tt;
begin
  n := degree(po);
  a := array(0..n); b := array(0..n); c := array(0..n);
  (a[i] := coeff(po, i)) $ i=0..n;
  
  eps1 := 10^(-DIGITS); eps2 := 10^(-2*DIGITS);
  if args(0)<2 then 
     max_it := max(50,n^2);  # Maximale Anzahl der Iterationen #
  end_if;
  r := {};       # Liste der Nullstellen           #

  if n > 2 then 
     repeat
        # Doppelzeiliges Hornerschema #
        A := 0; p := 1.0; q := 1.0;
        b[n] := a[n]; c[n] := a[n];
        repeat
           b[n-1] := b[n]*p+a[n-1];
           c[n-1] := b[n-1]+b[n]*p;
           for i from n-2 downto 0 do
               b[i] := b[i+2]*q+b[i+1]*p+a[i];
               c[i] := c[i+2]*q+c[i+1]*p+b[i];
           end_for;
           # Bairstow Iteration #
           C := c[2]^2-c[1]*c[3];
           if C = 0.0 then userinfo(1,"C=0"); return(FAIL) end_if;
           p1 := p-(b[1]*c[2]-b[0]*c[3])/C;
           q1 := q-(b[0]*c[2]-b[1]*c[1])/C;
           if abs(b[0]) >= eps1 or abs(b[1]) >= eps1 then
              if abs(p1-p) < eps2 and abs(q1-q) < eps2 then
                 userinfo(1,"does not converge");
                 return(FAIL); # Iteration steht #
              end_if;
              A := A+1;
              if A > max_it then
                 # Keine Konvergenz nach 'max_it' Iterationen #
                 userinfo(1,"no convergence after ".max_it." iterations");
                 return(FAIL); 
              end_if;
              p := p1; q := q1; cont := FALSE;
           else
              cont := TRUE
           end_if
        until cont end_repeat;
        # Nullstellen des quadratischen Faktors #
        s := p1/2;
        t := p1^2+4*q1; # Diskriminante #
        if t >= 0 then
           tt := sqrt(t)/2;
           r := r union {s+tt, s-tt};
        else
           tt := sqrt(-t)/2;
           r := r union {s+I*tt, s-I*tt};
        end_if;
        # Koeffizienten des abdividierten Polynoms #
        for i from 2 to n do
            a[i-2] := b[i]
        end_for;
        n := n-2;
      until n <= 2 end_repeat;
    end_if;
    # Auswerten des Restpolynoms #
    if n =1 then # Lineares Polynom #
       return( r union {-a[0]/a[1]} )
    end_if;
    # Quadratisches Restpolynom #
    s := -a[1]/(2*a[2]);
    t := a[1]^2-4*a[2]*a[0]; # Diskriminante #
    if t >= 0 then
       tt := sqrt(t)/(2*a[2]);
       return( r union {s+tt, s-tt} );
    else
       tt := sqrt(-t)/(2*a[2]);
       return( r union {s+I*tt, s-I*tt} );
    end_if
end_proc:

# end of file #
