#----------------------------------------------------------------#
# The abscissae c[j] of Gauss-Legendre quadrature                #
#   int(f(c),c=0..1) == sum(b[j]*f(c[j]),j=1..n)                 #
# are computed by a Newton search for the roots of               #
# the n.th Legendre-polynomial P(c)=(c-c[1])..(c-c[n])           #
# The weights b[j] are obtained by the formula                   #
#  b[j] = (n!)^4/((2n)!)^2 / Pp(c[j])/c[j]/(1-c[j])              #
# where Pp(c)=diff(P(c),c)                                       #
# Remark:  Let L be the j.th Lagrange-polynomial of              #
#          c[1],..,c[n], i.e. L(c)=P(c)/Pp(c[j])/(c-c[j])        #
#          With Lp=diff(L,c) one has                             #
#          L(c)^2                                                #
#           =diff((c-c[j])*L(c)^2,c)-2*(c-c[j])*L(c)*Lp(c)       #
#           =diff(P(c)^2/Pp(c[j])^2/(c-c[j]),c)                  #
#                     -2/Pp(c[j])*P(c)*Lp(c)                     #
#          Integration yields                                    #
#            b[j]=int(L(c)^2,c=0..1)                             #
#                = Pp(c[j])^(-2)*( P(1)^2/(1-c[j])+P(0)^2/c[j])  #
#                = Pp(c[j])^(-2)/c[j]/(1-c[j]) * P(0)^2          #
#          (note that  int(P(c)*Lp(c),c=0..1)=0 due to           #
#          orthogonality).                                       #
#                                                                #
# The data are returned as a list [b,c] of lists b and c.        #
#----------------------------------------------------------------#

#----------------------------------------------------------------#
# gldata(n,digits) -- computes numerical values of the weights   #
# and abscissae of Gauss-Legendre quadrature with n nodes.       # 
#                                                                #
# return value: list of lists [ b , c ],                         #
#               where b=[ b[1],..,b[n] ] , c=[c[1],..,c[n]]      #
#               are the weights and abscissae auf Gaussian       #
#               quadrature:                                      #  
#               int(f(x),x=0..1)= sum( b[i]*f(c[i]), i=1..n )    #
#               (exact for polynomials f(x) through degree 2n-1) #
# Example:                                                       #
# >>  ([b,c]):=gldata(50,DIGITS);                                #
#----------------------------------------------------------------#

numeric::gldata:=proc(n,digits) 
  local P,Pp,Pi1,Pi2,i,j,k,a,b,c,x,dx,f,DIGITS,macheps;
  option remember;
begin
  DIGITS:=digits+trunc(n/10);
  if digits<=200 then
     if n=20  then return(numeric::_gldata20)  end_if;
     if n=40  then return(numeric::_gldata40)  end_if;
     if n=80  then return(numeric::_gldata80)  end_if;
     if n=160 then return(numeric::_gldata160) end_if;
  end_if;
  userinfo(1,"Computing weights and abscissae of ".n."th Gauss formula with ".digits." digits");

  if digits > 200   # waehle folgende Startwerte fuer die Newton-Suche #
    then ([b,c]):=numeric::gldata(n,round(digits/2));
         c:=[ c[k]-1/2 $ hold(k)=1..n ];
    else Pi1:=float(PI/(n+1)):Pi2:=float(PI/2/n):
         b:=array(1..n):
         c:=[-float((cos(k*Pi1)+cos((2*k-1)*Pi2))/4) $ hold(k)=1..n];
  end_if;

  macheps:=float(10^(1-DIGITS));
  for i from 1 to n-1 do a[i]:=1/(16-4/i^2);end_for;
  f:=( (n!)^2/(2*n)! )^2 :
  Pi1:=float(PI/(n+1)):Pi2:=float(PI/2/n):
  P[0]:=1;
  for j from 1 to trunc(n/2) do
      x:=c[j]:
      while TRUE do
         P[1]:=x:
         for i from 1 to n-1 do P[i+1]:=x*P[i]-a[i]*P[i-1];end_for;
         Pp:= n/(4*x^2-1)*(4*x*P[n]-P[n-1]/(2-1/n));
         dx:=P[n]/Pp; x:=x-dx;
         if abs(dx)<macheps then break end_if;
      end_while;
      c[j]:=x+1/2; b[j]:=f/Pp^2/c[j]/(1-c[j]);
  end_for:
  if n mod 2 = 1 then 
      c[(n+1)/2]:=1/2;
      P[1]:=0;
      for i from 1 to n-2 do P[i+1]:=-a[i]*P[i-1];end_for;
      Pp:= n*(P[n-1]/(2-1/n));
      b[(n+1)/2]:= 4*f/Pp^2;
  end_if;
  for j from trunc((n+1)/2)+1 to n do 
     c[j]:=1-c[n+1-j]: b[j]:=b[n+1-j]:
  end_for;
  if not (digits<=200 and (n=20 or n=40 or n=80 or n=160))
        then userinfo(1,"Gauss data computed and stored"): end_if:
  [[b[k]$hold(k)=1..n],[c[k]$hold(k)=1..n]];
 end_proc:

# ------ the following data were generated and stored: --------- #
# _gldata20:=gldata(20,220):                                     #
# _gldata40:=gldata(40,220):                                     #
# _gldata80:=gldata(80,220):                                     #
# _gldata160:=gldata(160,220):                                   #
# write(Text,"gldata",_gldata20,_gldata40,_gldata80,_gldata160); #
# ---------------------------------------------------------------#

numeric::_gldata20:=hold([[0.008807003569576059155930981175926408181071552\
77166836626217466333867420962981092390870155277107304883435185811375628500\
41041614596086783578040550048278105689640330529777692042324908802967413082\
647228310726253685771080907380, 0.0203007149001934706655199761374660549395\
45319994975768408803427280916148375493664147769460311522192488094912854837\
53751130501942102946316212276350947813815211971088478115513801065968319575\
9701426215251664170733207803675480, 0.031336024167054531784753267593520803\
17580053828921818204979217271898740551683283932228188302841610175630162669\
97960366308786050453178528742652821817689686026087282566440478321548098912\
39164125141102226812311462757539598170, 0.04163837078835237436237907161102\
31030500889142915816453724410303928465414470397097356875954218954196745480\
58055966381885299574638544451391145511109384959659308035018219563756898984\
170654523000123496808347092207216507982530, 0.0509650599086202175183750677\
40174938083345828011696278130985808098426161012697173237674657884739929106\
87929517762741632966172038609859914737340559428976134827907921727402083811\
3238030456142119107507291503532850340897752960, 0.059097265980759208656188\
68885569114350252060977448438772344497601008737417525575815286434391290950\
87230313377154615829566943408968711049561271154878559256691928145459892844\
22100232497574160794368942590121959424316537884910, 0.06584431922458831344\
92472498740815674580552555734917634982182468544271782147404665717789875919\
86314622552990027318123508200158415303769352476466288844566100951010590833\
181439897570856762564402015262705462694013132391821600, 0.0710480546591910\
25664649162533582466517257706696010151668683541491914043748967183808474612\
14160029130066534286833100700947012186363710913686181268622437942838106383\
4413296135531348846249733479928752118442251450371112281560, 0.074586493236\
30187339391436850098471834633995204068415824810560890492221129779339034698\
06630176052405258545692728366900343287838896089324715470861712333766023178\
23638711435478034740191661985705125037750986945585185858515500, 0.07637669\
35653629253490421659775487967459743225561892986373505249087987265813688907\
67786243918251952967720009214068939130057513983980039150276501966092113867\
877700960156245755025924951909225364498960425586416904894242396510, 0.0763\
76693565362925349042165977548796745974322556189298637350524908798726581368\
89076778624391825195296772000921406893913005751398398003915027650196609211\
3867877700960156245755025924951909225364498960425586416904894242396510, 0.\
07458649323630187339391436850098471834633995204068415824810560890492221129\
77933903469806630176052405258545692728366900343287838896089324715470861712\
33376602317823638711435478034740191661985705125037750986945585185858515500\
, 0.0710480546591910256646491625335824665172577066960101516686835414919140\
43748967183808474612141600291300665342868331007009470121863637109136861812\
68622437942838106383441329613553134884624973347992875211844225145037111228\
1560, 0.065844319224588313449247249874081567458055255573491763498218246854\
42717821474046657177898759198631462255299002731812350820015841530376935247\
64662888445661009510105908331814398975708567625644020152627054626940131323\
91821600, 0.05909726598075920865618868885569114350252060977448438772344497\
60100873741752557581528643439129095087230313377154615829566943408968711049\
56127115487855925669192814545989284422100232497574160794368942590121959424\
316537884910, 0.0509650599086202175183750677401749380833458280116962781309\
85808098426161012697173237674657884739929106879295177627416329661720386098\
59914737340559428976134827907921727402083811323803045614211910750729150353\
2850340897752960, 0.041638370788352374362379071611023103050088914291581645\
37244103039284654144703970973568759542189541967454805805596638188529957463\
85444513911455111093849596593080350182195637568989841706545230001234968083\
47092207216507982530, 0.03133602416705453178475326759352080317580053828921\
81820497921727189874055168328393222818830284161017563016266997960366308786\
05045317852874265282181768968602608728256644047832154809891239164125141102\
226812311462757539598170, 0.0203007149001934706655199761374660549395453199\
94975768408803427280916148375493664147769460311522192488094912854837537511\
30501942102946316212276350947813815211971088478115513801065968319575970142\
6215251664170733207803675480, 0.008807003569576059155930981175926408181071\
55277166836626217466333867420962981092390870155277107304883435185811375628\
50041041614596086783578040550048278105689640330529777692042324908802967413\
082647228310726253685771080907380], [0.00343570040745253760693880576433986\
08886764345491720519259079343910076411861231095802752987517138953626355298\
26377904930992569806300611164380793160354816399076081830405826269277676603\
8339272075502793936493240707701835673080, 0.018014036361043104366166934401\
36138904396983609690557196823120305397920960780847150999093737201764218434\
47825420178847355697946027024579041993551513071937278818638897980327535003\
43080840087354133210556134639182162029133100, 0.04388278587433704706612377\
93983509434754076012881541126020588904209645543956404605317776369035393063\
10619804122676986794131568308530581759393113446361791973035456913405600772\
808165424113498744343741974622819277350058828600, 0.0804415140888905883027\
35469149239657335185317467181313373753637233569453000337595040385329717021\
17538969788982346542952721278661648302194216077215360641743481926211398087\
8876920781267161719179712257930367228358992480529020, 0.126834046769924603\
69284746482217920484463466021541177793022704696573232248092246765944794318\
96762396938075496741617192518442845135985546605312263799865392908051389965\
5183555104705748952256284933392787573242540214395444060, 0.181973159636742\
48727358165188685703162830544160031576803027668872672936870728493372064840\
22521193467089414453111370213189794594851453983797276502630278174731334046\
8392067556029214937626217089190198973102410724841413362020, 0.244566499024\
58645099781797452237450078725433539878658326382569005263251480461713592798\
41584745661104008352846557823688217173729887154426043652191306774696998082\
9332301635175922665450217192429494542049945212275680369340660, 0.313146955\
64229021966372591148753638130212683914715864408602569324217711781347023605\
20521581827333105276161389557359249234703249465777863022749668053200961561\
6839418654196529839216316258648981684065772697831590140444728710, 0.386107\
07442917746095975190231571268762845553115853626384268213039641432906822208\
61025239374045156459841131343421978489128504795617935939325736345263448244\
8844939556145554492365501360653435241763604168848889582338812571100, 0.461\
73673943325133312267979530058089449760186659325124959760237780787182897583\
15108792274429092218921965007471768179336477797310877143685576444056413615\
9330752596434826803838897133425155160117106879989365458450597786775090, 0.\
53826326056674866687732020469941910550239813340674875040239762219212817102\
41684891207725570907781078034992528231820663522202689122856314423555943586\
3840669247403565173196161102866574844839882893120010634541549402213224900,\
 0.61389292557082253904024809768428731237154446884146373615731786960358567\
09317779138974760625954843540158868656578021510871495204382064060674263654\
73655175511550604438544455076344986393465647582363958311511104176611874288\
90, 0.68685304435770978033627408851246361869787316085284135591397430675782\
28821865297639479478418172666894723838610442640750765296750534222136977250\
33194679903843831605813458034701607836837413510183159342273021684098595552\
71280, 0.75543350097541354900218202547762549921274566460121341673617430994\
73674851953828640720158415254338895991647153442176311782826270112845573956\
34780869322530300191706676983648240773345497828075705054579500547877243196\
30659330, 0.81802684036325751272641834811314296837169455839968423196972331\
12732706312927150662793515977478806532910585546888629786810205405148546016\
20272349736972182526866595316079324439707850623737829108098010268975892751\
58586637980, 0.87316595323007539630715253517782079515536533978458822206977\
29530342676775190775323405520568103237603061924503258382807481557154864014\
45339468773620013460709194861003448164448952942510477437150666072124267574\
59785604555930, 0.91955848591110941169726453085076034266481468253281868662\
62463627664305469996624049596146702829788246103021101765345704727872133835\
16978057839227846393582565180737886019121123079218732838280820287742069632\
77164100751947100, 0.95611721412566295293387622060164905652459239871184588\
73979411095790354456043595394682223630964606936893801958773230132058684316\
91469418240606886553638208026964543086594399227191834575886501255656258025\
37718072264994117140, 0.98198596363895689563383306559863861095603016390309\
44280317687969460207903921915284900090626279823578156552174579821152644302\
05397297542095800644848692806272118136110201967246499656919159912645866789\
44386536081783797086690, 0.99656429959254746239306119423566013911132356545\
08279480740920656089923588138768904197247012482861046373644701736220950690\
07430193699388835619206839645183600923918169594173730722323396166072792449\
72060635067592922981643270]]):
numeric::_gldata40:=hold([[0.002260638549266595629235866439092666363915555\
09985299535017097117133197350097411256564381791394501546975337347823810498\
99945179377024438002555406402102698315098574119414254982737902207169719768\
952830219201129805659638471670, 0.0052491422655764068073710855336398261883\
96310657898389857837561805812649448455415350268415237016122072935488380193\
77978648623088510542712020031142997331142511922483867810749227968567215920\
80345328773682354792204695324227950, 0.00821052919095394435643174244118196\
36461711466734788809859519577567952196506051368343276861311919417545337042\
40683103384885676082446942156281145765519014026153776006290858191711211721\
9427526511358097872092872342916090759220, 0.011122924597083478630752162092\
10428660351659833967721394036712000284565576007898749712410725965659951898\
72883053273579007760444230403920891779448671492911206543249054517841889289\
55688127868856541784922321252494742163269570, 0.01396850349001170054924457\
87538605386512754310253831992240429582251874497408454697208541741045320786\
18222461056553151033887614044407620514494031894092376516976599736939098554\
404834082261256812026001773231517736667770170610, 0.0167300976412739236963\
39091543205424488620893326882175206595588349526199885632576132150106303547\
10579588870690746874103516486597544115856605707020445472436532227440815626\
9191700991111406585535451308096234888254558994566470, 0.019391083987236008\
81998601564522308112672960561601292891863344445976936507184537690805539984\
27113617107880299268037054689948423547393572840542005063410606080751078405\
64362591777112635294179343129838154451709204190753741080, 0.02193545409283\
66359958373430208577479055034185851184448792629543249008929029589611444811\
03847769755671117402471971012509316500900587432889911117878001073270338178\
870143012729141222286663812622016766761434847961069530654150, 0.0243479038\
17536116030717080224073194033921513688559887570323025637589214208740437510\
40640610979809662606473504757190822252794141389252870860980083741719273234\
3756677425963265794226326329920612280162904497853707950161569440, 0.026613\
92349196841217749823988613025227766058591100322538218044026326040751093553\
73962225154703271330875787351184076432813140178042013212235188814050401557\
84665094144350872767245588855837501987497371142813705821392143533510, 0.02\
87198845496957756833088654552129928000524179272268132533406334627741808556\
95008376928381407686743421703528148767155421787685829556833406913761027373\
036973555856369061478575725274291871388945071590616604187981641703123920, \
0.030653121246464469583268998204199297951296881755587160993448819343434077\
91155872968446474991053807162582739616243488318171310101375056421794339747\
88344973447931695661227798033261554748953055947946650358896761640845281974\
80, 0.03240200672830051903727726478337636501634648210424445318159762213564\
19864628428130165116336747984880562291957100568111759435724231085555295423\
02473821459332307194688007336519620411728417067348654707609168460915248649\
901230, 0.0339560229076169519128450541159619929920986191896429773589481017\
96520390813439686841513509448360309675182950885958577819856358516208500560\
80137429709481656691919077171567874998139212153743888667326561670517768207\
9571869190, 0.035305823695643389847741815427643416179779551997793048655342\
13448133184865597423887730006802079789250579199270881138377222952155551328\
69719067441341296459340714486351195240528400162890022391449817827579058623\
99474736505000, 0.03644329119790202953025534172125891792877954049289931523\
81274084164952778653717010948543908749737005075355783486950370503138512197\
41504922972655743883917927366972045831433940706967742507886483006603415205\
913300355321257220, 0.0373615845289841321000946681306623365956014672101799\
10865670445708826701516361777719574153004517853056182588175241411113539125\
20206113768502512651281243821463089453092601490278789233655336101520216132\
1192834003834799079750, 0.038055180950313121185779037961247411506279776922\
53555352709122951464962031791629154265481286443261453611905337539907267598\
26692224147711229846931883311941684965795341618832478521322271597895572406\
07148782985749721577584240, 0.03851990908212398279415376714190512426221987\
70819702077445904462162708215452322186002541943661304181451859905241290732\
29221287451471062346802716198835438332093223141031637441951186757776597779\
729781213227060131713124851770, 0.0387529739892124056318619814791631634818\
34326394053217866477136927036066374535523186410374207268698144371801027288\
08694606457574460915888254718351668508552460727587808047829395095242953886\
6813137631332967187055947364779430, 0.038752973989212405631861981479163163\
48183432639405321786647713692703606637453552318641037420726869814437180102\
72880869460645757446091588825471835166850855246072758780804782939509524295\
38866813137631332967187055947364779430, 0.03851990908212398279415376714190\
51242622198770819702077445904462162708215452322186002541943661304181451859\
90524129073229221287451471062346802716198835438332093223141031637441951186\
757776597779729781213227060131713124851770, 0.0380551809503131211857790379\
61247411506279776922535553527091229514649620317916291542654812864432614536\
11905337539907267598266922241477112298469318833119416849657953416188324785\
2132227159789557240607148782985749721577584240, 0.037361584528984132100094\
66813066233659560146721017991086567044570882670151636177771957415300451785\
30561825881752414111135391252020611376850251265128124382146308945309260149\
02787892336553361015202161321192834003834799079750, 0.03644329119790202953\
02553417212589179287795404928993152381274084164952778653717010948543908749\
73700507535578348695037050313851219741504922972655743883917927366972045831\
433940706967742507886483006603415205913300355321257220, 0.0353058236956433\
89847741815427643416179779551997793048655342134481331848655974238877300068\
02079789250579199270881138377222952155551328697190674413412964593407144863\
5119524052840016289002239144981782757905862399474736505000, 0.033956022907\
61695191284505411596199299209861918964297735894810179652039081343968684151\
35094483603096751829508859585778198563585162085005608013742970948165669191\
90771715678749981392121537438886673265616705177682079571869190, 0.03240200\
67283005190372772647833763650163464821042444531815976221356419864628428130\
16511633674798488056229195710056811175943572423108555529542302473821459332\
307194688007336519620411728417067348654707609168460915248649901230, 0.0306\
53121246464469583268998204199297951296881755587160993448819343434077911558\
72968446474991053807162582739616243488318171310101375056421794339747883449\
7344793169566122779803326155474895305594794665035889676164084528197480, 0.\
02871988454969577568330886545521299280005241792722681325334063346277418085\
56950083769283814076867434217035281487671554217876858295568334069137610273\
73036973555856369061478575725274291871388945071590616604187981641703123920\
, 0.0266139234919684121774982398861302522776605859110032253821804402632604\
07510935537396222515470327133087578735118407643281314017804201321223518881\
40504015578466509414435087276724558885583750198749737114281370582139214353\
3510, 0.024347903817536116030717080224073194033921513688559887570323025637\
58921420874043751040640610979809662606473504757190822252794141389252870860\
98008374171927323437566774259632657942263263299206122801629044978537079501\
61569440, 0.02193545409283663599583734302085774790550341858511844487926295\
43249008929029589611444811038477697556711174024719710125093165009005874328\
89911117878001073270338178870143012729141222286663812622016766761434847961\
069530654150, 0.0193910839872360088199860156452230811267296056160129289186\
33444459769365071845376908055399842711361710788029926803705468994842354739\
35728405420050634106060807510784056436259177711263529417934312983815445170\
9204190753741080, 0.016730097641273923696339091543205424488620893326882175\
20659558834952619988563257613215010630354710579588870690746874103516486597\
54411585660570702044547243653222744081562691917009911114065855354513080962\
34888254558994566470, 0.01396850349001170054924457875386053865127543102538\
31992240429582251874497408454697208541741045320786182224610565531510338876\
14044407620514494031894092376516976599736939098554404834082261256812026001\
773231517736667770170610, 0.0111229245970834786307521620921042866035165983\
39677213940367120002845655760078987497124107259656599518987288305327357900\
77604442304039208917794486714929112065432490545178418892895568812786885654\
1784922321252494742163269570, 0.008210529190953944356431742441181963646171\
14667347888098595195775679521965060513683432768613119194175453370424068310\
33848856760824469421562811457655190140261537760062908581917112117219427526\
511358097872092872342916090759220, 0.0052491422655764068073710855336398261\
88396310657898389857837561805812649448455415350268415237016122072935488380\
19377978648623088510542712020031142997331142511922483867810749227968567215\
92080345328773682354792204695324227950, 0.00226063854926659562923586643909\
26663639155550998529953501709711713319735009741125656438179139450154697533\
73478238104989994517937702443800255540640210269831509857411941425498273790\
2207169719768952830219201129805659638471670], [0.0008811451447203998251886\
48789706753832114809202477046698331941451580568490136097461056906134395857\
82665140186559369519256605122470078670940125281687770491228057007984868689\
460774712433477200928852835552693155239290460165430, 0.0046368806502714967\
73472823889313922518888959324456731058461877847960548380727701685444747008\
74891609507521586197013318940142267835569070583425218312572415678348936529\
96977984991802261358691607020453356042454636222973661690, 0.01137002500811\
28686683148581435480965106660339810069517853689919944683927778927714061553\
49555635070663846828463735492345493469986647650131771989848508173890624744\
990795242172759855098868989095543415875461793096856692189460, 0.0210415903\
93104172097729500273620357452558254698626484215473078209120822188258135339\
78969418218148947901918487601327276254835491839357065940976642013529087387\
8871333232219891646437462609993424910627123807681088352649790370, 0.033593\
59586066173331957391657739714178262321235865377538532849229217973803832688\
52330726563291077268888167724825377851249677534929453528255191498907815024\
74098703599797414557326199206453272453247595069094261850563824632150, 0.04\
89505965155628516358733345657534482077559594711652661373699442298149092995\
07646302053006132959700898673853835888222388670273947558039696186355597992\
383294609489661109282010721809765219447563065868340714430804278744044680, \
0.067020248393870248089609095822690018214726723494449175270029872596658960\
57645872234954171889154026684228983249667988967541598484285504126949984661\
92983949233793784057295362522653775417361647190228871354876645488388671235\
70, 0.08769388458334416840183988466695061304637980787850610015410451842359\
64140786521377837752374839501079789238505307213467875409521862626660106124\
12969000645815609670312911634096460482950785512814057871567698035752999902\
380450, 0.1108471742867403061525142272467525759896543419365541028773289696\
85344029041909714279399954177202294952408497310238337624615067901472763857\
64200718392469234924701178802891427409417594088036756805521636267134955936\
080409190, 0.1363408724050364483595017741225347257213106632333359312262243\
77907035442676955763696229446987519481411281413965837442433965984600899611\
07204105380742787011815717504999615509694439587643046672102871250557643532\
607581531700, 0.1640216576929102258103227425192529450148370093080744249644\
53515506668762021690117026984027529287625792815316757667987529478301621266\
57861591377219636770555500485004450213553018440858130655678712282171622544\
224833608790130, 0.1937230551660098810236937748846525613099381091584251944\
32326218734904513787780882389939020190284945715585186167124602232271757042\
28080251838251921949107683487672746871612812868388322103691434176927168310\
273594916495543910, 0.2252664374524358989620343472352410148830124492021742\
67548717264139072127443258204764118353877182768357659219079111445100003876\
80395596367157736037206176017713228625150701494596644797743820872839046529\
985785883392642297270, 0.2584620991569106435457167128775884977004888022334\
42949605332378387028278629046570377062447589421260696440589395414834766856\
24505673129221906436229006084502164548158303898080545571851586357459297997\
728410645426782069792820, 0.2931103978141974992375601270981431585129501879\
73540880852798874465678668195389634281081970564256063211708593628390008593\
51957272382624426965087816167983637265509974561048451778329584475841104249\
107702261866805808152635460, 0.3290029545871207634962537594104028449665231\
89986337101768295207950928145528939558338402266675999867495140824372534198\
32227259531619514263222282136976024861159333737518532081260544806614459842\
526432021439611877732982368420, 0.3659239074963731594294078275957019082875\
97813381876509271370012563089403403132887823904812400734758947248442120759\
66856149973103223913935973907014837143727523759737311834629865877073846282\
581441335388949551713545865775560, 0.4036512096493144501422415739674250525\
92953989447395562557266203881748639168178294225037689628699240721844393951\
92373317479612705785084841108238128441364031594276982104350467178089038419\
997991848107146903497587939360126840, 0.4419579646623723957582743577959879\
43115635734572892025484851283919210418026066413242311443269453078610131598\
96586449863017690854767157836686906314968562728800200997197108367517590110\
720522436304158172876181423396689542900, 0.4806137912469745890334032779876\
88352660317682808429200415681409280996712499539621895697863096824257804962\
96395315829602730133396097548086655244099447629168693420800253998694580755\
734609924667634892184477896973177856724940, 0.5193862087530254109665967220\
12311647339682317191570799584318590719003287500460378104302136903175742195\
03703604684170397269866603902451913344755900552370831306579199746001305419\
244265390075332365107815522103026822143275050, 0.5580420353376276042417256\
42204012056884364265427107974515148716080789581973933586757688556730546921\
38986840103413550136982309145232842163313093685031437271199799002802891632\
482409889279477563695841827123818576603310457100, 0.5963487903506855498577\
58426032574947407046010552604437442733796118251360831821705774962310371300\
75927815560604807626682520387294214915158891761871558635968405723017895649\
532821910961580002008151892853096502412060639873150, 0.6340760925036268405\
70592172404298091712402186618123490728629987436910596596867112176095187599\
26524105275155787924033143850026896776086064026092985162856272476240262688\
165370134122926153717418558664611050448286454134224430, 0.6709970454128792\
36503746240589597155033476810013662898231704792049071854471060441661597733\
32400013250485917562746580167772740468380485736777717863023975138840666262\
481467918739455193385540157473567978560388122267017631570, 0.7068896021858\
02500762439872901856841487049812026459119147201125534321331804610365718918\
02943574393678829140637160999140648042727617375573034912183832016362734490\
025438951548221670415524158895750892297738133194191847364540, 0.7415379008\
43089356454283287122411502299511197766557050394667621612971721370953429622\
93755241057873930355941060458516523314375494326870778093563770993915497835\
451841696101919454428148413642540702002271589354573217930207170, 0.7747335\
62547564101037965652764758985116987550797825732451282735860927872556741795\
23588164612281723164234078092088855489999612319604403632842263962793823982\
286771374849298505403355202256179127160953470014214116607357702720, 0.8062\
76944833990118976306225115347438690061890841574805567673781265095486212219\
11761006097980971505428441481383287539776772824295771919748161748078050892\
316512327253128387187131611677896308565823072831689726405083504456080, 0.8\
35978342307089774189677257480747054985162990691925575035546484493331237978\
30988297301597247071237420718468324233201247052169837873342138408622780363\
229444499514995549786446981559141869344321287717828377455775166391209870, \
0.863659127594963551640498225877465274278689336766664068773775622092964557\
32304423630377055301248051858871858603416255756603401539910038892795894619\
25721298818428249500038449030556041235695332789712874944235646739241846829\
0, 0.889152825713259693847485772753247424010345658063445897122671030314655\
97095809028572060004582279770504759150268976166237538493209852723614235799\
28160753076507529882119710857259058240591196324319447836373286504406391959\
0800, 0.912306115416655831598160115333049386953620192121493899845895481576\
40358592134786221622476251604989202107614946927865321245904781373733398938\
75870309993541843903296870883659035395170492144871859421284323019642470000\
9761950, 0.932979751606129751910390904177309981785273276505550824729970127\
40334103942354127765045828110845973315771016750332011032458401515714495873\
05001533807016050766206215942704637477346224582638352809771128645123354511\
6113287640, 0.951049403484437148364126665434246551792244040528834733862630\
05577018509070049235369794699386704029910132614616411177761132972605244196\
03038136444020076167053905103388907179892781902347805524369341316592855691\
9572125595530, 0.966406404139338266680426083422602858217376787641346224614\
67150770782026196167311476692734367089227311118322751746221487503224650705\
46471744808501092184975259012964002025854426738007935467275467524049309057\
3814943617536780, 0.978958409606895827902270499726379642547441745301373515\
78452692179087917781174186466021030581781851052098081512398672723745164508\
16064293405902335798647091261211286667677801083535625373900065750893728761\
9231891164735020960, 0.988629974991887131331685141856451903489333966018993\
04821463100800553160722210722859384465044436492933615317153626450765450653\
00133523498682280101514918261093752550092047578272401449011310109044565841\
2453820690314330781050, 0.995363119349728503226527176110686077481111040675\
54326894153812215203945161927229831455525299125108390492478413802986681059\
85773216443092941657478168742758432165106347003022015008197738641308392979\
5466439575453637770263380, 0.999118854855279600174811351210293246167885190\
79752295330166805854841943150986390253894309386560414217334859813440630480\
74339487752992132905987471831222950877194299201513131053922528756652279907\
1147164447306844760709539830]]):
numeric::_gldata80:=hold([[0.000572475001593470767272085970657818059349696\
20278795143416312053706697756051137859270826040242253309254660277890987122\
18891156639605823200176287725205163143643799245384586372544159518301864474\
3472874850388226109681052379100, 0.001331766794756340834646767915834227732\
87227712119357428153678392970418764392796701812794147921567170003985285487\
55406669102795823754824290101894799654347610548646590095726447692842486435\
777779633532452287080336236838770420, 0.0020901565623474476183696521008406\
75661774748686553219761029401209299808473915671593190788433445098164076189\
46892937800072142174189659192498780353376715512857504598997501189101113547\
94487149921364536654441760324481864887070, 0.00284546122570159932463455355\
85810092384626319173385662609839505855502536704456336104362584404759241669\
91793104045212638182070245125973566195609257362313718188312559984895050053\
5696001482411312496284987510487681250609326360, 0.003596452384058656376337\
78543397825373825351909618265002231588200150993266780772581454175110380046\
85391677077124809866628441617726330057937699659715287641184083269405918595\
019247208512477083408436885110756455063328680537550, 0.0043419726346304292\
13204726102017140676203021477532704853841389898257569947822208691844206847\
32438769448256471358875989947082964373713112469577365690672928207332809292\
19124230850666429802046232025750744773777013182504570180, 0.00508088302055\
15322604159251762034718320228909398202626470343423378780892813409240864055\
91736511751194453861924810805668498696714732342097295185275093331549479642\
6144512937672705460941512563524010851221352701564915735219260, 0.005812057\
06039891345823338499771631742975659075148106648254502141041757719411432114\
89527148572966429952182909139524671880739623339819587535166358931954030005\
898845032077978440078777540637600782707876006904189141741541154250, 0.0065\
34380796200669646893412948528170155209317191207216772173946955633778407500\
59578816920304016300230881382590614114512127560575295394453893516547421964\
87419287776063033620709407891679505691090022738287721620563857978083720, 0\
.0072467540202545380584810372917302750279784360775577565282234655292445366\
21065412290360856569803616641812931221102155905627864354371147807059409598\
37707974866731975508464619763371260443154743679881271357391922429722557741\
20, 0.00794809179186284402245145461458926288548604630286096636342867173596\
70476356828994846395609275345200979743647935021692537545872399074773179789\
56060148968180305703275890349664522406080427332645208473234496002841444401\
3980650, 0.008637326028134653179292103565645499900155514652005170503587891\
07881962589689569147812533539255577276036643844876507567277021684174256034\
02988908318090572348011032726803666859795563749322800104477073035279838617\
871717724160, 0.0093134071041495157143677070760786045042238831680822117251\
79647269213178506677550601988789877026758541021039987634283060394475436063\
39808142785498151340193708621192992615847063684534834414620890844039090194\
91189008692423400, 0.00997530543907099946444596435755678168025053214250796\
53333736292310294022308705533759145327819757527995224171020922254107065537\
46559121835684874315712998012850696052620119495393243257720632690354469233\
8128901188605072390210, 0.010622013057891003194355368625306564273234462121\
65399025415468843410000095594165167993020618336025991206481895012451481326\
85752816461289504010655547211931221230836255492235040720595007334464824759\
34683179139897433169795810, 0.01125254512316623096311079484308436951028979\
41859185314922592121522041811166917762434154335590211474213709589475588861\
65077852933637877717311694140550648875162885940133493793258861808490635819\
045168379867654369095602916340, 0.0118659414329650506465962623067842081461\
71264554160947885795955512437268880270398511146428012820257107606921345391\
72219255084323664532375335320906814769547990607802070771837476079956993174\
3338214290071586104801447981562630, 0.012461267882057745552558923501609901\
17855124493778084775729618394903480984668937008447342153114040863688906240\
32214489027662176977381267514332110466441884940895044025283057911582395023\
35102332321127887609094339666558928380, 0.01303761788378255895148437180013\
46435628487379146196549756746490059785798517215082895369080551056166275498\
11544997981293431635867362326475816392242053768567902070718490414380860818\
619507852155281155521030025495140495967910, 0.0135941137502431903372093533\
40272129914938587850053535200042209014590046329154650448510855050952975466\
92881607623483052114820684163302786460593763408672807411757690182811261249\
5744298636441044123519573556753003712061119160, 0.014129908028638431198376\
59898250726514713274918656989590014344483169698988129689197852363783957095\
99258505371926174043753792654583916223514366578626964534582075273999466387\
30853120137184539097777368605791803319879310952380, 0.01464418479163392384\
63837930097895698306037155723153975949428749347751707411308936875323687173\
80455721907853480603593245156234372311237024377276953409818614686318366920\
574659747009252537787178721441189273473310305017284340, 0.0151361608797789\
90330610005045450587373671033779844639037890513864957811766188343284924991\
31340560965996429566416954789655305748320375758648087469273960184569320569\
7870873959927372531803989795144671525610686074459122531270, 0.015605087094\
05735082122143336030177593299986041014138300477390455773028548731915146161\
53922476139733493626879886742025751610272294059571686807775943167854125614\
80001390451348750038849260973684283106188658050390910332706230, 0.01605024\
93367438865740282451436253480447583819162968249133822192699086776854714570\
14750589235113438182058809635302947136243926451249585064663566241955050753\
058596689752603407719441720006456891378469418854062050309326001440, 0.0164\
70969698822700691418090450979768064013518846364308832099771420575857010433\
45611532682626115779004597047751312848668765095370894048302998956138145166\
7284365140222853970096651729117870756869156455194174425256472451383700, 0.\
01686660749230576140833758153211936422291325192408335139986613162415352863\
06718371720277009117050593381920955937350778075799691568793051248349343680\
46582999625575212394178220634356799546215911640080323070957330456219445600\
, 0.0172365602258769643971821133655149160383903983714764899516462426868764\
49849647049044277981331509768408985271470479269147235261089917493664460537\
02643675153830656477428029531502525468850800880135831016606292845562975498\
2840, 0.017580264522373796747763296194348440614581226155256749234881596182\
13680885619671584114666406580332049653177744176526232033269560029386166739\
06877186512174416600441408505401491789932159846257617159508381514942617144\
73274030, 0.01789719697670802730143079440807722712011806761528763539690006\
73031096556273502733579839822759411246948345072981737381673448026106226166\
32246686489757087350933167647452018679831330949895616707499959692401545339\
659559171730, 0.0181868749529179890219824955232614068300314108938050859728\
11480878201883167824945981878642715554419550474598777882679411215693231845\
93955768284507441919836981384167849591911725685161715957833710908583286913\
4190819546664140, 0.018448857319138004419575498286702609634284050565914312\
92065016135397458624760081768008704137361245045741398107744667799615186371\
63049059240787462729005732568552116131755279225907577195480497639683389232\
12495919215348964900, 0.01868274511936524501335268852891933458240345397471\
36898523924069179942919658287013007855034354602989280535416176331918020046\
86643109345346084561760585231580748598385972031170323673513319998567292296\
835938621219575864898370, 0.0188881821810006987448874882131605273853509620\
09784757380645423735199159734253292570249876271032133801993135468494340540\
01098916517971112048640220403631930680224313175724038543075722614419698920\
4485373339372299164232879610, 0.019064855657238819172103395782868100957071\
96195327951702644391348008804724782093111126226548466966897972658513351307\
67820491649138543139505587753402930595957676088036599583644518577002513027\
49432703095665286909654732036760, 0.01921249650347971159260621816474506921\
55109381354551231940509912683082962122985154758793046860125774241953632326\
99477378314655264727592753866686289173336604362373343221393007691209716646\
250652429173545342715651527209627020, 0.0193308798870382316635385551335783\
45630450463919932210008741811010081313272223429215362231367565088635248860\
15573456531433046879347802280686955564765100681921540968179524476298931072\
8434593219764678768244829352668517734910, 0.019419825529525984465887091334\
39358294544011467024292714551680892863198541994387668576553656453149300125\
01622839665250776352075048665834618275739490354478270152407332132610128475\
28257128853157113130207970639682503387750060, 0.01947919798138476559931276\
23861304111574660057931347807992357461916559806204427488037707307213407637\
11115082211912407724808799259438907911809921647564174672248756105530058636\
338526512837724121533415772299257710336280978030, 0.0195089068281533274056\
40219626377024164775237014802129898344447782006367857008783384429688899067\
03981797210656370806835547810541727522840837791869655561092639829958304414\
8250671186989975102727793205214848014772956879434380, 0.019508906828153327\
40564021962637702416477523701480212989834444778200636785700878338442968889\
90670398179721065637080683554781054172752284083779186965556109263982995830\
44148250671186989975102727793205214848014772956879434380, 0.01947919798138\
47655993127623861304111574660057931347807992357461916559806204427488037707\
30721340763711115082211912407724808799259438907911809921647564174672248756\
105530058636338526512837724121533415772299257710336280978030, 0.0194198255\
29525984465887091334393582945440114670242927145516808928631985419943876685\
76553656453149300125016228396652507763520750486658346182757394903544782701\
5240733213261012847528257128853157113130207970639682503387750060, 0.019330\
87988703823166353855513357834563045046391993221000874181101008131327222342\
92153622313675650886352488601557345653143304687934780228068695556476510068\
19215409681795244762989310728434593219764678768244829352668517734910, 0.01\
92124965034797115926062181647450692155109381354551231940509912683082962122\
98515475879304686012577424195363232699477378314655264727592753866686289173\
336604362373343221393007691209716646250652429173545342715651527209627020, \
0.019064855657238819172103395782868100957071961953279517026443913480088047\
24782093111126226548466966897972658513351307678204916491385431395055877534\
02930595957676088036599583644518577002513027494327030956652869096547320367\
60, 0.01888818218100069874488748821316052738535096200978475738064542373519\
91597342532925702498762710321338019931354684943405400109891651797111204864\
02204036319306802243131757240385430757226144196989204485373339372299164232\
879610, 0.0186827451193652450133526885289193345824034539747136898523924069\
17994291965828701300785503435460298928053541617633191802004686643109345346\
08456176058523158074859838597203117032367351331999856729229683593862121957\
5864898370, 0.018448857319138004419575498286702609634284050565914312920650\
16135397458624760081768008704137361245045741398107744667799615186371630490\
59240787462729005732568552116131755279225907577195480497639683389232124959\
19215348964900, 0.01818687495291798902198249552326140683003141089380508597\
28114808782018831678249459818786427155544195504745987778826794112156932318\
45939557682845074419198369813841678495919117256851617159578337109085832869\
134190819546664140, 0.0178971969767080273014307944080772271201180676152876\
35396900067303109655627350273357983982275941124694834507298173738167344802\
61062261663224668648975708735093316764745201867983133094989561670749995969\
2401545339659559171730, 0.017580264522373796747763296194348440614581226155\
25674923488159618213680885619671584114666406580332049653177744176526232033\
26956002938616673906877186512174416600441408505401491789932159846257617159\
50838151494261714473274030, 0.01723656022587696439718211336551491603839039\
83714764899516462426868764498496470490442779813315097684089852714704792691\
47235261089917493664460537026436751538306564774280295315025254688508008801\
358310166062928455629754982840, 0.0168666074923057614083375815321193642229\
13251924083351399866131624153528630671837172027700911705059338192095593735\
07780757996915687930512483493436804658299962557521239417822063435679954621\
5911640080323070957330456219445600, 0.016470969698822700691418090450979768\
06401351884636430883209977142057585701043345611532682626115779004597047751\
31284866876509537089404830299895613814516672843651402228539700966517291178\
70756869156455194174425256472451383700, 0.01605024933674388657402824514362\
53480447583819162968249133822192699086776854714570147505892351134381820588\
09635302947136243926451249585064663566241955050753058596689752603407719441\
720006456891378469418854062050309326001440, 0.0156050870940573508212214333\
60301775932999860410141383004773904557730285487319151461615392247613973349\
36268798867420257516102722940595716868077759431678541256148000139045134875\
0038849260973684283106188658050390910332706230, 0.015136160879778990330610\
00504545058737367103377984463903789051386495781176618834328492499131340560\
96599642956641695478965530574832037575864808746927396018456932056978708739\
59927372531803989795144671525610686074459122531270, 0.01464418479163392384\
63837930097895698306037155723153975949428749347751707411308936875323687173\
80455721907853480603593245156234372311237024377276953409818614686318366920\
574659747009252537787178721441189273473310305017284340, 0.0141299080286384\
31198376598982507265147132749186569895900143444831696989881296891978523637\
83957095992585053719261740437537926545839162235143665786269645345820752739\
9946638730853120137184539097777368605791803319879310952380, 0.013594113750\
24319033720935334027212991493858785005353520004220901459004632915465044851\
08550509529754669288160762348305211482068416330278646059376340867280741175\
76901828112612495744298636441044123519573556753003712061119160, 0.01303761\
78837825589514843718001346435628487379146196549756746490059785798517215082\
89536908055105616627549811544997981293431635867362326475816392242053768567\
902070718490414380860818619507852155281155521030025495140495967910, 0.0124\
61267882057745552558923501609901178551244937780847757296183949034809846689\
37008447342153114040863688906240322144890276621769773812675143321104664418\
8494089504402528305791158239502335102332321127887609094339666558928380, 0.\
01186594143296505064659626230678420814617126455416094788579595551243726888\
02703985111464280128202571076069213453917221925508432366453237533532090681\
47695479906078020707718374760799569931743338214290071586104801447981562630\
, 0.0112525451231662309631107948430843695102897941859185314922592121522041\
81116691776243415433559021147421370958947558886165077852933637877717311694\
14055064887516288594013349379325886180849063581904516837986765436909560291\
6340, 0.010622013057891003194355368625306564273234462121653990254154688434\
10000095594165167993020618336025991206481895012451481326857528164612895040\
10655547211931221230836255492235040720595007334464824759346831791398974331\
69795810, 0.00997530543907099946444596435755678168025053214250796533337362\
92310294022308705533759145327819757527995224171020922254107065537465591218\
35684874315712998012850696052620119495393243257720632690354469233812890118\
8605072390210, 0.009313407104149515714367707076078604504223883168082211725\
17964726921317850667755060198878987702675854102103998763428306039447543606\
33980814278549815134019370862119299261584706368453483441462089084403909019\
491189008692423400, 0.0086373260281346531792921035656454999001555146520051\
70503587891078819625896895691478125335392555772760366438448765075672770216\
84174256034029889083180905723480110327268036668597955637493228001044770730\
35279838617871717724160, 0.00794809179186284402245145461458926288548604630\
28609663634286717359670476356828994846395609275345200979743647935021692537\
54587239907477317978956060148968180305703275890349664522406080427332645208\
4732344960028414444013980650, 0.007246754020254538058481037291730275027978\
43607755775652822346552924453662106541229036085656980361664181293122110215\
59056278643543711478070594095983770797486673197550846461976337126044315474\
367988127135739192242972255774120, 0.0065343807962006696468934129485281701\
55209317191207216772173946955633778407500595788169203040163002308813825906\
14114512127560575295394453893516547421964874192877760630336207094078916795\
05691090022738287721620563857978083720, 0.00581205706039891345823338499771\
63174297565907514810664825450214104175771941143211489527148572966429952182\
90913952467188073962333981958753516635893195403000589884503207797844007877\
7540637600782707876006904189141741541154250, 0.005080883020551532260415925\
17620347183202289093982026264703434233787808928134092408640559173651175119\
44538619248108056684986967147323420972951852750933315494796426144512937672\
705460941512563524010851221352701564915735219260, 0.0043419726346304292132\
04726102017140676203021477532704853841389898257569947822208691844206847324\
38769448256471358875989947082964373713112469577365690672928207332809292191\
24230850666429802046232025750744773777013182504570180, 0.00359645238405865\
63763377854339782537382535190961826500223158820015099326678077258145417511\
03800468539167707712480986662844161772633005793769965971528764118408326940\
5918595019247208512477083408436885110756455063328680537550, 0.002845461225\
70159932463455355858100923846263191733856626098395058555025367044563361043\
62584404759241669917931040452126381820702451259735661956092573623137181883\
125599848950500535696001482411312496284987510487681250609326360, 0.0020901\
56562347447618369652100840675661774748686553219761029401209299808473915671\
59319078843344509816407618946892937800072142174189659192498780353376715512\
85750459899750118910111354794487149921364536654441760324481864887070, 0.00\
13317667947563408346467679158342277328722771211935742815367839297041876439\
27967018127941479215671700039852854875540666910279582375482429010189479965\
4347610548646590095726447692842486435777779633532452287080336236838770420,\
 0.00057247500159347076727208597065781805934969620278795143416312053706697\
75605113785927082604024225330925466027789098712218891156639605823200176287\
72520516314364379924538458637254415951830186447434728748503882261096810523\
79100], [0.000223088674184685059959750452716407504153221157074091091814390\
06633942658218908944754403303468391725557814385720916790580728615655682382\
42378217300551884420167271558723743413270721510315767012210170750748533916\
7355064635150, 0.001175067800881155550252895908438507334462582801718189259\
05798519130034790603907886318740571109260114400183838973402705480391086349\
21667728707153013823214640215318965339552639968389378125526613685384728667\
922341054339339450, 0.0028862295171558610539682481675441509555956975226439\
82090028368205724425088466259306639655861267879431081956746279500206234655\
97351140909994979461736166694010603318417461675071142037503367165418131434\
93535226572463177698620, 0.00535434875012223448674841643168430735892411108\
45491927312830611527973039312815021697140884685295916747661908029125612795\
30032221612336736134454071054593777167158320546416918692352002755002323067\
1583479143616254497111693450, 0.008575713630685464790855986145441763215652\
97059256008139623007101541207318526301264455880967294114830697102997507037\
53257671960643335859278829329646945224987824715868600994495063862436184438\
827161934883767045373133438992820, 0.0125454297071361033071773849654318618\
77520141981847761573149216272108977279944497007919800680360890125100418854\
99578378589457075908759209222982987845207587506369497610616551070451128294\
0732629301382128833501894382036506600, 0.017257455478100374273863422164272\
50074854825707569434776335525431770529297633848500588158251516910741000417\
15659414871627790492624989372650565550922821056895289770247445733494637977\
84182092522782851056892431507168845574390, 0.02270461682818254725325924148\
94852456081765926479503997403294058583291758106378932324054544257392992858\
04431685758259809203454116340250012442704683706252164504670830940805596733\
890865126359026101435698520913149280707992430, 0.0288786193450636626238669\
97749999132464583016879010318257980386729127509114018121484518816210403256\
77492422446733234448990603266886920687278587265540678322421674166531494994\
9657310126881641542169554731820721975966791306800, 0.035770061413777102023\
47702046227343310388250558020899397259849524843069741350680804986784462534\
22319590194813566736782382244917956166222461707138239286777756932336490141\
65000941960508064455689099595392424538833230446500070, 0.04336844871412117\
29176331719245262609442737658446126328429481976934400392487749653743372438\
02034297250906780124076922605628650779508251899245335311115574243495585230\
172561771600335247800768734959779854765966811278176419200, 0.0516622102806\
14658402837964016302006846350148974006507401970217817830475189861180088783\
93380383931831453262167985385435336406639386341103682566281625361680377376\
9253777437633579531775026844810970439727914953390217400314240, 0.060638716\
16089308564811332818043779603207411166161686630368836087878439731342848177\
29395010968898372917359988825382867726009568346875480492513071710682657685\
66686588880285223236717614961308372777804604753751873217111554150, 0.07028\
42966684444515114039382541717535800342551926778722772185117336742669543020\
99679924728764068441579959838869153658281829476465106784955367755574892513\
207029518053959008189874248417536394407973470856557878796679825749050, 0.0\
80584263209872362191688478048566467603188293694096142521647737198638365946\
27759727349229724380890256540213331129267844244327317259921037254002663011\
3546288625375750332832091506930377199528713507774789114358620955395611930,\
 0.09152293065926826481443750299385214612857939966426687534727660544601861\
28272944345744775587192612676983977445409767938379990285726252702405759038\
76732349761712648523877741127396946850912700113915744932277009784641633101\
900, 0.1030836412476972750256803441307728206946981246236163062009404877932\
43043163169994648990482623429370069404652057519650089371486346274063282666\
03585182785079092014365391948412312052506361596825941483648104034694634964\
465820, 0.1152487899324793130671919656254869580072970273323580307263619741\
13231853851840864639918721842979192610789841103443723901236121887462582276\
52386375952191077351805799168964413850358093594208713724316063931812293077\
435040570, 0.1279998512082013638417297360345431630956604576655408926555256\
47904739864000906038875070337712857278955583447740230011710868385054171209\
21789481829875228117875887334307990152065959572061033961689515282835978227\
461536922860, 0.1413174073189500598729658708530923607165847235370944504646\
50837135380792867416142699247153444946655924563215354429116655345864521993\
42052139903825046160631982575227203992167591695410194139612389929634271465\
433363086890480, 0.1551811778289861996143961937805323669551385272648935973\
31217452938171125116422326829865899664423435447483677304556483441355983820\
36246089402795428482919979308867369667510667090467084647956806535630845567\
531732984678940130, 0.1695700505069400991320164385778413825974699406422404\
02562486615029471780228420612933008558718422568335481865238221773536730182\
71284761033827370622743853429860137276245688727333028247779589400123512923\
589040070553754177980, 0.1844621134765640168760358063553315684554682649633\
30405530819505800758815428688760099260538509052640698342966160513538736478\
65834153818792841525070477356519156369692556407609300480379506305609536761\
567336294803801949684870, 0.1998346885851241284226268504179965757849724525\
93106629157173985840727002014631860818946578332865243755099483657914461135\
60239993763199502449686748919781679906053123999821348244748231019626089194\
528276176118493230887232810, 0.2156643659386451076372571066875864206290032\
13170193569271309527911658724438121710167589972721675269958743845966788829\
01378347006466383593143216416432938834330522690151183050544738495088505939\
530455164025995028857543757620, 0.2319270395514340339900713734372995475441\
53437460421538636627358749324531798299443536468484546742595722960002053451\
83845427395190305000271276399920791526212465470779237948004928186554707107\
647868769075141666356712283286030, 0.2485979440556075062031636248162159982\
17529663750200920058664791313665714991278545259488852649082340239125702616\
52346903549487575068467787200508638123761033701748706414949421444484715511\
293435871816781941687747142134995190, 0.2656516924147277614819608175320956\
71353021996755866598841291216069081440660186867223008108871471801058432135\
96970161670712555939821281848335565613308361593126533034603104433100179143\
505373297766771789728417778392987575380, 0.2830623145841219534688066498184\
09020848945847827402627331745935647057444731779906256191148733297655653355\
08136153328440913708660161920750669317495764939952255699061724444448569628\
351043898278565523486917842857966767879070, 0.3008032970590153864878101787\
41233121441375437643213670625651155911462202746270384869811493317904116935\
32494420088771078020437913304275227303282167721674566839360566490940669412\
584914502783580138892963454225104453615725400, 0.3188476232502563421904783\
56820518205991445187438193314087256152825591643488329499498485080399751775\
81587058366953536696041042061018376081268298188417641272726392334720360948\
723546724874158550503839330928783004690521951230, 0.3371678146261490426904\
43528186320652481507622445459801619341095097971378879512438140591873461949\
48952412423882434615988142386018303962185784715291312111699779352792130658\
082744845294040054606787830468899819186341840846420, 0.3557359725577440734\
45430349282643050751797355493117031789476785350946426733756351465655717436\
41660565030457808533218736487692102000652577433225594246308686450526126547\
029524291509747864053656720391676482039552833657639350, 0.3745238208038639\
39753420591982497601318475536218100393467582316928098748669762020358571014\
43247927019564987949226493815232596908615170069675661370165469341346065994\
452316672560354308934241175896983139218213339902553314050, 0.3935027485711\
66933713805730666839088452605624582613148925701647093116776084977228344873\
88731098648631575529957630014790524316024692345278954549419822059019685900\
005680488783021067002217716343530689015105976873119820718700, 0.4126438540\
83676593720330475994356902140594612511468710247995440241814541245218506786\
64318207989607566432508558853818854241894549869836659146378734102556735864\
902522571347264900768023241828001787161532116343080875201802080, 0.4319179\
88595428056720379460999641466033431875182558444735485293963368638862655402\
55131173561208631136480465685420228793438294056447933297995762466527601614\
786565453601269807043184065091160712790337729329544875307700865030, 0.4512\
95800779207700468360774947531548991466907781616953704576182384666221074750\
01322986725213723351788959220226305451727755511960945124151547908144299651\
732407261042717904570444545374678624950400109827579498682930531275950, 0.4\
70747781423789665685503339058291102787300662649042800895602230181104795836\
47368751249143828179601102950725763895242986301846516297864712576554537275\
743846995037359085910542033574047661562654222331441521585252295433069650, \
0.490244308371603001172824382946272726033224349176501092432714776209057813\
87730350006799243660741331099928591757363563564036555753356881394709426170\
46298057273272085352313036623876731827800026233181617540962956902215752887\
0, 0.509755691628396998827175617053727273966775650823498907567285223790942\
18612269649993200756339258668900071408242636436435963444246643118605290573\
82953701942726727914647686963376123268172199973766818382459037043097784247\
1120, 0.529252218576210334314496660941708897212699337350957199104397769818\
89520416352631248750856171820398897049274236104757013698153483702135287423\
44546272425615300496264091408945796642595233843734577766855847841474770456\
6930340, 0.548704199220792299531639225052468451008533092218383046295423817\
61533377892524998677013274786276648211040779773694548272244488039054875848\
45209185570034826759273895728209542955545462532137504959989017242050131706\
9468724040, 0.568082011404571943279620539000358533966568124817441555264514\
70603663136113734459744868826438791368863519534314579771206561705943552066\
70200423753347239838521343454639873019295681593490883928720966227067045512\
4692299134960, 0.587356145916323406279669524005643097859405387488531289752\
00455975818545875478149321335681792010392433567491441146181145758105450130\
16334085362126589744326413509747742865273509923197675817199821283846788365\
6919124798197920, 0.606497251428833066286194269333160911547394375417386851\
07429835290688322391502277165512611268901351368424470042369985209475683975\
30765472104545058017794098031409999431951121697893299778228365646931098489\
4023126880179281290, 0.625476179196136060246579408017502398681524463781899\
60653241768307190125133023797964142898556752072980435012050773506184767403\
09138482993032433862983453065865393400554768332743964569106575882410301686\
0781786660097446685940, 0.644264027442255926554569650717356949248202644506\
88296821052321464905357326624364853434428256358339434969542191466781263512\
30789799934742256677440575369131354947387345297047570849025213594634327960\
8323517960447166342360640, 0.662832185373850957309556471813679347518492377\
55454019838065890490202862112048756185940812653805051047587576117565384011\
85761398169603781421528470868788830022064720786934191725515470595994539321\
2169531100180813658159153580, 0.681152376749743657809521643179481794008554\
81256180668591274384717440835651167050050151491960024822418412941633046463\
30395895793898162391873170181158235872727360766527963905127645327512584144\
9496160669071216995309478048770, 0.699196702940984613512189821258766878558\
62456235678632937434884408853779725372961513018850668209588306467505579911\
22892197956208669572477269671783227832543316063943350905933058741508549721\
6419861107036545774895546384274600, 0.716937685415878046531193350181590979\
15105415217259737266825406435294255526822009374380885126670234434664491863\
84667155908629133983807924933068250423506004774430093827555555143037164895\
6101721434476513082157142033232120920, 0.734348307585272238518039182467904\
32864697800324413340115870878393091855933981313277699189112852819894156786\
40302983832928744406017871815166443438669163840687346696539689556689982085\
6494626702233228210271582221607012424610, 0.751402055944392493796836375183\
78400178247033624979907994133520868633428500872145474051114735091765976087\
42973834765309645051242493153221279949136187623896629825129358505057855551\
5284488706564128183218058312252857865004810, 0.768072960448565966009928626\
56270045245584656253957846136337264125067546820170055646353151545325740427\
70399979465481615457260480969499972872360007920847378753452922076205199507\
1813445292892352131230924858333643287716713960, 0.784335634061354892362742\
89331241357937099678682980643072869047208834127556187828983241002727832473\
00412561540332111709862165299353361640685678358356706116566947730984881694\
9455261504911494060469544835974004971142456242370, 0.800165311414875871577\
37314958200342421502754740689337084282601415927299798536813918105342166713\
47562449005163420855388643976000623680049755031325108021832009394687600017\
8651755251768980373910805471723823881506769112767180, 0.815537886523435983\
12396419364466843154453173503666959446918049419924118457131123990073946149\
09473593016570338394864612635213416584618120715847492952264348084363030744\
3592390699519620493694390463238432663705196198050315120, 0.830429949493059\
90086798356142215861740253005935775959743751338497052821977157938706699144\
12815774316645181347617782264632698172871523896617262937725614657013986272\
3754311272666971752220410599876487076410959929446245822010, 0.844818822171\
01380038560380621946763304486147273510640266878254706182887488357767317013\
41003355765645525163226954435165586440161796375391059720457151708002069113\
2630332489332909532915352043193464369154432468267015321059860, 0.858682592\
68104994012703412914690763928341527646290554953534916286461920713258385730\
07528465550533440754367846455708833446541354780065794786009617495383936801\
7424772796007832408304589805860387610070365728534566636913109510, 0.872000\
14879179863615827026396545683690433954233445910734447435209526013599909396\
11249296622871427210444165522597699882891316149458287907821051817012477188\
2124112665692009847934040427938966038310484717164021772538463077130, 0.884\
75121006752068693280803437451304199270297266764196927363802588676814614815\
91353600812781570208073892101588965562760987638781125374177234761362404780\
8922648194200831035586149641906405791286275683936068187706922564959420, 0.\
89691635875230272497431965586922717930530187537638369379905951220675695683\
68300053510095173765706299305953479424803499106285136537259367173339641481\
7214920907985634608051587687947493638403174058516351895965305365035534180,\
 0.90847706934073173518556249700614785387142060033573312465272339455398138\
71727055654255224412807387323016022554590232061620009714273747297594240961\
23267650238287351476122258872603053149087299886084255067722990215358366898\
10, 0.91941573679012763780831152195143353239681170630590385747835226280136\
16340537224027265077027561910974345978666887073215575567268274007896274599\
73369886453711374624249667167908493069622800471286492225210885641379044604\
38800, 0.92971570333155554848859606174582824641996574480732212772278148826\
63257330456979003200752712359315584200401611308463417181705235348932150446\
32244425107486792970481946040991810125751582463605592026529143442121203320\
17425090, 0.93936128383910691435188667181956220396792588833838313369631163\
91212156026865715182270604989031101627082640011174617132273990431653124519\
50748692828931734231433313411119714776763282385038691627222195395246248126\
78288844580, 0.94833778971938534159716203598369799315364985102599349259802\
97821821695248101388199112160661961606816854673783201461456466359336061365\
88963174337183746383196226230746222562366420468224973155189029560272085046\
60978259968570, 0.95663155128587882708236682807547373905572623415538736715\
70518023065599607512250346256627561979657027490932198759230773943713492204\
91748100754664688884425756504414769827438228399664752199231265040220145234\
03318872182358080, 0.96422993858622289797652297953772656689611749441979100\
60274015047515693025864931919501321553746577680409805186433263217617755082\
04383377753829286176071322224306766350985834999058039491935544310900404607\
57546116676955349990, 0.97112138065493633737613300225000086753541698312098\
96817420196132708724908859818785154811837895967432250757755326676555100939\
67331130793127214127344593216775783258334685050050342689873118358457830445\
26817927802403320869320, 0.97729538317181745274674075851051475439182340735\
20496002596705941416708241893621067675945455742607007141955683142417401907\
96545883659749987557295316293747835495329169059194403266109134873640973898\
56430147908685071929200750, 0.98274254452189962572613657783572749925145174\
29243056522366447456822947070236615149941184174848308925899958284340585128\
37220950737501062734943444907717894310471022975255426650536202215817907477\
21714894310756849283115442560, 0.98745457029286389669282261503456813812247\
98580181522384268507837278910227200555029920801993196391098748995811450042\
16214105429240912407907770170121547924124936305023893834489295488717059267\
37069861787116649810561796349340, 0.99142428636931453520914401385455823678\
43470294074399186037699289845879268147369873554411903270588516930289700249\
29624674232803935666414072117067035305477501217528413139900550493613756381\
55611728380651162329546268665610070, 0.99464565124987776551325158356831569\
26410758889154508072687169388472026960687184978302859115314704083252338091\
97087438720469967778387663263865545928945406222832841679453583081307647997\
24499767693284165208563837455028883060, 0.99711377048284413894603175183245\
58490444043024773560179099716317942755749115337406933603441387321205689180\
43253720499793765344026488590900050205382638333059893966815825383249288579\
62496632834581868565064647734275368223010, 0.99882493219911884444974710409\
15614926655374171982818107409420148086996520939609211368125942889073988559\
98161610265972945196089136507833227129284698617678535978468103466044736003\
16106218744733863146152713320776589456606600, 0.99977691132581531494004024\
95472835924958467788429259089081856099336605734178109105524559669653160827\
44421856142790832094192713843443176175762178269944811557983272844127625658\
67292784896842329877898292492514660832644935360]]):
numeric::_gldata160:=hold([[0.00014402926426054152232724969892699182341711\
61474749917907106524158541757788881977789600690354747134755230562415371078\
37161563767869053298613234110990426063810553556085882220624484327729667512\
14037293963289265215493739729770, 0.00033521916007615164073765313937787754\
94007066687434425232876645133009397459767426237878629659366055397621835786\
90524987458507527251755691455133960998727724135542260352713160511490692307\
10031513047598311030323323140314528550, 0.00052656383610686774411545949283\
26056932563394756504221807898257751381192064990298157463521380376901154161\
13486521149284015531131144272375895843676573535634654451337265667338222789\
73341936426534682623340265263055317285486270, 0.00071773137674159643149088\
71409390517092000606112676587834159713801214028451142911570285774836714157\
34376093275841292187632972068944677318656200458658563322693011827434926715\
09636664162176989746355383948855533143612235547090, 0.00090862887983259200\
45444007863047753695915237007754366566495421033059267416810356720969486911\
85500381017241715182161587952417453802638425167971547990570392981915153715\
17451763624495267057619136582519048753642855999781821320, 0.00109917974839\
82443724280734934424068467685387559178998352975133346647786852650971160139\
75007920613932120306553132109694990160416451289454292880376644152694851528\
3685219785202786777010008736583791310592783297302705781512510, 0.001289310\
06091876596134288278198892961335423905724679555123802307426726543111201749\
97132890277158387782714760738336664310087878952569508531655385136541959728\
762763105664117741321881673621710744180204378096398514967702410780, 0.0014\
78946662519708287468548184908943219207674063168355810894405097617343051156\
15355094614097920212212344397883446969676938312675085281603594133658474285\
93939448842595705757272084665599399034275976192400039426739101885664600, 0\
.0016680167739104331070009571736751071395367004894680827070670731046481183\
18015934457665914123463449115224645227742430049860632311131314600155889909\
87253572105633344085158900146925526395706281613365704852722074721846201916\
90, 0.00185644790129375785237689860342533353370320076179566762788349241983\
78413430818690978253453755809933340776924671266305047388381434456410292607\
14447802571154182374146114307857354141045574931228310928588064728104910175\
5567400, 0.002044167824283986338906729914484046011777578945941171086275758\
32392003999309738166163649181922038998667192666809792577376616682913550294\
29978222782641453209277243544832254798471454554992721842950467609728061197\
212310984940, 0.0022311046081674869763474925619427618916167744921433013230\
43738784903773147901981976995451778833027357750633828440135568646547700344\
93535384104501435525766352725661675051823717022263713052425692636099665170\
97244499272510860, 0.00241718662485138690858234658787889260486365526474024\
23849547413695498019094468813867616762156321902361606847915342528454691465\
62489207260114040075837953815841500785007606300652609046545080855030429103\
0548910619308689528900, 0.002602342577223633665139696795011581088923790053\
38914628485863942092687592899741148459622829272312758867413711503429763248\
14139641466130750157437318095748077070675385497832838749141418713482193477\
192554718627012575189158900, 0.0027865015249304524467569633192597322608816\
14018047653739721859620165356938689832205469195513524309577337586628931822\
11911367654195430168908484849295648501772131435686117441908804074377950297\
52642591596063856351970994940220, 0.00296959291074293877157003499008794117\
94924201692150533016255769333553935495953244693188965761067317689457567105\
25620335285545838980938236844830938364171594064726869094702575303139480398\
5641958500556427418764223954754640690, 0.003151546587139065862481432159829\
82768162569950774371307508285648118857846004719529547480592292602376341015\
73387303905851860806694748760407274751400985509150065456635637878021262233\
584959714427223711885106464321476545595810, 0.0033322928429189060687643396\
58385913441084763324498579837893414843899819349766803092470287424294528769\
81401562554848310759706613901526962012058337290246435129896496352177189326\
96715749341748104069756972746337504238223959550, 0.00351176242975702245955\
77869505735074298714782878714769020694990972237767065200264271396100065912\
75075587206872542503834966469823266055218002567556411945552448581630045657\
8563592100530636671637345365516553955859222931098070, 0.003689886588636988\
75997205085806931209484611479232732855185153022527842212405816594484925117\
58615936604016669049036451513840256502515478655549695239915338160726797435\
409965459555716381786487459475916293718484742838016054000, 0.0038665970761\
33493287710928674908786307278336638138194688995437969984944519479371007323\
31523602210755082674584220865044637371845214392344994532618203809425027900\
93963117737785534821760384701821222669036667493523935205242820, 0.00404182\
61905181899320980340775409047859034067342841346059137581833025194709023058\
76692543820014221260398028532165333740707760787846265388384041590886918572\
1011403461586663019852343322596133460884022332084858636816732857670, 0.004\
21550679767128241890081985984392606360833643792962312743895905947883814909\
53739226154016110771712996803634602767873765378580153498727616162484974826\
223642039669690961546176771341484498775281319534700854765500166935101760, \
0.004387572356784118549801234495919526734841078234754666522916232652077892\
87990617274599638671005068141349432505159331202047868066227705581370587277\
34357074996252864175094566334775702235527466686533822382018432561357170936\
520, 0.0045579569458400024520198080390797889708988471571329556741347738593\
84437103125671076915682455938341773605551068856305806214635082048090592789\
62193169510432885489813720037887298192236336501114441003421190657772178756\
00052090, 0.00472659528686161250305285450723503712762241508147825423556935\
21980276098558374636265204870649498234252532923173587914266140524293119761\
37473404038486672575797008565140166635704010707887752545926085169251980833\
8565272741900, 0.004893422770914165217830705680360456960802405790244986873\
63253213941505041513204536599364804887007588554735332925594984886777656906\
63887705566116771994630287741837779802980932553743273292881939352037035040\
412411867445963410, 0.0050583754828539697643184522790288370885316592194284\
25644413948449724295821343150999721469675684669524934080906007968568821838\
99168225460256589276977671483949340720765376342327666883705650083575354687\
41709957317095256185570, 0.00522139022581237498055546325991589874051687154\
02037829374426676510901870118978514843729317603652931651582254318600616279\
73117364118626120979094678342652034185426111291986368111779060361811774205\
7008515953692370036153205250, 0.005382404545405379350174449068752536166158\
02066484473540565680939235608607309995676357090512252184493114326509566158\
93354621803041203150603150316189354047598213270276479550633507273352952677\
610019360908466685021535168672810, 0.0055413567536593890302637007116750630\
48964600046928289736639090853847376360584099967828236810593887758583803130\
65997618899403994769435519342615607838969078439437672523046823635789222555\
42139517843293486638837258813095032280, 0.00569818595264379067181652428435\
30613738974109418651529406462016529183338514307909781599749443499038769561\
69464094977742263073732683100864247357122520000882519627470800409433889238\
0419090016648080068930356145347239812575370, 0.005852832057801167330751281\
84720944099901373821658645116091968141201123741676900366317954070213158024\
92865224352470971351851509744029474421689827827615941965612497262534268124\
085117390144563600522227129078281545828143571150, 0.0060052358209661353140\
10771499715966576710289342950695244141393260049563784858626186153258404313\
56919942690976786320265285693145321533237406618821589298557463316991566643\
10534312912150242586475403283869161298392036246758600, 0.00615533885306392\
24747074602053697296441141437734543581810709676499038640664433357922786327\
88300808367718232490316937193120308554998953056072356513376008971635212179\
8631424455501771042242524856744208857404174177637518281740, 0.006303083646\
47994757002645844285031610687591124204643071819618257264105009456209456379\
19634542289978304064419887162933377765837761955636472522716530597151186880\
737372899437869603066132361232552526854744930574661899120603650, 0.0064484\
13597091797980782288073696203142243875180004667030962023939320792314828909\
55067101865514108430487228121985388236079864860534204796401507255634766411\
94248458884916080574715966451776376601574933177102243884376731001300, 0.00\
65912730259551407795190894851185653858893633405279251953467704733942755142\
27084300817385904038516578011719231911041469737199314507408812713388469997\
7307491735301336967466216191789251723878650520265480443893300633730639630,\
 0.00673160720063524076302394915861230174524854000008541173554721181370966\
37630438002499833606222182701296966338794307108012475853404516845688748919\
73888264390642589083966043931200915499565892551337421261659726667606033542\
8760, 0.006869362356175899257793920441513844186932550124970193058999288547\
09292574939632880523947016663892484519571375507952676340267454147615965685\
41712892575933597816983820144938639977431242839558308008880551665899070812\
748176980, 0.0070044857156977696756800274522584672443609176801719055785891\
20711534609524724336986223159247302992353381393986941189620338666248648178\
09116080612261921455553112622705525028120831643380749081797946787419428650\
98366939883470, 0.00713692551061815021189531356677880464007677642305300630\
19525964960301714782835103919443626466379723308847643704876138194884675096\
05677949372456181495239072119067171762723274541051592907614442857548846701\
6911076988611858070, 0.007266631000484500912012861423544440438418085349792\
82159710102351623396786126017352830326146462806006416042330484613615576101\
59879160868547647633984379821497640002289083930286384343415907044936853298\
766745445307178326756510, 0.0073935524924140816945605421883488966385976578\
60041161127881363271919569019193920168187184203023901186635760056451046756\
11946906334665700362243379068729746517584140332859587133838418669238041931\
41148562233434294464633517440, 0.00751764136013225986044290567996023462347\
17101392247774833304854323004715302654555178424028348138465063337508542549\
07973042557125111832710860034826247321276531896675026418396162598386492976\
7231479143777306752216294689641100, 0.007638850062602190175537166614694883\
50978045618277338636341558830934656829365953763361374918070874529645157917\
85499476631074680700649841827194387864871634259722060182634818272558063017\
002135920691380152581411347716295424950, 0.0077571321622387277804429046376\
59831295568936632973336568962440932225392798806495936569674104795834466674\
66015149981357006183815347505200796090325687383339263198460968540035893109\
62000384071959878487997022275580148729251300, 0.00787244234269959394624815\
65461465075111968864020193995516231188912522550270820773449985742162260048\
27088382366902408371923267026136244544216553698105108902731123311841534370\
1163448005741926285279958962386777459592347631680, 0.007984736426246977029\
33139250144504890390038283893452552004165705648107832953132062513477610914\
67502519011632624926029053711442442138441596653362755150774969884808559206\
252249612817166449365463329223924915075428985381441920, 0.0080939713906729\
15844196689098314109421452518652798648763319644867512825427839441797637990\
83519347698949896115076843067741283075156817257761380540436310619340665845\
10912698732055986036427237511375029229119300706817732786360, 0.00820010538\
57819800265155632916292099786813833751212569934123552954569725285040327228\
42174054086317709836868520672098919546595431357465510865397021500911622230\
7826188630382514103933676106464574977665319954446020201699575050, 0.008303\
09774942493174878953561081668919862288560804575756315326805608648014456613\
42655298612484393008223586994020946407513590890795643764670342274516338575\
146215203038414407578924140838386959155627927325639582445816145457030, 0.0\
08402909023077225323911233142497730848502556261550584954854433400441102868\
56727557200718816199820947292266137121493333731833235893897156715524207512\
98618005199427835114273218068495662152519929041258846655583095478006703920\
, 0.0084995009669563757295215701062234359259707389708969873834381073678004\
52738757602391931216634290122332693405483180402383207959008032386427851533\
66129865798136201291619794458434041473583103846913038709445458333469845353\
70370, 0.00859283657467240384779510125483518050472654074446472819737496132\
52754864571234784442881499660425437419151806054285538936321581681909037963\
25809750072613559534816335115761362564474338910999439437988250134823697842\
2493107210, 0.008682880087405745178206698563824521663669405750293635728680\
52130566236955468631959736869983621701600511930710100271105997019412023435\
13327974317378620569449009561680535009818386540598448535794158602653580420\
768248173821630, 0.0087695970076071898800922910101675203836019161917291376\
03196059740013180591403416205518902779909342167669976698462999268011590322\
03362844859100863340379924179987766291357628287997811458528427064609176532\
31037716585550299840, 0.00885295411221460517092771568711261379614291046445\
93502826227245183949349472480224707109889783049524485056601732068725586296\
57914348881437944546354166591790369406078892535722078196227632010093713954\
3101842548821709745648450, 0.008932919465381376277304380789836884168176139\
87653428404262522772084600578211839104363982191514378933630595508657690890\
83034565270205668627823832312100023623167404159793189597546743737484253927\
481735402818656933132066594630, 0.0090094624307116892394190733138503639652\
58460319854097790731921729946008383321352492829007365707014387151018232414\
14739490147731757982929175626979109408590218810017027238600706405244920195\
63079106498532855434391643860965690, 0.00908255368299796783624351392999110\
79721111775656643169141420279495291985829683395097001128699988521093937243\
65826505247273640571708509235179768312846232643777795012282222977456333728\
9171418618961643356886694143541459716230, 0.009152165219455967656118160101\
33994587691706573492211120647097079444113003212407884048495153511349702737\
19974601820670647369879092089873110640617425327866301676730910104459082319\
365062688433509474843156909224591275023109020, 0.0092182703704532228141317\
30904057304539706680460480228759611571025182107729617567564843423332891609\
96374000064109883141478289208457796613146897110738765291517525914136344682\
31222121270338102846976118209512991377487475553740, 0.00928084380972673494\
02735899095035886683812781963797560992349649513292176608999362682103881357\
10151632064868815958423919994235184312849176354296492770494427699704205335\
5508483728861424038312725937580438700798476561550745530, 0.009339861564085\
98975718013682512020591559625517042852189436260419345485821208894349636517\
10615690051234453796432999583282902937837568196289087636358005869960858480\
483704230957032016811594515119918646034543507399352179878350, 0.0093953010\
22597583758825102567434439503496952086309634063631719232840524733856584780\
17933987381977644331733702998252664399545483640602162657192655411123442183\
29472282956467014110033183712202962841726948936672209148528456480, 0.00944\
71409452479421165516660860868327950538328594647135596637281967036685643298\
57288425384882439073059384525047088947413903976206371827053220016791718040\
6036830725893293647989543825706355488507285373430348043244630168127070, 0.\
00949536147108080890062106167733162215127968025141490635934055582910722378\
35494555447393059048554475609128078012984968674205553800058468704289394173\
82065769755786958653500089884701466018839761256392657160682265867118251926\
0, 0.009539944125806391937595286663551788093180912810435041543796716241662\
79534453024775376992093607501455344229828701146261135258998656010555771210\
87385044284575286256440141277712570179444949805232531789985951975386024456\
554460, 0.0095808718288792470494864978501270923291070178187150878749770971\
91690329086337361938568549426870056576239060508844977433368913544726700029\
92186413109425648062801001224633549660863701023876519236507239958643063467\
00053633940, 0.00961812890004218996230519003951315474367555408725860790772\
42135147669165778378970175866400407079700734254072141532607359040470746533\
09306317813242142784654870740388624644326061549712315438047131136259181131\
2738187084661080, 0.009651701065333728751579059331162038092790700877743070\
81979036055875994329301676187829164336449649123580225439809172202779379179\
60729155876453727362911803001347084474400147634840016189718841317206505234\
189697707148429859220, 0.0096815754625567152323290478613288221683258217838\
90722591390487485080171427197694272384859924067579560610564181659280120120\
91533986731737862534212447260478834563958627438186305636894688704249792227\
55404359697341618366517790, 0.00970774064620612012222609382880299167036739\
79059530939234099131363304753584147510913220036244222845735782670326327037\
58887661794992441649507653396164042129554672690107860996606575878250564906\
2505556094005865508661273486710, 0.009730186591854044030206133387232459041\
49091468758268127357301404032313743891922327176030525676546052433589517596\
13304241374860894797710137601685779598489094602058726830932141847675670880\
678236463591879833863454915649104190, 0.0097489046999902842693665880529810\
38360701337961895482657630082744139927178295906828027276609167324282231048\
88000316751033394438167455498262237962382431438459671494363496346833566967\
56739929569727738713577008930607569791940, 0.00976388779931698608289247114\
99775147983706975987070688682492913163164941666070303670213057254747036899\
89622344731978675511555404857930274825922257590790206771250431284305696109\
5826235253647819674627248755942559815290916090, 0.009775130149496116025197\
24236119464247167515624477053442279641736272216937295336684180989661641792\
82097106512915904041110523607500977049315020370696629080812469289847576195\
963545493415846591024868299529656927129103762673640, 0.0097826274433487048\
77323062342778042366091701723061179079703085802586740389903201101517974074\
70417867233976217643056573873247868622548715398563521177989077792794669840\
06013596027684017899098394781515631336998181022895144050, 0.00978637680850\
50175156471642664942644430308101549046961109994964393743743087346895655484\
17510465488746992738009920781833366492682912865212385925261069250439666669\
2382050622097712319966211449879019022920530551451448201254840, 0.009786376\
80850501751564716426649426444303081015490469611099949643937437430873468956\
55484175104654887469927380099207818333664926829128652123859252610692504396\
666692382050622097712319966211449879019022920530551451448201254840, 0.0097\
82627443348704877323062342778042366091701723061179079703085802586740389903\
20110151797407470417867233976217643056573873247868622548715398563521177989\
07779279466984006013596027684017899098394781515631336998181022895144050, 0\
.0097751301494961160251972423611946424716751562447705344227964173627221693\
72953366841809896616417928209710651291590404111052360750097704931502037069\
66290808124692898475761959635454934158465910248682995296569271291037626736\
40, 0.00976388779931698608289247114997751479837069759870706886824929131631\
64941666070303670213057254747036899896223447319786755115554048579302748259\
22257590790206771250431284305696109582623525364781967462724875594255981529\
0916090, 0.009748904699990284269366588052981038360701337961895482657630082\
74413992717829590682802727660916732428223104888000316751033394438167455498\
26223796238243143845967149436349634683356696756739929569727738713577008930\
607569791940, 0.0097301865918540440302061333872324590414909146875826812735\
73014040323137438919223271760305256765460524335895175961330424137486089479\
77101376016857795984890946020587268309321418476756708806782364635918798338\
63454915649104190, 0.00970774064620612012222609382880299167036739790595309\
39234099131363304753584147510913220036244222845735782670326327037588876617\
94992441649507653396164042129554672690107860996606575878250564906250555609\
4005865508661273486710, 0.009681575462556715232329047861328822168325821783\
89072259139048748508017142719769427238485992406757956061056418165928012012\
09153398673173786253421244726047883456395862743818630563689468870424979222\
755404359697341618366517790, 0.0096517010653337287515790593311620380927907\
00877743070819790360558759943293016761878291643364496491235802254398091722\
02779379179607291558764537273629118030013470844744001476348400161897188413\
17206505234189697707148429859220, 0.00961812890004218996230519003951315474\
36755540872586079077242135147669165778378970175866400407079700734254072141\
53260735904047074653309306317813242142784654870740388624644326061549712315\
4380471311362591811312738187084661080, 0.009580871828879247049486497850127\
09232910701781871508787497709719169032908633736193856854942687005657623906\
05088449774333689135447267000299218641310942564806280100122463354966086370\
102387651923650723995864306346700053633940, 0.0095399441258063919375952866\
63551788093180912810435041543796716241662795344530247753769920936075014553\
44229828701146261135258998656010555771210873850442845752862564401412777125\
70179444949805232531789985951975386024456554460, 0.00949536147108080890062\
10616773316221512796802514149063593405558291072237835494555447393059048554\
47560912807801298496867420555380005846870428939417382065769755786958653500\
0898847014660188397612563926571606822658671182519260, 0.009447140945247942\
11655166608608683279505383285946471355966372819670366856432985728842538488\
24390730593845250470889474139039762063718270532200167917180406036830725893\
293647989543825706355488507285373430348043244630168127070, 0.0093953010225\
97583758825102567434439503496952086309634063631719232840524733856584780179\
33987381977644331733702998252664399545483640602162657192655411123442183294\
72282956467014110033183712202962841726948936672209148528456480, 0.00933986\
15640859897571801368251202059155962551704285218943626041934548582120889434\
96365171061569005123445379643299958328290293783756819628908763635800586996\
0858480483704230957032016811594515119918646034543507399352179878350, 0.009\
28084380972673494027358990950358866838127819637975609923496495132921766089\
99362682103881357101516320648688159584239199942351843128491763542964927704\
944276997042053355508483728861424038312725937580438700798476561550745530, \
0.009218270370453222814131730904057304539706680460480228759611571025182107\
72961756756484342333289160996374000064109883141478289208457796613146897110\
73876529151752591413634468231222121270338102846976118209512991377487475553\
740, 0.0091521652194559676561181601013399458769170657349221112064709707944\
41130032124078840484951535113497027371997460182067064736987909208987311064\
06174253278663016767309101044590823193650626884335094748431569092245912750\
23109020, 0.00908255368299796783624351392999110797211117756566431691414202\
79495291985829683395097001128699988521093937243658265052472736405717085092\
35179768312846232643777795012282222977456333728917141861896164335688669414\
3541459716230, 0.009009462430711689239419073313850363965258460319854097790\
73192172994600838332135249282900736570701438715101823241414739490147731757\
98292917562697910940859021881001702723860070640524492019563079106498532855\
434391643860965690, 0.0089329194653813762773043807898368841681761398765342\
84042625227720846005782118391043639821915143789336305955086576908908303456\
52702056686278238323121000236231674041597931895975467437374842539274817354\
02818656933132066594630, 0.00885295411221460517092771568711261379614291046\
44593502826227245183949349472480224707109889783049524485056601732068725586\
29657914348881437944546354166591790369406078892535722078196227632010093713\
9543101842548821709745648450, 0.008769597007607189880092291010167520383601\
91619172913760319605974001318059140341620551890277990934216766997669846299\
92680115903220336284485910086334037992417998776629135762828799781145852842\
706460917653231037716585550299840, 0.0086828800874057451782066985638245216\
63669405750293635728680521305662369554686319597368699836217016005119307101\
00271105997019412023435133279743173786205694490095616805350098183865405984\
48535794158602653580420768248173821630, 0.00859283657467240384779510125483\
51805047265407444647281973749613252754864571234784442881499660425437419151\
80605428553893632158168190903796325809750072613559534816335115761362564474\
3389109994394379882501348236978422493107210, 0.008499500966956375729521570\
10622343592597073897089698738343810736780045273875760239193121663429012233\
26934054831804023832079590080323864278515336612986579813620129161979445843\
404147358310384691303870944545833346984535370370, 0.0084029090230772253239\
11233142497730848502556261550584954854433400441102868567275572007188161998\
20947292266137121493333731833235893897156715524207512986180051994278351142\
73218068495662152519929041258846655583095478006703920, 0.00830309774942493\
17487895356108166891986228856080457575631532680560864801445661342655298612\
48439300822358699402094640751359089079564376467034227451633857514621520303\
8414407578924140838386959155627927325639582445816145457030, 0.008200105385\
78198002651556329162920997868138337512125699341235529545697252850403272284\
21740540863177098368685206720989195465954313574655108653970215009116222307\
826188630382514103933676106464574977665319954446020201699575050, 0.0080939\
71390672915844196689098314109421452518652798648763319644867512825427839441\
79763799083519347698949896115076843067741283075156817257761380540436310619\
34066584510912698732055986036427237511375029229119300706817732786360, 0.00\
79847364262469770293313925014450489039003828389345255200416570564810783295\
31320625134776109146750251901163262492602905371144244213844159665336275515\
0774969884808559206252249612817166449365463329223924915075428985381441920,\
 0.00787244234269959394624815654614650751119688640201939955162311889125225\
50270820773449985742162260048270883823669024083719232670261362445442165536\
98105108902731123311841534370116344800574192628527995896238677745959234763\
1680, 0.007757132162238727780442904637659831295568936632973336568962440932\
22539279880649593656967410479583446667466015149981357006183815347505200796\
09032568738333926319846096854003589310962000384071959878487997022275580148\
729251300, 0.0076388500626021901755371666146948835097804561827733863634155\
88309346568293659537633613749180708745296451579178549947663107468070064984\
18271943878648716342597220601826348182725580630170021359206913801525814113\
47716295424950, 0.00751764136013225986044290567996023462347171013922477748\
33304854323004715302654555178424028348138465063337508542549079730425571251\
11832710860034826247321276531896675026418396162598386492976723147914377730\
6752216294689641100, 0.007393552492414081694560542188348896638597657860041\
16112788136327191956901919392016818718420302390118663576005645104675611946\
90633466570036224337906872974651758414033285958713383841866923804193141148\
562233434294464633517440, 0.0072666310004845009120128614235444404384180853\
49792821597101023516233967861260173528303261464628060064160423304846136155\
76101598791608685476476339843798214976400022890839302863843434159070449368\
53298766745445307178326756510, 0.00713692551061815021189531356677880464007\
67764230530063019525964960301714782835103919443626466379723308847643704876\
13819488467509605677949372456181495239072119067171762723274541051592907614\
4428575488467016911076988611858070, 0.007004485715697769675680027452258467\
24436091768017190557858912071153460952472433698622315924730299235338139398\
69411896203386662486481780911608061226192145555311262270552502812083164338\
074908179794678741942865098366939883470, 0.0068693623561758992577939204415\
13844186932550124970193058999288547092925749396328805239470166638924845195\
71375507952676340267454147615965685417128925759335978169838201449386399774\
31242839558308008880551665899070812748176980, 0.00673160720063524076302394\
91586123017452485400000854117355472118137096637630438002499833606222182701\
29696633879430710801247585340451684568874891973888264390642589083966043931\
2009154995658925513374212616597266676060335428760, 0.006591273025955140779\
51908948511856538588936334052792519534677047339427551422708430081738590403\
85165780117192319110414697371993145074088127133884699977307491735301336967\
466216191789251723878650520265480443893300633730639630, 0.0064484135970917\
97980782288073696203142243875180004667030962023939320792314828909550671018\
65514108430487228121985388236079864860534204796401507255634766411942484588\
84916080574715966451776376601574933177102243884376731001300, 0.00630308364\
64799475700264584428503161068759112420464307181961825726410500945620945637\
91963454228997830406441988716293337776583776195563647252271653059715118688\
0737372899437869603066132361232552526854744930574661899120603650, 0.006155\
33885306392247470746020536972964411414377345435818107096764990386406644333\
57922786327883008083677182324903169371931203085549989530560723565133760089\
716352121798631424455501771042242524856744208857404174177637518281740, 0.0\
06005235820966135314010771499715966576710289342950695244141393260049563784\
85862618615325840431356919942690976786320265285693145321533237406618821589\
29855746331699156664310534312912150242586475403283869161298392036246758600\
, 0.0058528320578011673307512818472094409990137382165864511609196814120112\
37416769003663179540702131580249286522435247097135185150974402947442168982\
78276159419656124972625342681240851173901445636005222271290782815458281435\
71150, 0.00569818595264379067181652428435306137389741094186515294064620165\
29183338514307909781599749443499038769561694640949777422630737326831008642\
47357122520000882519627470800409433889238041909001664808006893035614534723\
9812575370, 0.005541356753659389030263700711675063048964600046928289736639\
09085384737636058409996782823681059388775858380313065997618899403994769435\
51934261560783896907843943767252304682363578922255542139517843293486638837\
258813095032280, 0.0053824045454053793501744490687525361661580206648447354\
05656809392356086073099956763570905122521844931143265095661589335462180304\
12031506031503161893540475982132702764795506335072733529526776100193609084\
66685021535168672810, 0.00522139022581237498055546325991589874051687154020\
37829374426676510901870118978514843729317603652931651582254318600616279731\
17364118626120979094678342652034185426111291986368111779060361811774205700\
8515953692370036153205250, 0.005058375482853969764318452279028837088531659\
21942842564441394844972429582134315099972146967568466952493408090600796856\
88218389916822546025658927697767148394934072076537634232766688370565008357\
535468741709957317095256185570, 0.0048934227709141652178307056803604569608\
02405790244986873632532139415050415132045365993648048870075885547353329255\
94984886777656906638877055661167719946302877418377798029809325537432732928\
81939352037035040412411867445963410, 0.00472659528686161250305285450723503\
71276224150814782542355693521980276098558374636265204870649498234252532923\
17358791426614052429311976137473404038486672575797008565140166635704010707\
8877525459260851692519808338565272741900, 0.004557956945840002452019808039\
07978897089884715713295567413477385938443710312567107691568245593834177360\
55510688563058062146350820480905927896219316951043288548981372003788729819\
223633650111444100342119065777217875600052090, 0.0043875723567841185498012\
34495919526734841078234754666522916232652077892879906172745996386710050681\
41349432505159331202047868066227705581370587277343570749962528641750945663\
34775702235527466686533822382018432561357170936520, 0.00421550679767128241\
89008198598439260636083364379296231274389590594788381490953739226154016110\
77171299680363460276787376537858015349872761616248497482622364203966969096\
1546176771341484498775281319534700854765500166935101760, 0.004041826190518\
18993209803407754090478590340673428413460591375818330251947090230587669254\
38200142212603980285321653337407077607878462653883840415908869185721011403\
461586663019852343322596133460884022332084858636816732857670, 0.0038665970\
76133493287710928674908786307278336638138194688995437969984944519479371007\
32331523602210755082674584220865044637371845214392344994532618203809425027\
90093963117737785534821760384701821222669036667493523935205242820, 0.00368\
98865886369887599720508580693120948461147923273285518515302252784221240581\
65944849251175861593660401666904903645151384025650251547865554969523991533\
8160726797435409965459555716381786487459475916293718484742838016054000, 0.\
00351176242975702245955778695057350742987147828787147690206949909722377670\
65200264271396100065912750755872068725425038349664698232660552180025675564\
11945552448581630045657856359210053063667163734536551655395585922293109807\
0, 0.003332292842918906068764339658385913441084763324498579837893414843899\
81934976680309247028742429452876981401562554848310759706613901526962012058\
33729024643512989649635217718932696715749341748104069756972746337504238223\
959550, 0.0031515465871390658624814321598298276816256995077437130750828564\
81188578460047195295474805922926023763410157338730390585186080669474876040\
72747514009855091500654566356378780212622335849597144272237118851064643214\
76545595810, 0.00296959291074293877157003499008794117949242016921505330162\
55769333553935495953244693188965761067317689457567105256203352855458389809\
38236844830938364171594064726869094702575303139480398564195850055642741876\
4223954754640690, 0.002786501524930452446756963319259732260881614018047653\
73972185962016535693868983220546919551352430957733758662893182211911367654\
19543016890848484929564850177213143568611744190880407437795029752642591596\
063856351970994940220, 0.0026023425772236336651396967950115810889237900533\
89146284858639420926875928997411484596228292723127588674137115034297632481\
41396414661307501574373180957480770706753854978328387491414187134821934771\
92554718627012575189158900, 0.00241718662485138690858234658787889260486365\
52647402423849547413695498019094468813867616762156321902361606847915342528\
45469146562489207260114040075837953815841500785007606300652609046545080855\
0304291030548910619308689528900, 0.002231104608167486976347492561942761891\
61677449214330132304373878490377314790198197699545177883302735775063382844\
01355686465477003449353538410450143552576635272566167505182371702226371305\
242569263609966517097244499272510860, 0.0020441678242839863389067299144840\
46011777578945941171086275758323920039993097381661636491819220389986671926\
66809792577376616682913550294299782227826414532092772435448322547984714545\
54992721842950467609728061197212310984940, 0.00185644790129375785237689860\
34253335337032007617956676278834924198378413430818690978253453755809933340\
77692467126630504738838143445641029260714447802571154182374146114307857354\
1410455749312283109285880647281049101755567400, 0.001668016773910433107000\
95717367510713953670048946808270706707310464811831801593445766591412346344\
91152246452277424300498606323111313146001558899098725357210563334408515890\
014692552639570628161336570485272207472184620191690, 0.0014789466625197082\
87468548184908943219207674063168355810894405097617343051156153550946140979\
20212212344397883446969676938312675085281603594133658474285939394488425957\
05757272084665599399034275976192400039426739101885664600, 0.00128931006091\
87659613428827819889296133542390572467955512380230742672654311120174997132\
89027715838778271476073833666431008787895256950853165538513654195972876276\
3105664117741321881673621710744180204378096398514967702410780, 0.001099179\
74839824437242807349344240684676853875591789983529751333466477868526509711\
60139750079206139321203065531321096949901604164512894542928803766441526948\
515283685219785202786777010008736583791310592783297302705781512510, 0.0009\
08628879832592004544400786304775369591523700775436656649542103305926741681\
03567209694869118550038101724171518216158795241745380263842516797154799057\
039298191515371517451763624495267057619136582519048753642855999781821320, \
0.000717731376741596431490887140939051709200060611267658783415971380121402\
84511429115702857748367141573437609327584129218763297206894467731865620045\
86585633226930118274349267150963666416217698974635538394885553314361223554\
7090, 0.000526563836106867744115459492832605693256339475650422180789825775\
13811920649902981574635213803769011541611348652114928401553113114427237589\
58436765735356346544513372656673382227897334193642653468262334026526305531\
7285486270, 0.000335219160076151640737653139377877549400706668743442523287\
66451330093974597674262378786296593660553976218357869052498745850752725175\
56914551339609987277241355422603527131605114906923071003151304759831103032\
3323140314528550, 0.000144029264260541522327249698926991823417116147474991\
79071065241585417577888819777896006903547471347552305624153710783716156376\
78690532986132341109904260638105535560858822206244843277296675121403729396\
3289265215493739729770], [0.0000561238639183805706846684887361865236220436\
38357559773889497450658137120857495698678996613336383370632529508463006876\
34310766229704861932601450010356462767687035998827115308531708174277614913\
1205142317255459848245622755340, 0.000295689679347099056501272188430985107\
47124546216070733745799683418842722514251378281594066525963811194804542292\
45892859589935333817674995278324914983418276877636690494846166853333919848\
0569150818806963397966243021163162570, 0.000726590624241647921674324758547\
64433199594086908041306804849243704596740495225516788757512778286019304267\
88870514877684654857893188902689989306141931407599705801050168918093724325\
5980896215177543476306346298593857678832060, 0.001348757361360237708882206\
88297749671035030083682536565349103436785321218542041299268429696181728722\
02802836767203939783798613643849638190656795974548568318121913979530086667\
431210044523374776203225056583711418900716459770, 0.0021619632559874337174\
99856171321440919966984583896709034039687334710310704972325717639809462405\
97877039826243246688967860339807879445631381636808829649436256448569575704\
36471295601080694293857257702241965401026616135670570, 0.00316589955395158\
56824333483172454437899001130570060802174579315633398923316428646232943927\
81247444984363647311267100459677293424881458969864484683893289585224951138\
0901639040861915189839389461592967976390774685541301210100, 0.004360182560\
17676130576339507483286748737367473205539785496083741944624536386788429828\
79418785259269496068038290056905799068873260621266485782551331874553284283\
262166626648089787976569121576005966129008224476087030113503220, 0.0057443\
55101596398351544622892585399842185911654729508933631791954248662158230631\
19993559936099313595058714854278769203236495031288047517621954134413876293\
53019814802910980788002615663899279600876503447838866723651949351650, 0.00\
73178870520548279892052529566368146801105456340866930662438305993902008726\
00189724583772404857187710051284030711020561916771635979155950509793886049\
1500324524650188524675442784252360707571808556173907891324621103257355220,\
 0.00908017565173687331372764485183743740847965948783672188661006170085016\
30707443430190600943372317018668189041305591370966260338554244800760128796\
72198886218302510319090819120324792783360869460419749940507197153088946175\
1750, 0.011030545783347045666216464324035010293205855362755769517041387753\
76888717230085056840316512137184752479590089372907491639139750774544050508\
03095568016550395116873163673016915247582530989642723646959395001011502262\
26422100, 0.01316825025053301223564752666036335325009145435990588744692423\
18316736596963209224069503409909760156322166119110318408623388926988046566\
80677489020424492358893549804694055533287649974299434178643292240389500936\
824731716230, 0.0154924700736062642738684507364438860554359931257034480585\
13182600606695026783421593625533942416555173389730743799385380154018386962\
67063912530109659926004633670437321108481203262591242242254855683703673483\
2603397522677990, 0.018002314808168620471120908299125193123863855419945968\
60239756771699630225954956892610978790157923202637633133598163358912201129\
63774295635093390306153728976440874993481471413589430960916799146503969707\
20714884950028478300, 0.02069682288890836328250958370000419582535712768960\
40433826418726633253515992174177737446655767122905610284381412674922846919\
31462646627627173073647136460264091897361721296021529684002886159460698357\
000566671792308490820570, 0.0235749619995124774922842800315261993364281458\
24290857626306719711620038018578220454526595536058865739634786298833232104\
77750976687726074398996535477593637426621652187038705496330743995942805401\
9371143256232751680675705540, 0.026635629469074007083684466914047295843373\
97211101607431469164096454224974669613004067168073661770217259852075508378\
73282668856189915278658534078500361915071815995052028250196844966798796273\
97597348979496259479162677972210, 0.02987765269510863728521748990359761032\
81101233173231699827296852892312212881037997664043988590283406628182671300\
11973803846769954928514543988775532086492866334268696781339934821831804037\
087208881739165146161933019733798930, 0.0332997895931609408716823721179947\
57978845776674606687661926177547997071903871755985217536948570781587988504\
29176815947692100330416622175486959899436798357248731143711646950707056295\
1287111133997700033789236104310198512450, 0.036900729072907607766954568122\
47179864699487507425498694280643391195616755189959345306706958683015217254\
42081978066598648077312668448362615507920591866439903969798670185675600528\
19742903375394664884774370836004942322662810, 0.04067909154062131836483367\
78543028797205678114637653251065140942853121202921984946863757959323051526\
22808459309075397428850113451375432805549742363359609253519768543480564097\
744758057341789009471640982846963770830569857460, 0.0446334294278302865692\
10009009901667232404602006460759847542599413415498684737393739760602026614\
70959048291161203089564058633221048695889657608551872350330514581783602290\
9578421823367118127351086179382356895396265626060100, 0.048762227745987857\
20680777166642505573513039764956413944390007228762874307700182168386531844\
18080406158919798853790152496969059743664585234549273393957519368749988754\
97841658892790947492121009178203720784208163473938682350, 0.05306390466695\
03242375571766367041682135982365419937462630977419681071714952781590072145\
78171909998244710231375602111128022721412358483729772899573292405375171949\
799458896107974075678719753329627809334137159741909415161210, 0.0575368121\
29047460663314300493495093623858061955105821356688366086026344534403057728\
60135677020868078289595783144225470738206397488652742914818378954801792240\
1551785495831923527113795251499257598287643772356654698143810070, 0.062179\
23646851809502317558662801139484571289142371570849794378025385296452342110\
44591443126040925470813318327353904770401743541631551091128678887238289364\
12547616415140337241876555729700082107828946050188836236362490803560, 0.06\
69893990760718599090644403695039672844825205608869793765047398825180290198\
02003434120843648305899931073349968710463247996373298490175247374979797713\
190296246335048768778415471875328292880687632636751930147899983887617990, \
0.071965457078327646039445839426406719775203671415567889625639497035758653\
24931245333560530769098699391336151928205998410765049865491581688562860881\
90665482268463683240919749109306390538168124148999938838694087217007978170\
10, 0.07710550404386912805183024379373295171976015836012456241589171836949\
09157293118060395026143238562555585235175501196984702802867005478399621820\
72864950823317400894254466310412016047194886235861643830709056277202982874\
197020, 0.0824075707136478700116962184008539660887428815399006690025091751\
43507863626760721941106352576840952801141700100727525811210580830746359404\
38737968466697885490838936130601257393022739893908724521870750211828551801\
3298223450, 0.087869625755454899877021687230363575028892980170646596619075\
24166698769325383694689107977999233630562127217620291827963264896650668391\
63564588348688081623644013051773096159227423463380800656547380486986513386\
74415107292810, 0.09348957654217222019342895417614937887297534528369222607\
31802924915994098980588869994455178478434938102431369931790675357405590587\
91135995334027392128359624876038937371359113191590822917302735078626799242\
014430665747887550, 0.0992652699535064710953205515987002619998014431311796\
00912600128149615733004758132502312805071756608481430469428282516536250046\
36871987268198196258215046781631499216077262360429235142651387812922609586\
0184193521086249809270, 0.105194493200897865511981075760986275306303032774\
14207865835474889359535355753618656889030074971894564884349058719162133540\
62421396683195442052601047350735145957691593355391186022428874356370356488\
5448615744873546175155640, 0.111274974675288566070423582386276815133652441\
13997888926078744309573087096958489285690422395001310346310950287689680422\
36372252490153566800637557727562708559926134652040557284973171745484976893\
9188709378669024432307280990, 0.117504384817425863444457473986067187860736\
97729202326388572350527961594357978618837450733073409015715730755886417756\
00857432319795905564230270170367188569339831722328430979226910169981983601\
9428148020282328431317573221980, 0.123880337010366844405659546861093817084\
68024936799177973165124034739356374747940284656730871963587692573993483191\
53246996283531690172867322262695333725039757279906315671621925110414219953\
6128722620684782530755016063817060, 0.130400388493842703929657347425114934\
92689030538375908417562418960897657147752888704746572557392850643143276623\
91684038691679585262287551837901375440997283202133380856490686040013844901\
5788436912001257904929453397021998910, 0.137062041300132459904278405444903\
27021888643473593599609331056972065499434108238072525663747351473197222595\
10725245052557907048508634805627664486419290510736470669323938743684475812\
9877723183678498173759129093472213611730, 0.143862743211087572548234750893\
36316500345564435784946339502011214005664261663611397549943213265017036869\
61099596512298332913757173336407594093412413950795901797229350645329459739\
9235928842990385065876635917083071070104290, 0.150799888735940855364196478\
67135174371629153207161667662263635784985536296222780760423854740005030779\
24147248023321482043350545941869387234224820170020064369403933229509343512\
9724365582206183620899881467920055633024452060, 0.157870820109525092439190\
03237324034392687104052542063917838285978805234475440402234149962981044886\
15385028708249042726302905486836566520098669513199714893401836963188046231\
9065670847536183807284319280467721303483258399940, 0.165072828310518950507\
77407395703014083833065467110301229463194860201870710109236221199783738453\
28853872029784114957559062832234676037409280235422055183192889690162098466\
2307421795828214854075249317563287791844738191528520, 0.172403154099330095\
88654133571142405845499889131315669441246601575306942937997415767812833860\
11011459849570507241114239704007419294575684704094007387964943908236946283\
3660049164884491231498818736734294712010280605910944630, 0.179858989075217\
89873052436794241471127628237882583254659962754205791659531929136433551061\
41659526580597182421208296646890822414356149622740292107253463000690124373\
0499304520899859481644447373950147361709548272209985904230, 0.187437476752\
25073265293971743606896890952846939732457867632897024670375330614245548941\
20323378100476131587438207289043655118797875491060328149128243201909469939\
1088441876910620128010933141191246164821652046728155361299730, 0.195135713\
65368565920257063460667612197892335273458018871450213955164119736839835934\
60881320863478054689988120749862108553045374757860059682381962186695076295\
4696109028292619584309146665746972754759424776880699285285352940, 0.202950\
75042435122661407170798047098604134438790306633759340315897909239503011222\
62611088245177260605854804426672569407531563901973022161535336731900231082\
9676044583161530351908612768300745013862121807643219078274316803370, 0.210\
87959296060721321974955277960609148829458614687764882217759573632822841585\
15756535022738323974362326828426836134181102638706866915447923181577030037\
8393188611316555726709234835088267999938760771192581484514395789181550, 0.\
21891920355744841048816109875101291379989053893258748618077581272593392774\
09601438335385485297102750627157584003183293871500870246773958104119408729\
4739873524868126903689679901500058757946631093253089699628504459201060130,\
 0.22706650207231297134520498525366471211014122388704331929856245956728840\
97627277027353657279894459625711949653209814718105204105798454167457679836\
97877660132089966308979749179568676487426453103822414116671457065672006473\
20, 0.23531836710514944869979679679410176740865978635116591928866188370764\
46841123462810872379291749445217419207340810135966284110479965321707296243\
15661624266391646557015154681081053691322618133085641560951248616943638277\
36240, 0.24367163719429041934887726281044835959791355603336363464503107303\
61173684519315068089305314354649184955680492696229202040104358780228087528\
96242174718868976767050469315914618305399326632915819932332425068324711090\
50086720, 0.25212311202767453202934904537674110419918971837771133876324531\
55675763759786629185753420995997654653336758398595750366483796412190124485\
23253763275186811763627283607379749368410014036836753225259242107294825689\
05255763830, 0.26066955366895293761221651554799395544091931572267424794456\
60942314233564530746010864561910762362646896521592892523533594789071505286\
79690979246275604738804218297929336806247279842048924409711079754800984016\
58380439033420, 0.26930768779801035652953838139102619146242723126614380120\
56211268720114264160131834036332688904843168058188615259574378487498438719\
35130656587241302739404757937152833120291343802926219244755940782621744964\
75912738488084600, 0.27803420496542551565670484082035946989922528012144572\
86408318044568264801747695426492961994412892568621046330214294675170033014\
26432099378612835951496593401457766599162168594407882226340411304499593414\
45901510200127118020, 0.28684576186039034614424090915396348886982053804574\
76016129711162659837343187496274992388828935099251826664861091041328432921\
75393262196979519528147868559069111898957304515546990730528114686814813951\
62898403561667328452110, 0.29573898259160217714082399375862325569200974145\
02811800329996044092792850981739678534833845614595272031881404365406745597\
07842251075354069457370070660015827123026860900326203630897427259661497573\
85719209472050960605861820, 0.30471045998063818993994844857670322542178997\
24972778365168679360141103325827236026408735398489236088844885670586132135\
44131472510542936462883909700270714082759838236552679469781133835093208869\
28197015105464324322321092030, 0.31375675686731661471440753026130928848642\
45479294054834561823204232321008032057644423550202045447028632630216879742\
43162018047260477609428337413773218474894851147231211074334474334007331728\
38127708834444740046669849657400, 0.32287440742654455950244370168578069770\
75444414814167656890490994515372648807928734134700447552674001261619214307\
75680119988919571497166862896495220171169976648677846397353150912528094692\
84542011666104179991728488087238690, 0.33205991849614796023224868187397558\
57408841683782353239666650402817997050618181702103592822492713520170019194\
81289860640498233153055075575027433847684296257620213437647285273245185808\
15101645695925256026014619988954473310, 0.34130977091517493300006525400304\
23159124728261306993619129265319581155909574630462386026271120498870831601\
73869740296979210079357832396885431401787014531932663868867473721955066432\
23879020005537263382382434166594673796530, 0.35062042087215979716062387095\
81331367681069089991154783043960465978959689949799958778514184852613710288\
48256610921687964265283554895650824197381350133710018607382118546290888960\
39146114856568466485442664178762447330844660, 0.35998830126283122158204729\
54942578003420498444563685123390826134265158821954621471643258100643745853\
02988120178127928414349845605856409137918720240882577912948223158588381799\
01062017952989405387817221558188868284365277530, 0.36940982305674432812083\
60268169366295153311459989513098531708744637093420551583385711937471325918\
48247507200060616745922030730003718165440722381959748247968397559581559240\
66066335478094209209775048079373270803836450660890, 0.37888137667231316737\
07795146325136804528450330140706858705026510668983341283135085257550497315\
66975981365203549191859737638920972281542910836114673604057548400272554533\
08495918737970944034924947384156845286858366640366270, 0.38839933335971676\
33412165509658929311027511836211991521992932063156856610767033764251003145\
40029873957900764545015398552719176337287919338350853353869378148536621195\
47414941114734091312659371348584785649710909307428841520, 0.39796004659114\
89071569545526582121880566011273815964750061105647002874240919419777443031\
70336504432719978397562870600750636262255159266947436699918218324557328069\
95914028348930092528034073254808665311319471295273434117570, 0.40755985345\
78790662991695500571189999453105096819560390822623162177015426824500751383\
52749499470684805783282698299927044259839340426258037160529445246015251740\
63116139441406030687633719764572265090857516926653893181032600, 0.41719507\
60735891664009455627547263341049477742813871152676886865618046855101303000\
15925162256464604848017342100613781287867845745648547176852625114357742828\
98508750551313586962595256038844029165272137297125705696676542900, 0.42686\
20229834485981719148753465216040579543998621950812184970675982147832892268\
49911066692694037013055889558630710710381769784029331836497288018771775346\
50065907519592311498528652676715548976539046445568178302069260726300, 0.43\
65569905783876035744089542583849940217311743545596558659240290059933055301\
85612751774464551266473408551183761959126551111181766289833492242253091202\
55401994394959079198024756546131645880337796220036231850261367000636600, 0\
.4462762645140272037504711186440272214009020713432648460272788525085189042\
46248944918794225885188661455251336396405912589481984971034114967404584405\
20033574916486480969374986362523071466297492982409074277630347630550881220\
, 0.4560161211337220471676962108151192006192218447757166859883592454679666\
38297877841036380217868599617031742536161205878617913726572308903376415936\
17922167792830003196529978780809727858751338065264750232354003921130985863\
490, 0.4657728288951709806953574134091879071659079961310382024583701274474\
20272968549090105716059446370281117992050883462450877891691665620711216300\
27602162927262890599131108604413377830502623448337283557716112922142964414\
461790, 0.4755426498000487794441215624702440224973480722654691477881777818\
58899257691578699147384687891674455047901481541436585320595742950314465186\
13303628465215863487643610560869437373563302231148227392084290921005823615\
541785250, 0.4853218408261113137263263198019286478499817452518929468140082\
56743260440411877718601384817466673594860419989121093562910759679061054456\
42389683028432863204971971488966437851373569421765241530380854394561360954\
774160070290, 0.4951066553612254838625924661879287255720428567012801425066\
15120187105610275984678542718884567552785874961812678255745792969041427658\
19565661144724103919335257576485944076964424633898785379726527711283884928\
552509424716590, 0.5048933446387745161374075338120712744279571432987198574\
93384879812894389724015321457281115432447214125038187321744254207030958572\
34180434338855275896080664742423514055923035575366101214620273472288716115\
071447490575283400, 0.5146781591738886862736736801980713521500182547481070\
53185991743256739559588122281398615182533326405139580010878906437089240320\
93894554357610316971567136795028028511033562148626430578234758469619145605\
438639045225839929700, 0.5244573501999512205558784375297559775026519277345\
30852211822218141100742308421300852615312108325544952098518458563414679404\
25704968553481386696371534784136512356389439130562626436697768851772607915\
709078994176384458214740, 0.5342271711048290193046425865908120928340920038\
68961797541629872552579727031450909894283940553629718882007949116537549122\
10830833437928878369972397837072737109400868891395586622169497376551662716\
442283887077857035585538200, 0.5439838788662779528323037891848807993807781\
55224283314011640754532033361702122158963619782131400382968257463838794121\
38208627342769109662358406382077832207169996803470021219190272141248661934\
735249767645996078869014136500, 0.5537237354859727962495288813559727785990\
97928656735153972721147491481095753751055081205774114811338544748663603594\
08741051801502896588503259541559479966425083513519030625013637476928533702\
507017590925722369652369449118770, 0.5634430094216123964255910457416150059\
78268825645440344134075970994006694469814387248225535448733526591448816238\
04087344888881823371016650775774690879744598005605040920801975243453868354\
119662203779963768149738632999363390, 0.5731379770165514018280851246534783\
95942045600137804918781502932401785216710773150088933307305962986944110441\
36928928961823021597066816350271198122822465349934092480407688501471347323\
284451023460953554431821697930739273690, 0.5828049239264108335990544372452\
73665895052225718612884732311313438195314489869699984074837743535395151982\
65789938621871213215425435145282314737488564225717101491249448686413037404\
743961155970834727862702874294303323457090, 0.5924401465421209337008304499\
42881000054689490318043960917737683782298457317549924861647250500529315194\
21671730170007295574016065957374196283947055475398474825936883860558593969\
312366280235427734909142483073346106818967390, 0.6020399534088510928430454\
47341787811943398872618403524993889435299712575908058022255696829663495567\
28002160243712939924936373774484073305256330008178167544267193004085971651\
069907471965926745191334688680528704726565882420, 0.6116006666402832366587\
83449034107068897248816378800847800706793684314338923296623574899685459970\
12604209923545498460144728082366271208066164914664613062185146337880452585\
058885265908687340628651415214350289090692571158470, 0.6211186233276868326\
29220485367486319547154966985929314129497348933101665871686491474244950268\
43302401863479645080814026236107902771845708916388532639594245159972744546\
691504081262029055965075052615843154713141633359633720, 0.6305901769432556\
71879163973183063370484668854001048690146829125536290657944841661428806252\
86740815175249279993938325407796926999628183455927761804025175203160244041\
844075933933664521905790790224951920626729196163549339100, 0.6400116987371\
68778417952704505742199657950155543631487660917386573484117804537852835674\
18993562541469701187982187207158565015439414359086208127975911742208705177\
684141161820098937982047010594612182778441811131715634722460, 0.6493795791\
27840202839376129041866863231893091000884521695603953402104031005020004122\
14858151473862897115174338907831203573471644510434917580261864986628998139\
261788145370911103960853885143431533514557335821237552669155330, 0.6586902\
29084825066999934745996957684087527173869300638087073468041884409042536953\
76139737288795011291683982613025970302078992064216760311456859821298546806\
733613113252627804493356776120979994462736617617565833405326203460, 0.6679\
40081503852039767751318126024414259115831621764676033334959718200294938181\
82978964071775072864798299808051871013935950176684694492442497256615231570\
374237978656235271472675481419184898354304074743973985380011045526690, 0.6\
77125592573455440497556298314219302292455558518583234310950900548462735119\
20712658652995524473259987383807856922431988001108042850283313710350477982\
883002335132215360264684908747190530715457988333895820008271511912761300, \
0.686243243132683385285592469738690711513575452070594516543817679576767899\
19679423555764497979545529713673697831202575683798195273952239057166258622\
67815251051488527687889256655256659926682716187229116555525995333015034259\
0, 0.695289540019361810060051551423296774578210027502722163483132063985889\
66741727639735912646015107639111551143294138678645586852748945706353711609\
02997292859172401617634473205302188661649067911307180298489453567567767890\
7960, 0.704261017408397822859176006241376744307990258549718819967000395590\
72071490182603214651661543854047279681185956345932544029215774892464593054\
26299293399841728769731390996737963691025727403385024261428079052794903939\
4138170, 0.713154238139609653855759090846036511130179461954252398387028883\
73401626568125037250076111710649007481733351389089586715670782460673780302\
04804718521314409308881010426954844530092694718853131851860483710159643833\
2671547880, 0.721965795034574484343295159179640530100774719878554271359168\
19554317351982523045735070380055871074313789536697857053248299669857356790\
06213871640485034065985422334008378314055921177736595886955004065855409848\
9799872881970, 0.730692312201989643470461618608973808537572768733856198794\
37887312798857358398681659636673110951568319418113847404256215125015612806\
48693434127586972605952420628471668797086561970737807552440592173782550352\
4087261511915400, 0.739330446331047062387783484452006044559080684277325752\
05543390576857664354692539891354380892376373531034784071074764664052109284\
94713203090207537243952611957817020706631937527201579510755902889202451990\
1598341619560966570, 0.747876887972325467970650954623258895800810281622288\
66123675468443242362402133708142465790040023453466632416014042496335162035\
87809875514767462367248131882363727163926202506315899859631632467747407578\
9270517431094744236160, 0.756328362805709580651122737189551640402086443966\
63636535496892696388263154806849319106946856453508150443195073037707979598\
95641219771912471037578252811310232329495306840853816946006733670841800676\
6757493167528890949913270, 0.764681632894850551300203203205898232591340213\
64883408071133811629235531588765371891276207082505547825807926591898640337\
15889520034678292703756843383757336083534429848453189189463086773818669143\
5843904875138305636172263750, 0.772933497927687028654795014746335287889858\
77611295668070143754043271159023727229726463427201055403742880503467901852\
81894795894201545832542320163021223398679100336910202508204313235125735468\
9617758588332854293432799352670, 0.781080796442551589511838901248987086200\
10946106741251381922418727406607225903985616646145147028972493728424159968\
16706128499129753226041895880591270526012647513187309631032009849994124205\
3368906746910300371495540798939870, 0.789120407039392786780250447220393908\
51170541385312235117782240426367177158414842434649772616760256376731715731\
63865818897361293133084552076818422969962160681138868344427329076516491173\
2000061239228807418515485604210818440, 0.797049249575648773385928292019529\
01395865561209693366240659684102090760496988777373889117548227393941451955\
73327430592468436098026977838464663268099768917032395541683846964809138723\
1699254986137878192356780921725683196620, 0.804864286346314340797429365393\
32387802107664726541981128549786044835880263160164065391186791365219453100\
11879250137891446954625242139940317618037813304923704530389097170738041569\
0853334253027245240575223119300714714647050, 0.812562523247749267347060282\
56393103109047153060267542132367102975329624669385754451058796766218995238\
68412561792710956344881202124508939671850871756798090530060891155812308937\
9871989066858808753835178347953271844638700260, 0.820141010924782101269475\
63205758528872371762117416745340037245794208340468070863566448938583404734\
19402817578791703353109177585643850377259707892746536999309875626950069547\
9100140518355552626049852638290451727790014095760, 0.827596845900669904113\
45866428857594154500110868684330558753398424693057062002584232187166139889\
88540150429492758885760295992580705424315295905992612035056091763053716633\
9950835115508768501181263265705287989719394089055370, 0.834927171689481049\
49222592604296985916166934532889698770536805139798129289890763778800216261\
54671146127970215885042440937167765323962590719764577944816807110309837901\
5337692578204171785145924750682436712208155261808471470, 0.842129179890474\
90756080996762675965607312895947457936082161714021194765524559597765850037\
01895511384614971291750957273697094513163433479901330486800285106598163036\
8119537680934329152463816192715680719532278696516741600050, 0.849200111264\
05914463580352132864825628370846792838332337736364215014463703777219239576\
14525999496922075852751976678517956649454058130612765775179829979935630596\
0667704906564870275634417793816379100118532079944366975547930, 0.856137256\
78891242745176524910663683499654435564215053660497988785994335738336388602\
45005678673498296313038900403487701667086242826663592405906587586049204098\
2027706493546705402600764071157009614934123364082916928929895700, 0.862937\
95869986754009572159455509672978111356526406400390668943027934500565891761\
92747433625264852680277740489274754947442092951491365194372335513580709489\
2635293306760612563155241870122276816321501826240870906527786388260, 0.869\
59961150615729607034265257488506507310969461624091582437581039102342852247\
11129525342744260714935685672337608315961308320414737712448162098624559002\
7167978666191435093139599861550984211563087998742095070546602978001080, 0.\
87611966298963315559434045313890618291531975063200822026834875965260643625\
25205971534326912803641230742600651680846753003716468309827132677737304666\
2749602427200936843283780748895857800463871277379315217469244983936182930,\
 0.88249561518257413655554252601393281213926302270797673611427649472038405\
64202138116254926692659098428426924411358224399142567680204094435769729829\
63281143066016827767156902077308983001801639805718519797176715686824267780\
10, 0.88872502532471143392957641761372318486634755886002111073921255690426\
91290304151071430957760499868965368904971231031957763627747509846433199362\
44227243729144007386534795944271502682825451502310608112906213309755676927\
19010, 0.89480550679910213448801892423901372469369696722585792134164525110\
64046464424638134311096992502810543511565094128083786645937578603316804557\
94739895264926485404230840664460881397757112564362964351145513842551264538\
24844350, 0.90073473004649352890467944840129973800019855686882039908739987\
18503842669952418674976871949282433915185695305717174834637499536312801273\
18018037417849532183685007839227376395707648573486121870773904139815806478\
91375019070, 0.90651042345782777980657104582385062112702465471630777392681\
97075084005901019411130005544821521565061897568630068209324642594409412088\
64004665972607871640375123961062628640886808409177082697264921373200757985\
56933425211240, 0.91213037424454510012297831276963642497110701982935340338\
09247583330123067461630531089202200076636943787278237970817203673510334933\
16083643541165131191837635598694822690384077257653661919934345261951301348\
66132558489270720, 0.91759242928635212998830378159914603391125711846009933\
09974908248564921363732392780588936474231590471988582998992724741887894191\
69253640595612620315333021145091610638693987426069772601060912754781292497\
88171448198670177650, 0.92289449595613087194816975620626704828023984163987\
54375841082816305090842706881939604973856761437444414764824498803015297197\
13299452160037817927135049176682599105745533689587983952805113764138356169\
29094372279701712580290, 0.92803454292167235396055416057359328022479632858\
44321103743605029642413467506875466643946923090130060866384807179400158923\
49501345084183114371391180933451773153631675908025089069360946183187585100\
00611613059127829920218300, 0.93301060092392814009093555963049603271551747\
94391130206234952601174819709801979965658791563516941000689266500312895367\
52003626701509824752625020202286809703753664951231221584528124671707119312\
36736324806985210001611238200, 0.93782076353148190497682441337198860515428\
71085762842915020562197461470354765788955408556873959074529186681672646095\
22959825645836844890887132111276171063587452383584859662758123444270299917\
89217105394981116376363750919640, 0.94246318787095253933668569950650490637\
61419380448941786433116339139736554655969422713986432297913192171040421685\
57745292617936025113472570851816210451982077598448214504168076472886204748\
50074240171235622764334530185618990, 0.94693609533304967576244282336329583\
17864017634580062537369022580318928285047218409927854218280900017552897686\
24397888871977278587641516270227100426707594624828050200541103892025924321\
28024667037219066586284025809058483880, 0.95123777225401214279319222833357\
49442648696023504358605560999277123712569229981783161346815581919593841080\
20114620984750303094025633541476545072660604248063125001124502158341107209\
05250787899082179627921579183652606131760, 0.95536657057216971343078999099\
00983327675953979935392401524574005865845013152626062602393979733852904095\
17088387969104359413667789513041103423914481276496694854182163977090421578\
17663288187264891382061764310460373437393990, 0.95932090845937868163516632\
21456971202794321885362346748934859057146878797078015053136242040676948473\
77191540690924602571149886548624567194450257636640390746480231456519435902\
25524194265821099052835901715303622916943014250, 0.96309927092709239223304\
54318775282013530051249257450130571935660880438324481004065469329304131698\
47827455791802193340135192268733155163738449207940813356009603020132981432\
43994718025709662460533511522562916399505767733720, 0.96670021040683905912\
83176278820052420211542233253933123380738224520029280961282440147824630514\
29218412011495708231840523078996695833778245130401005632016427512688562883\
53049292943704871288886600229996621076389568980148750, 0.97012234730489136\
27147825100964023896718898766826768300172703147107687787118962002335956011\
40971659337181732869988026196153230045071485456011224467913507133665731303\
21866006517816819596291279111826083485383806698026620100, 0.97336437053092\
59929163155330859527041566260278889839256853083590354577502533038699593283\
19263382297827401479244916212671733114381008472134146592149963808492818400\
49479717498031550332012037260240265102050374052083732202780, 0.97642503800\
04875225077157199684738006635718541757091423736932802883799619814217795454\
73404463941134260365213701166767895222490233122739256010034645224063625733\
78347812961294503669256004057194598062885674376724831932429440, 0.97930317\
71110916367174904162999958041746428723103959566173581273366746484007825822\
26255334423287709438971561858732507715308068537353372372826926352863539735\
90810263827870397847031599711384053930164299943332820769150917940, 0.98199\
76851918313795288790917008748068761361445800540313976024322830036977404504\
31073890212098420767973623668664018366410877988703622570436490660969384627\
10235591250065185285864105690390832008534960302927928511504997152170, 0.98\
45075299263937357261315492635561139445640068742965519414868173993933049732\
16578406374466057583444826610269256200614619845981613037329360874698903400\
73995366329562678891518796737408757757745144316296326516739660247732200, 0\
.9868317497494669877643524733396366467499085456400941125530757681683263403\
03679077593049659009023984367783388088968159137661107301195343319322510979\
57550764110645019530594446671235002570056582135670775961049906317526828370\
, 0.9889694542166529543337835356759649897067941446372442304829586122462311\
12827699149431596834878628152475204099106270925083608602492254559494919690\
44319834496048831268363269830847524174690103572763530406049989884977377357\
790, 0.9909198243482631266862723551481625625915203405121632781133899382991\
49836929255656980939905662768298133181095869440862903373966144575519923987\
12032780111378169748968090918087967520721663913053958025005949280284691105\
382480, 0.9926821129479451720107947470433631853198894543659133069337561694\
00609799127399810275416227595142812289948715969288979438083228364020844049\
49020611395084996754753498114753245572157476392924281914438260921086753788\
967426440, 0.9942556448984036016484553771074146001578140883452704910663682\
08045751337841769368800064400639006864049412851457212307967635049687119524\
82378045865586123706469801851970890192119973843361007203991234965521611332\
763480506480, 0.9956398174398232386942366049251671325126263252679446021450\
39162580553754636132115701712058121474073050393196170994309420093112673937\
87335142174486681254467157167378333733519102120234308784239940338709917755\
239129698864960, 0.9968341004460484143175666516827545562100998869429939197\
82542068436660107668357135376705607218752555015636352688732899540322706575\
11854103013551531610671041477504886190983609591380848101606105384070320236\
092253144586987900, 0.9978380367440125662825001438286785590800330154161032\
90965960312665289689295027674282360190537594021229601737567533110321396601\
92120554368618363191170350563743551430424295635287043989193057061427422977\
580345989733838643290, 0.9986512426386397622911177931170225032896496991631\
74634346508965632146787814579587007315703038182712779719716323279606021620\
13863561503618093432040254514316818780860204699133325687899554766252237967\
749434162885810992835400, 0.9992734093757583520783256752414523556680040591\
30919586931951507562954032595047744832112424872217139806957321112948512231\
53451421068110973100106938580685924002941989498310819062756744019103784822\
456523693653701406142321170, 0.9997043103206529009434987278115690148925287\
54537839292662542003165811572774857486217184059334740361888051954577075410\
71404100646661823250047216750850165817231223633095051538331466660801519430\
849181193036602033756978836830, 0.9999438761360816194293153315112638134763\
77956361642440226110502549341862879142504301321003386663616629367470491536\
99312365689233770295138067398549989643537232312964001172884691468291825722\
385086879485768274454015175437720]]):

