# version 9.9.97 #
#------------------------------------------------------------------------#
# The following quadrature is the control routine for numerical          #
# quadrature. It should be called in the form                            #
#                                                                        #
#            quadrature(f(x),x=x0..x1)                                   #
# or                                                                     #
#            quadrature(f(x),x=x0..x1,method=n)                          #
# or                                                                     #
#            quadrature(f(x),x=x0..x1,method=n,Adaptive=TRUE/FALSE)      #
#                                                                        #
# with <method> taken from                                               #
#          {GaussLegendre,NewtonCotes,GaussTschebyscheff}                #
# Following alternative names for these methods are accepted:            #
#                                                                        #
#    GaussLegendre,gausslegendre,Gauss,gauss,GL,gl                       #
#    GaussTschebyscheff,gausstschebyscheff,GT,gt                         #
#    NewtonCotes,newtoncotes,NC,nc                                       #
#                                                                        #
# The adaptive mechanism is based on a fixed quadrature rule specified   #
# by one of these names with n nodes, where <n> may take any finite      #
# value from {1,2,3,...}.                                                #
# If no method is specified, then, depending on DIGITS, an adaptive      #
# Gauss-Legendre quadrature with n=20,40,80 or 160 nodes (with           #
# respective orders 40,80,160 or 320) is used as default.                #
#                                                                        # 
# Examples:                                                              #
# >>  quadrature( exp(x^2), x=0..1 );                                    #
# >>  quadrature( exp(x^2), x=0..1, GaussLegendre=40 );                  #
# >>  quadrature( exp(x^2), x=0..1, GL=160, Adaptive=FALSE );            #
# >>  quadrature( exp(x^2), x=0..1, NewtonCotes=10, Adaptive=TRUE );     #
#                                                                        #
# Technical remark: the data of Gaussian quadrature GL=20, GL=40, GL=80  #
#      and GL=160 are pre-computed and stored with 200 DIGITS, whence    #
#      there is no delay when using these methods with at most 200       #
#      DIGITS. For different values of <n> or higher DIGITS these data   #
#      will be computed before the actual quadrature. This may take some #
#      time. Therefore it is recommended to use these values of <n> and  #
#      to restrict ones ambitions to DIGITS<=200.                        #
#      The algorithm will converge eventually for arbitrarily high       #
#      DIGITS and arbitrary integrands (provided the integral exists).   #
#      However, convergence will be slow if very high precision is       #
#      required. Also, convergence will be slow if the integrand is not  #
#      smooth.                                                           #
#------------------------------------------------------------------------#

numeric::quadrature:=proc(alg_expr,eq)
local QuadOptions,adaptive,method,n,b,c,freestyle,
      GLnames,GTnames,NCnames,implementedMethods,
      Quadsum,adaptiveQuad,QuadVars,tmp;
begin
  if args(0)<2 then error(": call ".procname."(f(x),x=a..b)");end_if;
  if args(0)=2 then freestyle:=TRUE; end_if; # the default method: #
  if args(0)>2 then # find the requested options #
     QuadOptions:=[args(3..args(0))]:
     if has(QuadOptions,Adaptive=FALSE) 
        then adaptive:=FALSE else adaptive:=TRUE; 
     end_if;
     QuadOptions:= map(QuadOptions, proc(x) begin 
             if has(x,Adaptive) then null() else x end_if end_proc,QuadOptions):
     if QuadOptions=[]
       then if adaptive
               then freestyle:=TRUE
               else error("need to specify non-adaptive quadrature method");
            end_if;
       else if nops(QuadOptions)>1 then error("too many or wrong options") end_if;
            GLnames:={Gauss,gauss,GaussLegendre,gausslegendre,GL,gl};
            GTnames:={GaussTschebyscheff,GT,gausstschebyscheff,gt,
                      GaussChebyshev,GC,gausschebychev,gc}:
            NCnames:={NewtonCotes,NC,newtoncotes,nc};
            implementedMethods:=GLnames union GTnames union NCnames:
            QuadOptions:=op(QuadOptions);
            if type(QuadOptions) = "_equal"
               then method:=op(QuadOptions,1);n:=op(QuadOptions,2);
               else error("specification of the method must be of the form <method=n> ");
            end_if;
            if   (not contains(implementedMethods,method))
              or (not testtype(n, Type::PosInt)) then error("method not implemented")
            end_if;
            if contains(GLnames, method) then method:=GaussLegendre end_if:
            if contains(GTnames, method) then method:=GaussTschebyscheff;
                if adaptive then adaptive:=FALSE;
                warning("Warning: can use Gauss-Tschebyscheff method in non-adaptive mode only.   Switching to  GaussTschebyscheff=".n.", Adaptive=FALSE ; requested precision will not be achieved [numeric::quadrature");
                end_if;
            end_if;
            if contains(NCnames, method) then method:=NewtonCotes end_if;
     end_if;
 end_if;
 if freestyle=TRUE then
     method:=GaussLegendre: adaptive:=TRUE:
     if   DIGITS<=10 then  n:=20; 
     elif DIGITS<=50 then  n:=40; 
     elif DIGITS<=100 then n:=80; 
     else n:=160;
     end_if;
  end_if;

  # the following is to allow multiple integrations: #
  # alg_expr:=float(alg_expr); #                  # !!! new #
  if indets(map(op(eq,2),float))<>{}              # !!! new #
     then return( procname(args()) );             # !!! new #
  end_if;                                         # !!! new #
  QuadVars:= {op(eq,1)};                          # !!! new #
  tmp:=alg_expr;                                  # !!! new #
  while op(tmp,0) = procname do                   # !!! new #
     if indets(map(op(tmp,[2,2]),float)) minus QuadVars <>{}
         then return( procname(args()) );         # !!! new #
     end_if;                                      # !!! new #
     QuadVars:=QuadVars union {op(tmp,[2,1])};    # !!! new #
     tmp:=op(tmp,1);                              # !!! new #
  end_while;                                      # !!! new #
  if indets(float(tmp)) minus QuadVars <> {}      # !!! new #
     then return( procname(args()) );             # !!! new #
  end_if;                                         # !!! new #

# so far this was just to find out the requested method. Now the actual  #
# computation: compute weights b[i] and abscissae c[i] for the requested #
# method using the global routines gldata, gtdata, gtdata below          #

 if method=GaussLegendre then ([b,c]):=numeric::gldata(n,DIGITS) end_if;
 if method=GaussTschebyscheff then ([b,c]):=numeric::gtdata(n)   end_if;
 if method=NewtonCotes then ([b,c]):=numeric::ncdata(n)          end_if;


 #-----------------define local subroutine ------------------------------#
 # Quadsum computes                                                      #
 #  Quadsum( f(x),x=x0..x1):=(x1-x0)*sum(b[i]*f(x0+c[i]*(x1-x0)),i=1..n) #
 # based on the quadrature formula                                       #
 #    int(f(c),c=0..1)==sum( b[i]*f(c[i]),i=1..n)                        #
 # with abscissae  c[i]  and weights  b[i]                               #
 #-----------------------------------------------------------------------#
 Quadsum:=proc(alg_expr,eq)
 local x,x0,x1,L,i;
 begin  x:=op(eq,1): x0:=op(eq,[2,1]); x1:=op(eq,[2,2]); L:=x1-x0; 
        if iszero(L) then return(0);end_if;
        float(L)*_plus(float(b[i]*float(subs(alg_expr,x=x0+c[i]*L)))$hold(i)=1..n);
        eval(last(1));
 end_proc:

#------------- define local subroutine ------------------------------------
 adaptiveQuad is a general adaptive algorithm based on a non-adaptive 
 quadrature rule Quadsum:
   int(f(x),x=0..1)==sum(b[i]*f(c[i],i=1..n)=:Quadsum(f,x=0..1)
 given by the quadrature data b[i] (weights) and c[i] (abscissae).
 First,  an estimate  Is:= Quadsum(f,x=x0..x1) is computed.
 Then, by recursive bisectioning of the interval, the method Quadsum is used 
 to compute the integrals over each half of the interval.
 Bisectioning is stopped, when  
    Quadsum(first half)+ Quadsum(second half)-Quadsum(complete interval)
 has become so small that this difference cannot be added to the estimate
 Is within float precision.
 Note:  let  p be the order of the quadrature rule, i.e.
    Error( Quadsum( f,  x=x0..x1 ) )  =  constant( f ) * (x1-x0)^p
 Then bisectioning will occur down to subintervals of length L, until  
       constant(f) * L^p == Is * 10^(-DIGITS)
 holds,  i.e.,  L = O( 10^(-DIGITS/p ) :  the costs grow
 exponentially with DIGITS !
 For very low precision low order methods Quadsum suffice.  
 FOR HIGH PRECISION A HIGH ORDER METHOD Quadsum IS NECESSARY !
 A reasonable (heuristic) choice seems Gauss-Legendre-Quadrature
 with n nodes which has order (2n+1). 
 Example: n=20, for which the abscissae c[j] and weights b[j]
 can be computed in a fraction of a second, if DIGITS is not too
 high. Each bisection step then produces quadrature values with errors
 whose sum is approximately the error of the quadrature on the double 
 interval multplied by a factor 
     2^(-40)==10^(-12)
 Thus, for slowly variing integrands, approximately DIGITS/12
 bisection steps are needed to reduce the error of Quadsum(whole interval)
 by DIGITS decimal places. This leads to
   2^(DIGITS/12)
 subintervals, each requiring n=20 evaluations of the integrand.
---------------------------------------------------------------------#

  adaptiveQuad:=proc(alg_expr,eq,bisectionlevel)
     local x,x0,x1,mid,I1,Is,I21,I22,I2,macheps;
  begin
     x:=op(eq,1):x0:=op(eq,[2,1]):x1:=op(eq,[2,2]):mid:=(x0+x1)/2;
     if bisectionlevel=0
        then I1:=Quadsum(alg_expr,x=x0..x1);Is:=I1;
        else I1:=args(4);Is:=args(5);
     end_if;
     macheps:=float(10^(-DIGITS));
     I21:=Quadsum(alg_expr,x=x0..mid); 
     I22:=Quadsum(alg_expr,x=mid..x1);
     I2:=I21+I22;
     if abs(I2-I1)<= (abs(Is)+n*abs(I1)+2*n*abs(I2))*macheps 
         then return(I2) 
     end_if;
    #---------------------------------------------------------------#
    # in further calls of adaptiveQuad pass the estimates of the    #
    # integrals on the present half intervals and Is (the estimate  #
    # of the integral on the whole interval) as parameters 4 and 5, #
    # respectively.                                                 #
    # The bisectionlevel gives the length of the present bisected   #
    # interval = original interval / 2^(bisectionlevel)             #
    #---------------------------------------------------------------#
    return(adaptiveQuad(alg_expr,x=x0..mid,bisectionlevel+1,I21,Is)
         + adaptiveQuad(alg_expr,x=mid..x1,bisectionlevel+1,I22,Is)
          );
  end_proc:
  #----------- end of adaptiveQuad ----------------------------#

 if adaptive                      # now do the quadrature: #
    then adaptiveQuad(alg_expr,eq,0);
    else Quadsum(alg_expr,eq);
 end_if;
end_proc:
#----------- end of Quadrature ----------------------------#


#----------------------------------------------------------------#
# The data (weights b[i] and abscissae c[i]) are provided for    #
# Gauss-Legendre quadrature, for Gauss-Tschebyscheff-Quadrature  #
# and for Newton-Cotes quadrature:                               #
# See gldata.mu , gtdata.mu and gtdata.mu                        #
#----------------------------------------------------------------#
