
# ---------------  file butcher.mu 8.10.97 ---------------------------- #
#  Help file:

 butcher -- Butcher parameters of Runge-Kutta schemes

 Call:

 numeric::butcher( method )

 Parameter:

    method -- name of the numerical scheme, one of
              EULER1, RKF43, RK4, RKF34, RKF54a, RKF54b, DOPRI54, 
              RKF45a, RKF45b, DOPRI45, BUTCHER6, RKF87, RKF78 

 Synopsis:

   An explicit s-stage Runge-Kutta method for the numerical integration
   of a dynamical system dy/dt = f(t,y) with step size h is a map 
   (t,y) -> (t+h,Y) defined by Y = y + h*b[1]*k1 + ... + h*b[s]*k.s
   with k1,...,k.s given by

      k1  = f(t,y) 
      k2  = f(t+c[2]*h, y + h*a[2,1]*k1)
         ...
      k.s = f(t+c[s]*h, y + h*a[s,1]*k1 + ... + h*a[s,s-1]*k.(s-1) )

   Various numerical schemes arise from different choices of the Butcher 
   parameters: the (s times s)-matrix a[i,j], the weights b=[b[1],..,b[s]]
   and the abscissae c=[0,c[2],...,c[s]].

   Embedded pairs of Runge-Kutta methods consist of two methods that share 
   the matrix a[i,j] and the abscissae c[i], but use different weights b[j].
   
   The call butcher(method) returns the list [s,c,a,b1,b2,order1,order2].
   These are the Butcher data of the method: s is the number of stages,
   c is the list of abscissae, a is the strictly lower Butcher matrix, 
   b1 and b2 are lists of weights. The integers order1 and order2 are the
   orders of the scheme when using the weights b1 or b2, respectively,
   in conjunction with the matrix a and the abscissae c.

   The methods EULER1, RK4 and BUTCHER6 are single methods with b1=b2 and
   order1=order2. All other methods are embedded pairs of Runge-Kutta-Fehlberg
   or Dormand-Prince type.

 Example:

>> ([s,c,a,b1,b2,order1,order2]) := butcher(RKF34):

>> s, c, a;

                               +-                                    -+
                               |   0,       0,        0,       0,   0 |
                               |                                      |
                               |  1/4,      0,        0,       0,   0 |
      +-                   -+  |                                      |
   5, | 0, 1/4, 4/9, 6/7, 1 |, |  4/81,   32/81,      0,       0,   0 |
      +-                   -+  |                                      |
                               | 57/98, -432/343, 1053/686,    0,   0 |
                               |                                      |
                               |  1/6,      0,      27/52,  49/156, 0 |
                               +-                                    -+

>> b1, b2;

   +-                        -+  +-                                  -+
   | 1/6, 0, 27/52, 49/156, 0 |, | 43/288, 0, 243/416, 343/1872, 1/12 |
   +-                        -+  +-                                  -+

>> order1, order2;

                                   3 , 4

 See also:  numeric::odesolve
     
 ------------------------------------------------------------------------#

numeric::butcher:=proc(method)
local s,order1,order2,C,A,B1,B2;
begin
     case method 
     of EULER1 do
        s:=1;order1:=1;order2:=1;
        A:=array(1..s,1..s); A[1,1]:=0;
        B1:=array(1..s); B1[1]:=1;
        B2:=array(1..s); B2[1]:=1;
        C:=array(1..s); C[1]:=1;
        break;
     of RK4 do
        s:=4;order1:=4;order2:=4;
        A:=array(1..s,1..s,[
                           [ 0 , 0 , 0 , 0 ] ,
                           [1/2, 0 , 0 , 0 ] ,
                           [ 0 ,1/2, 0 , 0 ] ,
                           [ 0 , 0 , 1 , 0 ] ]);  
        B1:=array(1..s,[1/6,1/3,1/3,1/6]  );
        B2:=array(1..s,[1/6,1/3,1/3,1/6]  );
        C:=array(1..s,[0,1/2,1/2,1]);
        break;
     of RKF34 do
        s:=5; order1:=3; order2:=4;
        A:=array(1..s,1..s,[
            [   0   ,    0   ,    0   ,   0    , 0  ],
            [  1/4  ,    0   ,    0   ,   0    , 0  ],
            [  4/81 ,  32/81 ,    0   ,   0    , 0  ], 
            [ 57/98 ,-432/343,1053/686,   0    , 0  ],
            [  1/6  ,    0   ,  27/52 , 49/156 , 0  ] ]);
        B1:=array(1..s,
            [  1/6   ,   0   ,  27/52 , 49/156 , 0  ] );
        B2:=array(1..s,
            [ 43/288 ,    0   ,243/416,343/1872,1/12] );
        C:=array(1..s,[0,1/4,4/9,6/7,1]);  
        break;
     of RKF43 do
        s:=5; order1:=4; order2:=3;
        A:=array(1..s,1..s,[
            [   0   ,    0   ,    0   ,   0    , 0  ],
            [  1/4  ,    0   ,    0   ,   0    , 0  ],
            [  4/81 ,  32/81 ,    0   ,   0    , 0  ], 
            [ 57/98 ,-432/343,1053/686,   0    , 0  ],
            [  1/6  ,    0   ,  27/52 , 49/156 , 0  ] ]);
        B2:=array(1..s,
            [  1/6   ,   0   ,  27/52 , 49/156 , 0  ] );
        B1:=array(1..s,
            [ 43/288 ,    0   ,243/416,343/1872,1/12] );
        C:=array(1..s,[0,1/4,4/9,6/7,1]);  
        break;
     of RKF45a do
        s:=6;order1:=4;order2:=5;
        A:=array(1..s,1..s,[
            [   0    ,    0    ,   0   ,  0   , 0   , 0  ],
            [  2/9   ,    0    ,   0   ,  0   , 0   , 0  ],
            [  1/12  ,   1/4   ,   0   ,  0   , 0   , 0  ], 
            [ 69/128 ,-243/128 ,135/64 ,  0   , 0   , 0  ],
            [-17/12  ,  27/4   ,-27/5  ,16/15 , 0   , 0  ],
            [ 65/432 ,  -5/16  , 13/16 , 4/27 ,5/144, 0  ] ]);
        B1:=array(1..s,
            [  1/9   ,    0    ,  9/20 , 16/45,1/12 , 0  ] );
        B2:=array(1..s,
            [ 47/450 ,    0    , 12/25, 32/225,1/30 ,6/25] );
        C:=array(1..s, [0,2/9,1/3,3/4,1,5/6]);  
        break;
     of RKF54a do
        s:=6;order1:=5;order2:=4;
        A:=array(1..s,1..s,[
            [   0    ,    0    ,   0   ,  0   , 0   , 0  ],
            [  2/9   ,    0    ,   0   ,  0   , 0   , 0  ],
            [  1/12  ,   1/4   ,   0   ,  0   , 0   , 0  ], 
            [ 69/128 ,-243/128 ,135/64 ,  0   , 0   , 0  ],
            [-17/12  ,  27/4   ,-27/5  ,16/15 , 0   , 0  ],
            [ 65/432 ,  -5/16  , 13/16 , 4/27 ,5/144, 0  ] ]);
        B2:=array(1..s,
            [  1/9   ,    0    ,  9/20 , 16/45,1/12 , 0  ] );
        B1:=array(1..s,
            [ 47/450 ,    0    , 12/25, 32/225,1/30 ,6/25] );
        C:=array(1..s, [0,2/9,1/3,3/4,1,5/6]);  
        break;
     of RKF45b do
        s:=6;order1:=4;order2:=5;
        A:=array(1..s,1..s,[
            [    0    ,     0    ,     0    ,    0      ,   0  , 0  ],
            [   1/4   ,     0    ,     0    ,    0      ,   0  , 0  ],
            [   3/32  ,    9/32  ,     0    ,    0      ,   0  , 0  ], 
            [1932/2197,-7200/2197, 7296/2197,    0      ,   0  , 0  ],
            [ 439/216 ,    -8    , 3680/513 ,-845/4104  ,   0  , 0  ],
            [  -8/27  ,     2    ,-3544/2565,1859/4104  ,-11/40, 0  ] ]);
        B1:=array(1..s,
            [  25/216 ,     0    , 1408/2565, 2197/4104 , -1/5 , 0  ] );
        B2:=array(1..s,
            [  16/135 ,     0    ,6656/12825,28561/56430, -9/50,2/55] );
        C:=array(1..s,[0,1/4,3/8,12/13,1,1/2]);  
        break;
     of RKF54b do
        s:=6;order1:=5;order2:=4;
        A:=array(1..s,1..s,[
            [    0    ,     0    ,     0    ,    0      ,   0  , 0  ],
            [   1/4   ,     0    ,     0    ,    0      ,   0  , 0  ],
            [   3/32  ,    9/32  ,     0    ,    0      ,   0  , 0  ], 
            [1932/2197,-7200/2197, 7296/2197,    0      ,   0  , 0  ],
            [ 439/216 ,    -8    , 3680/513 ,-845/4104  ,   0  , 0  ],
            [  -8/27  ,     2    ,-3544/2565,1859/4104  ,-11/40, 0  ] ]);
        B2:=array(1..s,
            [  25/216 ,     0    , 1408/2565, 2197/4104 , -1/5 , 0  ] );
        B1:=array(1..s,
            [  16/135 ,     0    ,6656/12825,28561/56430, -9/50,2/55] );
        C:=array(1..s,[0,1/4,3/8,12/13,1,1/2]);  
        break;
     of DOPRI45 do
        s:=7;order1:=4;order2:=5;
        A:=array(1..s,1..s,[
        [     0    ,       0   ,     0   ,     0   ,     0     ,  0  , 0 ],
        [    1/5   ,       0   ,     0   ,     0   ,     0     ,  0  , 0 ],
        [    3/40  ,     9/40  ,     0   ,     0   ,     0     ,  0  , 0 ],
        [   44/45  ,   -56/15  ,   32/9  ,     0   ,     0     ,  0  , 0 ],
        [19372/6561,-25360/2187,64448/6561,-212/729,     0     ,  0  , 0 ],
        [ 9017/3168,  -355/33  ,46732/5247,  49/176,-5103/18656,  0  , 0 ],
        [   35/384 ,      0    ,  500/1113, 125/192,-2187/6784 ,11/84, 0 ]]);
        B1:=array(1..s,
        [ 5179/57600,     0    , 7571/16695,393/640,-92097/339200,
                                                                187/2100,1/40]);
        B2:=array(1..s,
        [   35/384 ,      0    ,  500/1113, 125/192,-2187/6784 ,11/84, 0 ]);
        C:=array(1..s,[0,1/5,3/10,4/5,8/9,1,1]);  
        break;
     of DOPRI54 do
        s:=7;order1:=5;order2:=4;
        A:=array(1..s,1..s,[
        [     0    ,       0   ,     0   ,     0   ,     0     ,  0  , 0 ],
        [    1/5   ,       0   ,     0   ,     0   ,     0     ,  0  , 0 ],
        [    3/40  ,     9/40  ,     0   ,     0   ,     0     ,  0  , 0 ],
        [   44/45  ,   -56/15  ,   32/9  ,     0   ,     0     ,  0  , 0 ],
        [19372/6561,-25360/2187,64448/6561,-212/729,     0     ,  0  , 0 ],
        [ 9017/3168,  -355/33  ,46732/5247,  49/176,-5103/18656,  0  , 0 ],
        [   35/384 ,      0    ,  500/1113, 125/192,-2187/6784 ,11/84, 0 ]]);
        B2:=array(1..s,
        [ 5179/57600,     0    , 7571/16695,393/640,-92097/339200,
                                                            187/2100,1/40]);
        B1:=array(1..s,
        [   35/384 ,      0    ,  500/1113, 125/192,-2187/6784 ,11/84, 0 ]);
        C:=array(1..s,[0,1/5,3/10,4/5,8/9,1,1]);  
        break;
     of BUTCHER6 do
        s:=7;order1:=6;order2:=6;
        A:=array(1..s,1..s,[
        [   0   ,   0   ,   0   ,    0   ,   0    ,   0   ,  0 ],
        [  1/2  ,   0   ,   0   ,    0   ,   0    ,   0   ,  0 ],
        [  2/9  ,  4/9  ,   0   ,    0   ,   0    ,   0   ,  0 ],
        [  7/36 ,  2/9  , -1/12 ,    0   ,   0    ,   0   ,  0 ],
        [-35/144,-55/36 , 35/48 ,  15/8  ,   0    ,   0   ,  0 ],
        [ -1/360,-11/36 , -1/8  ,   1/2  ,  1/10  ,   0   ,  0 ],
        [-41/260, 22/13 , 43/156,-118/39 , 32/195 , 80/39 ,  0 ]]);
        B1:=array(1..s,
        [ 13/200,   0   , 11/40 ,  11/40 ,  4/25  ,  4/25 ,13/200]);
        B2:=array(1..s,
        [ 13/200,   0   , 11/40 ,  11/40 ,  4/25  ,  4/25 ,13/200]);
        C:=array(1..s,[0,1/2,2/3,1/3,5/6,1/6,1]);  
        break;
     of RKF78 do
        s:=13;order1:=7;order2:=8;
        A:=array(1..s,1..s,[
[    0   , 0  ,   0  ,    0   ,    0    ,    0  ,    0    ,  0  ,  0   ,  0  ,0,0,0],
[   2/27 , 0  ,   0  ,    0   ,    0    ,    0  ,    0    ,  0  ,  0   ,  0  ,0,0,0],
[   1/36 ,1/12,   0  ,    0   ,    0    ,    0  ,    0    ,  0  ,  0   ,  0  ,0,0,0],
[   1/24 , 0  ,  1/8 ,    0   ,    0    ,    0  ,    0    ,  0  ,  0   ,  0  ,0,0,0],
[   5/12 , 0  ,-25/16,  25/16 ,    0    ,    0  ,    0    ,  0  ,  0   ,  0  ,0,0,0],
[   1/20 , 0  ,   0  ,   1/4  ,   1/5   ,    0  ,    0    ,  0  ,  0   ,  0  ,0,0,0],
[ -25/108, 0  ,   0  , 125/108, -65/27  , 125/54,    0    ,  0  ,  0   ,  0  ,0,0,0],
[  31/300, 0  ,   0  ,    0   ,  61/225 ,  -2/9 ,  13/900 ,  0  ,  0   ,  0  ,0,0,0],
[    2   , 0  ,   0  , -53/6  , 704/45  , -107/9,  67/90  ,  3  ,  0   ,  0  ,0,0,0],
[ -91/108, 0  ,   0  ,  23/108,-976/135 , 311/54, -19/60  ,17/6 ,-1/12 ,  0  ,0,0,0],
[ 2383/4100,0 ,   0  ,-341/164,4496/1025,-301/82,2133/4100,45/82,45/164,18/41,0,0,0],
[    3/205 ,0 ,   0  ,    0   ,    0    ,  -6/41,  -3/205 ,-3/41, 3/41 , 6/41,0,0,0],
[-1777/4100,0 ,   0  ,-341/164,4496/1025,-289/82,2193/4100,51/82,33/164,12/41,0,1,0]]);
        B1:=array(1..s,[41/840,0,0,0,0,34/105,9/35,9/35,9/280,9/280,41/840,0,0]);
        B2:=array(1..s,[0,0,0,0,0,34/105,9/35,9/35,9/280,9/280,0,41/840,41/840]);
        C:=array(1..s,[0,2/27,1/9,1/6,5/12,1/2,5/6,1/6,2/3,1/3,1,0,1]);  
        break;
     of RKF87 do
        s:=13;order1:=8;order2:=7;
        A:=array(1..s,1..s,[
[    0   , 0  ,   0  ,    0   ,    0    ,    0  ,    0    ,  0  ,  0   ,  0  ,0,0,0],
[   2/27 , 0  ,   0  ,    0   ,    0    ,    0  ,    0    ,  0  ,  0   ,  0  ,0,0,0],
[   1/36 ,1/12,   0  ,    0   ,    0    ,    0  ,    0    ,  0  ,  0   ,  0  ,0,0,0],
[   1/24 , 0  ,  1/8 ,    0   ,    0    ,    0  ,    0    ,  0  ,  0   ,  0  ,0,0,0],
[   5/12 , 0  ,-25/16,  25/16 ,    0    ,    0  ,    0    ,  0  ,  0   ,  0  ,0,0,0],
[   1/20 , 0  ,   0  ,   1/4  ,   1/5   ,    0  ,    0    ,  0  ,  0   ,  0  ,0,0,0],
[ -25/108, 0  ,   0  , 125/108, -65/27  , 125/54,    0    ,  0  ,  0   ,  0  ,0,0,0],
[  31/300, 0  ,   0  ,    0   ,  61/225 ,  -2/9 ,  13/900 ,  0  ,  0   ,  0  ,0,0,0],
[    2   , 0  ,   0  , -53/6  , 704/45  , -107/9,  67/90  ,  3  ,  0   ,  0  ,0,0,0],
[ -91/108, 0  ,   0  ,  23/108,-976/135 , 311/54, -19/60  ,17/6 ,-1/12 ,  0  ,0,0,0],
[ 2383/4100,0 ,   0  ,-341/164,4496/1025,-301/82,2133/4100,45/82,45/164,18/41,0,0,0],
[    3/205 ,0 ,   0  ,    0   ,    0    ,  -6/41,  -3/205 ,-3/41, 3/41 , 6/41,0,0,0],
[-1777/4100,0 ,   0  ,-341/164,4496/1025,-289/82,2193/4100,51/82,33/164,12/41,0,1,0]]);
        B2:=array(1..s,[41/840,0,0,0,0,34/105,9/35,9/35,9/280,9/280,41/840,0,0]);
        B1:=array(1..s,[0,0,0,0,0,34/105,9/35,9/35,9/280,9/280,0,41/840,41/840]);
        C:=array(1..s,[0,2/27,1/9,1/6,5/12,1/2,5/6,1/6,2/3,1/3,1,0,1]);  
        break;
     otherwise return([NIL,NIL,NIL,NIL,NIL,NIL,NIL]);
     end_case;
     [s,C,A,B1,B2,order1,order2];
end_proc:


