
#------   file  det.mu  25.9.97    ---------------------#
#  Help file:

 det - determinant of a matrix

 Call:

 numeric::det( A <, Symbolic> <, MinorExpansion> )
                                                                   
 Parameter:

  A  --  a  square real or complex matrix of domtype DOM_ARRAY
         or Dom::Matrix               
                                                                   
Synopsis:

  numeric::det(A); returns the determinant of A. Non-float numerical
  entries of the matrix are evaluated to floats in the course of the
  computation. The algorithm is based on QR factorization using
  Householder transformations.

  Conversion to floats can be suppressed by the option Symbolic. In
  this case a symbolic LU factorization is used, non-numeric symbolic
  entries are admissible. However, the presence of such entries will
  slow down the computation drastically.

  With the option MinorExpansion recursive minor expansion along the
  first row is used. This option is useful for small dense matrices 
  with symbolic entries as well as for large sparse matrices.

  Examples: 

>> A:=array(1..3,1..3,[ [1,1,I] , [1,exp(1),I] , [1,2,2] ]):

>> numeric::det(A);
                        3.436563656 - 1.718281828 I

>> A:=array(1..3,1..3,[ [1/x,1,I/(2+x)] , [1,1,1] , [1,x,1] ]):     

>> numeric::det(A,Symbolic);
                                                 2
                        2 - (1 + I) x - (1 - I) x
                        --------------------------
                                x (x + 2)
>> numeric::det(A,MinorExpansion);

                           1     I      I x
                           - - ----- + ----- - 1
                           x   x + 2   x + 2

See also:  linalg::det

------------------------------------------------------------------#

numeric::det:=proc(A)
local n,Options,use_normal,method,B,i,j,k,result,real,
      h,s,ss,tmp,pivindex,subdet;
begin
   if A::hasProp(Cat::MatrixCat)=TRUE   # A is of domtype Dom::Matrix  #
   then A:=A::convert_to(A,DOM_ARRAY)   # convert to domtype DOM_ARRAY #
   end_if;                              # for speed #
   if domtype(A) <> DOM_ARRAY
      then error("matrix must be of domtype DOM_ARRAY or Dom::Matrix");
   end_if:
   if op(A,[0,1])<>2 then error("argument is not a matrix");end_if;
   n:=op(A,[0,2,2]):
   if n <> op(A,[0,3,2]) then error("not a square matrix"); end_if;

   use_normal:=FALSE;

   if args(0)>1 then
     Options:={args(2..args(0))};
     if has(Options,MinorExpansion) then method:=3;  # minor expansion #
     else if has(Options,Symbolic) 
             then method:=1;                # LU factorization #
          end_if;
     end_if;
     if ( Options minus {Symbolic,MinorExpansion} ) <> {}
                 then error("unknown option") end_if;
   else method:=2;                  # floats: QR factorization #
   end_if;

if method=2 then
   B:=map(A,float):
   if {op(map(B,domtype))} minus {DOM_FLOAT, DOM_COMPLEX} <> {}
   then 
   warning("symbolic entry found, switching to symbolic mode [numeric::det]");
      method:=1;
   else A:=subs(B,float(0)=0):
        if ({op(map(A,(iszero@Im)))} = {TRUE})     # is matrix real ? #
           then real:=TRUE else real:=FALSE
        end_if:
   end_if;
end_if;

if method=1 then
   userinfo(1,"computing LU factorization");
   if (indets(A)<>{}) or has({op(map(A,domtype))},DOM_NIL)
      then use_normal:=TRUE else use_normal:=FALSE;end_if;
   if use_normal then A[1,1]:=normal(A[1,1]); end_if; 
   result:=1;
   for j from 1 to n-1 do
     if iszero(A[j,j]) then
         for pivindex from j+1 to n-1 do
             if not iszero(A[pivindex,j]) then break; end_if;
         end_for;
      else pivindex:=j;
      end_if;
      if pivindex<>j then
         result:=-result;
        (tmp:=A[j,k];A[j,k]:=A[pivindex,k];A[pivindex,k]:=tmp;) $ hold(k)=1..n;
      end_if;
      if iszero(A[j,j]) then return(0) end_if;
      result:=result*A[j,j];
      if use_normal
      then for i from j+1 to n do
               tmp:=A[i,j]/A[j,j];
               if not iszero(tmp) then
               (A[i,k]:=normal(A[i,k]-tmp*A[j,k])) $ hold(k)=j+1..n;
               end_if;
           end_for;
      else for i from j+1 to n do
               tmp:=A[i,j]/A[j,j];
               if not iszero(tmp) then
               (A[i,k]:=A[i,k]-tmp*A[j,k]) $ hold(k)=j+1..n;
               end_if;
           end_for;
      end_if;
   end_for;
   result:=result*A[n,n]:
   if use_normal then return(normal(result)):
                 else return(result):
   end_if;
end_if; #method=1#

if method=2 then            
   userinfo(1,"computing QR factorization by Householder transformations");
   result:=1;
   for j from 1 to n-1 do
        # Householder step from the left to obtain upper triangular form. #
        s:= _plus(stdlib::abs(A[i,j])^2 $ hold(i)=j+1..n);
        if iszero(s) then return(0)
        else ss:=s+stdlib::abs(A[j,j])^2;
             s:=stdlib::sqrt(ss):
             h := 1 / (ss + stdlib::abs(A[j,j]) * s);
             if not iszero(A[j,j]) then s:=sign(A[j,j])*s;end_if:
             A[j,j]:=A[j,j]+s; result:=result*s;
             for i from j+1 to n do
                 if real then   s:=_plus(A[k,j] *A[k,i]$hold(k)=j..n) ;
                 else s:=_plus(conjugate(A[k,j])*A[k,i]$hold(k)=j..n) ;
                 end_if;
                 s:=s*h;
                 if not iszero(s) then
                   (A[k,i]:=A[k,i]-s*A[k,j]) $ hold(k)=j+1..n ;
                 end_if;
             end_for;
        end_if;
   end_for;
   return(subs(result*A[n,n],float(0)=0));
end_if; #method=2#

if method=3 then
   userinfo(1,"using minor expansion along first row");
   A:=subs(A,float(0)=0);

   # define local subroutine  subdet = minor #

   subdet:=proc(r,c,n) local result,i; option remember;
   begin
      if n=3 then return(A[r[1],c[1]]*A[r[2],c[2]]*A[r[3],c[3]]
                       + A[r[1],c[2]]*A[r[3],c[1]]*A[r[2],c[3]]
                       + A[r[2],c[1]]*A[r[1],c[3]]*A[r[3],c[2]]
                       - A[r[1],c[1]]*A[r[2],c[3]]*A[r[3],c[2]]
                       - A[r[1],c[2]]*A[r[2],c[1]]*A[r[3],c[3]]
                       - A[r[1],c[3]]*A[r[2],c[2]]*A[r[3],c[1]]
                        ) end_if;
      if n=2 then return(A[r[1],c[1]]*A[r[2],c[2]]
                        -A[r[1],c[2]]*A[r[2],c[1]]) end_if;
      if n=1 then return(A[r[1],c[1]]) end_if;
      result:=0;
      for i from 1 to n do
         if not iszero(A[r[i],c[1]]) then
            result:=result+(-1)^(1+i)*A[r[i],c[1]]
            *subdet(subsop(r,i=null()),subsop(c,1=null()),n-1);
         end_if;
      end_for;
      result;
  end_proc; #subdet#


# find ordering c=[c[1],..,c[n]] of the columns s.th. #
#    number of zeroes in column c[1]                  #
# >= number of zeroes in column c[2]                  #
# >=   ...                                            #
# Then expand recursively along column c[1] ...       #

 #  for j from 1 to n do                        #
 #    z:=0;                                     #
 #    for i from 1 to n do                      #
 #       if iszero(A[i,j]) then z:=z+1;end_if;  #
 #    end_for;                                  #
 #    zeroes[j]:=z;                             #
 #  end_for;                                    #
 #                                              #
 #  c:=[i $ hold(i)=1..n ];                     #
 #  s:=1;  # # (s=signum of column permutation) # 
 #                                              #
 #  for j from 2 to n do                        #
 #    z:=zeroes[j];                             #
 #    for i from j downto 2 do                  #
 #       if zeroes[i-1]>=z then break end_if;   #
 #       zeroes[i]:=zeroes[i-1];c[i]:=c[i-1];   #
 #    end_for;                                  #
 #    zeroes[i]:=z; c[i]:=j; s:=(-1)^(j-i)*s;   #
 #  end_for;                                    #
 #                                              #
 #  return(s*subdet([i$ hold(i)=1..n],c,n));    #

  return(subdet([i$ hold(i)=1..n],[j$ hold(j)=1..n],n));

end_if; #method=3#

end_proc: #det#


