
#-----------------------------------------------------------------#
# eigenvalues(A)  - numerical eigenvalues of a matrix             #
#                                                                 #
# A - a square real or complex matrix (of domtype DOM_ARRAY or    #
#                                      Dom::Matrix )              #
#                                                                 #
# return value: sorted list of numerical eigenvalues              #
#                                                                 #
# remark: Non-float numerical entries are allowed. For non-       #
#         triangular matrices such entries are converted to       #
#         floats, for triangular matrices such entries are        #
#         processed symbolically.                                 #
#         Non-numeric symbolic entries will lead to an error.     #
#                                                                 #
# The routines eigenvalues/vectors are MuPAD versions of the      #
# standard algorithms from the Handbook of Automatic Computation  #
# by Wilkinson and Reinsch. They were adapted to suit the case of #
# complex matrices.                                               #
#                                                                 #
# Example:                                                        #
#                                                                 #
#   n:=2:                                                         # 
#   Hilbert:= array(1..n,1..n,                                    # 
#                [ [1/(i+j-1) $ hold(i)=1..n] $ hold(j)=1..n ]):  #
#                                                                 #
#   eigenvalues(Hilbert);                                         #
#                   [0.06574145409, 1.267591879]                  #
#                                                                 #
# The routines eigenvalues/vectors are MuPAD versions of the      #
# standard algorithms from the Handbook of Automatic Computation  #
# by Wilkinson and Reinsch. They were adapted to suit the case of #
# complex matrices.                                               #
# Remark:  Maple uses the same (or similar) routines taken from   #
#          the Handbook, whence major parts of the routines are   #
#          identical to the Maple versions. However, the MuPAD    # 
#          version of the complex case beats Maple by far.        #
#          Also, MuPAD is much faster for high values of DIGITS.  #
#          In general, the Maple implementation is not very clean.#
#          I spotted many cases where Maple returns wrong values  #
#          due to unstable computational steps.                   #
#-----------------------------------------------------------------#


numeric::eigenvalues := proc(A)
local n, oldDIGITS,
      hermitean, real, lowertriangular, uppertriangular,
      i, j, d, e ,  # d,e are lists generated by Hermitean2tridiagonal1 #
                    # and processed in tqli1()                          #
      Hermitean2tridiag1, balance1, elmhess1, tqli1,       # local #
      hqr1, sortEigenvalues, sortEigenvalues1;        #subroutines #
begin

# ----------- define local subroutine --------------------------------#
#                                                                     #
# Hermitean2tridiag1(real)                                            #
#         - reduction of a hermitean matrix to real tri-digonal form  #      
#                                                                     #    
# real - TRUE or FALSE                                                #
#                                                                     #    
# transforms a globally defined hermitean matrix A                    #
#                                                                     #    
# returns  d = list of real diagonal elements                         #
#          e = list of real subdiagonal elements                      #
# as global variables. The matrix A is replaced by a matrix           #
# containing the Householder data of the transformation               #
#                                                                     #    
# explanation: the unitary transformation B = Q A Q^T to              #
#            [ d[1] e[1]              ]                               #
#         B= [ e[1] d[2] e[2]         ]                               #
#            [      e[2] d[3] e[3]    ]                               #
#            [            ..   ..  .. ]                               #
#         is computed via Householder transformations Q=.. H2 H1      # 
#         For complex hermitean matrix A complex phase factors is     #    
#         introduced to convert the resulting complex tridiagonal     #     
#         matrix to real tridiagonal form with positive offdiagonal   #    
#         elements. This amounts to e[i] -> abs(e[i]). For real       # 
#         symmetric matrices arbitrary signs may occur in the         #
#         offdiagonal elements                                        #
#---------------------------------------------------------------------#

Hermitean2tridiag1 := proc(real) 
local j, t, i, tt, h, k;
begin  
   e:=[ 0 $ n ]:
   if real then # A is real and symmetric #
      for j from 1 to n-2 do
         t := _plus(stdlib::abs(A[i,j])^2 $ hold(i)=j+2..n);
         if iszero(t) then e[j]:=A[j+1,j]: # Householder transformation #
                       # is trivial only have to store subdiagonal in e #
         else tt:= t+stdlib::abs(A[j+1,j])^2 ; # Householder transformation #
              t:=stdlib::sqrt(tt):
              h := 1 / (tt + stdlib::abs(A[j+1,j]) * t);
              if not iszero(A[j+1,j]) then t:= sign(A[j+1,j]) * t; end_if:
              A[j+1,j] := t + A[j+1,j];    
              ( e[i] := h * ( _plus(A[i,k]*A[k,j] $ hold(k)=j+1..i)
                            + _plus(A[k,i]*A[k,j] $ hold(k)=i+1..n))
               ) $ hold(i)=j+1..n:
              h:= h/2 * _plus(A[k,j]*e[k] $ hold(k)=j+1..n);
              ( e[i] := e[i]-h*A[i,j] ) $ hold(i)=j+1..n:
              for i from j+1 to n do
                 (A[i,k]:=A[i,k]-e[i]*A[k,j]-A[i,j]*e[k])$hold(k)=j+1..i-1: 
                 A[i,i]:=A[i,i]-2*e[i]*A[i,j]:
              end_for;
              e[j] := - t;
         end_if;       
      end_for;
      e[n-1]:=A[n,n-1];       # the last subdiagonal #
      d := [ A[j,j] $ hold(j)=1..n ]:
  else  # real = FALSE #
     for j from 1 to n-2 do
        t := _plus(stdlib::abs(A[i,j])^2 $ hold(i)=j+2..n);
        if iszero(t) then e[j]:=stdlib::abs(A[j+1,j]):
        else tt:= t+stdlib::abs(A[j+1,j])^2 ; # Householder transformation #
             t:=stdlib::sqrt(tt):
             h := 1 / (tt + stdlib::abs(A[j+1,j]) * t);
             if not iszero(A[j+1,j]) then t:= sign(A[j+1,j]) * t; end_if:
             A[j+1,j] := t + A[j+1,j];    
             ( e[i] := h * ( _plus(A[i,k]*A[k,j] $ hold(k)=j+1..i)
                + _plus(conjugate(A[k,i])*A[k,j] $ hold(k)=i+1..n))
             ) $ hold(i)=j+1..n:
             h:= h/2 * _plus(conjugate(A[k,j])*e[k] $ hold(k)=j+1..n);
             ( e[i] := e[i]-h*A[i,j] ) $ hold(i)=j+1..n:
             for i from j+1 to n do
                (A[i,k]:= A[i,k]-e[i]*conjugate(A[k,j])-A[i,j]*conjugate(e[k])
                ) $ hold(k)=j+1..i-1 :
                # avoid imaginary numerical garbage on the diagonal: #
                A[i,i]:=A[i,i]-2*Re(e[i]*conjugate(A[i,j])):
             end_for;
             e[j] := stdlib::abs(t);
          end_if; 
     end_for;
     e[n-1]:=stdlib::abs(A[n,n-1]);
     d := [ A[j,j] $ hold(j)=1..n ]:
  end_if:
  d:=map(d, Re); e:=map(e,Re); 
  null():
end_proc:

# ----------- define local subroutine ---------------------------------#
#                                                                      #
# balance1()  - balance global matrix A to improve numerical stability #
#               of eigen computations                                  #
#                                                                      #
# uses - a global square real or complex matrix A                      #
#                                                                      #
# return value: A is replaced by D A D^{-1} with a diagonal            #
#               matrix D such that columns and rows of A               #
#               are of similar magnitude                               #
#----------------------------------------------------------------------#

balance1:=proc()
local base, base2, conv, i, j, c, r, f, s, g;
begin
 base:=float(2^16): base2:=base^2: # the PARI base is 2^32. Make sure #
 while TRUE do              # that division/multiplication with base2 #
   conv:=TRUE;              # is exact                                #
   for i from 1 to n do
     c:= _plus(stdlib::abs(A[j,i])$hold(j)=1..i-1)
        +_plus(stdlib::abs(A[j,i])$hold(j)=i+1..n);
     r:= _plus(stdlib::abs(A[i,j])$hold(j)=1..i-1)
        +_plus(stdlib::abs(A[i,j])$hold(j)=i+1..n);
     f:=1; s:=c+r;
     if iszero(c) or iszero(r) then next; end_if;
     g:=r/base: if c< g then f:=f*base;c:=c*base2: end_if:  # now              #
     g:=r*base: if c>=g then f:=f/base;c:=c/base2: end_if:  # c/base<=r<base*c #
     if (c+r)<0.95*s*f then
       conv:= FALSE:
       for j from 1 to i-1 do A[i,j]:=A[i,j]/f: A[j,i]:=A[j,i]*f; end_for:
       for j from i+1 to n do A[i,j]:=A[i,j]/f: A[j,i]:=A[j,i]*f; end_for:
     end_if:
   end_for;
   if conv then break; end_if: 
 end_while;
 null():
end_proc:

# ----------- define local subroutine ---------------------------------#
#                                                                      #
# elmhess1()  - reduction of a complex matrix to Hessenberg form       #
#                                                                      #
# uses  - globally defined matrix A                                    # 
#                                                                      #
# returns  Hessenberg form stored in A                                 #
#                                                                      #  
# explanation: Hessenberg = L.(n-2) .. L.1 A L.1^(-1) .. L.(n-2)^(n-2) #
#          with elementary matrices L.1, L.2 .. from Gauss-Elimination #
#----------------------------------------------------------------------#

elmhess1:=proc()
local m,i,pivot,j,tmp,y;
begin
  for m from 2 to n-1 do
    i:=m; pivot:=0:
    for j from m to n do                   # look for Pivot element #
       if stdlib::abs(A[j,m-1])>stdlib::abs(pivot) then pivot:=A[j,m-1];i:=j; end_if:
    end_for:
    if i<>m then 
                                                 # swap row i <-> m #
       for j from m-1 to n do tmp:=A[i,j]; A[i,j]:=A[m,j]: A[m,j]:=tmp:end_for:
                                             # swap columns i <-> m #
       for j from  1  to n do tmp:=A[j,i]; A[j,i]:=A[j,m]: A[j,m]:=tmp:end_for:
    end_if:
    if not iszero(pivot) then
     for i from m+1 to n do
         y:=A[i,m-1]:
         if not iszero(y) then
            y:=y/pivot; A[i,m-1]:=y;            # do the eliminations #
            for j from m to n do A[i,j]:=A[i,j]-y*A[m,j]:end_for:
            for j from 1 to n do A[j,m]:=A[j,m]+y*A[j,i]:end_for:
         end_if:
     end_for:
    end_if:
  end_for:
  null():
end_proc:

# ----------- define local subroutine ---------------------------#
#                                                                #
# tqli1() - compute eigenvalues of real tridiagonal matrix with  #
#           the globally defined diagonal d=[d[1],d[2],..,d[n]]  #
#           and subdiagonal e=[e[1],e[2],..,e[n-1]] using the QL #
#           method with implicit spectral shifts                 #
#                                                                #  
# return value - sorted list of eigenvalues                      #
#                                                                #  
# remark - small eigenvalues are computed well                   #
#----------------------------------------------------------------#

tqli1:=proc()
local macheps, maxiter, i, ip1, l, iter, m , g , r, s, c, f, b, p;
begin
  if n = 1 then return() end_if:
  e[n]:=0:
  macheps:=float(1/10^DIGITS)/2:
  maxiter:=max(30,3*DIGITS):
  for l from 1 to n do
    iter:=0:
    repeat #(1)#
      for m from l to n-1 do 
        if stdlib::abs(e[m])<=(stdlib::abs(d[m])
                              +stdlib::abs(d[m+1]))*macheps
        then break end_if 
      end_for:
      if m <> l then 
          iter:=iter+1:
          if iter = maxiter then error("no convergence") end_if:
          g:=(d[l+1]-d[l])/(2*e[l]):
          r:=stdlib::sqrt(1.0+g^2):
          if 0<=g then g:=d[m]-d[l]+e[l]/(g+r)
                  else g:=d[m]-d[l]+e[l]/(g-r)
          end_if;
          s:=1:c:=1:p:=0;
          for i from m-1 downto l do
              ip1:=i+1:
              f:=s*e[i]:
              b:=c*e[i]:
              r:=stdlib::sqrt(f^2+g^2):
              e[ip1]:=r:
              s:=f/r:
              c:=g/r:
              g:=d[ip1]-p:
              r:=(d[i]-g)*s+2*c*b;
              p:=s*r:
              d[ip1]:=g+p:
              g:=c*r-b:
         end_for:
         d[l]:=d[l]-p:
         e[l]:=g:
         e[m]:=0:
      else break; #goto(1)#
      end_if:
    until iter=maxiter end_repeat;
 end_for;
 null();
end_proc:

# ----------- define local subroutine --------------------------#
#                                                               #
# hqr1() - compute eigenvalues of complex Hessenberg matrix by  #
#          QR iteration with explicit spectral shifts           #
#                                                               #
# return value - sorted list of eigenvalues                     #
#---------------------------------------------------------------#

hqr1:= proc()
local nn, i, its, maxits, j, k, l, m, 
      macheps, na, notlast, p, q, r, s, t, w, tmp, tmpp,
      x, y, z;
begin
    d:=[0$n]:                    # global list to store eigenvalues   #
    macheps := float(1/10^DIGITS)/2; nn:=n; t:=0;
    maxits:=max(31,3*DIGITS):
    repeat #nextw#
        its:=1; na:=nn-1; 
        while TRUE do #nextit#
            for l from nn downto 2 do #look for small subdiagonal elements#
                if stdlib::abs(A[l,l-1]) 
                   <= macheps*(stdlib::abs(A[l-1,l-1])+stdlib::abs(A[l,l])) 
                   then break;
                end_if;
            end_for;
            x := A[nn,nn];
            if l<na then l:=1; end_if;
            if l = nn             # one eigenvalue found #
               then d[nn]:=x+t;nn:=na; break
            end_if;
            y := A[na,na]; w:=A[nn,na]*A[na,nn];
            if l = na then #twow#   #2 eigenvalues found #
                p := (y-x)/2; q:=p^2+w: x:=x+t;
                tmp:= p+stdlib::sqrt(q): tmpp:= p-stdlib::sqrt(q):
                if stdlib::abs(tmpp)>stdlib::abs(tmp) then tmp:= tmpp; end_if;
                d[na]:=x+tmp: d[nn]:=x-w/tmp:
                nn := nn-2; break
            end_if;
            if its = maxits then error("no convergence") end_if;
            if its mod 10 = 0 then       # exceptional shifts #
                t := x + t;
                for i from 1 to nn do A[i,i]:=A[i,i]-x end_for;
                s := stdlib::abs(A[nn,na]) + stdlib::abs(A[na,nn-2]);
                x:=.75*s; y:=x; w:=-.4375*s^2
            end_if;
            its:=its+1;
            for m from nn-2 downto l do
                z := A[m,m]; r:=x-z; s:=y-z;
                p := (r*s-w)/A[m+1,m]+A[m,m+1];
                q := A[m+1,m+1]-z-r-s; r := A[m+2,m+1];
                s := stdlib::abs(p)+stdlib::abs(q)+stdlib::abs(r);
                p:=p/s; q:=q/s; r:=r/s;
                if (m = l) or 
                   (stdlib::abs(A[m,m-1])*(stdlib::abs(q)
                                          +stdlib::abs(r)) 
                 <= macheps*stdlib::abs(p)*(stdlib::abs(A[m-1,m-1])
                                           +stdlib::abs(z)
                                           +stdlib::abs(A[m+1,m+1])))
                   then break
                end_if;
            end_for;
            for i from m+2 to nn do A[i,i-2]:=0; end_for:
            for i from m+3 to nn do A[i,i-3]:=0; end_for:
            for k from m to na do
                notlast := bool(k <> na);
                if k <> m then
                    p := A[k,k-1]; q := A[k+1,k-1];
                    if notlast then r := A[k+2,k-1] else r := 0 end_if;
                    x := stdlib::abs(p)+stdlib::abs(q)+stdlib::abs(r);
                    if  iszero(x) then break end_if;
                    p := p/x; q := q/x; r := r/x
                end_if;
                if iszero(p) then s:=stdlib::sqrt( stdlib::abs(q)^2
                                                  +stdlib::abs(r)^2)
                else s :=sign(p)*stdlib::sqrt( stdlib::abs(p)^2
                                              +stdlib::abs(q)^2
                                              +stdlib::abs(r)^2 ):
                end_if:
                if k <> m then A[k,k-1] := - s*x
                          else if l <> m then A[k,k-1] := -A[k,k-1] end_if:
                end_if;
                p := p+s; x := p/s; y := q/s; z := r/s;
                q :=conjugate(q/p); r := conjugate(r/p);
                for j from k to nn do          # row modifications #
                    p := A[k,j]+q*A[k+1,j];
                    if notlast then p := p+r*A[k+2,j]; 
                                    A[k+2,j] := A[k+2,j]-p*z
                    end_if;
                    A[k+1,j] := A[k+1,j]-p*y; A[k,j] := A[k,j]-p*x
                end_for;
                for i from l to min(k+3,nn) do # column modifications #
                    p := x*A[i,k]+y*A[i,k+1];
                    if notlast then p:=p+z*A[i,k+2];
                                    A[i,k+2]:=A[i,k+2]-p*r
                    end_if;
                    A[i,k+1]:=A[i,k+1]-p*q; A[i,k]:=A[i,k]-p
                end_for:
            end_for:
        end_while;
    until nn < 1 end_repeat;
    null():
end_proc:

# ----------- define local subroutine -------------------------------#
#                                                                    #
# sortEigenvalues(real) - sort list of real or complex eigenvalues   #
#                                                                    #
# return value: sorted global list d = [d[1],..,d[n]] of eigenvalues #
#--------------------------------------------------------------------#

sortEigenvalues:=proc(real)
local i,j,lambda;
begin
 if real 
 then for j from 2 to n do
        lambda:=d[j];    
        for i from j downto 2 do
            if d[i-1]<lambda then break end_if:
            d[i]:=d[i-1]:
        end_for;
        d[i]:=lambda;
       end_for;
 else for j from 2 to n do
        lambda:=d[j];   # search for sorted position of eigenvalue d[j] #
        # sorting criterion: from small real part to large real part.   #
        # If tie, then from small abs(imaginary part) to large          #
        # abs(imaginary part). For complex conjugate pair take the one  #
        # with positive imaginary part first                            #
        for i from j downto 2 do
          if Re(d[i-1])<Re(lambda) then break end_if:
          if (Re(d[i-1])=Re(lambda)) then
            if stdlib::abs(Im(d[i-1]))<stdlib::abs(Im(lambda))
                                                  then break end_if:
            if stdlib::abs(Im(d[i-1]))=stdlib::abs(Im(lambda))
                      and (Im(d[i-1])>Im(lambda)) then break end_if:
          end_if:
          d[i]:=d[i-1]: 
        end_for;
        d[i]:=lambda; 
      end_for:    # eigenvalues are ordered from small to large, now #
 end_if;
end_proc;

# ----------- define local subroutine -------------------------------#
#                                                                    #
# sortEigenvalues1() - numerical eigenvalues of a globally           #
#                      defined triangular matrix A                   #
#                                                                    #
# return value: sorted global list d = [d[1],..,d[n]] of eigenvalues #
#                                                                    #
# remark: same as sortEigenvalues, but non-float entries are allowed #
#--------------------------------------------------------------------#

sortEigenvalues1:=proc()
local i,j,lambda,fl,fd;
begin
  d:=[ A[i,i] $ hold(i)=1..n ]:                       # eigenvalues #
  for j from 2 to n do
    lambda:=d[j]; 
    fl:=float(lambda);        # non-float entries are allowed here! #
    for i from j downto 2 do
       fd:=float(d[i-1]):
       if Re(fd)<Re(fl) then break end_if:
       if (Re(fd)=Re(fl)) then
          if abs(Im(fd))<abs(Im(fl)) then break end_if:
          if abs(Im(fd))=abs(Im(fl)) and (Im(fd)>Im(fl)) 
                                     then break end_if:
       end_if:
       d[i]:=d[i-1]:
    end_for;
    d[i]:=lambda;
  end_for:   
  null():
end_proc;

#-----------------------------------------------------------#
#  the body of Eigenvalues using the subroutines above:     #
#-----------------------------------------------------------#

   if A::hasProp(Cat::MatrixCat)=TRUE    # A is of domtype Dom::Matrix  #
      then A:=A::convert_to(A,DOM_ARRAY) # convert to domtype DOM_ARRAY #
   end_if;                               # for speed #
   if domtype(A) <> DOM_ARRAY 
     then error("matrix must be of domtype DOM_ARRAY or Dom::Matrix");
   end_if:
   if op(A,[0,1]) <> 2 then error("first argument is not a matrix"); end_if;
   n:=op(A,[0,2,2]):
   if n <> op(A,[0,3,2]) then error("not a square matrix"); end_if;
   oldDIGITS:=DIGITS: DIGITS := DIGITS + trunc(n/10);
   hermitean := TRUE;                # is matrix hermitean ?        #
   real      := TRUE;                # is matrix real ?             #
   lowertriangular := TRUE:          # is matrix lower triangular ? #
   uppertriangular := TRUE:          # is matrix upper triangular ? #
   for i from 1 to n do              # find properties of matrix    #
    for j from 1 to i-1 do           # (hermitean/real/triangular)  #
      if  (not iszero(Im(A[i,j]))) or (not iszero(Im(A[j,i])))
                                   then     real := FALSE end_if:
      if A[i,j]<>conjugate(A[j,i]) then hermitean:= FALSE end_if:
      if not iszero(A[j,i]) then lowertriangular := FALSE end_if:
      if not iszero(A[i,j]) then uppertriangular := FALSE end_if:
    end_for:
    if (not iszero(Im(A[i,i]))) then real:=FALSE; hermitean:= FALSE
    end_if:
   end_for:

   if lowertriangular or uppertriangular then    # triangular case #
      sortEigenvalues1(); DIGITS:=oldDIGITS: return(d):
   end_if:
       # so far symbolic entries could be handled efficiently, now #
       # switch to floats                                          #
   A:=map(A,float):
   if {op(map(A,domtype))} minus {DOM_FLOAT, DOM_COMPLEX} <> {}  
      then error("non-numeric entries not handled"); 
   end_if: 
   A:=subs(A,float(0)=0):
   if hermitean then                              # hermitean case #
     Hermitean2tridiag1(real):    # transform to tridiagonal form  #
     tqli1():                     # QL-iteration without eigenvect #
     sortEigenvalues(TRUE):       # sort real (=TRUE) eigenvalues  #
   else                                             # general case #
     balance1():          # balance matrix for numerical stability #
     elmhess1():                    # transform to Hessenberg form #
     hqr1();            # compute eigenvalues of Hessenberg matrix #   
     sortEigenvalues(FALSE);      # sort complex (real=FALSE) data #
   end_if:
   DIGITS:=oldDIGITS:
   d;
end_proc:

#--------  end of procedure eigenvalues ------------------#

