#-----------------------------------------------------------------#
# eigenvectors(A) - numerical eigenvalues and vectors of a matrix #
#                                                                 #
# A - a square real or complex matrix (of domtype DOM_ARRAY or    #
#                                      Dom::Matrix )              #
#                                                                 #
# return value: list [ d, X , residues ]                          #
#                                                                 #
#         The sorted list d=[ d[1],d[2],... ] contains the        #
#         eigenvalues.                                            #
#                                                                 #
#         X is the matrix of eigenvectors, i.e., the i.th         #
#         column of X is a normalized numerical eigenvector       #
#         corresponding to the eigenvalue d[i].                   #
#         X is of domtype Dom::Matrix, whence columns can be      #
#         extracted easily via                                    #
#               eigenvector.i := X::col(X,i);                     #
#                                                                 #
#         The list residues = [ res[1],res[2],... ] contains the  #
#         residues res[i] = norm(A*x[i]-d[i]*x[i])/norm(x[i]),    #
#         where x[i] is the i.th column of X. For hermitean       #
#         matrices res[i] provides an estimate for the absolute   #
#         error of the approximative eigenvalue d[i].             #
#                                                                 #
# remark: Non-float numerical entries are allowed. For non-       #
#         triangular matrices such entries are converted to       #
#         floats, for triangular matrices such entries are        #
#         processed symbolically.                                 #
#         Non-numeric symbolic entries will lead to an error.     #
#         For matrices with multiple eigenvalues and an           #
#         insufficient number of eigenvectors some of the         #
#         returned eigenvectors may coincicde or may be zero !    #
#                                                                 #
# The routines eigenvalues/vectors are MuPAD versions of the      #
# standard algorithms from the Handbook of Automatic Computation  #
# by Wilkinson and Reinsch. They were adapted to suit the case of #
# complex matrices.                                               #
#                                                                 #
# Example:                                                        #
#                                                                 #
#   n:=2:                                                         # 
#   Hilbert:= array(1..n,1..n,                                    # 
#                [ [1/(i+j-1) $ hold(i)=1..n] $ hold(j)=1..n ]):  #
#   data:=eigenvectors(Hilbert);                                  #
# --                             +-                           -+  #
# |                              | 0.4718579255, -0.8816745987 |  #
# | [0.06574145409, 1.267591879],|                             |, #
# |                              |-0.8816745987, -0.4718579255 |  #
# --                             +-                           -+  #
#                                      --                         #
#                                       |                         #
#   [3.965706584e-20, 5.421010862e-20]  |                         #
#                                       |                         #
#                                      --                         #
#   eigenvalues:=op(data,1);                                      # 
#                    [0.06574145409, 1.267591879]                 #
#                                                                 #
#   eigenvectors:=op(data,2);                                     #
#                    +-                              -+           #
#                    |   0.4718579255, -0.8816745987  |           #
#                    |                                |           #
#                    |  -0.8816745987, -0.4718579255  |           #
#                    +-                              -+           #
#                                                                 #
#    errors:= op(data,3);                                         # 
#                    [3.965706584e-20, 5.421010862e-20]           #
#                                                                 #
#    first_eigenvector:=eigenvectors::col(eigenvectors,1);        #
#                    +-               -+                          #
#                    |   0.4718579255  |                          #
#                    |                 |                          #
#                    |  -0.8816745987  |                          #
#                    +-               -+                          #
#                                                                 #
# The routines eigenvalues/vectors are MuPAD versions of the      #
# standard algorithms from the Handbook of Automatic Computation  #
# by Wilkinson and Reinsch. They were adapted to suit the case of #
# complex matrices.                                               #
# Remark:  Maple uses the same (or similar) routines taken from   #
#          the Handbook, whence major parts of the routines are   #
#          identical to the Maple versions. However, the MuPAD    # 
#          version of the complex case beats Maple by far.        #
#          Also, MuPAD is much faster for high values of DIGITS.  #
#          In general, the Maple implementation is not very clean.#
#          I spotted many cases where Maple returns wrong values  #
#          due to unstable computational steps.                   #
#-----------------------------------------------------------------#

numeric::eigenvectors := proc(A)
local n, oldDIGITS, oldA, M,
      hermitean, real, lowertriangular, uppertriangular,
      i, j, d, e, phase, X , scale, perm, errors ,  
                 # d,e,phase,scale,perm  are lists generated in   #
                 # balance/Hermitean2tridiagonal/elmhess. X is    #
                 # the eigenvector matrix generated in tqli2/hqr2.#
     TriangularEigendata, Hermitean2tridiag2, tqli2,  # these are #
     QTXbyHouseholderData, balance2, elmhess2, hqr2,  # local     #
     backtransform, sortEigendata, sortEigendata1,    # sub-      #
     normalizeEigenvectors, normalizeEigenvectors1,   # routines  #
     residues, residues1;
begin

# ----------- define local subroutine --------------------------- #
#TriangularEigendata(lower,upper) - numerical eigenvalues and     #
#      eigenvectors of a globally defined triangular matrix A     #
#                                                                 #
#lower - TRUE or FALSE                                            #
#upper - TRUE or FALSE                                            #
#                                                                 #
#return value: global list d = [d[1],..,d[n]] of eigenvalues      #
#              and global matrix X.  The i.th column of X is a    #
#              (numerical) eigenvector of A corresponding to the  #
#              eigenvalue d[i]. For a multiple eigenvalue with    #
#              an insufficient number of eigenvectors some of     #
#              the returned eigenvectors are zero.                #
#-----------------------------------------------------------------#
TriangularEigendata:=proc(lower,upper)
local i,j,k,y,exists,t1,t2;
begin  
  d:=[ A[i,i] $ hold(i)=1..n ]:                      # eigenvalues #
  X:=array(1..n,1..n,[[0 $n]$n]):        # initialize eigenvectors #
  if lower and upper then                 # the matrix is diagonal #
     ( X[i,i]:=1 ) $ hold(i)=1..n;  return() end_if;
  if upper then                   # the matrix is upper triangular #
    for j from 1 to n do
      y[j]:=1; exists:=TRUE;
      for i from j-1 downto 1 do
        t1:=_plus( A[i,k]*y[k] $ hold(k)=i+1..j ):
        t2:=A[j,j]-A[i,i];
        if iszero(t2)        # multiple eigenvalue A[j,j] = A[i,i] # 
        then if iszero(t1) 
               then y[i]:=0       # a further eigenvector is found #
               else exists:=FALSE; break; # no further eigenvector #
             end_if:                      # exists                 #
        else y[i]:=t1/t2:
        end_if;
      end_for:
      if exists   # put j.th eigenvector (y[1],..,y[j],0,..,0) as  #
                  # column j of eigenvectormatrix X, else return 0 #
                  # vector if no further eigenvector was found for #
                  # multiple eigenvalue A[j,j]                     #
      then (X[i,j] := y[i]) $ hold(i)=1..j; end_if:
    end_for:
    return();
 end_if:
 if lower then                    # the matrix is lower triangular #
    for j from n downto 1 do
      y[j]:=1; exists:=TRUE;
      for i from j+1 to n do
        t1:=_plus( A[i,k]*y[k] $ hold(k)=j..i-1 ):
        t2:=A[j,j]-A[i,i];
        if iszero(t2)        # multiple eigenvalue A[j,j] = A[i,i] # 
        then if iszero(t1) 
               then y[i]:=0       # a further eigenvector is found #
               else exists:=FALSE; break; # no further eigenvector #
             end_if:                      # exists                 #
        else y[i]:=t1/t2:
        end_if;
      end_for:
      if exists   # put j.th eigenvector (0,..,0,y[j],..,y[n]) as  #
                  # column j of eigenvectormatrix X, else return 0 #
                  # vector if no further eigenvector was found for #
                  # multiple eigenvalue A[j,j]                     #
      then (X[i,j] := y[i]) $ hold(i)=j..n; end_if:
    end_for:
    return();
 end_if:
 null();
end_proc:

# ----------- define local subroutine ---------------------------- #
# Hermitean2tridiag2(real)  - reduction of a hermitean matrix      #
#                             to real tri-digonal form             #
#                                                                  #  
# real - TRUE or FALSE                                             #
#                                                                  #
# transforms a globally defined hermitean matrix A                 #
#                                                                  #
# returns  d = list of real diagonal elements                      #
#          e = list of real subdiagonal elements                   #
# as global variables. The matrix A is replaced by a matrix        #
# containing the Householder data of the transformation            #
#                                                                  #
# explanation: the unitary transformation B = Q A Q^T to           #
#           [ d[1] e[1]              ]                             #
#        B= [ e[1] d[2] e[2]         ]                             #
#           [      e[2] d[3] e[3]    ]                             #
#           [            ..   ..  .. ]                             #
#        is computed via Householder transformations Q=.. H2 H1    #
#        For complex hermitean matrix A complex phase factors are  #
#        introduced to convert the resulting complex tridiagonal   #
#        matrix to real tridiagonal form with positive offdiagonal #
#        elements. For real symmetric matrices arbitrary signs     #
#        may occur in the offdiagonal elements                     #
#------------------------------------------------------------------#

Hermitean2tridiag2 := proc(real) 
local i, j, k, t, tt, h;
begin  
   e:=[ 0 $ n ]:
   if real then # A is real and symmetric #
      for j from 1 to n-2 do
         t := _plus(stdlib::abs(A[i,j])^2 $ hold(i)=j+2..n);
         if iszero(t) then e[j]:=A[j+1,j]: # Householder transformation #
                       # is trivial only have to store subdiagonal in e #
         else tt:= t+stdlib::abs(A[j+1,j])^2 ; # Householder transformation #
              t:=stdlib::sqrt(tt):
              h := 1 / (tt + stdlib::abs(A[j+1,j]) * t);
              if not iszero(A[j+1,j]) then t:= sign(A[j+1,j]) * t; end_if:
              A[j+1,j] := t + A[j+1,j];    
              ( e[i] := h * ( _plus(A[i,k]*A[k,j] $ hold(k)=j+1..i)
                            + _plus(A[k,i]*A[k,j] $ hold(k)=i+1..n))
               ) $ hold(i)=j+1..n:
              h:= h/2 * _plus(A[k,j]*e[k] $ hold(k)=j+1..n);
              ( e[i] := e[i]-h*A[i,j] ) $ hold(i)=j+1..n:
              for i from j+1 to n do
                 (A[i,k]:=A[i,k]-e[i]*A[k,j]-A[i,j]*e[k])$hold(k)=j+1..i-1: 
                 A[i,i]:=A[i,i]-2*e[i]*A[i,j]:
              end_for;
              e[j] := - t;
         end_if;       
      end_for;
      e[n-1]:=A[n,n-1];       # the last subdiagonal #
      d := [ A[j,j] $ hold(j)=1..n ]:
  else  # real = FALSE #
     for j from 1 to n-2 do
        t := _plus(stdlib::abs(A[i,j])^2 $ hold(i)=j+2..n);
        if iszero(t) then e[j]:=A[j+1,j]:
        else tt:= t+stdlib::abs(A[j+1,j])^2 ; # Householder transformation #
             t:=stdlib::sqrt(tt):
             h := 1 / (tt + stdlib::abs(A[j+1,j]) * t);
             if not iszero(A[j+1,j]) then t:= sign(A[j+1,j]) * t; end_if:
             A[j+1,j] := t + A[j+1,j];    
             ( e[i] := h * ( _plus(A[i,k]*A[k,j] $ hold(k)=j+1..i)
                + _plus(conjugate(A[k,i])*A[k,j] $ hold(k)=i+1..n))
             ) $ hold(i)=j+1..n:
             h:= h/2 * _plus(conjugate(A[k,j])*e[k] $ hold(k)=j+1..n);
             ( e[i] := e[i]-h*A[i,j] ) $ hold(i)=j+1..n:
             for i from j+1 to n do
                (A[i,k]:= A[i,k]-e[i]*conjugate(A[k,j])-A[i,j]*conjugate(e[k])
                ) $ hold(k)=j+1..i-1 :
                # avoid imaginary numerical garbage on the diagonal: #
                A[i,i]:=A[i,i]-2*Re(e[i]*conjugate(A[i,j])):
             end_for;
             e[j] := - t;
          end_if; 
     end_for;
     e[n-1]:=A[n,n-1];
     d := [ A[j,j] $ hold(j)=1..n ]:

   # now [d[1],..,d[n]] and [e[1],..,e[n-1]] are the diagonal and
     offdiagonal of a complex tridiagonal matrix B=QAQ^T. Apply
     further transformation with a phase matrix 
            P=diag(phase[1],..,phase[n-1],1)
     to obtain real tridiagonal form PQAQ^TP^T. #

     phase[n]:=1:
     for j from n-1 downto 1 do
        if iszero(e[j]) then phase[j]:=1
                        else phase[j]:=phase[j+1]*sign(e[j]):
        end_if: 
        e[j]:=stdlib::abs(e[j]):
     end_for:
  end_if:
  d:=map(d, Re); e:=map(e,Re);
  null():
end_proc:

# ----------- define local subroutine -------------------------#
# tqli2() - compute eigenvalues and eigenvectors of real       #
#           tridiagonal matrix with the globally defined       #
#           diagonal d=[d[1],..,d[n]] and subdiagonal          #
#           e=[e[1],..,e[n-1]] using the QL iteration with     #
#           implicit spectral shifts. The eigenvectors are     #
#           stored in a globally defined matrix X              #
#                                                              #
# return value - list of eigenvalues stored in global list d   #
#               matrix X of eigenvectors                       #
#--------------------------------------------------------------#

tqli2:=proc()
local i, macheps, ip1, l, iter, maxiter, m, g, r, s, c, f, b,
      p, k, tmp;
begin
  if n=1 then X:=array(1..n,1..n):X[1,1]:=1:return():end_if:
  X:=array(1..n,1..n,[[float(0)$n]$n]): # initialize eigenvectors #
  (X[i,i]:=float(1)) $ hold(i)=1..n:    # as identity matrix      #
  e[n]:=0:
  macheps:=float(1/10^DIGITS)/2:
  maxiter:=max(30,3*DIGITS):
  for l from 1 to n do
    iter:=0:
    repeat #(1)#
      for m from l to n-1 do    # look for a single small subdiagonal #
                                # element to split the matrix         #
        if stdlib::abs(e[m])
            <=(stdlib::abs(d[m])+stdlib::abs(d[m+1]))*macheps
        then break end_if 
      end_for:
      if m <> l then 
          iter:=iter+1:
          if iter = maxiter then error("no convergence") end_if:
          g:=(d[l+1]-d[l])/(2*e[l]):     # form implicit shift #
          r:=stdlib::sqrt(1.0+g^2):
          if 0<=g then g:=d[m]-d[l]+e[l]/(g+r)  # this is d[m]-shift #
                  else g:=d[m]-d[l]+e[l]/(g-r)  # this is d[m]-shift #
          end_if;
          s:=1:c:=1:p:=0;
          for i from m-1 downto l do
              ip1:=i+1;
              f:=s*e[i]:        # a plane rotation as in the orig.  #
              b:=c*e[i]:        # QL, followed by Givens rotation   #
              r:=stdlib::sqrt(f^2+g^2): # to restore tridiagonal form #
              e[ip1]:=r:
              s:=f/r:
              c:=g/r:
              g:=d[ip1]-p:
              r:=(d[i]-g)*s+2*c*b;
              p:=s*r:
              d[ip1]:=g+p:
              g:=c*r-b:
                                               # update eigenvectors #
              for k from 1 to n do 
                 tmp:=X[k,i]:
                 X[k,i  ]:=c*tmp-s*X[k,ip1]:
                 X[k,ip1]:=s*tmp+c*X[k,ip1]:
              end_for: 
         end_for:
         d[l]:=d[l]-p:
         e[l]:=g:
         e[m]:=0:
      else break; #goto(1)#
      end_if:
    until iter=maxiter end_repeat;
  end_for;
  null();
end_proc:

# ----------- define local subroutine -----------------------------------#
#                                                                        #
# QTXbyHouseholderData(real)  - multiplies globally defined matrix X     #
#           from the left with a unitary matrix Q^T stored in globally   #
#           defined A by Householder data                                #
#                                                                        #
# real - TRUE or FALSE                                                   #
#                                                                        #
# uses - an arbitrary globally defined square matrix X                   #
#       -a globally defined matrix A containing Householder data         #
#                                                                        #
# return value: Q^T X                                                    #
#                                                                        #
# explanation: after Hermitean2tridiag2() or General2Hessenberg(real)    #
#              (transforming a matrix A=old_A to tridiagonal/Hessenberg  #
#              form) the global matrix A contains the data of Householder#
#              matrices H1,H2,..  The global list phase represents a     #
#              diagonal phase matrix P is generated, such that           #
#                P (.. H2 H1 oldA H1 H2 .. ) P^T =: B =                  #
#                                       real tridiagonal or Hessenberg,  #
#              i.e. oldA = H1 H2 .. P^T B P .. H2 H1                     #
#                          |___ Q^T___|   |__ Q ___|                     #
#              QTXbyHouseholderData(X,real) returns H1 H2 .. P^T X       #
#              for an arbitrary matrix X.                                #
#              If X represents the eigenvectors of B then this is the    #
#              matrix of eigenvectors of oldA                            #
#------------------------------------------------------------------------#

QTXbyHouseholderData:=proc(real)
  local i,j,k,h,vX;
begin
   if hermitean and (not real) then
     for k from 1 to n-1 do
       (X[k,j]:=X[k,j]/phase[k]) $ hold(j)=1..n;
     end_for:
   end_if:
   for j from n-2 downto 1 do
      h:= _plus( stdlib::abs(A[k,j])^2 $ hold(k)=j+2.. n) :
      if not iszero(h) then
        h:= (h+stdlib::abs(A[j+1,j])^2)/2;
        if real then 
             for i from 1 to n do
                vX:=_plus(A[k,j]*X[k,i] $ hold(k)=j+1..n)/h:
                (X[k,i]:=X[k,i]-vX*A[k,j]) $ hold(k)=j+1..n;
             end_for:
        else
             for i from 1 to n do
               vX:=_plus(conjugate(A[k,j])*X[k,i] $ hold(k)=j+1..n)/h:
               (X[k,i]:=X[k,i]-vX*A[k,j]) $ hold(k)=j+1..n;
             end_for:
        end_if:
      end_if:
   end_for:
   null()
end_proc:

# ----------- define local subroutine -------------------------------#
#                                                                    #
# balance2()  - balance global matrix A                              #
#                                                                    #
# uses A - a square real or complex matrix                           #
#                                                                    #
# return value: A is replaced by D^{-1} A D with a diagonal          #
#               matrix D such that columns and rows of A             #
#               are of similar magnitude. The entries of             #
#               D=diag(scale[1],..,scale[n]) are returned            #
#               as global list scale.                                #
#--------------------------------------------------------------------#

balance2:=proc()
local base, base2, conv, i, j, c, r, f, s, g;
begin
 base:=float(2^16): base2:=base^2:
        # the PARI base is 2^32. Make sure that division/multiplication #
        # with base2 is exact                                           #
 scale:=[ float(1) $n ]: # initialize list to store the balance factors # 
 while TRUE do 
   conv:=TRUE;  
   for i from 1 to n do
     c:=_plus(stdlib::abs(A[j,i])$hold(j)=1..i-1)
       +_plus(stdlib::abs(A[j,i])$hold(j)=i+1..n);
     r:=_plus(stdlib::abs(A[i,j])$hold(j)=1..i-1)
       +_plus(stdlib::abs(A[i,j])$hold(j)=i+1..n);
     f:=1; s:=c+r;
     if iszero(c) or iszero(r) then next; end_if;
     g:=r/base: if c< g then f:=f*base;c:=c*base2: end_if:  # now              #
     g:=r*base: if c>=g then f:=f/base;c:=c/base2: end_if:  # c/base<=r<base*c #
     if (c+r)<0.95*s*f then
       scale[i]:=scale[i]*f; conv:= FALSE:
       for j from 1 to i-1 do A[i,j]:=A[i,j]/f: A[j,i]:=A[j,i]*f; end_for:
       for j from i+1 to n do A[i,j]:=A[i,j]/f: A[j,i]:=A[j,i]*f; end_for:
     end_if:
   end_for;
   if conv then break; end_if: 
 end_while;
 null();
end_proc:

# ----------- define local subroutine -----------------------------------#
#                                                                        #
# elmhess2()  - reduction of a complex matrix to Hessenberg form         #
#                                                                        #
# uses  - globally defined matrix A                                      #
#                                                                        #
# returns  Hessenberg form stored in A, global list perm containing the  #
#          permutations used in pivoting steps                           #
#                                                                        #
# explanation: Hessenberg = L.(n-2) .. L.1 A L.1^(-1) .. L.(n-2)^(n-2)   #
#          with elementary matrices L.1, L.2 .. from Gauss-Elimination   #
#------------------------------------------------------------------------#

elmhess2:=proc()
local m,i,x,j,tmp,y;
begin 
  perm:=[i $ hold(i)=1..n]:
  for m from 2 to n-1 do
    i:=m; x:=0:
    for j from m to n do                   # look for Pivot element #
       if stdlib::abs(A[j,m-1])>stdlib::abs(x) then x:=A[j,m-1];i:=j; end_if:
    end_for:
    perm[m]:=i:
    if i<>m then 
                                                 # swap row i <-> m #
       for j from m-1 to n do tmp:=A[i,j]: A[i,j]:=A[m,j]: A[m,j]:=tmp:end_for:
                                              # swap column i <-> m #
       for j from  1  to n do tmp:=A[j,i]: A[j,i]:=A[j,m]: A[j,m]:=tmp:end_for:
    end_if:
    if not iszero(x) then
     for i from m+1 to n do
         y:=A[i,m-1]:
         if not iszero(y) then
            y:=y/x; A[i,m-1]:=y;                # do the eliminations #
            for j from m to n do A[i,j]:=A[i,j]-y*A[m,j]:end_for:
            for j from 1 to n do A[j,m]:=A[j,m]+y*A[j,i]:end_for:
         end_if:
     end_for:
    end_if:
  end_for:
  null():
end_proc:

# ----------- define local subroutine --------------------------#
#                                                               #
# hqr2() - compute eigenvalues and eigenvectors of complex      #
#          Hessenberg matrix by QR iteration with explicit      #
#          spectral shifts                                      #
#                                                               #
# return value - global list d = list of eigenvalues,           #
#                global matrix X , i.th column is numerical     #
#                eigenvectors corresponding to eigenvalue d[i]  #
#---------------------------------------------------------------#

hqr2:= proc()
local nn, i, its, maxits, j, k, l, m,  ssub, sssub ,
      macheps, na, notlast, p, q, r, t, w, tmp, tmpp,
      x, y, z, s, c, cs, b, QT, vec, exists ;
begin
    d:=[0$n]:                              # list to store eigenvalues  #
    ( ssub[j]:=A[j+2,j])$hold(j)=1..n-2;   # save subdiagonals, they    #
    (sssub[j]:=A[j+3,j])$hold(j)=1..n-3;   # need to be restored later  #
    QT:=array(1..n,1..n,[[float(0)$n]$n]): # initialize unitary transformation # 
    (QT[i,i]:=float(1)) $ hold(i)=1..n:    # matrix QT as identity matrix      #
    macheps := float(1/10^DIGITS)/2; nn:=n; t:=0;
    maxits:=max(31,3*DIGITS);
    repeat #nextw#
        its:=1; na:=nn-1; 
        while TRUE do #nextit#
            for l from nn downto 2 do #look for small subdiagonal elements#
                if stdlib::abs(A[l,l-1])
                   <= macheps*(stdlib::abs(A[l-1,l-1])+stdlib::abs(A[l,l])) 
                   then break;
                end_if;
            end_for;
            x := A[nn,nn];
            if l <na then l:=1; end_if;
            if l = nn then                  # one eigenvalue found #
                 A[nn,nn]:=x+t;nn:=na; break end_if;
            y := A[na,na]; w:=A[nn,na]*A[na,nn];
            if (l = na) then           #twow#   #2 eigenvalues found #
                p := (y-x)/2; q:=p^2+w: x:=x+t;
                tmp:= p+stdlib::sqrt(q): tmpp:= p-stdlib::sqrt(q):
                if stdlib::abs(tmpp)>stdlib::abs(tmp) then tmp:=tmpp; end_if;
                b:=A[na,nn]:
                if iszero(b) 
                   then s:=1: c:=0;
                        A[na,na]:=x: A[nn,nn]:=y+t: A[na,nn]:=-A[nn,na]:
                   else if not iszero(tmp) 
                           then tmpp:=stdlib::sqrt( stdlib::abs(b)^2
                                                   +stdlib::abs(tmp)^2 );
                                s:=tmp/sign(b)/tmpp: c:=stdlib::abs(b)/tmpp:
                                A[nn,nn]:=x+tmp: A[na,na]:=x-w/tmp:
                                A[na,nn]:=A[na,nn]-A[nn,na]/sign(s)^2:
                           else s:=0: c:=1:
                        end_if;
                end_if;
                cs:=conjugate(s):
                for i from 1 to na-1 do 
                     tmp:=A[i,na];
                     A[i,na]:=  c*tmp - s*A[i,nn]:
                     A[i,nn]:= cs*tmp + c*A[i,nn]:
                end_for:   # columns na and nn=na+1 modified in A #
                for j from nn+1 to n do
                     tmp:=A[na,j]:
                     A[na,j] := c*tmp - cs*A[nn,j]:
                     A[nn,j] := s*tmp + c *A[nn,j]:
                end_for:      # rows na and nn=na+1 modified in A #
                for i from 1 to n do
                     tmp:=QT[i,na]:
                     QT[i,na]:=  c*tmp - s*QT[i,nn]:
                     QT[i,nn]:= cs*tmp + c*QT[i,nn]:
                end_for:  # columns na and nn=na+1 modified in QT #
                
     #----------------------------------------------------------------------#
     # explanation of the previous steps:                                   #
     # consider block  (A[na,na] A[na,nn] ) = ( y  b )                      #
     #                 (A[ne,na] A[nn,nn] )   ( a  x )                      #
     # Apply unitary Givens rotation to eliminate lower entry a:            #
     #    ( c -s* )( y b )( c* s*) = (yy bb ) = ( c -s* )( yc*-bs  ys*+bc ) #
     #    ( s  c* )( a x )( -s c )   (aa xx)    ( s  c* )( ac*-xs  as*+xc ) #
     #                =  ( ycc* +xss* - bcs - ac*s* , bcc-as*s* +(y-x)cs* ) #
     #                   (-bss +ac*c* + (y-x) sc*  , yss*+xss* +bcs+ac*s* ) #
     # with  cc*+ss*=1. To eliminate aa, put c=c*=real, s= tc, i.e.         #
     #         -b t^2 + (y-x) t + a = 0 , i.e.  (bt)^2 -(y-x)bt-ab = 0      #
     # Take t= (stable) larger solution of this quadratic equation, then    #
     #           c:= 1/sqrt(1+|t|^2) , s:= t/sqrt(1+|t|^2).                 #
     # One obtains                                                          #
     #  yy:=(y+x|t|^2-bt-at*)/(1+|t|^2) = x-a/t                             #
     #  xx:=(y|t|^2+x+bt+at*)/(1+|t|^2) = x+b*t                             #
     #  bb:=(b-a(t*)^2+(y-x)t*)/(1+|t|^2) = b-at*/t                         #
     #  aa:=  0                                                             #
     # or, after rescaling t -> T=bt, w=ab , p=(y-x)/2:                     #
     #                                                                      #
     #   solve    T^2-(y-x)T-w=0, i.e.  T= p +- sqrt(p^2+w)                 #
     #   yy:= x - ((y-x)|b|^2+|b|^2T+wT*)/(|b|^2+|T|^2) = x - w/T           #
     #   xx:= x + ((y-x)|b|^2+|b|^2T+wT*)/(|b|^2+|T|^2) = x + T             #
     #   bb:= b-as*/s = b - a/sign(s)^2                                     #
     #   c := |b|/sqrt(|b|^2+|T|^2)                                         #
     #   s :=  T/sign(b)/sqrt(|b|^2+|T|^2)                                  #
     #----------------------------------------------------------------------#

                nn := nn-2; break
            end_if;
            if its = maxits then error("no convergence") end_if;
            if its mod 10 = 0 then       # exceptional shifts #
                t := x + t;
                for i from 1 to nn do A[i,i]:=A[i,i]-x end_for;
                s := stdlib::abs(A[nn,na]) + stdlib::abs(A[na,nn-2]);
                x:=.75*s; y:=x; w:=-.4375*s^2
            end_if;
            its:=its+1;
            for m from nn-2 downto l do        # double QR steps #
                z := A[m,m]; r:=x-z; s:=y-z;
                p := (r*s-w)/A[m+1,m]+A[m,m+1];
                q := A[m+1,m+1]-z-r-s; r := A[m+2,m+1];
                s := stdlib::abs(p)+stdlib::abs(q)+stdlib::abs(r);
                p:=p/s; q:=q/s; r:=r/s;
                if (m = l) or 
                    (stdlib::abs(A[m,m-1])*(stdlib::abs(q)+stdlib::abs(r)) 
                 <= macheps*stdlib::abs(p)*(stdlib::abs(A[m-1,m-1])
                                           +stdlib::abs(z)
                                           +stdlib::abs(A[m+1,m+1])))
                   then break
                end_if;
            end_for;
            for i from m+2 to nn do A[i,i-2]:=0; end_for:
            for i from m+3 to nn do A[i,i-3]:=0; end_for:
            for k from m to na do
                notlast := bool(k <> na);
                if k <> m then
                    p := A[k,k-1]; q := A[k+1,k-1];
                    if notlast then r := A[k+2,k-1] else r := 0 end_if;
                    x := stdlib::abs(p)+stdlib::abs(q)+stdlib::abs(r);
                    if  iszero(x) then break end_if;
                    p := p/x; q := q/x; r := r/x
                end_if;
                if iszero(p) then s:=stdlib::sqrt(stdlib::abs(q)^2
                                                 +stdlib::abs(r)^2)
                else s :=sign(p)*stdlib::sqrt( stdlib::abs(p)^2
                                              +stdlib::abs(q)^2
                                              +stdlib::abs(r)^2 ):
                end_if:
                if k <> m then A[k,k-1] := - s*x
                          else if l <> m then A[k,k-1] := -A[k,k-1] end_if:
                end_if;
                p := p+s; x := p/s; y := q/s; z := r/s;
                q :=conjugate(q/p); r := conjugate(r/p);
                for j from k to n do          # row modifications #
                    p := A[k,j]+q*A[k+1,j];
                    if notlast then p := p+r*A[k+2,j]; 
                                    A[k+2,j] := A[k+2,j]-p*z
                    end_if;
                    A[k+1,j] := A[k+1,j]-p*y; A[k,j] := A[k,j]-p*x
                end_for;
                for i from l to min(k+3,nn) do # column modifications #
                    p := x*A[i,k]+y*A[i,k+1];
                    if notlast then p:=p+z*A[i,k+2];
                                    A[i,k+2]:=A[i,k+2]-p*r
                    end_if;
                    A[i,k+1]:=A[i,k+1]-p*q; A[i,k]:=A[i,k]-p
                end_for:
                for i from 1 to n  do # column modifications #
                    p := x*QT[i,k]+y*QT[i,k+1];
                    if notlast then p:=p+z*QT[i,k+2];
                                    QT[i,k+2]:=QT[i,k+2]-p*r
                    end_if;
                    QT[i,k+1]:=QT[i,k+1]-p*q; QT[i,k]:=QT[i,k]-p
                end_for:
            end_for:
        end_while;
    until nn < 1 end_repeat;

                # Now matrix A is upper triangular. Compute eigendata #
   d:=[ A[j,j] $ hold(j)=1..n ]:                        # eigenvalues #
   X:=array(1..n,1..n,[[float(0)$n]$n]); #initialize eigenvector matrix #
   for j from 1 to n do # compute eigenvector vec for eigenvalue d[j] #
       vec[j]:=1; exists:=TRUE;
       for i from j-1 downto 1 do
         tmp:=_plus(A[i,k]*vec[k] $ hold(k)=i+1..j ): tmpp:=d[j]-d[i];
         if iszero(tmpp)            # multiple eigenvalue d[j] = d[i] # 
         then if iszero(tmp) 
                then vec[i]:=0       # a further eigenvector is found #
                else exists:=FALSE; break;   # no further eigenvector #
              end_if:                        # exists                 #
         else vec[i]:=tmp/tmpp:
         end_if;
       end_for:
       if exists # transform j.th eigenvector (vec[1],..,vec[j],0,..) #
                 # with matrix QT and store as column j of eigenvector#
                 # matrix X. If no further eigenvector was found for  #
                 # multiple eigenvalue d[j] then j.th column in X     #
                 # stays 0 as initialized                             #
       then (X[i,j]:=_plus(QT[i,k]*vec[k]$hold(k)=1..j))$hold(i)=1..n;
       end_if:
   end_for:
   (A[j+2,j]:=ssub[j] )$hold(j)=1..n-2; # restore the subdiagonals   #
   (A[j+3,j]:=sssub[j])$hold(j)=1..n-3; # containing the data of the #
                                        # transformation in elmhess2 #
   null();
end_proc:

# ----------- define local subroutine --------------------------#
#                                                               #
# backtransform - undo the transformation                       #
#                    A -> Hessenberg = L^{-1} A L               #
#          performed by elmhess2(), i.e. multiply the matrix X  #
#          of eigenvectors of Hessenberg  X -> LX.              #
#          Then undo the scaling of rows and columns performed  #
#          by balance2(), stored in the global list scale       #
#                                                               #
#explanation: the data of L= [ 1                    ]           #
#                            [     1                ]           #
#                            [   L[3,2]   1         ]           #
#                            [   L[4,2] L[4,3]   1  ]           #
#                            [     ..     ..    ..  ]           #
#        are stored in A by elmhess2() in the form              #
#            A = [ A[1,1]  ..                         ]         #
#                [ A[2,1] A[2,2] ..                   ]         #
#                [ L[3,2] A[3,2] A[3,2] ..            ]         #
#                [ L[4,2] L[4,3] A[4,3] A[4,4] ..     ]         #
#                [  ..     ..     ..      ..   ..  .. ]         #
#---------------------------------------------------------------#

backtransform:=proc()
local i,j,m,x,tmp;
begin
   for m from n-1 downto 2 do
      ( x:=A[i,m-1]; (X[i,j]:=X[i,j]+x*X[m,j] ) $ hold(j)=1..n;
      ) $ hold(i) = m+1 .. n;
      i:=perm[m];
      if i<>m then (tmp:=X[i,j];X[i,j]:=X[m,j];X[m,j]:=tmp;)$hold(j)=1..n;
      end_if:
   end_for:
   ((X[i,j]:=scale[i]*X[i,j]) $ hold(j)=1..n ) $ hold(i)=1..n;
   null();
end_proc:

#---------------------------------------------------------------------#
#---------------------------------------------------------------------#
sortEigendata:=proc(real)
local i,j,position,lambda,Y;
begin 
  position:=[i$hold(i)=1..n];     #brute force sorting of eigenvalues #
  if real
  then for j from 2 to n do             # search for sorted position  #
          lambda:=d[j];                 # of orig. eigenvalue d[j]    #
          for i from j downto 2 do      # which becomes d[position[j]]#
             if d[i-1]<lambda then break end_if:
             d[i]:=d[i-1]: position[i]:=position[i-1]:
          end_for;
          d[i]:=lambda; position[i]:=j:
       end_for:    # real eigenvalues are ordered from small to large #
  else for j from 2 to n do
         lambda:=d[j];   # search for sorted position of eigenvalue d[j] #
         # sorting criterion: from small real part to large real part.   #
         # If tie, then from small abs(imaginary part) to large          #
         # abs(imaginary part). For complex conjugate pair take the one  #
         # with positive imaginary part first                            #
         for i from j downto 2 do
            if Re(d[i-1])<Re(lambda) then break end_if:
            if (Re(d[i-1])=Re(lambda)) then
               if stdlib::abs(Im(d[i-1]))<stdlib::abs(Im(lambda))
                                                  then break end_if:
               if stdlib::abs(Im(d[i-1]))=stdlib::abs(Im(lambda)) and 
                          (Im(d[i-1])>Im(lambda)) then break end_if:
            end_if:
            d[i]:=d[i-1]: position[i]:=position[i-1]:
         end_for;
         d[i]:=lambda; position[i]:=j:
       end_for:    # eigenvalues are ordered from small to large, now #
  end_if:
  Y:=X: ((X[i,j]:=Y[i,position[j]])$hold(i)=1..n)$hold(j)=1..n:
  null();
end_proc:

#----------------------------------------------------------------------#
#----------------------------------------------------------------------#
sortEigendata1:=proc()             # same as sortEigendata, but also   #
local i,j,position,lambda,fd,fl,Y; # for non-float numeric eigenvalues #
begin 
  position:=[i$hold(i)=1..n];      #brute force sorting of eigenvalues #
  for j from 2 to n do
    lambda:=d[j];fl:=float(d[j]); # non-float entries are allowed here! #
    for i from j downto 2 do
       fd:=float(d[i-1]):
       if Re(fd)<Re(fl) then break end_if:
       if (Re(fd)=Re(fl)) then
          if abs(Im(fd))<abs(Im(fl)) then break end_if:
          if abs(Im(fd))=abs(Im(fl)) and (Im(fd)>Im(fl))
                                                     then break end_if:
       end_if:
       d[i]:=d[i-1]: position[i]:=position[i-1]:
    end_for;
    d[i]:=lambda; position[i]:=j:
  end_for: 
  Y:=X: ((X[i,j]:=Y[i,position[j]])$hold(i)=1..n)$hold(j)=1..n:
  null():
end_proc:

# ----------- define local subroutine --------------------------------#
#---------------------------------------------------------------------#
normalizeEigenvectors:=proc()
local i,j,mynorm;
begin
  for j from 1 to n do
     mynorm:= stdlib::sqrt(_plus(stdlib::abs(X[i,j])^2 $ hold(i)=1..n )):
     if not iszero(mynorm) then (X[i,j]:=X[i,j]/mynorm;)$hold(i)=1..n;
     end_if:
  end_for:
  null();
end_proc:
# ----------- define local subroutine --------------------------------#
# same as above, also for non-float entries                           #
#---------------------------------------------------------------------#
normalizeEigenvectors1:=proc()
local i,j,mynorm;
begin
  for j from 1 to n do
     mynorm:= sqrt(_plus(abs(X[i,j])^2 $ hold(i)=1..n )):
     if not iszero(mynorm) then (X[i,j]:=X[i,j]/mynorm;)$hold(i)=1..n;
     end_if:
  end_for:
  null();
end_proc:

# ----------- define local subroutine --------------------------------#
#                                                                     #
# residues( )  - computes the residues                                #   
#                         res[i] = norm( A x[i] - d[i] x[i] )         #
#                where x[i] is the normalized ith column of the       #
#                eigenvector matrix X and d[i] is the corresponding   #
#                numerical eigenvalue                                 #
# return value:  list [ res[1], res[2], .. ]                          #
#---------------------------------------------------------------------#
residues:=proc() local i,j,k,res;       # residue = |Ax-lambda x|/|x| #
begin
  (res[j]:=stdlib::sqrt(_plus(stdlib::abs(
            _plus((oldA[i,k]*X[k,j])$hold(k)=1..n) -d[j]*X[i,j] 
                          )^2 $ hold(i)=1..n))) $ hold(j)=1..n:
  [ res[j] $ hold(j)=1..n ]
end_proc:
#---------------------------------------------------------------------#
# same as above, also for non-float entries                           #
#---------------------------------------------------------------------#
residues1:=proc() local i,j,k,res;      # residue = |Ax-lambda x|/|x| #
begin
  (res[j]:=sqrt(_plus(abs(_plus((oldA[i,k]*X[k,j])$hold(k)=1..n)
                                  -d[j]*X[i,j] 
                          )^2 $ hold(i)=1..n))) $ hold(j)=1..n:
  [ res[j] $ hold(j)=1..n ]
end_proc:

#---------------------------------------------------------------------#
# the body of Eigenvectors using the subroutines above:               #
#---------------------------------------------------------------------#

   if A::hasProp(Cat::MatrixCat)=TRUE    # A is of domtype Dom::Matrix  #
      then A:=A::convert_to(A,DOM_ARRAY) # convert to domtype DOM_ARRAY #
   end_if;                               # for speed #
   if domtype(A) <> DOM_ARRAY
      then error("matrix must be of domtype DOM_ARRAY or Dom::Matrix");
   end_if:
   if op(A,[0,1])<>2 then error("first argument is not a matrix");end_if;
   n:=op(A,[0,2,2]):
   if n <> op(A,[0,3,2]) then error("not a square matrix"); end_if;
   oldDIGITS := DIGITS: DIGITS := DIGITS + trunc(n/10);

   hermitean := TRUE;            # is matrix hermitean ?        #
   real      := TRUE;            # is matrix real ?             #
   lowertriangular := TRUE:      # is matrix lower triangular ? #
   uppertriangular := TRUE:      # is matrix upper triangular ? #

   for i from 1 to n do          # find properties of matrix    #
    for j from 1 to i-1 do       # (hermitean/real/triangular)  #
      if  (not iszero(Im(A[i,j]))) or (not iszero(Im(A[j,i]))) 
                                       then real := FALSE end_if:
      if A[i,j]<>conjugate(A[j,i]) then hermitean:= FALSE end_if:
      if not iszero(A[j,i]) then lowertriangular := FALSE end_if:
      if not iszero(A[i,j]) then uppertriangular := FALSE end_if:
    end_for:
    if not iszero(Im(A[i,i]))then real:= FALSE; hermitean:= FALSE
    end_if:
   end_for:

   oldA:=A;
   M:=Dom::Matrix():               # constructor of Dom::Matrix #
   if lowertriangular or uppertriangular then # triangular case #
      TriangularEigendata(lowertriangular,uppertriangular):
      sortEigendata1():normalizeEigenvectors1():
      errors:=residues1():
      DIGITS:=oldDIGITS: return([d,M(X),errors]);
   end_if:
     # so far symbolic entries could be handled efficiently, now #
     # switch to floats                                          #
   A:=map(A,float):
   if {op(map(A,domtype))} minus {DOM_FLOAT, DOM_COMPLEX} <> {} 
      then error("non-numeric entries not handled");
   end_if:
   A:=subs(A,float(0)=0):
   oldA:=A:
   if hermitean then                            # hermitean case #
     Hermitean2tridiag2(real):  # transform to tri-diagonal form #
     tqli2();                   # QL-Iteration with eigenvectors #
     QTXbyHouseholderData(real):# transformation of eigenvectors #
     sortEigendata(TRUE):       # sort real eigenvalues/vectors  # 
   else                                           # general case #
     balance2();        # balance matrix for numerical stability #
     elmhess2();        # transform to Hessenberg form           #
     hqr2():        # compute spectral data of Hessenberg matrix #
     backtransform():      # transform eigenvectors back through #
                           # elmhess2 and balance2               #
     sortEigendata(FALSE): # sort complex (real=FALSE) data      #
   end_if:
   normalizeEigenvectors(): errors:=residues():
   DIGITS:=oldDIGITS: [d,M(X),errors]:
end_proc:
#--------  end of procedure eigenvectors ---------------#

