
#------   file  factorCh.mu   29.9.97  ---------------------#
#  Help file:

 factorCholesky - numerical Cholesky factorization of a positive
                  Hermitean matrix

 Call:

 numeric::factorCholesky( A <, Symbolic> <, NoCheck> )
                                                                   
 Parameter:

  A  --  a square real or complex matrix of domtype DOM_ARRAY
         or Dom::Matrix               
                                                                   
Synopsis:

  numeric::factorCholesky(A); returns the lower triangular
  Cholesky factor L of the factorization A=LL', where L' 
  denotes the Hermitean transpose of L. The result is of 
  domtype Dom::Matrix. It is normalized such that the diagonal
  of L is real and positive.
                                                                  
  Non-float numerical entries are admissible, they will be 
  evaluated to floats. Conversion to floats may be suppressed
  by using the option Symbolic.
 
  If the matrix is not Hermitean or not positive definite, then
  the computation will terminate with an error message.
  Checking of these properties can be suppressed with the option
  NoCheck. This option should be used, whenever the matrix contains 
  indeterminates. If numeric symbolic entries such as PI, sqrt(2) etc.
  are present, then these properties are checked numerically.

  Examples: 

>> A := array( 1..2 , 1..2 , [ [1,2*I],[-2*I,5] ] ):

>> numeric::factorCholesky(A);                                

                            +-              -+
                            |    1.0,    0   |
                            |                |
                            |  - 2.0 I, 1.0  |
                            +-              -+

>> numeric::factorCholesky(A,Symbolic);

                              +-          -+
                              |    1,   0  |
                              |            |
                              |  - 2 I, 1  |
                              +-          -+



See also: numeric::factorLU , numeric::factorQR 
                                                                  #
#-----------------------------------------------------------------#
numeric::factorCholesky:=proc(A)
local n,real,i,j,k,B,M,symbolic_mode,warning_given,do_check,Options;
begin
  if A::hasProp(Cat::MatrixCat)=TRUE # A is of domtype Dom::Matrix  #
  then A:=A::convert_to(A,DOM_ARRAY) # convert to domtype DOM_ARRAY #
  end_if;                            # for speed #
  if domtype(A) <> DOM_ARRAY
     then error("matrix must be of domtype DOM_ARRAY or Dom::Matrix");
  end_if:
  if op(A,[0,1])<>2 then error("argument is not a matrix");end_if;
  n:=op(A,[0,2,2]):
  if n <> op(A,[0,3,2]) then error("not a square matrix"); end_if;

  do_check:=TRUE; symbolic_mode:=FALSE; real:=FALSE;
  if args(0)>1 then
      Options:={args(2..args(0))};
      if has(Options,NoCheck)  then     do_check:=FALSE;end_if;
      if has(Options,Symbolic) then symbolic_mode:=TRUE;end_if;
      if ( Options minus {Symbolic,NoCheck}) <> {} then
                                 error("wrong option(s)"); end_if;
  end_if;
  if symbolic_mode and do_check and
       (  (indets(A)<>{}) or has({op(map(A,domtype))},DOM_NIL) )
  then do_check:=FALSE; 
       warning("indeterminate found, switching to option NoCheck [numeric::factorCholesky]");
  end_if;

  if not symbolic_mode then
    B:=map(A,float);
    if ({op(map(B,domtype))} minus {DOM_FLOAT,DOM_COMPLEX}) <> {}
      then symbolic_mode:=TRUE;
           do_check:=FALSE;
           warning("symbolic entry found, switching to options Symbolic, NoCheck [numeric::factorCholesky]");
      else A:=B;
    end_if:
  end_if;
  A:=subs(A,float(0)=0);

  if do_check then
    for i from 1 to n do
      for j from 1 to i-1 do
        if A[i,j]<>conjugate(A[j,i]) then error("matrix is not hermitean") end_if:
      end_for;
      if not iszero(Im(A[i,i]))then error("matrix is not hermitean"); end_if;
    end_for;
  end_if;

  if ({op(map(A,(iszero@Im)))}={TRUE}) then real:=TRUE else real:=FALSE; end_if:


  if do_check then
     warning_given:=FALSE;
     if traperror(bool(A[1,1]<=0))<>0 
     then if not warning_given then
             warning("positiveness checked numerically [numeric::factorCholesky]");
             warning_given:=TRUE;
          end_if;
          if float(A[1,1])<=0 then
            error("matrix is not positive definite within working precision");
          end_if;
     else if A[1,1]<=0 then
             if symbolic_mode
              then error("matrix is not positive definite");
              else error("matrix is not positive definite within working precision");
             end_if;
           end_if;
     end_if;
  end_if;

  for i from 1 to n do
    if real
    then (A[i,j]:=(A[i,j]-_plus(A[i,k]*A[j,k] $ hold(k)=1..j-1))/A[j,j])
                       $ hold(j)=1..i-1;
    else (A[i,j]:=(A[i,j]-_plus(A[i,k]*conjugate(A[j,k]) $ hold(k)=1..j-1)
                  )/A[j,j]) $ hold(j)=1..i-1;
    end_if;
    if symbolic_mode
      then A[i,i]:=A[i,i]-_plus(abs(A[i,k])^2 $ hold(k)=1..i-1);
      else A[i,i]:=A[i,i]-_plus(stdlib::abs(A[i,k])^2 $ hold(k)=1..i-1);
    end_if;
    if do_check then
      if traperror(bool(A[i,i]<=0))<>0 
      then if not warning_given then
             warning("positiveness checked numerically [numeric::factorCholesky]");
             warning_given:=TRUE;
           end_if;
           if float(A[i,i])<=0 then
              error("matrix is not positive definite within working precision");
           end_if;
      else if A[i,i]<=0 then
              if symbolic_mode
               then error("matrix is not positive definite");
               else error("matrix is not positive definite within working precision");
              end_if;
           end_if;
      end_if;
    end_if;
    A[i,i]:=(A[i,i])^(1/2); (A[i,j]:=0) $ hold(j)=i+1..n;
  end_for;

  A:=subs(A,float(0)=0);
  M:=Dom::Matrix():  # constructor of Dom::Matrix #
  return(M(A)):
end_proc:


