
#------   file  factorLU.mu  22.9.97         ---------------------#
#  Help file:

 factorLU - LU factorization of a matrix

 Call:

 numeric::factorLU( A <, Symbolic> )
                                                                   
 Parameter:

  A  --  a  square real or complex matrix of domtype DOM_ARRAY
         or Dom::Matrix               
                                                                   
Synopsis:

  The LU factorization of a real/complex matrix is PA=LU, where
  L is lower triangular, normalized to 1 along the diagonal,
  and R is upper triangular.  P is a permutation matrix 
  corresponding to row exchanges.

  numeric::factorLU(A) returns the list [L,U,p], where L and R are
  of domtype Dom::Matrix. The list of integers p=[ p[1],p[2], .. ]
  represents the row exchanges in pivoting steps, i.e., B = PA = LU, 
  where B[i,j] = A[p[i],j]. Multiplication y=Px of the permutation
  matrix P with a vector x is realized by y[i]:=x[p[i]].
                                                                  
  Non-float numerical entries are allowed in the matrix, they
  will be evaluated to floats in the course of the computation.
  Non-numeric symbolic entries will lead to an error, unless
  the option Symbolic is specified.

  The option Symbolic prevents conversion of the matrix entries
  to floats. No row exchanges will be performed unless necessary.

  Examples: 

>> A:=array(1..3,1..3,[[1,2,3],[2,4,6],[4,8,9]]);

                               +-         -+
                               |  1, 2, 3  |
                               |           |
                               |  2, 4, 6  |
                               |           |
                               |  4, 8, 9  |
                               +-         -+
>> ([L,U,p]):=numeric::factorLU(A);

          -- +-            -+  +-                -+            --
          |  |    1,  0, 0  |  |  4.0, 8.0,  9.0  |             |
          |  |              |  |                  |             |
          |  |   0.5, 1, 0  |, |   0,   0,   1.5  |, [3, 2, 1]  |
          |  |              |  |                  |             |
          |  |  0.25, 0, 1  |  |   0,   0,  0.75  |             |
          -- +-            -+  +-                -+            --

>> L*U;
                            +-               -+
                            |  4.0, 8.0, 9.0  |
                            |                 |
                            |  2.0, 4.0, 6.0  |
                            |                 |
                            |  1.0, 2.0, 3.0  |
                            +-               -+

>> A := array( 1..4,1..4, [ [2,1,0,0],[1,2,1,0],[0,1,2,1],[0,0,1,2] ]): 

>> numeric::factorLU(A,Symbolic);

    -- +-                  -+  +-                  -+               --
    |  |   1,   0,   0,  0  |  |  2,  1,   0,   0   |                |
    |  |                    |  |                    |                |
    |  |  1/2,  1,   0,  0  |  |  0, 3/2,  1,   0   |                |
    |  |                    |, |                    |, [1, 2, 3, 4]  |
    |  |   0,  2/3,  1,  0  |  |  0,  0,  4/3,  1   |                |
    |  |                    |  |                    |                |
    |  |   0,   0,  3/4, 1  |  |  0,  0,   0,  5/4  |                |
    -- +-                  -+  +-                  -+               --

See also:  linalg::factorLU, numeric::factorQR, numeric::factorCholesky
                                                                  #
#-----------------------------------------------------------------#
numeric::factorLU:=proc(A)
local n,method,B,i,j,k,tmp,pivot,pivindex,rownorm,L,p,M;
begin
   if A::hasProp(Cat::MatrixCat)=TRUE   # A is of domtype Dom::Matrix  #
   then A:=A::convert_to(A,DOM_ARRAY)   # convert to domtype DOM_ARRAY #
   end_if;                              # for speed #
   if domtype(A) <> DOM_ARRAY
      then error("matrix must be of domtype DOM_ARRAY or Dom::Matrix");
   end_if:
   if op(A,[0,1])<>2 then error("argument is not a matrix");end_if;
   n:=op(A,[0,2,2]):
   if n <> op(A,[0,3,2]) then error("not a square matrix"); end_if;

   if args(0)>1 then if args(2)=Symbolic then method:=1;
                     else error("unknown option") end_if;
                else method:=2;
   end_if;

if method=2 then            
   B:=map(A,float):
   if {op(map(B,domtype))} minus {DOM_FLOAT, DOM_COMPLEX} <> {}
       then warning("symbolic entry found, switching to option Symbolic [numeric::inverse]");
            method:=1;
       else A:=B;
   end_if:
end_if;
   A:=subs(A,float(0)=0):

if method=1 then
   userinfo(1,"symbolic mode, avoiding pivoting if possible ");
   p:=[ i $ hold(i)=1..n ]:   # initialize list to store row permutations #
   for j from 1 to n-1 do
      if iszero(A[j,j]) then
         for pivindex from j+1 to n-1 do
             if not iszero(A[pivindex,j]) then break; end_if;
         end_for;
      else pivindex:=j;
      end_if;
      if pivindex<>j then
        (tmp:=A[j,k]; A[j,k]:=A[pivindex,k];A[pivindex,k]:=tmp;) $ hold(k)=1..n;
        (tmp:=p[j];p[j]:=p[pivindex];p[pivindex]:=tmp;) $ hold(k)=1..n;
      end_if;
      if not iszero(A[j,j]) then
         for i from j+1 to n do
             A[i,j]:=A[i,j]/ A[j,j]:
             if not iszero(A[i,j]) then
              (A[i,k]:=A[i,k]-A[i,j]*A[j,k]) $ hold(k)=j+1..n;
             end_if;
         end_for;
      end_if;
   end_for;
   L:=array(1..n,1..n,[[0$n]$n]); (L[i,i]:=1) $ hold(i)=1..n;
   ((L[i,j]:=A[i,j]) $ hold(i)=j+1..n ) $ hold(j)=1..n-1;
   ((A[i,j]:=0) $ hold(i)=j+1..n ) $ hold(j)=1..n-1;
   M:=Dom::Matrix():                       # constructor of Dom::Matrix #
   return([M(L),M(A),p]);

end_if;

if method=2 then            
   userinfo(1,"using column pivoting");

   p:=[ i $ hold(i)=1..n ]: # initialize list to store row permutations #

   for j from 1 to n-1 do
      pivot:=0; pivindex:=j;
      for i from j to n do 
          rownorm:=max( stdlib::abs(A[i,k]) $ hold(k)=j+1..n );
          if not iszero(rownorm) then
             stdlib::abs(A[i,j])/rownorm;
             if pivot<last(1) then pivot:=last(1); pivindex:=i; end_if;
          end_if;
      end_for;
      if pivindex<>j then
        (tmp:=A[j,k]; A[j,k]:=A[pivindex,k];A[pivindex,k]:=tmp;) $ hold(k)=1..n;
        tmp:=p[j];p[j]:=p[pivindex];p[pivindex]:=tmp;
      end_if;
      if not iszero(A[j,j]) then
         for i from j+1 to n do
             A[i,j]:=A[i,j]/A[j,j]:
             if not iszero(A[i,j]) then
               (A[i,k]:=A[i,k]-A[i,j]*A[j,k]) $ hold(k)=j+1..n;
             end_if;
         end_for;
      end_if;
   end_for;
   L:=array(1..n,1..n,[[0$n]$n]); (L[i,i]:=1) $ hold(i)=1..n;
   ((L[i,j]:=A[i,j]) $ hold(i)=j+1..n ) $ hold(j)=1..n-1;
   ((A[i,j]:=0) $ hold(i)=j+1..n ) $ hold(j)=1..n-1;
   L:=subs(L,float(0)=0); A:=subs(A,float(0)=0);
   M:=Dom::Matrix():                     # constructor of Dom::Matrix #
   return([M(L),M(A),p]):
end_if;

end_proc:


