
#------   file  factorQR.mu  19.9.97         ---------------------#
#  Help file:

 factorQR - QR factorization of a matrix

 Call:

 numeric::factorQR( A <, Symbolic> )
                                                                   
 Parameter:

  A  --  a  square real or complex matrix of domtype DOM_ARRAY
         or Dom::Matrix               
                                                                   
Synopsis:

  The QR factorization of a real/complex matrix is A=QR, where
  Q is unitary/orthogonal and R is upper triangular.

  numeric::factorQR(A); returns the list [Q,R] with Q and R of
  domtype Dom::Matrix.
                                                                  
  Non-float numerical entries are allowed in the matrix, they
  will be evaluated to floats in the course of the computation.
  Non-numeric symbolic entries will lead to an error. Householder
  transformations are used to compute the factorization. The
  R factor is normalized such that its diagonal entries are
  real and non-negative.

  The option Symbolic prevents conversion of the matrix entries
  to floats. In this case symbolic entries are allowed, Gram-Schmidt
  orthonormalization is used to compute the factorization.

  Examples: 

>> A:=array(1..3,1..3,[ [1,1,I] , [1,exp(1),I] , [1,2,I] ]):

>> ([Q,R]):=numeric::factorQR(A);

    -- +-                                              -+
    |  | 0.5773502691, -0.7424322034,  - 0.339795659  I |
    |  |                                                |
    |  | 0.5773502691,  0.6654877745,  - 0.4730673192 I |,
    |  |                                                |
    |  | 0.5773502691,  0.07694442889,   0.8128629783 I |
    -- +-                                              -+
           +-                                           -+ --
           | 1.732050807, 3.301451553, 1.732050807 I     |  |
           |                                             |  |
           |      0,      1.220439979, 1.394208422e-19 I |  |
           |                                             |  |
           |      0,           0,      6.38099974e-20    |  |
           +-                                           -+ --
   
>> Q*R;
                       +-                       -+
                       | 1.0,     1.0,     1.0 I |
                       |                         |
                       | 1.0, 2.718281828, 1.0 I |
                       |                         |
                       | 1.0,     2.0,     1.0 I |
                       +-                       -+

>> A:=array(1..3,1..3,[ [1,1,I] , [1,1,I] , [1,2,I] ]):

>> ([Q,R]):=numeric::factorQR(A,Symbolic);

   -- +-                         -+                              --
   |  |  1/2     1/2  1/2    1/2  |                               |
   |  | 3       2    3      2     |  +-                       -+  |
   |  | ----, - ---------,  ----  |  |            1/2          |  |
   |  |  3          6        2    |  |  1/2    4 3         1/2 |  |
   |  |                           |  | 3   ,   ------,  I 3    |  |
   |  |  1/2     1/2  1/2     1/2 |  |           3             |  |
   |  | 3       2    3       2    |  |                         |  |
   |  | ----, - ---------, - ---- |, |        1/2  1/2         |  |
   |  |  3          6         2   |  |       2    3            |  |
   |  |                           |  |   0,  ---------,    0   |  |
   |  |  1/2    1/2  1/2          |  |           3             |  |
   |  | 3      2    3             |  |                         |  |
   |  | ----,  ---------,     0   |  |   0,      0,        0   |  |
   |  |  3         3              |  +-                       -+  |
   -- +-                         -+                              --

>> Q*R;
                               +-         -+
                               |  1, 1, I  |
                               |           |
                               |  1, 1, I  |
                               |           |
                               |  1, 2, I  |
                               +-         -+

See also:  linalg::factorQR, numeric::factorLU, numeric::factorCholesky
                                                                  #
#-----------------------------------------------------------------#
numeric::factorQR:=proc(A)
local n,method,B,rreal,i,j,k,l,d,h,s,ss,q,r,Q,R,M;
begin
   if A::hasProp(Cat::MatrixCat)=TRUE    # A is of domtype Dom::Matrix  #
   then A:=A::convert_to(A,DOM_ARRAY)    # convert to domtype DOM_ARRAY #
   end_if;                               # for speed #
   if domtype(A) <> DOM_ARRAY
      then error("matrix must be of domtype DOM_ARRAY or Dom::Matrix");
   end_if:
   if op(A,[0,1])<>2 then error("argument is not a matrix");end_if;
   n:=op(A,[0,2,2]):
   if n <> op(A,[0,3,2]) then error("not a square matrix"); end_if;

   if args(0)>1 then if args(2)=Symbolic then method:=1;
                     else error("unknown option") end_if;
                else method:=2;
   end_if;

if method=2 then            
   B:=map(A,float):
   if {op(map(B,domtype))} minus {DOM_FLOAT, DOM_COMPLEX} <> {}
       then warning("symbolic entry found, switching to option Symbolic [numeric::factorQR]");
            method:=1;
       else A:=B;
   end_if:
end_if;
   A:=subs(A,float(0)=0):

   if {op(map(A,(iszero@Im)))} = {TRUE}     # is matrix real ? #
       then rreal:=TRUE else rreal:=FALSE
   end_if:


if method=1 then
   userinfo(1,"using Gram-Schmidt orthonormalization");
   R:=array(1..n,1..n,[[0$n]$n]);              # initialize upper factor #
   for j from 1 to n do
      R[j,j]:=_plus( conjugate(A[i,j])*A[i,j] $ hold(i)=1..n );
      if iszero(R[j,j]) then       # insert an arbitrary column. Try the #
         for k from 1 to n do      # standard columns q=(0,..,1,..,0)^T  #
            q:=[0$n]; q[k]:=1;     # until q is independent of the first #
            for i from 1 to n do   # j-1 orthonormalized columns of A    #
               q[i]:=q[i]-_plus(A[i,l]*conjugate(A[k,l])/r[l]
                                $ hold(l)=1..j-1);
            end_for;
            r[j]:=_plus( conjugate(q[i])*q[i] $ hold(i)=1..n );
            if not iszero(r[j]) then
               (A[i,j]:=q[i]) $ hold(i)=1..n; break;
            end_if;
         end_for;
      else r[j]:=R[j,j];
      end_if;
      for k from j+1 to n do
          R[j,k]:=_plus( conjugate(A[i,j])*A[i,k] $ hold(i)=1..n );
      end_for;
      if j < n then
        for i from 1 to n do
           A[i,j+1]:= A[i,j+1]-_plus(A[i,k]*R[k,j+1]/r[k] $ hold(k)=1..j );
        end_for;
      end_if;
   end_for;
   r:=[r[i]^(1/2) $ hold(i)=1..n];
   for j from 1 to n do
       (A[i,j]:=A[i,j]/r[j]) $ hold(i)=1..n;
       if not iszero(R[j,j]) then R[j,j]:=r[j]; end_if;
       (R[j,i]:=R[j,i]/r[j]) $ hold(i)=j+1..n;
   end_for;

   M:=Dom::Matrix():                     # constructor of Dom::Matrix #
   return([M(A),M(R)]);

end_if;

if method=2 then            
   userinfo(1,"using Householder transformations");

   d:=[0$n]:           # initialize list to store diagonal #
   h:=[0$n]:           # initialize list to store factors  #

   for j from 1 to n do
        # Householder step from the left to obtain upper   #
        # triangular form. Householder data are stored in  #
        # lower triangular part of A.                      #
        s:= _plus(stdlib::abs(A[i,j])^2 $ hold(i)=j+1..n);
        if iszero(s) then d[j]:=A[j,j];
        else ss:=s+stdlib::abs(A[j,j])^2;
             s:=stdlib::sqrt(ss):
             h[j] := 1 / (ss + stdlib::abs(A[j,j]) * s);
             if not iszero(A[j,j]) then s:=sign(A[j,j])*s;end_if:
             A[j,j]:=s+A[j,j]; d[j]:=-s;
             for i from j+1 to n do
                 if rreal then   s:=_plus(A[k,j] *A[k,i]$hold(k)=j..n) ;
                 else s:=_plus(conjugate(A[k,j])*A[k,i]$hold(k)=j..n) ;
                 end_if;
                 s:=s*h[j];
                 (A[k,i]:=A[k,i]-s*A[k,j]) $ hold(k)=j..n ;
             end_for;
        end_if;
   end_for;

   # construct Q from the Householder data stored in the lower part of A#
   Q:=array(1..n,1..n,[[0$n]$n]); (Q[i,i]:=1) $ hold(i)=1..n;
   for j from n downto 1 do
     if h[j]<>0 then
        for i from j+1 to n do
           if rreal then s:=_plus(A[k,j]*Q[k,i] $ hold(k)=j..n) ;
           else s:=_plus(conjugate(A[k,j])*Q[k,i] $ hold(k)=j..n) ;
           end_if;
           s:=s*h[j];
           (Q[k,i]:=Q[k,i]-s*A[k,j]) $ hold(k)=j..n ;
        end_for;
       (Q[k,j]:=A[k,j]/d[j]) $ hold(k)=j..n;
       Q[j,j]:=Q[j,j]+1;
     end_if;
   end_for;

   # construct R from the upper part of A and diagonal d #
   for j from 1 to n do
     (A[i,j]:=0) $ hold(i)=j+1..n;
      A[j,j]:=stdlib::abs(d[j]);
      if not iszero(A[j,j]) then     
         s:=sign(d[j]);                       # normalize diagonal of #
         (A[j,i]:=A[j,i]/s) $ hold(i)=j+1..n; # R to be real positive #
         (Q[i,j]:=Q[i,j]*s) $ hold(i)=1..n;
      end_if;
   end_for;

   Q:=subs(Q,float(0)=0); A:=subs(A,float(0)=0);
   M:=Dom::Matrix():                     # constructor of Dom::Matrix #
   return([M(Q),M(A)]):
end_if;

end_proc:


