
#------   file  inverse.mu   26.9.97         ---------------------#
#-----------------------------------------------------------------#
#  Help file:

 inverse - inverse of a numerical matrix

 Call:

 numeric::inverse( A <, Symbolic> )
                                                                   
 Parameter:

  A  --  a  square real or complex matrix of domtype DOM_ARRAY
         or Dom::Matrix               
                                                                   
Synopsis:

  numeric::inverse(A); returns the inverse of A computed by
  Gauss elimination with partial pivoting. The result is of 
  domtype Dom::Matrix.
                                                                  
  Non-float numerical entries are admissible, they will be 
  evaluated to floats in the course of the computation.

  The option Symbolic prevents conversion of the matrix entries
  to floats. No pivoting will occur unless necessary.  With this
  option non-numeric symbolic entries are admissible.  However, 
  their presence will slow down the computation drastically.

  Examples: 

>> A := array( 1..2 , 1..2 , [ [1,2],[3,4] ] ):

>> numeric::inverse(A);
                             +-            -+
                             |  -2.0,  1.0  |
                             |              |
                             |   1.5, -0.5  |
                             +-            -+

>> numeric::inverse(A,Symbolic);

                              +-           -+
                              |   -2,   1   |
                              |             |
                              |  3/2, -1/2  |
                              +-           -+
                                                                  #
#-----------------------------------------------------------------#
numeric::inverse:=proc(A)
local n,method,use_normal,i,j,k,tmp,pivot,pivindex,rownorm,B,sparsity;
begin
  if A::hasProp(Cat::MatrixCat)=TRUE # A is of domtype Dom::Matrix  #
  then A:=A::convert_to(A,DOM_ARRAY) # convert to domtype DOM_ARRAY #
  end_if;                            # for speed #
  if domtype(A) <> DOM_ARRAY
     then error("matrix must be of domtype DOM_ARRAY or Dom::Matrix");
  end_if:
  if op(A,[0,1])<>2 then error("argument is not a matrix");end_if;
  n:=op(A,[0,2,2]):
  if n <> op(A,[0,3,2]) then error("not a square matrix"); end_if;

  if args(0)>1 then if args(2)=Symbolic then method:=1;
                    else error("unknown option") end_if;
               else method:=2;
  end_if;

  use_normal:=FALSE;

  if method=1 then
    if (indets(A)<>{}) or has({op(map(A,domtype))},DOM_NIL)
      then use_normal:=TRUE end_if;
  end_if;

  if method=2 then
    B:=map(A,float):
    if {op(map(B,domtype))} minus {DOM_FLOAT,DOM_COMPLEX}<>{} then
    warning("symbolic entry found, switching to symbolic mode [numeric::inverse]");
       method:=1; use_normal:=TRUE;
    else A:=B;
    end_if;  
  end_if;

  if method=1 then
    userinfo(1,"symbolic mode, no pivoting unless necessary");
  end_if;
  if method=2 then userinfo(1,"using column pivoting"); end_if;

  A:=subs(A,float(0)=0);
  B:=array(1..n,1..n,[[0$n]$n]); # initialize inverse as #
  (B[i,i]:=1) $ hold(i)=1..n;    # identity matrix       #

  for j from 1 to n-1 do
  # search for pivot element #
    if method=1 then
      if iszero(A[j,j]) then
         for pivindex from j+1 to n-1 do
             if not iszero(A[pivindex,j]) then break; end_if;
         end_for;
      else pivindex:=j;
      end_if;
    end_if;
    if method=2 then
      pivot:=0; pivindex:=j;
      for i from j to n do 
          rownorm:=max(stdlib::abs(A[i,k]) $ hold(k)=j+1..n );
          if not iszero(rownorm) then
             stdlib::abs(A[i,j])/rownorm;
             if pivot<last(1) then pivot:=last(1); pivindex:=i; end_if;
          else pivindex:=i;break;
          end_if;
      end_for;
    end_if;
    if pivindex<>j then
      (tmp:=A[j,k];A[j,k]:=A[pivindex,k];A[pivindex,k]:=tmp;) $ hold(k)=1..n;
      (tmp:=B[j,k];B[j,k]:=B[pivindex,k];B[pivindex,k]:=tmp;) $ hold(k)=1..n;
    end_if;
    if iszero(A[j,j]) then error("matrix is not invertible"); end_if;
    # Elimination : #
    for i from j+1 to n do
      tmp:=A[i,j]/A[j,j];
      if iszero(tmp) then next end_if;
      if use_normal then 
           (A[i,k]:=normal(A[i,k]-tmp*A[j,k])) $ hold(k)=j+1..n;
           (B[i,k]:=normal(B[i,k]-tmp*B[j,k])) $ hold(k)=1..n;
      else (A[i,k]:=       A[i,k]-tmp*A[j,k])  $ hold(k)=j+1..n;
           (B[i,k]:=       B[i,k]-tmp*B[j,k])  $ hold(k)=1..n;
      end_if;
    end_for;
  end_for;
  if iszero(A[n,n]) then error("matrix is not invertible"); end_if;
  # Now A is upper triangular. Backsubstitution: #

  sparsity:=0;
  for i from 1 to n-1 do
    for j from i+1 to n do
        if iszero(A[i,j]) then sparsity:=sparsity+1; end_if;
    end_for;
  end_for;
  if sparsity< 0.3*n^2
  then # original version, does not make use of sparsity in upper triangular A #
       for j from 1 to n do
         for i from n downto 1 do
           B[i,j]:=(B[i,j]-_plus(A[i,k]*B[k,j] $ hold(k)=i+1..n))/A[i,i] ;
           if use_normal then B[i,j]:=normal(B[i,j]) end_if;
         end_for;
       end_for;
  else # try to use sparsity #
       for i from n downto 1 do
         for k from i+1 to n do
           if not iszero(A[i,k]) then  
              (B[i,j]:=B[i,j]-A[i,k]*B[k,j];) $hold(j)=1..n;
           end_if;
         end_for;
         (B[i,j]:=B[i,j]/A[i,i];) $hold(j)=1..n;
         if use_normal then (B[i,j]:=normal(B[i,j]);)$hold(j)=1..n; end_if;
       end_for;
  end_if;
  return(Dom::Matrix()(subs(B,float(0)=0))):
end_proc:
