
# ---------------  file odesolve.mu 10.10.97 ------------------------ #
#  Help file:

 odesolve -- numerical solution of the initial value problem
             of a dynamical system

 Call:

 numeric::odesolve( t0 .. t, f, y0 <, method > <, Stepsize = h >
                                   <, Alldata = n > <, Symbolic> )

 Parameter:

    t0     -- numerical value for the initial time
    t      -- numerical value for the final time
    f      -- a function or procedure representing the vector field 
              of the dynamial system dy/dt = f(t,y)
    y0     -- a list or vector of initial conditions y(t0)=y0
    method -- name of the numerical scheme, one of
              EULER1, RKF43, RK4, RKF34, RKF54a, RKF54b, DOPRI54, 
              RKF45a, RKF45b, DOPRI45, BUTCHER6, RKF87, RKF78, 
              xRKF43, xRKF34, xRKF54a, xRKF54b, xDOPRI54, xRKF45a,
              xRKF45b, xDOPRI45, xRKF87, xRKF78 
    h      -- numerical value for the stepsize
    n      -- an integer controlling the size of the output list

 Synopsis:

  numeric::odesolve( t0..t, f, y0 <, Options> ) returns the solution
  y(t) of the initial value problem dy/dt= f(t,y), y(t0)=y0, where 
  y0, y(t) are real or complex vectors.

  The vector field f=f(t,y) must be represented by a function or a
  procedure with two input parameters: the scalar t and the vector y.
  It has to return the vector f(t,y) either as a list or as an array.
  Also for scalar equations the field y has to be represented as a
  1-dimensional list or array. For instance, the input data for
  the scalar initial value problem dy/dt = t*sin(y), y(0)=1, have
  to be of the form

    f := proc(t,y) begin [t*sin(y[1])] end_proc; y0 := [ 1 ];

  The result of numeric::odesolve is a vector of the same type as
  the initial condition y0, i.e., either a list or an array.

  As default an adaptive mechanism is used to control the step size
  such that accepted steps are accurate within the relative precision
  given by 10^(-DIGITS). If no method is specified, then the embedded
  Runge-Kutta-Fehlberg pair RKF78 of order 8 is used as default method.

  All methods presently implemented are Runge-Kutta type single step
  schemes. The methods RKF43, RKF34, RKF54a, RKF54b, DOPRI54, RKF45a, 
  RKF45b, DOPRI45, RKF87, RKF78 are embedded pairs of Runge-Kutta-Fehlberg
  and Dormand-Prince type. For them estimates of the local truncation error 
  are obtained in the usual way by comparing the results of the two 
  submethods of the pair. The names indicate the orders of the subprocesses. 
  For instance, RKF34 and RKF43 denote the same embedded Runge-Kutta-Fehlberg
  pair of order 3 and 4. However, in RKF34 the result of the fourth order 
  submethod is accepted, whereas RKF43 advances using the result of the 
  third order submethod. In both cases the truncation error of the lower 
  order subprocess is estimated and controlled.

  For single methods such as EULER1 (the first order Euler method),
  RK4 (the classical fourth order Runge-Kutta method) or BUTCHER6 (a 
  Runge-Kutta method of order 6) the relative local error is controlled
  by comparing steps with different stepsizes. By local extrapolation 
  the effective order of the method is increased by one. Local 
  extrapolation is also available for the implemented embedded pairs. 
  For instance, the method xRKF78 uses extrapolation of the 8th order 
  subprocess of RKF78, yielding a method of effective order 9. However, 
  the cheap error estimate based on the embedded pair is not used in this 
  case implying some loss of efficiency when comparing RKF78 and xRKF78.

  The Butcher data of the implemented methods are available via 
  numeric::butcher(method), where method is one of EULER1, RKF43, RK4,
  RKF34, RKF54a, RKF54b, DOPRI54, RKF45a, RKF45b, DOPRI45, BUTCHER6, 
  RKF87, RKF78.

  The option Stepsize = h switches off the adaptive mechanism and
  uses a fixed stepsize h in the Runge-Kutta iteration. A final step 
  with smaller stepsize is used to match the end t of the integration
  interval t0 .. t .

  The call odesolve( t0..t0+h, f, y0, Symbolic, <,method> ) returns
  a list/array of expressions representing a single step of the numerical 
  scheme with step size h. In this mode symbolic values for t0, h, and 
  the components of y0 are admissible.

  With the option Alldata = n numeric::odesolve returns the list
  [ [t0,y0], [t1,y1] , .. ] , where [ t.i, y.i ] are accepted
  intermediate points generated by the Runge-Kutta iteration. This
  list may be useful, e.g., for further graphical processing of the
  output. The integer n controls the size of the returned list.
  For n=1 all accepted data are returned. This case may also be
  invoked by entering the simplified option Alldata, which is
  equivalent to Alldata = 1. For n > 1 only each n.th accepted 
  data point is stored in the list. The initial point [t0,y0] and
  the final point [t,y(t)] are always present in the output list. 
  For n<=0 only the initial and the final data [ [t0,y0], [t,y(t)] ] 
  are returned. 

  WARNING: only local errors are controlled by the adaptive mechanism.
  No control of the global error is granted.

  WARNING: the run time of the numeric integration with a method of
  order p grows like O(10^(DIGITS/(p+1)), when DIGITS increases.
  Computations with high precision are very expensive.

  WARNING: presently only explicit single step methods of Runge-Kutta
  type are implemented. Stiff problems cannot be handled efficiently
  with these methods.

  Example: 

>>  f := proc(t,y) begin  [ t*sin(y[1]) ] end_proc:

>>  numeric::odesolve( 0..10, f, [1]);

                               [3.141592653]

>>  numeric::odesolve( t0..t0+h, f, [y0], Symbolic, EULER1 );

                             [y0 + h t0 sin(y0)]

>>  f := proc(t,y) begin array(1..2,[ y[1]*y[2], y[1]/y[2] ]) end_proc:

>>  DIGITS:=20:                                      

>>  numeric::odesolve( 0..PI/3 , f, [ 1 , 1 ] , xRKF78 );  

               [5.4131223316245629744, 2.4238006323952340528]

>>  DIGITS:=4:

>>  numeric::odesolve( 0.. 1 , f, [1,1] , Stepsize=0.2 ,  Alldata = 2 );

     [ [0.0, [1.0, 1.0]], [0.4, [1.619, 1.419]], [0.8, [3.16, 1.955]],

       [1.0, [4.839, 2.322]] ]

>> "plot the numerical solution y(t) of dy/dt = t*sin(y)-y, y(0)=1";

>>  DIGITS:=10:

>>  f:= proc(t,y) begin [t*sin(y[1])-y[1]] end_proc;

>>  data := numeric::odesolve( 0..5 , f , [1] , Alldata ):

>>  plotpoints := [ point(data[i][1],data[i][2][1]) $ hold(i)=1..nops(data) ]:

>>  plot2d( [ Mode=List , plotpoints ] );


  See also:  numeric::butcher 
     
 ------------------------------------------------------------------------#

numeric::odesolve := proc(x0_xn, diffequ, y0)
local x0, xn, n, nn, tmp, i, method, adaptive, symbolic, alldata, datacount,
      nextentry, datatable, append_to_datatable, h, eps, eps1,
      extrapol1, extrapol2, lo_order, hi_order, data, getButcherData,
      stages, C, A, B, Bb, RKPairStep, RKStep, RK4Step, symbolicRKStep,
      adaptiveRKPairStep, adaptiveRKStep, adaptiveRK4Step, startingstepsize,
      DIGITS;
begin 

   if args(0) < 3 then error ("not enough arguments"); end_if;

   if type(x0_xn) <> "_range" then
      error("wrong specification of integration interval");
   end_if;
   x0:=op(x0_xn,1);
   xn:=op(x0_xn,2);

   case domtype(diffequ) 
      of DOM_EXEC do
      of DOM_PROC do break;
      otherwise 
      error("vector field should be presented by a function or a procedure");
   end_case;

   case domtype(y0) 
      of DOM_ARRAY do n := op(y0,[0,2,2]); break;
      of DOM_LIST do  n := nops(y0); break;
      otherwise error("initial condition should be a vector (list or array)");
   end_case;

   tmp:= diffequ(x0,y0);
   case domtype( tmp ) 
      of DOM_ARRAY do nn := op(tmp,[0,2,2]); break
      of DOM_LIST do  nn := nops(tmp); break;
      otherwise error("vector field yields wrong type of output");
   end_case;
   if n<>nn 
    then error("size of vector field and initial vector do not match");
   end_if;

   # --------   define local subroutine   ---------- #
   getButcherData:=proc(method)
   local i,b;
   begin ([stages,C,A,b,B,lo_order,hi_order]):=numeric::butcher(method);
         if stages=1 then Bb:=array(1..stages);Bb[1]:=B[1]-b[1]
         else Bb:=array(1..stages,[ (B[i]-b[i]) $ hold(i)=1..stages ])
         end_if;
   end_proc;
   # ---------------------------------------------- #

   adaptive := TRUE;                 # default #
   symbolic := FALSE;                # default #
   alldata  := FALSE;                # default #
   method:=1; getButcherData(RKF78); # default #
   if args(0) > 3 then               # get options #
     for i from 4 to args(0) do
       case op(args(i),1)
         of  EULER1   do method:=2;getButcherData(EULER1);break
         of  RK4      do method:=3;getButcherData(RK4);   break
         of  RKF34    do method:=1;getButcherData(RKF34); break
         of xRKF34    do method:=2;getButcherData(RKF34);break
         of  RKF43    do method:=1;getButcherData(RKF43); break
         of xRKF43    do method:=2;getButcherData(RKF43);break
         of  RKF45a   do method:=1;getButcherData(RKF45a);break
         of xRKF45a   do method:=2;getButcherData(RKF45a);break
         of  RKF54a   do method:=1;getButcherData(RKF54a);break
         of xRKF54a   do method:=2;getButcherData(RKF54a);break
         of  RKF45b   do method:=1;getButcherData(RKF45b);break
         of xRKF45b   do method:=2;getButcherData(RKF45b);break
         of  RKF54b   do method:=1;getButcherData(RKF54b);break
         of xRKF54b   do method:=2;getButcherData(RKF54b);break
         of  DOPRI45  do method:=1;getButcherData(DOPRI45);break
         of xDOPRI45  do method:=2;getButcherData(DOPRI45);break
         of  DOPRI54  do method:=1;getButcherData(DOPRI54);break
         of xDOPRI54  do method:=2;getButcherData(DOPRI54);break
         of  BUTCHER6 do method:=2;getButcherData(BUTCHER6);break
         of  RKF78    do method:=1;getButcherData(RKF78); break
         of xRKF78    do method:=2;getButcherData(RKF78); break
         of  RKF87    do method:=1;getButcherData(RKF87); break 
         of xRKF87    do method:=2;getButcherData(RKF87); break
         of Stepsize do adaptive:=FALSE;
                        if type(args(i)) = "_equal" 
                           then h:=op(args(i),2);
                           else 
 error("step size specification should be of the form Stepsize = h ");
                        end_if;
                        break
         of Symbolic do symbolic:=TRUE; break
         of Alldata  do alldata :=TRUE;
                        if type(args(i)) = "_equal" 
                           then nn:=op(args(i),2)
                           else nn:=1;
                        end_if;
                        break
         otherwise error("unknown argument");
       end_case;
     end_for;       
   end_if;

   if symbolic then
     # --------   define local subroutine   ---------- #
     symbolicRKStep:=proc(h, x0, y0)
     local i,j,tmp,ii,Y,k;
     begin k[1]:=diffequ(x0,y0);
           for i from 2 to stages do
               Y:=y0; # initialize storage for intermediate stages #
               for j from 1 to i-1 do
                 tmp:=A[i,j]*h; (Y[ii]:=Y[ii]+tmp*k[j][ii]) $ hold(ii)=1..n;
               end_for;
               k[i]:= diffequ(x0+C[i]*h,Y);
           end_for;
           Y:=y0;  # initialize storage for result #
           for j from 1 to stages do
               tmp:=B[j]*h; (Y[ii]:=Y[ii]+tmp*k[j][ii]) $ hold(ii)=1..n;
           end_for;
           return(Y)
     end_proc:    # ---------------------------------- #
     return(symbolicRKStep(xn-x0,x0,y0))
   end_if;

   eps := float(10^(-DIGITS));

   if method=1 then eps1 := eps/2^(lo_order+1)/2;
               else eps1 := eps/2^(hi_order+2)/2; 
   end_if;
    
   DIGITS:=DIGITS+2+trunc(ln(n+stages+2^(hi_order+2))/ln(10));

   x0:=float(x0);
   if domtype(x0)<>DOM_FLOAT then error("initial time not admissible");end_if;

   xn:=float(xn);
   if domtype(xn)<>DOM_FLOAT then error("final time not admissible"); end_if;

   if not adaptive then
      h:=float(h);
      if domtype(h)<>DOM_FLOAT then error("step size not admissible"); end_if;
   end_if;

   y0:=map(y0,float);
   C:=map(C,float);
   A:=map(A,float);
   B:=map(B,float);
   Bb:=map(Bb,float);

   if alldata then
      if domtype(nn)<>DOM_INT then
         error("use integer n in option Alldata = n");
      end_if;
      datatable[1]:= [x0,y0];
      datacount:=1; nextentry:=2;
      #------------  define local subroutine -------------#
      append_to_datatable:=proc(x,y)
      begin  
          if datacount=nn then datacount:=0; end_if;
          if datacount=0 or iszero(x-xn)
              then datatable[nextentry]:=[x,y];
                   nextentry:=nextentry+1;
          end_if;
          datacount:=datacount+1;
      end_proc;
      #---------------------------------------------------#
   end_if;

   extrapol2:= 1/( 2^hi_order -1 ); # factors for extrapolating results  #
   extrapol1:= 2^hi_order;          # when comparing stepsizes h and h/2 #

if adaptive then

    # --------   define local subroutine   ----------------------- #
    # first choice of stepsize following Hairer, Noersett, Wanner, #
    # Solving ODEs I, chapter II.5.1 : " Starting step size"       #
    
    startingstepsize:=proc(x0,y0)
    local vz,k1,k1norm,ii,den,h0,h1;
    begin if float(xn-x0) >= 0 then vz:=1 else vz:=-1; end_if;
          k1:=map(diffequ(x0,y0),float); 
          k1norm:= _plus( stdlib::abs(k1[ii])^2 $ hold(ii)=1..n )^(1/2);
          den:=float(   1/max(abs(x0),abs(xn))^(hi_order+1) 
                      +                 k1norm^(hi_order+1)  );
          h0:=float( (eps/den)^(1/(hi_order+1)) );
          # one Euler step to get away from y0 ,because this often leads #
          # to untypical starting vectors k1 with vanishing components   #
          (y0[ii]:=y0[ii]+vz*h0*k1[ii] ) $ hold(ii)=1..n;
          k1:=map(diffequ(x0+vz*h0,y0),float); 
          k1norm:= _plus( stdlib::abs(k1[ii])^2 $ hold(ii)=1..n )^(1/2);
          den:=float(   1/max(abs(x0),abs(xn))^(hi_order+1) 
                      +                 k1norm^(hi_order+1)  );
          h1:=float( (eps/den)^(1/(hi_order+1)) );
          return(vz*min(h0,h1));
    end_proc: #--------------------------------#

end_if;

if method=1 then

# --------   define local subroutine   ---------- #
RKPairStep:=proc(h, x0, y0)
local i,j,tmp,ii,Y,k,finalstep,localerror;
begin if args(0)=4 then k[1]:=args(4);
                   else k[1]:=map(diffequ(x0,y0),float);
      end_if;
      if stdlib::abs(h) >= stdlib::abs(xn-x0)
         then finalstep:=TRUE; h:=xn-x0;
         else finalstep:=FALSE;
      end_if;
      for i from 2 to stages do
          Y:=y0; # initialize storage for intermediate stages #
          for j from 1 to i-1 do
             if not iszero(A[i,j]) then
                 tmp:=A[i,j]*h; (Y[ii]:=Y[ii]+tmp*k[j][ii]) $ hold(ii)=1..n;
             end_if;
          end_for;
          k[i]:= map(diffequ(x0+C[i]*h,Y),float);
      end_for;
      Y:=y0;  # initialize storage for result #
      for j from 1 to stages do
         if not iszero(B[j]) then
            tmp:=B[j]*h; (Y[ii]:=Y[ii]+tmp*k[j][ii]) $ hold(ii)=1..n;
         end_if;
      end_for;
      localerror:=[ 0 $ n ];
      for j from 1 to stages do
         if not iszero(Bb[j]) then
             (localerror[ii]:=localerror[ii]+Bb[j]*k[j][ii]) $ hold(ii)=1..n;
         end_if;
      end_for;
      localerror:= stdlib::abs(h)*
           _plus(stdlib::abs(localerror[ii])^2 $ hold(ii)=1..n )^(1/2);
      if finalstep
         then return(xn  ,Y,localerror)
         else return(x0+h,Y,localerror)
      end_if;
end_proc:
# --------   define local subroutine   ---------- #
adaptiveRKPairStep:=proc(h,x0,y0)
local k1,finalstep,dummy,Y,localerror,Y_norm;
begin if stdlib::abs(h) >= stdlib::abs(xn-x0)
         then finalstep:=TRUE; h:=xn-x0; 
         else finalstep:=FALSE;
      end_if;
      k1:=map(diffequ(x0,y0),float);
      while TRUE do
        ([dummy,Y,localerror]):=[RKPairStep(h,x0,y0,k1)];
        Y_norm:=(_plus(stdlib::abs(Y[ii])^2 $ hold(ii)=1..n ))^(1/2);
        if localerror<=Y_norm*eps1
           then if finalstep
                   then return(h,xn,Y);     # final step: make sure x0+h=xn  #
                   else return(2*h,x0+h,Y); # accept, but suggest 2h as next #
                end_if;                     # stepsize                       #
        end_if;
        if localerror>Y_norm*eps then
             finalstep:=FALSE; h:=h/2; next; end_if;
                                           # try again with smaller stepsize #
        break;
      end_while;
      h,x0+h,Y;  # accept step, suggest h as stepsize for next step #
end_proc:

end_if;

if method=2 then
# --------   define local subroutine   ---------- #
RKStep:=proc(h, x0, y0)
local i,j,tmp,ii,Y,k,finalstep;
begin if args(0)=4 then k[1]:=args(4);
                   else k[1]:=map(diffequ(x0,y0),float);
      end_if;
      if stdlib::abs(h) >= stdlib::abs(xn-x0)
         then finalstep:=TRUE; h:=xn-x0;
         else finalstep:=FALSE;
      end_if;
      for i from 2 to stages do
          Y:=y0; # initialize storage for intermediate stages #
          for j from 1 to i-1 do
             if not iszero(A[i,j]) then
                 tmp:=A[i,j]*h; (Y[ii]:=Y[ii]+tmp*k[j][ii]) $ hold(ii)=1..n;
             end_if;
          end_for;
          k[i]:= map(diffequ(x0+C[i]*h,Y),float);
      end_for;
      Y:=y0;  # initialize storage for result #
      for j from 1 to stages do
         if not iszero(B[j]) then
            tmp:=B[j]*h; (Y[ii]:=Y[ii]+tmp*k[j][ii]) $ hold(ii)=1..n;
         end_if;
      end_for;
      if finalstep then return(xn,Y) else return(x0+h,Y); end_if;
end_proc:
# --------   define local subroutine   ---------- #
adaptiveRKStep:=proc(h,x0,y0)
local k1,finalstep,y,data,Y,Y_norm,ii,dy_norm;
begin if stdlib::abs(h) >= stdlib::abs(xn-x0)
         then finalstep:=TRUE; h:=xn-x0; 
         else finalstep:=FALSE;
      end_if;
      k1:=map(diffequ(x0,y0),float);
      y:=(RKStep(h,x0,y0,k1))[2];
      data:=RKStep(h/2,x0,y0,k1);
      while TRUE do
        Y:=(RKStep(h/2,data))[2];
        Y_norm:=(_plus(stdlib::abs(Y[ii])^2 $ hold(ii)=1..n ))^(1/2);
        (Y[ii]:=(extrapol1*Y[ii]-y[ii])*extrapol2) $ hold(ii)=1..n ;
        dy_norm:= (_plus(stdlib::abs(Y[ii]-y[ii])^2 $ hold(ii)=1..n ))^(1/2);
        if dy_norm<=Y_norm*eps1
           then if finalstep
                   then return(h,xn,Y);     # final step: make sure x0+h=xn  #
                   else return(2*h,x0+h,Y); # accept, but suggest 2h as next #
                end_if;                     # stepsize                       #
        end_if;
        if dy_norm>Y_norm*eps
           then h:=h/2; y:=data[2]; data:=RKStep(h/2,x0,y0,k1);
                finalstep:=FALSE;
                next; # try again with smaller stepsize #
        end_if;
        break;
      end_while;
      h,x0+h,Y;  # accept step, suggest h as stepsize for next step #
end_proc:

end_if;

if method=3 then
# --------   define local subroutine   ---------- #
RK4Step:=proc(h, x0, y0)
local finalstep,i,Y,k1,k2,k3,k4;
begin if stdlib::abs(h) >= stdlib::abs(xn-x0)
         then finalstep:=TRUE; h:=xn-x0;
         else finalstep:=FALSE;
      end_if;
      Y:=y0; # initialize storage of same type as y0 #
      if args(0)=4 then k1:=args(4) else k1:=map(diffequ(x0,y0),float) end_if;
      (Y[i]:=y0[i]+h/2*k1[i])$hold(i)=1..n;k2:=map(diffequ(x0+h/2,Y),float);
      (Y[i]:=y0[i]+h/2*k2[i])$hold(i)=1..n;k3:=map(diffequ(x0+h/2,Y),float);
      (Y[i]:=y0[i]+h  *k3[i])$hold(i)=1..n;k4:=map(diffequ(x0+h  ,Y),float);
      (Y[i]:=y0[i]+h/6*(k1[i]+2*(k2[i]+k3[i])+k4[i]))$hold(i)=1..n;
      if finalstep then return(xn,Y) else return(x0+h,Y); end_if;
end_proc:
# --------   define local subroutine   ---------- #
adaptiveRK4Step:=proc(h,x0,y0)
local k1,finalstep,y,data,Y,Y_norm,ii,dy_norm;
begin if stdlib::abs(h) >= stdlib::abs(xn-x0)
         then finalstep:=TRUE; h:=xn-x0; 
         else finalstep:=FALSE;
      end_if;
      k1:=map(diffequ(x0,y0),float);
      y:=(RK4Step(h,x0,y0,k1))[2];
      data:=RK4Step(h/2,x0,y0,k1);
      while TRUE do
        Y:=(RK4Step(h/2,data))[2];
        Y_norm:=(_plus(stdlib::abs(Y[ii])^2 $ hold(ii)=1..n ))^(1/2);
        (Y[ii]:=(extrapol1*Y[ii]-y[ii])*extrapol2) $ hold(ii)=1..n ;
        dy_norm:= (_plus(stdlib::abs(Y[ii]-y[ii])^2 $ hold(ii)=1..n ))^(1/2);
        if dy_norm<=Y_norm*eps1
           then if finalstep
                   then return(h,xn,Y);     # final step: make sure x0+h=xn  #
                   else return(2*h,x0+h,Y); # accept, but suggest 2h as next #
                end_if;                     # stepsize                       #
        end_if;
        if dy_norm>Y_norm*eps
           then h:=h/2; y:=data[2]; data:=RK4Step(h/2,x0,y0,k1);
                finalstep:=FALSE;
                next; # try again with smaller stepsize #
        end_if;
        break;
      end_while;
      h,x0+h,Y;  # accept step, suggest h as stepsize for next step #
end_proc:
end_if;

   if adaptive then
      data:=startingstepsize(x0,y0),x0,y0;
      case method 
      of 1 do repeat
                data:=adaptiveRKPairStep(data);
                if alldata then append_to_datatable(data[2],data[3]); end_if;
              until data[2]=xn end_repeat;
              break
      of 2 do repeat
                data:=adaptiveRKStep(data);
                if alldata then append_to_datatable(data[2],data[3]); end_if;
              until data[2]=xn end_repeat;
              break
      of 3 do repeat
                data:=adaptiveRK4Step(data);
                if alldata then append_to_datatable(data[2],data[3]); end_if;
              until data[2]=xn end_repeat;
              break
      end_case;
      if alldata then return([datatable[i] $ hold(i)=1..nops(datatable)])
                 else return(data[3]); 
      end_if;
   end_if;

   if not adaptive then
      data:=x0,y0;
      case method
      of 1 do repeat 
                data:=RKPairStep(h,data[1],data[2]);
                if alldata then append_to_datatable(data[1],data[2]); end_if;
              until data[1]=xn end_repeat;
              break 
      of 2 do repeat 
                data:=RKStep(h,data);
                if alldata then append_to_datatable(data[1],data[2]); end_if;
              until data[1]=xn end_repeat;
              break 
      of 3 do repeat
                data:=RK4Step(h,data);
                if alldata then append_to_datatable(data[1],data[2]); end_if;
              until data[1]=xn end_repeat;
              break 
      end_case;
      if alldata then return([datatable[i] $ hold(i)=1..nops(datatable)])
                 else return(data[2]); 
      end_if;
  end_if;

end_proc:


