
#------   file  singval.mu  17.9.97   ---------------------#
#  Help file:

 singularvalues - numerical singular values of a matrix

 Call:

 numeric::singularvalues(A) 
                                                                   
 Parameter:

  A     --     a square real or complex matrix (of domtype 
               DOM_ARRAY or Dom::Matrix )               
                                                                   
 Synopsis:

 The singular values of an (m x n) matrix A are the p=min(m,n)
 real non-negative square roots of the eigenvalues of A'A (if p=n)
 or of AA' (if p=m), where A' is the Hermitean transpose of A.

 numeric::singularvectors(A); returns the sorted list [d[1],..,d[p]]
 of singular values d[1] >= .. >= d[p] >= 0 .
                                                                 
 Remark: Non-float numerical entries are allowed in the matrix. 
         They will be evaluated to floats in the course of the 
         computation.  Non-numeric symbolic entries will lead to 
         an error.      

 Remark: The singular values may also be computed via
               numeric::eigenvalues(AA');
         or    numeric::eigenvalues(A'A);
         However, in comparison with the singular value 
         routine, for ill-conditioned matrices about twice 
         as many DIGITS are required in the eigenvalue 
         routine to produce results of similar quality.
         
 Examples:                                                         
                                                                  
 >> m := 3: n := 2: A := array(1..m,1..n,[[1,2*I ],[2,3],[3,4]]):

 >> numeric::singularvalues(A);                                   

                        [6.411107219, 1.377571855]

 See also:  numeric::eigenvalues, numeric::eigenvectors,
            numeric::singularvectors, numeric:: vonMises
                                                                  #
#-----------------------------------------------------------------#
numeric::singularvalues := proc(A)
local m, n, mgtn, oldA,
      real, i, j, k, l, d, e, 
      s, ss, c, h , f, g,
      eps, docancel, x, y, z;
begin
    if A::hasProp(Cat::MatrixCat)=TRUE    # A is of domtype Dom::Matrix  #
       then A:=A::convert_to(A,DOM_ARRAY) # convert to domtype DOM_ARRAY #
    end_if;                               # for speed #
    if domtype(A) <> DOM_ARRAY
       then error("matrix must be of domtype DOM_ARRAY or Dom::Matrix");
    end_if:
    m:=op(A,[0,2,2]): n:=op(A,[0,3,2]): 

    A:=map(A,float):
    if {op(map(A,domtype))} minus {DOM_FLOAT, DOM_COMPLEX} <> {} 
       then error("non-numeric entries not handled");
    end_if:
    if {op(map(A,(iszero@Im)))} = {TRUE}     # is matrix real ? #
       then real:=TRUE else real:=FALSE
    end_if:
    A:=subs(A,float(0)=0):

    if m>=n then mgtn:=TRUE;
    else mgtn:=FALSE; ([m,n]):=([n,m]);
         oldA:=A: A:=array(1..m,1..n);
         if real then ((A[i,j]:=oldA[j,i] )          $hold(i)=1..m)$hold(j)=1..n;
                 else ((A[i,j]:=conjugate(oldA[j,i]))$hold(i)=1..m)$hold(j)=1..n;
         end_if;
    end_if;
    d:=[0$n  ];  # initialize list to store diagonal       #
    e:=[0$n-1];  # initialize list to store super diagonal #
    for j from 1 to n do 
        # Householder step from the left to obtain upper   #
        # triangular form. Householder data are stored in  #
        # lower triangular part of A.                      #
        s:= _plus(stdlib::abs(A[i,j])^2 $ hold(i)=j+1..m);
        if iszero(s) then d[j]:=stdlib::abs(A[j,j]);
        else ss:=s+stdlib::abs(A[j,j])^2;
             s:=stdlib::sqrt(ss):
             h := 1 / (ss + stdlib::abs(A[j,j]) * s);
             if not iszero(A[j,j]) then s:=sign(A[j,j])*s;end_if:
             A[j,j]:=s+A[j,j]; d[j]:=stdlib::abs(s);
             for i from j+1 to n do
                 if real then   s:=_plus(A[k,j] *A[k,i]$hold(k)=j..m) ;
                 else s:=_plus(conjugate(A[k,j])*A[k,i]$hold(k)=j..m) ;
                 end_if;
                 f:=s*h;
                 (A[k,i]:=A[k,i]-f*A[k,j]) $ hold(k)=j..m ;
             end_for;
        end_if;
        # Householder step from the right to obtain upper  #
        # bidiagonal form. Householder data are stored in  #
        # strictly upper triangular part of A.             #
        s:= _plus(stdlib::abs(A[j,i])^2 $ hold(i)=j+2..n);
        if iszero(s) then if j<n then e[j]:=stdlib::abs(A[j,j+1]); end_if;
        else ss:=s+stdlib::abs(A[j,j+1])^2;
             s:=stdlib::sqrt(ss):
             h := 1 / (ss + stdlib::abs(A[j,j+1]) * s);
             if not iszero(A[j,j+1]) then s:=sign(A[j,j+1])*s; end_if:
             A[j,j+1]:=s+A[j,j+1]; e[j]:=stdlib::abs(s); 
             for i from j+1 to m do
                 if real then   s:=_plus(A[j,k] *A[i,k] $ hold(k)=j+1..n) ;
                 else s:=_plus(conjugate(A[j,k])*A[i,k] $ hold(k)=j+1..n) ;
                 end_if;
                 f:=s*h;
                 (A[i,k]:=A[i,k]-f*A[j,k]) $ hold(k)=j+1..n ;
             end_for;
        end_if;
    end_for;

    # now A contains in its lower/strictly upper part the #
    # Householder data of the left/right transformation.  #
    # The resulting real bi-diagonal matrix               #
    #    [ d[1] e[1]         ]                            #
    #    [      d[2] e[2]    ]  is stored in d and e.     #
    #    [           ..   .. ]                            #

# ------ now QR iteration to diagonalize bi-diagonal matrix ---- #

  e:=[0,op(e)];
  eps:=max(stdlib::abs(d[i])+stdlib::abs(e[i]) $ hold(i)=1..n)/10^DIGITS;
  for k from n downto 1 do
     repeat
       docancel:=TRUE;
       for l from k downto 1 do
          if stdlib::abs(e[l])<=eps then docancel:=FALSE; break end_if;
          if stdlib::abs(d[l-1]) <= eps then break end_if;
       end_for;
 
       if docancel then # cancellation of e[l] for l>1 #
          c:=0;s:=1;
          for i from l to k do
              f:=s*e[i];e[i]:=c*e[i]:
              if stdlib::abs(f)<=eps then break; end_if;
              g:=d[i]; h:=stdlib::sqrt(f^2+g^2); d[i]:=h; c:=g/h; s:=-f/h;
          end_for;
       end_if;          # end of cancellation #

       z:=d[k]; if l=k then break; end_if;

       # compute shift #
       x:=d[l]; y:=d[k-1]; g:=e[k-1]; h:=e[k];
       f:= ( (y-z)*(y+z)+(g-h)*(g+h) ) / (2*h*y);
       g:=stdlib::sqrt(1.0+f^2); 
       if f<0 then f:= ( (x-z)*(x+z)+h*( y/(f-g)-h )  )/x ;
              else f:= ( (x-z)*(x+z)+h*( y/(f+g)-h )  )/x ;
       end_if;
 
       # do QR step #
       c:=1;s:=1;
       for i from l+1 to k do
          g:=e[i]; y:=d[i]: h:=s*g; g:=c*g;
          z:=stdlib::sqrt(f^2+h^2);  e[i-1]:=z; c:=f/z;s:=h/z;
          f:=x*c+g*s; g:=-x*s +g*c; h:=y*s; y:=y*c;
          z:=stdlib::sqrt(f^2+h^2); d[i-1]:=z; c:=f/z; s:=h/z;
          f:=c*g+s*y; x:=-s*g+c*y;
       end_for;
       e[l]:=0; e[k]:=f; d[k]:=x;
     until FALSE end_repeat;
     if z<0 then d[k]:=-z; end_if;
  end_for;

  # ------ sort singular values from large to small ------ #   
  for j from 2 to n do              
      z:=d[j];                      
      for i from j downto 2 do      
         if d[i-1]>=z then break end_if:
         d[i]:=d[i-1]:
      end_for;
      d[i]:=z;
  end_for:   

  d;
end_proc:
#--------  end of procedure singularvalues ---------------#


