
#------   file  singvec.mu  17.9.97  ---------------------#
#  Help file:

 singularvectors - numerical singular value decomposition of a matrix

 Call:

 numeric::singularvectors(A <, NoLeftVectors> <, NoRightVectors> <, NoErrors>) 
                                                                   
 Parameter:

  A  --  a real or complex matrix of domtype DOM_ARRAY or Dom::Matrix               
                                                                   
Synopsis:

  The singular value decomposition of an (m x n) matrix A is
  a factorisation A = U D V', where
  --  U is a unitary (m x m) matrix,
  --  D is a sparse (m x n) matrix with real entries D[i,i]=d[i],
      i=1..p, p=min(m,n),
  --  V is a unitary (n x n) matrix.
  V' denotes the Hermitean transpose of V.
  The i.th column of U is an eigenvector of AA' associated with 
  the eigenvalue d[i], where d[i]=0 for i>p.
  The i.th column of V is an eigenvector of A'A associated with 
  the eigenvalue d[i], where d[i]=0 for i>p.
 
  numeric::singularvectors(A,Options); returns the list 
  [ U,d,V,resU,resV ], where                           
  -- U is the unitary (m x m) matrix of left singular vectors. 
  -- d=[ d[1],...,d[p] ] is a sorted list of real singular 
     values d[1] >= d[2] >= .. >= d[p] >= 0, p=min(m,n).
  -- V is a unitary (n x n) matrix of right singular vectors. 
     It is normalized such that its diagonal entries are real
     and non-negative.
  -- resU = [ resU[1],..,resU[m] ] is a list of residues
     associated with the left singular vectors:
             resU[i] = < A'u.i , A'u.i > - d[i]^2,
     where u.i= i.th column of U and <.,.> is the usual complex
     Euclidean scalar product.
  -- resV = [ resV[1],..,resV[n] ] is a list of residues
     associated with the right singular vectors:
             resV[i] = < A v.i , A v.i > - d[i]^2,
     where v.i= i.th column of V.

  The list d represents the (m x n) matrix
              [ d[1]  0  ..  ]
              [   0  ..  ..  ]  
          D = [  ..  .. d[p] ]  
              [   0  ..   0  ]
              [  ..  ..  ..  ]
              [   0  ..   0  ]
  if p=min(m,n)=n, or, if p=min(m,n)=m,
              [ d[1]  0  ..  0 ..  0 ]
          D = [   0  ..  .. .. .. .. ] 
              [  ..  .. d[p] 0 ..  0 ]

  U,V are of domtype Dom::Matrix, whence columns can be       
  extracted easily via  u.i := U::col(U,i); v.i := V::col(V,i);                      

  If only right/left singular vectors are required, then the
  options NoLeftVectors/NoRightVectors suppress computation of 
  U/V and the corresponding residues resU/resV. The returned
  values for these data are NIL.

  Computation of the residues resU/resV may be suppressed by
  the option NoErrors. In this case the returned values for
  resU/resV are NIL.
                                                                  
  Remark: Non-float numerical entries are allowed in the matrix.
          They will be evaluated to floats in the course of the 
          computation. Non-numeric symbolic entries will lead to 
          an error.      

  Remark: If no singular vectors are required, then
          the singular values obtained from
          numeric::singularvectors(A,NoLeftVectors,NoRightVectors);
          are identical to those computed by
            numeric::singularvalues(A);
 
  Remark: The left/right singular data may also be 
          computed via
           ([d2,U,errorsU]):= numeric::eigenvectors(AA');
          or
           ([d2,V,errorsV]):= numeric::eigenvectors(A'A);
          where d2= [ d[1]^2,d[2]^2,.. ].  However, in comparison 
          with the singular routines, for ill-conditioned matrices 
          about twice as many DIGITS are required in the eigen 
          routines to produce results of similar quality.
          
  Examples: 

  >>  m:=3: n:=2: A:=array(1..m,1..n,[[1,2*I ],[2,3],[3,4]]):
  >>  sv:=numeric::singularvectors:
  >> ([U,d,V,resU,resV]):=sv(A,NoErrors):
  >> ([U,d,V,resU,resV]):=sv(A,NoLeftVectors):
  >> ([U,d,V,resU,resV]):=sv(A,NoLeftVectors,NoErrors):
  >> ([U,d,V,resU,resV]):=sv(A,NoRightVectors):
  >> ([U,d,V,resU,resV]):=sv(A,NoRightVectors,NoErrors):
  >> ([U,d,V,resU,resV]):=sv(A,NoLeftVectors,NoRightVectors):
  >>
  >> ([U,d,V,resU,resV]):=sv(A):                                   
  >> U;
               array(1..3, 1..3,
                 (1, 1) =  0.115306134   + 0.2577906207 I,
                 (1, 2) = -0.5998682768  + 0.73999056 I,
                 (1, 3) = -0.07612303987 + 0.08382001921 I,
                 (2, 1) =  0.5600113945  - 0.04296510345 I,
                 (2, 2) =  0.0102273326  - 0.1333040615 I,
                 (2, 3) = -0.8074122747  - 0.1214581624 I,
                 (3, 1) =  0.7755694367  - 0.05728680461 I,
                 (3, 2) = -0.1863196488  - 0.1999560922 I,
                 (3, 3) =  0.5636491964  + 0.05303210188 I
               )

  >> d;

                        [6.411107219, 1.377571855]

  >> V;

   +-                                                               -+
   |          0.5556040651,         - 0.8263616549 - 0.09181796165 I |
   | 0.8263616549 - 0.09181796165 I,          0.5556040651           |
   +-                                                               -+

  >> resU;

            [6.938893904e-18, 8.741117365e-19, 8.008055265e-38]

  >> resV;

                    [2.083474391e-17, 4.415536526e-19]


See also:  numeric::eigenvalues, numeric::eigenvectors, 
           numeric::singularvalues, numeric::vonMises
                                                                  #
#-----------------------------------------------------------------#

numeric::singularvectors := proc(A)
local Options, m, n, mgtn, oldA, U, V, M,
      real, i, j, k, l,  d, e, 
      s, ss, c, h , hh , f, g,
      eps, docancel, x, y, z,
      withU, withV, withErrors, position, Y,
      resU,resV, r;
begin

    if A::hasProp(Cat::MatrixCat)=TRUE    # A is of domtype Dom::Matrix  #
       then A:=A::convert_to(A,DOM_ARRAY) # convert to domtype DOM_ARRAY #
    end_if;                               # for speed #
    if domtype(A) <> DOM_ARRAY
       then error("matrix must be of domtype DOM_ARRAY or Dom::Matrix");
    end_if:

    withU:=TRUE; withV:=TRUE; withErrors:=TRUE;
    Options:=[args(2..args(0))];
    if has(Options,NoLeftVectors)  then withU:=FALSE; end_if;
    if has(Options,NoRightVectors) then withV:=FALSE; end_if;
    if has(Options,NoErrors) then withErrors:=FALSE end_if;

    m:=op(A,[0,2,2]): n:=op(A,[0,3,2]): 

    A:=map(A,float):
    if {op(map(A,domtype))} minus {DOM_FLOAT, DOM_COMPLEX} <> {} 
       then error("non-numeric entries not handled");
    end_if:
    if {op(map(A,(iszero@Im)))} = {TRUE}     # is matrix real ? #
       then real:=TRUE else real:=FALSE
    end_if:
    A:=subs(A,float(0)=0): oldA:=A:

    if m>=n then mgtn:=TRUE;
    else mgtn:=FALSE;
         ([m,n]):=([n,m]);
         ([withU,withV]):=([withV,withU]);
         A:=array(1..m,1..n);
         if real then ((A[i,j]:=oldA[j,i] )          $hold(i)=1..m)$hold(j)=1..n;
                 else ((A[i,j]:=conjugate(oldA[j,i]))$hold(i)=1..m)$hold(j)=1..n;
         end_if;
    end_if;

    d:=[0$n  ];  # initialize list to store diagonal       #
    e:=[0$n-1];  # initialize list to store super diagonal #
    h:=[0$n];    # initialize list to store left factors   #
    hh:=[0$n];   # initialize list to store right factors  #
    # Householder transformation to reduce matrix to real  #
    # bi-diagonal form                                     #
    for j from 1 to n do 
        # Householder step from the left to obtain upper   #
        # triangular form. Householder data are stored in  #
        # lower triangular part of A.                      #
        s:= _plus(stdlib::abs(A[i,j])^2 $ hold(i)=j+1..m);
        if iszero(s) then d[j]:=A[j,j];
        else ss:=s+stdlib::abs(A[j,j])^2;
             s:=stdlib::sqrt(ss):
             h[j] := 1 / (ss + stdlib::abs(A[j,j]) * s);
             if not iszero(A[j,j]) then s:=sign(A[j,j])*s;end_if:
             A[j,j]:=s+A[j,j]; d[j]:=-s;
             for i from j+1 to n do
                 if real then   s:=_plus(A[k,j] *A[k,i]$hold(k)=j..m) ;
                 else s:=_plus(conjugate(A[k,j])*A[k,i]$hold(k)=j..m) ;
                 end_if;
                 f:=s*h[j];
                 (A[k,i]:=A[k,i]-f*A[k,j]) $ hold(k)=j..m ;
             end_for;
        end_if;
        # Householder step from the right to obtain upper  #
        # bidiagonal form. Householder data are stored in  #
        # strictly upper triangular part of A.             #
        s:= _plus(stdlib::abs(A[j,i])^2 $ hold(i)=j+2..n);
        if iszero(s) then if j<n then e[j]:=A[j,j+1]; end_if;
        else ss:=s+stdlib::abs(A[j,j+1])^2;
             s:=stdlib::sqrt(ss):
             hh[j] := 1 / (ss + stdlib::abs(A[j,j+1]) * s);
             if not iszero(A[j,j+1]) then s:=sign(A[j,j+1])*s; end_if:
             A[j,j+1]:=s+A[j,j+1]; e[j]:=-s; 
             for i from j+1 to m do
                 if real then   s:=_plus(A[j,k] *A[i,k] $ hold(k)=j+1..n) ;
                 else s:=_plus(conjugate(A[j,k])*A[i,k] $ hold(k)=j+1..n) ;
                 end_if;
                 f:=s*hh[j];
                 (A[i,k]:=A[i,k]-f*A[j,k]) $ hold(k)=j+1..n ;
             end_for;
        end_if;
    end_for;

    # now A contains in its lower/strictly upper part the #
    # Householder data of the left/right transformation.  #
    # The resulting bi-diagonal matrix                    #
    #    [ d[1] e[1]         ]                            #
    #    [      d[2] e[2]    ]  is stored in d and e.     #
    #    [           ..   .. ]                            #

if withV then
    V:=array(1..n,1..n,[[0$n]$n]): (V[j,j]:=1)$ hold(j)=1..n;
    for j from n-1 downto 1 do     
        if hh[j]<>0 then
           for i from j+2 to n do
               s:=_plus(A[j,k]*V[k,i] $ hold(k)=j+2..n) ;
               f:=s*hh[j];
               if real then (V[k,i]:=V[k,i]-f*A[j,k])$hold(k)=j+1..n;
               else (V[k,i]:=V[k,i]-f*conjugate(A[j,k]))$hold(k)=j+1..n;
               end_if;
           end_for;
           if real then (V[k,j+1]:= A[j,k]/e[j]) $ hold(k)=j+1..n ; 
           else (V[k,j+1]:=conjugate(A[j,k]/e[j])) $ hold(k)=j+1..n ; 
           end_if;
           V[j+1,j+1]:=V[j+1,j+1]+1;
        end_if;
    end_for;
end_if;
 
if withU then
    U:=array(1..m,1..m,[[0$m]$m]); (U[j,j]:=1)$hold(j)=1..m;
    for j from n downto 1 do    
      if h[j]<>0 then 
         for i from j+1 to m do
            if real then s:=_plus(A[k,j]*U[k,i] $ hold(k)=j..m) ;
            else s:=_plus(conjugate(A[k,j])*U[k,i] $ hold(k)=j..m) ;
            end_if;
            f:=s*h[j];
            (U[k,i]:=U[k,i]-f*A[k,j]) $ hold(k)=j..m ;
         end_for;
        (U[k,j]:=A[k,j]/d[j]) $ hold(k)=j..m;
        U[j,j]:=U[j,j]+1;
      end_if;
    end_for;
end_if;

   # now [d[1],..,d[n]] and [e[1],..,e[n-1]] are the diagonal and
     superdiagonal of a complex bidiagonal matrix. Apply further
     transformation with phase matrices 
        [beta1     ] [d1 e1      ] [conjug(alpha1)         ]
        [   beta2  ] [   d2 e2   ] [       conjug(alpha2)  ]
        [       ...] [      .. ..] [                   ... ]
     (with alpha1=1) to obtain real bidiagonal form.
     Update matrices U and V.
   #

   if not real then
     if iszero(d[1]) then f:=1
                     else f:=sign(d[1]):    # f= beta.1 #
                          d[1]:=stdlib::abs(d[1]); 
                          if withU then
                            (U[i,1]:=U[i,1]*f) $ hold(i)=1..m;
                          end_if;
     end_if;
     for j from 2 to n do
         if iszero(e[j-1]) then f:=1;       
                           else f:=f/sign(e[j-1]); # f = alpha.j #
                                e[j-1]:=stdlib::abs(e[j-1]);
                                if withV then
                                  (V[i,j]:=V[i,j]*f) $ hold(i)=1..n;
                                end_if;
         end_if;
         if iszero(d[j]) then f:=1;  
                         else f:=f*sign(d[j]);     # f = beta.j #
                              d[j]:=stdlib::abs(d[j]):
                              if withU then
                                 (U[i,j]:=U[i,j]*f) $ hold(i)=1..m;
                              end_if;
         end_if;
     end_for;
  end_if;

# ------ now QR iteration to diagonalize bi-diagonal matrix ---- #

  e:=[0,op(e)];
  eps:=max(stdlib::abs(d[i])+stdlib::abs(e[i]) $ hold(i)=1..n)/10^DIGITS;
  for k from n downto 1 do
     repeat
       docancel:=TRUE;
       for l from k downto 1 do
          if stdlib::abs(e[l])<=eps then docancel:=FALSE; break end_if;
          if stdlib::abs(d[l-1]) <= eps then break end_if;
       end_for;
 
       if docancel then # cancellation of e[l] for l>1 #
          c:=0;s:=1;
          for i from l to k do
              f:=s*e[i];e[i]:=c*e[i]:
              if stdlib::abs(f)<=eps then break; end_if;
              g:=d[i]; h:=stdlib::sqrt(f^2+g^2); d[i]:=h; c:=g/h; s:=-f/h;
 
              if withU then
                for j from 1 to m do    # update matrix U #
                   y:=U[j,l-1]; z:=U[j,i]:
                   U[j,l-1]:= y*c+z*s; U[j,i]:=-y*s+z*c;
                end_for;
              end_if;
 
          end_for;
       end_if;          # end of cancellation #

       z:=d[k]; if l=k then break; end_if;

       # compute shift #
       x:=d[l]; y:=d[k-1]; g:=e[k-1]; h:=e[k];
       f:= ( (y-z)*(y+z)+(g-h)*(g+h) ) / (2*h*y);
       g:=stdlib::sqrt(1.0+f^2); 
       if f<0 then f:= ( (x-z)*(x+z)+h*( y/(f-g)-h )  )/x ;
              else f:= ( (x-z)*(x+z)+h*( y/(f+g)-h )  )/x ;
       end_if;
 
       # do QR step #
       c:=1;s:=1;
       for i from l+1 to k do
          g:=e[i]; y:=d[i]: h:=s*g; g:=c*g;
          z:=stdlib::sqrt(f^2+h^2);  e[i-1]:=z; c:=f/z;s:=h/z;
          f:=x*c+g*s; g:=-x*s +g*c; h:=y*s; y:=y*c;
 
          if withV then
            for j from 1 to n do
               x:=V[j,i-1];z:=V[j,i];V[j,i-1]:=x*c+z*s;V[j,i]:=-x*s+z*c;
            end_for;
          end_if;

          z:=stdlib::sqrt(f^2+h^2); d[i-1]:=z; c:=f/z; s:=h/z;
          f:=c*g+s*y; x:=-s*g+c*y;

          if withU then
            for j from 1 to m do
               y:=U[j,i-1];z:=U[j,i];U[j,i-1]:=y*c+z*s;U[j,i]:=-y*s+z*c;
            end_for;
          end_if;

       end_for;
       e[l]:=0; e[k]:=f; d[k]:=x;
     until FALSE end_repeat;
     if z<0 then d[k]:=-z;
                 if withV then (V[j,k]:=-V[j,k]) $ hold(j)=1..n; end_if;
     end_if;
  end_for;

# ------ sort singular values from large to small ------ #   

  position:=[i$hold(i)=1..n];
  for j from 2 to n do              # search for sorted position  #
      z:=d[j];                      # of orig. eigenvalue d[j]    #
      for i from j downto 2 do      # which becomes d[position[j]]#
         if d[i-1]>=z then break end_if:
         d[i]:=d[i-1]: position[i]:=position[i-1]:
      end_for;
      d[i]:=z; position[i]:=j:
  end_for:   

if withU then
  Y:=U: 
  for j from 1 to n do
    if j<>position[j] then (U[i,j]:=Y[i,position[j]])$hold(i)=1..m end_if
  end_for;
end_if;

if withV then
  Y:=V: 
  for j from 1 to n do
    if j<>position[j] then (V[i,j]:=Y[i,position[j]])$hold(i)=1..n end_if
  end_for;
end_if;

#
  make diagonal of V positive real by U-> UP, V->VP with
  diagonal phase matrix P . Note  A = U D V^T = UP D (VP)^T.
#

if not mgtn then ([m,n]):=([n,m]);
                 ([U,V]):=([V,U]);
                 ([withU,withV]):=([withV,withU]);
end_if;

if withV then
   for j from 1 to n do
       if not iszero(V[j,j]) then
          f:=1/sign(V[j,j]);
          if float(f)<>float(1) then
             (V[i,j]:=V[i,j]*f) $ hold(i)=1..j-1;
              V[j,j]:=stdlib::abs(V[j,j]);
             (V[i,j]:=V[i,j]*f) $ hold(i)=j+1..n;
             if withU and j<=m then
                 (U[i,j]:=U[i,j]*f) $ hold(i)=1..m;
             end_if;
          end_if;
       end_if;
   end_for;
end_if;

# compute residues
      norm( A^T u.j )^2 - d[j]^2 of left sing. vectors u.j:=column(U,j)
  and norm( A v.j )^2 - d[j]^2  of right sing. vectors v.j:=column(V,j)
#

if withErrors
then
   if withU then
     resU:=[0 $ hold(j)=1..m];
     for j from 1 to m do
        for i from 1 to n do
           if real then r[i]:=_plus(oldA[k,i]*U[k,j] $ hold(k)=1..m);
           else r[i]:=_plus(conjugate(oldA[k,i])*U[k,j] $ hold(k)=1..m);
           end_if;
        end_for;
        if j<=min(m,n) then z:=d[j]^2 else z:=0 end_if;
        if real then   resU[j]:=stdlib::abs(_plus(r[i] *r[i]$hold(i)=1..n)-z);
        else resU[j]:=stdlib::abs(_plus(conjugate(r[i])*r[i]$hold(i)=1..n)-z);
        end_if;
     end_for;
   end_if;
   if withV then
     resV:=[0 $ hold(j)=1..n];
     for j from 1 to n do
        for i from 1 to m do
           r[i]:=_plus( oldA[i,k]*V[k,j] $ hold(k)=1..n);
        end_for;
        if j<=min(m,n) then z:=d[j]^2 else z:=0; end_if;
        if real then   resV[j]:=stdlib::abs(_plus(r[i] *r[i]$ hold(i)=1..m)-z);
        else resV[j]:=stdlib::abs(_plus(conjugate(r[i])*r[i]$ hold(i)=1..m)-z);
        end_if;
     end_for;
   end_if;
end_if;


  M:=Dom::Matrix():               # constructor of Dom::Matrix #
  if withU then U:=subs(M(U),float(0)=0) end_if;
  if withV then V:=subs(M(V),float(0)=0) end_if;

if withU and withV then 
 if withErrors then return([U,d,V,resU,resV]) else return([U,d,V,NIL,NIL]) end_if
end_if;
if withU then
 if withErrors then return([U,d,NIL,resU,NIL]) else return([U,d,NIL,NIL,NIL]) end_if
end_if;
if withV then
 if withErrors then return([NIL,d,V,NIL,resV]) else return([NIL,d,V,NIL,NIL]) end_if
end_if;

return([NIL,d,NIL,NIL,NIL]); 

end_proc:
#--------  end of procedure singularvectors ---------------#


