#----------------------------------------------------------------------#
# vonMises(A,x0,maxiterations)                                         #
#     - compute the spectral radius of a matrix by vector iteration    #
#                     x.i = (A^i x0)/norm(A^i x0)                      #
#       converging to the eigenspace associated with the spectral      #
#       radius                                                         #
#                                                                      #
# A  - a square real or complex matrix                                 #
# x0 - a starting vector to be provided by the user                    #
# maxiterations - maximal number of iterations to be used              #
#                                                                      #
# return value: list  [ rho , x , residue ]                            #
#                                                                      #
#         Here rho is an approximation of the absolutely largest       #
#         eigenvalue (i.e., abs(rho) is the spectral radius).          #
#         For hermitean A the Rayleigh quotient rho = <x,Ax> is        #
#         used with a normalized vector x:=A^i x0/ norm(A^i x0),       #
#         i some suitable power. For non hermitean A rho is            #
#         computed as absolute largest value of the form               #
#         rho = (Ax)[j]/x[j].                                          #
#                                                                      #
#         x = A^i x0 / norm(A^i x0) is the corresponding normalized    #
#         eigenvector (of the same domtype as the input vector x0).    #
#                                                                      #
#         The  residue := norm(Ax - rho x)/norm(x) provides an         #
#         error estimate. For hermitean matrices this is a             #
#         rigorous upper bound for the error abs(rho-rho_exact).       #
#                                                                      #
# warning: the iteration is terminated, if the approximations of the   #
#          absolutely largest eigenvalue becomes stationary within     #
#          the relative precision given by DIGITS.                     #
#          The iteration may not converge, if the absolute value of    #
#          the eigenvalue defining the spectral radius is not well     #
#          separated from the absolute value of the other eigenvalues. #
#          In this case the iteration is terminated after              #
#          maxiterations with a warning.                               #
#                                                                      #
# The routine vonMises is an ad-hoc (and not very professional)        #
# implementation of the power method to compute the largest eigenvalue #
# and a corresponding eigenvector.                                     #
#                                                                      #
# Example:                                                             #
#                                                                      #
#   n:=2:                                                              #
#   Hilbert:= array(1..n,1..n,                                         #
#                [ [1/(i+j-1) $ hold(i)=1..n] $ hold(j)=1..n ]):       #
#   x0:=array(1..n, [1$n]):                                            #
#   data:= vonMises(Hilbert,x0,1000);                                  #
#   --            +-                          -+                 --    #
#   | 1.267591879,| 0.8816735651, 0.4718598569 |, 6.931409507e-12 |    #
#   --            +-                          -+                 --    #
#                                                                      #
#   eigenvalue:=op(data,1);                                            #
#                              1.267591879                             #
#   eigenvector:=op(data,2);                                           # 
#                            +-                          -+            #
#                            | 0.8816735651, 0.4718598569 |            #
#                            +-                          -+            #
#   error:=op(data,3);                                                 #
#                              6.931409507e-12                         #
#                                                                      #
#----------------------------------------------------------------------#

numeric::vonMises := proc(A,x0,maxiter)
local n,oldDIGITS,hermitean,iter,nnorm,x,i,y,j,rho,residue;
begin 
   if A::hasProp(Cat::MatrixCat)=TRUE
      then A:=A::convert_to(A,DOM_ARRAY)
   end_if;
   if domtype(A) <> DOM_ARRAY
     then error("matrix must be of domtype DOM_ARRAY or Dom::Matrix");
   end_if:
   if op(A,[0,1])<>2 then error("first argument is not a matrix"):end_if:
   n:=op(A,[0,2,2]):
   if n <> op(A,[0,3,2]) then error("not a square matrix"): end_if:
   oldDIGITS := DIGITS: DIGITS := DIGITS + trunc(n/10):
   hermitean:=TRUE:
   for i from 1 to n do
    for j from 1 to i do
        if A[i,j]<>conjugate(A[j,i]) then hermitean:=FALSE;break;end_if:
    end_for:
    if not hermitean then break;end_if:
   end_for:
   A:=map(A,float):
   if {op(map(A,domtype))} minus {DOM_FLOAT, DOM_COMPLEX} <> {}  
      then error("non-numeric entries not handled"); 
   end_if: 
   A:=subs(A,float(0)=0):
   x:=x0:            # make sure x is of same domtype as x0 #
   y:=map(x0,float): # initialize and make sure y is of same domtype as x0 #
   for iter from 1 to maxiter do
      nnorm  :=stdlib::sqrt(_plus( stdlib::abs(y[i])^2 $ hold(i)=1..n )):
      (x[i]  :=y[i]/nnorm) $ hold(i)=1..n:
      (y[i]  :=_plus(A[i,j]*x[j] $ hold(j)=1..n ) ) $ hold(i)=1..n:
      if hermitean # use Rayleigh quotient rho=<x,Ax> to extract eigenvalue #
      then rho:=_plus(conjugate(x[i])*y[i] $ hold(i)=1..n ):
      else rho:=0; # use rho=max((Ax)[i]/x[i]) to extract eigenvalue #
           for i from 1 to n do 
             if not iszero(x[i]) and stdlib::abs(rho*x[i])<stdlib::abs(y[i])
                then rho:=y[i]/x[i];
             end_if;
           end_for;
      end_if;
      residue:=_plus(stdlib::abs(y[i]-rho*x[i])^2 $ hold(i)=1..n ):
      if stdlib::abs(residue)< 10^(-oldDIGITS)*stdlib::abs(rho) 
                                                  then break: end_if:
      if iter=maxiter 
           then warning("no convergence of vector iteration [numeric::vonMises]");
      end_if;
   end_for:
   DIGITS:=oldDIGITS:
   [rho,x,residue]:
end_proc:
#--------  end of procedure vonMises -------------------#

