
#++ ---------------- ExpSol.mu ---------------------

Author: Eckhard Pflgel,  University of Grenoble
        Winfried Fakler,  University of Karlsruhe

Date Created: 13 August    1996
Last Update : 30 September 1997


Description:
This file contains functions for computing exponential solutions
of homogeneous linear ordinary differential equations 
over the field of rational functions. 

Functions: 

 - used Parameter:
    ++ Ly, y(x), v:DOM_EXPR
    ++ y, x       :DOM_IDENT
    ++ n          :DOM_INT
    ++ Ly    = ordinary linear homogeneous differential equation 
    ++         over the rational functions.
    ++ y(x)  = the operator (function) of Ly  
    ++ y     = name of the operator of Ly
    ++ x     = dependent variable of the operator y
    ++ n     = order of Ly

 - ode::exponentialSolutions(Ly, y(x), < "Generic" >)
    ++ determines all exponential solutions of Ly, i.e. solutions z such that
    ++ z'/z is a rational function.
    ++ When the option "Generic" is given, a generic form of them is returned.
    ++ NOTE: currently this function works properly only if all singularities
    ++ are in Q and no occuring candidate is involved with algebraic numbers.
    ++ Therefore, actually this function is far away from being a 
    ++ decision procedure. :-( 
 - ode::expsols(Ly, y, x, n)
    ++ determines all exponential solutions of Ly, i.e. solutions z such that
    ++ z'/z is a rational function.
    ++ NOTE: Ly should be normalized i.e. denom(Ly)=1.
    ++ NOTE: currently this function works properly only if all singularities
    ++ are in Q and no occuring candidate is involved with algebraic numbers.
    ++ Therefore, actually this function is far away from being a 
    ++ decision procedure. :-(   
 
See:
 - Bronstein, M. (1992). Linear Ordinary Differential Equations: breaking
   through the order 2 barrier. In: Proceedings of ISSAC'92, pp. 42-48.
 - Pflgel, E. (1997). An Algorithm for Computing Exponential Solutions of
   First Order Linear Differential Systems. In: Proceedings of ISSAC'97,
   pp. 164-171.

Examples:

++#

# Dependencies:
 
  read("isLODE.mu"): #

sysassign(ode::exponentialSolutions, proc(eq, z, o) #o: Option "Generic" #
  local sol;
  begin
    eq := ode::isLODE(rewrite(eq, diff), z, "HlodeOverRF");
    if nops(eq)<> 4 then 
      return(error("not an ordinary homogeneous linear differential ".
                           "equation over the rational functions")) end_if;    
    eq[1] := numer(normal(expr(eq[1])));
    #ode::expsols(eq);#
    sol := map(map(map( ode::expsols(eq) ,expand),simplify),combine,exp);
    if o="Generic" then
       _plus(op(map(sol, _mult, genident("C")))) 
     else sol end_if
  end_proc):



#*****************************************************************************#
#                                                                             #
#             An implementation of the p-adic Newton algorithm for            #
#           computing exponential solutions of linear diff. equations         #
#                                                                             #
#                              MuPAD version 1.1                              #
#                                                                             #
#*****************************************************************************#

# Last change: 02.05.96 EP first version #
# Last change: 29.07.96 EP bug fix, algebraic numbers #
# Last change: 12.09.96 WF ported to MuPAD 1.2.9a 
                           changed local variable polygon in ntPolygon in
                            function ode::padicNewton  
                           changed local variable phi in Phi in
                            function ode::conjTrace and ode::modSolve #
# Last change: 20.09.96 WF added top level function ode::exponentialSolutions #
# Last change: 09.11.96 WF adapted function ode::exponentialSolutions #
# Last change: 26.01.97 WF added rewrite in function ode::exponentialSolutions#
# Last change: 24.04.97 WF bugfix in ode::changeRiccati, 
                           comment out ode::polysols #
# Last change: 08.05.97 WF work around for a serious bug in padicExpParts
                           whereby we have to accept to loose solutions 
                           perhaps #
# Last change: 26.08.97 WF work around for a serious bug in padicSingParts
                           whereby we have to accept to loose solutions 
                           perhaps #

#setuserinfo(ode::expsols, 5):
 setuserinfo(ode::padicNewton, 5):#



# 
  ode::expsols(eq, y, x, n)
    INPUT:  eq - a linear differential equation with polynomial coefficients
            y  - a name (the unknown)
            x  - a name
            n  - an integer (the order of eq)
    OUTPUT: the set of all solutions y of eq with y'/y \in \bar(Q)(x)
#
# 
    This is a first incomplete version computing all solutions
    
    y with y'/y \in Q(x) 
# 
sysassign(ode::expsols, proc(eq, y, x, n)
    local lode, l, m, i, j, d, p, parts, parts1, part, places, 
          f, P, res, u;
begin
    lode := [];
    lode:=ode::vectorize(eq,y,x,n);
    l:=factor(lode[n+1]);
    m := nops(l) div 2;
    # Compute potential singularities #
    for i from 1 to m do p[i]:=l[2*i] end_for; 
    p[m+1] := 1/x;
    parts := []; places := [];
    for i from 1 to m+1 do
        userinfo(2, "computing p-adic exponential parts at: ", p[i]);
        # Compute p-adic exponential parts #
        part := ode::padicNewton(lode, x, p[i], n);
         # Eliminate apparent singularities #
        if part <> [[1, 0, 0]] then
            parts := append(parts, part);
            places := append(places, p[i])
        end_if;
        userinfo(3, "found: ", part);
        if part = [] then 
            userinfo(1, "proof for no exponential solutions");
            return({})
        end_if; 
    end_for;
    # compute possible denominator of rational part #
    userinfo(3, "computing rational part");
    d := 1;
    for i from 1 to nops(parts) do
        for j from 1 to nops(parts[i]) do
            d := lcm(d, parts[i][j][1]);
            parts[i][j] := [parts[i][j][2], parts[i][j][3]]
        end_for
    end_for;
    # and bound it #
    userinfo(3, "bounding denominator:", d);    
    lode := ode::changeRiccati(lode, x, -diff(d, x)/d);
    # eliminate equivalent exponential parts #
    parts1 := [];
    j := 0;
    f := fun([args(1)[2], args(1)[3]]);
    for i from 1 to nops(parts) do
        if parts[i] <> [[0, 0]] then 
            parts1 := append(parts1, parts[i]) 
        end_if
    end_for;
    parts := parts1;
    # combine all p-adic parts #
    parts := ode::possiblePadicParts(parts);
    res := {};
    # test for polynomial solutions #
    for part in parts do
         u := diff(part[1], x)+part[2];
         userinfo(3, "checking Riccati solution:", u);
         for P in ode::polysols(ode::changeRiccati(lode, x, u), x) do
             userinfo(3, "polynomial solution found:", P);
             res := res union {normal(P/d)*exp(part[1]+int(part[2], x))};
         end_for;
         if nops(res) = n then break end_if;
    end_for;
    res;
end_proc):


# The p-adic Newton algorithm #

# 
  ode::padicNewton(lode, x, p, m)                                             
    INPUT:  lode - list of coefficients of a linear differential operator    
            x    - a name                                                     
            p    - an irreducible polynomial or 1/x (representing infinity)   
            m    - an integer                                                 
    OUTPUT: the list of exponential parts of L at p - at most m parts         
#                                                                            
sysassign(ode::padicNewton, proc(lode, x, p, m)
    local u, m1, res, ntPolygon, c, slope;
begin
    res := [];
    userinfo(2, "computing Newton polygon");
    ntPolygon := ode::newtonPolygon(lode, x, p, u, m)[3];
    m1 := m;
    while m1 > 0 do
        slope := ntPolygon[1][1];
        c := ntPolygon[1][2];
        if slope = 0 then
            userinfo(5, "slope = ", slope,
                        "indicial equation =", c);
            if p <> 1/x then
                res := res . ode::padicSingParts(c, x, u, p);
            else
                res := [[1, 0, 0]] 
            end_if;
            m1 := m1  - degree(c, [u]);
        else
            if type(slope) = DOM_INT then
                userinfo(5, "slope = ", slope,
                            "characteristic equation =", c);
                res := res . ode::padicExpParts(lode, c, x, u, p, slope);
            end_if;
            m1 := m1 - degree(c, [u])*denom(slope)
        end_if;
        ntPolygon[1] := NIL
    end_while;
    res
end_proc):

# ode::padicSingParts(c, x, u, p) 
    INPUT:  c - a bivariate polynomial in x and u    
            x - a name                   
            u - a name                                                        
            p - an irreducible polynomial in x (or 1/x, representing infinity)
    OUTPUT: the list of p-adic singular parts associated to the generalized   
            exponents ui of the indicial equation c(ui,x) = 0 mod p(x)        
            only the representant mod Z are returned                          
#                                                                             
sysassign(ode::padicSingParts, proc(c, x, u, p)
    local res, numers, num, i, j, QPhi, P, R, Phi, num1, rest, tr,h;
begin
    res := [];
    numers := ode::modSolve(c, x, u, p);
    for i from 1 to nops(numers) div 2 do numers[i+1] := NIL end_for;
    numers := ode::reps(numers);
    #print(numers, "numers");#
    for i from 1 to nops(numers) do 
        num := numers[i];
        if (type(num) = DOM_INT) then
            if (num < 0) then 
                res := append(res, [p^(-num), 0, 0]) 
            else
                res := append(res, [1, 0, 0])
            end_if
        else
            if degree(p, x) > 1 then
                # We now interprete num as algebraic number: replace in num 
                  the variable x by Phi where p(Phi) = 0. We then have to 
                  compute the algebraic trace i.e. the sum of all conjugates 
                  of num/(x-Phi)., #
                QPhi := Dom::AlgebraicExtension(Dom::Rational, subs(p, x=Phi));
                P := Dom::DistributedPolynomial([x], QPhi);
                R := Dom::Fraction(P);
                # We divide p by x-Phi in QPhi #
                rest := R::numer(R::normalize(P(p), P(x-Phi)));
                # We then make the denominator rational by multiplying 
                  with rest #    
                num1 := P::_mult(P(subs(num, x=Phi)), rest);
                # Now we compute the algebraic trace of all coefficients 
                  of num1 #
                tr := 0;
                for j from 0 to degree(p, [x])-1 do
                    tr := tr + x^j*ode::conjTrace(P::coeff(num1, x, j), p, x)
                end_for;

    # this if statement stands for res := append(res, [1, 0, tr/p]) #
                if iszero(traperror( (h:=tr/p) )) then
                  res := append(res, [1, 0, h])
                else
                  userinfo(1, "NOTE: cannot treat this part in ".
                            "'padicSingParts'. \n ".
                  "        Try to skip it! Perhaps we will loose solutions.".
                  "\n ");
                  res:=append(res,[1,0,0])
                end_if;

                # This is the corresponding Maple command #
                #res := res,[1, 0, sum(subs(x = a, num[1])/(x-a), 
                                       a = RootOf(p))];#
            else

    # this if statement stands for res := append(res, [1, 0, num/p]) #
                if iszero(traperror( (h:=num/p) )) then
                  res := append(res, [1, 0, h])
                else
                  userinfo(1, "NOTE: cannot treat this part in ".
                            "'padicSingParts'. \n ".
                  "        Try to skip it! Perhaps we will loose solutions.".
                  "\n ");
                  res:=append(res,[1,0,0])
                end_if;

            end_if
        end_if
    end_for;
    if res <> [] then
        userinfo(5, "p-adic singular parts found: ", res)
    end_if;
    res
end_proc):

# ode::reps(lambda)                                                           
    INPUT:  lambda - a list of pairs [u, m] where u is a p-adic exponent     
                     and m its multiplicity                                  
    OUTPUT: a sublist l of lambda such if [u, m] in l, then there is         
            no positive integer i with [u+i, m] in l.                        
#                                                                            
sysassign(ode::reps, proc(lambda)
    local reps, parts, tmp, p, p1, d;
begin
    reps := {};
    tmp := {op(lambda)};
    parts := tmp;
    while parts <> {} do 
        p := op(parts, 1); 
        for p1 in parts do
            d := expr(p-p1);
            if type(expr(d)) = DOM_INT then
                tmp := tmp minus {p1};
                if d > 0 then p := p1 end_if
            end_if;
        end_for;
        reps := reps union {p};
        parts := tmp
    end_while;
    [op(reps)]
end_proc):

# This is an implementation of the algebraic trace. However, it would be     
  better to include it into the domain "AlgebraicExtension".             # 

#                                                                            
  ode::conjTrace(alpha, P, x)                                                 
    INPUT:  alpha - an element of an algebraic extension over the rationals  
            P     - the minimal polynomial of the extension                  
            x     - the variable of P                                        
    OUTPUT: its algebraic trace, i.e. the sum of the conjugates              
#                                                                            
sysassign(ode::conjTrace, proc(alpha, P, x)
    local tmp, a, Phi, i, tr; 
begin
    tmp := extop(alpha, 1);
    a := op(tmp, 1); 
    Phi := op(tmp, 2)[1];
    a := subs(a, Phi = x);
    tr := coeff(a, x, 0);
    for i from 1 to degree(P)-1 do
        a := divide(a*x, P, [x], Rem);
        tr := tr + coeff(a, x, i)
    end_for;
    tr
end_proc):
    
# ode::padicExpParts(c, x, u, p, k)                                           
   INPUT:  c - a bivariate polynomial in x and u                             
           x - a name                                                        
           u - a name                                                        
           p - an irreducible polynomial in x or 1/x (representing infinity)
           k - an integer                                                    
   OUTPUT: the list of p-adic exponential parts associated to the slope k and
           the p-adic characteristic equation c(u, x)                        
# 


sysassign(ode::padicExpParts, proc(lode, c, x, u, p, k)
    local i, c1, numers, part, rparts, res;
begin
    res := [];
    c1 := subs(c, u = -u*k);
    numers := ode::modSolve(c1, x, u, p);
    for i from 1 to nops(numers) div 2 do
        if iszero(traperror( (part := numers[2*i-1]/p^k))) 
          then part
          else 
            userinfo(1, "NOTE: cannot treat this part in ".
                            "'padicExpParts'. \n ".
                  "        Try to skip it! Perhaps we will loose solutions.".
                  "\n ");
            return( res . [[1,0,0]] )
          end_if;
        rparts := ode::padicNewton(
                     ode::changeRiccati(lode, x, diff(part, x)), x, 
                                   p, numers[2*i]);
        res := res . ode::insert(part, rparts);
    end_for;
    res
end_proc): 


# ode::insert(part, l)                                                        
   INPUT:  part - an expression                                              
           l    - a list of expressions                                      
   OUTPUT: if l is the empty list then [1, part, 0]                          
           else replace each element e of the form [*, e', *] in l           
           by [*, part+e', *].                                               
#                                                                             
sysassign(ode::insert, proc(part, l)
    local f;
begin
    if l = [] then [[1, part, 0]] else
        f := fun([args(1)[1], args(1)[2]+args(2), args(1)[3]]);
        map(l, f, part)
    end_if
end_proc):

# ode::possiblePadicParts
    INPUT:  parts - a list of p-adic exponential parts
    OUTPUT: all possible sums of elements of the different exponential 
            parts
#
sysassign(ode::possiblePadicParts, proc(part) 
    local p1, p2, pp, parts, res;
begin
    if part = [] then return([[0, 0]]) end_if;
    pp := part[1];
    part[1] := NIL;
    parts := ode::possiblePadicParts(part);
    res := [];
    for p1 in pp do
        for p2 in parts do
            res := append(res, [p1[1]+p2[1], p1[2]+p2[2]])
        end_for
    end_for;
    res
end_proc):

# Tools for computing Newton polygons and valuations #

#
 ode::newtonPolygon(lode, x, p, u[, m])                                      
    INPUT:  lode - list of coefficients of linear differential operator       
            x    - a name                                                     
            p    - a polynomial or 1/x                                        
            u    - a name                                                     
            m    - (optional) an integer                                      
    OUTPUT: the Newton polygon and of lode at place p as list                 
            [minimal y-value, u, list of couples [slope, newton polynomial]]  
            the polygon is constructed only for lode[i], i = 0,..,m           
#                                                                             
sysassign(ode::newtonPolygon, proc(lode, x, p, u, m)
    local lode1, n, s, yp, lcs, drop, npoly, d, i, j, k, slope, 
          tmp, res, dp;
begin
    lode1 := lode;
    if args(0) = 5 then n := m+1 else n := nops(lode) end_if;
    if p = 1/x then s := -1; dp := 1 else s := 1; dp := diff(p, x) end_if;
    yp := []; lcs := [];
    for i from 1 to n do
       tmp := ode::valuation(lode1[i], x, p);
       yp  := append(yp, tmp[1]-s*(i-1));
       lcs := append(lcs, tmp[2])
    end_for;
    drop := min(op(yp));
    npoly := 0;
    for j from 1 to n do
        if yp[j] = drop then  
            npoly := npoly + ode::tau(lcs[j], x, p, j-1)*
                             _mult(s*u-k+1$hold(k)=1..j-1)
        end_if
    end_for;
    d := degree(npoly, [u]);
    if d > 0 then res := [[0, collect(normal(primpart(npoly, [u])), [u])]] 
        else res := [] 
    end_if;
    i := d+1;
    while i < n do
        slope := min(seq((yp[j]-yp[i])/(j-i), j=[$(i+1..n)]));
        npoly := 0;
        for j from i to n do
            if yp[j] -(j-i)*slope = yp[i] then  
                npoly := npoly + ode::tau(lcs[j], x, p, j-i)*
                                 (s*u)^((j-i)/denom(slope))
            end_if
        end_for;
        res := append(res, [slope,collect(normal(primpart(npoly, [u])), [u])]);
        i := i+degree(npoly, [u])*denom(slope);
    end_while;
    [drop, u, res] 
end_proc):

# ode::val(f, x, p)                                                           
    INPUT:  f - a rational function
            x -  a name
            p - a polynomial or 1/x         
    OUTPUT: the list [v, lc] where v is the p-valuation of f and lc the       
            leading coefficient of f at p.                                    
#
sysassign(ode::val, proc(f, x, p)
    local t1, t2, G;
begin   
    t1 := ode::valuation(numer(f), x, p);
    t2 := ode::valuation(denom(f), x, p);
    if (p = x) or (p = 1/x) then 
        return([t1[1]-t2[1], t1[2]/t2[2]])
    else
        G := gcdex(t2[2], p, x);
        return(t1[1]-t2[1], divide(t1[2]*G[2], p, [x], Rem))
    end_if;
end_proc):

# ode::valuation(a, x, p)                                                     
    INPUT : a - a polynomial
            x - a name
            p - a polynomial or 1/x                
    OUTPUT: the list [v, lc] where v is the p-valuation of f and lc the       
            leading coefficient of a at p.                                    
#
sysassign(ode::valuation, proc(a, x, p)
    local ord, tmp;
begin
    if (a = 0) then 
        return([infinity, 0])
    elif (p = 1/x) then
        return([-degree(a, x), lcoeff(a, [x])])
    elif (p = x) then
        return([ldegree(a, x), tcoeff(a, [x])])
    end_if;
    ord := 0; tmp := a;
    repeat
        tmp := divide(a, p, [x]);
        if iszero(tmp[2]) then ord := ord+1; a := tmp[1] else break end_if;
    until FALSE end_repeat;
    [ord, divide(a, p, [x], Rem)]
end_proc):

# ode::tau(lc, x, p, i)                                                       
    INPUT:  lc - a univariate polynomial in x of degree < degree(p)           
            x  - a name                                                       
            p  - an irreducible polynomial in x                               
            i  - an integer                                                   
    OUTPUT: the ith remainder of a polynomial whose leading coefficient       
            is lc at p                                                        
#
sysassign(ode::tau, proc(lc, x, p, i)
begin   
    if (p = x) or (p = 1/x) then return(lc)
        else return(divide(lc*diff(p, x)^i, p, [x], Rem))
    end_if
end_proc):

# Additive change of riccati solution #

# ode::changeRiccati(L, x, u)                                                 
   INPUT:  L - list of coefficients of linear differential operator          
               with derivation d/dx                                          
           x - a name                                                        
           u - an expression                                                 
   OUTPUT: list of coefficients of linear differential equation which        
           results from the change of unknown variable y(x) to               
           exp(int(u))*z(x).                                                 
#
sysassign(ode::changeRiccati, proc(L, x, u)
    local n, i, j, T, cs, d;
begin
    n := nops(L);
    T := array(1..n+1);
    T[1] := 1;
    for i from 2 to n+1 do
        T[i] := (normal(diff(T[i-1], x)+u*T[i-1])) 
    end_for;
    cs := array(1..n);
    d := 1;
    for j from 1 to n do
        cs[j] := 0;
        for i from 1 to n-j+1 do
            cs[j] := cs[j] + (binomial(j+i-2, j-1)*T[i]*L[j+i-1])
        end_for;
        cs[j] := normal(cs[j]);
        d := lcm(d, denom(cs[j]))
    end_for;
    for j from 1 to n do cs[j] := normal(cs[j]*d) end_for;
    #cs #
    [op(cs)]  # WF 24.04.97 #
end_proc):
       
# Solving equations modulo an irreducible polynomial #

# ode::modSolve(c, x, u, p)                                                   
  INPUT:  c - a multivariate polynomial 
          x - a name                                                         
          u - a name                                                         
          p - an irreducible polynomial (or 1/x, representing infinity)      
  OUTPUT: a list of all solutions u \in C[x], deg(u) < deg(p) of c(x, u) = 0 
          (mod p) and their multiplicities                                   
#                                                                            
sysassign(ode::modSolve, proc(c, x, u, p) 
    local res, d, f, i, s, Qphi, P, C, Phi;
begin
    if p = 1/x then d := 1 else d := degree(p, [x]) end_if;
    res := [];
    if d = 1 then
        f := factor(poly(c, [u]));
        #print(f);#
        for i from 1 to (nops(f)-1) div 2 do
            s := f[2*i];
            if degree(s, u) > 1 then
                Qphi:=Dom::AlgebraicExtension(Dom::Rational, subs(s, u=Phi));
                #P:=Dom::DistributedPolynomial([u],Qphi);#
                res := append(res, Qphi(Phi), f[2*i+1])
            else
                s := op(solve(s, u));
                res := append(res, s, f[2*i+1]);
                userinfo(5, "solution", s);
            end_if
        end_for
    else
        Qphi:=Dom::AlgebraicExtension(Dom::Rational, subs(p, x=Phi));
        P:=Dom::DistributedPolynomial([u],Qphi);
        C := P(subs(c, x=Phi));
        f := factor(C);
        for i from 1 to (nops(f)-1) div 2 do
            s := f[2*i];
            if P::degree(s, u) > 1 then
                userinfo(1, "NOTE: Algebraic extension occurred - ".
                      "incomplete solutions");
                res := append(res, 0, f[2*i+1])
            else
                s := subs(op(s, 1), Phi=x);
                s := op(solve(s, u));
                res := append(res, s, f[2*i+1]);
                userinfo(5, "solution", s)
            end_if
        end_for
    end_if;
    res
end_proc):






