
#++ ---------------- ifReducible.mu ---------------------

Author: Winfried Fakler,  University of Karlsruhe
Date Created: 18 September 1996
Last Update : 29 December  1997


Description:
This file contains functions for solving *reducible* ordinary linear 
(homogeneous) differential equations over the rational functions.
 

Functions: 

 - used Parameter:
    ++ Ly, y(x), v:DOM_EXPR
    ++ y, x       :DOM_IDENT
    ++ n          :DOM_INT
    ++ fl         :DOM_LIST
    ++ Ly    = ordinary linear homogeneous differential equation 
    ++ y(x)  = the operator (function) of Ly  
    ++ y     = name of the operator of Ly
    ++ x     = dependent variable of the operator y
    ++ n     = order of Ly
    ++ v     = special nonzero solution of Ly
    ++ fl    = list of Ly, the (irreducible) factors of an ordinary linear 
    ++         homogeneous differential equation, where the last factor of fl 
    ++         corresponds to right factor of the equation

 - ode::solveIfReducible(Ly, y(x))
    ++ returns the liouvillian solutions of Ly if Ly is reducible, 
    ++ otherwise an empty set is returned.
    ++ NOTE: currently this function only works for second and third order 
    ++       equations for which the function ode::expsols works. 
 - ode::ifReducible(Ly, y, x, n)
    ++ returns the liouvillian solutions of Ly if Ly is reducible, 
    ++ otherwise an empty set is returned. 
    ++ NOTE: currently this function only works for second and third order 
    ++       equations for which the function ode::expsols works.
 - ode::firstOrder(Ly,y,x)
    ++ returns the liouvillian solution of the first order equation Ly. The
    ++ integral inside will not be evaluated.
 - ode::reducibleCase(Ly, y, x, n)
    ++ returns (maybe not all) liouvillian solutions of a reducible 
    ++ n-th order equation Ly. This function cannot be a decision procedure
    ++ since there is neither (currently) a complete algorithm for 
    ++ factorization of the corresponding differential operator nor an 
    ++ algorithm for solving liouvillian solutions of order >3. 
    ++ NOTE: currently this function only works for second and third order 
    ++       equations for which the function ode::expsols works.
 - ode::combineSolutionsOfFactors(fl,y,x,n)
    ++ returns liouvillian solutions, which can be determined by solving 
    ++ each factor and combining the solutions via variation of parameters.
    ++ Thus it is not a decision procedure for *all* liouvillian solutions,
    ++ see also ode::reducibleCase.

See:
 - Walter, W. (1990). Gew"ohnliche Differentialgleichungen. 4th Edition.
   Berlin, Heidelberg, New York: Springer.
   (for Variation of Parameters)

++#

# Dependencies:

    ExpSol.mu, dAlembert.mu, tools.mu, Lodo.mu
#

sysassign(ode::solveIfReducible, proc(eq, z)
  begin
    eq := ode::isLODE(rewrite(eq, diff), z, "HlodeOverRF");
    if nops(eq)<> 4 then 
      return(error("not an ordinary homogeneous linear differential ".
                           "equation over the rational functions")) end_if;    
    #eq[1] := numer(normal(expr(eq[1])));#
    ode::ifReducible(eq)
  end_proc):

sysassign(ode::ifReducible, proc(eq, y, x, n) 
  local sols, dim, v, w, EF, lodo, Df, eq_adj, eq2, fac, b, e,de;
  begin
    if n=1 then return(ode::firstOrder(eq,y,x)) end_if;    
    if n>3 then 
      error("currently only implemented up to third order equations") end_if;
    sysassign(ode::isReducibleFlag, FALSE);
    sols := ode::expsols(numer(normal(expr(eq))), y, x, n);
    dim  := nops(sols);
    if dim=n then return( sols ) end_if;
    EF   := Dom::ExpressionField(normal);  
    lodo := Dom::LinearOrdinaryDifferentialOperator(Df,x,EF);
    if iszero(dim) then
      userinfo(2, "found no exponential solution");
      if n=2 then return({}) 
        elif n=3 then    # try to find exponential solutions of the adjoint #
                         # equation                                         #
          userinfo(2,"try to find exponential solutions of the adjoint");
          eq_adj:= lodo::adjoint(lodo::mkLODO(eq,y(x))); 
          sols  := ode::expsols(numer(normal(expr(eq_adj(y(x),Unsimplified)))),
                                 y,x,n);
          dim   := nops(sols); 
          if iszero(dim) then return({})
            else         # dim needs to be equal 1 #
              sysassign(ode::isReducibleFlag, TRUE);
              fac := lodo::mkLODO([normal(-diff(op(sols),x)/op(sols)), 1]);
              eq2 := lodo::adjoint(lodo::rightQuotient(eq_adj, fac));
              eq2 := eq2(y(x),Unsimplified);
              sols:= ode::secondOrder(eq2,y,x, "Irreducible");
              dim   := nops(sols); 
              if iszero(dim) then return({})
               elif dim=1 then  # cannot treat this case currently, since #
                                # this means that we have only implicit   # 
                                # solutions given by a minimal polynomial #
                                #  --> solutions=RootOf(....)             #
                 userinfo(1,"NOTE: only found implicit solutions. \n".
                            "         Cannot search for further solutions");
                 return(sols)
                else     # search for a third solution by variation of  #
                         # parameters                                   #
                  userinfo(2, "two solutions found, searching for a ".
                              "third solution");
                  b := ode::firstOrder(
                              lodo::adjoint(fac)(y(x),Unsimplified),y,x);
                  b := op(b)/coeff(eq2,[diff(y(x),x,x)],1);
                  v := [op(sols)];
                  w := b/(ode::Wronskian(v,x,EF));
                  return( sols union 
                         {-v[1]*hold(int)(v[2]*w,x)+v[2]*hold(int)(v[1]*w,x)})
               end_if;
           end_if;
       end_if
     else  # found exponential solutions, try to find further solutions #
       sysassign(ode::isReducibleFlag, TRUE);
       v :=op(sols);
       if n=2 then
         userinfo(2, "found an exponential solution and".
                     " reduce the equation now...");
         de:=ode::secondOrderReduction(ode::normalize(eq,y,x,2),y,x,v);
         w := op(ode::firstOrder(de, y, x));
         return( map(map( sols union { v*hold(int)(w,x) },expand),simplify) )
       elif n=3 then
         # search for further solutions by variation of parameters #
         userinfo(2, "found an exponential solution, try to find further ".
                     "solutions"); 

         # should distinguish dim=1 or 2 but I don't feel to do it! #
         w   := ode::normalize(ode::Wronskian([v,y(x)],x,EF),y,x,dim);
         fac := lodo::mkLODO(w,y(x)); 
         #  w   := ode::Wronskian([v,y(x)],x,EF);     #
         #  fac := lodo::monic(lodo::mkLODO(w,y(x))); #
         eq2 := lodo::rightQuotient(lodo::mkLODO(eq,y(x)), fac);
         b := ode::liouvillianSolutions(eq2(y(x),Unsimplified),y(x) #,
                                        "Irreducible"# );
         if iszero(nops(b)) then return(sols) end_if;
         if dim=1 then
           for e in b do
             sols := sols union {v*hold(int)(e/v,x)};
           end_for;
           return(sols)
         else
           v := [v];
           w := op(b)/(ode::Wronskian(v,x,EF));
           return( sols union 
                    {-v[1]*hold(int)(v[2]*w,x)+v[2]*hold(int)(v[1]*w,x)})
         end_if
       end_if
    end_if  
  end_proc):

sysassign(ode::firstOrder, proc(eq, y, x)
  local peq;
  begin
    peq := ode::ode2poly(eq,y,x,1);
    {exp(hold(int)(
      #normal#(expr(-coeff(peq,y(x),1))/expr(coeff(peq,diff(y(x),x),1))),x))}
  end_proc):


sysassign(ode::reducibleCase, proc(eq, y, x, n) 
  local EF, lodo, Df, fl;
  begin
    if n=1 then return(ode::firstOrder(eq,y,x)) end_if;
    EF    := Dom::ExpressionField(normal);  
    lodo  := Dom::LinearOrdinaryDifferentialOperator(Df,x,EF); 
    userinfo(1, "factorize equation");
    fl    := lodo::factor(lodo::mkLODO(eq, y(x)));
    if iszero(nops(fl)) then return({}) end_if;
    fl    := map(fl, lodo::func_call, y(x), Unsimplified);
    ode::combineSolutionsOfFactors(fl,y,x,n)
  end_proc):

sysassign(ode::combineSolutionsOfFactors, proc(fl,y,x,n) 
  local i,sols,lsols,giveUp,ord,wr,w,v,u,nu,k,h,s,nls,EF;
  begin
    sols  := {};
    giveUp:= FALSE;
    i     := nops(fl);
    if iszero(i) then return(sols) end_if;
    userinfo(1, "solve factors and combine their solutions");
    while not(iszero(i) or giveUp) do 
      # tho following lines are written instead of:                  #
      # lsols:=ode::liouvillianSolutions(fl[i],y(x),"Irreducible");  #
      lsols:={};
      ord  :=ode::getOrder(fl[i],y(x));
      userinfo(5, "solve factor of degree", ord);
      case ord
        of 0 do break;
        of 1 do lsols:= {ode::firstord(fl[i],y,x)};
                        #ode::firstOrder(fl[i],y,x);# break;
        of 2 do if n>3 then lsols:=ode::secondOrder(fl[i],y,x)
                       else lsols:=ode::secondOrder(fl[i],y,x,"Irreducible")
                end_if;
                # this if-statement is necessary since currently the #
                # lodo factorization is not complete.                #
                break;
       otherwise 
         userinfo(1, "NOTE: Cannot treat factors of degree >= 3".
                                   " currently. \n".
                         "         Hence, we may lose some ".
                         "Liouvillian solutions.");
      end_case;
      nls:=nops(lsols);
      if iszero(nls) then break; end_if;

      # the following can only occure, when factor fl[i] is reducible 
        or we have found implicit solutions #
      if ord<>nls then 
        giveUp:=TRUE;
        if ord=2 and nls=1 then 
           userinfo(1,"NOTE: Found implicit solutions. \n".
                 "         Hence, cannot search for further solutions.");
        end_if;
      end_if;

      if iszero(nops(sols)) then sols:=lsols
       else

         # variation of parameters #

         userinfo(5, "combine solutions by variation of parameters");
         u := [op(sols)];
         EF:= Dom::ExpressionField(normal);
         wr:= normal(ode::Wronskian(u,x,EF));
         w :=[]; nu:=nops(u);
         if nu=1 then w := [1] else
           for k from 1 to nu do
             h := u[k]; u[k] := NIL;
             w := append(w,normal(ode::Wronskian(u,x,EF)));
             u := linsert(u,h,k);
           end_for;
         end_if;
         for v in lsols do 
           s := _plus(u[k]*(-1)^(nu+k)*hold(int)(normal(v*w[k]/wr),x) 
                      $hold(k)=1..nu);
           sols := sols union {s};   
         end_for;

      end_if;
      i:=i-1;
    end_while;
    sols
  end_proc):
   


