 #++ ---------------- PolySol.mu ---------------------

Author: Eckhard Pflgel,  University of Grenoble
        Paul Zimmermann,  INRIA, Lorraine
        Winfried Fakler,  University of Karlsruhe

Date Created: 14 February 1997
Last Update : 30 December 1997


Description:
This file contains functions for computing polynomial solutions of linear 
ordinary differential equations over the field of rational functions. 
The algorithm used here computes polynomial solutions monomial by monomial.

Functions:

 - used Parameter:
    ++ Ly, y(x):DOM_EXPR
    ++ A, b    :DOM_EXPR
    ++ y, x    :DOM_IDENT
    ++ n       :DOM_INT
    ++ Ly   = ordinary linear differential equation
    ++ y(x) = the operator (function) of Ly  
    ++ y    = name of the operator of Ly
    ++ x    = dependant variable of the operator y
    ++ n    = order of Ly
    ++ b    = rational function in x, may contain some parameters
    ++ A    = Ly in vectorized form (list of coefficients)

 - ode::polynomialSolutions(Ly, y(x), < "Generic" >)
    ++ returns a fundamental system of polynomial solutions of an ordinary
    ++ linear differential equation Ly over the rational functions.
    ++ When the option "Generic" is given, a generic form of them is returned.
 - ode::polysols(Ly, y, x, n,< b >)
   ode::polysols(A, x,< b >) 
    ++ finds the polynomial solutions of the linear differential  equation 
    ++ Ly = b. The output is the general polynomial solution and a set of 
    ++ conditions on the parameters that must hold. eq must be normalized, 
    ++ i.e. denom(eq)=1. 
    ++ NOTE: Ly needs to be homogeneous!
    ++ library call: Ly can also be a list of coeffs (from ode::vectorize)
    ++ Then the call is ode::polysols(Ly, x).
  
See:
 - Abramov, S.A., Kvansenko, K.Yu. (1991). Fast Algorithms to Search for the
   Rational Solutions of Linear Differential Equations with Polynomial 
   Coefficients. In: Proceedings of ISSAC '91, Bonn, Germany, 267-270.
 - Barkatou, M. (1997). An Efficient Algorithm for Computing Rational 
   Solutions of Systems of Linear Differential Equations. 
   Submitted to ISSAC '97

Examples:

>> ode::polynomialSolutions(-8*y(x)+8*x*diff(y(x),x)+156*x^2*diff(y(x),x$2)
	+258*x^3*diff(y(x),x$3)+122*x^4*diff(y(x),x$4)
	+20*x^5*diff(y(x),x$5)+x^6*diff(y(x),x$6),y(x));

                                    {x}

>> ode::polysols([-8,8*x,156*x^2,258*x^3,122*x^4,20*x^5,x^6],x);

                                    {x}

>> ode::polysols([12*x^3+6*x^2-2*x-3,8*x^4+2*x^3-5*x^2-x,x^5-x^3],x);

                                    {}

>> wy := ode::normalize(ode::Wronskian([x^2,x+1,x^3-x+1,y(x)],x),y,x,3):
>> ode::polynomialSolutions(wy,y(x));
 
                              3              2
                            {x  + 2, x + 1, x }
 
>> ode::polysols(wy,y,x,3);
 
                              3              2
                            {x  + 2, x + 1, x }
 
>> Ly := y(x)*4 + x*y(x)*3-x*diff(y(x), x) + x*diff(y(x), x, x)*2:
>> ode::polysols(Ly,y,x,2, a+b*x^10);
 
{                                                                  2
{ --       52911104 b --    13227776 b   18129664 b x   1225472 b x
{ |  a = - ----------  |, - ---------- + ------------ - ------------
{ --          729     --       729           729            243
 
               3            4            5                    7        8
     435520 b x    69208 b x    10888 b x          6   412 b x    5 b x
   - ----------- + ---------- + ---------- - 36 b x  - -------- + ------ +
         81            81           27                    27        9
 
      9 }
   b x  }
   ---- }
    3   }

>> Ln:=n*x*diff(y(x),x,x)-x*diff(y(x),x)+n^2*y(x):
   time((s:=ode::polynomialSolutions(subs(Ln,n=10),y(x))));
 
                                    910
++#

sysassign(ode::polynomialSolutions, proc(eq, z, o) #o: Option "Generic" #
  local sol;
  begin
    eq := ode::isLODE(rewrite(eq, diff), z, "HlodeOverRF");
    case nops(eq) 
      of 1 do return(error("not an ordinary homogeneous linear differential ".
                            "equation over the rational functions"));
      of 4 do if eq[4] < 1 then
                 return(error("only defined for positive order equations"))
              end_if;
    end_case;
    eq[1] := numer(normal(expr(eq[1])));
    sol := ode::polysols(eq, 0);
    if o="Generic" then
       _plus(op(map(sol, _mult, genident("C")))) 
     else sol end_if
  end_proc): 
 
# An implementation of the algorithm which computes polynomial solutions of  #
# linear differential equations monomial by monomial                         #
# See M.Barkatou "An Efficient Algorithm for Computing Rational Solutions of #
# Systems of Linear Differential Equations, submitted to ISSAC '97           #

# created:     14.02.97 EP #
# last change: 14.02.97 EP #
# change: arguments of polysols rearranged, 
          output is now standard output of ode-lib,
          added variable s to be local and removed local variable sm
          in polysols  14.03.97 WF #

# polysols(eq, y, x, n, b) finds the polynomial solutions of the linear
  differential  equation eq(y(x)) = b where b is rational function. b may
  contain parameters. n denotes the order of eq.
  The output is the general polynomial solution and a set of conditions on
  the parameters that must hold. eq must be normalized, i.e. denom(eq)=1     #

sysassign(ode::polysols, proc(eq, y, x, n)
    local a, k, params, c, b1, mon, lambda, h, axn, i, P, ir, sol, tmp, bb,
          s, conds, N, b, cis, bbhascis;
begin
    if type(eq)=DOM_LIST then a:=eq; x:=y; n:=nops(a)-1; b:=0
    else
       if args(0)=4 then b := 0 else b := args(5) end_if;
       a := ode::vectorize(eq,y,x,n);  
    end_if;

    # makes the coefficients polynomial in x #
    userinfo(15, "make coefficients polynomial");
    h:=lcm(op(map(map(a,denom),poly,[x])));
    if degree(h)<>0 then h:=expr(h); a:=map(map(a,_mult,h),normal); 
	b:=normal(b*h) end_if; 
    params := indets(b) minus {x};

    # action of the homogeneous equation on the monomial x^lambda #
    userinfo(15, "precompute action L(x^lambda)");
    mon:=1;
    axn := a[1]*mon*x^n;
    for i from 2 to n+1 do
        mon:=mon*(lambda+2-i);
        axn := axn+a[i]*mon*x^(n+1-i)
    end_for;

    userinfo(5, "computing height of Newton polygon at infinity");
    # height of the Newton polygon of eq at infinity #
    axn:=poly(axn,[x]);
    h:=degree(axn);
    axn:=divide(axn,poly(x^ldegree(axn),[x]),Exact);
    # make h = 0 #
    axn:=expr(axn)/x^degree(axn);

    # new rhs member #
    b1 := normal(b/x^(h-n));

    # indicial equation of the homogeneous equation at infinity #
    userinfo(5, "computing indicial equation at infinity");
    P := ode::lc_inf(axn, x);

    # list of the integer roots of the indicial equation #
    ir := sharelib::iroots(P);
    ir := sort([op(ir)]);
    while (ir <> []) and (ir[1] < 0) do
        N := nops(ir);
        ir := [op(ir, [2..N])];
    end_while;
    userinfo(5, "positive integer roots are: ", ir);

    # call the procedure 'nextMon' #
    tmp := ode::nextMon(axn, b1, x, P, lambda, ir, 0, k, c, {});
    cis := c[i] $hold(i)=0..max(0,op(ir));
    bb := tmp[2];
    conds := op(tmp[3]);

    if bb = 0 then # WF #
     //  return({coeff(poly(tmp[1], [cis])), conds} minus {0})  # PZ #
      if iszero(nops(conds)) then
        return({coeff(poly(tmp[1],[cis]))} minus {0})
       else
        return({coeff(poly(subs(tmp[1],conds),[cis]))} minus {0})
      end_if;
    else # WF # 
        if has(bb, [cis]) then
          s := solve({coeff(numer(bb),[x]), conds}, {cis});
          bbhascis := TRUE
         else
          s := solve({coeff(numer(bb),[x]), conds});
          bbhascis := FALSE end_if;
        if s={} then return({}) end_if;
        s := op(s);
        sol := {coeff(poly(subs(tmp[1],s),[cis]))} minus {0};
        if sol={} then
            return({})  
        elif bbhascis then 
               return(sol)  
          else return(sol union {conds, s})
        end_if
    end_if; 
end_proc):

# this procedure computes the leading monomial of a possible polynomial
  solution and iterates #
sysassign(ode::nextMon, proc(axn, b, x, P, lambda, ir, sol, k, c, conds)
    local lcb, delta, cc, N, r, g;
begin
  N := nops(ir);
  # use fraction-free representation: b=b[1]/b[2] where b[1] and b[2]
    are both polynomials, same for axn #
  b:=map(stdlib::normal(b),poly,[x]);
  axn:=map(stdlib::normal(axn),poly,[x]);
  repeat
    # valuation and leading coefficient of rhs member #
    delta:=(if iszero(b[1]) then -infinity 
            else degree(b[1])-degree(b[2]) end_if);
    lcb:=lcoeff(b[1])/lcoeff(b[2]);
    # stop, when all positive roots are taken into account and the valuation
      of b is negative #

    if (ir = [] or ir[N] < 0) and (delta < 0) then
        return([sol, expr(b[1])/expr(b[2]), conds])
    end_if;

    # Case 1: there is a (maximal) positive integer root not treated which is
      > delta #
    if (N > 0) and (ir[N] > delta) then
        g:=multcoeffs(axn[1]*b[2]*poly(x^ir[N],[x]),c[ir[N]]);
        b:=subs([b[1]*axn[2]-g,b[2]*axn[2]],lambda=ir[N]);
        userinfo(12, "free monomial found, degree is", ir[N]);
        sol:=sol+c[ir[N]]*x^(ir[N]); ir[N]:=NIL; N:=N-1;

    # Case 2: there is no positive integer root not treated which is > delta
              (this includes the case where all integer roots are treated) #
    elif subs(P, lambda = delta) <> 0 then
        # Case 2a: delta is no root of the indicial equation #
        cc := lcb/subs(P,lambda=delta);
        g:=(if delta=-infinity then 0 else
                multcoeffs(axn[1]*b[2]*poly(x^delta,[x]),cc) end_if);
        b:=subs([b[1]*axn[2]-g,b[2]*axn[2]],lambda=delta);
        sol:=sol+cc*x^delta;
        userinfo(12, "monomial found, degree is", delta);
    else
        # Case 2b: delta is a root of the indicial equation #
        userinfo(20, "enter solve, system of ".nops(lcb).
                     " equations, of total size ".length(lcb));
        r := linsolve({lcb},indets(lcb,RatExpr));
        if r=FAIL then
            return([sol, expr(b[1])/expr(b[2]), conds])
        else
           userinfo(20, "back from solve, ".nops(r).
                        " solution(s) of total size ".length(r));
	   b:=subs(b, r); sol:=subs(sol, r); conds:=conds union r;
        end_if;
    end_if
  until FALSE end_repeat
end_proc):

# leading coefficient of a rational function f at infinity #
sysassign(ode::lc_inf, proc(f, x)
begin
    if f = 0 then
        return(0)
    else
        return(lcoeff(numer(f), [x])/lcoeff(denom(f), [x]))
    end_if;
end_proc):












