
#++ ---------------- RatSol.mu ---------------------

Author: Winfried Fakler,  University of Karlsruhe
Date Created:  8 August   1996
Last Update : 24 October  1997


Description:
This file contains functions for computing polynomial and rational solutions
of linear ordinary differential equations over the field of rational functions.

Functions: 

 - used Parameter:
    ++ Ly, y(x), R:DOM_EXPR
    ++ y, x       :DOM_IDENT
    ++ n, m, mu   :DOM_INT
    ++ A, a, ci   :DOM_LIST
    ++ P, Q       :DOM_POLY
    ++ Ly   = ordinary linear differential equation over the rational functions
    ++ y(x) = the operator (function) of Ly  
    ++ y    = name of the operator of Ly
    ++ x    = dependant variable of the operator y
    ++ n    = order of Ly
    ++ m    = exponent of the symmetric power of Ly
    ++ mu   = sum of exponents of Ly
    ++ ci   = vectorized form Ly: [c_0,...,c_n] (denominators needs to be 
    ++        all of degree 0 in x)
    ++ A    = vectorized form Ly (i.e. all coefficients of Ly as polynomials)
    ++        [c_0,...,c_n] 
    ++ R    = Ansatz for rational solutions with a bounded but indeterminate
    ++        numerator and determined denominator
    ++ a    = list of the indeterminates of the numerator of R
    ++ P, Q = polynomials

 - ode::rationalSolutions(Ly, y(x), < "Generic" >)
    ++ returns a fundamental system of rational solutions of an ordinary
    ++ linear differential equation Ly over the rational functions.
    ++ When the option "Generic" is given, a generic form of them is returned.
 - ode::ratsols(Ly, y, x, n)
 - ode::ratsols(ci, x)
    ++ returns a fundamental system of rational solutions of an ordinary
    ++ linear differential equation Ly over the rational functions.
    ++ NOTE: Ly should be normalized (i.e. denom(Ly)=1).
 - ode::rationalInvariants(Ly, y(x), m)
    ++ returns a fundamental system of rational solutions of the m-th
    ++ symmetric power of an ordinary linear differential equation Ly
    ++ over the rational functions.
 - ode::ratInvars(Ly, y, x, n, m)
    ++ returns a fundamental system of rational solutions of the m-th
    ++ symmetric power of an ordinary linear differential equation Ly
    ++ over the rational functions.
    ++ NOTE: Ly should be normalized (i.e. denom(Ly)=1).
 - ode::denomAnsatz(A, x, n, < "mu" >)
    ++ returns the Ansatz of the numerator for rational solutions of A.
    ++ When the option "mu" is given, the sum of the exponents 
    ++ are returned additionally.
    ++ NOTE: A should be normalized (i.e. all denominators must be one).
 - ode::nu(Q, P, < Rem >)
    ++ computes the order of Q at P, which is the largest integer m
    ++ such that P^m divides Q.
    ++ When the option Rem is given, Q/P^m is returned.


See: e.g. M. Bronstein (1992). Integration and Differential Equations in
     Computer Algebra.  Programmirovanie 18, Nr. 5, pages 9-10,
     http://www.inf.ethz.ch/personal/bronstein/
 - Abramov, S.A., Kvansenko, K.Yu. (1991). Fast Algorithms to Search for the
   Rational Solutions of Linear Differential Equations with Polynomial 
   Coefficients. In: Proceedings of ISSAC '91, Bonn, Germany, 267-270.
 - Barkatou, M. (1997). An Efficient Algorithm for Computing Rational 
   Solutions of Systems of Linear Differential Equations. 
   Submitted to ISSAC '97


Acknowledgement: 
This implementation is based on the ode::ratsols implementation
from Paul Zimmermann.

Examples:
>> Ly := y(x)*6 + x*diff(y(x), x)*(-2) + diff(y(x), x, x)*(-x^2 + 1):
>> ode::rationalSolutions(Ly, y(x));
 
                                  2
                                {x  - 1/3}
 
>> ode::rationalInvariants(Ly, y(x), 2);
 
                           {      2            }
                           {   2 x     4       }
                           { - ---- + x  + 1/9 }
                           {    3              } 
++#


sysassign(ode::rationalInvariants, proc(eq, z, m)
  begin
    eq := ode::isLODE(rewrite(eq, diff), z, "HlodeOverRF");
    case nops(eq) 
      of 1 do return(error("not an ordinary homogeneous linear differential ".
                           "equation over the rational functions"));
      of 4 do if eq[4] < 1 then
                 return(error("only possible for positive order equations"))
              end_if; 
      otherwise
        if domtype(m) <> DOM_INT or m < 1
          then return(error("only defined for positive integers"))
        end_if
    end_case;
    eq[1] := numer(normal(expr(eq[1])));
    ode::ratInvars(eq, m)
  end_proc):

sysassign(ode::ratInvars, proc(eq, y, x, n, m)
  local h, ord, a;
  begin
    ode::symmetricPower; # to load ode::symPower #
    userinfo(5, "compute the", m,"-th symmetric power");
    if n=2 then 
      eq := ode::fracFreeSymPowerOrd2(eq, y, x, m);
      userinfo(5, "determine its rational solutions");
      return(ode::ratsols(ode::vectorize(eq,y,x,m+1),x))
    end_if;
    eq := ode::symPower(eq, y, x, n, m);
    userinfo(5, "...normalize it...");
    ord := ode::getOrder(eq, y(x));
    a := ode::vectorize(eq,y,x,ord);
    h:=lcm(op(map(map(a,denom),poly,[x])));
    if degree(h)<>0 then h:=expr(h); a:=map(map(a,_mult,h),normal) end_if; 
    userinfo(5, "determine its rational solutions");
    ode::ratsols(a,x)
  end_proc):       

sysassign(ode::rationalSolutions, proc(eq, z, o) #o: Option "Generic" #
  local sol;
  begin
    userinfo(12,"entering rationalSolutions at time ".time());
    eq := ode::isLODE(rewrite(eq, diff), z, "HlodeOverRF");
    case nops(eq) 
      of 1 do return(error("not an ordinary homogeneous linear differential ".
                            "equation over the rational functions"));
      of 4 do if eq[4] < 1 then
                 return(error("only defined for positive order equations"))
              end_if;
    end_case;
    eq[1] := numer(normal(expr(eq[1])));
    userinfo(12,"computed numerator at time ".time());
    sol := ode::ratsols(eq);
    if o="Generic" then
       _plus(op(map(sol, _mult, genident("C")))) 
     else sol end_if
  end_proc):  

# be careful: here the coefficients are assumed to be polynomial !!!
  (no normal is done)
  Use ode::rationalSolutions for rational coefficients #
sysassign(ode::ratsols, proc(eq, y, x, n) 
  local a, i, den, z, Ly, sol;
  begin
     if type(eq)=DOM_LIST then a:=eq; x:=y; n:=nops(a)-1;
       y := genident("_y")
     else
       a := ode::vectorize(eq,y,x,n) end_if;
    den := ode::denomAnsatz(map(a,poly,[x]),x,n);
    userinfo(12,"computed denominator ``ansatz'' at time ".time());
    userinfo(10,"degree of denominator is ".degree(den));
    userinfo(5, "ansatz of denominator", den);
    z   := y(x)/den;
    Ly  := a[1]*z; 
    for i from 1 to n do
      z  := diff(z,x);
      Ly := Ly + a[i+1]*z
    end_for;
    Ly := numer(normal(expr(Ly)));
    userinfo(15, "transformed equation is", Ly);
    sol:= ode::polysols(Ly,y,x,n);
    userinfo(12,"computed polynomial solutions at time ".time());
    map(map(sol, _mult, 1/den), normal)
  end_proc):


sysassign(ode::denomAnsatz, proc(A, x, n, o) 
  # Option: "mu", the sum of Exponents #
  local k, l, m, Q, mu, i, J, j, v, vmin, P, z, d, dp, minj;
  begin
    l := factor(A[n+1]); m := nops(l) div 2; mu := 0;
    for i from 1 to m do
      Q[i] := l[2*i]; 
      J[i] := {}; 
      vmin := infinity;
      for k from 0 to n do
         v := ode::nu(A[k+1], Q[i]) - k;
         if v<vmin then J[i] := {k}; vmin := v
           elif v=vmin then J[i] := J[i] union {k}
         end_if;
      end_for;
      dp := diff(Q[i],x); 
      P[i] := 0;
      minj:=min(op(J[i])); 
      for j in J[i] do
         # improvement with respect to [Bronstein92]: factor out the terms
           z*(z-1)*...*(z-(minj-1)) is minj>0 #
         P[i] := P[i] + multcoeffs(
                   dp^j*ode::nu(A[j+1],Q[i],Rem),_mult(z-k$hold(k)=minj..j-1) )
      end_for;
      P[i] := divide(P[i],Q[i],Rem);
      #P[i] := resultant(P[i],Q[i],x);#
      # as Q[i] is irrecucible, no resultant is needed, just get the
        roots of P[i](0) #
      P[i]:=lcoeff(P[i]); # P[i](0) may be 0 #
      # select negative roots #
      d[i]:=select(sharelib::iroots(P[i],FALSE),_less,0);
      if d[i]={} then d[i] := 0 else d[i] := -min(op(d[i])) end_if;
      mu := mu + d[i]*degree(Q[i]);
    end_for;
    Q:=expr(_mult(Q[i]^d[i]$hold(i)=1..m));
    if o="mu" then Q, mu else Q end_if
  end_proc):


# returns the largest integer m such that P^m divides Q
   or Q/P^m is o=Rem #
sysassign(ode::nu, proc(Q,P,o) 
  local m, R;
  begin
    if iszero(Q) then m := infinity
      else 
        m := 0; 
        repeat
          R := divide(Q,P);
          if iszero(R[2]) then m := m+1; Q := R[1] else break end_if
        until FALSE end_repeat
    end_if;
    if o=hold(Rem) then Q else m end_if
  end_proc):
    
