# ode::adjoint(l,x,f) solves the homogeneous linear equation 
  l[1]*u(x)+l[2]*diff(u(x),x)+...+l[n+1]*diff(u(x),x$n)+f = 0
  using the adjoint equation method (cf Zwillinger p. 149)

  Output: either a set of solutions for u(x) or FAIL.

  Example: ode::adjoint([8*x,2*x^2+4*x-3,x^2-x],x,-1);

     {       1            x             C1                C2        }
     { - ---------- + ---------- - ------------- + ---------------- }
     {            2            2      2        2         2        2 }
     {   4 (x - 1)    6 (x - 1)    2 x  (x - 1)    exp(x)  (x - 1)  }
#

ode::adjoint := proc(l,x,inho)
local u,T,w,i,n,adj_eq,s,wstar,B;
begin
   userinfo(1,"trying the adjoint equation method");
   n:=nops(l)-1; # order of the equation #
   if n<>2 then return(FAIL) end_if; # concomitant not yet implemented #
   for i from 0 to n do T[i]:=(-1)^i*diff(l[i+1]*w(x),x$i) end_for;
   adj_eq := _plus(T[i] $ hold(i)=0..n);
   userinfo(2,"the ajoint equation is",collect(adj_eq,[diff(w(x),x$i)$hold(i)=0..n]));
   # the ajoint equation is linear, thus call ode::undet
     (solve_linear would be more powerful, but more costly too) #
   sysassign(ode::depth,ode::depth+1);
   # we need only one solution here, therefore add 4th argument FALSE #
   s:=ode::undet([coeff(adj_eq,[diff(w(x),x$i)],1) $ hold(i)=0..n,0],x);
   sysassign(ode::depth,ode::depth-1);
   # ode::undet returns either FAIL or a non-empty set of solutions #
   if s=FAIL then return(FAIL) end_if;
   wstar:=op(s,1); # solution of adjoint equation #
   if type(wstar)="_mult" then wstar:=select(wstar,has,x) end_if;
   userinfo(2,"found a solution of the adjoint equation:",wstar);
   B:=ode::concomitant(u,w,x,l,n);
   if B=FAIL then return(FAIL) end_if;
   # B:=eval(subs(B,w(x)=wstar,D(w)(x)=diff(wstar,x))) #
   B:=subs(B,w(x)=wstar,D(w)(x)=diff(wstar,x),Simplified)
	+int(wstar*inho,x)+genident("C");
   solve(ode(B,u(x)))
end_proc:

ode::concomitant := proc(u,w,x,l,n)
begin
   if n=2 then u(x)*l[2]*w(x)+diff(u(x),x)*l[3]*w(x)-u(x)*diff(l[3]*w(x),x)
   else FAIL
   end_if
end_proc:
