#  eq is a first-order ODE in y(x),
   tries to recognize and solve a Bernoulli equation,
   i.e. and equation of the form

	y' + P(x)*y = Q(x)*y^n

   and returns FAIL otherwise.

   Examples:

>> ode::Bernoulli(diff(y(x),x)+y(x)-y(x)^3*sin(x),y,x);

                {                    1                    }
                { --------------------------------------- }
                { / 2 cos(x)   4 sin(x)            2 \1/2 }
                { | -------- + -------- + C1 exp(x)  |    }
                { \    5          5                  /    }

>> ode::Bernoulli(diff(x(y),y)-y^2*x(y)-y^5/x(y),x,y);

                  { /              /    3 \       \1/2 }
                  { |    3         | 2 y  |       |    }
                  { | - y  + C1 exp| ---- | - 3/2 |    }
                  { \              \  3   /       /    }
#
ode::Bernoulli := proc(eq,y,x) local yp,n,Q,P,s;
begin
   eq:=subs(eq,diff(y(x),x)=yp);
   if testtype(eq,Type::PolyExpr(yp)) and degree(eq,[yp])=1 then # linear in y'(x) #
      eq:=coeff(eq,[yp],0)/coeff(eq,[yp],1); # should match P(x)*y-Q(x)*y^n #
      loadlib("intlib");
      eq:=expand(subs(eq,y(x)=y));
      s:=intlib::match(eq,_pattern(P,not has,y)*y-_pattern(Q,not has,y)*y^_pattern(n,
	not has,{x,y}));
      if s<>FAIL then
         n:=subs(n,s); Q:=subs(Q,s); P:=subs(P,s);
         userinfo(1,"Bernoulli equation");
         userinfo(2,"P=",P,"Q=",Q,"n=",n);
         return(ode::bernoulli(P,Q,n,x)) 
      end_if
   end_if;
   FAIL
end_proc:

# solves y'(x) + P(x) y(x) = Q(x) y(x)^n, cf Zwillinger chap. 37 p. 157,
   changing to u(x) = y(x)^(1-n), we obtain the equation
   1/(1-n)*u'(x) + P(x) u(x) = Q(x),
   which can be solved exactly using integrating factors,
   and then y(x)=u(x)^(1/(1-n)) #
ode::bernoulli := proc(P,Q,n,x) local intP;
begin
   intP:=int(P,x);
   # here there is a typo in Zwillinger's book: a factor (1-n) is missing
     before int(exp((1-n)*intP)*Q,x) #
   {combine(expand(exp((n-1)*intP)*(genident("C")+(1-n)*int(exp((1-n)*intP)*Q,x))),exp)^(1/(1-n))}
end_proc:
