#++ ---------------- dAlember.mu ---------------------
(copied from file dAlembert.mu by Paul Zimmermann, 
names must not exceed 8 characters)

Author: Winfried Fakler,  University of Karlsruhe
	(ode::reduction written by Paul Zimmermann)
Date Created: 13 September 1996
Last Update :  8 November  1996

Description:
This file contains the reduction method of d'Alembert for
ordinary linear (homogeneous) differential equations.
 

Functions: 

 - used Parameter:
    ++ Ly, y(x), v:DOM_EXPR
    ++ y, x       :DOM_IDENT
    ++ n          :DOM_INT
    ++ Ly    = ordinary linear homogeneous differential equation 
    ++ y(x)  = the operator (function) of Ly  
    ++ y     = name of the operator of Ly
    ++ x     = dependent variable of the operator y
    ++ n     = order of Ly
    ++ v     = special nonzero solution of Ly

 - ode::dAlembert(Ly, y(x), v)
    ++ returns the reduced equation of an equation Ly of arbitrary order, i.e.
    ++ if u is a solution of the reduced equation then v*int(u) is
    ++ a solution of Ly.
 - ode::secondOrderReduction(Ly, y, x, v)
    ++ returns the reduced first order equation of a normal form
    ++ second order equation Ly, i.e. the leading coefficient must be 1.
    ++ NOTE: there is no test of arguments.
 - ode::generalReduction(Ly, y, x, n, v, < "Normal" >)
    ++ returns the reduced equation of an equation Ly of arbitrary order.
    ++ When the option "Normal" is used, the leading coefficient of the 
    ++ reduced equation will be normalized (i.e. is 1).
    ++ NOTE: there is no test of arguments.

See:
 e.g. Walter, W. (1990). Gew"ohnliche Differentialgleichungen. 4th Edition.
      Berlin, Heidelberg, New York: Springer, pp. 133-134.

Example:

>> wy := diff(y(x), x, x)*(-3) + x^(-2)*y(x)*6;

                                              6 y(x)
                       - 3 diff(y(x), x, x) + ------
                                                 2
                                                x

>> ode::dAlembert(wy, y(x), x^2);    

                          12 y(x)
                        - ------- - 3 diff(y(x), x)
                             x

++#

ode::dAlembert:= proc(eq, z, v)
  begin
    ode::generalReduction(ode::isLODE(eq, z, "Hlode"),v, "Normal")
  end_proc:
    
ode::secondOrderReduction:= proc(eq, y, x, v)
  local a;
  begin
    a := ode::vectorize(eq,y,x,2);
    if a[3] <> 1 then error("equation must be in normal form") end_if;
    diff(y(x),x) + normal( a[2] + 2*diff(v,x)/v )*y(x)
  end_proc:

ode::generalReduction:= proc(eq, y, x, n, v, o) # Option: "Normal" #
  local a, dv, i, j, b;
  begin
    a  := ode::vectorize(eq, y, x, n);
    dv := [ diff(v, x$i) $hold(i)=0..n ];
    b  := [_plus(binomial(i,j)*a[i+1]*dv[i-j+1] $hold(i)=j..n) $hold(j)=1..n];
    if o="Normal" then 
      b := map(map(b, _mult, 1/v), normal) end_if;
    ode::mkODE(b, y, x)
  end_proc:

# ode::reduction(eq,yofx,v) returns a complete solution of eq,
  v being a particular solution #
ode::reduction:= proc(eq, yofx, v) local s,n,C;
begin
   n:=ode::order(eq,{op(yofx,0)});
   eq:=ode::dAlembert(eq, yofx, v); # reduced equation #
   s:=ode::solve_linear(ode::vectorize(eq,op(yofx,0),op(yofx),n),op(yofx),n-1);
   if type(s)=DOM_SET and nops(s)=n-1 then
      C:=genident("C");
      map(s,fun(v*int(args(1),op(yofx))+C*v))
   else FAIL
   end_if
end_proc:

    
