# ode::equidimensional_in_x(eq,y,x,n) solves the nth order ODE eq=0 for y(x)
  if it is equidimensional in x (cf Zwillinger chap. 47 p. 189)
  Input: eq is an expression, y,x are variables, n is the order
  Output: FAIL or a list of solutions

  Examples: 
ode::equidimensional_in_x(x*diff(y(x),x,x)-2*y(x)*diff(y(x),x),y,x,2);
ode::equidimensional_in_x(diff(y(x),x$2)-2/x*diff(y(x),x)+3/x^2*y(x)-2*x+1,y,x,2);
#

# ok=TRUE when the equation is known to be equidimensional in x,
  for example when called from ode::scale_invariant #
ode::equidimensional_in_x := proc(eq,_0_y,_0_x,n,ok) local eq2,a,i,X,t,l;
begin
   if ok<>TRUE then
      # first substitute x by a*X #
      eq2:=subs(eq,(diff(_0_y(_0_x),_0_x$i)=diff(_0_y(X),X$i)/a^i) $ i=1..n, 
	_0_y(_0_x)=_0_y(X),_0_x=a*X, X=_0_x);
      ok:=not has(normal(eq2/eq),{_0_x,_0_y})
   end_if;
   if ok<>TRUE then FAIL
   else # equidimensional in x #
      userinfo(1,"equidimensional in x equation");
      # change x to exp(t), and exp(t) to 1 because it should disappear
	(autonomous equation) #
      l:=diff(_0_y(_0_x),_0_x$i)=ode::equidimx_change(_0_y,t,i) $ i=1..n, _0_y(_0_x)=_0_y(t), _0_x=1;
      eq:=subs(eq,l);
      eq:=numer(eq);
      userinfo(2,"new equation to solve is",eq);
      # ode::depth:=ode::depth+1;
      eq:=ode::solve_eq(eq,_0_y,t);
      sysassign(ode::depth,ode::depth-1); #
      eq:=ode::autonomous(eq,_0_y,t,n);
      if has(eq,FAIL) then
         userinfo(1,"equidimensional in x method failed"); return(FAIL)
      end_if;
      subs(eq,t=ln(_0_x))
   end_if;
end_proc:

# returns the value of diff(y(x),x$n) in terms of diff(Y(t),x$i) where
  y(x) = y(exp(t)) = Y(t) #
ode::equidimx_change := proc(_0_y,t,n)
local i;
option remember;
begin
   # exp(-n*t)*_plus(ode::euler_coeff(n,i)*diff(_0_y(t),t$i)$i=1..n) #
   # remove the exp(-n*t) factor because it will be replaced by 1 #
   _plus(ode::euler_coeff(n,i)*diff(_0_y(t),t$i)$i=1..n)
end_proc:

# Euler equations are linear ODEs of the form
   a[0] x^n y^(n)+a[1] x^{n-1} y^(n-1) + ... + a[n-1] x y' + a[n] y = 0.
   These are a special case of equidimensional-in-x equations,
   which are solved by the transformation x = exp(t)
   (cf Zwillinger chap. 49 p. 193).

  ode::euler(l,z,n) solves the equation 
    l[1]*y(z)+l[2]*diff(y(z),z)+...+l[n+1]*diff(y(z),z$n)=0

  Example: ode::euler([2,-2*x,x^2],x,2);

                                       2
                                  [x, x ]
>> ode::euler([8/x^4,8/x^3,-4/x^2,0,1],x,4);
#

ode::euler := proc(l,x,n) local L,i,m;
begin
   L:=[_plus(l[m+1]*ode::euler_coeff(m,i)/x^m$m=i..n)$i=0..n];
   if has(L,x) then FAIL
   else 
      userinfo(1,"Euler's equation");
      L:=ode::constant(L,x,n); # constant coefficient equation #
      # eval(subs(L,x=ln(x))) #
      subs(L,x=ln(x),Simplified)
   end_if
end_proc:

# returns the coefficient of diff(Y(t),t$i)/exp(n*t) in diff(y(x),x$n)
  for the change y(x) = y(exp(t)) = Y(t), cf table 49 p. 194 in Zwillinger.
  we should have 0<=i<=n #
ode::euler_coeff := proc(n,i)
option remember;
begin
   if n=i then 1
   elif i=0 then 0
   else ode::euler_coeff(n-1,i-1)-(n-1)*ode::euler_coeff(n-1,i)
   end_if
end_proc:
