# ode::exact_first_order(eq,y,x) tries to solve the first-order 
   equation eq=0 with respect to y(x) by using the method described
   in Zwillinger, chap. 50 p.195. Namely, if the equation has the form
   y'(x) = N(x,y)/M(x,y) and diff(M,x)+diff(N,y)=0, it is said to be an
   exact ODE.
   Input: eq is a first-order expression in y(x).
   Output: FAIL or a set of solutions.
 
   Example: ode::exact_first_order(
        (exp(y(x))+2*x*y(x)+1)*diff(y(x),x)-(3*x^2-y(x)^2-7),y,x);

                                               3      2    
                solve(C1 + 7 x + y + exp(y) - x  + x y , y)

      ode::exact_first_order(p^2*(diff(x(p),p)+2*x/p-3*a*p),x,p);

                              {      2      }
                              { 3 a p    C2 }
                              { ------ - -- }
                              {   4       2 }
                              {          p  }
#

ode::exact_first_order := proc(eq,y,x) local yp,N,M,f;
begin
   userinfo(2,"trying to recognize an exact first order equation");
   eq:=subs(eq,diff(y(x),x)=yp,y(x)=y);
   if testtype(eq,Type::PolyExpr(yp)) then
      eq:=poly(eq,[yp]);
      if degree(eq)=1 then
         # do not call normal to get N and M, not to destroy exact equations
            when called from ode::int_factors #
         N:=-coeff(eq,0); M:=coeff(eq,1);
         if normal(diff(M,x)+diff(N,y))=0 then
            userinfo(1,"exact first order equation");
            N:=-int(N,x); M:=expand(int(M,y));
            # the solution is N+f(y)=M+g(x) #
            userinfo(3,"solution is",N+f(y)=M+g(x));
            f:=select((if type(M)="_plus" then [op(M)] else [M] end_if),
		fun(has(args(1),y) and not has(args(1),x)));
            N:=_plus(expand(N),op(f)); # probable solution #
            if has(expand(N-M),y) then return(FAIL) end_if;
            userinfo(1,"exact 1st order method worked");
            return(map(solve(N+genident("C"),y,ode::maxdegree,PrincipalValue),
		expand))
         end_if
      end_if
   end_if;
   FAIL
end_proc:
