# ode::exact_second_order(eq,y,x) tries to solve the second order ODE eq=0
   with respect to y(x) using the method described in Zwillinger, chap. 51 p. 198.
   Namely, if eq is of the form f(x,y,y') y'' + g(x,y,y') =0, with the two conditions

   diff(f,x,x)+2*p*diff(f,x,y)+p^2*diff(f,y,y)=diff(g,x,p)+p*diff(g,y,p)-diff(g,y)
   diff(f,x,p)+p*diff(f,y,p)+2*diff(f,y)=diff(g,p,p)

   then eq is the total differential of some function.

   Input: eq is a second-order equation in y(x).
   Output: FAIL or a list of solutions.

   Example: ode::exact_second_order(
      x*y(x)*diff(y(x),x,x)+x*diff(y(x),x)^2+y(x)*diff(y(x),x),y,x);
#

ode::exact_second_order := proc(eq,y,x) local p,ypp,f,g,h;
begin
   eq:=subs(eq,diff(y(x),x)=p,diff(y(x),x,x)=ypp,y(x)=y);
   if testtype(eq,Type::PolyExpr(ypp)) then
      if degree(eq,[ypp])=1 then
         f:=coeff(eq,[ypp],1); g:=coeff(eq,[ypp],0);
         # check second condition first #
         if diff(f,x,p)+p*diff(f,y,p)+2*diff(f,y)=diff(g,p,p) then
            if diff(f,x,x)+2*p*diff(f,x,y)+p^2*diff(f,y,y)
                =diff(g,x,p)+p*diff(g,y,p)-diff(g,y) then
               userinfo(1,"exact second order equation");
               f:=int(f,p);
               # solution is phi=constant where phi=int(f,p)+h(x,y) #
               # we should also have g = diff(phi,x)+diff(phi,y)*y' #
               g:=g-diff(f,x)-diff(f,y)*p;
               # thus now we should have g = diff(h,x)+diff(h,y)*p #
               # we deal only with the two following easy cases:
                  (1) no y in g ==> h=int(g,x)
                  (2) no x in g ==> h=int(g/p,y) #
               if not has(g,y) then h:=int(g,x)
               elif not has(g,x) then h:=int(g/p,y)
               else return(FAIL)
               end_if;
               # now we obtain a first integral #
               eq:=f+h=genident("C");
               eq:=subs(eq,y=y(x),p=diff(y(x),x));
               userinfo(2,"first order integral is",eq);
               return(ode::solve_eq(eq,y,x))
            end_if
         end_if
      end_if
   end_if;
   FAIL
end_proc:
