# ode::exact_nth_order(l,x,n) tries to solve the linear homogeneous n-th order ODE
     l[1]*y(x)+l[2]*diff(y(x),x)+...+l[n+1]*diff(y(x),x$n)+l[n+2]=0
   with respect to y(x) using the method of Zwillinger, chap. 52 p. 201
   Input: l is a list of (n+2) coefficients, n is the order
   Output: FAIL or a solution 

   Example: ode::exact_nth_order([0,6,3+6*x,1+x+x^2,-6*x],x,3);
      --                                  4                 2      --
      |      C4          C2 x            x              C1 x        |
      |  ---------- - ---------- + -------------- - --------------  |
      |       2            2               2                2       |
      -- x + x  + 1   x + x  + 1   4 (x + x  + 1)   2 (x + x  + 1) --
   which is the solution of the 3rd order equation 
	(1+x+x^2)*diff(y(x),x$3)+(3+6*x)*diff(y(x),x$2)+6*diff(y(x),x)=6*x
#

ode::exact_nth_order := proc(l,x,n) local i,j;
begin
   # check if exact using condition (52.3) #
   if _plus((-1)^i*diff(l[i+1],x$i) $ i=0..n)=0 then
      userinfo(1,"exact n-th order equation");
      # applying formula (52.4) #
      l:=[_plus((-1)^j*diff(l[i+1+j],x$j) $ j=0..n-i) $ i=1..n, 
            int(l[n+2],x)+genident("C")];
      userinfo(2,"new equation is",_plus(l[i+1]*diff(y(x),x$i)$i=0..n-1)+l[n+1]=0);
      sysassign(ode::depth,ode::depth+1);
      l:=ode::solve_linear(l,x,n-1);
      sysassign(ode::depth,ode::depth-1);
      if l=FAIL then userinfo(1,"exact n-th order failed")
      else userinfo(1,"exact n-th order worked")
      end_if;
      l
   else FAIL
   end_if;
end_proc:
