# ode::factor(eq,y,z,n) tries to solve the ODE eq in y(z)
  using the factoring method, cf Zwillinger page 204.
  NOTE: The implementation below works only for linear second-order equations.

  Example: ode::factor((x^2-x^3)*diff(u(x),x,x)+(2*x^2-4*x)*diff(u(x),x)
	+(6-2*x)*u(x),u,x,2);

                             {             2 }
                             {     2   C1 x  }
                             { C2 x  + ----- }
                             {         x - 1 }

  Called with an element of domtype "ode", it returns a list of
  factors for the equation.
#

ode::factor := proc(eq,y,z,n) local l,sol;
begin
   userinfo(1,"trying to factor differential operator");
   if type(eq)=ode then
      l:=extop(eq,2); y:=op(l,0); z:=op(l,1); eq:=extop(eq,1);
      n:=ode::order(eq,{y});
      l:=ode::factor_aux(eq,y,z,n);
      map(l,ode,y(z))
   else
      l:=ode::factor_aux(eq,y,z,n);
      if nops(l)=1 then
         userinfo(1,"factoring method failed");
         return(FAIL) 
      end_if;
      userinfo(1,"differential operator factors");
      sol:={0};
      while l<>[] do
	 userinfo(3,"solving",l[1]);
	 sol:=map(sol,fun(op(ode::solve_eq(l[1]=args(1),y,z))));
	 l[1]:=NIL;
         if has(sol,FAIL) or (has(sol,int) and l<>[]) then
	    # do not allow nested Liouvillian solutions #
            userinfo(1,"factoring method failed"); return(FAIL)
         end_if;
      end_while;
      userinfo(1,"factoring method worked");
      sol
   end_if
end_proc:

# returns a list L of differential equations
  so that the original equation is L[1] @ L[2] @ ... @ L[nops(L)] #
ode::factor_aux := proc(eq,y,z,n) local l,a,b,c,d,f,g,e,g0,g1,i;
begin
   if n=2 then
      if (l:=Type::Linear(eq,[diff(y(z),z$i)$i=0..n]))<>FALSE then 
	 # if a*y''+b*y'+c*y factors into (d*y'+e) @ (f*y'+g),
	   then a=d*f, b=d*f'+d*g+e*f and c=d*g'+e*g #
	 a:=l[3]; b:=l[2]; c:=l[1];
	 factor:
	 g:=g0+g1*z;
	 for d in factors(a) do
	    f:=divide(a,d,Exact);
	    e:=(b-d*diff(f,z)-d*g)/f;
	    eq:=d*diff(g,z)+e*g-c;
	    eq:=numer(eq);
            eq:=coeff(eq,[z]);
	    if eq<>FAIL and type((eq:=solve({eq},{g0,g1})))=DOM_SET
	    and eq<>{} then
	       eq:=map(op(eq,1),normal);
               #
	       eq:=eval(subs(map(eq,map,expr),hold(RootOf)=proc(e,x) begin
		   e:=solve(e,x); if type(e)<>DOM_SET or e={} then FAIL
		   else op(e,1) end_if end_proc));
               #
               eq:=subs(map(eq,map,expr),hold(RootOf)=proc(e,x) begin
                   e:=solve(e,x); if type(e)<>DOM_SET or e={} then FAIL
                   else op(e,1) end_if end_proc,Simplified);
               # do 2 substitutions due to bug in gbasis
                 (univariate polynomial first instead of last) #
	       e:=normal(subs(e,op(eq),op(eq))); 
               g:=subs(g,op(eq),op(eq));
	       return([d*diff(y(z),z)+e*y(z)+l[4],f*diff(y(z),z)+g*y(z)])
	    end_if
	 end_for
      end_if;
   end_if;
   [eq]
end_proc:
