#++ ---------------- fuchsian.mu ---------------------

Author: Winfried Fakler,  University of Karlsruhe
Date Created: 30 June  1997
Last Update : 13 July  1997

Description:
This file contains function for testing and dealing with homogeneous linear 
differential equations of Fuchsian type.
A linear differential equation which has at worst only finitely many regular 
singular points (including infinity) is said to be of Fuchsian type.
 
Functions:

 - used Parameter:
    ++ Ly, y(x):DOM_EXPR
    ++ p       :DOM_EXPR
    ++ y, x, nu:DOM_IDENT
    ++ n       :DOM_INT
    ++ v       :DOM_LIST
    ++ Ly   = ordinary linear differential equation
    ++ y(x) = the operator (function) of Ly  
    ++ y    = name of the operator of Ly
    ++ x    = dependant variable of the operator y
    ++ n    = order of Ly
    ++ p    = place
    ++ nu   = indeterminate of indicial equation
    ++ v    = Ly in vectorized form

 - ode::exponents(Ly, y(x), p)
    ++ returns the exponents at place p of the given equation Ly. Hereby
    ++ p is an irreducible polynomial expression in x or for infinity 1/x.
 - ode::indicialEquation(Ly, y(x), p, nu)
    ++ returns the indicial equation in nu of equation Ly at place p. Hereby
    ++ p is an irreducible polynomial expression in x or for infinity 1/x. 
 - ode::indicialEq(v, x, p, nu)
    ++ see ode::indicialEquation.
 - ode::isFuchsian(Ly, y(x), <"AllExponents">)
    ++ tests, if the given equation Ly is fuchsian. When the option 
    ++ "AllExponents" is given, a list of table[place, indicial, exponents]
    ++ or FALSE is returned.

Examples:
>> Ly:=x*(1-x)*diff(y(x),x$2)+(1-x)*diff(y(x),x)+10*y(x):
>> ode::isFuchsian(Ly,y(x),"AllExponents");              

-- table(               , table(                        ,
|    place = x,             place = x - 1,
|                     2                               2
|    indicial = lambda ,    indicial = lambda - lambda ,
|    exponents = {0}        exponents = {0, 1}
|  )                      )
|
|
--

table(                             --
          1                         |
  place = -,                        |
          x                         |
                        2           |
  indicial = 10 - lambda ,          |
                 1/2    1/2         |
  exponents = {10   , 10    (-1)}   |
)                                  --

>> ode::indicialEquation(Ly,y(x),x-1,nu);

                                        2
                                 nu - nu

>> ode::exponents(Ly,y(x),1/x);          

                               1/2    1/2
                            {10   , 10    (-1)}

++# 

ode::expsols:  # to load necessary functions from ExpSol.mu #

sysassign(ode::isFuchsian, proc(eq, z, o) 
  local lode,l,m,i,p,t,lambda,indeq,all;
  begin
    eq := ode::isLODE(rewrite(eq, diff), z, "HlodeOverRF");
    if nops(eq)<> 4 then return(FALSE) end_if;    
    lode := ode::vectorize(numer(normal(expr(eq[1]))),eq[2],eq[3],eq[4]);
    t:=eq[3];
    all:=[];
    l:=factor(lode[eq[4]+1]);
    m := nops(l) div 2;
    for i from 1 to m do p[i]:=l[2*i] end_for; 
    p[m+1] := 1/t; 
    for i from 1 to m+1 do
      indeq:=ode::indicialEq(lode,t,p[i],lambda);
      if indeq=FAIL then return(FALSE) end_if;
      if o="AllExponents" then
        all := append(all,
          table(hold(place)=p[i], hold(indicial)=indeq,
                hold(exponents)=solve(indeq,lambda)))
      end_if
    end_for;
    if o="AllExponents" then all else TRUE end_if
  end_proc):         


sysassign(ode::indicialEquation, proc(eq, z, p, u) 
  begin
    eq := ode::isLODE(rewrite(eq, diff), z, "HlodeOverRF");
    if nops(eq)<> 4 then 
      return(error("not an ordinary homogeneous linear differential ".
                           "equation over the rational functions")) end_if;    
    eq[1] := numer(normal(expr(eq[1])));
    ode::indicialEq(ode::vectorize(eq),eq[3],p,u);
  end_proc):

sysassign(ode::indicialEq, proc(eq, x, p, u)
  local npg, indeq;
  begin
    npg := ode::newtonPolygon(eq,x,p,u);
    indeq := op(npg,3);
    if nops(indeq)=1 and iszero(indeq[1][1]) 
      then indeq[1][2] else FAIL
    end_if   
  end_proc):

sysassign(ode::exponents, proc(eq, z, p, u)
  local lambda, indeq;
  begin
    indeq:=ode::indicialEquation(eq,z,p,lambda);
    if indeq=FAIL then return({}) else solve(indeq,lambda) end_if
  end_proc):
 
