# ode::homogeneous(eq,y,x) tries to solve the ODE eq with respect to y(x)
   using the method of homogeneous equations. It returns either a set of
   solutions or FAIL.

   Applicable to: first order ODEs of the form y'=P(x,y)/Q(x,y)
   where P and Q are homogeneous in x and y of the same degree.

   Reference: Zwillinger pages 233-235.

Example: ode::homogeneous(diff(y(x),x)*(x^4-2*x*y(x)^3)-2*x^3*y(x)+y(x)^4,y,x);

                                                3
                    {x RootOf(- x + C3 v3 - x v3 , v3)}

   There is a typo in Zwillinger's example: in (61.5), the right hand side should be
   (2*v-v^4)/(1-2*v^3), or the initial equation should be y'=(x^3y+y^4)/(x^4-2xy^3).
#

ode::homogeneous := proc(eq,y,x) local yp,P,Q,v;
begin
   userinfo(2,"trying to recognize an homogeneous equation");
   eq:=subs(eq,diff(y(x),x)=yp);
   if testtype(eq,Type::PolyExpr(yp)) then
      if degree(eq,yp)=1 then
         eq:=-coeff(eq,yp,0)/coeff(eq,yp,1);
         v:=genident("v"); # to avoid name space problems #
         eq:=normal(subs(eq,y(x)=v*x));
         if not has(eq,x) then # homogeneous #
            userinfo(1,"homogeneous equation");
            eq:=solve(genident("C")*exp(combine(int(1/(eq-v),v),ln))-x,v,
		ode::maxdegree);
            if type(eq)=DOM_SET then eq:=map(eq,_mult,x) else eq:=x*eq end_if;
            return(eq)
         end_if;
      end_if
   end_if;
   FAIL
end_proc:
