# ode::interchange(eq,y,x,n) tries to solve the n-th order ODE eq=0 wrt y(x)
   by interchanging y and x (see Zwillinger page 263).
   There is a typo in (68.5): the solution is x(y)=sqrt(A*exp(2*y^3/3)-y^3-3/2).

Examples:
>> ode::interchange:=loadproc(ode::interchange,pathname("ODE"),"interch"):
>> ode::interchange(diff(y(x),x)=x/(x^2*y(x)^2+y(x)^5),y,x,1);

     / /                       /    3      3 \                         \1/2
     | |                       | 2 y    2 x  |                         |
     | |       /    3 \   3 exp| ---- - ---- |         /    3      3 \ |
     | |       | 2 y  |        \  3      3   /    3    | 2 y    2 x  | |
solve| | C7 exp| ---- | - -------------------- - x  exp| ---- - ---- | |
     \ \       \  3   /            2                   \  3      3   / /

        \
        |
        |
        |
 = x, y |
        /

>> ode::interchange(diff(y(x),x)^3-4*x*y(x)*diff(y(x),x)+8*y(x)^2,y,x,1);

                  {       2     2           3          }
                  { (x C11  - 1)           x           }
                  { -------------, ------------------- }
                  {        6                  3      3 }
                  {     C11        RootOf(4 C9  - 27)  }

>> setuserinfo(Any,1):
>> ode::interchange(diff(y(x),x,x)+x*y(x)*diff(y(x),x)^3,y,x,2);
ode::solve_eq_irred, solving DE, y*x(y)-diff(x(y), y, y)
(the above is Airy's equation)
#

ode::interchange := proc(eq,_0_y,_0_x,n) local i,l,X;
begin
   # the following lines are needed because one cannot deal with other
     functions of x, for example p(x)*diff(y(x),x)+q(x) #
   l:=select(map(select(indets(eq,PolyExpr),testtype,"diff"),op,1),has,_0_x);
   if l minus {_0_y(_0_x)} <> {} then return(FAIL) end_if;
   userinfo(1,"trying to interchange dependent and independent variable");
   l:=[1/diff(X(_0_y),_0_y)];
   for i from 2 to n do l:=append(l,expand(diff(l[i-1],_0_y)*l[1])) end_for;
   eq:=numer(subs(eq,diff(_0_y(_0_x),_0_x$i)=l[i]$hold(i)=1..n,
	_0_y(_0_x)=_0_y,_0_x=_0_x(_0_y),X=_0_x));
   # the following line avoids that we try to interchange back,
     but the call of "numer" above is crucial to have a normal form #
   sysassign(ode::interchange(eq,_0_x,_0_y,n), FAIL);
   # check the depth here in order not to enter an infinite loop #
   sysassign(ode::depth,ode::depth+1);
   eq:=ode::solve_eq(eq,_0_x,_0_y,{});
   sysassign(ode::depth,ode::depth-1);
   if eq={FAIL} then eq:=FAIL end_if;
   if eq=FAIL then userinfo(1,"interchanging failed");FAIL
   else userinfo(1,"interchanging successful");
      _union(solve(op(eq,i)=_0_x,_0_y)$hold(i)=1..nops(eq))
   end_if;
end_proc:
