# ode::riccati(eq,y,x) tries to solve the first-order ODE eq=0 wrt y(x)
   by rewriting it as a linear second order ODE. Reference: Zwillinger pages 288-291.

   Applicable to: equations of the form y' = a(x) y^2 + b(x) y + c(x).
   With the change y=w'/w/a, they rewrite to w''-(a'/a+b)w'+a*c*w=0.

   Examples:
>> ode::riccati:=loadproc(ode::riccati,pathname("ODE"),"riccati"):
>> ode::riccati(diff(y(x),x)-exp(x)*y(x)^2+y(x)-exp(-x),y,x);

                     {     - cos(x) - C3 sin(x)      }
                     { ----------------------------- }
                     { exp(x) (- sin(x) + C3 cos(x)) }

>> ode::riccati(diff(y(x),x)-y(x)^2+x*y(x)-1,y,x);

        {                               1                         }
        { x, x + ------------------------------------------------ }
        {              /    2 \      /    2 \    /    /  2 \    \ }
        {              |   x  |      |   x  |    |    | x  |    | }
        {        C1 exp| - -- | - exp| - -- | int| exp| -- |, x | }
        {              \   2  /      \   2  /    \    \ 2  /    / }
#

ode::riccati := proc(eq,y,x) local yp,a,b,c,w,d,e;
begin
   userinfo(2,"trying to recognize a Riccati equation");
   eq:=subs(eq,diff(y(x),x)=yp,y(x)=y);
   if testtype(eq,Type::PolyExpr(yp)) then
      if degree((eq:=poly(eq,[yp])))=1 then
         eq:=-coeff(eq,0)/coeff(eq,1); # should match a*y^2+b*y+c #
         if testtype(eq,Type::PolyExpr(y)) then
            if degree((eq:=poly(eq,[y])))=2 then
               userinfo(1,"Riccati equation");
               a:=coeff(eq,2); b:=coeff(eq,1); c:=coeff(eq,0);
               # first tries to solve the associated second order linear equation #
               d:=-expand(diff(a,x)/a+b); e:=expand(a*c);
               if not has([d,e],x) then 
                  eq:=ode::solve_eq(diff(w(x),x,x)+d*diff(w(x),x)+e*w(x),w,x,{});
                  if eq<>FAIL then
                     userinfo(1,"Riccati transformation worked");
                     d:=indets(eq) minus {x};
                     if nops(d)=2 then eq:=subs(eq,op(d,2)=1) end_if;
                     return(map(eq,fun(diff(args(1),x)/args(1)/a)))
                  end_if;
               end_if;
               # otherwise tries to find a particular solution #
               userinfo(2,"tries to find a particular solution");
               # eq:=solve(eval(subs(args(1),y(x)=w*x)),w,ode::maxdegree); #
               eq:=solve(subs(args(1),y(x)=w*x,Simplified),w,ode::maxdegree);
               if type(eq)=DOM_SET then
                  for w in eq do
                     if has(w,x) or type(w)=RootOf then next end_if;
                     w:=op(eq,1)*x; # particular solution #
                     if ode::bernoulli=FAIL then
                        sysassign(ode::bernoulli,
			 loadproc(ode::bernoulli,pathname("ODE"),"bernoull"))
                     end_if;
                     userinfo(1,"Riccati method worked");
                     return({w} union map(ode::bernoulli(-2*a*w-b,a,2,x),_plus,w)) # Eq. (76.2) #
                  end_for
               end_if;
               userinfo(1,"Riccati method failed");
            end_if
         end_if
      end_if
   end_if;
   FAIL
end_proc:
