# ode::scale_invariant(eq,y,x,n) tries to solve the n-th order ODE eq=0 with the method
   of scale invariant equations (see Zwillinger, pages 294-296). An equation
   is scale invariant if its form is unchanged by the transformation x->a*x,
   y->a^p*y.

   There is a typo in the example given by Zwillinger: the equation (84.2),
   x^2y''+3xy'=1/y^3/x^4, under the change y=u/x, becomes x^2u''+xu'-u=1/u^3
   and not x^2u''+xu'+u=1/u^3 as in (84.4).

   Output: either FAIL or a set of solutions

   Example:
ode::scale_invariant:=loadproc(ode::scale_invariant,pathname("ODE"),"scaleinv"):
ode::scale_invariant(x^2*diff(y(x),x,x)+3*x*diff(y(x),x)-1/y(x)^3/x^4,y,x,2);
ode::scale_invariant(x^5*y(x)^3*diff(y(x),x)*3+x^6*y(x)^3*diff(y(x),x,x)-1,y,x,2);
ode::scale_invariant(x^2*diff(y(x),x,x)+3*x*diff(y(x),x)+2*y(x)-1/y(x)^3/x^4,y,x,2);
#

ode::scale_invariant := proc(eq,_0_y,_0_x,n) local X,Y,p,i,l,s,a,u;
begin
   if type(eq)<>"_plus" then return(FAIL) end_if;
   l:=diff(_0_y(_0_x),_0_x$i)=a^(p-i)*diff(Y(X),X$i)$i=1..n,
	_0_y(_0_x)=a^p*Y(X),_0_x=a*X,Y=_0_y,X=_0_x;
   s:={map(op(eq),fun(subs(args(1),l)/args(1)))};
   s:=map(s,combine);
   s:=map(s,fun((if type(args(1))="_power" and op(args(1),1)=a then op(args(1),2)
                  elif args(1)=1 then 0 elif args(1)=a then 1 else FAIL end_if)));
   if has(s,FAIL) then return(FAIL) end_if;
   case nops(s)
   of 1 do s:={0}; break # any value of p works #
   of 2 do s:=solve(op(s,1)=op(s,2),p,ode::maxdegree); break
   otherwise return(FAIL)
   end_case;
   if type(s)="solve" or nops(s)=0 then return(FAIL) end_if;
   p:=op(s,1);
   userinfo(1,"scale invariant equation");
   u:=genident("u");
   # eq:=numer(eval(subs(eq,_0_y(_0_x)=_0_x^p*u(_0_x)))); #
   eq:=numer(subs(eq,_0_y(_0_x)=_0_x^p*u(_0_x),Simplified));
   userinfo(2,"new equation is",eq);
   # the equation is now equidimensional in x #
   eq:=ode::equidimensional_in_x(eq,u,_0_x,n,TRUE);
   if eq=FAIL then
      userinfo(1,"scale invariant method failed"); FAIL
   else
      userinfo(1,"scale invariant method worked");
      ode::mapsol(eq,_mult,_0_x^p)
   end_if
end_proc:
