
#++ ---------------- solvesys.mu ---------------------

Author(s): Paul Zimmermann,  INRIA, Lorraine
           Jay Belanger,
           Winfried Fakler,  University of Paderborn
Date Created:         1995
Last Update : 27 July 1999

Description:

  ode::solve_sys( sys, unks ) returns a set of assignments for the set unks of
unknown functions which solves the system of first order ordinary differential
equations sys with initial conditions. Initial conditions can be added in the 
set sys using the differential operator D, e.g. f(1)=2, D(f)(0)=0, 
D(D(f))(0) = 1 .
                                                 
Example:
>> sys:={diff(x(t),t)=-2*x(t)+y(t),diff(y(t),t)=x(t)-2*y(t),x(0)=0,y(0)=2};
>> ode::solve_sys(sys,{x(t),y(t)});
                                                             
++#

sysassign(ode::solve_sys,
proc(sys,unks) # unks is a set #
local is_linear,eq,inits,_l,y,z,maxord,s,i,j,A,ll,_b,m,k,sol,n,indet,
      initialConditions;
begin
  // handle initial conditions, (sol is supposed as a set)
  initialConditions:=
    proc(sol,inits,indet,_l,z)
    begin
      if inits<>{} then sol:=map(sol,ode::solvesysinit,indet,inits,_l,z)
      end_if;
      if domtype(sol)=DOM_SET and type(op(sol))=DOM_SET then sol:=op(sol)
      end_if;
      sol
    end_proc;

   _l:={}; z:={};
   for eq in unks do
      if type(eq)="function" then
         if type((y:=op(eq,0)))=DOM_IDENT and nops(eq)=1 then
            _l:=_l union {y};
            z:=z union {op(eq)};
            next
         end_if
      end_if;
      error("invalid unknown function")
   end_for;
   if nops(z)<>1 then error("wrong number of independent variables") end_if;
   z:=op(z);
   n:=nops(_l); # number of unknowns functions #

    // Get a list of the indeterminates from the original system
    // this will be needed to handle initial conditions
    indet := {};
    for eq in sys do
        indet := indet union indets(eq);
    end_for;
   
   # first split equations and initial conditions #
   eq:=ode::split(sys,z);
   sys:=eq[1]; inits:=eq[2];
   if nops(sys)<>n then
     error("number of equations differs from number of unknowns") end_if;
   # check the equations and compute the maximal order #
   maxord:=ode::order(sys,_l);
   userinfo(1,"differential system of order ".maxord." with ".n." functions");
   # recognize linear systems #
   _l:=[op(_l)]; # to set the order #
   s:=[(diff(_l[i](z),z$j) $ i=1..n) $ j=0..maxord];
   m:=n*(maxord+1);
   is_linear:=TRUE; _b:=[];
   for k from 0 to maxord do A[k]:=[[0$n]$n] end_for;
   for i from 1 to n do
      if (ll:=Type::Linear(op(sys,i),s))<>FALSE then
         _b:=append(_b,ll[m+1]);
         for k from 0 to maxord do
            for j from 1 to n do A[k][i][j]:=ll[k*n+j] end_for;
         end_for;
      else is_linear:=FALSE; break
      end_if
   end_for;
   sol:=FAIL;
   if is_linear then sol:=ode::linsys(A,_b,_l,z,n,maxord,n)
   else
      userinfo(1,"nonlinear system");
      if maxord=1 then
         sol:=ode::combination(sys,unks,z);
         if sol=FAIL then sol:=ode::matrixRiccati(sys,unks,z) end_if;
      end_if;
   end_if;

//      if sol<>FAIL then return(sol) end_if;
   if sol<>FAIL then return(initialConditions(sol,inits,indet,_l,z)) end_if;
  
   # try to recognize a triangular system #
   for eq in sys do
      hastype; # to load Indets #
      s:=Indets(eq,"function") intersect unks;
      if nops(s)=1 then
         userinfo(1,"found one independent equation");
         s:=op(s); # x(t) #
         sol:=ode::solve_eq(eq,op(s,0),op(s));
         if not has(sol,FAIL) and nops(sol)=1 then
            sol:={s=op(sol)};
            # sys:=eval(subs(sys minus {eq},sol)); #
            sys:=subs(sys minus {eq},sol,Simplified);
            unks:=unks minus {s};
            if nops(unks)=1 then
               s:=op(unks);
               sys:=ode::solve_eq(op(sys),op(s,0),op(s));
               if has(sys,FAIL) then sys:=FAIL 
               else
                  sys:=ode::mapsol(sys,fun({s=args(1)}));
                  sys:=map(sys,fun((if type(args(1))=DOM_SET then args(1)
                        else {args(1)} end_if)));
               end_if;
            else sys:=ode::solve_sys(sys,unks)
            end_if;
            if type(sys)=DOM_SET then

//               return(map(sys,_union,sol))
              return(initialConditions(map(sys,_union,sol),inits,indet,_l,z))
            end_if
         end_if
      end_if
   end_for;
   if sol=FAIL then sol else initialConditions({sol},inits,indet,_l,z) end_if;
end_proc):

sysassign(ode::combination,
        loadproc(ode::combination,pathname("ODE"),"combinat")):
sysassign(ode::matrixRiccati,
        loadproc(ode::matrixRiccati,pathname("ODE"),"mriccati")):


// Now the actual initial condition solver

sysassign(ode::solvesysinit,
proc (sol,indet,inits,fns,z)
local csts, eq, s, f, lhs, a, i, csteq, t, spsol, soln;
begin
    // Find the constants
    csts := {};
    for eq in sol do
        csts := csts union indets(eq);
    end_for;
    fns := {op(fns)};
    csts := csts minus (indet union fns union {z});
    userinfo(3,"general solution is",sol);
    // here solve_eq.mu has a test to see if the solution has type 
    // RootOf
    userinfo(3,"initial conditions are",inits);
    // Now put the initial conditions into a more useful form
    s := {};
    for eq in inits do
        for f in fns do
            if has(op(eq,2),f) then
                error("dependent variable not allowed in rhs of init conds");
            end_if;
        end_for;
        lhs := op(eq,1);
        i := 0;
        a := op(lhs);
        lhs := op(lhs,0); // This should be f or D(...(D(f))...)
        case type(lhs)
            of DOM_IDENT do break
            of "D"       do 
                            repeat
                                lhs := op(lhs);
                                i := i+1;
                            until type(lhs) <> "D"
                            end_repeat;
                            break
            otherwise error("invalid initial conditions")
        end_case;
        if not(contains(fns,lhs)) then
            error("invalid initial conditions");
        end_if;     
        s := s union {[lhs,i,a,op(eq,2)]};
        // [f,i,a,b] means (D@@i)(f)(a)=b
        
        // solve_eq has a check here to make sure that the denominator
        // of eq doesn't vanish

    end_for;

    // solve_eq.mu has the next part in "traperror"
    // it will take the initial conditions and create a system of 
    // equations which can be solved for the constants
    csteq := {};
    for t in s do
        eq := op(select(sol,has,t[1]),[1,2]);
        // I'm not sure of why there is the substitution "int=fun(0)"
        // in the next line, but it is from solve_eq.mu
        csteq := csteq union {subs(diff(eq,z$t[2]), hold(int)=fun(0),
                                z=t[3],Simplified) = t[4]};
    end_for;    
    // Now solve the equations for the constants
    csteq := solve(csteq,csts);
    // I would think that csteq would be a set of sets or lists,
    // but I again I get the following from solve_eq.mu
    csteq := select(csteq,fun(contains({DOM_LIST,DOM_SET},
                                type(args(1)))));

    // What to do if the system of equations for the constants cannot
    // be solved?  Return the general solution with the initial
    // conditions, I suppose

    if type(csteq) <> DOM_SET then
        return(sol, inits)
    elif nops(csteq) = 0 then
        return(null()) // no solution
    else 
        spsol := {};
        for soln in sol do
            soln := (subs(soln,op(csteq,i)) $ hold(i) = 1..nops(csteq));
            if has(soln,{sin,cos}) then
                soln := combine(expand(soln),sincos)
            end_if;
            spsol := spsol union {soln}
        end_for;
    return(spsol);
    end_if;
end_proc): /* ode::solvesysinit */
