#++ ---------------- sympower.mu ---------------------

Author: Winfried Fakler,  University of Karlsruhe
Date Created: 24 April    1996
Last Update : 13 April    1999

Description:
symmetricPower( Ly, y(x), m ) computes the m-th symmetricPower of the 
linear homogeneous differential equation Ly. This is the lowest order
linear homogeneous differential equation whose solution space consists 
exactly of all possible m-th powerproducts of solutions of Ly.

Functions:

 - used Parameter:
    ++ Ly, y(x):DOM_EXPR
    ++ y, x    :DOM_IDENT
    ++ n, m    :DOM_INT
    ++ Ly   = ordinary linear differential equation
    ++ y(x) = the operator (function) of Ly  
    ++ y    = name of the operator of Ly
    ++ x    = dependant variable of the operator y
    ++ n    = order of Ly
    ++ m    = exponent of the symmetric power of Ly

 - ode::symmetricPower(Ly, y(x), m)
    ++ returns the m-th symmetricPower of an ordinary homogeneous
    ++ linear differential equation Ly.
    ++ The given arguments are tested.
 - ode::symPower(Ly, y, x, n, m)
    ++ returns the m-th symmetricPower of an ordinary
    ++ linear differential equation Ly.
    ++ NOTE: the reductum of Ly needs to be non-constant! Otherwise the result
    ++       maybe wrong!
 - ode::symPowerOrd2(Ly, y, x, m)
    ++ returns the m-th symmetricPower of a second order ordinary
    ++ linear differential equation Ly.
    ++ NOTE: the reductum of Ly needs to be non-constant! Otherwise the result
    ++       maybe wrong!
 - ode::fracFreeSymPowerOrd2(Ly, y, x, m)
    ++ returns the m-th symmetricPower of a second order ordinary
    ++ linear differential equation Ly, computed by a fraction free version.

See:
   - Singer, M. F. (1985). Solving homogeneous linear differential 
     equations in terms of second order linear differential equations. 
     Amer. J. Math. 107
   - Bronstein, M., Mulders, T., Weil, J.A. (1997). On Symmetric Powers of
     Differential Operators. In: Proceedings of ISSAC'97, Maui, Hawaii,
     156-163.

Example:

>> eq := diff(y(x),x,x)+1/x*diff(y(x),x)-1/(4*x^2)*y(x):
>> ode::symmetricPower(eq, y(x), 2);
 
                 3 diff(y(x), x, x)
                 ------------------ + diff(y(x), x, x, x)
                         x
Hurwitz-equation:
>> eq:=x^2*(x-1)^2*diff(y(x),x,x,x)+(7*x-4)*x*(x-1)*diff(y(x),x,x)+
 (72/7*(x^2-x)-20/9*(x-1)+3/4*x)*diff(y(x),x)+(72*11/7^3*(x-1)+5/8+2/63)*y(x):
>> ode::symmetricPower(eq, y(x), 2);
++#

sysassign(ode::symmetricPower, proc(eq, z, m) 
  begin
    eq := ode::isLODE(eq, z, "Hlode");
    case nops(eq) 
      of 1 do return(error("not an ordinary homogeneous linear differential ".
                           "equation"));
      of 4 do if eq[4] < 1 then
                 return(error("only defined for positive order equations"))
              end_if;
      otherwise
        if domtype(m) <> DOM_INT or m < 1
          then return(error("only defined for positive integers"))
        end_if
    end_case;
    if eq[4]=2 
      then eq[4] := NIL; ode::symPowerOrd2(eq, m)
      else ode::symPower(eq, m) end_if
  end_proc):

sysassign(ode::symPower, proc(eq, z, x, n, m) 
  local t0, c, Y, i, j, dxn,
        ansatz, dimBasis, newDimBasis, diffvars, solset, term;
  begin
    t0 := time();
    z := z(x);
    Y := z^m;
    dxn := diff(z, x$n);
    term := expand(- coeff(eq, [dxn],0)/coeff(eq, [dxn],1));
    ansatz := c[0]*Y;
    diffvars := [diff(z, x$j) $ j=0..n-1];
    for i from 1 to n-1 do
      Y := diff(Y, x);
      ansatz := ansatz + c[i]*Y;
    end_for;    
    i := n-1;
    newDimBasis := nterms(ansatz, diffvars);
    repeat
      i := i+1;
      dimBasis := newDimBasis;
      userinfo(5, "order is now: ", i);
      Y := expand(subs(diff(Y, x), diff(z, x$n)=term));
      ansatz := expand(ansatz + c[i]*Y);
      newDimBasis := nterms(ansatz, diffvars);
    until dimBasis = newDimBasis end_repeat; 
    repeat
      userinfo(5, "...solve ansatz");
      [coeff(poly(subs(ansatz, c[i]=1), diffvars))];
      solset := linsolve({op(%, j)=0 $j=1..dimBasis}, {c[j] $j=0..i-1});
      if solset <> FAIL and solset <> null() then
        userinfo(10, "Time: ", time()-t0, "msec");
        [op({c[i]=1} union solset, j) $j=1..i+1];
        return( subs( _plus(c[j]*diff(z, x$j) $j=0..i), %) ) 
      end_if;
      i := i+1;
      userinfo(5, "order is now: ", i);
      Y := expand(subs(diff(Y, x), diff(z, x$n)=term));
      ansatz :=  expand(ansatz + c[i]*Y) 
    until FALSE end_repeat;
  end_proc): 

sysassign(ode::symPowerOrd2, proc(eq, z, x, m) 
  local t0, c, Y, i, j, dxn, sys, unk, ind, e, eqn, sols, 
        ansatz, dimBasis, newDimBasis, diffvars, solset, term;
  begin
    t0 := time();
    z := z(x);
    Y := z^m;
    dxn := diff(z, x$2);
    diffvars := [diff(z, x$j) $hold(j)=0..2]; 
    term := expand(- coeff(eq, [dxn],0)/coeff(eq, [dxn],1));
    c[0] := genident("c");
    ansatz := c[0]*Y;
    Y := diff(Y, x);
    c[1] := genident("c");
    ansatz := ansatz + c[1]*Y;  
    i := 1; 
    newDimBasis := nterms(ansatz, diffvars);
    repeat
      i := i+1;
      dimBasis := newDimBasis;
      userinfo(5, "order is now: ", i);
      Y := expand(subs(diff(Y, x), dxn=term));
      c[i] := genident("c");
      ansatz := ansatz + c[i]*Y;
      newDimBasis := nterms(ansatz, diffvars);
    until dimBasis = newDimBasis end_repeat; 
    repeat
      userinfo(5, "...solve ansatz");
      sys := {coeff(poly(ansatz, diffvars))};
      unk := {c[j] $hold(j)=0..i-1};
      # the following lines avoid an expensive call to linsolve #
      sols := null();
      repeat
        e:=FALSE; 
        for eqn in sys do
          ind := indets(eqn) intersect unk;
          if nops(ind)=1 then
            ind := op(ind);
            if degree(eqn,[ind])=1 then
              sys := sys minus {eqn}; unk := unk minus {ind};
              # normal is the expensive call #
              #eqn := ind=normal(-coeff(eqn,[ind],0)/coeff(eqn,[ind],1));#
              eqn := ind=expand(-coeff(eqn,[ind],0)/coeff(eqn,[ind],1));
              sols := sols,eqn; sys := subs(sys,eqn); 
              e := TRUE; break;
            end_if
          end_if
        end_for;
      until e=FALSE end_repeat;
      sys:=sys minus {0};
      if sys<>{} then
        solset := linsolve(sys,unk);
        #print("linsolve");#
       else solset := {} end_if;
      if solset <> FAIL and solset <> null() then
        solset := {sols} union solset;
        if has(solset, c[i])  
         then solset := subs(solset, c[i]=1) union {c[i]=1}
         else solset := solset union {c[i]=0} end_if;
        #assign(solset);#
        userinfo(10, "Time: ", time()-t0, "msec");
        return( subs(_plus(c[j]*diff(z,x$j) $hold(j)=0..i), solset))
      end_if;
      i := i+1;
      userinfo(5, "order is now: ", i);
      Y := expand(subs(diff(Y, x), dxn=term));
      c[i] := genident("c");
      ansatz :=  ansatz + c[i]*Y 
    until FALSE end_repeat;
  end_proc):

ode::fracFreeSymPowerOrd2:=
  proc(eq,y,x,m)
    local veq,h,p,a,b,i,L,vl,p1,pe,po;
  begin
    vl:=[diff(y(x),x $ (m+1-i)) $ i=0..m,y(x)];
    L:=genident("_L");
    veq := ode::vectorize(eq,y,x,2);
    h:=lcm(op(map(map(veq,denom),poly,[x])));
    if degree(h)<>0 then h:=expr(h); veq:=map(map(veq,_mult,h),normal) end_if; 
    p:=veq[3];
    a:=veq[2];
    b:=veq[1];
    p1:=collect(diff(p,x),x);
    pe:=gcd(gcd(p,p1),a-p1);
    po:=gcd(pe,normal(b*p/pe));
    L[0]:=y(x);
    L[1]:=normal(diff(L[0],x));
    L[2]:=p*diff(y(x),x,x)+a*diff(y(x),x)+m*b*y(x);
    L[3]:=collect(p/pe*diff(L[2],x)+(2*(a-p1)/pe+p1/pe)*L[2]
                  +2*(m-1)*b*p/pe*L[1],vl,normal);
    for i from 2 to trunc((m+1)/2) do
      L[2*i]:=op(mapcoeffs(
                 poly(p/po*diff(L[2*i-1],x)+L[2*i-1]*((2*i-1)*(a-p1)/po
                      +p1/po+(i-1)*p*diff(pe,x)/po/pe+(i-2)*p*diff(po,x)/po^2)
                      +(2*i-1)*(m-(2*i-2))*b*p/po/pe*L[2*i-2],
                      vl), normal),1);
      L[2*i+1]:=op(mapcoeffs(
                 poly(p/pe*diff(L[2*i],x)+L[2*i]*(2*i*(a-p1)/pe+p1/pe
                        +(i-1)*p*diff(pe,x)/pe^2+(i-1)*p*diff(po,x)/pe/po)
                        +2*i*(m-(2*i-1))*b*p/po/pe*L[2*i-1],
                        vl), normal),1);
    end_for;
    collect(L[m+1],vl);
  end_proc:
