
#++ ---------------- tools.mu ---------------------

Author: Winfried Fakler,  University of Karlsruhe
Date Created: 13 September 1996
Last Update : 16 April     1999


Description:
This file contains functions for dealing with
linear ordinary (homogeneous) differential equations.
NOTE: almost all functions do not check arguments! 

Functions: 

 - used Parameter:
    ++ Ly, y(x)   :DOM_EXPR
    ++ v_i, v, mp :DOM_EXPR
    ++ y, x, u    :DOM_IDENT
    ++ n          :DOM_INT
    ++ R,F        :DOM_DOMAIN
    ++ p          :DOM_POLY
    ++ Ly    = ordinary linear homogeneous differential equation 
    ++ y(x)  = the operator (function) of Ly  
    ++ y     = name of the operator of Ly
    ++ x     = dependant variable (of the operator y)
    ++ n     = order of Ly
    ++ v_i,v = function in x 
    ++ R     = differential ring
    ++ F     = factorial domain
    ++ p     = (minimal) polynomial in u over rational functions in x
    ++ mp    = (minimal) polynomial expression in u over rational 
    ++         functions in x
    ++ u     = dependant variable of p (or mp)

 - ode::vectorize(Ly, y, x, n)
    ++ returns a list of *all* coefficients of Ly: [a_0, a_1, ... a_n].
 - ode::mkODE([a_i], y, x)
    ++ returns a linear homogenous differential equation Ly in y(x), 
    ++ where the last element of the list [a_i] corresponds to
    ++ the leading coefficient of Ly.
 - ode::getOrder(Ly, y(x))
    ++ returns the order of Ly in y(x), even if Ly is not linear.
 - ode::wronskianMatrix([v_i], x, < R >)
    ++ returns the Wronskian matrix of the functions v_i  w.r.t. x.
    ++ When the option R is given, then the specified differential ring will
    ++ be choosed for reprensenting the matrix entries, otherwise 
    ++ Dom::ExpressionField(id, iszero@normal) will be used instead. 
 - ode::Wronskian([v_i], x, < R >)
    ++ returns the Wronskian (determinant) of the functions v_i  w.r.t. x.
 - ode::ode2poly(Ly, y, x, n)
    ++ converts Ly in a polynomial in the derivations of y(x).
 - ode::normalize(Ly, y, x, n)
    ++ returns Ly normalized, i.e. the leading coefficient is 1.
 - ode::derivationOfRootOf(p, u, x, n)
    ++ returns the n-th derivative of u, where u=RootOf(p, u) using 
    ++ p(u)=0. The result diff(u, x$n) is a polynomial in u over the
    ++ rational functions in x of degree lower than degree(p).
 - ode::applyRootOf(Ly, y, x, n, v)
    ++ applies v to Ly, if v contains RootOf else returns an error.
 - ode::evalOde(Ly, y(x)=v)
    ++ evaluates Ly with y(x)=v. 
 - ode::printMinPol(p, u, d, m, < F >)
    ++ returns a reasonable output of a computed minimal polynomial p in u,
    ++ i.e. the coefficients are in factored form in it's numerator and 
    ++ denominator:
    ++         mp(u) = u^(d*m)+c_(m-1)*u^(d*(m-1))+ ... + c_0
    ++ Thereby the parameter d gives the sparsity of p and 
    ++ the parameter m the number of it's coefficients. 
  

See:
 e.g. Walter, W. (1990). Gew"ohnliche Differentialgleichungen. 4th Edition.
      Berlin, Heidelberg, New York: Springer.
 - Comtet, L. (1964). Calcul pratique des coefficients de Taylor d'une 
   fonction alg`ebrique. Einseignement Math. 10, 267-270.
 - Salvy, B., Zimmermann, P. (1994). Gfun: A Maple Package for the 
   Manipulation of Generating and Holonomic Functions in One Variable. 
   toms, Vol.20 No.2, 163-177.
   
Acknowlegdements:
function ode::derivtionOfRootOf stems from Paul Zimmermann.

Examples:

>> eq := 4*x^2*diff(y(x), x$2)+4*x*diff(y(x),x)-y(x);
>> ode::vectorize(eq,y,x,2);   
                                           2
                              [-1, 4 x, 4 x ]

>> ode::mkODE(%, y, x);

                                            2
            - y(x) + 4 x diff(y(x), x) + 4 x  diff(y(x), x, x)


>> ode::wronskianMatrix([sqrt(x),1/sqrt(x),y(x)],x);                    

                +-                                      -+
                |     1/2       1                        |
                |    x   ,     ----,         y(x)        |
                |               1/2                      |
                |              x                         |
                |                                        |
                |     1          1                       |
                |   ------,  - ------,   diff(y(x), x)   |
                |      1/2        3/2                    |
                |   2 x        2 x                       |
                |                                        |
                |      1        3                        |
                |  - ------,  ------,  diff(y(x), x, x)  |
                |       3/2      5/2                     |
                |    4 x      4 x                        |
                +-                                      -+


>> ode::Wronskian([sqrt(x),1/sqrt(x),y(x)],x);      

                   diff(y(x), x, x)   y(x)   diff(y(x), x)
                 - ---------------- + ---- - -------------
                          x              3         2
                                      4 x         x


>> ode::normalize(%,y,x,2);

                  diff(y(x), x)                      y(x)
                  ------------- + diff(y(x), x, x) - ----
                        x                               2
                                                     4 x


++#


sysassign(ode::vectorize, proc(eq, y, x, n)
  local k, vars;
  begin
    vars := [diff(y(x), x$k) $hold(k)=0..n ];
    eq := poly(eq, vars);
    [expr(coeff(eq, vars[k], 1)) $hold(k)=1..n+1 ]
  end_proc):

sysassign(ode::mkODE, proc(coef, y, x)
  begin
    _plus( coef[i+1]*diff(y(x), x$i) $hold(i)=0..nops(coef)-1 )  
  end_proc):


sysassign(ode::getOrder,
  func(max(-infinity, op(map(ode::indets(eq, {op(z,0)}), nops)))-1, eq, z)):


sysassign(ode::wronskianMatrix, proc(vis, x, R) #option: R #
  local i, mat;
  begin
    if R::hasProp(Dom::BaseDomain) = FAIL 
      then R := Dom::ExpressionField(id, iszero@normal) 
     elif not(R::hasProp(Cat::CommutativeRing) and
              R::hasProp(Cat::DifferentialRing) and
              iszero( R::characteristic() ) or
              R::hasProp(Dom::ExpressionField(id, iszero@normal))) 
          then error("illegal coefficient ring") end_if;
    mat := [map(vis, R::new)];
    for i from 1 to nops(vis)-1 do
      mat := append(mat, map(mat[i], diff, x)) 
    end_for;
    (Dom::Matrix(R))::newThis(mat)
  end_proc):

sysassign(ode::Wronskian, 
            fun( expr(linalg::det(ode::wronskianMatrix( args() ))) )):

sysassign(ode::ode2poly, proc(eq, y, x, n)
  begin
     poly(eq, [diff(y(x), x$k) $hold(k)=0..n ])
  end_proc):     

sysassign(ode::normalize, proc(eq, y, x, n)
  begin
    ode::ode2poly(eq, y, x, n);
    #expr(multcoeffs(%, 1/expr(coeff(%, diff(y(x),x$n), 1))))#
    expr(mapcoeffs(multcoeffs(%, 1/expr(coeff(%, diff(y(x),x$n), 1))), normal))
  end_proc):

# computes diff(RootOf(p,y),x$n) as a polynomial in y #
sysassign(ode::derivationOfRootOf, proc(p,y,x,n) local py,g;
option remember;
begin
   if n=0 then y
   elif n=1 then
      py:=diff(p,y);
      g:=gcdex(p,py,y); # g[1]=g[2]*p+g[3]*py #
      collect(divide(-diff(p,x)*g[3]/g[1],p,[y],Rem),[y],normal)
   else
      ode::derivationOfRootOf(p,y,x,n-1);
      diff(%,x)+diff(%,y)*ode::derivationOfRootOf(p,y,x,1);
      collect(divide(%,p,[y],Rem),[y],normal)
   end_if
end_proc):


sysassign(ode::applyRootOf, proc(eq, y, x, n, v)
  local r, p, u, i;
  begin
    v := context(v);
    if hastype(v, RootOf) then r := op(Indets(v, RootOf))
      else error("there is no RootOf(...) to apply") end_if;
    p := expr(op(r, 1)); 
    u := op(r, 2); 
#
    eq := eval( subs(eq, y(x)=subs(v, r=u(x))) );
    subs(eq, [diff(u(x),x$i)=ode::derivationOfRootOf(p,u,x,i) $hold(i)=0..n]); 
    simplify(numer(eval(%)))
#
    eq := subs(eq, y(x)=subs(v, r=u(x)),Simplified); 
    simplify(
       numer(
         subs(eq, 
              [diff(u(x),x$i)=ode::derivationOfRootOf(p,u,x,i) $hold(i)=0..n],
              Simplified
             )
        )
    )
  end_proc):

sysassign(ode::evalOde, proc(eq, subeq)
  local r,s;
  begin
    ode::isLODE(eq, op(subeq, 1), "Lode");
    if nops(%)=1 
      then return(error("not an ordinary linear differential"))
    elif iszero(traperror( (r:=ode::applyRootOf(%,op(subeq, 2))) )) 
      then r
      else
        r:=subs(eq, subeq, Simplified);
        s:=simplify(numer(r));
        if iszero(s) then s
        else normal(s/simplify(denom(r)))
        end_if;
      end_if
    end_proc):
    
sysassign(ode::printMinPol, proc(mp, u, d, m)
  local l, ci, i, F;
  begin
    userinfo(10, "compute a pretty representation of the minimal polynomial");
    if args(0)=5 then
      F:=args(5);
      if F::hasProp(Dom::BaseDomain)=FAIL or 
         not(F::hasProp(Cat::FactorialDomain))
        then error("illegal coefficient ring") end_if
    else F:=Expr end_if; 
    mp := poly(mp, [u], F);
    l := [u^(d*m)];
    for i from m-1 downto 1 do
      ci := coeff(mp, d*i);
      if not iszero(ci) then
        l  := append(l, hold(_mult)(Factor(normal(ci)), u^(d*i)))
      end_if
    end_for;
    hold(_plus)(op(l), Factor(normal(coeff(mp,0)))) 
  end_proc):

    

