# rec::polysol(r,u,n,ord) tries to find polynomial solutions to the
   linear recurrence r for u(n) or order ord with polynomial coefficients.
   Author: Paul Zimmermann <Paul.Zimmermann@loria.fr> 15 February 1996
   Reference: Calcul formel: mode d'emploi. Gomez, Salvy, Zimmermann, 1995, Masson,
      pages 175-176.
   Input: r is an equation in u(n), u(n+1),  ..., u(n+ord)
             u,n are indeterminates
             ord is a positive integer
   Output: a set of polynomial solutions, {} if no poly. solution
               or if recurrence not linear with poly. coeff.
   Examples: 
>> rec::polysol(n*u(n+2)-5*u(n+1)-(n+1)*u(n),u,n,2);

                         {    /        2      \ }
                         {    | n   3 n     3 | }
                         { a3 | - - ---- + n  | }
                         {    \ 2    2        / }

>> rec::polysol(r(n+2)-2*r(n+1)+r(n)-2,r,n,2);

                                           2
                             {a0 + n a1 + n }

>> rec::polysol(-n-u(n) + u(n + 1), u, n, 1);

                             {             2 }
                             {   n   a0   n  }
                             { - - + -- + -- }
                             {   2   2    2  }
#
rec::polysol := proc(r,u,n,ord,mon) local i,l,d,m,alpha,s,j,inho,newr;
# returns only monic solutions if mon=TRUE #
begin
   inho:=r; for i from 0 to ord do
      if not testtype(inho,Type::PolyExpr(u(n+i))) then return({}) end_if;
      if degree(inho,[u(n+i)])>=2 then return({}) end_if;
      l[i]:=coeff(inho,[u(n+i)],1);
      if not testtype(l[i],Type::PolyExpr(n)) then return({}) end_if;
      d[i]:=degree(l[i],[n]); inho:=coeff(inho,[u(n+i)],0);
   end_for;
   if inho<>0 then
      if not testtype(inho,Type::PolyExpr(n)) then return({}) end_if;
      # r(n) = r0(n)+inho(n) = 0 where r0 is homogeneous, 
         therefore inho(n)*r(n+1)-inho(n+1)*r(n) is homogeneous #
      newr:=expand(inho*subs(r,n=n+1)-subs(inho,n=n+1)*r);
      s:={}; for i in rec::polysol(newr,u,n,ord+1,TRUE) do
         # check in the initial recurrence #
         l:=expand(subs(r,u(n+j)=alpha*subs(i,n=n+j)$j=0..ord));
         l:=solve(l,alpha);
         if l<>{} then s:=s union map(l,_mult,i) end_if;
      end_for;
      return(s)
   end_if;
   m:=max(d[i] $ hold(i)=0..ord)+1;
   repeat m:=m-1;
      # compute the coefficient of n^(alpha+m) if deg(u(n),n)=alpha #
      s:=0; for i from 0 to ord do
         # [n^(alpha+m)] l[i]*(n+i)^alpha #
         for j from max(0,m) to d[i] do
            s:=s+coeff(l[i],[n],j)*binomial(alpha,j-m)*i^(j-m);
         end_for
      end_for;
      s:=expand(s);
      if has(s,alpha) then break elif s<>0 then return({}) end_if
   until FALSE end_repeat;
   s:=primpart(s,[alpha]);
   if indets(s)<>{alpha} then return({}) end_if;
   s:=sharelib::iroots(s,FALSE);
   s:=select(s,not _less,0);
   s:=map(s,rec::polysol2,r,u,n,ord);
   if mon<>TRUE then s:=map(s,_mult,genident("a")) end_if;
   s
end_proc:

# rec::polysol2(k,r,u,n,ord) returns a monic polynomial of degree k
   solution of r, if it exists, or null() otherwise,
   r must be homogeneous #
rec::polysol2 := proc(k,r,u,n,ord) local p,a,i;
begin
   if k=0 then
      a:=expand(subs(r,u(n+i)=1$i=0..ord));
      if a=0 then 1 else null() end_if
   else
      p:=_plus(n^k,a.i*n^i$i=0..k-1);
      r:=expand(subs(r,u(n+i)=subs(p,n=n+i)$i=0..ord));
      if r=0 then return(p) end_if;
      a:={coeff(r,[n])},{a.i $ i=0..k-1};
      a:=linsolve(a);
      if a=FAIL or a=null() then null() else subs(p,op(a)) end_if
   end_if
end_proc:

