# This is the main file for solving recurrences.
  rec::solve(r,u(n),inits) solves the recurrence r (or r=0) for u(n)
  with initial conditions inits.
  Output: a set of solutions or solve(rec(...)) if no solution is found

Examples: 
>> solve(rec(r(n+2)-2*r(n+1)+r(n)=2,r(n),{r(0)=-1,r(1)=m}));

                                      2     
                              {m n + n  - 1}

>> solve(rec(r(n+2)-3*r(n+1)+r(n)=2,r(n),{r(0)=-1,r(1)=m}));

 solve(rec(r(n) - 3 r(n + 1) + r(n + 2) = 2, r(n), {r(0) = -1, r(1) = m}))

>> solve(rec(n*u(n+2)-5*u(n+1)-(n+1)*u(n),u(n)));

                         {    /        2      \ }
                         {    | n   3 n     3 | }
                         { a3 | - - ---- + n  | }
                         {    \ 2    2        / }

>> solve(rec(u(n+1)=2*u(n)*(n+1)/n,u(n)));

                                       n - 1
                              {n u(1) 2     }

>> solve(rec(2*u(n)+2*(n+1)*u(n+1)+(n+1)*(n+2)*u(n+2),u(n),{u(0)=0,u(1)=1}));
                {                    n                  n }
                { (- 1/2 I) (- 1 + I)  + 1/2 I (- 1 - I)  }
                { --------------------------------------- }
                {                 fact(n)                 }
#

# this function does argument checking,
  checks that the recurrence only contains linear indices,
  that all increments are integers,
  puts the recurrence in normal form u(n+ord) ... u(n),
  and calls the main solving function rec::Solve #
rec::solve := proc(r,uofn,inits) local u,n,s,ord,islin,m;
begin
   case domtype(r)
   of rec do # interactive call #
      uofn:=extop(r,2);
      inits:=(if extnops(r)>=3 then extop(r,3) else {} end_if);
      r:=extop(r,1);
      # break through #
   of DOM_EXPR do # library call #
      if type(uofn)<>"function" then error("invalid arguments") end_if;
      if nops(uofn)<>1 then error("invalid arguments") end_if;
      u:=op(uofn,0); n:=op(uofn,1);
      if map({u,n},type)<>{DOM_IDENT} then error("invalid arguments") end_if;
      case type(r)
      of DOM_SET do error("invalid arguments")
      of "_equal" do r:=op(r,1)-op(r,2)
      end_case;
      s:=select(indets(r,PolyExpr),testtype,"function");
      s:=select(s,fun(op(args(1),0)=u and has([op(args(1))],n)));
      islin:=select(s,fun(testtype(op(args(1))-n,DOM_INT)));
      if s<>islin then return(procname(args())) end_if;
      # now all terms u(f(n)) are of the form u(n+i) #
      ord:=map(s,fun(op(args(1))-n));
      # don't deal with non-integer increment i #
      if select(ord,testtype,DOM_INT)<>ord then return(procname(args())) end_if;
      # now put the recurrence in normal form, i.e. wrt u(n+ord) ... u(n) #
      m:=min(op(ord));
      if m=0 then ord:=max(op(ord))
      else
         r:=subs(r,n=n-m); ord:=max(op(ord))-m
      end_if;
      s:=rec::Solve(r,u,n,ord,inits);
      if s=FAIL then
         if type((r:=args(1)))<>rec then r:=rec(args()) end_if;
         hold(solve)(r)
      else s
      end_if; break
   otherwise error("invalid arguments")
   end_case
end_proc:

# library call: r must be an expression (equal to 0)
                involving u(n), u(n+1), ..., u(n+ord)
  Output: a set of solutions or FAIL #
rec::Solve := proc(r,u,n,ord,inits) local sol,i,l,s;
begin
   userinfo(1,"solving recurrence of order ".ord);
   r:=numer(r); sol:={};
   userinfo(2,"recurrence is ",r);
   if (l:=Type::Linear(r,[u(n+i)$i=0..ord]))<>FALSE then
      userinfo(1,"recurrence is linear");
      if map({op(l)},testtype,Type::PolyExpr(n))={TRUE} then
         userinfo(1,"   with polynomial coefficients");
         if ord=1 and l[3]=0 then
            # homogeneous of order 1 l[1]*u(n)+l[2]*u(n+1)=0 #
            sol:=rec::hypergeom(-l[1]/l[2],inits,u,n);
            if sol<>FAIL then return({sol}) end_if;
         else
            sol:={};
            if not has(l,n) then sol:={rec::constant(l,n,ord)} end_if;
            if sol={} then sol:=rec::polysol(r,u,n,ord) end_if;
            if sol={} and l[ord+2]=0 then # tries the recurrence for u(n)/n! #
               s:=1; for i from 1 to ord do
                  s:=s*(n+i);
                  l[i+1]:=normal(l[i+1]/s);
               end_for;
               if not has(l,n) then
                  sol:={rec::constant(l,n,ord)};
                  if sol<>{} then sol:=map(sol,_mult,1/fact(n)) end_if
               end_if
            elif ord=1 and nops(sol)=1 and not has(sol,n) then
               sol:=map(subs(rec::Solve(subs(r,u=u+op(sol,1),Simplified),u,n,ord,{}),u=u-op(sol,1),Simplified),_plus,op(sol,1))
            end_if
         end_if
      end_if;
   end_if;
   if sol={} or sol=FAIL then return(FAIL) end_if;
   if args(0)>=5 and inits<>{} then
      userinfo(1,"trying to fulfill initial conditions");
      case type(inits) of "_equal" do inits:={inits} of DOM_SET do break
      otherwise error("illegal initial conditions")
      end_case;
      sol:=map(sol,rec::Solve_inits,r,u,n,inits)
   end_if;
   if sol={} then return(FAIL) end_if;
   sol
end_proc:

rec::Solve_inits := proc(sol,r,u,n,inits) local sys,eq,unk;
begin
   sol:=subs(sol,op(inits));
   sys:={}; for eq in inits do
      # sys:=sys union {eval(subs(sol,n=op(eq,[1,1])))=op(eq,2)} #
      sys:=sys union {subs(sol,n=op(eq,[1,1]),Simplified)=op(eq,2)}
   end_for;
   unk:=indets(sys) minus indets(r) minus indets(inits);
   if unk={} then
      sys:=map(sys,fun((if type(args(1))="_equal" then 
	op(args(1),1)-op(args(1),2) end_if)));
      if sys={0} or sys={} then sol else null() end_if
   else
      sys:=solve(sys,unk);
      if nops(sys)=0 then return(null()) end_if;
      subs(sol,op(op(sys,1)));
   end_if
end_proc:
