
#--

stats::Tdist -- Compute the T distribution

Call:

stats::Tdist(x, v)

x -- value
v -- degrees of freedom (positive integer)

f(u,v) := (1+u^2/v)^(-1/2*v-1/2) * gamma((v+1)/2)/(sqrt(PI*v)*gamma(v/2))

stats::Tdist(x, v) = int(f(u), u = -infinity..x)
                   = int((1+u^2/v)^(-1/2*v-1/2), u = -infinity..x) *
                     gamma((v+1)/2)/(sqrt(PI*v)*gamma(v/2))
--#

stats::Tdist := proc(x, v)
   local F, r, y;
begin
   if args(0) <> 2 then
      error("Two arguments expected")
   elif not testtype(v, Type::PosInt) then
      error("2nd argument must be a positiv integer")
   end_if;
   F:=stats::I(v, y) * gamma((v+1)/2)/(sqrt(PI*v)*gamma(v/2));
   # r:=eval(subs(F, y=x)); #
   r:=subs(F, y=x,Simplified);
   if testtype(x, DOM_FLOAT) then r := float(r) end_if;
   r
end_proc:

#--
  returns I(v,x)=int((1+u^2/v)^(-1/2*v-1/2),u=-infinity..x)
  where v is a positive integer.
 
  I(v,x)=int(1/(1+u^2/v)^(v/2+1/2),u=-infinity..x)
        =sqrt(v)*int(1/(1+u^2)^(v/2+1/2),u=-infinity..x/sqrt(v))
 
  Reference: Calcul formel : mode d'emploi. Exemples en Maple.
        Gomez, Salvy, Zimmermann, 1995, Masson.
--#
 
stats::I := proc(v,x) 
   local g,y;
begin
   if v mod 2 = 1 then 
      g:=stats::In((v+1)/2,y) else g:=stats::Jn(v/2,y) end_if;
   # expand(sqrt(v)*(eval(subs(g,y=x/sqrt(v)))-limit(g,y=-infinity))) #
   expand(sqrt(v)*(subs(g,y=x/sqrt(v),Simplified)-limit(g,y=-infinity)))
end_proc:
 
#--
  returns int(1/(1+x^2)^n,x) where n>=1 
--#

stats::In := proc(n,x)
   option remember; # to accelerate further calls #
begin
   if n=1 then 
      atan(x)
   else    
      # using integration by parts on 1/(1+x^2)^n, we get the recurrence #
      # I(n)=x/(1+x^2)^n+2*n*I(n)-2*n*I(n+1), see [GoSaZi95] p. 252      #
      (x/(1+x^2)^(n-1)+(2*n-3)*stats::In(n-1,x))/(2*n-2)
   end_if
end_proc:

#--
  returns int(1/(1+x^2)^(n+1/2),x) where n>=1
  Using integration by parts, we get the recurrence
  J(n)=x/(1+x^2)^(n+1/2)+(2*n+1)*J(n)-(2*n+1)*J(n+1)
--#

stats::Jn := proc(n,x)
  option remember; # to accelerate further calls #
begin
   if n=1 then 
      x/(1+x^2)^(1/2)
   else 
      (x/(1+x^2)^(n-1/2)+(2*n-2)*stats::Jn(n-1,x))/(2*n-1)
   end_if
end_proc:
