
/*

Usage:
reg("advanced", data, expr, v, p, p_init)
reg(data, expr, v, p)

nonlinear least-squares-fit of an arbitrary function
(nonlinear concerning independent variables and nonlinear concerning 
 the fit-parameters)

Parameters:
data  data - this may be a sample with specified columns 
                      or a sequence of columns
                      or a list of rows.
             the number of columns depends on the number of 
             independent variables.
             the order of columns is:
             independent variables (x_i), dependent variable (y),
                      errors/standard deviations of the dependent 
                      variable (sigma_y)
             the sigma_y are optional
expr         function to fit
v            list of independent variables
p            list of fit-parameters
p_init       list of starting values for fit-parameters

Example:

S:=[[1,2,3], [4,5,9], [7,8,15]]:
R:=stats::reg(S, a0*x+a1*y, [x,y], [a0, a1]);

	[ [1.0, 1.0], [1.312334645, 1.105541596],

        0.00000000000000000000000000000000004326994705 ]


a0 := R[1][1]; a1 := R[1][2];

*/

stats::reg := proc()
  local llll, x, y, fitf, fitderiv, xvar_, xvar, param, 
        N, n, i, j, k, m, p, a, deltaa, al, alpha, alphal,
        lambda, bet, betav, chis, chisqu, chisqu_old, w,
        minv, det, counter_out, counter_in, redigits,
        iter, conv, max_out, sigmay, number_x,
        dummy, dummy2, aout, errout;

  /*

  x[i]          x-data
  y[i]          y-data
  fitf          fitting function
  fitderiv[i]  fitting function's derivatives
  xvar_         user's fitting variable
  xvar          symbolic fitting variable in fitting function
  param[i]      user's fitting parameters
  N             number of (x,y)-pairs
  n             number of fitting parameters
  i, j, k, m    indices
  p[i]          symbolic fitting parameters in fitting function
  a[i]          current values of the fitting parameters
  aout[i]       list of converged parameters
  errout[i]     list of errors of those parameters (sigma_aout)
  deltaa[i]     vector in parameter space (step to next parameter)
  al[i,j]       symbolic matrix alpha
  alpha[i,j]    matrix alpha
  alphal[i,j]   matrix alpha(lambda)
  lambda        marquard-factor
  bet[i]        symbolic gradient vector
  betav[i]      gradient vector
  chis          symbolic chisquare
  chisqu        chisquare
  w             weighting factor
  minv()        matrix inversion of a symmetric square matrix
  det           determinant of alphal
  counter_out   counter for outer loop
  counter_in    counter for inner loop
  redigits      1/10^(DIGITS-1)
  iter          FALSE if converged
  conv          number of converged parameters in current loop
  max_out       maximum number of outer loops
  sigmay        TRUE if x-y-data includes sigma_y
  
  */

begin

  /* subroutine to invert matrix and calculate det */

  minv:=proc()
    local i, j, ol, ii, ji, i_ol, j_ol, h, maxentry, store;
    /* alphal[i,j], n, det are treated global */
  begin
    /* outer loop */
    det:=1;
    for ol from 1 to n do
      i_ol:=-1;
      j_ol:=-1;

      /* order matrix (get largest elements in diagonal) */

      while j_ol < 0 do
        while i_ol < 0 do

          /* find largest entry in remaining matrix */
          maxentry:=0;
          for i from ol to n do
            for j from ol to n do
              if abs(alphal[i,j]) >= abs(maxentry) then  
                maxentry:=alphal[i,j];
                ii[ol]:=i;
                ji[ol]:=j;
              end_if;
            end_for
          end_for;

          if maxentry = 0.0 then
            det:=0; 
            alphal:=FAIL; 
            return([]); 
          end_if;

          i:=ii[ol];
          i_ol:=i - ol;
        end_while;

        /* move largest entry to alphal[ol, ol] */
        if i_ol > 0 then
          for j from 1 to n do
            store:=alphal[ol, j];
            alphal[ol, j]:=alphal[i, j];
            alphal[i, j]:=-store;
          end_for;
        end_if;

        j:=ji[ol];
        j_ol:=j - ol;
      end_while;

      if j_ol > 0 then
        for i from 1 to n do
          store:=alphal[i, ol];
          alphal[i, ol]:=alphal[i, j];
          alphal[i, j]:=-store;
        end_for;
      end_if;

      /* inversion of ordered matrix */

      for i from 1 to n do
        if _not(i = ol) then
          alphal[i, ol]:=-alphal[i, ol]/maxentry
        end_if
      end_for;

      for i from 1 to n do
        for j from 1 to n do
          if _not(i = ol) then
            if _not(j = ol) then
              alphal[i, j]:=alphal[i, j] + alphal[i, ol]*alphal[ol, j]
            end_if
          end_if
        end_for
      end_for;

      for j from 1 to n do
        if _not(j = ol) then
          alphal[ol, j]:=alphal[ol, j]/maxentry
        end_if
      end_for;

      det:=det*maxentry;
      alphal[ol, ol]:=1/maxentry;
    end_for;
    /* end of outer loop */

    /* restore original arrangement of entries */
    for h from 1 to n do
      ol:= n - h + 1;

      j:=ii[ol];
      if ol < j then
        for i from 1 to n do
          store:=alphal[i, ol];
          alphal[i, ol]:=-alphal[i, j];
          alphal[i, j]:=store
        end_for
      end_if;

      i:=ji[ol];
      if ol < i then
        for j from 1 to n do
          store:=alphal[ol, j];
          alphal[ol, j]:=-alphal[i, j];
          alphal[i, j]:=store
        end_for
      end_if;

    end_for
  end_proc;

  /* get data */

  if args(1) <> "advanced" then
    if testargs() then 
      if domtype(args(args(0)-2)) <> DOM_EXPR then 
        error("3rd-last argument (function to fit) has to be of type DOM_EXPR")
      elif domtype(args(args(0)-1)) <> DOM_LIST then
        error("2nd-last argument (independent variables) has to be of type DOM_LIST")
      elif domtype(args(args(0))) <> DOM_LIST then
        error("last argument (fit-parameters) has to be of type DOM_LIST")
      end_if;

      number_x:=nops(args(args(0)-1));
      llll:=stats::getdata(testargs(), "numeric_only", number_x+1, args(1..args(0)-3));
      if domtype(llll) = DOM_STRING then 
        llll:=stats::getdata(testargs(), "numeric_only", number_x+2, args(1..args(0)-3));
      end_if;
      if testargs() then
        if domtype(llll) = DOM_STRING then error(llll) end_if
      end_if;  

    end_if;
    stats::reg("advanced", args(), [$1..nops(args(args(0)))])
  else

  if testargs() then
    if domtype(args(args(0)-3)) <> DOM_EXPR then 
      error("4th-last argument (function to fit) has to be of type DOM_EXPR")
    elif domtype(args(args(0)-2)) <> DOM_LIST then
      error("3rd-last argument  (independent variables) has to be of type DOM_LIST")
    elif domtype(args(args(0)-1)) <> DOM_LIST then
      error("2nd-last argument (fit-parameters) has to be of type DOM_LIST")
    elif domtype(args(args(0))) <> DOM_LIST then
      error("last argument (starting values for fit-parameters) has to be of type DOM_LIST")
    end_if
  end_if;

  number_x:=nops(args(args(0)-2));
  llll:=stats::getdata(testargs(), "numeric_only", number_x+1, args(2..args(0)-4));
  if domtype(llll) = DOM_STRING then 
    llll:=stats::getdata(testargs(), "numeric_only", number_x+2, args(2..args(0)-4));
    sigmay:=TRUE
  else
    sigmay:=FALSE
  end_if;
  if testargs() then
    if domtype(llll) = DOM_STRING then error(llll) end_if
  end_if;  

  /* initialize */
  
  fitf:=args(args(0)-3);
  param:=args(args(0)-1);
  xvar_:=args(args(0)-2);
  N:=nops(llll[1]); n:=nops(param);

  if testargs() then
    if _not(N-n > 0) then error("too many parameters") end_if;
    for i from 1 to n do
      if (diff(fitf, op(param, i))) = 0 
        then error("parameter ".expr2text(op(param,i))." missing in function to fit")
      end_if
    end_for;
    for i from 1 to number_x do
      if (diff(fitf, xvar_[i])) = 0 
          then error(expr2text(xvar_[i])." missing in function to fit") end_if
    end_for
  end_if;

  unassign(i);

  xvar:=genident("X");
  fitf:=subs(fitf, xvar_[i]=xvar[i] $i=1..number_x);

  p:=genident("p");
  fitf:=subs(fitf, op(param, i)=p[i] $i=1..n);

  for i from 1 to number_x do
    x[i]:=llll[i];
  end_for;
  y:=llll[number_x+1];

  /* starting values of parameters */
  unassign(a);
  a:=args(args(0));

  /* calculate weights */
  if sigmay = FALSE then
    for i from 1 to N do
      w[i]:=1 
    end_for
  else
    for i from 1 to N do
      w[i]:=1/(llll[3][i]^2)
    end_for
  end_if;

  redigits:=1/(10^(DIGITS-1));
  max_out:=500;
  iter:=TRUE;

  /* calculate all derivatives fitderiv[i] of function to fit */

  for i from 1 to n do
    fitderiv[i]:=diff(fitf, p[i]);
  end_for;

  /* calculate symbolic matrix al[i,j] */

  for j from 1 to n do
    for k from 1 to n do
      al[j, k]:=0
    end_for;
  end_for;
  for i from 1 to N do
    dummy:=subs(fitderiv, xvar[m]=x[m][i] $hold(m)=1..number_x);
    for j from 1 to n do
      for k from 1 to n do
        al[j, k]:=al[j, k] + w[i]*eval(dummy[j]*dummy[k]);
      end_for
    end_for
  end_for;

  /* calculate symbolic gradient bet[i,j] */

  unassign(m);

  for k from 1 to n do
    bet[k]:=0;
  end_for;
  for i from 1 to N do
    dummy:=subs(fitderiv, xvar[m]=x[m][i] $m=1..number_x);
    dummy2:=subs(fitf, xvar[m]=x[m][i] $m=1..number_x);
    for k from 1 to n do
      bet[k]:=bet[k] + w[i]* ( (y[i] - dummy2)*dummy[k] )
    end_for
  end_for;

  /* calculate symbolic chisquare chis */

  chis:=0;
  for i from 1 to N do
    dummy2:=subs(fitf, xvar[m]=x[m][i] $m=1..number_x);
    chis:=chis + w[i]*( (y[i] - dummy2)^2 )
  end_for;

  /* calculate initial chisquare(a) */

  unassign(i);

  chisqu_old:=chis;
  chisqu_old:=subs( chisqu_old, p[i]=a[i] $i=1..n);
  /* this is vital for dealing with trigonometric functions */
  chisqu_old:=Re(float(chisqu_old)); 
  if chisqu_old > 0.0 then 
    chisqu:=2*chisqu_old
  elif chisqu_old = 0.0 then 
    chisqu:=10
  else
    return(FAIL) /* illegal starting parameters */
  end_if;

  counter_out:=0;
  lambda:=0.001;

  /* begin outer loop (searching parameter space) */

  while (iter and counter_out < max_out) do

    counter_out:=counter_out+1;

    /* calculate alpha and betav */

    alpha:=subs( al, p[i]=a[i] $hold(i)=1..n);
    betav:=subs( bet, p[i]=a[i] $hold(i)=1..n);
    alpha:=map(map(alpha, float), Re);
    betav:=map(map(betav, float), Re);

    counter_in:=0;

    /* begin inner loop (computing step in parameter space (deltaa)) */

    while chisqu > chisqu_old do

      counter_in:=counter_in+1;
      if counter_in > 500 then 
        print("no convergence in inner loop!");
        return(FAIL)
      end_if;

      /* alphal(lambda) */

      alphal:=alpha;
      for i from 1 to n do
        alphal[i,i]:=(1 + lambda)*alphal[i,i]
      end_for;

      /* invert alphal(lambda) */

      minv();
      if det = 0 then 
        print("matrix of curvature is singular!");
        return(FAIL)
      end_if;

      /* calculate deltaa */

      for j from 1 to n do
        deltaa[j]:=0;
        for k from 1 to n do
          deltaa[j]:=deltaa[j] + betav[k]*alphal[j, k]
        end_for
      end_for;

      /* calculate chisquare(a+deltaa) */

      chisqu:=subs( chis, p[i]=a[i]+deltaa[i] $hold(i)=1..n);
      chisqu:=Re(float(chisqu));

      if chisqu > chisqu_old then 
        lambda:=lambda*10
      end_if;

    end_while;

    /* end inner loop */

    /* check convergence */

    conv:=0;
    for i from 1 to n do
      if abs(deltaa[i]/a[i]) <= redigits/100 then 
        conv:=conv+1
      end_if
    end_for;
    if conv = n then
      iter:=FALSE
    end_if;

    lambda:=lambda/10;
    chisqu_old:=chisqu;
    chisqu:=chisqu+10;
    for i from 1 to n do
      a[i]:=a[i]+deltaa[i]
    end_for;

  end_while;

  /* end outer loop */

  if counter_out = max_out then 
    print("Warning: poor convergence!");
  end_if;

  aout:=[$1..n];
  errout:=[$1..n];

  for i from 1 to n do
    aout[i]:=a[i];
  end_for;

  /* calculate alphal(lambda=0) and invert ( => error matrix ) */

  alphal:=subs( al, p[i]=a[i] $hold(i)=1..n);
  alphal:=map(map(alphal, float), Re);

  minv();
  if det = 0 then 
     errout:=FAIL;
  else
    for i from 1 to n do
      errout[i]:=sqrt(alphal[i, i]);
    end_for;
  end_if;

  if errout <> FAIL then
    if _plus(op(map(map(errout, float), Im))) <> 0.0 then
      return(FAIL);
    end_if
  end_if;

  if chisqu_old < 0 then return(FAIL) end_if;

  return([aout, errout, chisqu_old]);

  end_if;

end_proc:

/*

usage:
unzipCol(l)

the sub-lists' elements are gathered in lists according to their
element index. these lists form the resulting sequence.

parameters:
l  list of lists

*/

stats::unzipCol:=proc(l)
  local i;
begin
  if testargs() then
    if args(0) <> 1 then
      error("Wrong number of arguments")
    end_if;
    if domtype(l) <> DOM_LIST
       or op(l) = null()
       or has(map([op(l)], proc() begin bool(domtype(args(1)) = DOM_LIST
                                           ) end_proc ), FALSE)
    then
      error("Argument must be list of lists")
    end_if
  end_if;
  [];
  for i from 1 to max(0, op(map(l, proc() begin nops(args(1)) end_proc ))) do
    append(last(1), map(l, proc() begin op(args(1), i) end_proc ))
  end_for;
  op(last(1))
end_proc:


/*

usage:
getdata(test_args, opt, nc, l) / getdata(test_args, opt, nc, s, c)

internal procedure, required by procedures like mean, median,...
to get and / or test their arguments / elements of arguments.
if test_args is TRUE, then all arguments are tested, if test_args is
FALSE, no arguments are tested.
if a data-column of a sample is specified, the column index is verified
if no data-column of a sample is specified, then all data-columns are
selected.
option "all_data":
                allows DOM_INT, DOM_FLOAT, DOM_RAT, DOM_EXPR, DOM_IDENT
option "numeric_only":
                allows DOM_INT, DOM_FLOAT, DOM_RAT
if called by a procedure or function, all errors are returned as
strings. if no errors are found, a sequence of data-columns is returned.

parameters:
test_args  TRUE or FALSE
opt option, "all_data" or "numeric_only"
nc  number of allowed data-columns
l   list of rows or sequence of columns
c   (list of) data-column index/indices (optional)

*/

stats::getdata := proc(test_args, opt, nc, li, co)
  local l, c, err, err_msg;
begin
  l:=li;
  if args(0)>=5 then
	c:=co
  end_if; 
  if testargs() then /* return errors as errors */
    err:=error;
    if args(0) < 4 then error("Wrong number of arguments") end_if;
    if  not testtype(test_args, DOM_BOOL) or
        (opt <> "all_data" and opt <> "numeric_only") or
        domtype(nc) <> DOM_INT or nc < 1 then 
      error("Invalid arguments")
    end_if
  else     /* return errors as strings (if getdata is called by procedure) */
    err:=return
  end_if;

  if domtype(l) = DOM_LIST then 
    if domtype(op(l, 1)) = DOM_LIST then                   /* list of rows */
      if test_args then
        if args(0) <> 4 then
          err("Wrong number of arguments")
        elif has(map(l, proc() begin (if nops(args(1)) <> nops(op(l, 1)) then 
                               FAIL 
                             end_if) end_proc ), FAIL)
        then
          err("Rows are not of equal length")
        end_if
      end_if;
      l:=stats::unzipCol(l);
      if test_args then
        map(l, proc() begin (stats::testdata(opt, args(1));
                    if last(1) <> TRUE then err_msg:=last(1) end_if) end_proc );
        if domtype(err_msg) = DOM_STRING then err(err_msg) end_if
      end_if

    elif nc > 1 then                       /* sequence of columns (nc > 1) */
      l:=args(4..args(0));
      if test_args then
        if l = [] then err("Column empty") end_if;
        if has(map([l], proc() begin (if nops(args(1)) <> nops(op([l], 1)) then 
                               FAIL 
                             end_if) end_proc ), FAIL)
        then
          err("Columns are not of equal length")
        end_if;
        map(l, proc() begin (stats::testdata(opt, args(1));
                    if last(1) <> TRUE then err_msg:=last(1) end_if) end_proc );
        if domtype(err_msg) = DOM_STRING then err(err_msg) end_if
      end_if

    else                                                       /* 1 column */
      if test_args then
        stats::testdata(opt, args(4..args(0)));
        if last(1) <> TRUE then err(last(1)) end_if
      end_if
    end_if;

 
  else                                             
    l:=[args(4..args(0))];
    if test_args then
      stats::testdata(opt, l);
      if last(1) <> TRUE then err(last(1)) end_if
    end_if
  end_if;

  if test_args and nops([l]) <> nc then
    err("Wrong number of data-columns")
  end_if;
  return(l)
end_proc:


/*

usage:
testdata(opt, l) 

internal procedure, required by procedures like getdata to
test arguments / elements of arguments.
if a data-column of a sample is specified, the column index is verified
option "all_data":
                allows DOM_INT, DOM_FLOAT, DOM_RAT, DOM_EXPR, DOM_IDENT
option "numeric_only":
                allows DOM_INT, DOM_FLOAT, DOM_RAT
if called by a procedure or function, all errors are returned as
strings. if no errors are found, TRUE is returned.

parameters:
opt option, "all_data" or "numeric_only"
l   list or sequence

*/

stats::testdata :=
proc(opt, li, co)
  local l, c, err;
begin
  l:=li; 
  if testargs() then /* return errors as errors */
    err:=error;
    if args(0) < 2 then error("Wrong number of arguments") end_if;
    if opt <> "all_data" and opt <> "numeric_only" then 
      error("Invalid arguments")
    end_if
  else /* return errors as strings (if testdata is called by procedure) */
    err:=return
  end_if;
   
  if domtype(l) = DOM_LIST then
    if args(0) <> 2 then /* 1 + opt */
      err("Wrong number of arguments")
    end_if;
  else /* sequence to list */
    l:=[args(2..args(0))]; /* exclude opt from sequence */
  end_if;
  case opt
    of "all_data" do
      if has(map(l, proc() begin testtype(args(1), DOM_INT)
                     or testtype(args(1), DOM_FLOAT) 
                     or testtype(args(1), DOM_EXPR)
                     or testtype(args(1), DOM_IDENT)
                     or testtype(args(1), DOM_RAT) end_proc ), FALSE) then 
        err("Invalid operand (Type DOM_INT, DOM_FLOAT, DOM_RAT,
               DOM_EXPR or DOM_IDENT expected)")
      end_if; break;
    of "numeric_only" do
      if has(map(l, proc() begin testtype(args(1), DOM_INT) 
                     or testtype(args(1), DOM_FLOAT) 
                     or testtype(args(1), DOM_RAT) end_proc ), FALSE) then 
        err("Invalid operand (Type DOM_INT, DOM_FLOAT or DOM_RAT, expected)")
      end_if;
  end_case;
return(TRUE)
end_proc:

 

