# #
# $Date: 1997/08/04 14:56:07 $ $Author: stefanw $ $Revision: 1.18 $ #
# #
# frankp, 18.08.1994 #

#++
Im.mu

	Im -- A function for computing the imaginary part of 
              an expression


	For computing the imaginary part of an expression x, 
	the method "Im" of the domain of x will be used.
	If this method does not exist, then 'Im(x)' will be
	returned.

	The basis types DOM_INT, DOM_RAT, DOM_FLOAT, DOM_COMPLEX
	and DOM_EXPR be an exception. These types are treated by
	the procedure  'Im'.
	For expressions of type DOM_EXPR the procedure 'Im'
	looks for a function attribute 'Im', which will be used 
	for x if such exist.
	Otherwise either, if the function environment of x is one
	of the built-in environments "_plus", "_mult" or "_power", 
	x will be treated by the procedure 'Im' or the expression 
	'Im(x)' will be returned respectivly.
++#

Im := func_env(
proc(x)
    name Im;
    local t, multSplit, a, ai, i;
begin
    if (t:= x::Im) <> FAIL then return( t(args()) ) end_if;
    if args(0) <> 1 then error("wrong no of args") end_if;

    case (t:= domtype(x))
    of DOM_INT     do
    of DOM_RAT     do return( 0 )
    of DOM_FLOAT   do return( 0.0 )
    of DOM_COMPLEX do return( op(x,2) )
    end_case;

    if testtype( x,Type::Constant ) then
        # is x real ? #
        if domtype( float(x) ) = DOM_FLOAT then return( 0 )
	# is x an indexed identifier ? #
	elif type(x) = "_index" then
            return( procname(x) )
        end_if
    end_if;

    if t = DOM_EXPR then
	t := op(x,0);
	if domtype(level(t,2)) = DOM_FUNC_ENV and
	   (a:= funcattr(level(t,2),"Im")) <> FAIL 
	then
	    return( a(op(x)) )
	elif domtype(level(t,2)) = DOM_IDENT then
	# f(x) #
	    return( procname(args()) )
	end_if;
    
        multSplit := proc(z) local t,a,b,ci,cr,d;
            begin
            # split product into real and imaginary parts if possible #
                cr := 1; ci := 0;
                for t in op(z) do
                    a := Re(t);
                    if has(a,Re) then return( FAIL ) end_if;
                    b := Im(t);
                    if has(b,Im) then return( FAIL ) end_if;
                    d := cr;
                    cr := cr*a - ci*b;
                    ci := d*b + ci*a
                end_for;
                [cr,ci]
            end_proc:

	case t
	of hold(_plus) do
	    return( _plus( Im( op(x,i) ) $ hold(i)=1..nops(x) ) );
        of hold(_mult) do
            # expressions of sign -1 must be treated separatedly #
            #if has(x,-1) then
		return( -Im(-x) )
	    end_if;#

            # find constant terms #
	    t:= split( x,testtype,Type::RealNum );
            if t[1] <> 1 and t[1] <> 1.0 then 
		return( t[1]*Im(t[2]*t[3]) )
            end_if;

	    # split product into real and imaginary parts if possible #
	    t := multSplit(x);
            if t <> FAIL then return( t[2] ) end_if;

            # test if the expression is a symbolic complex number #
            if contains( {op(x)}, I ) then
		# I*ex = (Re(ex)+Im(ex)*I)*I = Re(ex)*I-Im(ex) #
		# note: using 'contains', 1/I = -I will not be considered where
		        this rule would lead to a wrong result
		#
                return( Re( subs(x,I=1) ) )
	    else
		return( procname(x) )
	    end_if
        of hold(_power) do
            if op(x,2) = 1/2 then
                case domtype( op(x,1) )
                of DOM_INT     do
                of DOM_RAT     do
                of DOM_FLOAT   do return( 0 )
                of DOM_COMPLEX do
                    t := op( op(x,1) );
                    return( sqrt((-op(t,1)+sqrt(op(t,1)^2+op(t,2)^2))/2) )
	        otherwise
		    if is(op(x,1),Type::RealNum)=TRUE then
		    	if is(op(x,1)>=0)=TRUE then
				return(0)
		    	elif is(op(x,1)>=0)=FALSE then
				return(sqrt(-op(x,1)))
			end_if
                    end_if				
                end_case
            elif domtype(op(x,2)) = DOM_INT then
                case domtype(op(x,1))
                of DOM_INT      do
                of DOM_RAT      do
                of DOM_FLOAT    do return( 0 )
                otherwise
                    a := Re(op(x,1));
                    ai := Im(op(x,1));
                    if has(a,{Re,Im}) or has(ai,{Re,Im}) then
                        return( procname(x) )
                    end_if;
                    t := [a,ai];
                    if op(x,2) > 0 then
                        t := multSplit( [(a+I*ai) $ op(x,2)] );
                        if t = FAIL then
			    return( procname(x) )
			else 
			    return( t[2] ) 
			end_if
                    end_if;
                    if op(x,2) < -1 then
                        t := multSplit( [(a+I*ai) $ -op(x,2)] );
                        if t = FAIL then return( procname(x) ) end_if
                    end_if;
                    # case op(x,2) = -1 #
                    return( -t[2]/(t[1]^2+t[2]^2) )
		end_case
            elif testtype( op(x,2),NUMERIC ) then
                # hold is *very* important, otherwise
		  exp simplifies back to op(x,1)^op(x,2)
                  and we get an infinite loop #
		t := Im(hold(exp)(op(x,2)*ln(op(x,1))));
		if type(t) <> "Im" then return( t ) end_if
            end_if
        end_case
    end_if;

    # uses information from assume #
    if is(x,Type::RealNum)=TRUE then return(0) end_if;

    procname(x)
end_proc,
NIL,
table( "type"="Im", "print"="Im", "Im"=0, "Re"=fun(hold(Im)(args(1))),
       "info"="Im(x) -- the imaginary part of x" )
):

# #
# Further Function Attributes #
# #

Im:= funcattr(Im,"rectform",
   loadproc(funcattr(Im,"rectform"),pathname("STDLIB","RECTFORM"),"Im")
):

# end of file #
