# $Date: 1999/08/02 12:14:43 $  $Author: fakler $  $Revision: 1.25.2.1 $ #

proc()
begin

#++
O -- the domain of order terms of polynomial type

O's are used to represent order terms of series for example. They may
be of polynomial type only.

f(x) = O(g(x), x=x_0) means that there is a constant c with
|f(x)/g(x)| < c as x --> x_0. Note that x_0 may not be 'infinity'!

An order term has 2 operands:
- an expression representing the term,
- a set of equations v=p where v is an indeterminate and p a limit point.

The expression is simplified by O::create to get some crude 'normal
form' (not that is really has a normal form).

The operators +, -, *, 1/ and ^ are defined for order terms. Note that
1/ may cause an error if the operator is not O(1).
++#

O:= domain("O"):
O::info:= "Domain 'O': Big O order terms of series.":
O::interface:= {}:


#--
O::new -- create order term

O::new(e, [v1=p1,...])

e  - polynomial expression
vi - variables
pi - limit points

Create order term from expression e. The limit points are all assumed to
be 0, if not given explicitly.
--#

O::new:= proc(x) local p, flatO; name O; begin
    if (domtype(x))::O <> FAIL then return((domtype(x))::O(args())) end_if;
    case args(0)
    of 0 do error("wrong no of args");
    of 1 do
	p:= map(indets(x), _equal, 0);
	break;
    otherwise
        p:= {args(2..args(0))};
	if testargs() then
	    if map(p, type) <> {"_equal"} then
		error("illegal limit point")
	    end_if;
	    if nops(map(p, op, 1)) <> nops(p) then
		error("multiple limit point")
	    end_if;
	    if has(p, {infinity, complexInfinity}) then
		error("infinite limit point");
	    end_if;
	end_if;
    end_case;

    # flat out nested O's #
    if hastype(x, O) then
	flatO:= fun((
	    p:= p union extop(args(1),2);
	    extop(args(1),1);
	));
	x:= maprec(x, {O}=flatO, PreMap);
	if nops(map(p, op, 1)) <> nops(p) then
	    error("multiple limit point")
	end_if;
    end_if;

    O::create(x, p);
end_proc:


#--
O::expr -- convert order term into expression

O::expr(o)

o - order term
--#

O::expr:= fun((
    hold(O)(expr(extop(args(1),1)), op(
		select(extop(args(1),2), fun(op(args(1),2) <> 0))))
));

O::print:= O::expr:

#--
O::TeX -- return TeX-string for order term

O::TeX(o)

o - order term
--#

O::TeX:= fun(generate::TeX(O::expr(args(1)))):

#--
O::indets -- return list of indets

O::indets(o)

o - order term
--#

O::indets:= fun(map(extop(args(1),2), op, 1)):

#--
O::points -- return list of limit points

O::points(o)

o - order term
--#

O::points:= fun(extop(args(1),2)):

#--
O::create -- create order term

O::create(e, p)

e - expression representing order term
p - set of equations 'indet=limit_point'
--#

O::create:= proc(x, p) local v; begin
    v:= map(p, op, 1);
    if select(map(p, op, 2), not iszero) = {} then
	x:= O::basicSimplify(x, v);
    else
	# shift vars to 0, simplify and shift back #
	x:= O::basicSimplify(subs(x, [ op(map(p,
		    fun(subsop(args(1),2=op(args(1),1)+op(args(1),2)))
		)) ]), v);
	x:= subs(x, [ op(map(p,
		    fun(subsop(args(1),2=op(args(1),1)-op(args(1),2)))
		)) ]);
    end_if;
    if iszero(x) then 0 else new(O, x, p) end_if
end_proc:

O::basicSimplify:= proc(x, v) local p, q, i, j;
begin
    i:= indets(x);
    if i minus v <> {} then
	x:= subs(x, map([op(i minus v)], _equal, 1));
	return(O::basicSimplify(x, v));
    end_if;
    if i = {} then return(1) end_if;
    p:= poly(x, [op(v)]);
    if p = FAIL then return(x / O::icontent(x)) end_if;
    if iszero(p) then return(0) end_if;
    if has([coeff(p)], v) then return(x / O::icontent(x)) end_if;
    if ldegree(p) = 0 then return(1) end_if;
    if nops(op(p,2)) = 1 then return(op(p,[2,1])^ldegree(p)) end_if;
    # remove any term which may be divided by another one #
    x:= [ nthmonomial(p,i) $ hold(i)=1..nterms(p) ];
    q:= { op(x) };
    for i in x do
        for j in q do
            if i <> j and divide(i, j, hold(Exact)) <> FAIL then
                q:= q minus {i};
                break;
            end_if
        end_for
    end_for;
    x:= _plus(op(map(q, expr)));
    x / O::icontent(x)
end_proc:

O::icontent := proc(x) local i;
begin
   case type(x)
   of "_mult" do
      if type((i:=op(x,nops(x))))=DOM_INT then i else 1 end_if; break;
   of "_plus" do
      igcd(O::icontent(op(x,i)) $ i=1..nops(x)); break;
   of DOM_INT do
      x; break;
   otherwise
      1;
   end_case
end_proc:

#--
O::_plus -- add order terms

O::_plus(t1, t2,...)

ti - order terms
--#

O::_plus:= proc()
    local a, b, s, o, i, v;
begin
    b:= split([args()], O::thisDomain);
    a:= _plus(op(b[2]));
    if a = undefined then return(undefined) end_if;
    if a = complexInfinity then return(complexInfinity) end_if;
    b:= b[1];
    if nops(b) = 1 then
	s:= b;
    else
	# collect O-terms with different vars/points #
	s:= [];
	for o in b do
	    for i from 1 to nops(s) do
		v:= extop(o,2) union extop(s[i],2);
		if nops(v) = nops(map(v, op, 1)) then
		    s[i]:= O::create(_plus(extop(s[i],1), extop(o,1)), v);
		    o:= 0;
		    break;
		end_if;
	    end_for;
	    if o <> 0 then s:= append(s, o) end_if;
	end_for;
    end_if;

    if a = 0 then
	if nops(s) = 1 then
	    s[1]
	else
	    hold(_plus)(op(s))
	end_if
    elif type(a) = "_plus" then  # flat operands of a #
	hold(_plus)(op(a), op(s))
    else
	hold(_plus)(a, op(s))
    end_if
end_proc:


#--
O::_negate -- negate order term

O::_negate(t)

t - order term
--#

O::_negate:= fun(args(1)):


#--
O::_mult -- multiply order terms

O::_mult(t1, t2,...)

ti - order terms
--#

O::_mult:= proc()
    local a, b, s, v, o, i;
begin
    b:= split([args()], O::thisDomain);
    a:= _mult(op(b[2]));
    if a = undefined then return(undefined) end_if;
    if a = complexInfinity then return(complexInfinity) end_if;
    b:= b[1];
    if nops(b) = 1 then
	s:= b;
    else
	# collect O-terms with different vars/points #
	s:= [];
	for o in b do
	    for i from 1 to nops(s) do
		v:= extop(o,2) union extop(s[i],2);
		if nops(v) = nops(map(v, op, 1)) then
		    s[i]:= O::create(_mult(extop(s[i],1), extop(o,1)), v);
		    o:= 0;
		    break;
		end_if;
	    end_for;
	    if o <> 0 then s:= append(s, o) end_if;
	end_for;
    end_if;

    if domtype(a) = domtype(infinity) then 
        return((domtype(infinity))::_mult(a, op(s)))
    end_if;
    if nops(s) = 1 then
	s:= s[1];
	O::create(a * extop(s,1), extop(s,2));
    else
	# try to find matching O-term #
	v:= indets(a);
	for i from 1 to nops(s) do
	    o:= map(extop(s[i],2), op, 1);
	    if o union v = o then # o contains v #
		s[i]:= O::create(a * extop(s[i],1), extop(s[i],2));
		return(hold(_mult)(op(s)));
	    end_if;
	end_for;
	# be fatalistic #
	s[1]:= O::create(a * extop(s[1],1), extop(s[1],2));
	hold(_mult)(op(s))
    end_if
end_proc:


#--
O::intmult -- multiply order term by integer

O::intmult(t, i)

t - order term
i - integer
--#

O::intmult:= fun((if args(2)=0 then 0 else args(1) end_if)):


#--
O::_invert -- invert order term

O::_invert(t)

t - order term
--#

O::_invert:= fun(O::create(1 / extop(args(1),1), extop(args(1),2))):


#--
O::_power -- power of order term

O::_power(t, p)

t - order term
p - power
--#

O::_power:= proc(x,y) local v; begin
    if domtype(x) = O then
        if domtype(y) = O then
	    v:= extop(x,2) union extop(y,2);
	    if nops(v) = nops(map(v, op, 1)) then
		O::create(extop(x,1) ^ extop(y,1), v)
	    else
		hold(_power)(args())
	    end_if
        else
	    O::create(extop(x,1) ^ y, extop(x,2))
        end_if
    else
	O::create(x ^ extop(y,1), extop(y,2))
    end_if
end_proc:


#--
O::simplify -- simplify order term

O::simplify(o)

o - order term
--#

O::simplify:=proc(x) local p, i, j, c, di, k, v;
begin
    v:=extop(x,2);
    x:=extop(x,1);
    # removes any term in the complex-hull of the other ones #
    if nops(v)>=2 then
       p:= poly(x, map([op(v)], op, 1));
       if p <> FAIL then
	   x:= [ nthmonomial(p,i) $ i=1..nterms(p) ];
	   v:= { op(x) };
	   # the following line is needed because linopt::feasible does not
	     like indexed identifiers like c[1], c[2], ... #
	   for i from 1 to nops(x)-1 do c[i]:=genident("c") end_for;
	   for i in x do
	      p:=v minus {i};
	      if nops(p)<2 then break end_if;
	      di:=degreevec(i); p:=map(p,degreevec);
	      p:={di[j]>=_plus(c[k]*op(p,k)[j]$k=1..nops(p))
						$hold(j)=1..nops(di)};
	      j:=_plus(c[k]$k=1..nops(p));
	      p:=linopt::feasible(p union {j>=1},NonNegative);
	      if p then v:= v minus {i} end_if
	   end_for;
	   x:= _plus(op(map(v, expr)));
       end_if;
    end_if;
    new(O,x,extop(args(1),2))
end_proc:


#--
O::thisDomain -- test if argument if from domain O

O::thisDomain(x)

x - expression
--#

O::thisDomain:= fun(bool(domtype(args(1)) = O)):


#--
O::subs -- substitution

O::subs(x,eq...)
--#

O::subs:= fun(O::new(subs(extop(args(1),1), args(2..args(0))),
		     subs(op(extop(args(1),2)), args(2..args(0))))):


#--
O::diff -- differentiation

O::diff(f,x...)
--#

O::diff:= fun(O::create(diff(extop(args(1),1), args(2..args(0))),
			extop(args(1),2))):

end_proc():

# end of file #
