# #
# $Date: 1997/08/04 15:01:38 $ $Author: stefanw $ $Revision: 1.16 $ #
# #
# frankp, 18.08.1994 #

#++
Re.mu

	Re -- A function for computing the real part of
              an expression 

	For computing the real part of an expression x, 
        the method "Re" of the domain of x will be used respectivly.
	If this method does not exist, then 'Re(x)' will be
	returned.

	The basis types DOM_INT, DOM_RAT, DOM_FLOAT, DOM_COMPLEX
	and DOM_EXPR be an exception. These types are treated by
	the procedure 'Re'.
	For expressions of type DOM_EXPR the procedure 'Re' looks
	for a function attribute 'Re' respectivly, which will be 
	used for x if such exist.
	Otherwise either, if the function environment of x is one
	of the built-in environments "_plus", "_mult" or "_power", 
	x will be treated by the procedure 'Re' or the expression 
	'Re(x)' will be returned respectivly.
++#

Re := func_env(
proc(x)
    name Re;
    local t, multSplit, a, ai, i;
begin
    if (t:= x::Re) <> FAIL then return( t(args()) ) end_if;
    if args(0) <> 1 then error("wrong no of args") end_if;

    case (t:= domtype(x))
    of DOM_INT     do
    of DOM_RAT     do
    of DOM_FLOAT   do return( x )
    of DOM_COMPLEX do return( op(x,1) )
    end_case;

    if testtype( x,Type::Constant ) then
        # is x real ? #
        if domtype( float(x) ) = DOM_FLOAT then return( x )
	# is x an indexed identifier ? #
	elif type(x) = "_index" then
	    return( procname(x) )
        end_if
    end_if;

    if t = DOM_EXPR then
	t := op(x,0);
	if domtype(level(t,2)) = DOM_FUNC_ENV and
	   (a:= funcattr(level(t,2),"Re")) <> FAIL
	then
	    return( a(op(x)) )
	elif domtype(level(t,2)) = DOM_IDENT then
	# f(x) #
	    return( procname(args()) )
	end_if;
    
	multSplit := proc(z) local t,a,b,ci,cr,d;
	    begin
 	    # split product into real and imaginary parts if possible #
		cr:= 1; ci:= 0;
		for t in op(z) do
       		    a:= Re(t);
        	    if has(a,hold(Re)) then return( FAIL ) end_if;
         	    b:= Im(t);
                    if has(b,hold(Im)) then return( FAIL ) end_if;
                    d:= cr;
                    cr:= cr*a - ci*b;
                    ci:= d*b + ci*a
		end_for;
		[cr,ci]
	    end_proc:
 
	case t
	of hold(_plus) do
	    return( _plus( Re( op(x,i) ) $ hold(i)=1..nops(x) ) )
	of hold(_mult) do
	    # expressions of sign -1 must be treated separatedly #
	    #if has(x,{-1,-I}) then
		return( -Re(-x) )
	    end_if;#

	    # select constant terms #
	    t:= split( x,testtype,Type::RealNum );
            if t[1] <> 1 and t[1] <> 1.0 then 
		return( t[1]*Re(t[2]*t[3]) )
            end_if;

	    # split product into real and imaginary part if possible #
	    t := multSplit(x);
	    if t <> FAIL then return( t[1] ) end_if;

	    # test if the expression is a symbolic complex number #
            if contains( {op(x)}, I ) then
                # I*ex = (Re(ex)+Im(ex)*I)*I = Re(ex)*I-Im(ex) #
                # note: using 'contains', 1/I = -I will not be considered where
                        this rule would lead to a wrong result
                #
		return( -Im( subs(x,I=1) ) )
            else
                return( procname(x) )
            end_if
	of hold(_power) do
	    if op(x,2) = 1/2 then
		case domtype(op(x,1))
		of DOM_INT      do
		of DOM_RAT      do
		of DOM_FLOAT    do return( x )
		of DOM_COMPLEX  do
		    t := op( op(x,1) );
		    return( sqrt((op(t,1)+sqrt(op(t,1)^2+op(t,2)^2))/2) )
  		otherwise
		    if is( op(x,1), Type::RealNum)=TRUE then
			if is(op(x,1)>=0)=TRUE then
				return(sqrt(op(x,1)))
			elif is(op(x,1)>=0)=FALSE then
				return(0)
			end_if
		    end_if
		end_case
	    elif domtype(op(x,2)) = DOM_INT then
		a := Re(op(x,1));
		ai := Im(op(x,1));
		if has(a,{Re,Im}) or has(ai,{Re,Im}) then
		    return( procname(x) )
		end_if;
		t := [a,ai];
		if op(x,2) > 0 then
		    t := multSplit( [(a+I*ai) $ op(x,2)] );
		    if t = FAIL then
			return( procname(x) )
		    else
			return( t[1] )
		    end_if
		end_if;
		if op(x,2) < -1 then
		    t := multSplit( [(a+I*ai) $ -op(x,2)] );
		    if t = FAIL then return( procname(x) ) end_if
		end_if;
		# case op(x,2) = -1 #
		return( t[1]/(t[1]^2+t[2]^2) )
	    elif testtype( op(x,2),NUMERIC ) then
		t := Re( hold(exp)(op(x,2)*ln(op(x,1))) );
		if type(t) <> "Re" then return( t ) end_if
	    end_if
	end_case
    end_if;

    # uses info from assume #
    if is(x,Type::RealNum)=TRUE then return(x) end_if;
    procname(x)
end_proc,
NIL,
table( "type"="Re", "print"="Re", "Re"=fun(hold(Re)(args(1))), "Im"=0,
       "info"="Re(x) -- the real part of x" )
):

Re:= funcattr(Re,"rectform",
    loadproc(funcattr(Re,"rectform"),pathname("STDLIB","RECTFORM"),"Re")
):

# end of file #
