# #
# $Date: 1997/10/16 10:53:45 $ $Author: olli $ $Revision: 1.18 $ #
# #
# frankp, 25.11.1994 #

#++
  rectform.mu

    rectform -- A function for computing the rectangular form of 
		complex expressions

    rectform( x [,rl] )

    x  -- expression
    rl -- (optional) set of unknown variables


    rectform(x) tries to split x into its real and imaginary
    parts and return x - if possible - in the form x_r + I*x_i.
    If this is not possible then rectform(x) tries to find for each
    subexpression in x such a complex canonical form.

    The result of rectform is an element of the domain 'rectform'.
    Such an element consists of four operands. The first two operands
    give the real and imaginary parts of the expression. The third
    operand represents all subexpressions of x for which such a
    rectangular form can not be computed (possible 0 if there are
    not such subexpressions). The fourth operand will be described later.

    The functions 'Re' and 'Im' are overloaded. If the third operand
    is not 0 then 'Re' returns the first operand and 'Im' the
    second operand of an element of type rectform.
    Otherwise the new expressions which results by 'Re' and 'Im' 
    will be computed to their rectangular form.

    rectform maps onto sets, lists and arrays, and also onto polynomials
    and series, where rectform maps onto the coefficients.

    Arithmetical operations between ordinary expressions and elements
    of type rectform are possible, whereby the results will be of
    type 'rectform'.

    The MuPAD function 'expr' is overloaded for arguments of type
    'rectform'. If x is of type 'rectform' then 'expr(x)' will return
    the expression  extop(x,1)+I*extop(x,2)+extop(x,3), i.e. an
    expression of type DOM_EXPR.

    The general assumption of rectform is that unkown variables
    represent complex-valued quantities.
    But there are two possibilities to change this assumption:
    First -- let us call this the 'global' method --
    with the function call \Mex{rectform::globalReal(S)} all unknown
    variables in a set \Mup{S} will represent real variables.
    For all subsequent operations, e.g. arithmetical
    operations, rectform will test if a corresponding unknown
    variable can be found in S and treated this as real if true.
    The function call \Mex{rectform::globalReal(S)} returns either
    the empty set or a set of unknowns that were defined as real
    valued before.
    The second method -- let us call this the 'local' method -- 
    is to state a set rl of unknown variables by calling rectform
    which elements should represent real-valued quantities. 
    Then for this expression returned by the call of rectform
    the variables in rl will ever be treated as real.
    This set is part of any element of type 'rectform' and it is
    stored in the fourth operand of such an element.
        The fourth operand of an element x of rectform can be
    changed by calling rectform with x by stating rl. Then
    the expression will be recalculated in respect to the new variables
    in rl defined to be real. If rectform is called with x without the 
    second argument rl, then rectform will return x without changes.
	The method 'real' returns the fourth operand of an rectform
    element x.
    
    The main difference of the global method and the local method is
    that the global method will not change the internal representation
    of an element of type rectform, but will influence all further 
    operations, where this element is be a part of.

    For arithmetical operations with different sets of unknown variables
    set to be real, the result will be computed for the intersection of
    both sets (see examples). Thus the local method may not influence
    futher operations.

    For expressions, for which the 0-th operand is of type DOM_FUNC_ENV,
    rectform will used the function attribute "rectform" by calling
    the attribute with the operands of x, if it exist.

    The following example shows how the function attribute "rectform"
    should be defined:

    sin := funcattr( sin, "rectform", proc(x)
        local a, b, rl;
    begin
        if args(0) = 2 then
            x := rectform::new(x,args(2))
        else
            x := rectform::new(x)
        end_if;
        rl := extop(x,4);
        if extop(x,3) <> 0 then
            new(rectform,0,0,sin(x),rl)
        else
            a := extop(x,1);
            b := extop(x,2);
            if domtype(a+I*b) = DOM_COMPLEX
            and domtype(a) = DOM_FLOAT or domtype(b) = DOM_FLOAT
            then
                rectform::new(sin(a+I*b))
            else
                new(rectform,sin(a)*cosh(b),cos(a)*sinh(b),0,rl)
            end_if
        end_if
    end_proc):

    rectform treates expressions of type "_plus", "_mult" and "_power"
    and system functions explicitly.


    Examples:

    >> rectform( tan(x) );

                sin(2 Re(x))                 I sinh(2 Im(x))       
        ---------------------------- + ----------------------------
        cos(2 Re(x)) + cosh(2 Im(x))   cos(2 Re(x)) + cosh(2 Im(x))

    >> Im(%);

                              sinh(2 Im(x))        
                       ----------------------------
                       cos(2 Re(x)) + cosh(2 Im(x))

    >> a := rectform(x);

                              Re(x) + I Im(x)

    >> b := rectform(y,{y});

                                    y

    >> c := a+b;

                    (Re(x) + Re(y)) + I (Im(x) + Im(y))

    >> rectform::globalReal({x,y}):
    >> a, b;

                           Re(x) + I Im(x), y 

    >> c;                    

                    (Re(x) + Re(y)) + I (Im(x) + Im(y)) 

    >> rectform(c);

                                  x + y 

    >> b::real(b);
                                   {y}
++#

#--
        Methods for rectform:

	new(z)		expand(z)	simplify(z)   
	_plus()		_mult()		iszero(z)    
	expr(z)		length(z)	testtype(z,T)
	Re(z)		Im(z)		conjugate(z) 
	convert(z)	diff()		intmult()
	globalReal	info 		abs(z)
	real(z)
--#

rectform:= domain("rectform"):

rectform::name:= "rectform":

rectform::globalReal:= proc(S)
    local oldS;
begin
    if args(0) = 0 then rectform::globalRealSet
    elif not testtype(S,Type::SetOf(DOM_IDENT)) then
        error("expecting set of identifiers")
    else
        oldS:= rectform::globalRealSet;
        sysassign( rectform::globalRealSet,S );
        oldS
    end_if
end_proc:

rectform::globalRealSet:= {}:

#--
	the method info
--#
domattr(rectform,"info") := 
"rectform(x) -- splits x into real and imaginary part [try ?rectform for options]":

#--
	the method expr
--#
domattr(rectform,"expr") := fun(
    (if domtype(args(1)) <> rectform then args(1)
    else
	_plus( extop(args(1),1),extop(args(1),2)*I,extop(args(1),3) )
    end_if)
):

#--
	the method float
--#
domattr(rectform,"float") := float@expr:

#--
	the method real
--#
domattr(rectform,"real") := fun( extop(args(1),4) ):

#--
	pI  --  be used for ordinary output of a complex expression
--#
domattr(rectform,"pI") := new(rectform,I,0,0,{}):

#--
	the method print
--#
domattr(rectform,"print") := fun(
	(if extop(args(1),1) = 0 then
	    if extop(args(1),2) = 0 then extop(args(1),3)
	    elif extop(args(1),3) = 0 then 
		hold(_mult)(extop(args(1),2),rectform::pI)
	    else hold(_plus)(
		hold(_mult)(extop(args(1),2),rectform::pI),extop(args(1),3)
	    )
	    end_if
	elif extop(args(1),2) = 0 then 
	    if extop(args(1),3) = 0 then extop(args(1),1)
	    else hold(_plus)(extop(args(1),1),extop(args(1),3))
	    end_if
	elif extop(args(1),3) = 0 then
	    hold(_plus)(extop(args(1),1),
        	hold(_mult)(extop(args(1),2),rectform::pI)
	    )
	else hold(_plus)(
            extop(args(1),1),hold(_mult)(extop(args(1),2),rectform::pI),
	    extop(args(1),3)
        )
	end_if)
):

#--
	the function iszero 
--#
domattr(rectform,"iszero") := fun(
    bool(
	iszero(extop(args(1),1)) and iszero(extop(args(1),2))
	and iszero(extop(args(1),3))
    )
):

#--
	the function expand
--#
domattr(rectform,"expand") := fun(
    (rectform::new( args(1) );
    rectform::new( 
        expand(rectform::expr(%),args(2..args(0))),extop(%,4))
    )
):

#--
        the function expand
--#
domattr(rectform,"subs") := fun(
    (if args(0) = 1 then return( args(1) )
    else rectform::new(
        subs(rectform::expr(args(1)),args(2..args(0))),extop(args(1),4)
    )
    end_if)
):

domattr(rectform,"subsex") := fun(
    (if args(0) = 1 then return( args(1) )
    else rectform::new(
        subsex(rectform::expr(args(1)),args(2..args(0))),extop(args(1),4)
    )
    end_if)
):

#--
	the function simplify
--#
domattr(rectform,"simplify") := fun(
    (rectform::new( args(1) );
    rectform::new( 
	simplify(rectform::expr(%),args(2..args(0))),extop(%,4))
    )
):

#--
        the function rewrite
--#
domattr(rectform,"rewrite") := fun(
    (rectform::new( args(1) );
    rectform::new(
        rewrite(rectform::expr(%),args(2..args(0))),extop(%,4))
    )
):

#--
	the function normal
--#
domattr(rectform,"normal") := fun(
    (rectform::new( args(1) );
    rectform::new( normal(rectform::expr(%)),extop(%,4) ))
):

#--
	the function diff
--#
domattr(rectform,"diff") := fun(
    (rectform::new( args(1) );
    if args(0) > 1 then
    rectform::new(
	diff(rectform::expr(%),args(2..args(0)))
     ,extop(%,4))
    else 
	%
    end_if) 
):

#--
	the method intmult (for the derivative of poly's)
--#
domattr(rectform,"intmult") := fun( rectform::_mult(args(1),args(2)) ):

#--
	the function testtype
--#
domattr(rectform,"testtype") := fun(
    (if args(2) = rectform or domtype(args(2)) = rectform then TRUE
    else testtype(rectform::expr(args(1)),args(2))
    end_if)
):

#--
	the method convert
--#
domattr(rectform,"convert") := fun(rectform::new(args(1),extop(args(1),4))):

#--
	the method convert_to
--#
domattr(rectform,"convert_to") := fun(
    (if domtype(args(1)) = rectform then
	domattr(args(2),"convert")(rectform::expr(args(1)))
    else
	FAIL
    end_if)
):

#--
	the function _plus 
--#
domattr(rectform,"_plus"):= loadproc(
    domattr(rectform,"_plus"),pathname("STDLIB","RECTFORM"),"_plus"
):

#--
        the function _mult
--#
domattr(rectform,"_mult") := loadproc(
    domattr(rectform,"_mult"),pathname("STDLIB","RECTFORM"),"_mult"
):

#--
        the function _power
--#
domattr(rectform,"_power") := loadproc(
    domattr(rectform,"_power"),pathname("STDLIB","RECTFORM"),"_power"
):

#--
	the function length
--#
domattr(rectform,"length") := fun(
    length(rectform::expr( rectform::new(args(1)) ))
):

#--
	the function op
--#
domattr(rectform,"op") := fun(
    (if args(0) = 1 then
        _exprseq(
    extop(args(1),1),extop(args(1),2),extop(args(1),3),extop(args(1),4)
        )
    elif type(args(2)) = "_range" then
	rectform::op( args(1),j ) $ hold(j)=args(2);
	if contains( {%},FAIL ) then FAIL else % end_if
    elif args(2) > 4 then FAIL
    else extop(args(1),args(2))
    end_if)
):

#--
	the function has
--#
domattr(rectform,"has") := fun(
    (rectform::new( args(1) );
    has( rectform::expr(%),args(2) ))
):

#--
	the function Re
--#
domattr(rectform,"Re") := fun(
    (rectform::new( args(1) );
    if extop(%,3) = 0 then
	new(rectform,extop(%,1),0,0,extop(%,4))
    else
	new(rectform,0,0,Re(rectform::expr(%)),extop(%,4))
    end_if)
):

#--
	the function Im
--#
domattr(rectform,"Im") := fun(
    (rectform::new( args(1) );
    if extop(%,3) = 0 then
	new(rectform,extop(%,2),0,0,extop(%,4))
    else
	new(rectform,0,0,Im(rectform::expr(%)),extop(%,4))
    end_if)
):

#--
	the function conjugate
--#
domattr(rectform,"conjugate") := fun(
    (rectform::new( args(1) );
    if extop(%,3) = 0 then
	new(rectform,extop(%,1),-extop(%,2),0,extop(%,4))
    else
	new(rectform,0,0,conjugate(rectform::expr(%)),extop(%,4))
    end_if)
):

#--
	the method new
--#
domattr(rectform,"new") := proc(x) 
    local t, a, rl;
begin
    if testargs() then
        if args(0) < 1 or args(0) > 2 then error("wrong no of args") end_if;
	if args(0) = 2 then
	    if domtype(args(2)) <> DOM_SET then
		error("expecting a set of indents")
	    end_if
	end_if
    end_if;
    if args(0) = 2 then rl := args(2) else rl := null() end_if;
    if x::rectform <> FAIL then return( x::rectform(args()) ) end_if;

    t := domtype(x);
    case t
    of rectform    do if args(0) = 1 then rl := extop(x,4) end_if;
		      return( rectform::new(rectform::expr(x),rl) )
    of DOM_IDENT   do if domtype(rl) = DOM_NULL then rl := {} end_if;
		      if has(x,rectform::globalRealSet) or has(x,rl) then
		          return( new(rectform,x,0,0,rl) )
		      else
		          return( new(rectform,Re(x),Im(x),0,rl) )
		      end_if
    of DOM_INT     do 
    of DOM_RAT     do
    of DOM_FLOAT   do if domtype(rl) = DOM_NULL then rl := {} end_if;
		      return( new(rectform,x,0,0,rl) )
    of DOM_COMPLEX do if domtype(rl) = DOM_NULL then rl := {} end_if;
		      return( new(rectform,op(x,1),op(x,2),0,rl) )
    of DOM_LIST    do
    of DOM_SET     do
    of DOM_ARRAY   do return( map(x,rectform::new,rl) )
    of DOM_EXEC    do
    of DOM_PROC    do if domtype(rl) = DOM_NULL then rl := {} end_if;
		      return( new(rectform,0,0,x,rl) )
    of DOM_POLY    do return( mapcoeffs(x,rectform::new,rl) )
    of Puiseux     do return( x::map(x,rectform::new,rl) )
    of DOM_EXPR    do
	    t := op(x,0);
	    if domtype(level(t,2)) = DOM_FUNC_ENV then
	        a := funcattr(level(t,2),"rectform");
		if a <> FAIL then
		    return( a(op(x),rl) )
	        end_if
	    end_if;
	    if t = hold(_plus) or t = hold(_mult) then
		return( map(x,rectform::new,rl) )
	    elif t = hold(_power) then
		return( rectform::_power(
		    rectform::new(op(x,1),rl),rectform::new(op(x,2),rl)
		) )
	    end_if
    end_case;

    if domtype(rl) = DOM_NULL then rl := {} end_if;
    new(rectform,0,0,x,rl)
end_proc:

# define some methods to avoid creation via domattr #
rectform::evaluate:= FAIL:
rectform::posteval:= FAIL:
rectform::domtype:= FAIL:
rectform::type:= FAIL:
rectform::elemattr:= FAIL:
rectform::rectform:= FAIL:

#--
args2expr -- map args which are 'rectform's to expressions
--#

rectform::args2expr:= fun((
    if args(0) = 1 then
        expr(args(1))
    else
	(if domtype(args(i)) = rectform then rectform::expr(args(i))
	 else args(i) end_if) $ hold(i)=1..args(0)
    end_if
)):

#--
domattr -- create default method

domattr creates a function from the method index by converting the
index to an expression; arguments which are 'rectform's are converted
to expressions before the function is called. 

NOTE: This MUST be the last method defined for rectform.
--#

rectform::domattr:= fun((
    if domtype(args(2))=DOM_STRING then
        text2expr(args(2));
        # (eval(%) @ rectform::args2expr); #
        (subs(%,Simplified) @ rectform::args2expr);
    else
	FAIL
    end_if
)):

null():

# end of file #
