# $Date: 1997/04/22 14:08:59 $ $Author: cube $ $Revision: 1.2 $ #
# #
# stefanw #

#++
lagrange.mu

	lagrange - lagrange interpolation of data 
			
	lagrange(pointlist,valuelist,x <,F>)

	pointlist - list of evaluation points
	valuelist - list of the values at the evaluation points
	x	  - variable 
	F	  - (optional) coefficient ring

Returns the polynomial of lowest degree which interpolates the given data.

Note that if F is not a field, the interpolation may be impossible.
F::convert must be able to handle the operands of pointlist and valuelist.
++#

lagrange:=proc()
local i,m,n,t,k,a,pointlist,valuelist,x,xp,yp,F,result;

begin
if testargs() then
	if args(0)<3 or args(0)>5 then
		 error("wrong number of arguments");
	elif {domtype(args(1)),domtype(args(2))}<>{DOM_LIST} then
		error("First and second argument must be lists");
        elif nops(args(1)) = 2 and 
             {domtype(op(args(1),1)),domtype(op(args(1),2))}={DOM_LIST} then
             xp := op(args(1),1); yp := op(args(1),2);
             n := nops(xp); m := nops(yp);
             if n<>nops({op(xp)}) or m<>nops({op(yp)}) then
                error("Evaluation points should be distinct");
             elif not testtype(args(2), Type::ListOf(DOM_LIST,n,n)) then
                error("List of ".expr2text(n)." lists of values expected");
             elif _and(testtype(xp[i],Type::ListOf(Type::AnyType,m,m))$i=1..n) then
                error("Lists with ".expr2text(m)." values expected");
             elif domtype(args(3))<>DOM_LIST or nops(args(3))<>2 or
                  {domtype(op(args(3),1)),domtype(op(args(3),2))}<>{DOM_IDENT} then
                error("Third argument must be a list of 2 variables");
             end_if
	elif nops(args(1))<>nops(args(2)) then
		error("Number of points does not equal number of values");
	elif nops(args(1)) <> nops({op(args(1))}) then
		error("Evaluation points should be distinct");
	elif domtype(args(3))<> DOM_IDENT then
		error("Third argument must be variable");
        end_if;
	if args(0)=4 then
		if args(4)=hold(Expr) then
			# ok, do nothing #	
		elif (args(4))::hasProp(Cat::Ring) <> TRUE then
			error("Fourth argument must be a Cat::Ring");
	        end_if
	end_if
end_if;			
pointlist:=args(1);
valuelist:=args(2);
x:=args(3);
if args(0)=3 then 
	F:=hold(Expr);
else
	F:=args(4);
end_if;

if domtype(pointlist[1]) = DOM_LIST then
   return(lagrange2d(pointlist, valuelist, x, F))
end_if;

n:=nops(pointlist)-1;

a:=array(0..n);
t:=array(0..n);
for i from 0 to n do
	t[i]:=valuelist[i+1];
	k:=i-1;
	while k>=0 do
		t[k]:=(t[k+1]-t[k])/(pointlist[i+1]-pointlist[k+1]);
		k:=k-1;
	end_while;
	a[i]:=t[0];
end_for;


result:=poly(a[n],[x],F);
k:=n-1;
while k>=0 do
	result:=result*poly([[1,1],[-pointlist[k+1],0]],[x],F)+poly(a[k],[x],F);
	k:=k-1;
end_while;
result
end_proc:

#--

Lagrange interpolation (2 dimensional)

    Input:

    * m by n rectangular grid

    * x[i] <> x[j] ; i,j = 1,...,m ; i<>j ;

    * y[i] <> y[j] ; i,j = 1,...,n ; i<>j ;

    * z[i][j] = f(x[i], y[j]) ; i = 1,...,m , j = 1,...,n

    Output:

                m   n
               --- ---
    P(x, y) =  \   \    X(x,i) * Y(y,j) * z[i][j]
               /   /
               --- ---
               i=1 j=1


               m                            n
              ---    x  - x[k]             ---    y  - y[k]
    X(x,i) =  | |  -----------   Y(y,j) =  | |  -----------
              | |  x[i] - x[k]             | |  y[j] - y[k]
              k=1                          k=1
              k<>i                         k<>j


    degree(P, x) < m , degree(P, y) < n

    >> lagrange2d([[1,2,4],[3,5]], [[5,10],[24,-3],[6,5]], [x,y], Expr);


         /                              2       2                   \
         | 325 x   67 y   77 x y   191 x    15 x  y                 |
     poly| ----- + ---- - ------ - ------ + ------- - 799/6, [x, y] |
         \   2      2       2        6         2                    /

--#

lagrange2d := proc(pointlist, valuelist, vars, F)
   local i, j, k, m, n, x, xp, y, yp, zp;
begin
   xp := pointlist[1]; yp := pointlist[2]; zp := valuelist;
   m := nops(xp); n := nops(yp); x := vars[1]; y := vars[2];
   poly(_plus(
          _plus(
            _mult((x-xp[k])/(xp[i]-xp[k])$k=1..i-1) *
            _mult((x-xp[k])/(xp[i]-xp[k])$k=i+1..m) *
            _mult((y-yp[k])/(yp[j]-yp[k])$k=1..j-1) *
            _mult((y-yp[k])/(yp[j]-yp[k])$k=j+1..n) *
            zp[i][j] $ j = 1 .. n
         ) $ i = 1 .. m
       ),
       [x,y], F
   )
end_proc:

# end of file #
