# $Date: 1999/04/30 12:14:04 $  $Author: stefanw $  $Revision: 1.29.2.1 $ #

#++
loadproc -- load an object from file on demand

loadproc(p, l, f [,c [,d,...]])

p - an expression
l - a path name
f - a file name without suffix
c - a function (optional)
d - data (optional)

loadproc on demand reads in the file f containing the definition of
the object p. On demand means: the file is read when the result
of the loadproc-call is evaluated for the first time. The argument
p is not evaluated.

After the result of the loadproc-call is evaluated and the file is
read p is evaluated and returned instead of the result of the
loadproc-call.

The typical use is loading a procedure p on demand as in
    p:= loadproc(p, "LIB/", "p");
Here the definition of p is read when p is evaluated for the first
time.

The file name f must not contain a suffix. First the binary file
	l."BIN/".f.".mb"
is searched for. If this binary file doesn't exist the t file
	l.f.".mu"
is searched for.

Note that the varaible LIB_PATH is used to locate the file.

If the additional options c and d,... are given, the expression 'c(d,...)'
is evaluated after p is read from file, but before p is executed. The
callback function c may be used to change p in a certain way after it is
read from file.
++#

proc()
    option noDebug;
    local dom;
begin
 
loadproc := proc()
    option noDebug, hold;
begin
    if testargs() then
	if args(0) < 3 then
	    error("wrong no of arguments")
	end_if;

	case domtype(args(1))
	of DOM_IDENT do
	of DOM_EXPR do
	    break;
	otherwise
	    error("wrong procedure name")
	end_case;
    end_if;

    (stdlib::LoadProc)::new( args(1), 
        op(context([args(2..args(0))])))
end_proc:

# LoadProc -- domain to load library functions on demand #

dom:= domain("stdlib::LoadProc"):

dom::new := proc()
    option noDebug;
    name loadproc;
begin
    new(stdlib::LoadProc, args())
end_proc:

dom::read_file:= proc(path, fnam)
    option noDebug;
    name loadproc;
    local p, f, at, af, pref_val, PRETTY_PRINT;
begin
    pref_val := Pref::_verboseRead;
    if pref_val <> 0 then
	PRETTY_PRINT:= FALSE
    end_if;
    for p in LIB_PATH do
	# read file, try mb-file first, then mu-file #
	f:= fopen(p.path.stdlib::BinPath.fnam.".mb");
	if f <> FAIL then
            case pref_val
	    of 0 do
                break
            of 1 do
            	if strmatch(path,"\*LIBFILES\*") then
                    print(Unquoted,"loading package '".fnam."' from ".p);
            	end_if;
		break
            of 2 do 
		if strmatch(path,"\*LIBFILES\*") then
                    print(Unquoted,"loading package '".fnam."' [".p."]");
		else
                    print(Unquoted,"reading file ".p.path.stdlib::BinPath.fnam.".mb");
		end_if
            end_case;

            stdlib::syseval(fread(f));
	    fclose(f);
	    break;
	end_if;

	f:= fopen(p.path.fnam.".mu");
	if f = FAIL then next end_if;
	case pref_val
	of 0 do 
	    break
	of 1 do
	    if strmatch(path,"\*LIBFILES\*") then
                print(Unquoted,"loading package '".fnam."' [".p."]");
            end_if;
	    break
	of 2 do
	    if strmatch(path,"\*LIBFILES\*") then
                print(Unquoted,"loading package '".fnam."' [".p."]");
	    else
	        print(Unquoted,"reading file ".p.path.fnam.".mu");
	    end_if
	end_case;
    
	# un-install actual aliases #
	at:= stdlib::Alias;
	af:= (if domtype(_dom_alias_element_) = DOM_IDENT then FAIL
	      else _dom_alias_element_ end_if);
	sysassign(stdlib::Alias, FAIL);
	sysassign(_dom_alias_element_, NIL);

        stdlib::syseval(fread(f));
	fclose(f);
    
	# re-install actual aliases #
	sysassign(stdlib::Alias, at);
	sysassign(_dom_alias_element_,
	      (if af = FAIL then NIL else af end_if));
	break
    end_for;

    if f = FAIL then
        error("can't read file '".path.fnam.".mu'")
    end_if;
end_proc:



dom::evaluate := proc()
    option noDebug;
    name loadproc;
begin
    (stdlib::LoadProc)::read_file(op(args(1),2), op(args(1),3));
    if nops(args(1)) > 3 then
    #~ call float attribute ~#
	op(args(1),4)(op(args(1), 5..nops(args(1))))
    end_if;
    context(op(args(1),1))
end_proc:

dom::func_call := proc()
    option noDebug;
    name loadproc;
begin
   
    (stdlib::LoadProc)::read_file(op(args(1),2), op(args(1),3));
    if nops(args(1)) > 3 then
    #~ call float attribute ~#
      op(args(1),4)(op(args(1), 5..nops(args(1))))
    end_if;
   
    context(op(args(1),1)(args(2..args(0))))
end_proc:

dom::set_func_call := proc()
    option noDebug;
    name loadproc;

begin

    (stdlib::LoadProc)::read_file(op(args(1),2), op(args(1),3));
    if nops(args(1)) > 3 then
    #~ call float attribute ~#
      op(args(1),4)(op(args(1), 5..nops(args(1))))
    end_if;
    context(op(args(1),1))    
end_proc:

dom::posteval := proc()
    option noDebug;
    name loadproc;
begin

    (stdlib::LoadProc)::read_file(op(args(1),2), op(args(1),3));
    if nops(args(1)) > 3 then
    #~ call float attribute ~#
	op(args(1),4)(op(args(1), 5..nops(args(1))))
    end_if;

    context(op(args(1),1));
end_proc:

sysassign(stdlib::LoadProc, dom):
null():
end_proc():


# end of file #





