/*

Replacement of plotfunc for 2-dimensional function plots

New scene options:
ViewingBox: List of 4 real values
GrideLines: On, Off or a list of 2 integers > 0

Changed scene options:
Ticks: List of 2 real values > 0 : Ticks = [a, b]
       Tickmarks are drawn and these ticks are a multiple 
       of a in the range XMin..XMax resp. a multiple 
       of b within the range YMin..YMax

Remarks:
You can specify plot2d objects like 
plotfunc2d([Mode=List,[point(2,2)]],sin(x), x=-PI..PI): 
Note, that the option ViewingBox is not yet implemented 
for points and polygons.

       
Examples:

plotfunc2d(x^3, x=-3..3):
plotfunc2d({Axes=Corner, Ticks=[0.5, 10]}, x^3, x=-3..3):
plotfunc2d(x^3, x=-2..2, YRange=-4..4):
plotfunc2d({GridLines=On}, x^3, x=-2..2, YRange=-5..5):
plotfunc2d({Ticks=[1,2],GridLines=[2,1]}, x^3, x=-2..2, YRange=-10..10):
plotfunc2d({ViewingBox=[-2,2,-5,5]}, x^3, x=-4..4, YRange=-10..10):
plotfunc2d({ViewingBox=[-3,3,-6,6]}, x^3, x=-2..2, YRange=-5..5):

Define default values for plotting:

plotfunc2d_options := { Axes=Corner, Labels=["x","y"],
                        ViewingBox=[-4,4,-4,4], GridLines=On}:

Now plotfunc2d uses the options defined in plotfunc2d_options

plotfunc2d(x^3, x=-3..3):

Undefine default options:

unassign(plotfunc2d_options):

*/

plotfunc2d := proc()
   local arg, axes, draw_gridlines, err, gl, 
         gridcolor, gridlines, gridstyle, 
         new_args, origin, plot2d_objects, s, 
         scene_options, show_gridlines, sys_plot, ticks, 
         Ticks_x_delta, Ticks_x_max, Ticks_x_min, 
         Ticks_x_step, Ticks_y, Ticks_y_delta, 
         Ticks_y_max, Ticks_y_min, Ticks_y_step, 
         x_max, x_min, x_var, y_max, y_min, vbox;
   // Global variable: plotfunc2d_options - do not protect it.
begin

   // Definition of a local procedure

   draw_gridlines := proc(xmin, xmax, ymin, ymax, xgrid, ygrid)
      local xminf,xmaxf,yminf,ymaxf,rx,ry,a,
            xList,yList,ix,iy,ixmax,iymax,
            xLines,yLines;
      // Uses global variables axes, gridcolor, gridstyle
      // and origin defined in the procedure plotfunc2d 
   begin 
      xminf:=float(xmin): xmaxf:=float(xmax):
      yminf:=float(ymin): ymaxf:=float(ymax): 
      ixmax:=ceil((xmaxf-xminf)/xgrid): iymax:=ceil((ymaxf-yminf)/ygrid):

      if axes <> Origin then
         xList:=[xminf+ix*xgrid $ ix=0..ixmax]:
         yList:=[yminf+iy*ygrid $ iy=0..iymax]:
         if axes = Box then
            rx := 2..ixmax; ry := 2..iymax;
         elif axes = Corner then
            rx := 2..ixmax+1; ry := 2..iymax+1;
         else // None
            rx := 1..ixmax+1; ry := 1..iymax+1;
         end_if;
      else // Origin 
         a := (origin[1]-xminf)/xgrid;
         if frac(a) = 0.0 and float(origin[1]) <> xminf 
            and float(origin[1]) <> xmaxf then
            a := trunc(a);
            xList:=[xminf+ix*xgrid $ ix=0..a-1,
                     xminf+ix*xgrid $ ix=a+1..ixmax];
            ixmax := ixmax-1;  
         else
            xList:=[xminf+ix*xgrid $ ix=0..ixmax]:
         end_if;
         if float(origin[1]) = xminf then
            rx := 2..ixmax+1;
         elif float(origin[1]) = xmaxf then
            rx := 1..ixmax;
         else
            rx := 1..ixmax+1;
         end_if;
         a := (origin[2]-yminf)/ygrid;
         if frac(a) = 0.0 and float(origin[2]) <> yminf 
            and float(origin[2]) <> ymaxf then
            a := trunc(a);
            yList:=[yminf+iy*ygrid $ iy=0..a-1,
                     yminf+iy*ygrid $ iy=a+1..iymax];
            iymax := iymax-1;  
         else
            yList:=[yminf+iy*ygrid $ iy=0..iymax]:
         end_if;
         if float(origin[2]) = yminf then
            ry := 2..iymax+1;
         elif float(origin[2]) = ymaxf then
            ry := 1..iymax;
         else
            ry := 1..iymax+1;
         end_if;
      end_if;

      xLines:=polygon(point(xList[1],yList[iy]),
                      point(xList[ixmax+1],yList[iy])) $ iy=ry:
      yLines:=polygon(point(xList[ix],yList[1]),
                      point(xList[ix],yList[iymax+1])) $ ix=rx:
      return( [Mode=List, [xLines,yLines],
               LineStyle=gridstyle, Color = [Flat, gridcolor] ] );
   end_proc:


   // Initializiation: Default values (change it if you want)
   axes           := Origin; // Axes=Origin (Posible values: None, Box, Corner)     
   origin         := [0, 0]; // AxesOrigin=[0,0] (real values)           
   Ticks_y        := 5;      // Number of ticks, if length of y-axes is unknown     
   Ticks_y_min    := YMin;   // min(f(x)|x=xmin..xmax) 
   Ticks_y_max    := YMax;   // min(f(x)|x=xmin..xmax) 
   Ticks_x_delta  := 1;      // x-axes: ticks at ...,-1,0,1,...  (real value > 0)   
   Ticks_y_delta  := 1;      // y-axes: ticks at ...,-1,0,1,...  (real value > 0)
   Ticks_x_step   := 1;      // integer value > 0
   Ticks_y_step   := 1;      // integer value > 0
   show_gridlines := FALSE;  // GridLines=Off, TRUE corresponds to On
   gridcolor      := RGB::Gray;   // Color of grid lines
   gridstyle      := DashedLines; // Style of grid lines

   // Do not change the following initialization
   ticks          := FALSE;
   new_args       := null();
   plot2d_objects := null();
   gridlines      := {};
   scene_options  := {};
   

   for arg in args() do

       if type(arg) = DOM_SET then
          scene_options := scene_options union arg;

       elif type(arg) = DOM_LIST then
          plot2d_objects := plot2d_objects, arg;

       elif type(arg) = "_equal" then

          case op(arg, 1) 
           of YRange do
              if type(op(arg, 2)) <> "_range" then
                 error("YRange: Range expected.");
              end_if;
              y_min := float(op(arg,[2,1]));
              y_max := float(op(arg,[2,2]));
              if not (testtype(y_min, Type::RealNum) and 
                      testtype(y_max, Type::RealNum)) then
                 error("YRange: Numerical values expected.")
              elif y_min >= y_max then
                 error("YRange: a..b with a < b expected.");
              end_if;
              break;
           otherwise
              if type(op(arg, 2)) <> "_range" then
                 error("Illegal option or illegal x-range.");
              end_if;
              x_var := op(arg, 1);
              x_min := float(op(arg,[2,1]));
              x_max := float(op(arg,[2,2]));
              if type(x_var) <> DOM_IDENT then
                 error("x-range: Variable expected.");
              end_if;
              if not (testtype(x_min, Type::RealNum) and 
                      testtype(x_max, Type::RealNum)) then
                 error("x-range: Numerical values expected.")
              elif x_min >= x_max then
                 error("x-range: a..b with a < b expected.");
              end_if;
        end_case;
     else
        new_args := new_args, arg;
     end_if;

   end_for;

   if type(plotfunc2d_options) = DOM_SET then 
      for arg in plotfunc2d_options do
          if type(arg) <> "_equal" then
             error("Illegal option in set plotfunc2d_options.");
          end_if;
          if not has(scene_options, op(arg, 1)) then
             scene_options := scene_options union {arg}
          end_if
      end_for;
   elif type(plotfunc2d_options) <> DOM_IDENT then
      error("plotfunc2d_options: Set expected.");
   end_if;

   s := {};
   for arg in scene_options do
       case op(arg, 1)          
         of ViewingBox do
            vbox := float(op(arg, 2));
            if not testtype(vbox, Type::ListOf(Type::RealNum, 4, 4)) or
               vbox[1] >= vbox[2] or vbox[3] >= vbox[4] then
               s := "ViewingBox: List of 4 numerical values [a,b,c,d]\n";
               s := s."         with a < b and c < d expected. ";
               error(s)
            end_if;
            Ticks_x_min := vbox[1]; Ticks_x_max := vbox[2];
            Ticks_y_min := vbox[3]; Ticks_y_max := vbox[4];
            break;
         of Ticks do
            ticks := float(op(arg, 2));
            if not testtype(ticks, Type::ListOf(Type::RealNum, 2, 2)) or
               ticks[1] <= 0 or ticks[2] <= 0 then
               error("Ticks: List of 2 numerical values > 0 expected.");
            end_if;
            Ticks_x_delta := ticks[1]; Ticks_y_delta := ticks[2];
            ticks := TRUE;
            break;
         of GridLines do
            gl := op(arg, 2);
            if gl = Off then
               show_gridlines := FALSE;
            elif gl = On then
               show_gridlines := TRUE;
            elif testtype(gl, Type::ListOf(DOM_INT, 2, 2)) and
                 gl[1] > 0 and gl[2] > 0 then
               Ticks_x_step  := gl[1];
               Ticks_y_step  := gl[2];
               show_gridlines := TRUE;
            else
               error("Illegal value for GridLines.");
            end_if;
            break;
         of Axes do
            axes := op(arg, 2); 
            s := s union {arg};
            break;
         of AxesOrigin do
            origin := float(op(arg, 2));
            if type(origin) <> DOM_LIST or nops(origin) <> 2 or
               (origin[1] <> XMin and origin[1] <> XMax and 
                not testtype(origin[1], Type::RealNum)) or
               (origin[2] <> YMin and origin[2] <> YMax and
                not testtype(origin[2], Type::RealNum)) then
               error("AxesOrigin: List of 2 values expected.");
            end_if;
            break;
         otherwise
            s := s union {arg};
       end_case;
   end_for;
   scene_options := s;

   if type(Ticks_x_min) = DOM_IDENT then
      Ticks_x_min := x_min; Ticks_x_max := x_max;
   else
      x_min := max(x_min, Ticks_x_min);
      x_max := min(x_max, Ticks_x_max);
   end_if;
   new_args := new_args, x_var = x_min..x_max;

   if type(y_min) <> DOM_IDENT then
      if type(Ticks_y_min) = DOM_IDENT then
         Ticks_y_min := y_min; Ticks_y_max := y_max;
      else
         y_min := max(y_min, Ticks_y_min);
         y_max := min(y_max, Ticks_y_max);
      end_if;
      new_args := new_args, YRange = y_min..y_max;
   elif type(Ticks_y_min) <> DOM_IDENT then
      new_args := new_args, YRange = Ticks_y_min..Ticks_y_max;      
   end_if;

   if type(Ticks_y_min) = DOM_IDENT and not ticks then
      scene_options := scene_options union 
        {Ticks=[[Ticks_x_min..Ticks_x_max, Ticks_x_delta, Ticks_x_step],Ticks_y]}; 
   else 
      scene_options := scene_options union 
        {Ticks=[[Ticks_x_min..Ticks_x_max, Ticks_x_delta, Ticks_x_step],
                [Ticks_y_min..Ticks_y_max, Ticks_y_delta, Ticks_y_step]]
        };
   end_if;

   if axes = Origin then
      if origin[1] = XMin then
         origin[1] := Ticks_x_min;
      elif origin[1] = XMax then
         origin[1] := Ticks_x_max;
      elif origin[1] < Ticks_x_min then 
         origin[1] := Ticks_x_min;
      elif origin[1] > Ticks_x_max then 
         origin[1] := Ticks_x_max; 
      end_if;
      if Ticks_y_min <> YMin then
         if origin[2] = YMin then
            origin[2] := Ticks_y_min;
         elif origin[2] = YMax then
            origin[2] := Ticks_y_max;
         elif origin[2] < Ticks_y_min then 
            origin[2] := Ticks_y_min;
         elif origin[2] > Ticks_y_max then 
            origin[2] := Ticks_y_max; 
         end_if;
      end_if;
      scene_options := scene_options union {AxesOrigin=origin};
   end_if;
   if not has(scene_options, Axes) then
      scene_options := scene_options union {Axes=axes};
   end_if;
   if axes = Origin and not has(scene_options, AxesOrigin) then
      scene_options := scene_options union {AxesOrigin=origin};
   end_if;

   // Redefine plot to avoid plotting. Therefore plotfunc returns only
   // the evaluated arguments without drawing.
 
   unprotect(plot); sys_plot := plot; plot := proc() begin args() end_proc:

   hold(plotfunc)(scene_options, new_args);
   err := traperror((s:=eval(%)));

   // Reset the original plot function.
   plot:=sys_plot; protect(plot);

   if err <> 0 then
      error("Illegal option.");
   end_if;

   if show_gridlines then
      if type(Ticks_y_min) = DOM_IDENT then
         warning("Cannot draw grid lines.\nPlease specify YRange or ViewingBox.");
      else
         Ticks_x_step := float(Ticks_x_delta/Ticks_x_step);
         Ticks_y_step := float(Ticks_y_delta/Ticks_y_step);
         if frac(Ticks_x_min/Ticks_x_step) <> 0.0 then
            Ticks_x_min := (trunc(Ticks_x_min/Ticks_x_step)+sign(Ticks_x_min))
                           * Ticks_x_step;
         end_if;
         if frac(Ticks_y_min/Ticks_y_step) <> 0.0 then
            Ticks_y_min := (trunc(Ticks_y_min/Ticks_y_step)+sign(Ticks_y_min))
                           * Ticks_y_step;
         end_if;
         s := s, draw_gridlines(Ticks_x_min,  Ticks_x_max, 
                                Ticks_y_min,  Ticks_y_max,
                                Ticks_x_step, Ticks_y_step);                     
      end_if;
   end_if;

   s := s, plot2d_objects;

   if stdlib::ShowPlot = FALSE then s else plot(s) end_if;
   
end_proc:
