#===========================================================================

sum -- (in)definite summation of rational functions

Calling sequence:
sum(f, x)

Parameters:
f -- an expression (rational function)
x -- an

Summary:
sum(f,x) computes the indefinite sum of f(x) with respect to x.

Copyright:
Paul Zimmermann and Inria, November 1994

--------------------------------------------------------------------------

Examples of the implementation of the "sum" function:

1) Polynomial case:

>> sum(1,k);

                                     k

>> sum(k^2,k);

                                     2    3
                                k   k    k
                                - - -- + --
                                6   2    3
 
>> sum(a*x^2+b*x+c,x);
 
                    3
                 a x      / a   b     \    2 /   a   b  \
                 ---- + x | - - - + c | + x  | - - + -  |
                  3       \ 6   2     /      \   2   2  /
 
 
2) Fractional case: 

a) Decomposition into the polynomial part and the rational part:
 
>> sum(k/(k+1),k);
 
                              k - psi(k + 1)
 
>> sum((x^2+1)/(x-1),x);
 
                              2
                         x   x     
                         - + -- + 2 psi(x - 1) 
                         2   2 
 
 
b) Reduced rational expressions:
   (degree of numerator is less than degree of denominator)

>> sum(1/k/(k-2),k);

                                1           1    
                          - --------- - ---------
                            2 (k - 1)   2 (k - 2)


>> sum(1/k/(2*k-1),k);

                          - psi(k) + psi(k - 1/2)


==========================================================================#

sum := proc(_f_,_x) local g,_k_,Rec,u,_d_,l,_p,n,q,z,toE,_E;
begin
   if args(0)<>2 then error("two arguments expected") end_if;

   case type(_f_)
   of "limit" do
   of DOM_PROC do
   of DOM_EXEC do
   of "fun" do
   of Puiseux do
   of gseries do
   of "diff" do
   of "int" do
   of DOM_SET do
   of DOM_LIST do
        return( procname(args()) )
   end_case;

   if has(_f_,{hold(cos),hold(sin),hold(cosh),hold(sinh)}) then
	_f_:=rewrite(_f_,exp) end_if;
   if (toE:=has(_f_,exp)) then 
	# _f_:=eval(subs(_f_,hold(exp)=fun(_E^args(1)))) end_if; #
	_f_:=subs(_f_,hold(exp)=fun(_E^args(1)),Simplified) end_if;
   if type(_x)=DOM_IDENT then Sum::indefinite(_f_,_x)
   elif type(_x)="_equal" and type((_k_:=op(_x,1)))=DOM_IDENT then
      if type(op(_x,2))="_range" then
         # definite summation #
         if type((g:=op(_x,[2,2])-op(_x,[2,1])))=DOM_INT then # finite range #
            _f_:=subs(args(1),_k_=hold(_k_)+op(_x,[2,1]));
            return((if g>=0 then _plus(level(_f_,2)$hold(_k_)=0..g)
            else -_plus(level(_f_,2)$hold(_k_)=g+1..-1)
            end_if))
         elif has(op(_x,2),_k_) then error("summation variable appears in bounds")
	 elif hastype(op(_x,2),DOM_FLOAT) then error("floating-point numbers not allowed in bounds")
         end_if;
         g:=Sum::indefinite(_f_,_k_);
         if type(g)="sum" and type(_f_)="_plus" then # try to map #
            l:=[]; for q in _f_ do
               u:=Sum::indefinite(q,_k_);
               if type(u)="sum" then break else l:=append(l,u) end_if
            end_for;
            if nops(l)=nops(_f_) then g:=_plus(op(l)) end_if;
         end_if;
	 if type(g)="sum" then 
            userinfo(1,"indefinite summation failed");
            if Sum::ishypergeom(_f_,_k_) then
               userinfo(1,"summand is hypergeometric");
	       # try Zeilberger's algorithm #
               # choose in priority indets that are in the bounds #
               l:=indets(op(_x,2));
               l:=[op(l),op(indets(_f_) minus (l union {_k_}))];
	       for n in l do
		  if Sum::ishypergeom(_f_,n) then
                     userinfo(1,"trying Zeilberger's algorithm with respect to",n);
                     u:=genident("u");
		     Rec:=Sum::zeilberger(_f_,_k_,n,u,op(op(_x,2)));
		     if Rec=FAIL then 
                        userinfo(1,"Zeilberger's algorithm fails"); next
                     else userinfo(1,"Zeilberger's algorithm succeeds") end_if;
                     g:=Rec[2]; Rec:=Rec[1];
		     _d_:=op(Rec,[1,1])-n;
		      _p:=subsop(proc(n) begin n end_proc,1=n,4=hold(sum)(_f_,_k_=op(_x,2)));
                     if traperror((g:={u(z)=_p(z)$hold(z)=g-_d_..g-1}))<>0 then
                        g:={u(z)=_p(z)$hold(z)=g-_d_+1..g}
                     end_if;
                     Rec:=solve(rec(Rec,u(n),g));
                     if not has(Rec,undefined) then
                        if type(Rec)=DOM_SET and nops(Rec)>=1 then
			   Rec:=op(Rec,1);
                           g:=select(Indets(Rec,"function"),fun(op(args(1),0)=u));
                           g:=select(map(g,op,1),testtype,DOM_INT);
                           Rec:=subs(Rec,u(op(g,z))=_p(z)$hold(z)=1..nops(g));
                        end_if;
                        return(Rec)
                     end_if
		  end_if
               end_for;
               # f is hypergeometric but Zeilberger's algorithm fails #
               userinfo(1,"trying definite summation of hypergeometrics");
               g:=Sum::hypergeomtype(_f_,_k_);
               userinfo(2,"hypergeometric type is",g);
               if g<>FAIL then
                  g:=Sum::hypergeomeval(g,op(op(_x,2)),_f_,_k_);
                  if g<>FAIL then return(g) end_if
               end_if;
               userinfo(1,"definite summation of hypergeometrics failed");
            end_if;
            # not hypergeometric in k or in another variable #
            procname(_f_,_x)
	 else # indefinite summation succeeded #
            userinfo(2,"indefinite summation gave",g);
	    # eval is needed to evaluate psi(1,1) into PI^2/6 for example #
	    if traperror((g:=Sum::eval(g,_k_,op(_x,[2,2])+1)
		-Sum::eval(g,_k_,op(_x,[2,1]))))=0 then normal(level(g,2))
            else procname(_f_,_x)
            end_if
	 end_if
      elif type((_p:=op(_x,2)))=RootOf then # sum(..., k=RootOf(q,z)) #
         userinfo(1,"sum over RootOf");
         z:=extop(_p,2);
         q:=subs(extop(_p,1),z=_k_);
         if testtype(_f_,Type::RatExpr(_k_)) then # first simplify to a sum over a polynomial #
            userinfo(2,"summand is rational");
            _f_:=RootOf::evala(_f_,q,_k_)
         end_if;
         if testtype(_f_,Type::PolyExpr(_k_)) then # evaluate #
            RootOf::polysum(_f_,q,_k_)
         else hold(sum)(args())
         end_if
      else
         error("invalid second argument")
      end_if
   else error("invalid arguments")
   end_if;
   if toE then subs(maprec(combine(%),{"_power"}=
      fun((if op(args(1),1)=_E then exp(op(args(1),2)) else args(1) end_if))),
      _E=exp(1))
   else %
   end_if
end_proc:

sum:= funcattr(sum, "type", "sum"):
sum:= funcattr(sum, "print", "sum"):

Sum:= domain("Sum"):
Sum::info:= "Sum: Utilities for the sum function":
Sum::interface:= {}:

Sum::eval := proc(_f_,_x_,v) # evaluate value of f at x=v #
begin
   if v=infinity then
      limit(_f_,_x_=infinity)
   else # do no eval here ==> problems if f contains the name "x",
          for example sum(x^k*y^(n-k),k=0..n) or sum(x^k,k=1..2) #
      subs(_f_,_x_=v)
   end_if
end_proc:

# returns h such that h(x+1)-h(x)=f(x) #
Sum::indefinite:=proc(_f_,_x_)
begin
   userinfo(1,"entering indefinite summation");
   if testtype(_f_,Type::PolyExpr(_x_,Type::AnyType)) then Sum::poly(_f_,_x_)
   elif testtype(_f_,Type::RatExpr(_x_,Type::AnyType)) then Sum::rat(_f_,_x_)
   else Sum::gosper(_f_,_x_)
   end_if
end_proc:

# p is a polynom in x, returns h such that p = subs(h,x=x+1)-h #
Sum::poly := proc(p,x)
local d,i,b,a;
begin
  if not has(p,x) then p*x
  else # method of indeterminate coefficients #
   p:=poly(p,[x]); d:=degree(p);
   # first tabulate a[i]=(x+1)^i #
   a[1]:=poly(x+1,[x]); for i from 2 to d+1 do a[i]:=a[1]*a[i-1] end_for;
   for i from d+1 downto 2 do
      b[i]:=coeff(p,i-1)/i;
      p:=p+multcoeffs(lmonomial(a[i],Rem)[2],-b[i]);
   end_for;
   b[1]:=coeff(p,0);
   _plus(b[i]*x^i $ hold(i)=1..d+1)
  end_if
end_proc:

Sum::rat := proc(_f_,x)
local num,den,d,dnum,dden;
begin
   num:=numer(_f_); den:=denom(_f_);
   dnum:=degree(num,x); dden:=degree(den,x);
   if dnum>=dden then
      d:=divide(poly(num,[x]),poly(den,[x]));
      Sum::poly(expr(d[1]),x)+Sum::rat(expr(d[2])/den,x);
   else # now deg(num)<deg(den) #
      d:=Sum::rat_abramov(_f_,x);
      if d=FAIL then hold(sum)(_f_,x) else d end_if
   end_if
end_proc:

# f is minimal : sum(f,x) can not be written p+sum(r,x)
  where p is a polynomial and denom(r) has lower degree than denom(f)
#
Sum::to_psi := proc(_f_,x)
local ff,t,s,a,d,lin;
begin
   ff:=partfrac(_f_,x,hold(full));
   userinfo(3,"full partial fraction decomposition=",ff);
   if type(ff)="_plus" then ff:=[op(ff)] else ff:=[ff] end_if;
   s:=0;
   for t in ff do
      d:=stdlib::denom(t); # because denom expands the denominator #
      d:=d/maprat(d,fun(content(args(1),[x]))); # primpart(d,[x]) expands too !!! #
      if type(d)="_power" and testtype(op(d,2),Type::NonNegInt) then lin:=op(d,1)
      else lin:=d
      end_if;
      if testtype(lin,Type::PolyExpr(x,Type::AnyType)) then
         case degree(lin,[x])
         of 1 do a:=coeff(lin,[x],0)/coeff(lin,[x],1);
                 s:=s+subs(Sum::to_psi2(normal(subs(t,x=x-a)),x),x=x+a);
                 break;
         otherwise return(FAIL)
         end_case
      else # non polynomial #
         return(FAIL)
      end_if
   end_for;
   s
end_proc:

Sum::is_linear:=proc(p,x) # checks for a*x+b #
begin
   if testtype(p,Type::PolyExpr(x,Type::AnyType)) then
      if degree(p,x)=1 then return(TRUE) end_if
   end_if;
   FALSE
end_proc:

Sum::to_psi2 := proc(_f_,x) # the denominator looks like a*x^j #
# f must be in normal form #
local n,j,res,d,c,dd,t;
begin
   res:=0;
   d:=denom(_f_);
   c:=maprat(d,fun(content(args(1),[x])));
   d:=normal(d/c); # should be given by primpart #
   if d=-x then d:=x; c:=-c end_if;
   if not ((d=x) or (type(d)="_power" and op(d,1)=x and type(op(d,2))=DOM_INT))
   then return(FAIL) # should not happen #
   end_if;
   dd:=degree(d,x)-1;
   n:=numer(_f_);
   if type(n)="_plus" then n:=[op(n)] else n:=[n] end_if;
   for t in n do
      j:=dd-degree(t,x);
      res:=res+maprat(t,fun(lcoeff(args(1),[x])))/c*(-1)^j/fact(j)*psi(x,j);
   end_for;
   res
end_proc:

Sum::rat_abramov := proc(_f_,x)
begin
   userinfo(1,"entering Abramov's algorithm");
   Sum::abr(_f_,Sum::dispersion(denom(normal(_f_)),x),x)
end_proc:

Sum::abr := proc(_f_,dis,x)
local p,q,cp,vwp,vwm,ba,u,newf;
begin
   userinfo(2,"dispersion=",dis);
   if _f_=0 then return(0) end_if;
   if dis=0 then return(Sum::to_psi(_f_,x)) end_if;
   p:=numer(_f_); q:=denom(_f_);
   cp:=gcd(q,subs(q,x=x+dis));
   if cp=1 then return(Sum::abr(_f_,dis-1,x)) end_if;
   vwp:=Sum::abr_part(q,cp);
   vwm:=Sum::abr_part(q,subs(cp,x=x-dis));
   if degree(vwm[1],x)>degree(vwp[1],x) then
      ba:=pdioe(vwp,p,x);
      u:=subs(ba[2]/vwp[1],x=x-1);
      newf:=normal(ba[1]/vwp[2]+u);
   else
      ba:=pdioe(vwm,p,x);
      u:=-ba[2]/vwm[1];
      newf:=normal(ba[1]/vwm[2]+subs(ba[2]/vwm[1],x=x+1))
   end_if;
   u+Sum::abr(newf,dis-1,x)
end_proc:

Sum::abr_part := proc(p,h)
local g,v,w;
begin
   g:=gcd(p,h); v:=1; w:=p;
   while g<>1 do w:=divide(w,g,Quo); v:=v*g; g:=gcd(w,v) end_while;
   v,w
end_proc:

Sum::dispersion := proc(q,x) # q is a polynomial #
local _k,r,i,s;
begin
   q:=normal(q/content(q,[x]));
   r:=resultant(q,subs(q,x=x+_k),x);
   r:=factor(r);
   s:={};
   for i from 2 to nops(r) step 2 do
      # this is because iroots does not like other variables #
      if indets(op(r,i))={_k} then s:=s union sharelib::iroots(op(r,i),FALSE) end_if;
   end_for;
   # no multiplicity for iroots #
   max(op(s))
end_proc:

Sum::ishypergeom := proc(_f_,n)
begin
   testtype(Sum::ratio(_f_,n),Type::RatExpr(n))
end_proc:

# f being hypergeometric in k, puts f(k+1)/f(k) in the form
  (k+a1)...(k+ap)/(k+b1)/.../(k+bq) x/(k+1)
  and returns [a1,...,ap], [b1,...,bq], x,
  cf Wilf page 5 #
Sum::hypergeomtype := proc(_f_,k)
local g,P,Q,a,b,x,t,d;
begin
   g:=normal(subs(Sum::ratio(_f_,k),k=k+1));
   P:=Factor(numer(g));
   if type(P)<>"_mult" then P:=[P] end_if;
   a:=[]; b:=[]; x:=1;
   for t in P do
      if not has(t,k) then x:=x*t
      else
         if type(t)="_power" then d:=op(t,2); t:=op(t,1) else d:=1 end_if;
         if degree(t,k)<>1 then return(FAIL) end_if;
         x:=x*coeff(t,k,1);
         a:=append(a,coeff(t,k,0)/coeff(t,k,1) $ d);
      end_if
   end_for;
   Q:=Factor(denom(g));
   if type(Q)<>"_mult" then Q:=[Q] end_if;
   for t in Q do
      if not has(t,k) then x:=x/t
      else
         if type(t)="_power" then d:=op(t,2); t:=op(t,1) else d:=1 end_if;
         if degree(t,k)<>1 then return(FAIL) end_if;
         x:=x/coeff(t,k,1);
         b:=append(b,coeff(t,k,0)/coeff(t,k,1) $ d);
      end_if
   end_for;
   t:=contains(b,1);
   if t=0 then # no factor k+1 in the denominator, add one #
      a:=append(a,1)
   else # delete it #
      b[t]:=NIL
   end_if;
   a,b,x
end_proc:

# sum(f,k=a..b) when sum(f,k={0,1}..infinity) = pFq(P,Q,z) #
Sum::hypergeomeval := proc(P,Q,z,a,b,_f_,k) local init,kk;
begin
   userinfo(1,"hypergeometric of type ".nops(P)."F".nops(Q));
   if nops(P)=1 and nops(Q)=0 then # 1F0 #
      if testtype(a,Type::NonNegInt) and b=infinity then
         userinfo(2,"1F0 with right bound at infinity");
         if P=[0] then
            init:=_plus(subs(_f_,k=kk, Simplified) $ kk=1..a-1);
            # return(-ln(1-z)*eval(subs(_f_,k=1))/z-init) #
            return(-ln(1-z)*subs(_f_,k=1,Simplified)/z-init)
         else
           init:=_plus(subs(_f_,k=kk, Simplified) $ kk=0..a-1);
           # return(((1-z)^(-P[1])-init)*eval(subs(_f_,k=0))) #
           return(((1-z)^(-P[1])-init)*subs(_f_,k=0,Simplified))
         end_if
      end_if
   elif nops(P)=0 and nops(Q)=0 then # 0F0 #
      if testtype(a,Type::NonNegInt) and b=infinity then
           # init:=eval(_plus(subs(_f_,k=kk) $ kk=0..a-1)); #
           init:=_plus(subs(_f_,k=kk,Simplified) $ kk=0..a-1);
           # return(exp(z)*eval(subs(_f_,k=0))-init) #
           return(exp(z)*subs(_f_,k=0,Simplified)-init)
      end_if
   elif nops(P)=2 and nops(Q)=1 then # 2F1 #
      return((if P=[1/2,1] and Q=[3/2] and a=0 and b=infinity then
         atan(I*sqrt(z))/I/sqrt(z)
// sum((-1)^k/k^2,k=1..infinity) yields in this case a wrong result
//      elif P=[0,0] and Q=[1] and testtype(a,Type::PosInt) and b=infinity then
//	 init:=_plus(subs(_f_,k=kk) $ kk=1..a-1);
//	 # 2*(PI^2/12-ln(2)^2/2-init)*eval(subs(_f_,k=1)) #
//	 2*(PI^2/12-ln(2)^2/2-init)*subs(_f_,k=1,Simplified)
      else FAIL
      end_if))
   elif nops(P)=3 and nops(Q)=2 then # 3F2 #
      if P=[1,1,1] and Q=[2,3/2] and z=1/4 and b=infinity and 
	testtype(a,Type::NonNegInt) then # sum(n!^2/(2*n+2)!,n=a..infinity) #
           init:=level(_plus(_f_ $ k=0..a-1));
           # return((PI^2/9-2*init)*eval(subs(_f_,k=0))) #
           return((PI^2/9-2*init)*subs(_f_,k=0,Simplified))
      end_if
   end_if;
   FAIL
end_proc:

# computes f/subs(f,n=n-1) and returns a normalized result #
Sum::ratio := proc(_f_,n) 
local p,a,b,i;
option remember;
begin
   case type(_f_)
   of "_mult" do return(normal(map(_f_,Sum::ratio,n)))
   of "_power" do
      if not has(op(_f_,2),n) then return(Sum::ratio(op(_f_,1),n)^op(_f_,2)) 
      elif not has(op(_f_,1),n) then
	return(op(_f_,1)^(op(_f_,2)-subs(op(_f_,2),n=n-1)))
      else break end_if
   of "fact" do
      if not has((p:=Sum::ratio(rewrite(_f_,gamma),n)),gamma) then p end_if;
      break
   of "gamma" do
      if not has(_f_,n) then return(1)
      elif Sum::is_linear((p:=op(_f_)),n) then
         if type((a:=coeff(p,n,1)))=DOM_INT then
            b:=coeff(p,n,0);
            if a>0 then return(_mult(a*n+b-i$i=1..a))
            # elif a<0 then return(1/eval(_mult(a*n+b+i$i=0..-a-1))) #
            elif a<0 then return(1/subs(_mult(a*n+b+i$i=0..-a-1),Simplified)) # PR-6 #
            # a cannot be 0 otherwise p would be independent of n #
            end_if
         end_if
      end_if;
   end_case;
   # other cases #
   if type(_f_)="function" then
      if op(_f_,0)=hold(rfact) then
         if not has(op(_f_,1),n) and Sum::is_linear(op(_f_,2),n) and
	    coeff(op(_f_,2),n,1)=1 then return(op(_f_,1)+op(_f_,2)-1)
         end_if
      end_if
   elif type(_f_)="binomial" then return(Sum::ratio(expand(_f_),n))
   end_if;
   normal(_f_/expand(subs(_f_,n=n-1)))
end_proc:

# from GKP pages 224-226 #
Sum::gosper := proc(_f_,_n_) # returns g such that f = subs(g,n=n+1)-g #
local rat,qn,rn,res;
begin
   userinfo(1,"enter Gosper");
   if not has(_f_,_n_) then _f_*_n_
   else
      # write f(k+1)/f(k) = p(k+1)/p(k) * q(k)/r(k+1) (5.117)
        where roots of q and r do not differ by an integer #
      rat:=Sum::ratio(_f_,_n_);
      # check for geometric summations #
      # if not has(rat,_n_) then return(eval(subs(_f_,_n_=0))*rat^_n_/(rat-1)) end_if; #
      if not has(rat,_n_) then return(subs(_f_,_n_=0,Simplified)*rat^_n_/(rat-1)) end_if;
      if has(rat,{binomial,fact}) then rat:=normal(expand(rat)) end_if;
      userinfo(2,hold(f)(_n_)/hold(f)(_n_-1)=rat);
      qn:=numer(rat);
      rn:=denom(rat);
      if not (testtype(qn,Type::PolyExpr(_n_,Type::AnyType)) 
         and testtype(rn,Type::PolyExpr(_n_,Type::AnyType))) then
            return(hold(sum)(_f_,_n_))
      end_if;
      userinfo(1,"input is hypergeometric");
      res:=Sum::gosper2(1,qn,rn,_n_,{},_f_);
      if res=FAIL then 
         userinfo(1,"Gosper's algorithm fails");
         hold(sum)(_f_,_n_)
      else normal(res*_f_)
      end_if
   end_if
end_proc:

# returns R=g/f such that f = subs(g,n=n+1)-g for f(n)/f(n-1)=p(n)/p(n-1)*qn/rn
  with pn,qn,rn polynomials,
  inds is a set of auxiliary unknowns (used only in Zeilberger).
  As f[n] = g[n+1]-g[n]. we must have f[n]=R[n+1]*f[n+1]-R[n]*f[n],
  i.e. 1 = R[n]*p(n)/p(n-1)*qn/rn-R[n-1]
#
Sum::gosper2 := proc(pn,qn,rn,n,inds,f)
local jj,rnj,res,i,p_zeros,z,j,gn,gnj,dp,R,dR,Q,dQ,d,k0,fn,eq,F,ii,m,unk,ff;
begin
   userinfo(3,"pn=",pn,"qn=",qn,"rn=",rn);
      rnj:=subs(rn/content(rn,[n]),n=n+jj); # do not expand ! #
      res:=resultant(qn,rnj,n);
      res:=factor(res);
      p_zeros:=[];
      for i from 1 to nops(res) div 2 do
         ff:=op(res,2*i);
         if degree(ff,jj)=1 then
            z:=-coeff(ff,jj,0)/coeff(ff,jj,1);
            if testtype(z,Type::PosInt) then p_zeros:=append(p_zeros,z) end_if
         end_if
      end_for;
      for j from nops(p_zeros) downto 1 do
         gn:=gcd(qn,expand(subs(rn,n=n+p_zeros[j])));
         qn:=divide(qn,gn,Exact);
         gnj:=gn;
         for i from 1 to p_zeros[j] do
            pn:=expand(pn*gnj);
            gnj:=expand(subs(gn,n=n-i));
         end_for;
         rn:=normal(rn/gnj); # divide(rn,gnj,Exact) ? #
      end_for;
      # make the polynoms monic (cf Prop. 3.2 of [LiPaSt93]) #
      qn:=qn/lcoeff(rn,[n]);
      rn:=rn/lcoeff(rn,[n]);
      qn:=expand(subs(qn,n=n+1));
      dp:=degree(pn,n);
      R:=expand(qn+rn); dR:=degree(R,n);
      Q:=expand(qn-rn); dQ:=degree(Q,n);
      if dQ=FAIL or dR=FAIL then return(FAIL) end_if;
      if dR<=dQ then # case 1 #
         userinfo(2,"case 1 of Gosper's algorithm");
         if dQ=0 then # polynomial case #
            d:=dp+1
         else
            d:=dp-dQ
         end_if
      else # case 2 : deg(qn)=deg(rn) and lcoeff(qn)=lcoeff(rn) #
         m:=dR; # to have the same notation as [LiPaSt93] #
         k0:=coeff(rn-qn,n,m-1); # (5.2) of [LiPaSt93] #
         if type(k0)<>DOM_INT or m=1 # cf [LiPaSt93] page 255 #
         or testtype(f,Type::RatExpr(n,Type::AnyType)) then # case 2a #
            userinfo(2,"case 2a of Gosper's algorithm");
            d:=dp-m+1
         else # case 2b #
            userinfo(2,"case 2b of Gosper's algorithm");
            d:=max(k0,dp-m+1)
         end_if
      end_if;
      userinfo(2,"d=",d);
      if d>=0 then
         fn:=_plus(F[ii]*n^ii$ii=0..d);
         # eq. (GE) of [LiPaSt93] p. 248 #
         # where qn is q(n+1) #
         # put minus sign before fn because of PR302-303-310 #
         eq:=expand(pn+qn*subs(-fn,n=n+1)+rn*fn);
         userinfo(2,"number of equations is",degree(eq,[n])+1);
         eq:={coeff(eq,n,i)$hold(i)=0..degree(eq,[n])};
         unk:=inds union {F[ii]$ii=0..d};
         if nops(eq)<nops(unk) then eq:=eq union {F[0]=0} end_if;
         userinfo(3,"system to solve is",eq);
         eq:=linsolve(eq,unk);
         userinfo(1,"solution is",eq);
         if eq=FAIL then FAIL # no solution #
         else
            # do the substitution into pn too because it can
              contain some indeterminates C[l] in Zeilberger's alg. #
            fn:=Sum::normal(subs(fn/pn,eq)*rn);
            if has(fn,F) then FAIL
            elif inds={} then fn # for Sum::gosper #
            else fn,eq # for Sum::zeilberger #
            end_if
         end_if
      else FAIL
      end_if
end_proc:

# required because normal expands #
Sum::normal := proc(f)
begin
   Sum::factor(numer(f))/Sum::factor(denom(f))
end_proc:

Sum::factor := proc(f)
local i,l;
begin
   l:=factor(f);
   _mult(l[1],l[2*i]^l[2*i+1]$i=1..nops(l) div 2)
end_proc:

# returns a..b such that f=0 for j<a or j>b #
Sum::support := proc(f,j) local a,b,x,k,c,d;
begin
   hastype: # to load Indets #
   a:=-infinity; b:=infinity;
   for x in Indets(f,"binomial") do
      for k in [op(x,2),op(x,1)-op(x,2)] do
         # k must be >= 0 #
	 if testtype(k,Type::PolyExpr(j)) then
	    if degree(k,[j])=1 then
	       d:=coeff(k,[j],1); c:=-coeff(k,[j],0)/d;
	       if is(coeff(k,[j],1)>0) then
		  if a=-infinity then a:=c else a:=max(a,c) end_if
	       elif is(coeff(k,[j],1)<0) then
		  if b=infinity then b:=c else b:=min(b,c) end_if
	       end_if
	    end_if
	 end_if
      end_for
   end_for;
   a..b
end_proc:

# u is the name of the output recurrence #
Sum::zeilberger:=proc(f,_k,n,u,a,b) # sum(f,k=a..b) #
local ratn,d,l,m,C,p,ratk,res,rnk,cert,va,vb,A,fcert,corr,i,sup,aa,bb;
begin
   if has(a,n) then return(FAIL) end_if; # not yet treated #
   ratn:=Sum::ratio(f,n); # f(n)/f(n-1) #
   if not has(ratn,{n,_k}) then return(FAIL) 
   elif not testtype(ratn,Type::RatExpr(n)) then return(FAIL)
   elif not testtype(ratn,Type::RatExpr(_k)) then return(FAIL)
   end_if;
   d:=1;
   repeat
      rnk:=1; m:=1;
      for l from 1 to d do
         m:=m*subs(ratn,n=n+l); # now m=f(n+l)/f(n) #
         rnk:=rnk+C[l]*m;
      end_for;
      # now apply Gosper to rnk*f with respect to k #
      # i.e. search if F=f(n)+C[1]*f(n+1)+...+C[d]*f(n+d)
        satisfies F(k)=G(k+1)-G(k) #
      userinfo(3,"rnk=",rnk);
      l:=C[l]$hold(l)=1..d;
      p:=numer(rnk);
      ratk:=Sum::ratio(f/denom(rnk),_k);
      res:=Sum::gosper2(p,numer(ratk),denom(ratk),_k,{l},f);
      if res<>FAIL then 
         cert:=subs(res[1]*rnk,res[2]); # g = cert*f #
         userinfo(1,"certificate=",cert);
         # one should have subs(G,k=k+1)-G-F=0 where
           F=f(n)+C[1]*f(n+1)+...=rnk*f and G=(cert/rnk)*F=cert*f,
           i.e. subs(cert*f,k=k+1)/f-cert-rnk=0 #
         # thus sum(f,k=a..b)+...+sum(subs(f,n=n+d),k=a..b) 
		= subs(f*cert,k=b+1) - subs(f*cert,k=a) #
         # only deal with the cases where a is independent from n
           and b is independent from n or b=n #
         A:=_plus(u(n),C[l]*u(n+l)$hold(l)=1..d);
         fcert:=normal(f*cert);
         sup:=Sum::support(f,_k);
         # if a<=op(sup,1) we can extend the lower bound to -infinity #
         aa:=simplify(min(a,op(sup,1)));
         if traperror((va:=eval(subs(expand(subs(f*cert,_k=a+_k)),_k=0))))<>0 
         then va:=hold(limit)(fcert,_k=a)
         end_if;
         # if op(sup,2)<=b we can extend the upper bound to infinity #
         bb:=simplify(max(b,subs(op(sup,2),n=n+d)));
         if traperror((vb:=eval(subs(expand(subs(f*cert,_k=b+1+_k)),_k=0))))<>0
         then vb:=hold(limit)(fcert,_k=b+1)
         end_if;
         A:=A-vb+va;
         # correcting terms if b depends on n #
         if A=undefined then return(FAIL)
         elif bb<>b and not has(b-n,n) then
	       corr:=_plus((C[l]*eval(subs(f,n=n+l,_k=b+i))$hold(i)=1..l)$hold(l)=1..d);
	       A:=A-corr
         else return(FAIL)
         end_if;
         A:=expand(subs(A,res[2]));
         # to check: subs(rnk,res[2])-subs(cert*Sum::ratio(f,k),k=k+1)+cert
	   should be zero #
         userinfo(1,"recurrence=",A);
         res:=numer(A);
         res:=-coeff(res,u(n+d),0)/coeff(res,u(n+d),1);
         res:=u(n+d)=collect(res,[u(n+l)$hold(l)=0..d-1],Factor);
         if subs(denom(cert),n=0)=0 then d:=d+1 end_if;
         return([res,d])
      else
         userinfo(1,"Gosper's algorithm fails for order ",d);
      end_if;
      d:=d+1; # try a higher order #
   until d>Sum::maxorder end_repeat;
   FAIL
end_proc:

Sum::maxorder:=2:

sum := funcattr(sum,"diff",
proc(e) # e = sum(f,x) #
local i,l,_x;
begin
  l:=[args(i)$i=2..args(0)]; # variables to differentiate #
  if map({op(l)},type)={DOM_IDENT} and not has((_x:=op(e,2)),l) then 
	# summation independent from differentiation #
     sum(diff(op(e,1),op(l)),_x)
  else
     hold(diff)(e,op(l))
  end_if
end_proc
):

sum := funcattr(sum,"expand",
proc()
  local s;
begin
  s:=expand(args(1));
  if type(s)="_plus" then
    _plus(map(s,expand@sum,args(2)))
  elif type(s)="_mult" then
    split(s,has,(if type(args(2))=DOM_IDENT then
                         args(2)
                       else
                         op(args(2),1)
                       end_if));
    %[2]*hold(sum)(%[1],args(2))
  else
    hold(sum)(s, args(2..args(0)))
  end_if
end_proc):

sum := funcattr(sum,"int",
proc(s,x) begin
   sum(int(op(s,1),x),op(s,2))
end_proc):

sum := funcattr(sum,"float",
   loadproc(funcattr(sum,"float"),pathname("NUMERIC"),"floatsum")):
