#-

string::match -- string matching 

Calling Sequence:

string::match( s, t, n, m, k)

Parameter:

s - non-empty string ( s = s[a] s[a+1] ... s[b] , b = a+n-1)
t - non-empty string
n - positive integer (length of s, 1 <= n <= strlen(s))
m - positive integer (length of t, 1 <= m <= strlen(t))
k - integer (  a <= k <= b )

Summary:

string::match returns an integer i , a <= i <= b, 
if the string s contains the string t and s[i..i+m-1] = t holds. 
Otherwise FAIL is returned. Note, that no check of the arguments is 
done.

The value a is determined by the integer variable string::FirstIndex.
The default value of string::FirstIndex is 0. All integers are valid
values of string::FirstIndex.

The running time of string::match( s, t, n, m, k) is O(m + n) (see [1]).
Note, that successive calls of string::match( s, t, n, m, k) with
different values of k needs only O( n ), because the first step
of the algorithm is computed only once  (see string::init_match, option 
remember). n = n - k + a

Examples:

>> s := "abcdabe": t := "ab": n := strlen(s): m := strlen(t):
>> i := string::match(s, t, n, m, 0); substring(s, i, m);

           0 
		   
	   "ab"
>> i := string::match(s, t, n, m, i+m); substring(s, i, m);

           4
		   
	   "ab"
		   
>> string::match(s, t, n, m, i+m);

           FAIL
		   
See also:

strmatch, strlen, substring

References: 

[1] D.E. Knuth, J.H. Morris, R.V. Pratt. Fast Pattern Matching in Strings, 
    SIAM Journal of Computing 6:2, 1977, p. 323-350

[2] R. Karp, M. Rabin. Efficient Randomized Pattern Matching Algorithms,
    IBM Journal res. Development, 1987

-#

string::match := proc( s, t, n, m, k)
   local OFFSET, index, j;
begin
   OFFSET := string::FirstIndex-1:
   index := string::init_match(t, m, OFFSET);
   j := 1; k := k-OFFSET;
   while j <= m and k <= n do
         while j > 0 and s[k+OFFSET] <> t[j+OFFSET] do
                j := index[j];
         end_while;
         k := k + 1; j := j + 1;
   end_while;
   if j > m then k - m + OFFSET else FAIL end_if
end_proc:

string::init_match := proc( t, m, OFFSET)
   option remember;
   local  index, i, j;
begin
   index := array(1..m);
   i := 0; j := 1; index[1] := 0;
   while j < m do
         while i > 0 and t[j+OFFSET] <> t[i+OFFSET] do
               i := index[i];
         end_while;
         i := i + 1; j := j + 1;
         if t[j+OFFSET] <> t[i+OFFSET] then
             index[j] := i
         else
             index[j] := index[i]
         end_if
   end_while;
   index
end_proc:

#-

Use this algorithm with MuPAD 1.2.2 / 1.2.9

string::match := proc( s, t, n, m, k)
   local OFFSET, index, j;
begin
   OFFSET := string::FirstIndex-1:
   index := string::init_match(t, m, OFFSET);
   j := 1; k := k-OFFSET;
   while j <= m and k <= n do
      if j > 0 then
         while substring(s, k+OFFSET, 1) <> substring(t, j+OFFSET, 1) do
	        j := index[j];
		if j = 0 then break end_if
	     end_while
	  end_if;
	  k := k + 1; j := j + 1;
   end_while;
   if j > m then k - m + OFFSET else FAIL end_if
end_proc:

string::init_match := proc( t, m, OFFSET)
   option remember;
   local  index, i, j;
begin
   index := array(1..m); 
   i := 0; j := 1; index[1] := 0;
   while j < m do
      if i > 0 then
         while substring(t, j+OFFSET, 1) <> substring(t, i+OFFSET, 1) do
	       i := index[i];
		   if i = 0 then break end_if
	     end_while
	  end_if;
	  i := i + 1; j := j + 1;
	  if substring(t, j+OFFSET, 1) <> substring(t, i+OFFSET, 1) then
	     index[j] := i
	  else
	     index[j] := index[i]
	  end_if
   end_while;
   index
end_proc:

End of algorithm for MuPAD 1.2.2 / 1.2.9 -#

#- End of algorithm D.E. Knuth, J.H. Morris, R.V. Pratt -#

