# paul #
#++
	fourier(f(x),x,y) returns the Exponential Fourier transform of f(x) 
	with respect to y,

	g_e(y) = int(f(x)*exp(I*y*x),x=-infinity..infinity).

	There are also the Fourier Cosine Transform

	g_c(y) = int(f(x)*cos(y*x),x=0..infinity)

	and the Fourier Sine Transform

	g_s(y) = int(f(x)*sin(y*x),x=0..infinity)

	If f is even, then g_e(y)=2*g_c(y); if f is odd, then g_e(y)=2*I*g_s(y),
	thus g_e(f(x)) = g_c(f(x)+f(-x))/2+I*g_s(f(x)-f(-x))/2

	f is any expression, x must be an identifier, y is any expression

	Reference: F. Oberhettinger, Tables of Fourier Transforms and Fourier 
	           Transforms of Distributions, Springer-Verlag, 1990
++#

alias(Match=intlib::match):
alias(Match2=intlib::matchassign):
alias(indep(a,x)=_pattern(a,Type::Independent(x))):
alias(_p=_pattern):

transform::fourier:= proc(f,x,y) 
   local _a,k,g,h,z,_b_,c,m,n,v,yy;
begin
   # allow overloading #
   if f::fourier<>FAIL then return(f::fourier(args())) 
   elif type(x)=DOM_IDENT then
     if not has(f,x) then return(f*2*PI*dirac(y)) end_if
   elif type(x/2/PI)=DOM_IDENT then # frequency form #
        x:=x/2/PI; return(2*PI*transform::fourier(f,x,2*PI*y))
   else error("invalid arguments")
   end_if;
   case type(f)
   of "heaviside" do # formula 3.1 p. 199 of Oberhettinger90 #
      if (k:=Type::Linear(op(f),[x]))<>FALSE then
         _a:=-k[2]/k[1]; return(PI*dirac(y)-I/y*exp(I*_a*y))
      end_if; break
   of "dirac" do
      if op(f)=x then return(1) end_if; break
   of "exp" do
      if not has((_a:=-op(f)/x^2),x) then # formula 3.17 p. 11 (even function) #
         return(sqrt(PI/_a)*exp(-y^2/4/_a))
      end_if; break
   of DOM_IDENT do return(2*I*PI*dirac(y,1)) # can be only x #
   of "_equal" do
   of "_plus" do return(map(f,transform::fourier,x,y))
   of "sin" do 
      if not has((_a:=op(f)/x),x) then return(I*PI*(dirac(y-_a)-dirac(y+_a))) end_if; break
   of "cos" do
      if not has((_a:=op(f)/x),x) then return(PI*(dirac(y+_a)+dirac(y-_a))) end_if; break
   of "_power" do # expr^k #
      k:=op(f,2);
      if op(f,1)=x and testtype(k,Type::PosInt) then
         return(2*PI*I^k*dirac(y,k))
      elif k=-1 then
         if Match2(op(f,1),indep(_a,x)*x+indep(_b_,x)) then
            return(I*PI*exp(-I*y*_b_/_a)/_a*(2*heaviside(y)-1))
         elif Match2(op(f,1),indep(_a,x)*x^2+indep(_b_,x)*x+indep(c,x)) then
            _b_:=_b_/2/_a; c:=c/_a;
            if _b_=0 then
               if sign(c)=-1 then
                  c:=sqrt(-c); return(-PI/c/_a*sin(c*y)*sign(c*y)) 
			# formula 1.29 p. 5 #
               else c:=sqrt(c); return(PI/c/_a*exp(-c*y)) # formula 1.25 p. 5 #
               end_if
            end_if; 
            # 1/(a*x^2+b*x+c) = 1/a*1/((x+b/2/a)^2+c/a-b^2/4/a^2) #
            c:=sqrt(_b_^2-c);
            # now 1/a*1/((x+b)^2-c^2) = 1/a/2/c*(1/(x+b-c)-1/(x+b+c)) #
            return(1/(2*_a*c)*(transform::fourier(1/(x+_b_-c),x,y)-transform::fourier(1/(x+_b_+c),x,y)))
         elif Match2(op(f,1),x^4+indep(_a,x)) then
               _a:=sqrt(sqrt(_a)); # formula 1.51 p. 7 #
               return(PI/_a^3*exp(-_a*y/2^(1/2))*sin(PI/4+_a*y/2^(1/2)))
         end_if
      elif not has((_a:=op(f,1)-x^2),x) then
         _a:=sqrt(_a); # (x^2+a^2)^k #
         if testtype((m:=-k-1),Type::PosInt) then # formula 1.57 p. 7 #
            return(2*PI*(2*_a)^(-m-1)*exp(-_a*y)*y^m/fact(m)*_plus(fact(m+k)/fact(m-k)/(2*_a*y)^k
		$hold(k)=0..m))
         elif is(Re(-k)>0)=TRUE then # formula 2.7 p. 8, includes formula 1.32 p. 5 #
            v:=-k-1/2;
            return(2*PI^(1/2)*(y/2/_a)^v/gamma(v+1/2)*besselK(v,_a*y))
         end_if
      end_if; break
   of "diff" do
      if op(op(f,1))=x then return((I*y)^(nops(f)-1)*procname(op(f,1),x,y)) end_if; break
   of "_mult" do
      loadlib("intlib"); # for match #
      if (k:=select(f,has,x))<>f then return(f/k*transform::fourier(k,x,y))
      # now all factors contain x #
      elif Match2(f,x^_p(n,Type::PosInt)*_p(hold(k))) then
         # fourier(x^n*f(x),x,y)=(-I)^n*diff(fourier(f(x),x,y),y$n) #
         return((-I)^n*subs(diff(transform::fourier(k,x,yy),yy$n),yy=y))
      elif contains([op(f)],heaviside(x))<>0 then
         return(laplace(f/heaviside(x),x,-I*y))
      elif (g:=Match(f,dirac(x-indep(_a,x))*_p(k)))<>FAIL then
         return(subs(k*exp(I*y*x),x=_a,op(g)))
      elif (g:=Match(f,1/(x^2+indep(_a,x)^2)*1/(x^2+indep(_b_,x)^2)))<>FAIL then
         _a:=subs(_a,op(g)); _b_:=subs(_b_,op(g));
         return(PI/(_a^2-_b_^2)*(exp(-_b_*y)/_b_-exp(-_a*y)/_a)) # formula 1.34 p. 5 #
      elif (g:=expand(f))<>f then return(transform::fourier(g,x,y))
      end_if; break
   of "int" do
      loadlib("intlib"); # for match #
      if (_a:=Match(f,hold(int)(_p(g)(_p(z))*_p(h)(x-_p(z)),_p(z)=-infinity..infinity)))<>FAIL then
         g:=subs(g,op(_a)); h:=subs(h,op(_a));
         return(transform::fourier(g(x),x,y)*transform::fourier(h(x),x,y))
      end_if; break
   of "transform::fourier" do
      if op(f,3)=x then return(2*PI*subs(op(f,1),op(f,2)=-y)) end_if; break
   end_case;
   loadlib("intlib");
   if (g:=Match(f,x^(2*_p(m,Type::PosInt))/(x^2+indep(z,x))^(_p(n)+1)))<>FAIL then
      m:=subs(m,op(g)); n:=subs(n,op(g));
      if testtype(n+1-m,Type::PosInt) then
         # formula 1.55 p. 7 #
         return((-1)^(m+n)*PI/fact(n)*subs(diff(z^(m-1/2)*exp(-y*z^(1/2)),z$n),op(g)))
      end_if
   elif (g:=Match(f,x^(2*_p(m,Type::PosInt))/(indep(_a,x)+x^(2*_p(n,Type::PosInt)))))<>FAIL then
      m:=subs(m,op(g)); n:=subs(n,op(g)); _a:=subs(_a,op(g))^(1/2/n);
      # formula 1.56 p. 7 #
      return(PI/n*_a^(2*m-2*n+1)*_plus(sin((2*m+1)*(k-1/2)*PI/n+_a*y
	*cos((k-1/2)*PI/n))*exp(-_a*y*sin((k-1/2)*PI/n))$hold(k)=1..n))
   end_if;
   procname(args())
end_proc:

transform::fourier:=funcattr(transform::fourier,"type","transform::fourier"):
